
/**************************************************/
/*                                                */
/* File: DyDump .c                                */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*                        **********                     */
/*                        *  DUMP  *                     */
/*                        **********                     */

/*------------------------------------------- DyXformAddDataToDump ---*/

void DyXformAddDataToDump (FL_OBJECT *obj, long arg)
{
 int level;
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyXformAddDataToDump> arg=%d", arg);
 DyDebugPrint(dummy,0);

 /*--- Add full frame to dump ---*/

 if (arg == 0)
    {
     DyXformAddToDumpList(NULL);  
     level = atoi(fl_get_input(dumpdata->dump_level));
     if (level <= 0) fl_set_input(dumpdata->dump_level,"1");
    }

 /*--- Add data to dump ---*/

 if (arg == ADD_DATA)
    {
     /*--- Loop over sms,adc and proc browsers and take the lines selected ---*/
     if (fl_get_browser(mainbrowser->sms)  != 0)  DyXformAddToDumpList(mainbrowser->sms);
     if (fl_get_browser(mainbrowser->adc)  != 0)  DyXformAddToDumpList(mainbrowser->adc);
     if (fl_get_browser(mainbrowser->proc) != 0)  DyXformAddToDumpList(mainbrowser->proc);
     if (fl_get_browser(mainbrowser->sim)  != 0)  DyXformAddToDumpList(mainbrowser->sim);
     level = atoi(fl_get_input(dumpdata->dump_level));
     if (level <= 0) fl_set_input(dumpdata->dump_level,"1");
    }

 /*--- Remove data to dump ---*/

 if (arg == REMOVE_DATA)
    {
     DyXformRemoveFromDumpList(dumpdata->datatodump);  
    }

 return;
}


/*------------------------------------------- DyXformAddToDumpList ---*/

void DyXformAddToDumpList (FL_OBJECT *obj)
{
 long nvar, ndump, i, j, j0;
 const char *str=NULL, *strdump=NULL;
 char dummy[LSTRING_MAX], strname[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (obj == NULL)
    {fl_add_browser_line(dumpdata->datatodump,"FULL FRAME");
     DyInfoLine ("Added full frame to the list of data to dump");
     return;
    }

 nvar = fl_get_browser_maxline(obj);

 for (i=0; i<nvar; i++)
    {
     if (fl_isselected_browser_line(obj, i+1))
        {
         str = fl_get_browser_line(obj, i+1);
         sscanf(str,"%s",strname);            /* to get only the first word */
                                  
         /*--- Check that it is not already in the list ---*/         
         j0 = -1;
         ndump = fl_get_browser_maxline(dumpdata->datatodump);
         for (j=0; j<ndump; j++)
            {strdump = fl_get_browser_line(dumpdata->datatodump, j+1);
             if (strcmp(strname,strdump) == 0) {j0 = j; break;}
            }
         if (j0 == -1)
            {fl_add_browser_line(dumpdata->datatodump, strname);
             sprintf(dummy, "Added %s to the list of data to dump", strname);
             DyInfoLine (dummy);
            }
        }
    }

 return;
}
/*-------------------------------------------- DyXformRemoveFromDumpList ---*/

void DyXformRemoveFromDumpList (FL_OBJECT *obj)
{
 char strnew[LSTRING_MAX];
 long ndump, i, i0, lastSelected=0;

 DyDebugPrint(__FUNCTION__,0);

 ndump = fl_get_browser_maxline(obj);
 if (ndump <= 0) return;

 i0 = 1;
 while (i0 > 0)
   {
    ndump = fl_get_browser_maxline(obj);
    if (ndump <= 0) break;

    i0 = -1;
    for (i=1; i<=ndump; i++)
       {if (fl_isselected_browser_line(obj,i)) {i0 = i; break;}}

    if (i0 <= 0) break;
    else         lastSelected = i0;

    for (i=i0; i<=ndump-1; i++)
      {
       strcpy(strnew, fl_get_browser_line(obj,i+1));
       fl_replace_browser_line(obj,i, strnew);
       if (fl_isselected_browser_line(obj,i+1))
          {fl_select_browser_line(obj,i);}
       else
          {fl_deselect_browser_line(obj,i);}
      }

   fl_delete_browser_line(obj,ndump);
  }

 ndump = fl_get_browser_maxline(obj);
 if (ndump > 0  &&  lastSelected > 0) fl_select_browser_line(obj,lastSelected);

 return;
}


/*------------------------------------------- DyXformDumpData ---*/

void DyXformDumpData (struct FrameH *frame, long not_frame)
{
 const char *str;
 char varName[LSTRING_MAX], dummy[LSTRING_MAX], tag[LBIGSTRING_MAX];
 long ndump, i, newtopline;
 int level;

 if (frame == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 sprintf(dummy," --------------------------------------------------------------------- \n");
 fl_add_browser_line(dumpdata->datadumped, dummy);
 sprintf(dummy," gps%d  fr%d  run%d  (%s)\n",
                 frame->GTimeS, frame->frame,frame->run,FrStrGTime(frame->GTimeS));
 fl_add_browser_line(dumpdata->datadumped, dummy);
 sprintf(dummy," --------------------------------------------------------------------- \n");
 fl_add_browser_line(dumpdata->datadumped, dummy);

 /*--- Set the dump level ---*/

 level = atoi(fl_get_input(dumpdata->dump_level));
 if (level < 0)
    {level = 2; fl_set_input(dumpdata->dump_level,"2");}
 
 /*--- If there is name FULL FRAME in the list of data to dump, we dump full frame ---*/

 ndump = fl_get_browser_maxline(dumpdata->datatodump);
 if (ndump <= 0) return;
 for (i=0; i<ndump; i++)
    {str = fl_get_browser_line(dumpdata->datatodump,i+1);
     if (strcmp(str,"FULL FRAME") == 0) {not_frame = 0; break;}
    }

 if (not_frame)
    {
     ndump = fl_get_browser_maxline(dumpdata->datatodump);
     if (ndump <= 0) return;

     strcpy(tag, " ");
     for (i=0; i<ndump; i++)
        {str = fl_get_browser_line(dumpdata->datatodump,i+1);
         sscanf(str, "%s", varName);
         strcat(tag,varName);
         strcat(tag," ");
/*
         DyDumpOneVar(frame, varName, level);
*/
        }
     FrameTagAdc(frame,tag);
     FrameTagSer(frame,tag);
     DyFillFrameDump(frame, level);
     FrameUntagAdc(frame);
     FrameUntagSer(frame);

    }
 else
    {DyFillFrameDump(frame, level);}

 newtopline = fl_get_browser_maxline(dumpdata->datadumped) - fl_get_browser_screenlines(dumpdata->datadumped) + 1;
 fl_set_browser_topline(dumpdata->datadumped, newtopline);

 return;
}


/*-------------------------------------------- DyDumpOneVar ----*/

void DyDumpOneVar (struct FrameH *frame, char *varName, long level)
{
 struct FrAdcData  *adc;
 struct FrProcData *proc;
 struct FrSerData  *sms;
 char *dummy, dummy2[LSTRING_MAX];
 double period;

 if (frame == NULL)   return;
 if (varName == NULL) return;

 sprintf(dummy2,"DyDumpOneVar> frame=%d  varName=%s", frame->GTimeS, varName);
 DyDebugPrint(dummy2,0);

 if (level < 3)  dummy = (char *)malloc(LBIGSTRING_MAX);
 if (level == 3) dummy = (char *)malloc(65536);
 if (level > 3)  dummy = (char *)malloc(512000);

 /*--- data to dump is ADC data ---*/

 if ((adc = FrAdcDataFind(frame, varName)) != NULL)
    {
     sprintf(dummy, " ADC: %s \n", adc->name);
     DyXformAddInDump (dummy);
     sprintf(dummy, "   nBits=%d  sampleRate=%.2f  crate=%d  channel=%d \n",
                    adc->nBits, adc->sampleRate, adc->channelGroup, adc->channelNumber);
     DyXformAddInDump (dummy);
     if (adc->data != NULL)
        {DyFrVectDumpToBuf(adc->data, dummy, level);
         DyXformAddInDump (dummy);
        }
    }

 /*--- data to dump is PROC data ---*/

 else if ((proc = FrProcDataFind(frame, varName)) != NULL)
    {
     sprintf(dummy, " PROC: %s \n", proc->name);
     DyXformAddInDump (dummy);
     sprintf(dummy, "   sampleRate=%.2f  fShift=%.2f \n", 1./proc->data->dx[0], proc->fShift);
     DyXformAddInDump (dummy);
     if (proc->data != NULL)
        {DyFrVectDumpToBuf(proc->data, dummy, level);
         DyXformAddInDump (dummy);
        }
    }

 /*--- data to dump is SMS data ---*/

 else if ((sms = FrSerDataFind(frame, varName, NULL)) != NULL)
    {
     sprintf(dummy, " SMS: %s \n", sms->name);
     DyXformAddInDump (dummy);
     if (sms->sampleRate > 0.) period = 1. / sms->sampleRate;
     else                      period = 999999.;
     sprintf(dummy, "   timeSec=%s  timeNsec=%d  sampleRate=%g (period=%.2g) \n",
                    FrStrGTime(sms->timeSec), sms->timeNsec, sms->sampleRate, period);
     DyXformAddInDump (dummy);

     if (level > 0)
        {sprintf(dummy,"   dataString=%s \n", sms->data);
         DyXformAddInDump (dummy);
        }

     if (sms->serial != NULL)
        {DyFrVectDumpToBuf(sms->serial, dummy, level);
         DyXformAddInDump (dummy);
        }
    }
 else
    {sprintf(dummy, "    No %s present in this frame \n", varName);
     DyXformAddInDump (dummy);
    }

 free(dummy);

 return;
}


/*------------------------------------------- DyXformSaveDump ---*/

void DyXformSaveDump (FL_OBJECT *obj, long arg)
{
 long nb_ligne, i;
 const char *str;
 char dummy[LSTRING_MAX];
 FILE *fp;

 nb_ligne = fl_get_browser_maxline(dumpdata->datadumped);
 if (nb_ligne <= 0) return;

 sprintf(dummy,"%s_dump.txt",Dy->curConfig);
 str = fl_show_input("Save Dump in file: ",dummy);
 if (!str) return;

 fp = fopen(str,"w");

 if (fp==NULL)
    {printf("  Cannot open file to save dumped data.\n");
     return;
    }

 for (i=0; i<nb_ligne; i++)
    {str = fl_get_browser_line(dumpdata->datadumped, i+1);
     fprintf(fp,"%s\n",str);
    }

 fclose(fp);
 return;
}


/*------------------------------------------- DyFillFrameDump ---*/

void DyFillFrameDump (struct FrameH *frame, int level)
{
 int notaNumber, buffSize;
 char *buff;
 char *bufout;
 struct FrAdcData *adc;
 struct FrProcData *rec;
 FILE *fp;

 if (frame == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 notaNumber = 0;

 if (frame->rawData != NULL)
    {for (adc = frame->rawData->firstAdc; adc != NULL; adc = adc->next)
        {if (adc != NULL)  notaNumber += DyCheckData(adc->data);}
    }

 for (rec = frame->procData; rec != NULL; rec = rec->next)
    {if (rec != NULL)  notaNumber += DyCheckData(rec->data);}

 if (notaNumber == 0)
    {
     if (level < 10) 
        {if (level < 3) buffSize = 65536;
         else           buffSize = 540000;
         buff = (char *)malloc(buffSize);
         bufout = FrameDumpToBuf(frame, level, buff, buffSize);
         if (bufout)  DyXformAddInDump (bufout);
/*
FrameDump(frame,stdout,level);
printf("\n\n\n***********************************************************\n");
printf("***********************************************************\n");
printf("***********************************************************\n\n\n");
printf("bufout=%s\n", bufout);
printf("\n\n\n***********************************************************\n");
printf("***********************************************************\n");
printf("***********************************************************\n\n\n");
printf("buff=%s\n", buff);
*/
        free(buff);
        }

     else
        {fp = fopen(DYDUMP_FILE,"w");
         FrameDump(frame,fp,level);
         fl_load_browser(dumpdata->datadumped,DYDUMP_FILE);
         fclose(fp);
         unlink(DYDUMP_FILE);
        }
    }

 return;
}


/*-------------------------------------------------- DyCheckData ----*/
int DyCheckData (struct FrVect *data)
{
 int i, flag, notValid;
 double value;

 notValid = 0;
 flag     = 0;

 if (data == NULL) return(0);

 for (i=0; i<data->nData; i++)
    {
     if (data->type == FR_VECT_4R)  value = data->dataF[i];
     if (data->type == FR_VECT_8R)  value = data->dataD[i];
     if (data->type == FR_VECT_C)   value = data->data[i];
     if (data->type == FR_VECT_2S)  value = data->dataS[i];
     if (data->type == FR_VECT_8S)  value = data->dataL[i];
     if (data->type == FR_VECT_4S)  value = data->dataI[i];

     if (isnan(value))
        {if (flag == 0) printf("DyCheckData: Got NaN at index %d of vector %s (type %d)\n", i,data->name,data->type);
         flag     = 1;
         notValid = 1;
        }
     else
        {flag = 0;}
    }
			 
 return(notValid);
}



/*-------------------------------------------- DyFrVectDumpToBuf ----*/

char *DyFrVectDumpToBuf (struct FrVect *vect, char *buf, long dumplevel)
{
 int i, imax;
 char *dC;
 short  *dS;
 int    *dI;  
 long   *dL; 
 float  *dF, ratio;  
 double *dD;
 unsigned char *dU;
 unsigned short *dUS;
 unsigned int *dUI;
 unsigned long *dUL; 

 if (buf  == NULL) return(NULL);
 if (vect == NULL) return(NULL);
 if (dumplevel < 1) return(NULL);

 strcpy(buf," ");

 sprintf(buf,"%s  Vector:%s ndata=%d nDim=%d", buf, vect->name, vect->nData, vect->nDim);

 if (vect->unitY != NULL) 
    {sprintf(buf,"%s dataUnits=%s", buf, vect->unitY);}
 
 for (i=0; i<vect->nDim; i++)
    {sprintf(buf,"%s      dimension=%d nx=%10d dx=%g", buf,i,vect->nx[i], vect->dx[i]);
     if (vect->unitX[i] != NULL)
        {sprintf(buf,"%s unit%d=%s\n",buf, i, vect->unitX[i]);}
     else
        {sprintf(buf,"%s unit%d=a.u.\n",buf,i);}
    }

 if (vect->compress == 0)
    {
     if (vect->type == FR_VECT_4R) 
        {dF = (float *) vect->data;
         if (dumplevel == 2)
            {sprintf(buf,"%s(F) %e %e %e ...\n",buf, dF[0],dF[1],dF[2]);}
         if (dumplevel == 3)
            {sprintf(buf,"%s(F) ",buf);
             if (vect->nData/10 < 3) imax=vect->nData;
             else                    imax = vect->nData/10;
             for (i=0; i<imax; i++)
                 {sprintf(buf,"%s%e ", buf, dF[i]);
                  if (i>0 && i%10==0) sprintf(buf,"%s\n",buf);
                 }
             sprintf(buf,"%s...\n",buf);
            }
         if (dumplevel >3)
            {sprintf(buf,"%s(F) ",buf);
             for (i=0; i<vect->nData; i++)
                 {sprintf(buf,"%s%e ", buf, dF[i]);
                  if (i>0 && i%10==0) sprintf(buf,"%s\n",buf);
                 }
             sprintf(buf,"%s...\n",buf);
            }
        }
     else if(vect->type == FR_VECT_8R)  
        {dD = (double *) vect->data;
         sprintf(buf,"%s(D) %e %e %e ...\n",buf, dD[0],dD[1],dD[2]);
        }
     else if(vect->type == FR_VECT_C)  
        {dC = vect->data;
         sprintf(buf,"%s(C) %d %d %d %d %d ...\n", 
                             buf, dC[0],dC[1],dC[2],dC[3],dC[4]);}
     else if(vect->type == FR_VECT_2S)  
        {dS = (short *) vect->data;
         sprintf(buf,"%s(S) %d %d %d %d %d ...\n", 
                            buf, dS[0],dS[1],dS[2],dS[3],dS[4]);}
     else if(vect->type == FR_VECT_8S)  
        {dL = (long *) vect->data;
         sprintf(buf,"%s(L) %ld %ld %ld ...\n",buf, dL[0],dL[1],dL[2]);
        }
     else if(vect->type == FR_VECT_4S)  
        {dI = (int *) vect->data;
         sprintf(buf,"%s(I) %d %d %d ...\n",buf, dI[0],dI[1],dI[2]);
        }
     else if(vect->type == FR_VECT_1U)  
        {dU = (unsigned char *)vect->dataU;
         sprintf(buf,"%s(C) %d %d %d ...\n",buf, dU[0],dU[1],dU[2]);
        }
     else if(vect->type == FR_VECT_2U)  
        {dUS = (unsigned short *) vect->data;
         sprintf(buf,"%s(S) %d %d %d ...\n",buf, dUS[0],dUS[1],dUS[2]);
        }
     else if(vect->type == FR_VECT_8U)  
        {dUL = (unsigned long *) vect->data;
         sprintf(buf,"%s(L) %ld %ld %ld ...\n",buf, dUL[0],dUL[1],dUL[2]);
        }
     else if(vect->type == FR_VECT_4U)  
        {dUI = (unsigned int *) vect->data;
         sprintf(buf,"%s(I) %d %d %d ...\n",buf, dUI[0],dUI[1],dUI[2]);
        }
     else if(vect->type == FR_VECT_C8) 
        {dF = (float *) vect->data;
         sprintf(buf,"%s(CF) (%e,%e) (%e,%e) ...\n",buf, dF[0],dF[1],dF[2],dF[3]);
        }
     else if(vect->type == FR_VECT_C16) 
        {dD = (double *) vect->data;
         sprintf(buf,"%s(CD) (%e,%e) (%e,%e) ...\n",buf, dD[0],dD[1],dD[2],dD[3]);
        }
     else 
        {sprintf(buf,"%sunknown type: %d \n",buf,vect->type );}
    }

 else 
    {ratio = ((float) vect->nBytes) /(vect->nData*vect->wSize);
     sprintf(buf,"%s\n     the vector is %4.2f compressed (%d) nBytes=%d wSize=%d\n",
             buf,ratio,vect->compress,vect->nBytes, vect->wSize);
    }

 if (vect->next != NULL)
    {sprintf(buf,"%s  Attached information:\n",buf);
     DyFrVectDumpToBuf(vect->next, buf, dumplevel) ;
    }

 return(buf);
}


