

/*****************************************************************/
/*                                                               */
/*               FRAMES IN SHARED MEMORY                         */
/*                     LIBRARY                                   */
/* MODULE  : DyFdShm.c (taken from FdShm.c v4r13)                */
/* AUTHOR  : D. Verkindt, LAPP (verkindt@lapp.in2p3.fr)          */
/* DATE: 16/02/2005                                              */
/*                                                               */
/*   This library allows a server (for instance the Main Frame   */
/*   Builder producer) to create a shared memory segment and     */
/*   to put frames into this shared memory. It allows also       */
/*   several consumers (for instance the Main Frame Builder      */
/*   consumers) to read those frames in shared memory.           */
/*                                                               */
/*****************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#include "memwatch.h"
#endif


/*------------------------------------------ FdShmGetCsmN --------*/

long FdShmGetCsmN (struct ShmInfo *shm, char *name)
{
 long csmN,i;

 csmN = -1;

 if (shm->nConsumer >= 0)
    {for (i=0; i<=shm->nConsumer; i++)
        {if (strcmp(name,shm->csmName[i]) == 0) {csmN = i; break;}}
    }

 return(csmN);
}


/*----------------------------------------- FdShmGetI ----------*/

long FdShmGetI (struct ShmInfo *shm, char *csmName)
{
 long i,csmN;

 csmN = FdShmGetCsmN (shm,csmName);

 if (csmN == 0)  i = shm->iwrite;
 else            i = shm->iread[csmN];

 return(i);
}



/*------------------------------------------ FdShmRemoveCsm --------*/

int FdShmRemoveCsm (struct ShmInfo *shm, int csmN)
{
 int j, stillAlive;

 if (csmN == 0) return(0);  /* do not remove the producer */

 stillAlive = kill (shm->csmPid[csmN],0);  /* do not remove an active process */
 if (stillAlive == 0  &&  shm->toBeRemoved[csmN] == 0) return(-1);

 shm->selfStatus = 1;

 if (shm->nConsumer > 1)
    {
     for (j=csmN; j<=shm->nConsumer-1; j++)
        {
         shm->iread[j]       = shm->iread[j+1];
         shm->csmPid[j]      = shm->csmPid[j+1];
         shm->toBeRemoved[j] = shm->toBeRemoved[j+1];
         strcpy(shm->csmName[j], shm->csmName[j+1]);
        }
    }
 shm->nConsumer--;
 shm->selfStatus = 0;

 return(csmN);
}


/*------------------------------------------ FdShmFree --------*/

void FdShmFree (struct ShmInfo *shm, char *name)
{
 long csmN;

 if (name == NULL  ||  shm == NULL) return;

 csmN = FdShmGetCsmN (shm,name);

 if (csmN == 0)
    {
     shmctl(shm->shmid, IPC_RMID, 0);
     printf ("FdShmFree> Shared memory segment marked for deletion\n");
    }
 else
    {
     shm->toBeRemoved[csmN] = 1;
     FdShmRemoveCsm (shm, csmN);
    }

 shmdt((void *)shm);

 return;
}


/*------------------------------------------ FdShmClose --------*/
/* only for consumers */

void FdShmClose (struct ShmInfo *shm, char *name)
{
 long csmN;

 if (name == NULL  ||  shm == NULL) return;

 csmN = FdShmGetCsmN (shm,name);

 if (csmN == 0)
    {
     return;
    }
 else
    {
     shm->toBeRemoved[csmN] = 1;
    }

 shmdt((void *)shm);

 return;
}



/*-------------------------------------------  FdShmOpen -------*/
/* only for consumers */

struct ShmInfo *FdShmOpen (char *idName, char *name)
{
 struct ShmInfo *temp;
 void *segptr;
 key_t key;
 int shmid;

 key = FdShmMakeKey(idName);

 if ((shmid = shmget(key, 0, 0)) == -1)  return(NULL);

 /*--- Attach (map) the shared memory segment into the current process ---*/

 if ((segptr = shmat(shmid, 0, 0)) == (void *)-1)
    {
     printf("FdShmOpen> Could not attach Shm segment shmid=%d\n", shmid);
     return(NULL);
    }

 if (segptr == NULL)
    {
     printf("FdShmOpen> shm segptr is NULL\n");
     return(NULL);
    }

 temp = (struct ShmInfo *)segptr;

 FdShmCsmIni (temp, name);

 if (temp->nConsumer > SHM_NOUTMAX)
    {
     printf("FdShmNew> Max nbr of consumer is %d. Cannot add %s\n",SHM_NOUTMAX, name);
     temp->nConsumer--;
     return(NULL);
    }

 return(temp);
}


/*------------------------------------------- FdShmMakeKey -------*/

int FdShmMakeKey (char *name)
{
 int i, length, key, val, nbytes;
 
 length = strlen(name);

 key = 0;
 for (i=0; i<length; i=i+4)
     {val = 0;
      if (length-i < 4) nbytes=length-i;
      else              nbytes = 4;
      memcpy(&val, &name[i], nbytes);
      key += val;
      if (key < 0) key += INT_MAX;
     }

 return(key);
}


/*-------------------------------------------  FdShmCsmIni -------*/

void FdShmCsmIni (struct ShmInfo *temp, char *name)
{
 int csmN, i;

 csmN = 0;
 for (i=1; i<=temp->nConsumer; i++)
    {if (strcmp(name, temp->csmName[i]) == 0) {csmN = i; break;}
    }

 if (csmN > 0)
    {temp->csmPid[csmN]      = getpid();
     temp->toBeRemoved[csmN] = 0;
    }
 else
    {
     temp->nConsumer++;
     strcpy(temp->csmName[temp->nConsumer], name);
     temp->toBeRemoved[temp->nConsumer] = 0;
     temp->csmPid[temp->nConsumer]      = getpid();
     temp->iread[temp->nConsumer]       = temp->iwrite;
     if (temp->iwrite < temp->nFrameMax)
        {temp->iread[temp->nConsumer] = 0;}
     else
        {temp->iread[temp->nConsumer] = temp->iwrite - temp->nFrameMax + 1;}
    }

 if (temp->nConsumer <= SHM_NOUTMAX)
    {if (csmN > 0)
        {
         printf("FdShmCsmIni> Consumer %s (%ld) back in the list. Positionned at iread=%ld\n",
                 temp->csmName[temp->nConsumer], temp->nConsumer, temp->iread[temp->nConsumer]);
        }
     else
        {
         printf("FdShmCsmIni> New consumer %s (%ld) added to the list. Positionned at iread=%ld\n",
                 temp->csmName[temp->nConsumer], temp->nConsumer, temp->iread[temp->nConsumer]);
        }
    }

 return;
}




/*------------------------------------------- FdShmRead -------------*/

struct FrameH *FdShmRead (struct ShmInfo *shm, char *name, long compress, int *nBytes)
{
 struct FrameH *frame;
 long csmN, address, iread, iwrite;
 char *debut;

 if (shm == NULL) return(NULL);

 csmN = FdShmGetCsmN (shm,name);

 iwrite = shm->iwrite;

 /*--- don't try to read a frame currently written ---*/
 if (shm->iread[csmN] >= iwrite)  {*nBytes = 0; return(NULL);}

 /*--- if too late with respect to producer, go behind it (skipping nFrameMax frames) ---*/
 if (shm->iread[csmN] <= iwrite - shm->nFrameMax)
    {
     printf ("FdShmRead> late in reading shm (%d/%d). Skeeping %d frames, I go to iread=%d\n",
                    shm->iread[csmN], iwrite, iwrite-shm->iread[csmN], iwrite-1);
     shm->iread[csmN] = iwrite - 1;
    }

 /*--- ok, read the frame in share memory ---*/

 iread = shm->iread[csmN] % shm->nFrameMax;

 *nBytes = shm->frameSize[iread];
 address = shm->frameSh[iread] + (long) shm - (long) shm->segptr;


 if (*nBytes <= 0 || address == (long)NULL) return(NULL);

 frame = FrameReadFromBuf((char *)address, *nBytes, compress);

 if (frame == NULL) *nBytes = 0;

 shm->iread[csmN]++;

 return(frame);
}
