/**************************************************/
/*                                                */
/* File: DyOutput.c                               */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h>
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*------------------------------------------- DyOutputDefault ---*/

void DyOutputDefault()
{
 strcpy(Dy->output_tag,"Qc_Moni* Alp_Main* Alp_Cali* Pr_B1_ACp h*");
 strcpy(Dy->output_dir, DYOUTPUTDIR);
 Dy->output_nframe_perfile = 240;
 Dy->output_flag = OFF;

 return;
}


/*------------------------------------------- DyOutputWrite ---*/

void DyOutputWrite (struct FrameH *frame)
{
 char filename[LSTRING_MAX], dummy[LSTRING_MAX], date_heure[18];
 time_t utc;
 int error, nBytes;
 DIR  *dirp;

 if (Dy->output_flag == OFF) return;

 if (Dy->output_nframe_written == 0)
    {dirp = opendir(Dy->output_dir);
     if (dirp == NULL)
        {printf("Creating directory %s \n", Dy->output_dir);
         sprintf(dummy, "mkdir %s", Dy->output_dir);
         system(dummy);
        }
     Dy->output_nbytes_tot = 0;
    }

 if (Dy->output_nframe_written%2 == 1) fl_show_object(mainpanel->writing_data);
 else                                  fl_hide_object(mainpanel->writing_data);

 if (Dy->output_nframe_written == Dy->output_nframe_perfile)
    {FrFileOEnd(Dy->output_file);
     Dy->output_file = NULL;
     Dy->output_nframe_written = 0;
    }

 if (Dy->output_file == NULL)
    {utc = frame->GTimeS + FRGPSTAI - frame->ULeapS + FRGPSOFF;
     strftime(date_heure,18,"%d-%b-%Y-%Hh%M",gmtime(&utc));
     sprintf(filename, "%s/Dy-%d-%s-%dF.gwf", Dy->output_dir, frame->GTimeS, date_heure, Dy->output_nframe_perfile);
     printf("DyOutputWrite> Writing output file %s \n", filename);
     Dy->output_file = FrFileONew(filename,-1);
    }

 nBytes = Dy->output_file->nBytes;

 error = FrameWrite (frame, Dy->output_file);

 if (error == 0)
    {Dy->output_nframe_written++;
     Dy->output_nbytes_tot += Dy->output_file->nBytes - nBytes;
     printf("DyOutputWrite> frame=%d  Input_framesize=%d  Output_framesize=%d  Written on disk=%.3f MB\n",
             frame, UFrCurFrameHSize, Dy->output_file->nBytes - nBytes, Dy->output_nbytes_tot/1024./1024.);
     if (Dy->output_nbytes_tot > 10.*1024.*1024.*1024.)
        {Dy->output_flag = OFF;
         printf("DyOutputWrite> Written %.1f kB in %s\n", Dy->output_nbytes_tot/1024., Dy->output_dir);
         DyInfoLine("Written already 10 GB of data! You cannot continue to output data");
         Dy->output_nbytes_tot = 0;
        }
    }

 return;
}


/*------------------------------------------- DyOutputClose ---*/

void DyOutputClose()
{
 FILE *fp;
 struct FrFile *iFile;
 char *output_files;
 char fflname[LSTRING_MAX], filename[LSTRING_MAX];
 DIR  *dirp;
 struct dirent *entry;
 int length, nfile;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->output_file)
    {FrFileOEnd(Dy->output_file);
     Dy->output_file = NULL;
    }

/***
 length = 0;
 dirp   = opendir(Dy->output_dir);
 while ((entry = readdir(dirp)) != NULL)
   {
    if (strncmp(entry->d_name,".",1) == 0) continue;
    if (strstr(entry->d_name,".gwf") == 0) continue;
    length += 1 + strlen(Dy->output_dir) + strlen(entry->d_name);
   }
 closedir(dirp);
 output_files = (char *)malloc(length+1);

 nfile = 0;
 dirp  = opendir(Dy->output_dir);
 while ((entry = readdir(dirp)) != NULL)
   {
    if (strncmp(entry->d_name,".",1) == 0) continue;
    if (strstr(entry->d_name,".gwf") == 0) continue;
    sprintf(filename,"%s/%s",Dy->output_dir,entry->d_name);
    if (nfile == 0) strcpy(output_files,filename);
    else            {strcat(output_files," "); strcat(output_files,filename);}
    nfile++;
   }
 closedir(dirp);

 if (nfile > 0)
    {sprintf(fflname,"%s/%s", Dy->output_dir, DYOUTPUTFFL);
     printf("DyOutpuClose> Building %s\n", fflname);
     printf("DyOutputClose> Output files: %s\n", output_files);
     fp = fopen(fflname,"w");
     iFile = FrFileINew(output_files);
     printf("DyOutputClose> Output files: iFile=%x  fp=%x\n", iFile, fp);
     FrFileIDumpT (iFile, fp, 1, NULL, 0., 1.e+12);
     FrFileIEnd(iFile);
     fclose(fp);
    }

 free(output_files);
***/

 fl_hide_object(mainpanel->writing_data);
 Dy->output_nframe_written = 0;

 return;
}



