
/**************************************************/
/*                                                */
/* File: DyHelp.c                                 */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"
#include "coffee.xpm"
#include "cristalBall.xpm"

#include "RVersion.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*                 **************************                 */
/*                 * HELPS, INIT and RESET  *                 */
/*                 **************************                 */

/*------------------------------------------ DyXformHelpPanel ---*/

void DyXformHelpPanel (FL_OBJECT *obj, long arg)
{
 FL_OBJECT *ob, *browser,*close;
 long xsize, ysize;
 char dummy[LSTRING_MAX];

 if (fl_form_is_visible(helppanel)) return;

 xsize      = 550;
 ysize      = 410;
 helppanel  = fl_bgn_form(FL_NO_BOX, xsize, ysize);
 ob         = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
 browser    = fl_add_browser(FL_HOLD_BROWSER,10,10,xsize-20,ysize-50,"Help");
    fl_set_object_color(browser,FL_LEFT_BCOL,FL_WHEAT);
    fl_set_object_lstyle(browser,FL_TIMESBOLD_STYLE);
 close = fl_add_button(FL_NORMAL_BUTTON,xsize-70,ysize-30,60,20,"Close");
    fl_set_object_color(close,FL_WHEAT,FL_COL1);
    fl_set_object_lstyle(close,FL_TIMESBOLD_STYLE);
    fl_set_object_callback(close,DyXformClosePanel,HELPPANEL);
 fl_end_form();

 fl_add_browser_line(browser,"\n");
 sprintf(dummy,"             dataDisplay (%s)\n ", DYVERSION);
 sprintf(dummy,"%s  using:  Fr(%s), Root(%s) , GUI Xforms (v%dr%d)",
                dummy, FrVersion, ROOT_RELEASE, FL_VERSION, FL_REVISION);
#ifndef NOCM
 sprintf(dummy,"%s , DyFd (%s)\n", dummy, DYFDVERSION);
#endif
 fl_add_browser_line(browser, dummy);
 fl_add_browser_line(browser,"********************   HELP SUMMARY   ***********************\n\n"
                             "  First, you must get some data (under frame or wave format).\n"
                             "  Use the 'Input Data' menu and choose a source of data      \n"
                             "  (for instance a 'Read File' or a 'Connect online').        \n"
                             "  Then, you will see in the bottom line of the mainpanel that\n"
                             "  a first frame has been read successfully. The data channels\n"
                             "  contained in the frame can now be selected for doing plots,\n"
                             "  audio output or dump of data, using 'Data and Plots' panel.\n\n");
 fl_add_browser_line(browser,"                              *********                      \n\n"
                             "  - For more help, you can look at the README file           \n");
 sprintf(dummy,"    in the /virgoApp/Dy/%s directory.                \n", DYVERSION);
 fl_add_browser_line(browser,dummy);
 fl_add_browser_line(browser,"  - For detailed help, you can use the 'Help ballons' button \n"
                             "    of the mainpanel. In this case, each field or button     \n"
                             "    shows a message each time the mouse passes on it.        \n");
 sprintf(dummy,"  - For help, see also paper documentation: /virgoApp/Dy/%s/doc/DyDoc.ps. \n\n", DYVERSION);
 fl_add_browser_line(browser,dummy);
 fl_add_browser_line(browser,"                              *********                     \n\n"
                             "  D. Verkindt                                               \n"
                             "  (verkindt@lapp.in2p3.fr)                                  \n\n");

 fl_set_form_atclose(helppanel, DyAtClose, "helppanel");
 DyXformRaisePanel(NULL,HELPPANEL);
 return;
}


/*------------------------------------------ DyXformCoffeePanel ---*/

void DyXformCoffeePanel (FL_OBJECT *obj, long arg)
{
 FL_OBJECT *ob;
 long xsize, ysize;

 if (fl_form_is_visible(coffeepanel)) return;

 xsize      = 670;
 ysize      = 190;
 coffeepanel= fl_bgn_form(FL_NO_BOX, xsize, ysize);
 ob = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
      fl_set_object_color(ob,FL_WHITE,FL_COL1);
 ob = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
 ob = fl_add_text(FL_NORMAL_TEXT,230,60,430,80,"And now, dataDisplay can also make the coffee!!");
/*
    fl_set_object_color(ob,FL_WHITE,FL_MCOL);
*/
    fl_set_object_lsize(ob,FL_MEDIUM_SIZE);
    fl_set_object_lalign(ob,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(ob,FL_TIMESBOLD_STYLE);
/*    fl_set_object_boxtype(ob,FL_DOWN_BOX); */
    ob = fl_add_button(FL_NORMAL_BUTTON,xsize-70,ysize-30,60,20,"Close");
    fl_set_object_color(ob,FL_WHEAT,FL_COL1);
    fl_set_object_lsize(ob,FL_NORMAL_SIZE);
    fl_set_object_lstyle(ob,FL_TIMESBOLD_STYLE);
    fl_set_object_callback(ob,DyXformClosePanel,COFFEEPANEL);
 ob = fl_add_pixmap(FL_NORMAL_PIXMAP,20,20,170,150,"");
 fl_set_pixmap_data(ob, coffee);
 fl_end_form();
 fl_set_form_atclose(coffeepanel, DyAtClose, "coffeepanel");

 DyXformRaisePanel(NULL,COFFEEPANEL);
 return;
}


/*------------------------------------------ DyXformSurprisePanel ---*/

void DyXformSurprisePanel (FL_OBJECT *obj, long arg)
{
 FL_OBJECT *ob;
 long xsize, ysize;

 if (fl_form_is_visible(coffeepanel)) return;

 xsize      = 670;
 ysize      = 190;
 coffeepanel= fl_bgn_form(FL_NO_BOX, xsize, ysize);
 ob = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
      fl_set_object_color(ob,FL_WHITE,FL_COL1);
 ob = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
 ob = fl_add_text(FL_NORMAL_TEXT,230,60,430,80,"Since 32nd of July 2004, dataDisplay can also read in the future!!");

    fl_set_object_color(ob,FL_WHITE,FL_MCOL);

    fl_set_object_lsize(ob,FL_MEDIUM_SIZE);
    fl_set_object_lalign(ob,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(ob,FL_TIMESBOLD_STYLE);
    fl_set_object_boxtype(ob,FL_DOWN_BOX);
    ob = fl_add_button(FL_NORMAL_BUTTON,xsize-70,ysize-30,60,20,"Close");
    fl_set_object_color(ob,FL_WHEAT,FL_COL1);
    fl_set_object_lsize(ob,FL_NORMAL_SIZE);
    fl_set_object_lstyle(ob,FL_TIMESBOLD_STYLE);
    fl_set_object_callback(ob,DyXformClosePanel,COFFEEPANEL);
 ob = fl_add_pixmap(FL_NORMAL_PIXMAP,20,20,170,150,"");
 fl_set_pixmap_data(ob, cristalBall);
 fl_end_form();
 fl_set_form_atclose(coffeepanel, DyAtClose, "coffeepanel");

 DyXformRaisePanel(NULL,COFFEEPANEL);
 return;
}


/*------------------------------------------ DyHelpMainPanel ---*/

void DyHelpMainPanel (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == mainpanel->inputMenu)
            fl_show_oneliner("To select a frames source is the first\n"
                             "action to do. You can read frames from\n"
                             "one file (Read FFL or Read File), \n"
                             "a Cm server (Connect Online),\n"
                             "a Data Distribution server (Connect to DataSender).\n"
                             "or read data from a shared memory (Read Shared Memory).\n"
                             "You can also read a whoile directory (Read Directory).\n"
                             "or read a set of files while they are written (Read Files Online).\n"
                             "Data in wave format can also be read (Read Wave File).\n"
                             "You can also connect on Ethernet with a fixed Cm Name (Use fixed CmName).\n"
                             "The current list of files can be checked with the item (Files List)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->configMenu)
            fl_show_oneliner("Menu to load or save the\n"
                             "current configuration in a file",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->userFuncPanel)
            fl_show_oneliner("To load sets of user's defined functions.\n"
                             "that takes several input channels. Output is user's defined",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->mainbrowser)
            fl_show_oneliner("Shows the panel where you can:\n"
                             "- select the data to plot,\n"
                             "- select the data to dump,\n"
                             "- manage, edit or remove plots,\n"
                             "- do operations with the channels...",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->dataDumped)
            fl_show_oneliner("Shows a dump of the full frame or of the channels selected",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->cleaning)
            fl_show_oneliner("Remove from the screen the various\n"
                             "plot windows created by the data display",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->resetAll)
            fl_show_oneliner("Put the data display in a 'just started' configuration",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->currentInput)
            fl_show_oneliner("Name of the current source of data",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->currentRun)
            fl_show_oneliner("Current run number",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->currentFrame)
            fl_show_oneliner("Current frame number",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->startRun)
            fl_show_oneliner("Put here the run number of the first frame\n"
                             "to read when button Start is pushed.\n"
                             "Useful only if no starting Time has been defined.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->startFrame)
            fl_show_oneliner("Put here the frame number of the first frame\n"
                             "to read when button Start is pushed.\n"
                             "Useful only if no starting Time has been defined.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->startTime)
            fl_show_oneliner("Put here the GPS time of first data to read\n"
                             "when button Start has been pushed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->startUTCdate)
            fl_show_oneliner("Show the starting UTC date in a standard format.\n"
                             "Put here the UTC date in some approximative time format\n"
                             "The starting GPS time will be automatically updated\n"
                             "and UTC date will be put into a standard format.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->duration)
            fl_show_oneliner("Put here the number of seconds\n"
                             "over which you want to read data \n"
                             "If this number is negative,\n"
                             "it will be interpreted as infinity.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->currentGTime)
            fl_show_oneliner("GPS time of the last frame read.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->UTCdate)
            fl_show_oneliner("UTC date of the last frame read.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == mainpanel->start)
            fl_show_oneliner("Start reading frames and displaying plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->pause)
            fl_show_oneliner("Pause the frames reading until button.\n"
                             "Continue, Next Refresh or Next Frame is pushed",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->continuer)
            fl_show_oneliner("Continue to read data.\n"
                             "Active only after button Pause has been pushed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->nextUpdate)
            fl_show_oneliner("Continue to read data up to the next plot refresh.\n"
                             "Active only after button Pause has been pushed",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->nextFrame)
            fl_show_oneliner("To continue only up to the next frame.\n"
                             "Use after button Pause has been pushed",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->stop)
            fl_show_oneliner("Stop reading data and displaying plots\n"
                             "Disconnect from Ethernet or Shared Memory if needed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->output)
            fl_show_oneliner("To write on disk the frames received.\n"
                             "You can select the channels to write and the name of the output directory.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->setRefPlots)
            fl_show_oneliner("Save in a ROOT file the plots currently displayed\n"
                             "Those plots can then be used as reference plots to be\n"
                             "superposed to the plots displayed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->getRefPlots)
            fl_show_oneliner("Get from a ROOT file a set of reference plots\n"
                             "to be superposed to the plots currently displayed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->editRefPlots)
            fl_show_oneliner("Open a panel to manage to list of reference plots\n"
                             "in memory. Useful also to choose the color of those plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->options)
            fl_show_oneliner("Open panel to select various options for\n"
                             "plots display or data read.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->debuglvl)
            fl_show_oneliner("Open panel to choose debug type and level\n"
                             "or to do some checks",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->infoLine)
            fl_show_oneliner("Shows the last action made\n"
                             "or the current status of the data display",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->latency)
            fl_show_oneliner("Difference between the current GPS time and the data GPS time",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->helpme)
            fl_show_oneliner("Shows a minimal help that explains how to\n"
                             "use the data display and points to other helps.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->currentSize)
            fl_show_oneliner("Size in bytes of the last frame read",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->checks)
            fl_show_oneliner("Output to stdout the current configuration:\n"
                             "Input data type, list of clocks, plots parameters...",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->scanplots)
            fl_show_oneliner("Slider to move along the X axis of the last zoomed plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->datenow)
            fl_show_oneliner("Button that sets the start UTC date and GPS time\n"
                             "to the current time",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->maxduration)
            fl_show_oneliner("Button that sets the duration of data read\n"
                             "to the max time window of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (mainpanel->inputMenu,     DyHelp);
     fl_set_object_posthandler (mainpanel->configMenu,    DyHelp);
     fl_set_object_posthandler (mainpanel->userFuncPanel, DyHelp);
     fl_set_object_posthandler (mainpanel->mainbrowser,   DyHelp);
     fl_set_object_posthandler (mainpanel->dataDumped,    DyHelp);
     fl_set_object_posthandler (mainpanel->resetAll,      DyHelp);
     fl_set_object_posthandler (mainpanel->cleaning,      DyHelp);
     fl_set_object_posthandler (mainpanel->currentRun,    DyHelp);
     fl_set_object_posthandler (mainpanel->currentFrame,  DyHelp);
     fl_set_object_posthandler (mainpanel->startRun,      DyHelp);
     fl_set_object_posthandler (mainpanel->startFrame,    DyHelp);
     fl_set_object_posthandler (mainpanel->startTime,     DyHelp);
     fl_set_object_posthandler (mainpanel->startUTCdate,  DyHelp);
     fl_set_object_posthandler (mainpanel->currentGTime,  DyHelp);
     fl_set_object_posthandler (mainpanel->UTCdate,       DyHelp);
     fl_set_object_posthandler (mainpanel->duration,      DyHelp);
     fl_set_object_posthandler (mainpanel->currentInput,  DyHelp);
     fl_set_object_posthandler (mainpanel->start,         DyHelp);
     fl_set_object_posthandler (mainpanel->pause,         DyHelp);
     fl_set_object_posthandler (mainpanel->continuer,     DyHelp);
     fl_set_object_posthandler (mainpanel->nextUpdate,    DyHelp);
     fl_set_object_posthandler (mainpanel->nextFrame,     DyHelp);
     fl_set_object_posthandler (mainpanel->stop,          DyHelp);
     fl_set_object_posthandler (mainpanel->output,        DyHelp);
     fl_set_object_posthandler (mainpanel->setRefPlots,   DyHelp);
     fl_set_object_posthandler (mainpanel->getRefPlots,   DyHelp);
     fl_set_object_posthandler (mainpanel->editRefPlots,  DyHelp);
     fl_set_object_posthandler (mainpanel->options,       DyHelp);
     fl_set_object_posthandler (mainpanel->debuglvl,      DyHelp);
     fl_set_object_posthandler (mainpanel->infoLine,      DyHelp);
     fl_set_object_posthandler (mainpanel->latency,       DyHelp);
     fl_set_object_posthandler (mainpanel->helpme,        DyHelp);
     fl_set_object_posthandler (mainpanel->currentSize,   DyHelp);
     fl_set_object_posthandler (mainpanel->checks,        DyHelp);
     fl_set_object_posthandler (mainpanel->scanplots,     DyHelp);
     fl_set_object_posthandler (mainpanel->datenow,       DyHelp);
     fl_set_object_posthandler (mainpanel->maxduration,   DyHelp);
    }
 return;
}


/*------------------------------------------ DyHelpMainBrowser ---*/

void DyHelpMainBrowser (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == mainbrowser->sms)
            fl_show_oneliner("List of Slow Monitoring\n"
                             "Stations (SMS) seen in the first frame read.\n"
                             "It is updated each time you push button Stop.\n"
                             "You can select one or several channel\n"
                             "and press for instance TIME button\n"
                             "if you want a time plot of all those channels\n"
                             "To deselect a line, click on it. To deselect all\n"
                             "the lines, click on 'deselect sms data'.\n"
                             "If two lines are selected and a 2D or TrFct plot\n"
                             "is chosen, X is the upper and Y is the lower line.\n"
                             "If only one line was selected, it is put in X\n"
                             "and next line selected will be Y",
                              ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->deselectsms)
            fl_show_oneliner("Allows to deselect all the channels selected.\n"
                             "Also, clicking on a selected channel deselects it.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->inputSms)
            fl_show_oneliner("Allows to search for a SMS variable name.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == mainbrowser->adc)
            fl_show_oneliner("List of adc's channels\n"
                             "seen in the first frame read.\n"
                             "It is updated each time you push button Stop.\n"
                             "You can select one or several channels\n"
                             "and press for instance TIME button\n"
                             "if you want a time plot of all those channels.\n"
                             "To deselect a line, click on it. To deselect all\n"
                             "the lines, click on 'deselect ADC'.\n"
                             "If two lines are selected and a 2D or TrFct plot\n"
                             "is chosen, X is the upper and Y is the lower line.\n"
                             "If only one line was selected, it is put in X\n"
                             "and next line selected will be Y", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->deselectadc)
            fl_show_oneliner("Allows to deselect all the channels selected.\n"
                             "Also, clicking on a selected channel deselects it.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->inputAdc)
            fl_show_oneliner("Allows to search for an ADC channel name.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == mainbrowser->proc)
            fl_show_oneliner("List of processed data channels\n"
                             "seen in the first frame read.\n"
                             "It is updated each time you push button Stop.\n"
                             "You can select one or several channels\n"
                             "and press for instance TIME button\n"
                             "if you want a time plot of all those channels.\n"
                             "To deselect a line, click on it. To deselect all\n"
                             "the lines, click on 'deselect ADC'.\n"
                             "If two lines are selected and a 2D or TrFct plot\n"
                             "is chosen, X is the upper and Y is the lower line.\n"
                             "If only one line was selected, it is put in X\n"
                             "and next line selected will be Y", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->deselectproc)
            fl_show_oneliner("Allows to deselect all the channels selected.\n"
                             "Also, clicking on a selected channel deselects it.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->inputProc)
            fl_show_oneliner("Allows to search for a PROCessed channel name.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == mainbrowser->sim)
            fl_show_oneliner("List of simulated data's channels\n"
                             "seen in the first frame read.\n"
                             "It is updated each time you push button Stop.\n"
                             "You can select one or several channels\n"
                             "and press for instance TIME button\n"
                             "if you want a time plot of all those channels.\n"
                             "To deselect a line, click on it. To deselect all\n"
                             "the lines, click on 'deselect ADC'.\n"
                             "If two lines are selected and a 2D or TrFct plot\n"
                             "is chosen, X is the upper and Y is the lower line.\n"
                             "If only one line was selected, it is put in X\n"
                             "and next line selected will be Y", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->deselectsim)
            fl_show_oneliner("Allows to deselect all the channels selected.\n"
                             "Also, clicking on a selected channel deselects it.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->inputSim)
            fl_show_oneliner("Allows to search for a SIM channel name.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == mainbrowser->plotTime)
            fl_show_oneliner("Create, for the data selected, a time\n"
                             "plot with suitable default parameters",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotCorr)
            fl_show_oneliner("Create, for the data selected, a Correlation Function\n"
                             "plot with suitable default parameters.\n"
                             "Such plot shows versus time\n"
                             "the cross-correlation of the two signals.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotFFT)
            fl_show_oneliner("Create, for the data selected, a Spectrum\n"
                             "plot with suitable default parameters",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotTrFct)
            fl_show_oneliner("Create, for the data selected, a Transfert Function\n"
                             "plot with suitable default parameters.\n"
                             "Transfert Function is FFT(X)/FFT(Y).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotCohe)
            fl_show_oneliner("Create, for the data selected, a Coherence Function\n"
                             "plot with suitable default parameters.\n"
                             "Coherence function has module 1 if the\n"
                             "two signals are correlated.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plot1D)
            fl_show_oneliner("Create, for the data selected, a 1D\n"
                             "plot with suitable default parameters",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plot2D)
            fl_show_oneliner("Create, for the data selected, a 2D\n"
                             "plot with suitable default parameters.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotFFTTime)
            fl_show_oneliner("Create, for the data selected, a time-frequency\n"
                             "plot with suitable default parameters.\n"
                             "It is based on a FFT made over timeWindow seconds\n"
                             "and moving by steps of timeStep seconds\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotTrFctTime)
            fl_show_oneliner("Create, for the data selected, a time-transfer_function\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the transfer function module\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotCoheTime)
            fl_show_oneliner("Create, for the data selected, a time-coherence_function\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the coherence function module\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plot1DTime)
            fl_show_oneliner("Create, for the data selected, a time-1d distribution\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the 1D distribution of the data\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotRawTime)
            fl_show_oneliner("Create, for the data selected, a time-raw\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the raw\n"
                             "vector taken from the frame (like spectra)\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotRaw)
            fl_show_oneliner("Create, for the data selected, a raw\n"
                             "plot with suitable default parameters, where\n"
                             "are plotted the data of the raw vector taken \n"
                             "from the frame (like spectra or camera images)\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotAudio)
            fl_show_oneliner("Create, for the data selected, a virtual plot used\n"
                             "to hear online or offline the selected data.\n"
                             "By default, a wave audio file will be written on disk.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->dump)
            fl_show_oneliner("Puts the selected data into the list of data to dump",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plots)
            fl_show_oneliner("List of plots created.\n"
                             "You can remove a plot from the list\n"
                             "or select it and edit its parameters.\n"
                             "The 'Edit Clone' button allows to automatically\n"
                             "clone your parameters change to all the other\n"
                             "plots of the same type as the one selected",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->editPlot)
            fl_show_oneliner("Open an edition panel for the selected plot.\n"
                             "This panel shows you the parameters\n"
                             "of the plot that you can modify",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->editPlotClone)
            fl_show_oneliner("Open an edition panel for the selected plot.\n"
                             "This panel shows the parameters of this plot.\n"
                             "But every change you make is automatically\n"
                             "cloned to all the other plots of\n"
                             "the same type as the one selected",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->editGenePar)
            fl_show_oneliner("Open an edition panel for the parameters.\n"
                             "common to almost all the plots.\n"
                             "Any change of parameter in this panel is propagated\n"
                             "to all the plots\n"
                             "Remark: Time window in this panel is\n"
                             "automatically changed to get nBin=a power of 2.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->removePlot)
            fl_show_oneliner("Remove the selected plot from the list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->savePlots)
            fl_show_oneliner("According to the filename extension, save in a\n"
                             "root, jpg or ps formatted file all the last plots made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->clear_plotbrowser)
            fl_show_oneliner("Remove ALL the plots from the list\n"
                             "Take care! There is no undo in the dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->userFct)
            fl_show_oneliner("List of current operations that can be performed\n"
                             "on the signals. First select some input signals in\n"
                             "the channels browsers. Then choose an operation.\n"
                             "It will create a new signal that you can use\n"
                             "to do plots. Note that you can also create your own\n"
                             "operation by using the item -Operation String- in this menu.\n"
                             "In this case, if you write the string\n"
                             "myop=sin(Pr_B1_ACq)*Pr_B1_ACp , it will create a new operation\n"
                             "named myop, with a corresponding file myop.C and a corresponding\n"
                             "new proc data channel **myop,Pr_B1_ACq,Pr_B1_ACp that you can\n"
                             "use for plots like the others.\n"
                             "The operation myop is than available in the 'User Fct' menu and it\n"
                             "can be used with other channels as input.\n"
                             "And you can create an other proc data like, for instance\n"
                             "**myop,Gx_BS_ty,Gx_MC_tx.\n"
                             "You can also put parameters by using the item -Set Parameters-\n"
                             "in this menu, after having selected the desired proc data\n"
                             "channels. Then, if the operations creating those proc data\n"
                             "channels contains par[0], par[1], etc... the values provided\n"
                             "in the parameters string (for instance 2.17_3.2_0.711) replace\n"
                             "the par[i] variables.\n"
                             "Finally, you can also modify the file myop.C as you want\n"
                             "and just type Start to get the plots of variables **myop\n"
                             "with the operation changed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->superposed)
            fl_show_oneliner("This button superpose the plots you have selected in\n"
                             "the browser. A sign (s) appears then in front of each plot\n"
                             "superposed, except the first one.\n"
                             "Selecting just one plot which has a sign (s) and clicking\n"
                             "on this button will remove it from the superposition.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->unsame)
            fl_show_oneliner("This button removes the superposition\n"
                             "for the selected plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->exchange)
            fl_show_oneliner("This button exchanges the variables in each of the 2D,\n"
                             " TRFCT or COHE plot selected in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->updateChList)
            fl_show_oneliner("This button updates the list of channels in all browser\n"
                             "according to the Start GPS time put in the mainpanel.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->copyPlot)
            fl_show_oneliner("Create a new plot with same\n"
                             "parameters as the plot selected.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->transform)
            fl_show_oneliner("Transform the selected plots into an other type of plots\n"
                             "with same number of channels. New type plot is set with default parameters.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->up)
            fl_show_oneliner("Move upwards in the list the selected plots.\n"
                             "The move takes into account the superposed plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->down)
            fl_show_oneliner("Move downwards in the list the selected plots.\n"
                             "The move takes into account the superposed plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->show)
            fl_show_oneliner("The selected plots are shown while running.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->hide)
            fl_show_oneliner("The selected plots are hidden while running.\n"
                             "This allow to temporarily hide some plots from the display.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->deselectPlots)
            fl_show_oneliner("Deselect all the plots currently selected.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->colrow)
            fl_show_oneliner("Allows to define the displayed plots configuration.\n"
                             "New row and columns configuration is active\n"
                             "only after Start button has been pushed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->trigger)
            fl_show_oneliner("This menu allows to define various type of triggers\n"
                             "thats condition the display of plots:\n"
                             "Item -Triggers from channels- allows to define a condition\n"
                             "based on logical and mathematical operations on the channels.\n"
                             "For instance: 'Pr_B5_DC>0.05 AND Qc_Moni_ScienceMode=1'\n"
                             "Then, plots will be computed only for data that verify this condition.\n"
                             "Item -Trigger from ASCII file- allows to read from an ASCII file\n"
                             "a list of start GPS time and duration (or stop GPS time)\n"
                             "defining the chunck of data to be used for the plots.\n"
                             "Item - Trigger from FrEvents- allows to use an FrEvent signal\n"
                             "to define the chunck of signal to be used for the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->filterSms)
            fl_show_oneliner("Allows to put as first items of browser's list a selection of channels.\n"
                             "For instance '*BMS*DC' will show first all the channels containing\n"
                             "BMS in their name and finishing with DC",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->filterAdc)
            fl_show_oneliner("Allows to put as first items of browser's list a selection of channels.\n"
                             "For instance '*BMS*DC' will show first all the channels containing\n"
                             "BMS in their name and finishing with DC",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->filterProc)
            fl_show_oneliner("Allows to put as first items of browser's list a selection of channels.\n"
                             "For instance '*BMS*DC' will show first all the channels containing\n"
                             "BMS in their name and finishing with DC",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->filterSim)
            fl_show_oneliner("Allows to put as first items of browser's list a selection of channels.\n"
                             "For instance '*BMS*DC' will show first all the channels containing\n"
                             "BMS in their name and finishing with DC",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (mainbrowser->sms,               DyHelp);
     fl_set_object_posthandler (mainbrowser->deselectsms,       DyHelp);
     fl_set_object_posthandler (mainbrowser->inputSms,          DyHelp);
     fl_set_object_posthandler (mainbrowser->adc,               DyHelp);
     fl_set_object_posthandler (mainbrowser->deselectadc,       DyHelp);
     fl_set_object_posthandler (mainbrowser->inputAdc,          DyHelp);
     fl_set_object_posthandler (mainbrowser->proc,              DyHelp);
     fl_set_object_posthandler (mainbrowser->deselectproc,      DyHelp);
     fl_set_object_posthandler (mainbrowser->inputProc,         DyHelp);
     fl_set_object_posthandler (mainbrowser->sim,               DyHelp);
     fl_set_object_posthandler (mainbrowser->deselectsim,       DyHelp);
     fl_set_object_posthandler (mainbrowser->inputSim,          DyHelp);
     fl_set_object_posthandler (mainbrowser->plotTime,          DyHelp);
     fl_set_object_posthandler (mainbrowser->plotCorr,          DyHelp);
     fl_set_object_posthandler (mainbrowser->plotFFT,           DyHelp);
     fl_set_object_posthandler (mainbrowser->plotTrFct,         DyHelp);
     fl_set_object_posthandler (mainbrowser->plotCohe,          DyHelp);
     fl_set_object_posthandler (mainbrowser->plotFFTTime,       DyHelp);
     fl_set_object_posthandler (mainbrowser->plotTrFctTime,     DyHelp);
     fl_set_object_posthandler (mainbrowser->plotCoheTime,      DyHelp);
     fl_set_object_posthandler (mainbrowser->plot1DTime,        DyHelp);
     fl_set_object_posthandler (mainbrowser->plotRawTime,       DyHelp);
     fl_set_object_posthandler (mainbrowser->plotRaw,           DyHelp);
     fl_set_object_posthandler (mainbrowser->plot1D,            DyHelp);
     fl_set_object_posthandler (mainbrowser->plot2D,            DyHelp);
     fl_set_object_posthandler (mainbrowser->plotAudio,         DyHelp);
     fl_set_object_posthandler (mainbrowser->dump,              DyHelp);
     fl_set_object_posthandler (mainbrowser->plots,             DyHelp);
     fl_set_object_posthandler (mainbrowser->editPlot,          DyHelp);
     fl_set_object_posthandler (mainbrowser->editPlotClone,     DyHelp);
     fl_set_object_posthandler (mainbrowser->editGenePar,       DyHelp);
     fl_set_object_posthandler (mainbrowser->removePlot,        DyHelp);
     fl_set_object_posthandler (mainbrowser->savePlots,         DyHelp);
     fl_set_object_posthandler (mainbrowser->clear_plotbrowser, DyHelp);
     fl_set_object_posthandler (mainbrowser->close,             DyHelp);
     fl_set_object_posthandler (mainbrowser->userFct,           DyHelp);
     fl_set_object_posthandler (mainbrowser->superposed,        DyHelp);
     fl_set_object_posthandler (mainbrowser->unsame,            DyHelp);
     fl_set_object_posthandler (mainbrowser->exchange,          DyHelp);
     fl_set_object_posthandler (mainbrowser->updateChList,      DyHelp);
     fl_set_object_posthandler (mainbrowser->copyPlot,          DyHelp);
     fl_set_object_posthandler (mainbrowser->transform,         DyHelp);
     fl_set_object_posthandler (mainbrowser->up,                DyHelp);
     fl_set_object_posthandler (mainbrowser->down,              DyHelp);
     fl_set_object_posthandler (mainbrowser->show,              DyHelp);
     fl_set_object_posthandler (mainbrowser->hide,              DyHelp);
     fl_set_object_posthandler (mainbrowser->deselectPlots,     DyHelp);
     fl_set_object_posthandler (mainbrowser->colrow,            DyHelp);
     fl_set_object_posthandler (mainbrowser->trigger,           DyHelp);
     fl_set_object_posthandler (mainbrowser->filterSms,         DyHelp);
     fl_set_object_posthandler (mainbrowser->filterAdc,         DyHelp);
     fl_set_object_posthandler (mainbrowser->filterProc,        DyHelp);
     fl_set_object_posthandler (mainbrowser->filterSim,         DyHelp);
    }

 return;
}



/*------------------------------------------ DyHelpDumpData ---*/

void DyHelpDumpData (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dumpdata->datatodump)
            fl_show_oneliner("List of data\n"
                             "that will be dumped once the\n"
                             "Start button is pressed",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dumpdata->remove_datatodump)
            fl_show_oneliner("Removes the selected data\n"
                             "from the list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dumpdata->clear_datatodump)
            fl_show_oneliner("Removes all the data\n"
                             "from the list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dumpdata->save,                 DyHelp);
     fl_set_object_posthandler (dumpdata->remove_datatodump, DyHelp);
     fl_set_object_posthandler (dumpdata->clear_datatodump,  DyHelp);
     fl_set_object_posthandler (dumpdata->close,             DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpPlottime ---*/

void DyHelpPlottime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plottime->name)
            fl_show_oneliner("Name of the plot.\n"
                             "By default, it is made of the variable name \n"
                             "with the sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->sampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->tstep)
            fl_show_oneliner("Size (in % of time window size)\n"
                             "of the time window steps",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->yMin)
            fl_show_oneliner("The lower y edge of the plot. To get an automatic\n"
                             "set of lower and upper edges, put 0",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->yMax)
            fl_show_oneliner("The upper y edge of the plot. To get an automatic\n"
                             "set of lower and upper edges, put 0",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plottime->yoffset)
            fl_show_oneliner("A user defined offset that will be added to\n"
                             "the plot on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plottime->yscale)
            fl_show_oneliner("A user defined scaling factor that will rescale\n"
                             "the plot on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->autoScale)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->realUnits)
            fl_show_oneliner("This button, if switched on, sets the Y units\n"
                             "in real units instead of ADC counts",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->ok)
            fl_show_oneliner("Acknowledges the changes you made\n"
                             "and closes the panel",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancels any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plottime->name,      DyHelp);
     fl_set_object_posthandler (plottime->bydefault, DyHelp);
     fl_set_object_posthandler (plottime->tsize,     DyHelp);
     fl_set_object_posthandler (plottime->sampFreq,  DyHelp);
     fl_set_object_posthandler (plottime->tstep,     DyHelp);
     fl_set_object_posthandler (plottime->yMin,      DyHelp);
     fl_set_object_posthandler (plottime->yMax,      DyHelp);
     fl_set_object_posthandler (plottime->gridx,     DyHelp);
     fl_set_object_posthandler (plottime->gridy,     DyHelp);
     fl_set_object_posthandler (plottime->logx,      DyHelp);
     fl_set_object_posthandler (plottime->logy,      DyHelp);
     fl_set_object_posthandler (plottime->realUnits, DyHelp);
     fl_set_object_posthandler (plottime->autoScale, DyHelp);
     fl_set_object_posthandler (plottime->ok,        DyHelp);
     fl_set_object_posthandler (plottime->close,     DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpPlotfft ---*/

void DyHelpPlotfft (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotfft->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name with the\n"
                             "sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used to compute FFT",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->sampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->tstep)
            fl_show_oneliner("Size (in % of time window size)\n"
                             "of the time window steps",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->average_fft)
            fl_show_oneliner("Number of spectra you average",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotfft->xMin)
            fl_show_oneliner("Lower frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->xMax)
            fl_show_oneliner("Upper frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotfft->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotfft->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotfft->name,        DyHelp);
     fl_set_object_posthandler (plotfft->bydefault,   DyHelp);
     fl_set_object_posthandler (plotfft->tsize,       DyHelp);
     fl_set_object_posthandler (plotfft->sampFreq,    DyHelp);
     fl_set_object_posthandler (plotfft->average_fft, DyHelp);
     fl_set_object_posthandler (plotfft->tstep,       DyHelp);
     fl_set_object_posthandler (plotfft->xMin,        DyHelp);
     fl_set_object_posthandler (plotfft->xMax,        DyHelp);
     fl_set_object_posthandler (plotfft->nodc,        DyHelp);
     fl_set_object_posthandler (plotfft->gridx,       DyHelp);
     fl_set_object_posthandler (plotfft->gridy,       DyHelp);
     fl_set_object_posthandler (plotfft->logx,        DyHelp);
     fl_set_object_posthandler (plotfft->logy,        DyHelp);
    fl_set_object_posthandler (plotfft->ok,          DyHelp);
     fl_set_object_posthandler (plotfft->close,       DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpPlotffttime ---*/

void DyHelpPlotffttime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotffttime->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name with the\n"
                             "sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->timeSize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "of the time-frequency plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->sampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->tstep)
            fl_show_oneliner("Size of the time window steps\n"
                             "(in % of FFT time window size)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plotffttime->dtime)
            fl_show_oneliner("Time resolution (in seconds)\n"
                             "of the time-frequency plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->dfreq)
            fl_show_oneliner("Frequency resolution (in Hertz)\n"
                             "of the time-frequency plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
/**
      if (ob == plotffttime->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used to compute the FFT",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->average_fft)
            fl_show_oneliner("Number of spectra averaged",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
**/
     if (ob == plotffttime->xMin)
            fl_show_oneliner("Lower frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->xMax)
            fl_show_oneliner("Upper frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotffttime->refresh_percent)
            fl_show_oneliner("refresh period of the plot, in percent of time window",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotffttime->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotffttime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->gridz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->logz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotffttime->name,        DyHelp);
     fl_set_object_posthandler (plotffttime->bydefault,   DyHelp);
     fl_set_object_posthandler (plotffttime->timeSize,    DyHelp);
     fl_set_object_posthandler (plotffttime->sampFreq,    DyHelp);
     fl_set_object_posthandler (plotffttime->dtime,       DyHelp);
     fl_set_object_posthandler (plotffttime->dfreq,       DyHelp);
/**
     fl_set_object_posthandler (plotffttime->tsize,       DyHelp);
     fl_set_object_posthandler (plotffttime->average_fft, DyHelp);
**/
     fl_set_object_posthandler (plotffttime->tstep,       DyHelp);
     fl_set_object_posthandler (plotffttime->xMin,        DyHelp);
     fl_set_object_posthandler (plotffttime->xMax,        DyHelp);
     fl_set_object_posthandler (plotffttime->nodc,        DyHelp);
     fl_set_object_posthandler (plotffttime->gridx,       DyHelp);
     fl_set_object_posthandler (plotffttime->gridy,       DyHelp);
     fl_set_object_posthandler (plotffttime->gridz,       DyHelp);
     fl_set_object_posthandler (plotffttime->logx,        DyHelp);
     fl_set_object_posthandler (plotffttime->logy,        DyHelp);
     fl_set_object_posthandler (plotffttime->logz,        DyHelp);
     fl_set_object_posthandler (plotffttime->ok,          DyHelp);
     fl_set_object_posthandler (plotffttime->close,       DyHelp);
    }

 return;
}



/*------------------------------------------ DyHelpPlot1dtime ---*/

void DyHelpPlot1dtime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plot1dtime->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name with the\n"
                             "sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->timeSize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "of the time-1d plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plot1dtime->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used to do 1d statistics",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->sampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->xMin)
            fl_show_oneliner("Lower edge of the 1d plot (y axis)\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->xMax)
            fl_show_oneliner("Upper edeg of the 1d plot (y axis)\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->refresh_percent)
            fl_show_oneliner("refresh period of the plot, in percent of time window",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->autoScale)
            fl_show_oneliner("This button, if switched on, allows to \n"
                             "automatically set the z axis min and max",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->realUnits)
            fl_show_oneliner("This button, if switched on, put z axis of the\n"
                             "time-1d plot with the real units of the data",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->gridz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->logz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plot1dtime->name,        DyHelp);
     fl_set_object_posthandler (plot1dtime->bydefault,   DyHelp);
     fl_set_object_posthandler (plot1dtime->timeSize,    DyHelp);
     fl_set_object_posthandler (plot1dtime->tsize,       DyHelp);
     fl_set_object_posthandler (plot1dtime->sampFreq,    DyHelp);
     fl_set_object_posthandler (plot1dtime->xMin,        DyHelp);
     fl_set_object_posthandler (plot1dtime->xMax,        DyHelp);
     fl_set_object_posthandler (plot1dtime->zmax,       DyHelp);
     fl_set_object_posthandler (plot1dtime->autoScale,  DyHelp);
     fl_set_object_posthandler (plot1dtime->realUnits,  DyHelp);
     fl_set_object_posthandler (plot1dtime->nodc,        DyHelp);
     fl_set_object_posthandler (plot1dtime->gridx,       DyHelp);
     fl_set_object_posthandler (plot1dtime->gridy,       DyHelp);
     fl_set_object_posthandler (plot1dtime->gridz,       DyHelp);
     fl_set_object_posthandler (plot1dtime->logx,        DyHelp);
     fl_set_object_posthandler (plot1dtime->logy,        DyHelp);
     fl_set_object_posthandler (plot1dtime->logz,        DyHelp);
     fl_set_object_posthandler (plot1dtime->ok,          DyHelp);
     fl_set_object_posthandler (plot1dtime->close,       DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpPlotrawtime ---*/

void DyHelpPlotrawtime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotrawtime->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name with the\n"
                             "sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->timeSize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "of the time-1d plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plotrawtime->average_fft)
            fl_show_oneliner("Number of data raw vectors to\n"
                             "average (one vector per frame)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->xMin)
            fl_show_oneliner("Lower edge of the 1d plot (y axis)\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->xMax)
            fl_show_oneliner("Upper edge of the 1d plot (y axis)\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->zmax)
            fl_show_oneliner("Upper edge of the z axis of the time-raw plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotrawtime->refresh_percent)
            fl_show_oneliner("refresh period of the plot, in percent of time window",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotrawtime->x2av)
            fl_show_oneliner("This button, if switched on, allows to average\n"
                             "squares instead of values (useful if raw data \n"
                             "averaged are spectra)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->autoScale)
            fl_show_oneliner("This button, if switched on, allows to \n"
                             "automatically set the z axis min and max",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->realUnits)
            fl_show_oneliner("This button, if switched on, put z axis of the\n"
                             "time-raw plot with the real units of the data",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotrawtime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->gridz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->logz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotrawtime->name,        DyHelp);
     fl_set_object_posthandler (plotrawtime->bydefault,   DyHelp);
     fl_set_object_posthandler (plotrawtime->timeSize,    DyHelp);
     fl_set_object_posthandler (plotrawtime->average_fft, DyHelp);
     fl_set_object_posthandler (plotrawtime->xMin,        DyHelp);
     fl_set_object_posthandler (plotrawtime->xMax,        DyHelp);
     fl_set_object_posthandler (plotrawtime->zmax,        DyHelp);
     fl_set_object_posthandler (plotrawtime->x2av,        DyHelp);
     fl_set_object_posthandler (plotrawtime->autoScale,   DyHelp);
     fl_set_object_posthandler (plotrawtime->realUnits,   DyHelp);
     fl_set_object_posthandler (plotrawtime->gridx,       DyHelp);
     fl_set_object_posthandler (plotrawtime->gridy,       DyHelp);
     fl_set_object_posthandler (plotrawtime->gridz,       DyHelp);
     fl_set_object_posthandler (plotrawtime->logx,        DyHelp);
     fl_set_object_posthandler (plotrawtime->logy,        DyHelp);
     fl_set_object_posthandler (plotrawtime->logz,        DyHelp);
     fl_set_object_posthandler (plotrawtime->ok,          DyHelp);
     fl_set_object_posthandler (plotrawtime->close,       DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpPlottrfct ---*/

void DyHelpPlottrfct (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plottrfct->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name with the\n"
                             "sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used to compute FFT's",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->sampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->tstep)
            fl_show_oneliner("Size (in % of time window size)\n"
                             "of the time window steps",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->average_fft)
            fl_show_oneliner("Number of spectra you average",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plottrfct->xMin)
            fl_show_oneliner("Lower frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->xMax)
            fl_show_oneliner("Upper frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plottrfct->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plottrfct->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->gridx2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->gridy2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logx2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logy2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plottrfct->name,        DyHelp);
     fl_set_object_posthandler (plottrfct->bydefault,   DyHelp);
     fl_set_object_posthandler (plottrfct->tsize,       DyHelp);
     fl_set_object_posthandler (plottrfct->sampFreq,    DyHelp);
     fl_set_object_posthandler (plottrfct->average_fft, DyHelp);
     fl_set_object_posthandler (plottrfct->tstep,       DyHelp);
     fl_set_object_posthandler (plottrfct->xMin,        DyHelp);
     fl_set_object_posthandler (plottrfct->xMax,        DyHelp);
     fl_set_object_posthandler (plottrfct->nodc,        DyHelp);
     fl_set_object_posthandler (plottrfct->gridx,       DyHelp);
     fl_set_object_posthandler (plottrfct->gridy,       DyHelp);
     fl_set_object_posthandler (plottrfct->logx,        DyHelp);
     fl_set_object_posthandler (plottrfct->logy,        DyHelp);
     fl_set_object_posthandler (plottrfct->gridx2,      DyHelp);
     fl_set_object_posthandler (plottrfct->gridy2,      DyHelp);
     fl_set_object_posthandler (plottrfct->logx2,       DyHelp);
     fl_set_object_posthandler (plottrfct->logy2,       DyHelp);
     fl_set_object_posthandler (plottrfct->ok,          DyHelp);
     fl_set_object_posthandler (plottrfct->close,       DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpPlot1d ---*/

void DyHelpPlot1d (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plot1d->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name \n"
                             "with the sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xnBin)
            fl_show_oneliner("Number of x bins used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xMin)
            fl_show_oneliner("Min value for the 1D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xMax)
            fl_show_oneliner("Max value for the 1D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->tstep)
            fl_show_oneliner("Number of data read\n"
                             "before the plot is updated",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->stat)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "statistical informations in the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->autoEdge)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plot1d->name,      DyHelp);
     fl_set_object_posthandler (plot1d->bydefault, DyHelp);
     fl_set_object_posthandler (plot1d->xnBin,     DyHelp);
     fl_set_object_posthandler (plot1d->xMin,      DyHelp);
     fl_set_object_posthandler (plot1d->xMax,      DyHelp);
     fl_set_object_posthandler (plot1d->tstep,     DyHelp);
     fl_set_object_posthandler (plot1d->stat,      DyHelp);
     fl_set_object_posthandler (plot1d->gridx,     DyHelp);
     fl_set_object_posthandler (plot1d->gridy,     DyHelp);
     fl_set_object_posthandler (plot1d->logx,      DyHelp);
     fl_set_object_posthandler (plot1d->logy,      DyHelp);
     fl_set_object_posthandler (plot1d->autoEdge,  DyHelp);
     fl_set_object_posthandler (plot1d->ok,        DyHelp);
     fl_set_object_posthandler (plot1d->close,     DyHelp);

    }

 return;
}


/*------------------------------------------ DyHelpPlot2d ---*/

void DyHelpPlot2d (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plot2d->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name \n"
                             "with the sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xnBin)
            fl_show_oneliner("Number of x bins used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xMin)
            fl_show_oneliner("Min x value for the 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xMax)
            fl_show_oneliner("Max x value for the 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->ynBin)
            fl_show_oneliner("Number of y bins used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->yMin)
            fl_show_oneliner("Min y value for 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->yMax)
            fl_show_oneliner("Max y value for the 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->tstep)
            fl_show_oneliner("Number of data read\n"
                             "before the plot is updated",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->stat)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "statistical informations in the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->logz)
            fl_show_oneliner("This button, if switched on, introduces\n"
                             "z log scale in the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->cont)
            fl_show_oneliner("This button, if switched on, gives\n"
                             "a 2D plot in contour mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->lego)
            fl_show_oneliner("This button, if switch on, gives\n"
                             "a 2D plot in lego mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->surf)
            fl_show_oneliner("This button, if switched on, gives\n"
                             "a 2D plot in surface mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->scat)
            fl_show_oneliner("This button, if switched on, gives\n"
                             "a 2D plot in scatter mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->autoEdge)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plot2d->name,      DyHelp);
     fl_set_object_posthandler (plot2d->bydefault, DyHelp);
     fl_set_object_posthandler (plot2d->xnBin,     DyHelp);
     fl_set_object_posthandler (plot2d->xMin,      DyHelp);
     fl_set_object_posthandler (plot2d->xMax,      DyHelp);
     fl_set_object_posthandler (plot2d->ynBin,     DyHelp);
     fl_set_object_posthandler (plot2d->yMin,      DyHelp);
     fl_set_object_posthandler (plot2d->yMax,      DyHelp);
     fl_set_object_posthandler (plot2d->tstep,     DyHelp);
     fl_set_object_posthandler (plot2d->stat,      DyHelp);
     fl_set_object_posthandler (plot2d->gridx,     DyHelp);
     fl_set_object_posthandler (plot2d->gridy,     DyHelp);
     fl_set_object_posthandler (plot2d->logx,      DyHelp);
     fl_set_object_posthandler (plot2d->logy,      DyHelp);
     fl_set_object_posthandler (plot2d->logz,      DyHelp);
     fl_set_object_posthandler (plot2d->cont,      DyHelp);
     fl_set_object_posthandler (plot2d->lego,      DyHelp);
     fl_set_object_posthandler (plot2d->surf,      DyHelp);
     fl_set_object_posthandler (plot2d->scat,      DyHelp);
     fl_set_object_posthandler (plot2d->autoEdge,  DyHelp);
     fl_set_object_posthandler (plot2d->ok,        DyHelp);
     fl_set_object_posthandler (plot2d->close,     DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpAudio ---*/

void DyHelpAudio (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotaudio->name)
            fl_show_oneliner("Name of the plot.\n"
                             "It is composed of the variable name \n"
                             "with the sms name as prefix if it is a sms variable.\n"
                             "Do not put any blank space in this name!",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->t0)
            fl_show_oneliner("Begin time for data audio play\n"
                             "(relative to first frame read)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->duration)
            fl_show_oneliner("Duration of audio play",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotaudio->name,      DyHelp);
     fl_set_object_posthandler (plotaudio->bydefault, DyHelp);
     fl_set_object_posthandler (plotaudio->t0,        DyHelp);
     fl_set_object_posthandler (plotaudio->duration,  DyHelp);
     fl_set_object_posthandler (plotaudio->ok,        DyHelp);
     fl_set_object_posthandler (plotaudio->close,     DyHelp);
    }

 return;
}

/*------------------------------------------ DyHelpPlotGenePar ---*/

void DyHelpPlotGenePar (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotgenepar->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters in all the plots. Use it carefully.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->tsize)
            fl_show_oneliner("Change the size (in seconds) of the time window\n"
                             "used, in all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->sampFreq)
            fl_show_oneliner("Change the resampling of the data\n"
                             "on all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->tstep)
            fl_show_oneliner("Define the size (in % of time window size)\n"
                             "of the time window steps on all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component\n"
                             "on all the concerned plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->autoScale)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on Y axis of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->autoEdge)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on X axis of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->realUnits)
            fl_show_oneliner("This button, if switched on, sets the Y units\n"
                             "in real units instead of ADC counts of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
/**
     if (ob == plotgenepar->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
**/
     if (ob == plotgenepar->minmax_resample)
            fl_show_oneliner("This button, if switched on, says that any data resampling\n"
                             "will be done using min and max instead of mean",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->all_clone)
            fl_show_oneliner("If switched on, this button allows to clone all\n"
                             " the parameters, even those you did not touched.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotgenepar->bydefault,   DyHelp);
     fl_set_object_posthandler (plotgenepar->tsize,       DyHelp);
     fl_set_object_posthandler (plotgenepar->sampFreq,    DyHelp);
     fl_set_object_posthandler (plotgenepar->tstep,       DyHelp);
     fl_set_object_posthandler (plotgenepar->autoScale,   DyHelp);
     fl_set_object_posthandler (plotgenepar->autoEdge,    DyHelp);
     fl_set_object_posthandler (plotgenepar->realUnits,   DyHelp);
     fl_set_object_posthandler (plotgenepar->nodc,        DyHelp);
     fl_set_object_posthandler (plotgenepar->gridx,       DyHelp);
     fl_set_object_posthandler (plotgenepar->gridy,       DyHelp);
/***
     fl_set_object_posthandler (plotgenepar->logx,        DyHelp);
     fl_set_object_posthandler (plotgenepar->logy,        DyHelp);
**/
     fl_set_object_posthandler (plotgenepar->minmax_resample, DyHelp);
     fl_set_object_posthandler (plotgenepar->all_clone,   DyHelp);
     fl_set_object_posthandler (plotgenepar->ok,          DyHelp);
     fl_set_object_posthandler (plotgenepar->close,       DyHelp);
    }

 return;
}

/*------------------------------------------ DyHelpCmConnect ---*/

void DyHelpCmConnect (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == cmservers->liste)
            fl_show_oneliner("This browser shows the list of all Cm servers\n"
                             "currently available on the Ethernet network.\n"
                             "Selecte one of them. Then button OK will do the connection.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == cmservers->fullList)
            fl_show_oneliner("If on, this button show the full list of Cm names\n"
                             "provided by the Cm Name Server. Otherwise, only\n"
                             "default dataDisplay providers are shown.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == cmservers->update_cmlist)
            fl_show_oneliner("This button updates the liste of Cm names present in\n"
                             "the browser, by doing a request to the Cm Name Server.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == cmservers->searchCmName)
            fl_show_oneliner("Begin to type a name in this input field to search\n"
                             "for a Cm name in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == cmservers->ok)
            fl_show_oneliner("This button do the connection to the Cm server\n"
                             "selected in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);


    }
 else
    {
     fl_set_object_posthandler (cmservers->liste,          DyHelp);
     fl_set_object_posthandler (cmservers->update_cmlist,  DyHelp);
     fl_set_object_posthandler (cmservers->fullList,       DyHelp);
     fl_set_object_posthandler (cmservers->searchCmName,   DyHelp);
     fl_set_object_posthandler (cmservers->ok,             DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpUserFct ---*/

void DyHelpUserFct (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == userfuncbrowser->userFuncLoaded)
            fl_show_oneliner("List of user functions currently loaded. This list\n"
                             "is the same as in the menu UserFct of mainbrowser panel.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == userfuncbrowser->newFunc)
            fl_show_oneliner("To create a new file x.C that you can fill with your own\n"
                             "algorithm's code operating on the channels to create new\n"
                             "channels to plot.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == userfuncbrowser->loadInterpret)
            fl_show_oneliner("Load a file x.C already compiled (x.so available).\n"
                             "Then, this user fonction x appears in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == userfuncbrowser->loadCompile)
            fl_show_oneliner("Compile and load a file x.C.\n"
                             "Then, this user fonction x appears in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (userfuncbrowser->userFuncLoaded,  DyHelp);
     fl_set_object_posthandler (userfuncbrowser->newFunc,         DyHelp);
     fl_set_object_posthandler (userfuncbrowser->loadInterpret,   DyHelp);
     fl_set_object_posthandler (userfuncbrowser->loadCompile,     DyHelp);
    }

 return;
}



/*------------------------------------------ DyHelpOptions ---*/

void DyHelpOptions (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dyoptions->notoc)
            fl_show_oneliner("This button, when checked, allows to disable the\n"
                             "use of the Table of Contents and to read frames\n"
                             "sequentially without taking care of their GPS time",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->noplot)
            fl_show_oneliner("This button, when checked, allows to disable the\n"
                             "display for all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->skipFrame)
            fl_show_oneliner("Put on this button if you want to skip the\n"
                             "missing frames. This has the advantage to scan\n"
                             "more rapidly a file of selected frames, but it has\n"
                             "the drawback that not consecutive data are concatenated\n"
                             "in time plots for instance.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->trigused)
            fl_show_oneliner("This button, when checked, allows to use the\n"
                             "trigger defined with TRIGGER in Data and Plots panel",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->trigpause)
            fl_show_oneliner("This button, when checked, allows to have an automatic \n"
                             "pause between each event found by the trigger",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->color2d)
            fl_show_oneliner("Each time this button is pushed, a new colormap is \n"
                             "chosen for the plots 2D, FFTTIME, etc...",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->autodeselectplot)
            fl_show_oneliner("This button, when checked, allows an automatic deselection\n"
                             "of plots in the browser each time an action is made on the plots list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->autodeselectch)
            fl_show_oneliner("This button, when checked, allows an automatic deselection \n"
                             "of channels in the browsers each time a new plot is created",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->noBox)
            fl_show_oneliner("This button, when checked, removes the red (corrupted data),\n"
                             "green (missing channel) or blue (missing frame)\n"
                             "colored boxes in the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomlegendtitle)
            fl_show_oneliner("Increase (+) or decrease (-) the font size\n"
                             "in the plots legends",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomstattitle)
            fl_show_oneliner("Increase (+) or decrease (-) the font size\n"
                             "in the plots statistics boxes",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomlabelxtitle)
            fl_show_oneliner("Increase (+) or decrease (-) the font size\n"
                             "in the plots labels on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomlabelytitle)
            fl_show_oneliner("Increase (+) or decrease (-) the font size\n"
                             "in the plots labels on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomtimestringtitle)
            fl_show_oneliner("Increase (+) or decrease (-) the font size\n"
                             "of the time string under the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomtitletitle)
            fl_show_oneliner("Increase (+) or decrease (-) the font size\n"
                             "of the plots title",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->loglinx)
            fl_show_oneliner("This button, when checked, put alternately log or\n"
                             "linear scale on X axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->logliny)
            fl_show_oneliner("This button, when checked, put alternately log or\n"
                             "linear scale on Y axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->fullwhite)
            fl_show_oneliner("This button, when checked, puts white color over\n"
                             "the full Canvas of the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->indelebile)
            fl_show_oneliner("This button, when checked, allows to superpose the reference\n"
                             "plots in memory to the current plots while dataDisplay is running",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->nicefont)
            fl_show_oneliner("This button, when checked, allows to use helvetica font (62)\n"
                             "instead of times font (22) in all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dyoptions->notoc,              DyHelp);
     fl_set_object_posthandler (dyoptions->noplot,             DyHelp);
     fl_set_object_posthandler (dyoptions->skipFrame,          DyHelp);
     fl_set_object_posthandler (dyoptions->trigused,           DyHelp);
     fl_set_object_posthandler (dyoptions->trigpause,          DyHelp);
     fl_set_object_posthandler (dyoptions->color2d,            DyHelp);
     fl_set_object_posthandler (dyoptions->autodeselectplot,   DyHelp);
     fl_set_object_posthandler (dyoptions->autodeselectch,     DyHelp);
     fl_set_object_posthandler (dyoptions->noBox,              DyHelp);
     fl_set_object_posthandler (dyoptions->zoomlegendtitle,    DyHelp);
     fl_set_object_posthandler (dyoptions->zoomstattitle,      DyHelp);
     fl_set_object_posthandler (dyoptions->zoomlabelxtitle,    DyHelp);
     fl_set_object_posthandler (dyoptions->zoomlabelytitle,    DyHelp);
     fl_set_object_posthandler (dyoptions->zoomtimestringtitle, DyHelp);
     fl_set_object_posthandler (dyoptions->zoomtitletitle,     DyHelp);
     fl_set_object_posthandler (dyoptions->loglinx,            DyHelp);
     fl_set_object_posthandler (dyoptions->logliny,            DyHelp);
     fl_set_object_posthandler (dyoptions->fullwhite,          DyHelp);
     fl_set_object_posthandler (dyoptions->indelebile,         DyHelp);
     fl_set_object_posthandler (dyoptions->nicefont,           DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpOutput ---*/

void DyHelpOutput (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dyoutput->output_default)
            fl_show_oneliner("This button allows to put default values\n"
                             "in all the fields of this panel",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_dir)
            fl_show_oneliner("Name of the directory where will be written the\n"
                             "files containing data frames",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_nframe_perfile)
            fl_show_oneliner("Number of frames per file written on disk",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_flag)
            fl_show_oneliner("This button, when checked, allows the writing of data on disk",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_tag)
            fl_show_oneliner("Write here the list of channels you want to write on disk\n"
                             "You can use wildcard like in \042Em*BD*01\042 to choose\n"
                             "all channels beginning with Em, containing BD and finishing with 01",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dyoutput->output_default,        DyHelp);
     fl_set_object_posthandler (dyoutput->output_dir,            DyHelp);
     fl_set_object_posthandler (dyoutput->output_nframe_perfile, DyHelp);
     fl_set_object_posthandler (dyoutput->output_flag,           DyHelp);
     fl_set_object_posthandler (dyoutput->output_tag,            DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpDebug ---*/

void DyHelpDebug (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dydebug->dbg_plot)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in the plots browser management",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_vdyplot)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in the plots drawing and painting",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_urplot)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in the plots processing",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_file)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in the source of data management",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_cm)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in the data through Ethernet management",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_config)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in the configuration management",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_frame)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in frames read and processing",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_channel)
            fl_show_oneliner("This button, if checked, allows debugging\n"
                             "the functions implied in the channels management",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_speed)
            fl_show_oneliner("This button, if checked, allows to know how much time\n"
                             "is passed in various functions",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_calls)
            fl_show_oneliner("This button, if checked, allows to know the name of\n"
                             "each function where the dataDisplay is passing",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_others)
            fl_show_oneliner("This button, if checked, allows debugging of various\n"
                             "useful functions of the dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->debuglvl)
            fl_show_oneliner("The higher this value is the more debugging output\n"
                             "you get for each of the debugging types below",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->checks)
            fl_show_oneliner("This button dump many parameters of the data inputs and plots\n"
                             "in the xterm window where you started dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dydebug->dbg_plot,     DyHelp);
     fl_set_object_posthandler (dydebug->dbg_vdyplot,  DyHelp);
     fl_set_object_posthandler (dydebug->dbg_urplot,   DyHelp);
     fl_set_object_posthandler (dydebug->dbg_file,     DyHelp);
     fl_set_object_posthandler (dydebug->dbg_cm,       DyHelp);
     fl_set_object_posthandler (dydebug->dbg_config,   DyHelp);
     fl_set_object_posthandler (dydebug->dbg_frame,    DyHelp);
     fl_set_object_posthandler (dydebug->dbg_channel,  DyHelp);
     fl_set_object_posthandler (dydebug->dbg_calls,    DyHelp);
     fl_set_object_posthandler (dydebug->dbg_speed,    DyHelp);
     fl_set_object_posthandler (dydebug->dbg_others,   DyHelp);
     fl_set_object_posthandler (dydebug->debuglvl,     DyHelp);
     fl_set_object_posthandler (dydebug->checks,       DyHelp);
    }

 return;
}


/*------------------------------------------ DyHelpReffile ---*/

void DyHelpReffile (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == reffile->file_list)
            fl_show_oneliner("This browser contains the list of reference plots files that\n"
                             "have been loaded into memory. The last number in the name is the code\n"
                             "color of the reference plots when they are superposed to the current plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->remove2d)
            fl_show_oneliner("This button removes from memory all the\n"
                             "2-dimensional reference plots of the files selected",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->removefile)
            fl_show_oneliner("This button removes from the list all the files selected",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->clear_filebrowser)
            fl_show_oneliner("This button removes all the files from the list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->edit)
            fl_show_oneliner("This button allows to change the code color of the selected file\n"
                             "Colors code it the ROOT one (1=black, 2=red, ... 51=pink, 52=dark blue, ...)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->addfile)
            fl_show_oneliner("This button allows to add a new file to the list.\n"
                             "The code color is defined automatically.\n"
                             "To change it, use the Edit button",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (reffile->file_list,         DyHelp);
     fl_set_object_posthandler (reffile->removefile,        DyHelp);
     fl_set_object_posthandler (reffile->clear_filebrowser, DyHelp);
     fl_set_object_posthandler (reffile->edit,              DyHelp);
     fl_set_object_posthandler (reffile->addfile,           DyHelp);
     fl_set_object_posthandler (reffile->remove2d,          DyHelp);
    }
 return;
}


/*------------------------------------------ DyUserProcHelp ---*/

void DyUserProcHelp (FL_OBJECT *obj, long arg)
{
 FL_OBJECT *ob, *browser,*close;
 long xsize, ysize;

 xsize      = 450;
 ysize      = 320;
 userprochelp  = fl_bgn_form(FL_NO_BOX, xsize, ysize);
 ob         = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
 browser    = fl_add_browser(FL_HOLD_BROWSER,10,10,xsize-20,ysize-60,"User processing and Macros Help");
    fl_set_object_color(browser,FL_LEFT_BCOL,FL_WHEAT);
    fl_set_object_lstyle(browser,FL_TIMESBOLD_STYLE);
    fl_set_object_lsize(browser,FL_MEDIUM_SIZE);
 close = fl_add_button(FL_NORMAL_BUTTON,xsize-80,ysize-40,70,30,"Close");
    fl_set_object_color(close,FL_WHEAT,FL_COL1);
    fl_set_object_lstyle(close,FL_TIMESBOLD_STYLE);
    fl_set_object_lsize(close,FL_MEDIUM_SIZE);
    fl_set_object_callback(close,DyXformClosePanel,USERFCTHELP);
 fl_end_form();

 fl_add_browser_line(browser,"\n                        ****  USER FUNCTIONS HELP  **** \n\n"
                             " Using this panel or using the \"User Fct\" button, you can create any function \n"
                             " operating on a set of input channels and outputing a new channel whose name will \n"
                             " begin by default with UserOp. \n"
                             "    \n"
                             " When you click on the \"User Fct\" menu and choose \"Operation String\" \n"
                             " you get an input line where you can write any C function to combine channels. \n"
                             " This line begins by default with \"UserOp=\". This is the default name's start \n"
                             " of the new channel that you creates with this line. \n"
                             "    \n"
                             " Once you click on Ok, two files UserOp.C and UserOp.h are created. \n"
                             " You can modify them afterwards if you want to change the function. \n");
 fl_add_browser_line(browser,"                                     *********                \n"
                             " You can also create a new function by using the button \"New Function\" \n"
                             " of this panel. You can also Load and Compile a function you already had \n"
                             " written (files UserOp.C and UserOp.h) and that you want to use on some \n"
                             " channels to process them before passing the result to the plots. \n"
                             "   \n"
                             " Each time, you click on the \"Start\" button of the dataDisplay mainpanel, \n"
                             " the function that you have created are stored at the end of the dd.car \n"
                             " configuration file of the dataDisplay. Then, any UserOp.C and UserOp.h files \n"
                             " disappear since the information is kept in dd.car \n\n");

 fl_set_form_atclose(userprochelp, DyAtClose, "userprochelp");
 DyXformRaisePanel(NULL,USERFCTHELP);
 return;
}


/*------------------------------------------ DyHelp ---*/
/*                            post-handler (for help ballons)       */
int DyHelp (FL_OBJECT *ob, int ev, FL_Coord mx, FL_Coord y, int key, void *xev)
{
 int help_selected;

 help_selected = fl_get_button(mainpanel->help);
 if (!help_selected) return(0);

 /*--- show help ballons ---*/

 if (ev == FL_ENTER)
    {
     DyHelpMainPanel(ob,0);     /*--- show help ballons for mainpanel panel   ---*/
     DyHelpMainBrowser(ob,0);   /*--- show help ballons for mainbrowser panel ---*/
     DyHelpDumpData (ob,0);     /*--- show help ballons for dumpdata panel    ---*/
     DyHelpPlottime(ob,0);      /*--- show help ballons for plottime panel    ---*/
     DyHelpPlotfft(ob,0);       /*--- show help ballons for plotfft panel     ---*/
     DyHelpPlotffttime(ob,0);   /*--- show help ballons for plotffttime panel ---*/
     DyHelpPlot1dtime(ob,0);    /*--- show help ballons for plot1dtime panel  ---*/
     DyHelpPlotrawtime(ob,0);   /*--- show help ballons for plotrawtime panel ---*/
     DyHelpPlottrfct(ob,0);     /*--- show help ballons for plottrfct panel   ---*/
     DyHelpPlot1d(ob,0);        /*--- show help ballons for plot1d panel      ---*/
     DyHelpPlot2d(ob,0);        /*--- show help ballons for plot2d panel      ---*/
     DyHelpAudio(ob,0);         /*--- show help ballons for plotaudio panel   ---*/
     DyHelpPlotGenePar(ob,0);   /*--- show help ballons for plotgenepar panel ---*/
     DyHelpCmConnect(ob,0);     /*--- show help ballons for cmservers panel   ---*/
     DyHelpUserFct(ob,0);       /*--- show help ballons for userfuncbrowser panel ---*/
     DyHelpOptions(ob,0);       /*--- show help ballons for options panel     ---*/
     DyHelpOutput(ob,0);        /*--- show help ballons for output panel      ---*/
     DyHelpDebug(ob,0);         /*--- show help ballons for debug panel       ---*/
     DyHelpReffile(ob,0);       /*--- show help ballons for reffile panel     ---*/
    }

 else if (ev != FL_MOTION)
    {fl_hide_oneliner();}

 return(0);
}
