/**************************************************/
/*                                                */
/* File: DyXform.c                                */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "dataDisplay.h"
#include "logo.xpm"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*------------------------------------------- DyXformSetDebug ---*/

void DyXformSetDebug (FL_OBJECT *obj, long arg)
{
 int debugType;

 debugType = 0;

 if (fl_get_button(dydebug->dbg_vdyplot)) debugType |= (1 << DBG_VDYPLOT);
 if (fl_get_button(dydebug->dbg_urplot))  debugType |= (1 << DBG_URPLOT);
 if (fl_get_button(dydebug->dbg_plot))    debugType |= (1 << DBG_PLOT);

 if (fl_get_button(dydebug->dbg_file))    debugType |= (1 << DBG_FILE);
 if (fl_get_button(dydebug->dbg_cm))      debugType |= (1 << DBG_CM);
 if (fl_get_button(dydebug->dbg_frame))   debugType |= (1 << DBG_FRAME);
 if (fl_get_button(dydebug->dbg_channel)) debugType |= (1 << DBG_CHANNEL);

 if (fl_get_button(dydebug->dbg_config))  debugType |= (1 << DBG_CONFIG);
 if (fl_get_button(dydebug->dbg_speed))   debugType |= (1 << DBG_SPEED);
 if (fl_get_button(dydebug->dbg_calls))   debugType |= (1 << DBG_CALLS);
 if (fl_get_button(dydebug->dbg_others))  debugType |= (1 << DBG_OTHERS);

 if (debugType != Dy->debugType && Dy->debug == 0) fl_set_input(dydebug->debuglvl,"1");

 Dy->debugType = debugType;

 Dy->debug = atoi(fl_get_input(dydebug->debuglvl));

 printf("debugType=%d  debugLvl=%d \n", Dy->debugType, Dy->debug);

 return;
}


/*-------------------------------------------- DyXformScanPlots ---*/

void DyXformScanPlots (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];
 double value;
 float min, max;

 fl_get_scrollbar_bounds (obj, &min, &max);
 value = fl_get_scrollbar_value (obj);
 value = value / (max-min);

 RA_ScanPlots (value);

 return;
}


/*------------------------------------------- DyXformSetMaxDuration ---*/

void DyXformSetMaxDuration (FL_OBJECT *obj, long arg)
{
 int i, nplot;
 double maxduration;
 char dummy[64];

 maxduration = 0.;

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 for (i=1; i<=nplot; i++)
    {if (maxduration < Plot[i]->timeSize) maxduration = Plot[i]->timeSize;
     if (maxduration < Plot[i]->tsize)    maxduration = Plot[i]->tsize;
    }

 sprintf(dummy,"%d",(int)maxduration);
 fl_set_input(mainpanel->duration,dummy);

 return;
}


/*------------------------------------------- DyXformOutputDefault ---*/

void DyXformOutputDefault (FL_OBJECT *obj, long arg)
{
 DyOutputDefault();
 DyXformOutputToPanel();
 return;
}


/*------------------------------------------- DyXformOutputToPanel ---*/

void DyXformOutputToPanel()
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"%d", Dy->output_nframe_perfile);
 fl_set_input(dyoutput->output_nframe_perfile, dummy);
 fl_set_input(dyoutput->output_dir, Dy->output_dir);
 fl_set_input(dyoutput->output_tag, Dy->output_tag);
 fl_set_button(dyoutput->output_flag, Dy->output_flag);

 return;
}


/*------------------------------------------- DyXformPanelToOutput ---*/

void DyXformPanelToOutput (FL_OBJECT *obj, long arg)
{
 long answer=1;
 DIR  *dirp;
 char dummy[LSTRING_MAX];
 const char *output_dir, *output_tag;

 output_tag = fl_get_input(dyoutput->output_tag);
 output_dir = fl_get_input(dyoutput->output_dir);

 if (output_tag) strcpy(Dy->output_tag, output_tag);
 else            strcpy(Dy->output_tag," ");
 if (output_dir) strcpy(Dy->output_dir, output_dir);
 else            {strcpy(Dy->output_dir, "."); fl_set_input(dyoutput->output_dir,".");}
 Dy->output_nframe_perfile = atoi(fl_get_input(dyoutput->output_nframe_perfile));
 Dy->output_flag = fl_get_button(dyoutput->output_flag);

 dirp = opendir(Dy->output_dir);
 if (dirp != NULL)
    {sprintf(dummy, "Directory %s already exists. Do you want to erase its content ?", Dy->output_dir);
     answer = fl_show_question (dummy,1);
     if (answer) {sprintf(dummy,"rm -rf %s/Dy-*.gwf", Dy->output_dir); system(dummy);}
    }

 if (arg == 0 || arg == OUTPUTDATA) DyXformClosePanel(dyoutput->close,OUTPUTDATA);

 return;
}


/*------------------------------------------- DyColorPalette ---*/

void DyColorPalette (FL_OBJECT *obj, long arg)
{
 char dummy[16];

 if (arg == -1) Dy->colorpalette++;
 else           Dy->colorpalette = arg;

 Dy->colorpalette = RA_SetColorPalette(Dy->colorpalette);
 sprintf(dummy, "Color2D=%d", Dy->colorpalette);
 fl_set_object_label(dyoptions->color2d,dummy);

 return;
}


/*------------------------------------------- DyXformGetButton ---*/
                                            /*  called from mainpanel */
void DyXformGetButton (FL_OBJECT *obj, long arg)
{
 int fullwhite;

 if (obj == dyoptions->fullwhite)          {fullwhite = fl_get_button (dyoptions->fullwhite); RA_SetCanvasColor(fullwhite);}
 if (obj == dyoptions->squareplot)         Dy->squarePlot        = fl_get_button (dyoptions->squareplot);
 if (obj == dyoptions->nicefont)           Dy->nicefont          = fl_get_button (dyoptions->nicefont);
 if (obj == dyoptions->trigused)           Dy->triggerUsed       = fl_get_button (dyoptions->trigused);
 if (obj == dyoptions->trigpause)          Dy->triggerPause      = fl_get_button (dyoptions->trigpause);
 if (obj == dyoptions->noplot)             Dy->plotsNotDisplayed = fl_get_button (dyoptions->noplot);
 if (obj == dyoptions->skipFrame)          Dy->skipFrame         = fl_get_button (dyoptions->skipFrame);
 if (obj == dyoptions->autodeselectplot)   Dy->AutoDeselectPlot  = fl_get_button (dyoptions->autodeselectplot);
 if (obj == dyoptions->autodeselectch)     Dy->AutoDeselectCh    = fl_get_button (dyoptions->autodeselectch);
 if (obj == dyoptions->noBox)              Dy->noBox             = fl_get_button (dyoptions->noBox);
 if (obj == dyoutput->output_flag)        {Dy->output_flag       = fl_get_button(dyoutput->output_flag); if (Dy->output_flag == OFF) fl_hide_object(mainpanel->writing_data);} 
 if (obj == dyoptions->indelebile)
    {Dy->refplot_indelebile = fl_get_button (dyoptions->indelebile);
     if (Dy->Order == STOP) {RA_DrawReferencePlot(); RA_MainCanvasUpdate();}
    }

 return;
}


/*------------------------------------------- DyXformAutoDeselect ---*/
                                            /*  called from plots browser */
void DyXformAutoDeselect (FL_OBJECT *obj, long arg)
{
 int nelement, i0, i;

 if (Dy->AutoDeselectPlot == ON && obj == mainbrowser->plots)
    {
     i0 = fl_get_browser(obj);
     if (i0 <= 0) return;
     nelement = fl_get_browser_maxline(obj);
     for (i=0; i<nelement; i++)
        {if (fl_isselected_browser_line(obj,i+1)) fl_deselect_browser_line(obj,i+1);}
    }

 if (Dy->AutoDeselectCh == ON && obj != mainbrowser->plots)
    {
     i0 = fl_get_browser(obj);
     if (i0 <= 0) return;
     nelement = fl_get_browser_maxline(obj);
     for (i=0; i<nelement; i++)
        {if (fl_isselected_browser_line(obj,i+1)) fl_deselect_browser_line(obj,i+1);}
    }

 return;
}


/*------------------------------------------ DyXformNow ---*/

void DyXformNow (FL_OBJECT *obj, long arg)
{
 char dummy[32];

 Dy->startTime = time(NULL) - FRGPSOFF + FRGPSDELTA;

 if (FRGPSDELTA <= 13 && Dy->startTime>=820108813) Dy->startTime += 1;

 sprintf(dummy,"%d", Dy->startTime);
 fl_set_input(mainpanel->startTime, dummy);
 fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));

 return; 
}


/*------------------------------------------ DyAudioControl ---*/

void DyAudioControl (FL_OBJECT *obj, long arg)
{
 return;
}


/*-------------------------------------- DyXformAddUserChannel ---*/

void DyXformAddUserChannel (struct FrameH *frame, FL_OBJECT *obj, char *className, char *inputVect)
{
 int i, nData, i0, nline;
 double sampleRate, dt;
 const char *str;
 char *word, *outName, chaine[LBIGSTRING_MAX], dummy[LBIGSTRING_MAX], unitY[LSTRING_MAX];
 struct FrVect *vect;
 struct FrProcData *proc;

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformAddUserChannel> inputVect=%s \n", inputVect);

 if (frame == NULL)     return;
 if (inputVect == NULL) return;

 outName = (char *)malloc(strlen(OPSIGNAL)+strlen(className)+strlen(inputVect)+LSTRING_MAX);
 strcpy(outName,OPSIGNAL);
 strcat(outName,className);

 /*--- Get list of input vector names and nvect ---*/

 nData = 0;
 strcpy(chaine,inputVect);
 word = strtok(chaine,OPSEP);

 while (word != NULL) 
    {
     if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("DyXformAddUserChannel> inputVect=%s  word=%s \n", inputVect, word);
     vect = FrameFindVect (frame, word);
     if (vect != NULL)
        {if (nData <= 1 && vect->unitY != NULL) strcpy(unitY, vect->unitY);
         if (nData < vect->nData) nData = vect->nData;
        }
     else if (nData == 0) nData = 1;
     strcat(outName,OPSEP);
     strcat(outName,word);
     word = strtok(NULL,OPSEP);
    }

 dt = DyGetFramedt(frame);
 sampleRate = (double)(nData) / dt;
 proc = FrProcDataNew (frame, outName, sampleRate, nData, -64);
 FrStrCpy(&(proc->data->unitY),unitY);
 proc->nAuxParam = 0;

 nline = fl_get_browser_maxline (obj);
 i0 = 0;
 for (i=0; i<nline; i++)
    {str = fl_get_browser_line(obj,i+1);
     sscanf(str, "%s", dummy);
     if (strcmp(outName, dummy) == 0) {i0=i+1; break;}
    }
 sprintf(dummy, "%s : %d sampling/frames at %.2fHz", outName, nData, sampleRate);
 if (i0 == 0) fl_add_browser_line (obj, dummy);
 else         fl_replace_browser_line (obj, i0, dummy);

 sprintf(dummy, "Made new signal %s at %.2fHz", outName, sampleRate);
 DyInfoLine (dummy);
 free(outName);

 return;
}


/*------------------------------------------ DyXformGetstartTime ---*/

void DyXformGetstartTime (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];

 if (arg == 0)
    {Dy->startTime = atoi(fl_get_input(mainpanel->startTime));
     if (FRGPSDELTA <= 13 && Dy->startTime>=820108813)
        {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));}
     else 
        {fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));}
    }
 else
    {Dy->startTime = DyGTimeFromDate((char *)fl_get_input(mainpanel->startUTCdate));
     sprintf(dummy,"%ld",Dy->startTime);
     fl_set_input(mainpanel->startTime, dummy);
    }

 return;
}

      
/*------------------------------------------ DyXformFilterBrowser ---*/

void DyXformFilterBrowser (FL_OBJECT *obj, long arg)
{
 const char *browserline, *filtertag;
 char name[64][64], varName[LSTRING_MAX], dummy[LSTRING_MAX], *str, *str0;
 int i, j, nvar, nname, todelete, length;
 FL_OBJECT *browser;
 char **varlist;
 int ndel;

 /*--- Get the concerned browser ---*/

 browser = NULL;
 if (obj == mainbrowser->filterSms)     browser = mainbrowser->sms;
 if (obj == mainbrowser->filterAdc)     browser = mainbrowser->adc;
 if (obj == mainbrowser->filterProc)    browser = mainbrowser->proc;
 if (obj == mainbrowser->filterSim)     browser = mainbrowser->sim;
 if (obj == dumpdata->filterDump)       browser = dumpdata->datadumped;
 if (browser == NULL) return;

 /*--- Get filter tag ---*/

 filtertag = fl_get_input(obj);
 nvar      = fl_get_browser_maxline(browser);

 if (filtertag == NULL)          {DyXformSortBrowser(browser); return;}
 if (strcmp(filtertag,"") == 0)  {DyXformSortBrowser(browser); return;}
 if (strcmp(filtertag,"*") == 0) {DyXformSortBrowser(browser); return;}
 if (nvar < 3)                    return;

 /*--- Allocate for browser's line storage ---*/

 varlist = (char **)malloc(nvar*sizeof(char *));
 for (i=0; i<nvar; i++) varlist[i] = (char *)malloc(LSTRING_MAX);

 DyXformClosePanel(mainbrowser->close,MAINBROWSER);

 /*--- Search for channels prefix ---*/

 if (browser == mainbrowser->sms || browser == mainbrowser->adc || browser == mainbrowser->proc || browser == mainbrowser->sim)
    {DyXformGetPrefix(browser);}

 /*--- Search for names in the filter tag ---*/

 strcpy(dummy,filtertag);
 length = strlen(filtertag);
 nname = 0;
 str  = strtok(dummy,"*");
 while (str != NULL) {strcpy(name[nname],str); str = strtok(NULL,"*"); nname++;}
 if (nname == 0) {DyXformSortBrowser(browser); DyXformRaisePanel(NULL,MAINBROWSER); return;}

 /*--- Search for browser's line that do not contain the filter tag names ---*/

 ndel = 0;
 for (i=nvar-1; i>=0; i--)
    {
     browserline = fl_get_browser_line(browser, i+1);
     sscanf(browserline,"%s", &varName[0]);

     todelete = 0;
     str = varName;
     for (j=0; j<nname; j++)
        {
         /*--- If filter tag is one name without wildcard ---*/
         if (nname == 1 && filtertag[length-1] != '*' && filtertag[0] != '*')
            {if (strcmp(varName, filtertag) != 0)
                {sprintf(dummy,"%s%s", Dy->chPrefix, filtertag);
                 if (strcmp(varName, dummy) != 0) {todelete = 1; break;}
                }
             break;
            }

         /*--- If filter tag does not begin with a wildcard ---*/
         if (j == 0 && filtertag[0] != '*')
            {if (strncmp(str, name[j], strlen(name[j])) != 0)
                {sprintf(dummy,"%s%s", Dy->chPrefix, name[j]);
                 if (strncmp(str, dummy, strlen(dummy)) != 0) {todelete = 1; break;}
                 else str += strlen(dummy);
                }
             else str += strlen(name[j]);
             continue;
            }

         /*--- General case, search for name within the remaining part of the browserline ---*/
         str0 = str;
         str = strstr(str, name[j]);
         if (str == NULL) {todelete = 1; break;}

         /*--- If filter tag begins with a wildcard but do not end with a wildcard ---*/
         if (j == nname-1 && filtertag[length-1] != '*' && filtertag[0] == '*')
            {if (strcmp(str, name[j]) != 0) {todelete = 1; break;}}

        }

     if (todelete == 1)
        {strcpy(varlist[ndel],browserline); ndel++;
         fl_delete_browser_line(browser,i+1);
        }
     else if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformFilterBrowser> browserline=%d  %s OK\n", i, varName);
    }

 /*--- Put back at the end of browser all the deleted lines ---*/

 if (ndel > 0)
    {DyXformSortBrowser(browser);
     for (i=ndel-1; i>=0; i--) fl_add_browser_line(browser,varlist[i]);
     fl_set_browser_topline(browser, 1);
    }

 for (i=0; i<nvar; i++) free(varlist[i]);
 free(varlist);

 DyXformRaisePanel(NULL, MAINBROWSER);

 return;
}


/*----------------------------------- Search for a prefix in the channels names ---*/

void DyXformGetPrefix (FL_OBJECT *browser)
{
 const char *varName;
 char *str;
 int i, nvar, lprefix;

 nvar = fl_get_browser_maxline(browser);

 strcpy(Dy->chPrefix,"");

 if (nvar > 6)
    {for (i=0; i<5; i++)
        {varName = fl_get_browser_line(browser,i+1);
         str = strstr(varName,":");
         if (str && *(str+1) != ' ')
           {lprefix = str-varName+1;
            if (lprefix < 32) {strncpy(Dy->chPrefix, varName, lprefix); *(Dy->chPrefix+lprefix) = '\0'; break;}
           }
        }
     for (i=nvar-5; i<nvar; i++)
        {varName = fl_get_browser_line(browser,i+1);
         str = strstr(varName,":");
         if (str && *(str+1) != ' ')
            {lprefix = str-varName+1;
             if (lprefix < 32) {strncpy(Dy->chPrefix, varName, lprefix); *(Dy->chPrefix+lprefix) = '\0'; break;}
            }
        }
    }
 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("DyXformGetPrefix> channels_prefix=%s  found at line=%d \n", Dy->chPrefix,i);

 return;
}


/*------------------------------------------ DyXformFindInBrowser ---*/

void DyXformFindInBrowser (FL_OBJECT *obj, long arg)
{
 const char *varName, *name;
 char *str, name1[LSTRING_MAX], name2[LSTRING_MAX], dummy[LSTRING_MAX];
 int i, i0, imin, nvar, length, lprefix;
 FL_OBJECT *browser;

 browser = NULL;
 if (obj == mainbrowser->inputSms)     browser = mainbrowser->sms;
 if (obj == mainbrowser->inputAdc)     browser = mainbrowser->adc;
 if (obj == mainbrowser->inputProc)    browser = mainbrowser->proc;
 if (obj == mainbrowser->inputSim)     browser = mainbrowser->sim;
 if (obj == cmservers->searchCmName)   browser = cmservers->liste;
 if (obj == dumpdata->inputDump)       browser = dumpdata->datadumped;
 if (browser == NULL) return;

 name = fl_get_input(obj);
 nvar = fl_get_browser_maxline(browser);

 /*--- Search for a prefix in the channels names ---*/

 if (browser == mainbrowser->sms || browser == mainbrowser->adc || browser == mainbrowser->proc || browser == mainbrowser->sim)
    {DyXformGetPrefix(browser);}

 if (name == NULL)
    {fl_set_browser_topline(browser,1);
     Dy->lastSearchI = 1;
     if (browser == mainbrowser->sms || browser == mainbrowser->adc || browser == mainbrowser->proc || browser == mainbrowser->sim)
        {return;}
    }

 /*--- Search for name in the data to dump list ---*/

 if (obj == dumpdata->inputDump)
    {i0 = -1;
     if (strcmp(Dy->lastSearchName,name) == 0) imin = Dy->lastSearchI;
     else                                      imin = 0;
     for (i=imin; i<nvar; i++)
        {varName = fl_get_browser_line(browser, i+1);
         if (strstr(varName, name) != NULL) {i0 = i; break;}
        }
     if (i0 >= 0) {fl_set_browser_topline(browser, i0+1); Dy->lastSearchI = i0+1;}
     strcpy(Dy->lastSearchName,name);
    }

 /*--- Search for name in the channels list ---*/

 else
    {
     strcpy(name1,DYEMPTY);
     strcpy(name2,DYEMPTY);
     length = 0;
     i0 = -1;
     for (i=0; i<nvar; i++)
        {varName = fl_get_browser_line(browser, i+1);

         if (strstr(name,"*") == NULL)
            {if (strncmp(varName, name, strlen(name)) == 0) {i0 = i; break;}}
         else
            {str = strstr(name,"*");
             strcpy(name2, str+1);
             length = str-name;
             strncpy(name1,name,length); name1[length] = '\0';
             if (strncmp(varName, name1, strlen(name1)) == 0 && strstr(varName+length,name2) != 0) {i0 = i; break;}
            }
         if (varName[0] > name[0]) break;
        }

     if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1) printf("DyXformFindInBrowser> name1=%s  name2=%s  name=%s  length=%d \n", name1, name2, name, length);

     /*--- If channel name found go to it, else put the prefix and search again ---*/

     if (i0 >= 0) fl_set_browser_topline(browser, i0+1);

     else if (obj == mainbrowser->inputSms  || obj == mainbrowser->inputAdc ||
              obj == mainbrowser->inputProc || obj == mainbrowser->inputSim)
        {
         if (strlen(name) >= 1 && strncmp(name, Dy->chPrefix,2) != 0)
            {
             strcpy(name1,DYEMPTY);
             strcpy(name2,DYEMPTY);
             length = 0;
             sprintf(dummy, "%s%s", Dy->chPrefix, name);
             fl_set_input(obj,dummy);
             name = fl_get_input(obj);
             i0 = -1;
             for (i=0; i<nvar; i++)
                {varName = fl_get_browser_line(browser, i+1);
                 if (strstr(name,"*") == NULL)
                    {if (strncmp(varName, name, strlen(name)) == 0) {i0 = i; break;}}
                 else
                    {str = strstr(name,"*");
                     strcpy(name2, str+1);
                     length = str-name;
                     strncpy(name1,name,length); name1[length] = '\0';
                     if (strncmp(varName, name1, strlen(name1)) == 0 && strstr(varName+length,name2) != 0) {i0 = i; break;}
                    }
                 if (varName[0] > name[0]) break;
                }
             if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1) printf("DyXformFindInBrowser> Second try: name1=%s  name2=%s  name=%s  length=%d \n", name1, name2, name, length);
             if (i0 >= 0) fl_set_browser_topline(browser, i0+1);
            }
        }
    }

 return;
}


/*------------------------------------------ DyXformSetScanStep ---*/

void DyXformSetScanStep (double step)
{
 fl_set_scrollbar_increment(mainpanel->scanplots, step, step);
 fl_set_scrollbar_step(mainpanel->scanplots, step);
 return;
}


/*------------------------------------------ DyXformIni ---*/

void DyXformIni (char **argv, int *myargc)
{
 char** myargv;
 char dummy[LSTRING_MAX];

 fl_set_border_width(-2);

#ifdef USE_VEGA
 myargv = (char**)malloc(sizeof(char*));
 myargv[0] = (char*)malloc(5*sizeof(char));
 sprintf(myargv[0],"Dy");
 fl_initialize(myargc, myargv,"dataDisplay", 0, 0);
 free(myargv[0]);
 free(myargv);
#else
 fl_initialize(myargc, argv, "dataDisplay", 0, 0);
#endif
 
 dumpdata        = create_form_dumpdata();
 mainbrowser     = create_form_mainbrowser();
 inputfile       = create_form_inputfile();
 mainpanel       = create_form_mainpanel();
 userfuncbrowser = create_form_userfuncbrowser();
 plot1d          = create_form_plot1d();
 plot2d          = create_form_plot2d();
 plottime        = create_form_plottime();
 plotfft         = create_form_plotfft();
 plotffttime     = create_form_plotffttime();
 plot1dtime      = create_form_plot1dtime();
 plotrawtime     = create_form_plotrawtime();
 plottrfct       = create_form_plottrfct();
 plotaudio       = create_form_plotaudio();
 plotgenepar     = create_form_plotgenepar();
 getdd           = create_form_getdd();
 getshm          = create_form_getshm();
 cmservers       = create_form_cmservers();
 plotraw         = create_form_plotraw();
 dydebug         = create_form_dydebug();
 dyoptions       = create_form_dyoptions();
 dyoutput        = create_form_dyoutput();
 reffile         = create_form_reffile();

 /*--- Init some inputs and browsers ---*/

 fl_set_object_label (mainpanel->currentInput,"Please, give me input data...");
 DyXformUpdateCounters(0,0,0,0);
 fl_set_input (mainpanel->startRun, "-1");
 fl_set_input (mainpanel->startFrame, "0");
 fl_set_input (mainpanel->UTCdate, "0");
 fl_clear_browser (mainbrowser->sms);
 fl_clear_browser (mainbrowser->adc);
 fl_clear_browser (mainbrowser->proc);
 fl_clear_browser (mainbrowser->sim);
 fl_clear_browser (mainbrowser->plots);
 fl_clear_browser (inputfile->file_list);
 fl_clear_browser (reffile->file_list);

 fl_set_button (dyoptions->autodeselectplot,ON);

 if (Dy->startTime <= 0) DyXformNow(NULL,0);
 else
    {sprintf(dummy,"%d", Dy->startTime); 
     fl_set_input (mainpanel->startTime, dummy);    
     fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));
    }

 fl_set_input (mainpanel->duration, "-1");
 if (Dy->duration > 0)
    {sprintf(dummy,"%d", Dy->duration); 
     fl_set_input (mainpanel->duration, dummy);
    }

 /*--- Init the slider used for scanning zoomed plots ---*/

 fl_set_scrollbar_increment(mainpanel->scanplots, 0.001, 0.001);
 fl_set_scrollbar_bounds(mainpanel->scanplots,0.,1.);
 fl_set_scrollbar_step(mainpanel->scanplots,0.001);

 /*--- Init the user functions browser ---*/

 fl_clear_browser (userfuncbrowser->userFuncLoaded);
 DyXformUpdateUserProcBrowser();

 /*--- Init data dump ---*/

 fl_clear_browser (dumpdata->datadumped);
 fl_clear_browser (dumpdata->datatodump);
 fl_set_browser_fontstyle (dumpdata->datadumped, FL_FIXED_STYLE);

 /*--- Init the CmServers panel and the dataSender panel ---*/
/**
 fl_set_input (cmservers->cmdomain, CASCINA);
 fl_set_input (getdd->cmdomain, CASCINA);
**/
 fl_set_input (cmservers->cmdomain, getenv("CMDOMAIN"));
 fl_set_input (getdd->cmdomain, getenv("CMDOMAIN"));
 printf("\n *** Your current CMDOMAIN is %s ***\n", getenv("CMDOMAIN"));
 printf(" *** To access online data, you must type \042setenv CMDOMAIN %s\042 before starting dataDisplay ***\n", CASCINA);
 printf(" *** To connect to dataSender, you must type \042setenv CMDOMAIN %sLV\042 before starting dataDisplay ***\n\n", CASCINA);
 fl_set_browser_dblclick_callback (cmservers->liste, DyXformCmConnect, 0);
 fl_set_browser_fontsize (cmservers->liste, FL_MEDIUM_SIZE);
 fl_set_browser_fontstyle (cmservers->liste, FL_TIMES_STYLE);
 fl_set_browser_fontsize (getdd->liste, FL_MEDIUM_SIZE);
 fl_set_browser_fontstyle (getdd->liste, FL_TIMES_STYLE);
 fl_set_browser_fontsize (getshm->liste, FL_MEDIUM_SIZE);
 fl_set_browser_fontstyle (getshm->liste, FL_TIMES_STYLE);

 /*--- Init the search input fields ---*/

 fl_set_input_return (cmservers->searchCmName,  FL_RETURN_ALWAYS);
 fl_set_input_return (mainbrowser->inputSms,    FL_RETURN_ALWAYS);
 fl_set_input_return (mainbrowser->inputAdc,    FL_RETURN_ALWAYS);
 fl_set_input_return (mainbrowser->inputProc,   FL_RETURN_ALWAYS);
 fl_set_input_return (mainbrowser->inputSim,    FL_RETURN_ALWAYS);
 fl_set_input_return (dumpdata->inputDump,      FL_RETURN_ALWAYS);

 fl_set_input_return (mainpanel->startUTCdate,  FL_RETURN_ALWAYS);
 fl_set_input_return (mainpanel->startTime,   FL_RETURN_ALWAYS);

 /*--- Initialize options and buttons of the main panel ---*/

 fl_activate_object   (mainpanel->start);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->start, FL_BLACK);
 fl_set_object_lcol   (mainpanel->stop, FL_INACTIVE);
 fl_set_object_lcol   (mainpanel->pause, FL_INACTIVE);
 fl_set_object_lcol   (mainpanel->continuer, FL_INACTIVE);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
 fl_set_object_lcol   (mainpanel->nextFrame, FL_INACTIVE);
/*
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/

 /*--- Init the Input menu button ---*/

 fl_addto_menu (mainpanel->inputMenu, READ_FFL);
 fl_addto_menu (mainpanel->inputMenu, READ_FILE);
#ifndef NOCM
 fl_addto_menu (mainpanel->inputMenu, CONNECT_CMSERVER);
 fl_addto_menu (mainpanel->inputMenu, CONNECT_DD);
#endif
 fl_addto_menu (mainpanel->inputMenu, READ_SHM);
 fl_addto_menu (mainpanel->inputMenu, "-----------------");
 fl_addto_menu (mainpanel->inputMenu, READ_DIRECTORY);
 fl_addto_menu (mainpanel->inputMenu, READ_FILE_ONLINE);
 fl_addto_menu (mainpanel->inputMenu, READ_WAVE);
 fl_addto_menu (mainpanel->inputMenu, "-----------------");
#ifndef NOCM
 fl_addto_menu (mainpanel->inputMenu, DECLARE_CM);
#endif
 fl_addto_menu (mainpanel->inputMenu, CHECK_FILES);

 /*--- Init the Config menu button ---*/

 fl_addto_menu (mainpanel->configMenu, LOAD_CONFIG);
 fl_addto_menu (mainpanel->configMenu, SAVE_CONFIG_AS);

 /*--- Init the User Fct menu button ---*/

 fl_addto_menu (mainbrowser->userFct, SET_PARAMETERS);
 fl_addto_menu (mainbrowser->userFct, OPERATION_STRING);

 /*--- Init the Trigger  menu button ---*/
 fl_addto_menu (mainbrowser->trigger, TRIGCH);
 fl_addto_menu (mainbrowser->trigger, TRIGASCII);
 fl_addto_menu (mainbrowser->trigger, TRIGEVENTS);

 /*--- Init the Cm servers menu ---*/
/**
 fl_addto_menu (cmservers->csmMenu, "AddToMenu");
 fl_addto_menu (cmservers->csmMenu, "FbmStDy");
 fl_addto_menu (cmservers->csmMenu, "Fbt50FbDy");
 fl_addto_menu (cmservers->csmMenu, "FbmMainUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmMoniUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmSuspUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmDetUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmMainShift");
 fl_addto_menu (cmservers->csmMenu, "FbmMoniShift");
 fl_addto_menu (cmservers->csmMenu, "FbmSuspShift");
 fl_addto_menu (cmservers->csmMenu, "FbmDetShift");
**/

 /*--- Init the Output parameters ---*/

 DyXformOutputToPanel();
 fl_set_button(mainpanel->writing_data,ON);
 fl_hide_object(mainpanel->writing_data);

 /*--- Init the Dy Debug panel ---*/

 fl_set_input_return (dydebug->debuglvl, FL_RETURN_ALWAYS);

 /*--- Get Virgo logo as a pixmap or bitmap picture ---*/

 fl_set_pixmap_data(mainpanel->logo,logo);
 fl_set_object_label (mainpanel->dyversion, DYVERSION);

 /*--- Init the goodies fonts ---*/

 fl_set_goodies_font(FL_TIMESBOLD_STYLE, FL_MEDIUM_SIZE);
 fl_set_oneliner_color(FL_BLACK, FL_WHEAT);
 fl_set_oneliner_font(FL_TIMESBOLD_STYLE, FL_LARGE_SIZE);
 fl_set_resource("flAlert.dismiss.label","Continue");

 return;
}


/*--------------------------------------------------- DyXformRaz ---*/

void DyXformRaz (FL_OBJECT *obj, long arg)
{
 long nplot, nfile, answer=1;

 DyDebugPrint(__FUNCTION__,0);

 if (obj == mainpanel->resetAll)
    {answer = fl_show_question ("Do you really want to RESET All ?",1);
     if (!answer)  return;
    }

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 nfile = fl_get_browser_maxline(inputfile->file_list);
     
 DyRaz (arg, nplot);

 /*--- reset file list ---*/

 if (arg == 1 || arg == 0)
    {
     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DyXformRaz> Reset files list (%ld files) \n",nfile);
     fl_clear_browser (inputfile->file_list);
     fl_set_object_label (mainpanel->currentInput,"I need some Input Data...");
     DyXformUpdateCounters(0,0,0,0);
    }

 /*--- reset plots and clocks ---*/

 if (arg == 2) 
    {
     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DyXformRaz> Reset plots list (%ld plots) \n",nplot);
     fl_clear_browser(mainbrowser->plots);
    }

 /*--- reset datatodump ---*/

 if (arg == 3 || arg == 0)
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DyXformRaz> Reset datatodump browser \n");
     fl_clear_browser(dumpdata->datatodump);
    }

 /*--- reset datadump ---*/

 if (arg == 4 || arg == 0)
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DyXformRaz> Reset dumpdata browser \n");
     fl_clear_browser(dumpdata->datadumped);
     fl_set_browser_fontstyle(dumpdata->datadumped,FL_FIXED_STYLE);
    }

 /*--- reset reference plots files list ---*/

 if (arg == 6 || arg == 0)
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("DyXformRaz> Reset reference files browser \n");
     RA_DelReferencePlot(NULL);
     fl_clear_browser(reffile->file_list);
     Dy->refplot_color = 0;
     Dy->refplot_indelebile = OFF; fl_set_button(dyoptions->indelebile,OFF);
    }


 /*--- Initialize options and buttons of the main panel ---*/

 if (arg == 0)
    {
     /*--- reset counters and browsers ---*/
     fl_set_object_label (mainpanel->currentInput,"Give me input data...");
     DyXformUpdateCounters(0,0,0,0);
     fl_set_input (mainpanel->startRun, "-1");
     fl_set_input (mainpanel->startFrame, "0");
     fl_set_input (mainpanel->startTime, "-1");
     fl_set_input (mainpanel->startUTCdate, "0");
     fl_set_input (mainpanel->duration, "-1");
     fl_set_input (mainpanel->UTCdate, "0");
     fl_clear_browser (mainbrowser->sms);
     fl_clear_browser (mainbrowser->adc);
     fl_clear_browser (mainbrowser->proc);
/***
     fl_clear_browser (mainbrowser->plots);
***/
     fl_clear_browser (inputfile->file_list);

     /*--- reset buttons states ---*/
     fl_activate_object   (mainpanel->start);
     fl_set_object_lcol   (mainpanel->start, FL_BLACK);
     fl_set_object_lcol   (mainpanel->stop, FL_INACTIVE);
    }

 if (arg == 0) DyInfoLine("All reset!");
 if (arg == 1) DyInfoLine("Input data is reset");
 if (arg == 2) DyInfoLine("List of plots is reset");
 if (arg == 3) DyInfoLine("List of data to dump is reset");
 if (arg == 4) DyInfoLine("Dumped Data is reset");
 if (arg == 5) DyInfoLine("ROOT Canvas removed");
 if (arg == 6) DyInfoLine("Reference plots files removed");

 return;
}


/*------------------------------------------ DyInfoLine ---*/

void DyInfoLine (char *msg)
{
 char dummy[LSTRING_MAX];
char date_heure[24];
 time_t tt;

 tt = time(NULL);
 strftime(date_heure,24,"%d_%b_%Y_%T",localtime(&tt));

 sprintf(dummy,"STATUS_%s : %s", date_heure, msg);
 printf("\033[35;1m%s \033[0m \n",dummy);
 fl_set_object_label (mainpanel->infoLine, dummy);
 fl_check_forms();

 return;
}


/*------------------------------------------ DyInfoPanel ---*/

void DyInfoPanel (char *msg, int alarm)
{
 char *str0=NULL, *str=NULL, *str2=NULL, *ret=NULL;

 if (msg == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 str0 = strstr(msg,"\n");
 if (str0 == NULL) str0 = msg + strlen(msg);

 if (str0-msg > 50)
    {str = msg;
     while (str && str < msg +strlen(msg) - 50)
        {ret = strstr(str+45, "\n");
         if (ret && (ret-str-45 < 10)) {str = ret; *str = '\n'; continue;}
         ret = strstr(str+45, ". ");
         if (ret && (ret-str-45 < 10)) {str = ret; *(str+1) = '\n'; continue;}
         str = strstr(str+45, " ");
         if (str) *str = '\n';
        }
    }

 if (alarm > 0) {fl_trigger_object(mainpanel->stop); fl_show_messages(msg);}
 else           {fl_trigger_object(mainpanel->pause); fl_show_messages(msg); fl_trigger_object(mainpanel->continuer);}

 return;
}


/*------------------------------------------ DyCurrentInput ---*/

void DyCurrentInput (char *msg)
{
 fl_set_object_label (mainpanel->currentInput, msg);
 return;
}


/*------------------------------------------  DyXformUpdateCounters ---*/

void DyXformUpdateCounters (int frameN, int runN, int gtime, int uleaps)
{
 char dummy[LSTRING_MAX];

 sprintf (dummy, "%d", frameN);
 fl_set_input (mainpanel->currentFrame, dummy);

 sprintf (dummy, "%d", runN);
 fl_set_input (mainpanel->currentRun, dummy);

 sprintf (dummy, "%d", gtime);
 fl_set_input (mainpanel->currentGTime, dummy);

 fl_set_input (mainpanel->UTCdate, FrStrUTC(gtime,uleaps));

 sprintf (dummy, "%.1f kB", (double)UFrCurFrameHSize/1024.);
 fl_set_input (mainpanel->currentSize, dummy);

 sprintf (dummy, "Reading data GPS=%d frame=%d size=%.1f kB (%s UTC)", gtime, frameN, (double)UFrCurFrameHSize/1024., FrStrUTC(gtime,uleaps));
 DyInfoLine(dummy);

 return;
}


/*------------------------------------------  DyXformUpdateGTime ---*/

void DyXformUpdateGTime (double gtime, int uleaps)
{
 char dummy[LSTRING_MAX];

 sprintf (dummy, "%.3f", gtime);
 fl_set_input (mainpanel->currentGTime, dummy);
 fl_set_input (mainpanel->UTCdate, FrStrUTC((int)gtime,uleaps));

 return;
}


/*------------------------------------------ DyXformFirstLast ---*/

void DyXformFirstLast (long *start, long *duration, int *startFrame, int *startRun)
{
 const char *dum1, *dum2, *dum3, *dum4;

 DyDebugPrint(__FUNCTION__,0);

 if (fl_get_input(mainpanel->startTime)  == NULL)
    {fl_set_input(mainpanel->startTime,"0");
     fl_set_input (mainpanel->startUTCdate, "0");
    }

 if (fl_get_input(mainpanel->duration)   == NULL)  fl_set_input (mainpanel->duration,"-1");
 if (fl_get_input(mainpanel->startFrame) == NULL)  fl_set_input (mainpanel->startFrame,"-1");
 if (fl_get_input(mainpanel->startRun)   == NULL)  fl_set_input (mainpanel->startRun,"-1");

 /*--- Get starting Time ---*/

 dum1 = fl_get_input(mainpanel->startTime);
 *start = atol(dum1);

 /*--- Get duration ---*/

 dum2 = fl_get_input(mainpanel->duration);
 *duration = atol(dum2);

 /*--- Get startFrame if needed ---*/

 dum3 = fl_get_input(mainpanel->startFrame);
 *startFrame = atoi(dum3);

 /*--- Get startRun if needed ---*/

 dum4 = fl_get_input(mainpanel->startRun);
 *startRun = atoi(dum4);

 return;
}


/*--------------------------------------------------- DyXformStart ---*/

void DyXformStart (FL_OBJECT *obj, long arg)
{
 long nPlot, nDump, nFile;

 DyDebugPrint(__FUNCTION__,1);

 /*--- If no input data, we stop ---*/

 nFile = fl_get_browser_maxline(inputfile->file_list);  

 if (Dy->InputType == INOFF && nFile <= 0 && Dy->frameChannel==0)
    {DyInfoLine ("Please, first choose a source of data by using the Input menu!");
     return;
    }

 /*--- If no plot defined, we stop. Except if we output frames on files ---*/

 nPlot = fl_get_browser_maxline(mainbrowser->plots);
 nDump = fl_get_browser_maxline(dumpdata->datatodump);

 if (nPlot+nDump <= 0 && Dy->output_flag == OFF)
    {DyInfoLine ("There is nothing to display or to dump!");
     return;
    }

 /*--- Configure the main panel buttons ---*/

 fl_deactivate_object (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_INACTIVE);
 fl_deactivate_object (mainpanel->continuer);
 fl_set_object_lcol   (mainpanel->continuer, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextFrame);
 fl_set_object_lcol   (mainpanel->nextFrame, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
/*
 fl_deactivate_object (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/
 fl_activate_object   (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_BLACK);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->stop, FL_BLACK);

 /*--- Deactivate some main panel buttons ---*/

 fl_set_object_lcol   (mainpanel->resetAll, FL_INACTIVE);
 fl_deactivate_object (mainpanel->resetAll);
 fl_set_object_lcol   (mainpanel->cleaning, FL_INACTIVE);
 fl_deactivate_object (mainpanel->cleaning);

 if (Dy->InputType == INCM)
    {fl_deactivate_object (mainpanel->pause);
     fl_set_object_lcol (mainpanel->pause, FL_INACTIVE);
    }

 /*--- Really start reading frames ---*/

 DyStart (nPlot);

 return;
}


/*--------------------------------------------------- DyXformSetNoToc ---*/

void DyXformSetNoToc (FL_OBJECT *obj, long arg)
{
 Dy->notoc = fl_get_button (dyoptions->notoc);

 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {if (Dy->notoc == ON) DyInfoLine("Frames files TOC will NOT be used to access frames");
     else                 DyInfoLine("Frames files TOC will be used to access frames");
    }

 return;
}


/*--------------------------------------------------- DyXformPause ---*/

void DyXformPause (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 fl_deactivate_object (obj);
 fl_set_object_lcol   (obj, FL_INACTIVE);
 fl_deactivate_object (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_INACTIVE);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->stop, FL_BLACK);
 fl_deactivate_object (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_INACTIVE);
 fl_activate_object   (mainpanel->nextFrame);
 fl_set_object_lcol   (mainpanel->nextFrame, FL_BLACK);
 fl_activate_object   (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_BLACK);
/*
 fl_activate_object   (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_BLACK);
*/
 fl_activate_object   (mainpanel->continuer);
 fl_set_object_lcol   (mainpanel->continuer, FL_BLACK);

 DyPause();
 return;
}


/*--------------------------------------------------- DyXformContinue ---*/

void DyXformContinue (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 fl_deactivate_object (obj);
 fl_set_object_lcol   (obj, FL_INACTIVE);
 fl_deactivate_object (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextFrame);
 fl_set_object_lcol   (mainpanel->nextFrame, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
/*
 fl_deactivate_object (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/
 fl_activate_object   (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_BLACK);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->stop, FL_BLACK);

 DyContinue();
 return;
}


/*--------------------------------------------------- DyXformNext ---*/

void DyXformNext (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);
 DyNext(arg);
 return;
}


/*---------------------------------------------------  DyXformStop ---*/

void DyXformStop (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 /*--- Deactivate some main panel buttons ---*/

 fl_deactivate_object (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->stop, FL_INACTIVE);
 fl_activate_object   (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_BLACK);
 fl_deactivate_object (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextFrame);
 fl_set_object_lcol   (mainpanel->nextFrame, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
/*
 fl_deactivate_object (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/
 fl_deactivate_object (mainpanel->continuer);
 fl_set_object_lcol   (mainpanel->continuer, FL_INACTIVE);

 /*--- Reactivate some other main panel buttons ---*/

 fl_set_object_lcol (mainpanel->resetAll, FL_BLACK);
 fl_activate_object (mainpanel->resetAll);
 fl_set_object_lcol (mainpanel->cleaning, FL_BLACK);
 fl_activate_object (mainpanel->cleaning);

 DyStop();
 return;
}


/*------------------------------------------ DyXformMenu ---*/

void DyXformMenu (FL_OBJECT *obj, long arg)
{
 const char *menuItem, *cmname, *cmdomain;
 int error;

 menuItem = fl_get_menu_text(obj);
 if (!menuItem) return;

 /*--- First part of the Input menu of mainpanel ---*/

 if (strcmp(menuItem,READ_FFL) == 0)
    {
     if (Dy->InputType == INCM) DyXformRaz(NULL,0);
     DyXformAddFile(NULL,1);
     Dy->InputType = INFILE;
    }

 if (strcmp(menuItem,READ_FILE) == 0)
    {
     if (Dy->InputType == INCM) DyXformRaz(NULL,0);
     DyXformAddFile(NULL,0);
     Dy->InputType = INFILE;
    }

 if (strcmp(menuItem,CONNECT_CMSERVER) == 0)
    {
     if (Dy->InputType != INCM) DyXformRaz(NULL,0);
     fl_set_button(cmservers->fullList,OFF);
     DyXformUpdateCmList(NULL,0);
     DyXformRaisePanel(NULL,CMSERVERS);
     Dy->InputType = INCM;
    }

 if (strcmp(menuItem,CONNECT_DD) == 0)
    {
     fl_set_input(getdd->ddserver,Dy->DdServer);
     fl_set_input(getdd->inputfile,Dy->Ddinputfile);
     fl_clear_menu (getdd->datasender_list);
     Dy->DdNS = ON;
     DyXformUpdateCmList(getdd->datasender_list,GETDD);
     if (strstr(Dy->Ddinputfile,".") == NULL) fl_trigger_object(getdd->ok);
     DyXformRaisePanel(NULL,GETDD);
     Dy->InputType=INDD;
     Dy->DdNS = OFF;
    }

 if (strcmp(menuItem,READ_SHM) == 0)
    {
     if (Dy->InputType == INCM) DyXformRaz(NULL,0);
     Dy->nshm = DyGetShmList();
     if (Dy->nshm == 0) {DyInfoPanel("No shared memory segments available",1); return;}
     else if (Dy->nshm == 1) DyAddShm(Dy->shmlist[0]);
     else {DyXformUpdateShmList(NULL,0); DyXformRaisePanel(NULL, GETSHM);}
     Dy->InputType = INSHM;
    }

 /*--- Second part of the Input menu of mainpanel ---*/

 if (strcmp(menuItem,READ_WAVE) == 0)
    {
     if (Dy->InputType == INCM) DyXformRaz(NULL,0);
     DyXformAddFile(NULL,-1);
     Dy->InputType = INWAVE;
    }

 if (strcmp(menuItem,READ_DIRECTORY) == 0)
    {
     if (Dy->InputType == INCM) DyXformRaz(NULL,0);
     DyXformAddDir(NULL,0);
    }

 if (strcmp(menuItem,READ_FILE_ONLINE) == 0)
    {
     if (Dy->InputType == INCM) DyXformRaz(NULL,0);
     DyXformAddDir(NULL,1);
     Dy->InputType = INONLINE;
    }

 if (strcmp(menuItem,DECLARE_CM) == 0)
    {
     cmname = fl_show_input("local Cm Name of the data Display", Dy->CmName);
     if (!cmname) return;
     strcpy(Dy->CmName,cmname);
     DyXformRaz(NULL,0);

     strcpy(Dy->CmServer,"...");

     if (strstr(Dy->CmName,"Shift") != 0)
        {if (strstr(Dy->CmName,"Det")  != 0)  strcpy(Dy->CmServer,"FbmDetShift");
         if (strstr(Dy->CmName,"Susp") != 0)  strcpy(Dy->CmServer,"FbmSuspShift");
         if (strstr(Dy->CmName,"Moni") != 0)  strcpy(Dy->CmServer,"FbmMoniShift");
         if (strstr(Dy->CmName,"Main") != 0)  strcpy(Dy->CmServer,"FbmMainShift");
        }

     cmdomain = fl_get_input(cmservers->cmdomain);
     if (cmdomain) strcpy(Dy->CmDomain, cmdomain);
     else          strcpy(Dy->CmDomain,CASCINA);

     if (DyCmConnect(Dy->CmName) == -1) return;
     Dy->InputType = INCM;
     DyReadFirstFrame(NULL, Dy->InputType);
     DyCmDisconnect(Dy->CmName);
    }

 if (strcmp(menuItem,CHECK_FILES) == 0)
    {
     DyXformRaisePanel(NULL,INPUTFILE);
    }

 /*--- Used from getdd panel, choice of a dataSender ---*/

 if (strncmp(menuItem,DDSERVER,10) == 0)
    {
     strcpy(Dy->DdServer, menuItem);
     fl_set_input(getdd->ddserver,Dy->DdServer);
     error = DyDdFFLRequest(Dy->DdServer);
     if (error < 0)
        {strcpy(Dy->Ddffldir,DDDIR);
         fl_clear_browser(getdd->liste);
         fl_add_browser_line(getdd->liste,"NO FFL DIRECTORY SENT BY DATASENDER");
        }
     strcpy(Dy->Ddinputfile, Dy->Ddffldir);
     fl_set_input(getdd->inputfile,Dy->Ddinputfile);
     if (error == 0) fl_trigger_object(getdd->ok);
    }


 /*--- Config Menu of the mainpanel ---*/

 if (strcmp(menuItem,LOAD_CONFIG) == 0)     DyXformGetConfigFile(NULL,0);
 if (strcmp(menuItem,SAVE_CONFIG_AS) == 0)  DyXformSaveConfig(NULL,0);

 return;
}



/************/
/*  PANELS  */
/************/

/*------------------------------------------ DyXformRaisePanel ---*/

void DyXformRaisePanel (FL_OBJECT *obj, long arg)
{
 FL_FORM *form;
 long xmin, ymin, xsize, ysize, offset, xpos, ypos, width, height;
 char dummy[LSTRING_MAX];

 if (arg < NARG_MAX) sprintf(dummy,"DyXformRaisePanel> arg=%d  panel=%s", arg, Dy->typeName[arg]);
 DyDebugPrint(dummy,0);

 if (Dy->BgRun == ON) return;

 xmin = -1200;
 ymin = -760;

 xsize = -mainpanel->mainpanel->w;
 ysize = -mainpanel->mainpanel->h;
 offset = 5;

 switch(arg)
   {
    case MAINPANEL:
       form   = mainpanel->mainpanel;
       width  = form->w;
       height = form->h;
       printf("DyXformRaisePanel> Mainpanel: xsize=%d  ysize=%d  xpos=%d  ypos=%d \n", -xsize, -ysize, XREF-width, YREF-height);
       fl_set_form_position(form, XREF-width, YREF-height);
       if (Dy->AutoConf) sprintf(dummy,"Data_Display_%s:_%s", DYVERSION, Dy->AutoConf);
       else              sprintf(dummy,"Data_Display_%s", DYVERSION);
       fl_show_form(form,FL_PLACE_POSITION,FL_FULLBORDER,dummy);
       break;

    case MAINBROWSER:
       form   = mainbrowser->mainbrowser;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "mainbrowser");
       fl_raise_form(form);
       break;

    case INPUTFILE:
       form   = inputfile->inputfile;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "inputfile");
       fl_raise_form(form);
       break;

    case CMSERVERS:
       form   = cmservers->cmservers;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Cm Servers");
       fl_raise_form(form);
       break;

    case GETDD:
       form   = getdd->getdd;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "getdd");
       fl_raise_form(form);
       break;

    case GETSHM:
       form   = getshm->getshm;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Shared Memory segments");
       fl_raise_form(form);
       break;

    case DUMPDATA:
       form   = dumpdata->dumpdata;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "dumpdata");
       fl_raise_form(form);
       break;

    case PLOTRAW:
       form   = plotraw->plotraw;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotraw");
       fl_raise_form(form);
       break;

    case PLOTTIME:
       form   = plottime->plottime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plottime");
       fl_raise_form(form);
       break;

    case PLOTCORR:
       form   = plottime->plottime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotcorr");
       fl_raise_form(form);
       break;

    case PLOTFFT:
       form   = plotfft->plotfft;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotfft");
       fl_raise_form(form);
       break;

    case PLOTTR:
       form   = plottrfct->plottrfct;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plottr");
       fl_set_object_label(plottrfct->title,"Transfert Function Plot");
       fl_raise_form(form);
       break;

    case PLOTCOHE:
       form   = plottrfct->plottrfct;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotcohe");
       fl_set_object_label(plottrfct->title,"Coherence Plot");
       fl_raise_form(form);
       break;

    case PLOTFFTTIME:
       form   = plotffttime->plotffttime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotffttime");
       fl_raise_form(form);
       break;

    case PLOTTRTIME:
       form   = plotffttime->plotffttime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plottrfcttime");
       fl_set_object_label(plottrfct->title,"TrFct-Time Plot");
       fl_raise_form(form);
       break;

    case PLOTCOHETIME:
       form   = plotffttime->plotffttime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotcohetime");
       fl_set_object_label(plottrfct->title,"Coherence-Time Plot");
       fl_raise_form(form);
       break;

    case PLOT1DTIME:
       form   = plot1dtime->plot1dtime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plot1dtime");
       fl_raise_form(form);
       break;

    case PLOTRAWTIME:
       form   = plotrawtime->plotrawtime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotrawtime");
       fl_raise_form(form);
       break;

    case PLOT1D:
       form   = plot1d->plot1d;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plot1d");
       fl_raise_form(form);
       break;

    case PLOT2D:
       form   = plot2d->plot2d;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plot2d");
       fl_raise_form(form);
       break;

    case PLOT3D:
       form   = plot2d->plot2d;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plot2d");
       fl_raise_form(form);
       break;

    case PLOTAUDIO:
       form   = plotaudio->plotaudio;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotaudio");
       fl_raise_form(form);
       break;

    case GENEPAR:
       form   = plotgenepar->plotgenepar;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotgenepar");
       fl_raise_form(form);
       break;

    case USERFUNCBROWSER:
       form   = userfuncbrowser->userfuncbrowser;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "userfuncbrowser");
       fl_raise_form(form);
       break;

    case HELPPANEL:
       form   = helppanel;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       if (xpos < xmin) xpos = xmin; if (ypos < ymin) ypos = ymin;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Help");
       fl_raise_form(form);
       break;

    case COFFEEPANEL:
       form   = coffeepanel;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Coffee");
       fl_raise_form(form);
       break;

    case USERFCTHELP:
       form   = userprochelp;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_set_form_position(form, XREF-width, YREF+ysize-offset-height);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Help for User Functions");
       fl_raise_form(form);
       break;

    case DYDEBUG:
       form   = dydebug->dydebug;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       if (((Dy->debugType)&(1<<DBG_VDYPLOT))) fl_set_button(dydebug->dbg_vdyplot,ON); 
       if (((Dy->debugType)&(1<<DBG_URPLOT)))  fl_set_button(dydebug->dbg_urplot,ON); 
       if (((Dy->debugType)&(1<<DBG_PLOT)))    fl_set_button(dydebug->dbg_plot,ON); 
       if (((Dy->debugType)&(1<<DBG_FILE)))    fl_set_button(dydebug->dbg_file,ON); 
       if (((Dy->debugType)&(1<<DBG_CM)))      fl_set_button(dydebug->dbg_cm,ON); 
       if (((Dy->debugType)&(1<<DBG_FRAME)))   fl_set_button(dydebug->dbg_frame,ON); 
       if (((Dy->debugType)&(1<<DBG_CHANNEL))) fl_set_button(dydebug->dbg_channel,ON); 
       if (((Dy->debugType)&(1<<DBG_CONFIG)))  fl_set_button(dydebug->dbg_config,ON); 
       if (((Dy->debugType)&(1<<DBG_SPEED)))   fl_set_button(dydebug->dbg_speed,ON); 
       if (((Dy->debugType)&(1<<DBG_OTHERS)))  fl_set_button(dydebug->dbg_others,ON); 
       if (((Dy->debugType)&(1<<DBG_CALLS)))   fl_set_button(dydebug->dbg_calls,ON); 
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Debugging");
       fl_raise_form(form);
       break;

    case DYOPTIONS:
       form   = dyoptions->dyoptions;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Options");
       fl_raise_form(form);
       break;

    case OUTPUTDATA:
       DyXformOutputToPanel();
       form   = dyoutput->dyoutput;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Outputs");
       fl_raise_form(form);
       break;

    case REFFILE:
       form   = reffile->reffile;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Reference files");
       fl_raise_form(form);
       break;

    default:
       printf("DyXformRaisePanel> called with unvalid argument %d\n", arg);
       break;
   }
 return;
}


/*------------------------------------------ DyXformClosePanel ---*/

void DyXformClosePanel (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyXformClosePanel> arg=%d  panel=%s", arg, Dy->typeName[arg]);
 DyDebugPrint(dummy,0);

 if (obj == NULL)                                        return;
 if (obj && obj->form && !fl_form_is_visible(obj->form)) return;
 if (obj && obj->visible != 1)                           return;

 if (arg == HELPPANEL)   {fl_hide_form(helppanel);    fl_free_form(helppanel);}
 if (arg == COFFEEPANEL) {fl_hide_form(coffeepanel);  fl_free_form(coffeepanel);}
 if (arg == USERFCTHELP) {fl_hide_form(userprochelp); fl_free_form(userprochelp);}

 if (obj == mainbrowser->close)     fl_hide_form(mainbrowser->mainbrowser);
 if (obj == inputfile->close)       fl_hide_form(inputfile->inputfile);
 if (obj == cmservers->close)       fl_hide_form(cmservers->cmservers);
 if (obj == getdd->close)           fl_hide_form(getdd->getdd);
 if (obj == getshm->close)          fl_hide_form(getshm->getshm);
 if (obj == dumpdata->close)        fl_hide_form(dumpdata->dumpdata);
 if (obj == userfuncbrowser->close) fl_hide_form(userfuncbrowser->userfuncbrowser);

 if (obj == plotraw->close     ||  obj == plotraw->ok)     fl_hide_form(plotraw->plotraw);
 if (obj == plottime->close    ||  obj == plottime->ok)    fl_hide_form(plottime->plottime);
 if (obj == plotfft->close     ||  obj == plotfft->ok)     fl_hide_form(plotfft->plotfft);
 if (obj == plotffttime->close ||  obj == plotffttime->ok) fl_hide_form(plotffttime->plotffttime);
 if (obj == plot1dtime->close  ||  obj == plot1dtime->ok)  fl_hide_form(plot1dtime->plot1dtime);
 if (obj == plotrawtime->close ||  obj == plotrawtime->ok) fl_hide_form(plotrawtime->plotrawtime);
 if (obj == plottrfct->close   ||  obj == plottrfct->ok)   fl_hide_form(plottrfct->plottrfct);

 if (obj == plot1d->close      ||  obj == plot1d->ok)      fl_hide_form(plot1d->plot1d);
 if (obj == plot2d->close      ||  obj == plot2d->ok)      fl_hide_form(plot2d->plot2d);
 if (obj == plotaudio->close   ||  obj == plotaudio->ok)   fl_hide_form(plotaudio->plotaudio);
 if (obj == plotgenepar->close ||  obj == plotgenepar->ok) fl_hide_form(plotgenepar->plotgenepar);

 if (obj == dydebug->close)   fl_hide_form(dydebug->dydebug);
 if (obj == dyoptions->close) fl_hide_form(dyoptions->dyoptions);
 if (obj == dyoutput->close)  fl_hide_form(dyoutput->dyoutput);
 if (obj == reffile->close)   fl_hide_form(reffile->reffile);

 return;
}



/*------------------------------------------ DyXformCleanPanel ---*/

void DyXformCleanPanel (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyXformCleanPanel> arg=%d  panel=%s", arg, Dy->typeName[arg]);
 DyDebugPrint(dummy,0);

 switch(arg)
   {
    case PLOTRAW:
       fl_set_button(plotraw->average_fft_clone, OFF);
       fl_set_button(plotraw->yMin_clone, OFF);
       fl_set_button(plotraw->yMax_clone, OFF);
       break;

    case PLOTTIME:
       fl_set_button (plottime->tsize_clone, OFF);
       fl_set_button (plottime->sampFreq_clone, OFF);
       fl_set_button (plottime->tstep_clone, OFF);
       fl_set_button (plottime->yMin_clone, OFF);
       fl_set_button (plottime->yMax_clone, OFF);
       fl_set_button (plottime->yoffset_clone, OFF);
       fl_set_button (plottime->yscale_clone, OFF);
       fl_set_button (plottime->filterMin_clone, OFF);
       fl_set_button (plottime->filterMax_clone, OFF);
       break;

    case PLOTCORR:
       fl_set_button (plottime->tsize_clone, OFF);
       fl_set_button (plottime->sampFreq_clone, OFF);
       fl_set_button (plottime->tstep_clone, OFF);
       fl_set_button (plottime->yMin_clone, OFF);
       fl_set_button (plottime->yMax_clone, OFF);
       fl_set_button (plottime->yoffset_clone, OFF);
       fl_set_button (plottime->yscale_clone, OFF);
       fl_set_button (plottime->filterMin_clone, OFF);
       fl_set_button (plottime->filterMax_clone, OFF);
       break;

    case PLOTFFT:
       fl_set_button (plotfft->tsize_clone, OFF);
       fl_set_button (plotfft->sampFreq_clone, OFF);
       fl_set_button (plotfft->tstep_clone, OFF);
       fl_set_button (plotfft->average_fft_clone, OFF);
       fl_set_button (plotfft->refresh_fft_clone, OFF);
       fl_set_button (plotfft->xMin_clone, OFF);
       fl_set_button (plotfft->xMax_clone, OFF);
       break;

    case PLOTTR:
       fl_set_button (plottrfct->tsize_clone, OFF);
       fl_set_button (plottrfct->sampFreq_clone, OFF);
       fl_set_button (plottrfct->tstep_clone, OFF);
       fl_set_button (plottrfct->average_fft_clone, OFF);
       fl_set_button (plottrfct->refresh_fft_clone, OFF);
       fl_set_button (plottrfct->xMin_clone, OFF);
       fl_set_button (plottrfct->xMax_clone, OFF);
       break;

    case PLOTCOHE:
       fl_set_button (plottrfct->tsize_clone, OFF);
       fl_set_button (plottrfct->sampFreq_clone, OFF);
       fl_set_button (plottrfct->tstep_clone, OFF);
       fl_set_button (plottrfct->average_fft_clone, OFF);
       fl_set_button (plottrfct->refresh_fft_clone, OFF);
       fl_set_button (plottrfct->xMin_clone, OFF);
       fl_set_button (plottrfct->xMax_clone, OFF);
       fl_set_button (plottrfct->crossCohe_fmin_clone, OFF);
       fl_set_button (plottrfct->crossCohe_fmax_clone, OFF);
       break;

    case PLOTFFTTIME:
       fl_set_button (plotffttime->xMin_clone, OFF);
       fl_set_button (plotffttime->xMax_clone, OFF);
       fl_set_button (plotffttime->refresh_percent_clone, OFF);
       fl_set_button (plotffttime->timeSize_clone, OFF);
       fl_set_button (plotffttime->dfreq_clone, OFF);
       fl_set_button (plotffttime->dtime_clone, OFF);
       fl_set_button (plotffttime->sampFreq_clone, OFF);
       fl_set_button (plotffttime->tstep_clone, OFF);
       fl_set_button (plotffttime->zmin_clone, OFF);
       fl_set_button (plotffttime->zmax_clone, OFF);
       break;

    case PLOTTRTIME:
       fl_set_button (plotffttime->xMin_clone, OFF);
       fl_set_button (plotffttime->xMax_clone, OFF);
       fl_set_button (plotffttime->refresh_percent_clone, OFF);
       fl_set_button (plotffttime->timeSize_clone, OFF);
       fl_set_button (plotffttime->dfreq_clone, OFF);
       fl_set_button (plotffttime->dtime_clone, OFF);
       fl_set_button (plotffttime->sampFreq_clone, OFF);
       fl_set_button (plotffttime->tstep_clone, OFF);
       fl_set_button (plotffttime->zmin_clone, OFF);
       fl_set_button (plotffttime->zmax_clone, OFF);
       break;

    case PLOTCOHETIME:
       fl_set_button (plotffttime->xMin_clone, OFF);
       fl_set_button (plotffttime->xMax_clone, OFF);
       fl_set_button (plotffttime->refresh_percent_clone, OFF);
       fl_set_button (plotffttime->timeSize_clone, OFF);
       fl_set_button (plotffttime->dfreq_clone, OFF);
       fl_set_button (plotffttime->dtime_clone, OFF);
       fl_set_button (plotffttime->sampFreq_clone, OFF);
       fl_set_button (plotffttime->tstep_clone, OFF);
       fl_set_button (plotffttime->zmin_clone, OFF);
       fl_set_button (plotffttime->zmax_clone, OFF);
       break;

    case PLOT1DTIME:
       fl_set_button (plot1dtime->timeSize_clone, OFF);
       fl_set_button (plot1dtime->tsize_clone, OFF);
       fl_set_button (plot1dtime->sampFreq_clone, OFF);
       fl_set_button (plot1dtime->xnBin_clone, OFF);
       fl_set_button (plot1dtime->refresh_percent_clone, OFF);
       fl_set_button (plot1dtime->xMin_clone, OFF);
       fl_set_button (plot1dtime->xMax_clone, OFF);
       fl_set_button (plot1dtime->zmin_clone, OFF);
       fl_set_button (plot1dtime->zmax_clone, OFF);
       break;

    case PLOTRAWTIME:
       fl_set_button (plotrawtime->timeSize_clone, OFF);
       fl_set_button (plotrawtime->xnBin_clone, OFF);
       fl_set_button (plotrawtime->average_fft_clone, OFF);
       fl_set_button (plotrawtime->refresh_percent_clone, OFF);
       fl_set_button (plotrawtime->xMin_clone, OFF);
       fl_set_button (plotrawtime->xMax_clone, OFF);
       fl_set_button (plotrawtime->zmin_clone, OFF);
       fl_set_button (plotrawtime->zmax_clone, OFF);
       break;

    case PLOT1D:
       fl_set_button (plot1d->xnBin_clone, OFF);
       fl_set_button (plot1d->xMin_clone, OFF);
       fl_set_button (plot1d->xMax_clone, OFF);
       fl_set_button (plot1d->tstep_clone, OFF);
       fl_set_button (plot1d->sampFreq_clone, OFF);
       break;

    case PLOT2D:
       fl_set_button (plot2d->xnBin_clone, OFF);
       fl_set_button (plot2d->xMin_clone, OFF);
       fl_set_button (plot2d->xMax_clone, OFF);
       fl_set_button (plot2d->ynBin_clone, OFF);
       fl_set_button (plot2d->yMin_clone, OFF);
       fl_set_button (plot2d->yMax_clone, OFF);
       fl_set_button (plot2d->nSampling_clone, OFF);
       fl_set_button (plot2d->tstep_clone, OFF);
       break;

    case PLOT3D:
       fl_set_button (plot2d->xnBin_clone, OFF);
       fl_set_button (plot2d->xMin_clone, OFF);
       fl_set_button (plot2d->xMax_clone, OFF);
       fl_set_button (plot2d->ynBin_clone, OFF);
       fl_set_button (plot2d->yMin_clone, OFF);
       fl_set_button (plot2d->yMax_clone, OFF);
       fl_set_button (plot2d->nSampling_clone, OFF);
       fl_set_button (plot2d->tstep_clone, OFF);
       break;

    case PLOTAUDIO:
       fl_set_button (plotaudio->duration_clone, OFF);
       fl_set_button (plotaudio->t0_clone, OFF);
       break;

    case GENEPAR:
       fl_set_button (plotgenepar->all_clone, OFF);
       break;

    default:
       break;
   }
 return;
}



/*******************/
/*  Cm CONNECTION  */
/*******************/


/*------------------------------------------ DyXformCmName ---*/

void DyXformCmName (FL_OBJECT *obj, long arg)
{
 int i0;
 const char *name;

 DyDebugPrint(__FUNCTION__,0);

 i0 = fl_get_browser(cmservers->liste);
 if (i0 == 0) return;
 if (i0 < 0) i0 = -i0;

 name = fl_get_browser_line(cmservers->liste, i0);
 fl_set_input(cmservers->cmname, name);

 return;
}


/*------------------------------------------ DyXformCmConnect ---*/

void DyXformCmConnect (FL_OBJECT *obj, long arg)
{
 const char *name, *cmdomain;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 name = fl_get_input(cmservers->cmname);

 strcpy(Dy->CmServer, name);

 cmdomain = fl_get_input(cmservers->cmdomain);
 if (cmdomain) strcpy(Dy->CmDomain, cmdomain);
 else          strcpy(Dy->CmDomain,CASCINA);

 sleep(2);
 if (DyCmConnect(NULL) == -1) return;

 DyReadFirstFrame (NULL, INCM);

 DyCmDisconnect(Dy->CmName);

 if (obj != NULL)  DyXformClosePanel (cmservers->close, CMSERVERS);

 return;
}


/*------------------------------------------ DyXformUpdateCmList ---*/

void DyXformUpdateCmList (FL_OBJECT *obj, long arg)
{
 const char *cmdomain;

 DyDebugPrint(__FUNCTION__,0);

 if (obj == getdd->datasender_list) cmdomain = fl_get_input(getdd->cmdomain);
 else                               cmdomain = fl_get_input(cmservers->cmdomain);

 if (cmdomain) strcpy(Dy->CmDomain, cmdomain);
 else          strcpy(Dy->CmDomain,CASCINA);

 DyUpdateCmList();
 return;
}


/*******************/
/* Shm CONNECTION  */
/*******************/

/*------------------------------------------ DyXformAddShm ---*/

void DyXformAddShm (FL_OBJECT *obj, long arg)
{
 const char *shmname;
 int i0;

 DyDebugPrint(__FUNCTION__,0);

 i0 = fl_get_browser(getshm->liste);
 if (i0 == 0) return;
 if (i0 < 0) i0 = -i0;

 shmname = fl_get_browser_line(getshm->liste, i0);

 DyAddShm((char *)shmname);
 return;
}

/*------------------------------------------ DyXformUpdateShmList ---*/

void DyXformUpdateShmList (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 DyUpdateShmList();
 return;
}



/*************************/
/*  Data Sender CONNECT  */
/*************************/

/*-------------------------------------------- DyXformDdGetFile ---*/

void DyXformDdGetFile (FL_OBJECT *obj, long arg)
{
 const char *filename;
 char dummy[LSTRING_MAX];
 int i0,i;

 DyDebugPrint(__FUNCTION__,0);

 i0 = fl_get_browser(getdd->liste);
 if (i0 <= 0) return;

 filename = fl_get_browser_line(getdd->liste, i0);

 if (filename)
    {if (strcmp(filename,"..") == 0)
        {for (i=strlen(Dy->DdDir)-1; i>0; i--) {if (Dy->DdDir[i] == '/') {Dy->DdDir[i]='\0'; break;}}
         if (strlen(Dy->DdDir) == 0) strcpy(Dy->DdDir,"/");
         strcpy(dummy, Dy->DdDir);
        }
     else
        {sprintf(dummy, "%s/%s", Dy->DdDir, filename);}

     fl_set_input(getdd->inputfile, dummy);

     if (strstr(filename,".") == NULL  || strcmp(filename,"..") == 0)
        {fl_deactivate_object(getdd->ok);
         DyXformDdGetChlist(getdd->ok,0);
         fl_activate_object(getdd->ok);
        }
    }

 return;
}


/*-------------------------------------------- DyXformDdGetChlist ---*/

void DyXformDdGetChlist (FL_OBJECT *obj, long arg)
{
 int result;
 const char *ddserver, *inputfile, *cmdomain;
 char dummy[LSTRING_MAX];
 const char *answer;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Ddpasswd == NULL)
    {answer = fl_show_input("You need to provide a password: ", "");
     if (answer)
        {Dy->Ddpasswd = (char *)malloc(64);
         if (strlen(answer) > 0) strcpy(Dy->Ddpasswd,answer);
         else                    strcpy(Dy->Ddpasswd,"NULL");
        }
     else return;
    }

 cmdomain = fl_get_input(getdd->cmdomain);
 if (cmdomain) strcpy(Dy->CmDomain, cmdomain);
 else          strcpy(Dy->CmDomain,CASCINA);

 ddserver = fl_get_input(getdd->ddserver);
 if (!ddserver) return;
 strcpy(Dy->DdServer, (char *)ddserver);

 inputfile = fl_get_input(getdd->inputfile);
 if (!inputfile) return;
 strcpy(Dy->Ddinputfile, (char *)inputfile);

 sleep(2);
 if (DyCmConnect(Dy->CmName) == -1) return;

 result = DyReadFirstFrame (NULL, INDD);

 DyCmDisconnect(Dy->CmName);

 if (result == -2) strcpy(Dy->DdDir,Dy->Ddinputfile);

 if (obj != NULL && result >= 0)
    {
     fl_deselect_browser(getdd->liste); 
     DyXformClosePanel (getdd->close, GETDD);
    }

 return;
}




/***********/
/*  PLOTS  */
/***********/

/*-------------------------------------------- DyXformSavePlot ---*/

void DyXformSavePlot (FL_OBJECT *obj, long arg)
{
 const char *str;
 char dummy[LSTRING_MAX];
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Save current plots in a file ---*/

 if (Dy->savedPlots) strcpy(dummy, Dy->savedPlots);
 str = fl_show_input("Plots saved in file", dummy);
 if (str) strcpy(Dy->savedPlots, (char *)str);
 else     return;
 ok = DySavePlot (Dy->savedPlots);
 if (ok)
    {sprintf(dummy, "Current plots saved in %s", Dy->savedPlots);
     DyInfoLine (dummy);
    }

 return;
}


/*-------------------------------------------- DyXformSetRefPlot ---*/

void DyXformSetRefPlot (FL_OBJECT *obj, long arg)
{
 const char *str;
 char dummy[LSTRING_MAX];
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Save reference plots in a file ---*/

 if (Dy->refPlots) strcpy(dummy, Dy->refPlots);
 else              strcpy(dummy, DDREFPLOTS);
 str = fl_show_input("Reference Plots saved in file", dummy);
 if (!str) return;
 if (Dy->refPlots == NULL) Dy->refPlots = (char *)malloc(LSTRING_MAX);
 strcpy(Dy->refPlots, (char *)str);
 ok = DySavePlot (Dy->refPlots);
 if (ok)
    {sprintf(dummy, "Reference plots saved in %s", Dy->refPlots);
     DyInfoLine (dummy);
    }

 return;
}


/*-------------------------------------------- DyXformGetRefPlot ---*/

void DyXformGetRefPlot (FL_OBJECT *obj, long arg)
{
 const char *answer;
 char dummy[LSTRING_MAX], filename[LSTRING_MAX], *blank;
 int color, ok, nFile, size, result_length;
 char *result, *str, pattern_str[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 size = LSTRING_MAX * NFILE_MAX;
 result = (char *)malloc(size);
 while (result == NULL)
    {size = size / 2;
     result = (char *)malloc(size);
    }

 if (Dy->refPlots) strcpy(dummy, Dy->refPlots);
 else              strcpy(dummy, DDREFPLOTS);
/*
 answer = fl_show_input("Get Reference Plots from file", dummy);
 if (answer)
    {
     sscanf(answer, "%s", &filename[0]);
     if ((blank = strstr(answer,SEPTITLE)) != NULL)
        {color = atoi(blank+strlen(SEPTITLE));}
     else
        {if (Dy->refplot_color <= 1)      color = 51;
         else if (Dy->refplot_color < 10) color = Dy->refplot_color + 1;
         else                             color = 10 * ((Dy->refplot_color-6)/10 + 1) + 6;
        }
     ok = RA_GetReferencePlot (filename, color);
     if (ok)
        {DyXformAddToRefList(filename, color);
         sprintf(dummy, "Superposed with color %d the reference plots from %s", Dy->refplot_color, filename);
         DyInfoLine (dummy);
        }
    }
*/

 sprintf(pattern_str, "*%s", ROOT_SUFFIX);
 result_length = ClXFSelector("Get Reference Plots from file", Dy->refplot_dir, pattern_str, "", result, size);

 if (result_length == 0 || !result) return;

 str = strtok(result," ");
 while (str != NULL)
    {if (is_a_file(str) == 0) {str = strtok(NULL," "); continue;}
     nFile++;
     if (nFile == 1) Dy->refplot_dir = DyGetDir(Dy->refplot_dir, str);
      printf("DyXformGetRefPlot> file%d=%s\n",nFile, str);
      sscanf(str, "%s", &filename[0]);
      if ((blank = strstr(str,SEPTITLE)) != NULL)
         {color = atoi(blank+strlen(SEPTITLE));}
      else
         {if (Dy->refplot_color <= 1)      color = 51;
          else if (Dy->refplot_color < 10) color = Dy->refplot_color + 1;
          else                             color = 10 * ((Dy->refplot_color-6)/10 + 1) + 6;
         }
      ok = RA_GetReferencePlot (filename, color);
      if (ok > 0)
         {DyXformAddToRefList(filename, color);
          sprintf(dummy, "Superposed with color %d the reference plots from %s", Dy->refplot_color, filename);
          DyInfoLine (dummy);
         }
      else
         {sprintf(dummy, "No reference plot found in %s", filename);
          DyInfoLine (dummy);
         }
     str = strtok(NULL," ");
    }
 free(result);

 return;
}


/*------------------------------------------ DyXformAddToRefList ---*/

void DyXformAddToRefList (char *fileName, int color)
{
 int i0, i, nfile;
 const char *str;
 char name[LSTRING_MAX], dummy[LSTRING_MAX];

 nfile = fl_get_browser_maxline(reffile->file_list);
 i0 = -1;
 for (i=0; i<nfile; i++)
    {str = fl_get_browser_line(reffile->file_list, i+1);
     sscanf(str,"%s",name);
     if (strcmp(fileName, name) == 0) {i0 = i; break;}
    } 
 if (i0 == -1) {sprintf(dummy,"%s__%d", fileName, color); fl_add_browser_line (reffile->file_list, dummy);}
 else          printf("DyXformAddToRefList> %s already present in reference files list \n", fileName);

 Dy->refplot_indelebile = ON; fl_set_button(dyoptions->indelebile,ON);
 Dy->refplot_color = color;
 if (Dy->refPlots == NULL) Dy->refPlots = (char *)malloc(LSTRING_MAX);
 strcpy(Dy->refPlots,fileName);

 return;
}


/*------------------------------------------- DyXformRemoveRefFile ---*/

void DyXformRemoveRefFile (FL_OBJECT *obj, long arg)
{
 long i, i0, nfile;
 const char *str0, *str, *last;
 char filename[LSTRING_MAX], *sep;
 int color;

 nfile = fl_get_browser_maxline(reffile->file_list);
 if (nfile <= 0) return;

 if (nfile == 1)
    {DyXformRaz(NULL,6);}
 else
    {i0 = fl_get_browser(reffile->file_list);
     str0 = fl_get_browser_line(reffile->file_list,i0);
     sep = strstr(str0,SEPTITLE);
     strcpy(filename, str0); filename[sep-str0]='\0';
     color = atoi(sep+strlen(SEPTITLE));
     RA_DelReferencePlot(filename);
     printf("DyXformRemoveRefFile> Removed %s  color=%d from reference files list\n", filename, color);
     for (i=i0-1; i<nfile-1; i++)
        {str = fl_get_browser_line(reffile->file_list,i+2);
         fl_replace_browser_line(reffile->file_list,i+1,str);
        }
     fl_delete_browser_line(reffile->file_list,nfile);
     last = fl_get_browser_line(reffile->file_list,nfile-1);
     sep = strstr(last,SEPTITLE);
     strcpy(Dy->refPlots, last); Dy->refPlots[sep-last]='\0';
     Dy->refplot_color = atoi(sep+strlen(SEPTITLE));
    }

 return;
}

/*------------------------------------------- DyXformRemove2DRefFile ---*/

void DyXformRemove2DRefFile (FL_OBJECT *obj, long arg)
{
 long i, i0, nfile;
 const char *str0, *str, *last;
 char filename[LSTRING_MAX], *sep;
 int color;

 nfile = fl_get_browser_maxline(reffile->file_list);
 if (nfile <= 0) return;

 i0 = fl_get_browser(reffile->file_list);
 if (i0 == 0) return;

 for (i=i0; i<=nfile; i++)
    {
     if (fl_isselected_browser_line(reffile->file_list,i))
        {str0 = fl_get_browser_line(reffile->file_list,i);
         sep = strstr(str0,SEPTITLE);
         strcpy(filename, str0); filename[sep-str0]='\0';
         color = atoi(sep+strlen(SEPTITLE));
         RA_Del2DReferencePlot(filename);
         printf("DyXformRemove2DRefFile> Removed all 2D plots of %s color=%d from reference files list\n", filename, color);
        }
    }

 return;
}


/*------------------------------------------- DyXformEditRefFile ---*/

void DyXformEditRefFile (FL_OBJECT *obj, long arg)
{
 int i0, nfile, color;
 const char *str, *answer;
 char dummy[LSTRING_MAX], filename[LSTRING_MAX], *blank, *sep;

 nfile = fl_get_browser_maxline(reffile->file_list);
 if (nfile <= 0) return;

 i0 = fl_get_browser(reffile->file_list);
 str = fl_get_browser_line(reffile->file_list,i0);

 answer = fl_show_input("Change color number:", (char *)str);

 if (answer && strlen(answer) > 0)
    {sep = strstr(answer,SEPTITLE);
     strcpy(filename, answer); filename[sep-answer]='\0';
     color = atoi(sep+strlen(SEPTITLE));
     if (color <=0 && (blank = strstr(answer," ")) != NULL) color = atoi(blank+1);
     sep = strstr(str,SEPTITLE);
     strcpy(filename, str); filename[sep-str]='\0';
     sprintf(dummy,"%s%s%ld", filename, SEPTITLE, color);
     fl_replace_browser_line(reffile->file_list,i0,dummy);
     RA_ModifyReferencePlot(filename, color);
    }

 return;
}


/*------------------------------------------ DyXformHideShowPlot ---*/

void DyXformHideShowPlot (FL_OBJECT *obj, long arg)
{
 const char *str;
 char browserLine[LSTRING_MAX];
 long nplot, i;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 if (arg == 0)
    {
     for (i=1; i<=nplot; i++)
        {if (fl_isselected_browser_line(mainbrowser->plots,i))
            {Plot[i]->hidden = ON;
             str = fl_get_browser_line(mainbrowser->plots,i);
             if (str && strncmp(str,"@C",2) != 0)
                {sprintf (browserLine, "@C7 %s", str);
                 fl_replace_browser_line(mainbrowser->plots, i, browserLine);
                }
             fl_deselect_browser_line(mainbrowser->plots,i);
            }
        }
    }

 else
    {
     for (i=1; i<=nplot; i++)
        {if (fl_isselected_browser_line(mainbrowser->plots,i))
            {Plot[i]->hidden = OFF;
             str = fl_get_browser_line(mainbrowser->plots,i);
             if (str && strncmp(str,"@C",2) == 0)
                {sprintf (browserLine, "%s", &str[4]);
                 fl_replace_browser_line(mainbrowser->plots, i, browserLine);
                }
             fl_deselect_browser_line(mainbrowser->plots,i);
            }
        }
    }

 return;
}


/*------------------------------------------ DyXformTransformPlot ---*/

void DyXformTransformPlot (FL_OBJECT *obj, long arg)
{
 const char *str;
 const char *answer;
 char  browserLine[LSTRING_MAX];
 long nplot, i, i0, flag2ch, newtype;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 /*--- Check if  at least one plot is selected ---*/

 i0 = fl_get_browser(mainbrowser->plots);
 if (i0 == 0) return;
/**
 i0 = 0;
 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i)) {i0 = i; break;}}
 if (i0 == 0) return;
**/

 /*--- Get and plot type number or plot type name ---*/

 answer = fl_show_input("New plot type", Dy->typeName[Plot[i0]->type]);
 if (!answer) return;
 newtype = -1;
 for (i=0; i<NARG_MAX; i++) {if (strcmp(answer,Dy->typeName[i]) == 0) {newtype = i; break;}}
 if (newtype == -1)
    {if (strcmp(answer,"TIME") == 0)      newtype = PLOTTIME;
     if (strcmp(answer,"CORR") == 0)      newtype = PLOTCORR;
     if (strcmp(answer,"FFT") == 0)       newtype = PLOTFFT;
     if (strcmp(answer,"FFTTIME") == 0)   newtype = PLOTFFTTIME;
     if (strcmp(answer,"TRFCT") == 0)     newtype = PLOTTR;
     if (strcmp(answer,"TRFCTTIME") == 0) newtype = PLOTTRTIME;
     if (strcmp(answer,"COHE") == 0)      newtype = PLOTCOHE;
     if (strcmp(answer,"COHETIME") == 0)  newtype = PLOTCOHETIME;
     if (strcmp(answer,"1D") == 0)        newtype = PLOT1D;
     if (strcmp(answer,"1DTIME") == 0)    newtype = PLOT1DTIME;
     if (strcmp(answer,"2D") == 0)        newtype = PLOT2D;
     if (strcmp(answer,"3D") == 0)        newtype = PLOT3D;
     if (strcmp(answer,"RAW") == 0)       newtype = PLOTRAW;
     if (strcmp(answer,"RAWTIME") == 0)   newtype = PLOTRAWTIME;
     if (strcmp(answer,"AUDIO") == 0)     newtype = PLOTAUDIO;
    }

 /*--- Check if new plot type is a 2 channels plot ---*/

 flag2ch = 0;
 if (newtype == PLOTTR || newtype == PLOTCOHE || newtype == PLOTTRTIME || newtype == PLOTCOHETIME ||
     newtype == PLOT2D || newtype == PLOTCORR) flag2ch = 1;

 /*--- Transform into newtype plot all the plots selected ---*/

 for (i=1; i<=nplot; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i))
       {
        if ((strstr(Plot[i]->name,SEP2VAR) || strstr(Plot[i]->name,SEP2VARTR)) && flag2ch == 0)           {printf("type=2ch  newtype=1ch\n"); continue;}
        if (strstr(Plot[i]->name,SEP2VAR)==NULL && strstr(Plot[i]->name,SEP2VARTR)==NULL && flag2ch == 1) {printf("type=1ch  newtype=2ch\n"); continue;}

        Plot[i]->type = newtype;
        if (Plot[i]->type == PLOTTR || Plot[i]->type == PLOTTRTIME) sprintf(Plot[i]->name, "%s%s%s", Plot[i]->vVarx, SEP2VARTR, Plot[i]->vVary);
        else if (flag2ch == ON)                                     sprintf(Plot[i]->name, "%s%s%s", Plot[i]->vVarx, SEP2VAR, Plot[i]->vVary);
        else                                                        sprintf(Plot[i]->name, "%s", Plot[i]->vVarx);
        DyPutDefaultInPlot(i);
        str = fl_get_browser_line(mainbrowser->plots,i);
        if (strncmp(str,"@C",2) == 0) sprintf (browserLine, "@C7 %d %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
        else                          sprintf (browserLine, "%d %s %s", i, Dy->typeName[Plot[i]->type], Plot[i]->name);
        browserLine[strlen(browserLine)] = '\0';
        fl_replace_browser_line(mainbrowser->plots, i, browserLine);
        fl_deselect_browser_line(mainbrowser->plots,i);
       }
    }

 return;
}


/*------------------------------------------ DyXformUpdatePlotBrowser ---*/

void DyXformUpdatePlotBrowser (long id, long arg)
{
 const char *str;
 char browserLine[LSTRING_MAX];
 long i,nplot;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 if (id > 0)
    {
     if (nplot >= NPLOT_MAX-1 && arg == ADD_PLOT) {printf("  Maximum number of plots reached. Cannot add more. \n"); return;}

     if (Plot[id]->superposed == ON)
        {if (Plot[id]->hidden == ON) sprintf (browserLine, "@C7 %ld %s %s (s)", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
         else                        sprintf (browserLine, "%ld %s %s (s)", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
        }
     else
        {if (Plot[id]->hidden == ON) sprintf (browserLine, "@C7 %ld %s %s", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
         else                        sprintf (browserLine, "%ld %s %s", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
        }

     if (arg == ADD_PLOT)     fl_add_browser_line(mainbrowser->plots, browserLine);
     if (arg == REPLACE_PLOT) fl_replace_browser_line(mainbrowser->plots, id, browserLine);
    }

 else
    {
     fl_clear_browser(mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {
         if (Plot[id]->superposed == ON)
            {if (Plot[id]->hidden == ON) sprintf (browserLine, "@C7 %ld %s %s (s)", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
             else                        sprintf (browserLine, "%ld %s %s (s)", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
            }
         else
            {if (Plot[id]->hidden == ON) sprintf (browserLine, "@C7 %ld %s %s", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
             else                        sprintf (browserLine, "%ld %s %s", id, Dy->typeName[Plot[id]->type], Plot[id]->name);
            }
        fl_add_browser_line(mainbrowser->plots, browserLine);
       }
   }

 return;
}


/*------------------------------------------- DyXformPutDefaultInPanel ---*/

void DyXformPutDefaultInPanel (FL_OBJECT *obj, long arg)
{
 long id, state;

 DyDebugPrint(__FUNCTION__,0);

 id = fl_get_browser(mainbrowser->plots);
 if (id <= 0) return;

 DyCopyPlot(NPLOT_MAX-1,id);
 DyPutDefaultInPlot(NPLOT_MAX-1);
 DyXformPlotToPanel(NPLOT_MAX-1);

 return;
}



/***************/
/*  VARIABLES  */
/***************/

/*------------------------------------------- DyXformRootUtil ---*/

void DyXformRootUtil (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 if (obj == dyoptions->zoomstatm)       RA_ZoomAllStat (-0.1);
 if (obj == dyoptions->zoomstatp)       RA_ZoomAllStat (0.1);

 if (obj == dyoptions->zoomlegendm)     RA_ZoomAllLegends (-0.1);
 if (obj == dyoptions->zoomlegendp)     RA_ZoomAllLegends (0.1);

 if (obj == dyoptions->zoomtimestringm) RA_ZoomAllTimeString (-0.1);
 if (obj == dyoptions->zoomtimestringp) RA_ZoomAllTimeString (0.1);

 if (obj == dyoptions->zoomtitlem)      RA_ZoomAllTitle (-0.1);
 if (obj == dyoptions->zoomtitlep)      RA_ZoomAllTitle (0.1);

 if (obj == dyoptions->zoomlabelxm)     RA_ZoomAllLabels (-0.1,"xaxis");
 if (obj == dyoptions->zoomlabelxp)     RA_ZoomAllLabels (0.1,"xaxis");
 if (obj == dyoptions->zoomlabelym)     RA_ZoomAllLabels (-0.1,"yaxis");
 if (obj == dyoptions->zoomlabelyp)     RA_ZoomAllLabels (0.1,"yaxis");

 if (obj == dyoptions->loglinx)         RA_LogLinAll ("xaxis");
 if (obj == dyoptions->logliny)         RA_LogLinAll ("yaxis");

 return;
}


/*------------------------------------------- DyXformDeSelect ---*/

void DyXformDeSelect (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 if (obj == mainbrowser->deselectsms)     fl_deselect_browser(mainbrowser->sms);
 if (obj == mainbrowser->deselectadc)     fl_deselect_browser(mainbrowser->adc);
 if (obj == mainbrowser->deselectproc)    fl_deselect_browser(mainbrowser->proc);
 if (obj == mainbrowser->deselectsim)     fl_deselect_browser(mainbrowser->sim);
 if (obj == mainbrowser->deselectPlots)   fl_deselect_browser(mainbrowser->plots);

 return;
}



/*------------------------------------------ DyXformSortBrowser ---*/

void DyXformSortBrowser (FL_OBJECT *browser)
{
 int i, j, nline, flag, sort_order;
 const char *str1, *str2;
 char **namelist, dummy[LSTRING_MAX];

 if (browser == NULL) return;

 nline = fl_get_browser_maxline(browser); 
 if (nline <= 1) return;

 sprintf(dummy, "DyXformSortBrowser> browser=%s  nline=%d", browser->label, nline);
 DyDebugPrint(dummy,0);

 /*--- Do not sort if this is already sorted ---*/

 flag = 0;
 str1 = fl_get_browser_line (browser, 1);
 str2 = fl_get_browser_line (browser, 2);
 sort_order = strcmp(str1, str2);
 for (i=1; i<nline; i++)
    {str1 = fl_get_browser_line (browser, i);
     str2 = fl_get_browser_line (browser, i+1);
     if (strcmp(str1, str2) != sort_order) flag = 1;
    }

 if (flag == 0  &&  sort_order < 0)
    {if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("DyXformSortBrowser> Channels list already sorted\n"); return;}

 if (flag == 0  &&  sort_order > 0)
    {
     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("DyXformSortBrowser> Channels list already sorted but in wrong order.\n"); 
     namelist = (char **)calloc(nline+1,sizeof(char *));
     for (i=1; i<=nline; i++)
        {
         str1 = fl_get_browser_line (browser, i);
         namelist[i] = (char *)malloc(512);
         strcpy(namelist[i], str1);
        }
     fl_clear_browser(browser);
     for (i=1; i<=nline; i++)
        {fl_add_browser_line (browser, namelist[nline-i+1]);
         free(namelist[nline-i+1]);
        }
     free(namelist);
     return;
    }

 /*--- Sorting the channels list ---*/
 
 namelist = (char **)calloc(nline+1,sizeof(char *));
 for (i=1; i<=nline; i++)
    {
     str1 = fl_get_browser_line (browser, i);
     namelist[i] = (char *)malloc(512);
     strcpy(namelist[i], str1);
    }

 for (i=1; i<nline; i++)
    {
     for (j=i+1; j<=nline; j++)
        {
         if (strcmp(namelist[i], namelist[j]) > 0)
            {
             strcpy(dummy,namelist[i]);
             strcpy(namelist[i], namelist[j]);
             strcpy(namelist[j],dummy);
            }
        }
    }

 fl_clear_browser(browser);
 for (i=1; i<=nline; i++)
    {fl_add_browser_line (browser, namelist[i]);
     free(namelist[i]);
    }
 free(namelist);

 return;
}



/*************/
/*   FILES   */
/*************/


/*------------------------------------------ DyXformAddToFileList ---*/

void DyXformAddToFileList (char *fileName)
{
 int i0, i, nfile;
 const char *str;
 char dummy[LSTRING_MAX];

 if (fileName == NULL) return;

 sprintf(dummy, "DyXformAddToFileList> filename=%s\n", fileName);
 DyDebugPrint(dummy,0);

 nfile = fl_get_browser_maxline(inputfile->file_list);
 i0 = -1;
 for (i=0; i<nfile; i++)
    {str = fl_get_browser_line(inputfile->file_list, i+1);
     if (strcmp(fileName, str) == 0) {i0 = i; break;}
    } 
 if (i0 == -1) fl_add_browser_line (inputfile->file_list, fileName);
 else          printf("DyXformAddToFileList> %s already present in files list \n", fileName);

 return;
}


/*------------------------------------------- DyXformAddDir ---*/

void DyXformAddDir (FL_OBJECT *obj, long arg)
{
 const char *str;
 long test=1;

 DyDebugPrint(__FUNCTION__,0);

 if (arg != 11)
    {while(test)
        {str = fl_show_fselector("chose one file of the directory to add",Dy->Dir,"*","");
         if (!str) return;
         if (is_a_file((char *)str)) test = 0;
         else                        DyInfoLine ("Please, select one file of the directory");
        }
     Dy->Dir = DyGetDir(Dy->Dir, (char *)str);
    }

 if (arg == 1)
    {fl_set_input(mainpanel->startRun,"-1");
     fl_set_input(mainpanel->startFrame,"0");
     fl_set_input(mainpanel->startTime,"-1");
     fl_set_input (mainpanel->startUTCdate, "0");
    }

 DyAddDir (Dy->Dir, (char *)str, arg);

 return;
}

/*------------------------------------------ DyXformSortFileList ---*/

void DyXformSortFileList()
{
 char **fileName, dummy[LSTRING_MAX];
 int gts, firstTime[NFILE_MAX];
 long i, j, nfile;
 const char *str;
 struct FrFile *iFile;

 nfile = fl_get_browser_maxline(inputfile->file_list);
 if (nfile < 2) return;

 DyDebugPrint(__FUNCTION__,0);

 fileName = (char **) calloc(nfile, sizeof(char *));

 for (i=0; i<nfile; i++)
    {str = fl_get_browser_line(inputfile->file_list,i+1);
     fileName[i] = (char *) calloc(LSTRING_MAX,sizeof(char));
     strcpy(fileName[i], str);
     iFile = FrFileINew(fileName[i]);
     if (iFile != NULL)
        {firstTime[i] = FrFileITStart(iFile);
         FrFileIEnd(iFile);
        }
    }

 for (i=0; i<nfile; i++)
    {for (j=i+1; j<nfile; j++)
        {if (firstTime[i] > firstTime[j])
            {strcpy(dummy,fileName[i]);
             strcpy(fileName[i],fileName[j]);
             strcpy(fileName[j],dummy);
             gts          = firstTime[i];
             firstTime[i] = firstTime[j];
             firstTime[j] = gts;
            }
        }
    }

 fl_clear_browser(inputfile->file_list);

 for (i=0; i<nfile; i++)
    {fl_add_browser_line (inputfile->file_list, fileName[i]);}

 for (i=0; i<nfile; i++)  free(fileName[i]);
 free(fileName);

 return;
}


/*------------------------------------------ DyXformAddInDump ---*/

void DyXformAddInDump (char *str)
{
 fl_add_browser_line(dumpdata->datadumped, str);

 return;
}


