/* *********************************************************************
******* UArrayL.c ******************************** UArrayL.c ***********
******* UArrayL.c ******************************** UArrayL.c ***********
********************************************************************* */
/* By B.Caron, F.Marion, B.Mours (ESIA/LAPP Annecy) */
 
char UAversion[] = "June 08, 1998";
 
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "UArrayL.h"
#ifdef MEMWATCH
#    include "memwatch.h"
#endif

struct UJstructH *UArrayFillDUJstructH;
struct UJstructH *UArrayFillFUJstructH;


/*--------------------------------------------------------- UALibIni -*/
void UALibIni()
/*--------------------------------------------------------------------*/
{
 static BOOL first=YES;

 if (first == NO) return;
 first = NO; 

/**
 fprintf(UJobH.fOut,"   UArrayL   Version:%s\n",UAversion);
**/

 UArrayFillDUJstructH = UJDicAddS("UArrayFillD",NULL,UAfillDSim,NULL,UAfillDRead,NULL);
 UArrayFillFUJstructH = UJDicAddS("UArrayFillF",NULL,UAfillFSim,NULL,UAfillFRead,NULL);

 return;
}
 

/* ---------------------------------------------------- UACDAdd ------*/
void UACDAdd(mat1, mat2, matOut)
struct UArrayCD *mat1, *mat2, *matOut;
/*                       Add 2 COMPLEX matrices: matOut = mat1 + mat2 */
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut;
 long index;
    
 if (mat1->nData != mat2->nData)    UJError(3,"UACDAdd","nData missmatch");
 if (mat1->nData != matOut->nData)  UJError(3,"UACDAdd","nData missmatch");

 Phi1   = mat1->dataD;
 Phi2   = mat2->dataD;
 PhiOut = matOut->dataD;
 
 for (index=0;index<mat1->nData*2;index++)
    {PhiOut[index] = Phi1[index] + Phi2[index];}
		
 return;
}


/*---------------------------------------------------UACDCopy --------*/
void UACDCopy(In, Out)
struct UArrayCD *In, *Out;
/*
  Copy the Complexe matrix PhiIn into PhiOut.
     Input  ->  PhiIn : Matrix which will BE COPIED. 
     Output ->  PhiOut: Matrix INTO which PhiIn will be copied.

*/
/*--------------------------------------------------------------------*/
{
 double *PhiIn, *PhiOut;
 long index;
 
 if (In->nData != Out->nData)  UJError(3,"UACDCopy","nData missmatch");

 PhiIn  = In->dataD;
 PhiOut = Out->dataD;

 for (index=0; index<In->nData*2; index++)
    {PhiOut[index] = PhiIn[index];}

 return;
}


/*---------------------------------------------------- UACDCscal-----*/
void UACDCscal(in, c_factor, out)
struct UArrayCD *in, *out;
double c_factor[2];
/*
  This function multiplies a COMPLEX matrix by a complexe number factor
 
  Input ->  in:  Pointer to the complex input matrix
            c_factor: Complex number factor[0]: real,[1]: imaginary
 Output ->  out: After multiplication by factor (could be in) 
*/
/*--------------------------------------------------------------------*/
{
 long index;
 double xreal, ximag, *phiOut, *phiIn;
 
 if (in->nData != out->nData)  UJError(3,"UACDCscal","nData missmatch");

 phiIn  = in->dataD;
 phiOut = out->dataD;

 for (index=0; index<in->nData*2; index=index+2)
    {
     xreal           = phiIn[index];
     ximag           = phiIn[index+1];
     phiOut[index]   = xreal * c_factor[0] - ximag * c_factor[1];
     phiOut[index+1] = ximag * c_factor[0] + xreal * c_factor[1];
    }
 return;

}


/*------------------------------------------------ UACDEcartV --------*/
double UACDEcartV( Vect1, Vect2)
struct UArrayCD *Vect1, *Vect2;
/*
   Computes the 'distance' between 2 COMPLEX matrices Phi1 andd Phi2
   and puts the deviation into sigma.
 
     Input  ->  Phi1  : Pointer to first matrix.
                Phi2  : Pointer to second matrix.
     Output ->  sigma : distance between the 2 matrices
*/
/*--------------------------------------------------------------------*/
{
 int index;
 double delR,delI,aveR,aveI,sigma1,sigma2,*Phi1,*Phi2;
 
 if (Vect1->nData != Vect2->nData)  UJError(3,"UACDEcartV","nData missmatch");

 Phi1   = Vect1->dataD;
 Phi2   = Vect2->dataD;
 sigma1 = 0.;
 sigma2 = 0.;
 
 for (index=0; index<Vect1->nData*2; index=index+2)
    {
     delR   = Phi1[index]   - Phi2[index];
     delI   = Phi1[index+1] - Phi2[index+1];
     aveR   = Phi1[index]   + Phi2[index];
     aveI   = Phi1[index+1] + Phi2[index+1];
     sigma1 = sigma1 + delR*delR + delI*delI;
     sigma2 = sigma2 + aveR*aveR + aveI*aveI;
    }

 sigma1 = sqrt(sigma1*4. / sigma2);
 return(sigma1);
}


/*------------------------------------------------------UACDeqn-------*/
void UACDeqn(va,vx,vb)
struct UArrayCD *va, *vx, *vb;
/* This routines solves the system va * vx = vb with va a 
   complex square matrix (nxn) and vb a (nxk) complex array.
   This routine was translated from fortran CERNLIB routine CEQN.
*/
/*--------------------------------------------------------------------*/
{
 long n,k,i,j,m1,m2,m3,fact,p,*ir;
 double *a,*b,*x,re,im,s[2],det[2],b1[2],t1,t2,t3,temp[2],y1[2],y2[2];
 double l11[2],l21[2],l22[2],l31[2],l32[2],l33[2],u12[2],u13[2],u23[2];
 struct UArrayCD *vfa,*vfb;

 n = va->nx[0];
 if (va->nx[1] != n)  UJError(3,"UACDeqn","nData missmatch");
 if (vx->nx[0] != n)  UJError(3,"UACDeqn","nData missmatch");
 if (vb->nx[0] != n)  UJError(3,"UACDeqn","nData missmatch");
 k = vb->nx[1];
 if (vx->nx[1] != k) UJError(3,"UACDeqn","nData missmatch");

 a   = va->dataD;
 b   = vb->dataD;
 x   = vx->dataD;
 vfb = UACDNew(n,k);
 UACDCopy(vb,vfb);

 if (n == 1)
    {
     if (a[0] == 0.  &&  a[1] == 0.)
        {UJError(3,"UACDeqn","singular matrix");
         return;
        }
     s[0] =  a[0]*a[0] + a[1]*a[1];
     s[1] = -a[1] / s[0];
     s[0] =  a[0] / s[0];
     for (j=0; j<k; j++)
        {re       = b[2*j]*s[0] - b[2*j+1]*s[1];
         im       = b[2*j]*s[1] + b[2*j+1]*s[0];
         b[2*j]   = re;
         b[2*j+1] = im;
        }
     UACDCopy(vb,vx);
     UACDCopy(vfb,vb);
     UACDFree(vfb);
     return;
    }

 else if (n == 2)
    {
     det[0] = (a[0]*a[6] - a[1]*a[7])  -  (a[2]*a[4] - a[3]*a[5]);
     det[1] = (a[0]*a[7] + a[1]*a[6])  -  (a[2]*a[5] + a[3]*a[4]);
     if (det[0] == 0.  &&  det[1] == 0.)
        {UJError(3,"UACDeqn","singular matrix");
         return;
        }
     s[0] =  det[0]*det[0] + det[1]*det[1];
     s[1] = -det[1] / s[0];
     s[0] =  det[0] / s[0];
     for (j=0; j<k; j++)
        {b1[0] = b[2*j];
         b1[1] = b[2*j+1];
         re       = s[0] * ( (a[6]*b1[0]-a[7]*b1[1]) - (a[2]*b[2*(k+j)]-a[3]*b[2*(k+j)+1]) )
                  - s[1] * ( (a[6]*b1[1]+a[7]*b1[0]) - (a[2]*b[2*(k+j)+1]+a[3]*b[2*(k+j)]) );
         im       = s[0] * ( (a[6]*b1[1]+a[7]*b1[0]) - (a[2]*b[2*(k+j)+1]+a[3]*b[2*(k+j)]) )
                  + s[1] * ( (a[6]*b1[0]-a[7]*b1[1]) - (a[2]*b[2*(k+j)]-a[3]*b[2*(k+j)+1]) );
         b[2*j]   = re;
         b[2*j+1] = im;
         re       = s[0] * ( -(a[4]*b1[0]-a[5]*b1[1]) + (a[0]*b[2*(k+j)]-a[1]*b[2*(k+j)+1]) )
                  - s[1] * ( -(a[4]*b1[1]+a[5]*b1[0]) + (a[0]*b[2*(k+j)+1]+a[1]*b[2*(k+j)]) );
         im       = s[0] * ( -(a[4]*b1[1]+a[5]*b1[0]) + (a[0]*b[2*(k+j)+1]+a[1]*b[2*(k+j)]) )
                  + s[1] * ( -(a[4]*b1[0]-a[5]*b1[1]) + (a[0]*b[2*(k+j)]-a[1]*b[2*(k+j)+1]) );
         b[2*(k+j)]   = re;
         b[2*(k+j)+1] = im;
        }
     UACDCopy(vb,vx);
     UACDCopy(vfb,vb);
     UACDFree(vfb);
     return;
    }

 else if (n == 3)
    {
     fact = 0;
     /*--- FACTORIZE MATRIX A=L*U ---*/
     /*--- (FIRST PIVOT SEARCH)   ---*/
     t1 = fabs(a[0])+fabs(a[1]);
     t2 = fabs(a[6])+fabs(a[7]);
     t3 = fabs(a[12])+fabs(a[13]);
     if (t1 >= t2  &&  t1 >= t3)     /* (PIVOT IS A11) */
        {m1 = 0;
         m2 = 1;
         m3 = 2;
        }
     else if(t2 >= t1 && t2 >= t3)    /* (PIVOT IS A21) */
        {m1 = 1;
         m2 = 0;
         m3 = 2;
        }
     else                             /* (PIVOT IS A31) */
        {m1 = 2;
         m2 = 1;
         m3 = 0;
        }
     temp[0] = a[2*m1*n];
     temp[1] = a[2*m1*n+1];
     if (temp[0] == 0.  &&  temp[1] == 0.)
        {fact = 1;}
     else
        {l11[0] = temp[0]*temp[0] + temp[1]*temp[1];
         l11[1] = -temp[1]/l11[0];
         l11[0] = temp[0]/l11[0];
         u12[0] = l11[0]*a[2*(m1*n+1)] - l11[1]*a[2*(m1*n+1)+1];
         u12[1] = l11[0]*a[2*(m1*n+1)+1] + l11[1]*a[2*(m1*n+1)];
         u13[0] = l11[0]*a[2*(m1*n+2)] - l11[1]*a[2*(m1*n+2)+1];
         u13[1] = l11[0]*a[2*(m1*n+2)+1] + l11[1]*a[2*(m1*n+2)];
         l22[0] = a[2*(m2*n+1)]   - (a[2*(m2*n)]*u12[0]-a[2*(m2*n)+1]*u12[1]);
         l22[1] = a[2*(m2*n+1)+1] - (a[2*(m2*n)]*u12[1]+a[2*(m2*n)+1]*u12[0]);
         l32[0] = a[2*(m3*n+1)]   - (a[2*(m3*n)]*u12[0]-a[2*(m3*n)+1]*u12[1]);
         l32[1] = a[2*(m3*n+1)+1] - (a[2*(m3*n)]*u12[1]+a[2*(m3*n)+1]*u12[0]);

         /*--- (SECOND PIVOT SEARCH) ---*/
         t2 = fabs(l22[0]) + fabs(l22[1]);
         t3 = fabs(l32[0]) + fabs(l32[1]);
         if (t2 < t3)
            {i  = m2;
             m2 = m3;
             m3 = i;
             for (p=0; p<2; p++) 
                {temp[p] = l22[p];
                 l22[p] = l32[p];
                 l32[p] = temp[p];
                }
            }
         for (p=0; p<2; p++) 
            {l21[p] = a[2*(m2*n)+p];
 	     l31[p] = a[2*(m3*n)+p];
            }
         if (l22[0] == 0.  &&  l22[1] == 0.)
            {fact = 1;}
         else
            {re = l22[0]*l22[0]+l22[1]*l22[1];
             l22[0] = l22[0]/re;
             l22[1] = -l22[1]/re;
             u23[0] = l22[0]*(a[2*(m2*n+2)]-(l21[0]*u13[0]-l21[1]*u13[1]))
                    - l22[1]*(a[2*(m2*n+2)+1]-(l21[0]*u13[1]+l21[1]*u13[0]));
             u23[1] = l22[0]*(a[2*(m2*n+2)+1]-(l21[0]*u13[1]+l21[1]*u13[0]))
                    + l22[1]*(a[2*(m2*n+2)]-(l21[0]*u13[0]-l21[1]*u13[1]));
             temp[0] = a[2*(m3*n+2)]
                     - (l31[0]*u13[0]-l31[1]*u13[1])
                     - (l32[0]*u23[0]-l32[1]*u23[1]);
             temp[1] = a[2*(m3*n+2)+1]
                     - (l31[0]*u13[1]+l31[1]*u13[0])
                     - (l32[0]*u23[1]+l32[1]*u23[0]);
             if (temp[0] == 0.  &&  temp[1] == 0.)
                {fact = 1;}
             else
                {l33[0] = temp[0]*temp[0] + temp[1]*temp[1];
                 l33[1] = -temp[1]/l33[0];
                 l33[0] = temp[0]/l33[0];
                 /*--- SOLVE L*Y=B AND U*X=Y ---*/
                 for (j=0; j<k; j++)
	            {y1[0] = l11[0] * b[2*(m1*k+j)]   - l11[1]*b[2*(m1*k+j)+1];
                     y1[1] = l11[0] * b[2*(m1*k+j)+1] + l11[1]*b[2*(m1*k+j)];
                     y2[0] = l22[0] * (b[2*(m2*k+j)] - (l21[0]*y1[0]-l21[1]*y1[1]))
                           - l22[1] * (b[2*(m2*k+j)+1] - (l21[0]*y1[1]+l21[1]*y1[0]));
                     y2[1] = l22[0] * (b[2*(m2*k+j)+1] - (l21[0]*y1[1]+l21[1]*y1[0]))
                           + l22[1] * (b[2*(m2*k+j)]   - (l21[0]*y1[0]-l21[1]*y1[1]));
                     re    = l33[0] * (b[2*(m3*k+j)]
                                       - (l31[0]*y1[0]-l31[1]*y1[1])
                                       - (l32[0]*y2[0]-l32[1]*y2[1]))
                           - l33[1] * (b[2*(m3*k+j)+1]
                                       -(l31[0]*y1[1]+l31[1]*y1[0])
                                       -(l32[0]*y2[1]+l32[1]*y2[0]));
                     im    = l33[0] * (b[2*(m3*k+j)+1]
                                       -(l31[0]*y1[1]+l31[1]*y1[0])
                                       -(l32[0]*y2[1]+l32[1]*y2[0]))
                           + l33[1] * (b[2*(m3*k+j)]
                                       -(l31[0]*y1[0]-l31[1]*y1[1])
                                       -(l32[0]*y2[0]-l32[1]*y2[1]));
                     b[2*(2*k+j)]   = re;
                     b[2*(2*k+j)+1] = im;
                     b[2*(k+j)]     = y2[0] 
                                    - (u23[0]*b[2*(2*k+j)]-u23[1]*b[2*(2*k+j)+1]);
                     b[2*(k+j)+1]   = y2[1] 
                                    - (u23[0]*b[2*(2*k+j)+1]+u23[1]*b[2*(2*k+j)]);
                     b[2*j]         = y1[0] 
                                    - (u12[0]*b[2*(k+j)]-u12[1]*b[2*(k+j)+1])
                                    - (u13[0]*b[2*(2*k+j)]-u13[1]*b[2*(2*k+j)+1]);
                     b[2*j+1]       = y1[1] 
                                    - (u12[0]*b[2*(k+j)+1]+u12[1]*b[2*(k+j)])
		                    - (u13[0]*b[2*(2*k+j)+1]+u13[1]*b[2*(2*k+j)]);
                    }
                }
            }
        }

     if (fact == 0)
        {UACDCopy(vb,vx);
         UACDCopy(vfb,vb);
         UACDFree(vfb);
         return;
        }
    }

 /*--- N.GT.3 CASES.  FACTORIZE MATRIX AND SOLVE SYSTEM. ---*/

 vfa = UACDNew(n,n);
 UACDCopy(va,vfa);
 UACDCopy(vfb,vb);
 ir  = (long*)malloc(n*sizeof(long));

 UACDfact(vfa,ir);
 UACDfeqn(vfa,vb,ir);

 UACDCopy(vb,vx);
 UACDCopy(vfb,vb);

 free(ir);
 UACDFree(vfa);
 UACDFree(vfb);

 return;
}


/*------------------------------------------------------UACDfact-------*/
void UACDfact(va,ir)
struct UArrayCD *va;
long *ir;
/* This routine factorizes the complex square matrix va.
   It must be called before UACDfeqn or UACDfinv is called.
   This routine was translated from fortran CERNLIB routine CFACT.
*/
/*--------------------------------------------------------------------*/
{
 long n,nxch,i,j,k,jp1,jm1,l;
 double *a,det[2],p,q,tf[2],re,im,s11[2],s12[2];

 n = va->nx[0];
 if (va->nx[1] != n)  UJError(3,"UACDfact","nData missmatch");
 a = va->dataD;

 nxch   = 0;
 det[0] = 1.;
 det[1] = 0.;

 for (j=0; j<n; j++)
    {k = j;
     p = SIESTA_MAX(fabs(a[2*(j*n+j)]), fabs(a[2*(j*n+j)+1]));
     if (j == n-1  &&  p <= 0.)
        {UJError(3,"UACDfact","singular matrix");
         return;
        }
     if (j != n-1)
        {jp1 = j+1;
         for (i=jp1; i<n; i++)
	    {q = SIESTA_MAX(fabs(a[2*(i*n+j)]), fabs(a[2*(i*n+j)+1]));
             if (q <= p) continue;
             k = i;
             p = q;
            }
        if (k == j && p <= 0.)
	   {UJError(3,"UACDfact","singular matrix");
	    return;
           }
        if (k != j)
	   {for (l=0; l<n; l++)
	       {tf[0]          = a[2*(j*n+l)];
                tf[1]          = a[2*(j*n+l)+1];
                a[2*(j*n+l)]   = a[2*(k*n+l)];
                a[2*(j*n+l)+1] = a[2*(k*n+l)+1];
                a[2*(k*n+l)]   = tf[0];
                a[2*(k*n+l)+1] = tf[1];
               }
            nxch ++;
            ir[nxch-1] = (j+1)*4096 + k+1;
           }
       }
    re = a[2*(j*n+j)]*a[2*(j*n+j)] + a[2*(j*n+j)+1]*a[2*(j*n+j)+1];
    a[2*(j*n+j)]   =  a[2*(j*n+j)]   / re;
    a[2*(j*n+j)+1] = -a[2*(j*n+j)+1] / re;
    if (j != n-1)
       {jm1 = j-1;
        jp1 = j+1;
        for (k=jp1; k<n; k++)
	   {s11[0] = -a[2*(j*n+k)];
            s11[1] = -a[2*(j*n+k)+1];
            s12[0] = -a[2*(k*n+j+1)];
            s12[1] = -a[2*(k*n+j+1)+1];
            if (j != 0)
	       {for (i=0; i<=jm1; i++)
		   {s11[0] += a[2*(i*n+k)]*a[2*(j*n+i)]     - a[2*(i*n+k)+1]*a[2*(j*n+i)+1];
                    s11[1] += a[2*(i*n+k)]*a[2*(j*n+i)+1]   + a[2*(i*n+k)+1]*a[2*(j*n+i)];
                    s12[0] += a[2*(i*n+j+1)]*a[2*(k*n+i)]   - a[2*(i*n+j+1)+1]*a[2*(k*n+i)+1];
                    s12[1] += a[2*(i*n+j+1)]*a[2*(k*n+i)+1] + a[2*(i*n+j+1)+1]*a[2*(k*n+i)];
                   }
               }
            re = - (s11[0]*a[2*(j*n+j)]   - s11[1]*a[2*(j*n+j)+1]);
            im = - (s11[0]*a[2*(j*n+j)+1] + s11[1]*a[2*(j*n+j)]);
            a[2*(j*n+k)]   = re;
            a[2*(j*n+k)+1] = im;
            re = - (s12[0] + a[2*(j*n+j+1)]*a[2*(k*n+j)]   - a[2*(j*n+j+1)+1]*a[2*(k*n+j)+1]);
            im = - (s12[1] + a[2*(j*n+j+1)]*a[2*(k*n+j)+1] + a[2*(j*n+j+1)+1]*a[2*(k*n+j)]);
            a[2*(k*n+j+1)]   = re;
            a[2*(k*n+j+1)+1] = im;
           }
        }
    }
 ir[n-1] = nxch;

 return;
}


/*------------------------------------------------------UACDfeqn-------*/
void UACDfeqn(va,vb,ir)
struct UArrayCD *va, *vb;
long *ir;
/* This routines solves the system va * vx = vb with va a factorized
   complex square matrix (nxn) and vb a (nxk) complex array.
   The result is put in vb.
   This routine may be called only after UACDfact has been called, 
   with va and ir unchanged.
   This routine was translated from fortran CERNLIB routine CFEQN.
*/
/*--------------------------------------------------------------------*/
{
 long n,k,nxch,m,ij,l,i,j,im1,nm1,nmi,nmjp1;
 double te[2],s21[2],s22[2],re,im,*a,*b;

 n = va->nx[0];
 if (va->nx[1] != n)  UJError(3,"UACDfeqn","nData missmatch");
 if (vb->nx[0] != n)  UJError(3,"UACDfeqn","nData missmatch");
 k = vb->nx[1];

 a = va->dataD;
 b = vb->dataD;

 nxch = ir[n-1];

 if (nxch != 0)
    {for (m=0; m<nxch; m++)
        {ij = ir[m];
         i = ij/4096 - 1;
         j = ij - (i+1)*4096 - 1;
         for (l=0; l<k; l++)
	    {te[0]          = b[2*(i*k+l)];
             te[1]          = b[2*(i*k+l)+1];
             b[2*(i*k+l)]   = b[2*(j*k+l)];
             b[2*(i*k+l)+1] = b[2*(j*k+l)+1];
             b[2*(j*k+l)]   = te[0];
             b[2*(j*k+l)+1] = te[1];
            }
         }
     }

 for (l=0; l<k; l++)
    {re       = a[0]*b[2*l]   - a[1]*b[2*l+1];
     im       = a[0]*b[2*l+1] + a[1]*b[2*l];
     b[2*l]   = re;
     b[2*l+1] = im;
    }
 if (n == 1) return;

 for (l=0; l<k; l++)
    {for (i=1; i<n; i++)
        {im1    = i-1;
         s21[0] = - b[2*(i*k+l)];
         s21[1] = - b[2*(i*k+l)+1];
         for (j=0; j<=im1; j++)
	    {s21[0] += a[2*(i*n+j)]*b[2*(j*k+l)]   - a[2*(i*n+j)+1]*b[2*(j*k+l)+1];
             s21[1] += a[2*(i*n+j)]*b[2*(j*k+l)+1] + a[2*(i*n+j)+1]*b[2*(j*k+l)];
            }
         b[2*(i*k+l)]   = - (a[2*(i*n+i)]*s21[0] - a[2*(i*n+i)+1]*s21[1]);
         b[2*(i*k+l)+1] = - (a[2*(i*n+i)]*s21[1] + a[2*(i*n+i)+1]*s21[0]);
        }
     nm1 = n-1;
     for (i=0; i<nm1; i++)
        {nmi    = n-i-2;
         s22[0] = - b[2*(nmi*k+l)];
         s22[1] = - b[2*(nmi*k+l)+1];
         for (j=0; j<=i; j++)
	    {nmjp1   = n-j-1;
             s22[0] += a[2*(nmi*n+nmjp1)]*b[2*(nmjp1*k+l)]
                     - a[2*(nmi*n+nmjp1)+1]*b[2*(nmjp1*k+l)+1];
             s22[1] += a[2*(nmi*n+nmjp1)]*b[2*(nmjp1*k+l)+1]
		     + a[2*(nmi*n+nmjp1)+1]*b[2*(nmjp1*k+l)];
            }
         b[2*(nmi*k+l)]   = -s22[0];
         b[2*(nmi*k+l)+1] = -s22[1];
        }
    }

 return;
}


/*------------------------------------------------------UACDinv------*/
void UACDinv(vIn,vOut)
struct UArrayCD *vIn, *vOut;
/* This routines computes the inverse of a 
   complex square matrix vIn and puts it into vOut.
   This routine was translated from fortran CERNLIB routine CINV.
*/
/*--------------------------------------------------------------------*/
{
 long n,*ir;
 double *a,s[2],det[2],re,im,t1,t2,t3,temp[2];
 double c11[2],c12[2],c13[2],c21[2],c22[2],c23[2],c31[2],c32[2],c33[2];
 struct UArrayCD *vInc,*vOutc;

 n = vIn->nx[0];
 if (vIn->nx[1] != n)   UJError(3,"UACDinv","nData missmatch");
 if (vOut->nx[0] != n)  UJError(3,"UACDinv","nData missmatch");
 if (vOut->nx[1] != n)  UJError(3,"UACDinv","nData missmatch");

 a = vIn->dataD;

 vInc  = UACDNew(n,n);
 vOutc = UACDNew(n,n);
 UACDCopy(vIn,vInc);

 if (n == 1)
    {if (a[0] == 0.  &&  a[1] == 0.)
        {UJError(3,"UACDinv","singular matrix");
         return;
        }
     s[0] =  a[0]*a[0] + a[1]*a[1];
     a[0] =  a[0]/s[0];
     a[1] = -a[1]/s[0];
    }

 else if(n == 2)
    {det[0] = (a[0]*a[6]-a[1]*a[7]) - (a[2]*a[4]-a[3]*a[5]);
     det[1] = (a[0]*a[7]+a[1]*a[6]) - (a[2]*a[5]+a[3]*a[4]);
     if (det[0] == 0.  &&  det[1] == 0.)
        {UJError(3,"UACDinv","singular matrix");
         return;
        }
     s[0]   =  det[0]*det[0] + det[1]*det[1];
     s[1]   = -det[1]/s[0];
     s[0]   =  det[0]/s[0];
     c11[0] =   s[0]*a[6] - s[1]*a[7];
     c11[1] =   s[0]*a[7] + s[1]*a[6];
     re     = -(s[0]*a[2] - s[1]*a[3]);
     im     = -(s[0]*a[3] + s[1]*a[2]);
     a[2]   = re;
     a[3]   = im;
     re     = -(s[0]*a[4] - s[1]*a[5]);
     im     = -(s[0]*a[5] + s[1]*a[4]);
     a[4]   = re;
     a[5]   = im;
     a[6]   =   s[0]*a[0] - s[1]*a[1];
     a[7]   =   s[0]*a[1] + s[1]*a[0];
     a[0]   = c11[0];
     a[1]   = c11[1];
    }

 else if (n == 3)
    {/*--- COMPUTE COFACTORS ---*/

     c11[0] = (a[8]*a[16]  - a[9]*a[17])   -  (a[10]*a[14] - a[11]*a[15]);
     c11[1] = (a[8]*a[17]  + a[9]*a[16])   -  (a[10]*a[15] + a[11]*a[14]);
     c12[0] = (a[10]*a[12] - a[11]*a[13])  -  (a[6]*a[16]  - a[7]*a[17]);
     c12[1] = (a[10]*a[13] + a[11]*a[12])  -  (a[6]*a[17]  + a[7]*a[16]);
     c13[0] = (a[6]*a[14]  - a[7]*a[15])   -  (a[8]*a[12]  - a[9]*a[13]);
     c13[1] = (a[6]*a[15]  + a[7]*a[14])   -  (a[8]*a[13]  + a[9]*a[12]);
     c21[0] = (a[14]*a[4]  - a[15]*a[5])   -  (a[16]*a[2]  - a[17]*a[3]);
     c21[1] = (a[14]*a[5]  + a[15]*a[4])   -  (a[16]*a[3]  + a[17]*a[2]);
     c22[0] = (a[16]*a[0]  - a[17]*a[1])   -  (a[12]*a[4]  - a[13]*a[5]);
     c22[1] = (a[16]*a[1]  + a[17]*a[0])   -  (a[12]*a[5]  + a[13]*a[4]);
     c23[0] = (a[12]*a[2]  - a[13]*a[3])   -  (a[14]*a[0]  - a[15]*a[1]);
     c23[1] = (a[12]*a[3]  + a[13]*a[2])   -  (a[14]*a[1]  + a[15]*a[0]);
     c31[0] = (a[2]*a[10]  - a[3]*a[11])   -  (a[4]*a[8]   - a[5]*a[9]);
     c31[1] = (a[2]*a[11]  + a[3]*a[10])   -  (a[4]*a[9]   + a[5]*a[8]);
     c32[0] = (a[4]*a[6]   - a[5]*a[7])    -  (a[0]*a[10]  - a[1]*a[11]);
     c32[1] = (a[4]*a[7]   + a[5]*a[6])    -  (a[0]*a[11]  + a[1]*a[10]);
     c33[0] = (a[0]*a[8]   - a[1]*a[9])    -  (a[2]*a[6]   - a[3]*a[7]);
     c33[1] = (a[0]*a[9]   + a[1]*a[8])    -  (a[2]*a[7]   + a[3]*a[6]);
     t1 = fabs(a[0])  + fabs(a[1]);
     t2 = fabs(a[6])  + fabs(a[7]);
     t3 = fabs(a[12]) + fabs(a[13]);

     /*--- (SET TEMP=PIVOT AND DET=PIVOT*DET) ---*/
     if (t1 >= t2  &&  t1 >= t3)                     /* (PIVOT IS A11) */
        {temp[0] = a[0];
         temp[1] = a[1];
         det[0]  = (c22[0]*c33[0] - c22[1]*c33[1]) - (c23[0]*c32[0] - c23[1]*c32[1]);
         det[1]  = (c22[0]*c33[1] + c22[1]*c33[0]) - (c23[0]*c32[1] + c23[1]*c32[0]);
        }
     else if(t2 >= t1  &&  t2 >= t3)                 /* (PIVOT IS A21) */
        {temp[0] = a[6];
         temp[1] = a[7];
         det[0]  = (c13[0]*c32[0] - c13[1]*c32[1]) - (c12[0]*c33[0] - c12[1]*c33[1]);
         det[1]  = (c13[0]*c32[1] + c13[1]*c32[0]) - (c12[0]*c33[1] + c12[1]*c33[0]);
        }
     else                                            /* (PIVOT IS A31) */
        {temp[0] = a[12];
         temp[1] = a[13];
         det[0]  = (c23[0]*c12[0] - c23[1]*c12[1]) - (c22[0]*c13[0] - c22[1]*c13[1]);
         det[1]  = (c23[0]*c12[1] + c23[1]*c12[0]) - (c22[0]*c13[1] + c22[1]*c13[0]);
        }

     /*--- SET ELEMENTS OF INVERSE IN A ---*/
     if (det[0] == 0.  &&  det[1] == 0.)
        {UJError(3,"UACDinv","singular matrix");
         return;
        }
     s[0]  =  det[0]*det[0] + det[1]*det[1];
     s[1]  = -det[1]/s[0];
     s[0]  =  det[0]/s[0];
     re    = s[0]*temp[0] - s[1]*temp[1];
     im    = s[0]*temp[1] + s[1]*temp[0];
     s[0]  = re;
     s[1]  = im;
     a[0]  = s[0]*c11[0] - s[1]*c11[1];
     a[1]  = s[0]*c11[1] + s[1]*c11[0];
     a[2]  = s[0]*c21[0] - s[1]*c21[1];
     a[3]  = s[0]*c21[1] + s[1]*c21[0];
     a[4]  = s[0]*c31[0] - s[1]*c31[1];
     a[5]  = s[0]*c31[1] + s[1]*c31[0];
     a[6]  = s[0]*c12[0] - s[1]*c12[1];
     a[7]  = s[0]*c12[1] + s[1]*c12[0];
     a[8]  = s[0]*c22[0] - s[1]*c22[1];
     a[9]  = s[0]*c22[1] + s[1]*c22[0];
     a[10] = s[0]*c32[0] - s[1]*c32[1];
     a[11] = s[0]*c32[1] + s[1]*c32[0];
     a[12] = s[0]*c13[0] - s[1]*c13[1];
     a[13] = s[0]*c13[1] + s[1]*c13[0];
     a[14] = s[0]*c23[0] - s[1]*c23[1];
     a[15] = s[0]*c23[1] + s[1]*c23[0];
     a[16] = s[0]*c33[0] - s[1]*c33[1];
     a[17] = s[0]*c33[1] + s[1]*c33[0];}

 else
    {/*--- N.GT.3 CASES.  FACTORIZE MATRIX AND SOLVE SYSTEM ---*/
     ir = (long*)malloc(n*sizeof(long));
     UACDfact(vIn,ir);
     UACDfinv(vIn,ir);
     free(ir);
    }

 UACDCopy(vIn,vOutc);
 UACDCopy(vInc,vIn);
 UACDCopy(vOutc,vOut);
 UACDFree(vInc);
 UACDFree(vOutc);

 return;
}


/*------------------------------------------------------UACDfinv-------*/
void UACDfinv(vIn,ir)
struct UArrayCD *vIn;
long *ir;
/* This routines computes the inverse of a factorized 
   complex square matrix vIn. The result is put in vIn.
   This routine may be called only after UACDfact has been called, 
   with vIn and ir unchanged.
   This routine was translated from fortran CERNLIB routine CFINV.
*/
/*--------------------------------------------------------------------*/
{
 long n,i,j,k,im2,nm1,nmi,m,nxch,ij;
 double ti[2],s31[2],s32[2],s33[2],s34[2],re,im,*a;

 n = vIn->nx[0];
 if (vIn->nx[1] != n)  UJError(3,"UACDfinv","nData missmatch");

 a = vIn->dataD;

 if (n == 1) return;

 re       = - (a[2*n+2]*(a[0]*a[2*n]-a[1]*a[2*n+1]) - a[2*n+3]*(a[0]*a[2*n+1]+a[1]*a[2*n]));
 im       = - (a[2*n+2]*(a[0]*a[2*n+1]+a[1]*a[2*n]) + a[2*n+3]*(a[0]*a[2*n]-a[1]*a[2*n+1]));
 a[2*n]   = re;
 a[2*n+1] = im;
 a[2]     = -a[2];
 a[3]     = -a[3];

 if (n > 2)
    {for (i=2; i<n; i++)
        {im2 = i-1;
         for (j=0; j<im2; j++)
            {s31[0] = 0.;
             s31[1] = 0.;
             s32[0] = a[2*(j*n+i)];
             s32[1] = a[2*(j*n+i)+1];
             for (k=j; k<im2; k++)
	        {s31[0] += a[2*(k*n+j)]*a[2*(i*n+k)]         - a[2*(k*n+j)+1]*a[2*(i*n+k)+1];
                 s31[1] += a[2*(k*n+j)]*a[2*(i*n+k)+1]       + a[2*(k*n+j)+1]*a[2*(i*n+k)];
                 s32[0] += a[2*(j*n+k+1)]*a[2*((k+1)*n+i)]   - a[2*(j*n+k+1)+1]*a[2*((k+1)*n+i)+1];
                 s32[1] += a[2*(j*n+k+1)]*a[2*((k+1)*n+i)+1] + a[2*(j*n+k+1)+1]*a[2*((k+1)*n+i)];
                }
             re = - (a[2*(i*n+i)] * (a[2*((i-1)*n+j)]*a[2*(i*n+i-1)]
                                   - a[2*((i-1)*n+j)+1]*a[2*(i*n+i-1)+1]+s31[0])
                - a[2*(i*n+i)+1] * (a[2*((i-1)*n+j)]*a[2*(i*n+i-1)+1]
                                  + a[2*((i-1)*n+j)+1]*a[2*(i*n+i-1)]+s31[1]));
             im = -(a[2*(i*n+i)] * (a[2*((i-1)*n+j)]*a[2*(i*n+i-1)+1]
                                  + a[2*((i-1)*n+j)+1]*a[2*(i*n+i-1)]+s31[1])
                + a[2*(i*n+i)+1] * (a[2*((i-1)*n+j)]*a[2*(i*n+i-1)]
                                  - a[2*((i-1)*n+j)+1]*a[2*(i*n+i-1)+1]+s31[0]));
             a[2*(i*n+j)]   = re;
             a[2*(i*n+j)+1] = im;
             a[2*(j*n+i)]   = -s32[0];
             a[2*(j*n+i)+1] = -s32[1];}
             re             = - (a[2*(i*n+i)] * (a[2*((i-1)*n+i-1)]*a[2*(i*n+i-1)]
                                               - a[2*((i-1)*n+i-1)+1]*a[2*(i*n+i-1)+1])
                              - a[2*(i*n+i)+1] * (a[2*((i-1)*n+i-1)]*a[2*(i*n+i-1)+1]
                                                + a[2*((i-1)*n+i-1)+1]*a[2*(i*n+i-1)]));
             im             = - (a[2*(i*n+i)] * (a[2*((i-1)*n+i-1)]*a[2*(i*n+i-1)+1]
                                               + a[2*((i-1)*n+i-1)+1]*a[2*(i*n+i-1)])
                              + a[2*(i*n+i)+1] * (a[2*((i-1)*n+i-1)]*a[2*(i*n+i-1)]
                                                - a[2*((i-1)*n+i-1)+1]*a[2*(i*n+i-1)+1]));
             a[2*(i*n+i-1)]     = re;
             a[2*(i*n+i-1)+1]   = im;
             a[2*((i-1)*n+i)]   = - a[2*((i-1)*n+i)];
             a[2*((i-1)*n+i)+1] = - a[2*((i-1)*n+i)+1];
            }
        }
 nm1 = n-1;
 for (i=0; i<nm1; i++)
    {nmi = n-i-1;
     for (j=0; j<=i; j++)
        {s33[0] = a[2*(i*n+j)];
         s33[1] = a[2*(i*n+j)+1];
         for (k=0; k<nmi; k++)
	    {s33[0] += a[2*((i+k+1)*n+j)]*a[2*(i*n+i+k+1)]
                     - a[2*((i+k+1)*n+j)+1]*a[2*(i*n+i+k+1)+1];
             s33[1] += a[2*((i+k+1)*n+j)]*a[2*(i*n+i+k+1)+1]
	             + a[2*((i+k+1)*n+j)+1]*a[2*(i*n+i+k+1)];
            }
         a[2*(i*n+j)] = s33[0];
         a[2*(i*n+j)+1] = s33[1];
        }
    for (j=0; j<nmi; j++)
       {s34[0] = 0.;
        s34[1] = 0.;
        for (k=j; k<nmi; k++)
           {s34[0] += a[2*((i+k+1)*n+i+j+1)]*a[2*(i*n+i+k+1)]
                    - a[2*((i+k+1)*n+i+j+1)+1]*a[2*(i*n+i+k+1)+1];
            s34[1] += a[2*((i+k+1)*n+i+j+1)]*a[2*(i*n+i+k+1)+1]
	            + a[2*((i+k+1)*n+i+j+1)+1]*a[2*(i*n+i+k+1)];
           }
        a[2*(i*n+i+j+1)]   = s34[0];
        a[2*(i*n+i+j+1)+1] = s34[1];
       }
    }

 nxch = ir[n-1];
 if (nxch == 0) return;

 for (m=0; m<nxch; m++)
    {k = nxch - m - 1;
     ij = ir[k];
     i  = ij/4096 - 1;
     j  = ij - (i+1)*4096 - 1;
     for (k=0; k<n; k++)
        {ti[0]          = a[2*(k*n+i)];
         ti[1]          = a[2*(k*n+i)+1];
         a[2*(k*n+i)]   = a[2*(k*n+j)];
         a[2*(k*n+i)+1] = a[2*(k*n+j)+1];
         a[2*(k*n+j)]   = ti[0];
         a[2*(k*n+j)+1] = ti[1];
        }
    }

 return;
}


/*------------------------------------------------ UACDft2d -----*/
void  UACDft2d(Vect, msign)
struct UArrayCD *Vect;
long msign;
/*                   Computes the TF of a 2 dimensions Complex Matrix */
/*--------------------------------------------------------------------*/
{
 double *c_field, xnorm;
 long nBinP2,nBin,index,i,j, indexVect;
 static struct UArrayCD *vectTemp;
 static long nBinLast=0;
 
 c_field = Vect->dataD;

 if (msign == 0)  return;
 nBinP2 = msign;
 if (msign < 0)  nBinP2 = -msign;
 
 nBin=1;
 for (i=0; i<nBinP2; i++) nBin = nBin * 2;
 
 if (nBin >  nBinLast)
    {if(vectTemp != NULL) UACDFree(vectTemp);
     vectTemp = UACDNew(1,nBin);
     nBinLast = nBin;
    }
 
 for (j=0; j<nBin; j++)
    {
     for (i=0; i<nBin; i++)
        {index     = (nBin*i+j) * 2;
         indexVect = 2*i;
         vectTemp->dataD[indexVect]   = c_field[index];
         vectTemp->dataD[indexVect+1] = c_field[index+1];
        }
 
     UACDfft(vectTemp,msign);
 
     for (i=0; i<nBin; i++)
        {index     = (nBin*i+j) * 2;
         indexVect = 2*i;
         c_field[index]   = vectTemp->dataD[indexVect];
         c_field[index+1] = vectTemp->dataD[indexVect+1];
        }
    }
 
 for (i=0; i<nBin; i++)
    {index = nBin*i*2;
     for (j=0; j<2*nBin; j++)
        {vectTemp->dataD[j] = c_field[index + j];}
     UACDfft(vectTemp,msign);
     for (j=0; j<2*nBin; j++)
        {c_field[index + j] = vectTemp->dataD[j];}
    }
	  
/*---  Normalisation  for inverse Fourier transform only ---*/
 if (msign < 0)
    {xnorm = 1. / ((double)nBin*(double)nBin);
     for (index=0; index<2*nBin*nBin; index++)
        {c_field[index] = c_field[index] * xnorm;}
    }

 return;
}


/*----------------------------------------------------- UACDfft --------*/
void UACDfft(Vect, msign)
struct UArrayCD *Vect;
long msign;
/*--------------------------------------------------------------------*/
/* cdfft computes the finite Fourier transform of a complex periodic  */
/* sequence, whose period n must be a power of 2 (n=2**|msign|).      */
/* To ensure optimum use of storage, the same array is used for input */
/* and output. If msign is positive the direct Fourier transform is   */
/* computed. If msign is negative, the inverse Fourier transform is   */
/* computed. This code is a c translation of the routine D704 (CFFT)  */
/* from CERNLIB. See the CERN writeup for more information.           */
/* Author: B. Mours        27/02/91                                   */
/*--------------------------------------------------------------------*/
 
{
 double c,s,ur,ui,ur0,wr,wi,tr,ti,*a,temp;
 long  m,n,nv2,nm1,i,j,k,le,l,le1,ip;
 long indexI,indexJ,indexIp;
 
 a   = Vect->dataD;

 if (msign == 0) return;
 m   = msign;
 if (msign < 0) m = -m;
 n   = 1;
 for (i=0; i<m; i++) {n=2*n;}
 nv2 = n/2;
 nm1 = n-1;
 j   = 0;
 
 for (i=0; i<nm1; i++)
    {indexI = 2*i;
     if (i < j)
        {indexJ      = 2*j;
         tr          = a[indexJ  ];
         ti          = a[indexJ+1];
         a[indexJ  ] = a[indexI  ];
         a[indexJ+1] = a[indexI+1];
         a[indexI  ] = tr;
         a[indexI+1] = ti;
        }
     k = nv2-1;
     while ( k < j )
        {j = j - k - 1;
         k = (k-1) / 2;
        }
    j = j + k + 1;
   }
 
 for (i=0; i<n; i=i+2)
    {indexI       = 2*i;
     tr           = a[2*(i+1)];
     ti           = a[2*(i+1)+1];
     a[2*(i+1)  ] = a[indexI  ] - tr;
     a[2*(i+1)+1] = a[indexI+1] - ti;
     a[indexI  ]  = a[indexI  ] + tr;
     a[indexI+1]  = a[indexI+1] + ti;
    }
 
 if (m == 1) return;
 c = 0;
 s = 1.;
 if (msign < 0) s = -1.;
 le = 2;
 
 for (l=2; l<=m; l++)
    {wr  = c;
     wi  = s;
     ur  = wr;
     ui  = wi;
     c   = sqrt(c*.5+.5);
     s   = wi/(c+c);
     le1 = le;
     le  = le1+le1;
 
     for (i=0; i<n; i=i+le)
        {ip      = i + le1;
         indexI  = 2*i;
         indexIp = 2*ip;
         tr      = a[indexIp];
         ti      = a[indexIp+1];
         a[indexIp  ] = a[indexI  ] - tr;
         a[indexIp+1] = a[indexI+1] - ti;
         a[indexI  ]  = a[indexI  ] + tr;
         a[indexI+1]  = a[indexI+1] + ti;
        }

     for (j=1; j<le1; j++)
        {for (i=j; i<n; i=i+le)
            {ip      = i + le1;
             indexI  = 2*i;
             indexIp = 2*ip;
             tr      = a[indexIp  ]*ur - a[indexIp+1]*ui;
             ti      = a[indexIp+1]*ur + a[indexIp  ]*ui;
             a[indexIp  ] = a[indexI  ] - tr;
             a[indexIp+1] = a[indexI+1] - ti;
             a[indexI  ]  = a[indexI  ] + tr;
             a[indexI+1]  = a[indexI+1] + ti;
            }
 
         ur0 = ur;
         ur  = ur*wr - ui*wi;
         ui  = ui*wr + ur0*wi;
        }
    }
 
 for (i=1; i<n/2; i++)
    {temp         = a[2*i];
     a[2*i]       = a[2*(n-i)];
     a[2*(n-i)]   = temp;
     temp         = a[2*i+1];
     a[2*i+1]     = a[2*(n-i)+1];
     a[2*(n-i)+1] = temp;
    }

 return;
}


/* --------------------------------------------------- UACDinvertV ---*/
void UACDinvertV(VectIn, VectOut)
struct UArrayCD *VectIn, *VectOut;
/*
     VectOut = invert of the complex number VectIn
*/
/*--------------------------------------------------------------------*/
{
 double *PhiIn, *PhiOut, mod;
 long nBin;
    
 if (VectIn->nData != VectOut->nData)  UJError(3,"UACDinvertV","nData missmatch");
 if (VectIn->nData != 2)               UJError(3,"UACDinvertV","nData missmatch");

 PhiIn  = VectIn->dataD;
 PhiOut = VectOut->dataD;
 
 nBin = sqrt(VectIn->nData);
 if (VectIn->nData != nBin*nBin)  UJError(3,"UACDinvertV","not a squared matix");

 if (VectIn->nData !=  1) UJError(3,"UACDinvertV","nBin is too big");

 mod = PhiIn[0]*PhiIn[0] + PhiOut[1]*PhiOut[1];
 PhiOut[0] =   PhiIn[0] / mod;
 PhiOut[1] = - PhiIn[1] / mod;
		  
 return;
}


/* ----------------------------------------------------- UACDNormalV----*/
void UACDNormalV(Vect, xnnorm)
struct UArrayCD *Vect;
double xnnorm;
/*
   This routine normalizes the incoming COMPLEX vector Vect into itself.
   After the routine, Vect will have its norm = xnnorm^2 * Vect->nData.
       Input  ->  Vect: pointer to the vector to normalize.
                  xnnorm: value used for the normalisation of Vect.
       Output ->  Vect: renormalized vector.
*/
/*--------------------------------------------------------------------*/
{
 double xnorm, xsnnorm, *Phi;
 long i;

 if ((Vect->nx[0] != 1) && (Vect->nx[1] != 1))
    {UJError(3,"UACDNormalV","nData missmatch");}

 Phi = Vect->dataD;

 xnorm = 0.;
 for (i=0; i<Vect->nData*2; i=i+2)
    {xnorm += Phi[i]*Phi[i] + Phi[i+1]*Phi[i+1];}
 
 if (xnorm <= 0.)  
    {xsnnorm = 0.;}
 else
    {xnorm   = sqrt(xnorm)/sqrt((double) Vect->nData);
     xsnnorm = xnnorm / xnorm;
    }

 for (i=0; i<Vect->nData*2; i=i+2)
    {Phi[i]   = xsnnorm * Phi[i];
     Phi[i+1] = xsnnorm * Phi[i+1];
    }

 return;
}


/*---------------------------------------------------------UACDProd---*/
void UACDProd(mat1, mat2, matOut)
struct UArrayCD *mat1, *mat2, *matOut;
/*                                  compute the product of 2 matrices */
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut, *temp;
 long i,j,k,index,index1,index2,nData,nC1,nC2,nCout;
 static struct UArrayCD *vectTemp;
 static long nDataLast=0;

 if (mat1->nx[1]   !=  mat2->nx[0])  UJError(3,"UACDProd","nData missmatch");
 if (matOut->nx[1] !=  mat2->nx[1])  UJError(3,"UACDProd","nData missmatch");
 if (matOut->nx[0] !=  mat1->nx[0])  UJError(3,"UACDProd","nData missmatch");

 Phi1   = mat1->dataD;
 Phi2   = mat2->dataD;
 PhiOut = matOut->dataD;
 nData  = matOut->nData;
 nC1    = mat1->nx[1];
 nC2    = mat2->nx[1];
 nCout  = matOut->nx[1];

 if (nData > nDataLast)
    {if (vectTemp != NULL) UACDFree(vectTemp);
     vectTemp  = UACDNew(1,nData);
     nDataLast = nData;
    }
 temp = vectTemp->dataD;
 for (i=0; i<nData*2; i++)  temp[i] = 0.;

 for (i=0; i<2*matOut->nx[0]; i=i+2)
    {for (j=0; j<2*nCout; j=j+2)
        {index = i*nCout + j;
         for (k=0; k<2*nC1; k=k+2)
	    {index1 = i*nC1 + k;
             index2 = k*nC2 + j;
             vectTemp->dataD[index]   += Phi1[index1]*Phi2[index2]
                                       - Phi1[index1+1]*Phi2[index2+1];
	     vectTemp->dataD[index+1] += Phi1[index1]*Phi2[index2+1]
                                       + Phi1[index1+1]*Phi2[index2];
            }
        }
    }

 for (i=0; i<nData*2; i++)  PhiOut[i] = vectTemp->dataD[i];

 return;
}


/*-------------------------------------------------UACDProdSq ---------*/
void UACDProdSq(Vect1, Vect2, VectOut)
struct UArrayCD *Vect1, *Vect2, *VectOut;
/*
   Computes the product of the 2 SQUARE COMPLEX matrices Phi1 andd Phi2 and puts it into PhiOut
       Input  ->  Vect1  : Pointer to first matrix.
                  Vect2  : Pointer to second matrix 
                          (nData2 could be nData1, nData1*nData1 or sqrt(nData1))
       Output ->  VectOut: Pointer to the matrix where the product will be put.
                          (could be Vect1 or Vect2)

     If one of the matrices has only one line, then it means that it is a 
     diagonal matrix stored as a vector.
*/
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut, *temp;
 long i,j,k,index,nData,nData1,nData2,nBin,nL1,nL2,nC1,nC2;
 static struct UArrayCD *vectTemp;
 static long nDataLast=0;

 nData1  = Vect1->nData;
 nData2  = Vect2->nData;
 nData   = (long) SIESTA_MAX( (double) nData1, (double) nData2);

 if (VectOut->nData != nData)  UJError(3,"UACDProdSq","nData missmatch");
 
 Phi1    = Vect1->dataD;
 Phi2    = Vect2->dataD;
 PhiOut  = VectOut->dataD;
 nL1     = Vect1->nx[0];
 nL2     = Vect2->nx[0];
 nC1     = Vect1->nx[1];
 nC2     = Vect2->nx[1];
 nBin    = (long) sqrt((double) nData);

 if (nData >  nDataLast)
    {if (vectTemp != NULL)  UACDFree(vectTemp);
     vectTemp  = UACDNew(1,nData);
     nDataLast = nData;
    }
 temp = vectTemp->dataD;
 for (i=0; i<nData*2; i++)  temp[i] = 0.;
      
 if (nL1 == nC1  &&  nL2 == nC2)
    {for (i=0; i<nBin*2; i=i+2)
        {for (j=0; j<nBin*2; j=j+2)
	    {index = i*nBin + j;
             for (k=0; k<nBin*2; k=k+2)
                {temp[index]   += Phi1[i*nBin+k] * Phi2[k*nBin+j] 
                                - Phi1[i*nBin+k+1] * Phi2[k*nBin+j+1];
                 temp[index+1] += Phi1[i*nBin+k+1] * Phi2[k*nBin+j] 
                                + Phi1[i*nBin+k] * Phi2[k*nBin+j+1];
                }
            }
        }
     for (i=0; i<nData*2; i++)  PhiOut[i] = temp[i];
    }

 else if (nL1 == nC1  &&  nL2 == 1)
    {for (i=0; i<nBin*2; i=i+2)
        {for (j=0; j<nBin*2; j=j+2)
	    {index         = i*nBin + j;
             temp[index]   = Phi1[index]   * Phi2[j]  -  Phi1[index+1] * Phi2[j+1];
             temp[index+1] = Phi1[index+1] * Phi2[j]  +  Phi1[index]   * Phi2[j+1];
            }
        }
     for (i=0; i<nData*2; i++)  PhiOut[i] = temp[i];
    }

 else if (nL1 == 1  &&  nL2 == nC2)
    {for (i=0; i<nBin*2; i=i+2)
        {for (j=0; j<nBin*2; j=j+2)
	    {index         = i*nBin + j;
             temp[index]   = Phi2[index]   * Phi1[i]  -  Phi2[index+1] * Phi1[i+1];
             temp[index+1] = Phi2[index+1] * Phi1[i]  +  Phi2[index]   * Phi1[i+1];
            }
        }
     for (i=0; i<nData*2; i++)  PhiOut[i] = temp[i];
    }

 else if (nL1 == 1  &&  nL2 == 1)
    {for (i=0; i<nData*2; i=i+2)
	{temp[i]     = Phi1[i]   * Phi2[i]  -  Phi1[i+1] * Phi2[i+1];
 	 temp[i+1]   = Phi1[i+1] * Phi2[i]  +  Phi1[i]   * Phi2[i+1];
	 PhiOut[i]   = temp[i];
	 PhiOut[i+1] = temp[i+1];
        }
    }

 else
    {UJError(3,"UACDProdSq","nData missmatch");}

 return;
}


/*---------------------------------------------------------UACDProdV---*/
void UACDProdV(mat1, mat2, matOut)
struct UArrayCD *mat1, *mat2, *matOut;
/*                                   compute the product of 2 vectors */
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut;
 long i,nData;
 double  PhiO[2];

 /* Element by element product. 
    The number of data of mat1, mat2 and matOut should be the same but 
    no check is made to same time. */

 Phi1   = mat1->dataD;
 Phi2   = mat2->dataD;
 PhiOut = matOut->dataD;
 nData  = matOut->nData;

 for (i=0; i<nData*2; i=i+2)
    {PhiO[0]     = Phi1[i  ] * Phi2[i]  -  Phi1[i+1] * Phi2[i+1];
     PhiO[1]     = Phi1[i+1] * Phi2[i]  +  Phi1[i  ] * Phi2[i+1];
     PhiOut[i]   = PhiO[0];
     PhiOut[i+1] = PhiO[1];
    }

 return;
}


/* ------------------------------------------------- UACDRScal ----- */                           
void UACDRScal( in, factor, out)
struct UArrayCD *in, *out;
double factor;
/* This function multiplies the COMPLEX matrix in by a real factor    */
/* The output matrix could be the input matrix                        */
/*--------------------------------------------------------------------*/
{
 double *phiIn, *phiOut;
 long index;
 
 if (in->nData != out->nData)  UJError(3,"UACDRScal","nData missmatch");

 phiIn  = in->dataD;
 phiOut = out->dataD;

 for (index=0; index<in->nData*2; index++)
    {phiOut[index] = phiIn[index] * factor;}
      
 return;
}


/*----------------------------------------------------- UACDScalV ---*/
void UACDScalV(Vect1, Vect2, ps)
struct UArrayCD *Vect1, *Vect2;
double ps[2];
/*
      Function to compute scalar-product of 2 complexe vectors
      ps = Phi1*conj(Phi2)   (i.e. ps[0]=real,ps[1]=imag)
*/
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2;
 long index;
 
 if (Vect1->nData != Vect2->nData)  UJError(3,"UACDScalV","nData missmatch");
 if (Vect1->nx[0] != 1)             UJError(3,"UACDScalV","nData missmatch");
 if (Vect2->nx[0] != 1)             UJError(3,"UACDScalV","nData missmatch");

 Phi1  = Vect1->dataD;
 Phi2  = Vect2->dataD;
 ps[0] = 0.;
 ps[1] = 0.;
 
 for (index=0;index<Vect1->nData*2;index=index+2)
    {ps[0] = ps[0] + Phi1[index]*Phi2[index]   + Phi1[index+1]*Phi2[index+1];
     ps[1] = ps[1] - Phi1[index]*Phi2[index+1] + Phi1[index+1]*Phi2[index];
    }
 ps[0] = ps[0] / (double)Vect1->nData;
 ps[1] = ps[1] / (double)Vect1->nData;
 
 return;
}


/*------------------------------------------------------UACDSolve3V----*/
void UACDSolve3V(va, vx, vb)
struct UArrayCD *va, *vx, *vb;
/*----------- this function solves the equation:  a*x = b
               x and b are vectors of 3 complex numbers
               a is a 3x3 matrix of complex numbers           
                     a0*x0 + a1*x1 + a2*x2 = b0
                     a3*x0 + a4*x1 + a5*x2 = b1
                     a6*x0 + a7*x1 + a8*x2 = b2                       */
/*--------------------------------------------------------------------*/
{
 double ap1r,ap1i,ap2r,ap2i,bp0r,bp0i,scale,t,cr,ci,dr,di,er,ei,zr,zi;
 double *a,*x,*b;

 if (vx->nData != 3)  UJError(3,"UACDSolve3V","nData missmatch");
 if (vb->nData != 3)  UJError(3,"UACDSolve3V","nData missmatch");
 if (va->nData != 9)  UJError(3,"UACDSolve3V","nData missmatch");
        
 a = va->dataD;
 b = vb->dataD;
 x = vx->dataD;
 
 /*--- first compute a'1=a1/a0  a'2=a2/a0 b'0=b0/a1 ---*/
 /*--- x0 = b'0 - a'1*x1 - a'2*x2 ---*/
   
 scale = 1. / (a[0]*a[0] + a[1]*a[1]);
 ap1r  = (a[2]*a[0] + a[3]*a[1]) * scale;
 ap1i  =-(a[2]*a[1] - a[3]*a[0]) * scale;
 ap2r  = (a[4]*a[0] + a[5]*a[1]) * scale;
 ap2i  =-(a[4]*a[1] - a[5]*a[0]) * scale;
 bp0r  = (b[0]*a[0] + b[1]*a[1]) * scale;
 bp0i  =-(b[0]*a[1] - b[1]*a[0]) * scale;

 /*--- compute  e = 1/(a4-a3*a'1) --- */
   
 er    = a[8] - a[6]*ap1r + a[7]*ap1i;
 ei    = a[9] - a[6]*ap1i - a[7]*ap1r;
 scale = 1. / (er*er + ei*ei);
 er    =  er * scale;
 ei    = -ei * scale;
 
 /*---- compute c = e*(b1-a3*b'0) and d = e*(a3*a'2-a5) ---*/
 /*---  x1 = c + d*z2 ---*/
   
 cr = b[2] - a[6]*bp0r + a[7]*bp0i;
 ci = b[3] - a[6]*bp0i - a[7]*bp0r;
 t  = er*cr - ei*ci;
 ci = er*ci + ei*cr;
 cr = t;
 dr = a[6]*ap2r - a[7]*ap2i - a[10];
 di = a[6]*ap2i + a[7]*ap2r - a[11];
 t  = er*dr - ei*di;
 di = er*di + ei*dr;
 dr = t;
 
 /*---  e = 1./(a8 + a7*d - a6*(ap1*d + a'2)) ---*/
  
 er    = ap1r*dr - ap1i*di + ap2r;
 ei    = ap1r*di + ap1i*dr + ap2i;
 t     = a[16] + a[14]*dr - a[15]*di - a[12]*er + a[13]*ei;
 ei    = a[17] + a[14]*di + a[15]*dr - a[12]*ei - a[13]*er;
 er    = t;
 scale = 1. / (er*er + ei*ei);
 er    = er * scale;
 ei    =-ei * scale;
 
 /*--- x2 = (b2 - a7*c - a6*(b'0 - c*a'1))*e ---*/ 
 
 zr   = bp0r - cr*ap1r + ci*ap1i;
 zi   = bp0i - cr*ap1i - ci*ap1r;
 t    = b[4] - a[14]*cr + a[15]*ci - a[12]*zr + a[13]*zi;
 zi   = b[5] - a[14]*ci - a[15]*cr - a[12]*zi - a[13]*zr;
 zr   = t;
 t    = zr*er - zi*ei;
 x[5] = zr*ei + zi*er;
 x[4] = t;

 /*--- x1 = c + d*z2 ---*/
 
 x[2] = cr + dr*x[4] - di*x[5];
 x[3] = ci + dr*x[5] + di*x[4];

 /*--- x0 = b'0 - a'1*x1 - a'2*x2 ---*/
 
 x[0] = bp0r - ap1r*x[2] + ap1i*x[3] - ap2r*x[4] + ap2i*x[5];
 x[1] = bp0i - ap1r*x[3] - ap1i*x[2] - ap2r*x[5] - ap2i*x[4];

 return ;
}


/*---------------------------------------------------------UACDSub ---*/
void UACDSub(mat1, mat2, matOut)
struct UArrayCD *mat1, *mat2, *matOut;
/*                  Subtract 2 COMPLEX matrices: matOut = mat1 - mat2 */
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut;
 long index;
 
 if (mat1->nData != mat2->nData)    UJError(3,"UACDSub","nData missmatch");
 if (mat1->nData != matOut->nData)  UJError(3,"UACDSub","nData missmatch");

 Phi1   = mat1->dataD;
 Phi2   = mat2->dataD;
 PhiOut = matOut->dataD;

 for (index=0; index<mat1->nData*2; index++)
    {PhiOut[index] = Phi1[index] - Phi2[index];}

 return;
}


/*---------------------------------------------------------UACDSub ---*/
void UACDsymLines(matIn, matOut)
struct UArrayCD *matIn, *matOut;
/* Perform symetry wrt horizontal axis on matIn 
   (switch first and last lines, etc)
   matIn and matOut could be the same                                 */
/*--------------------------------------------------------------------*/
{
 double *PhiIn,*PhiOut,phi;
 long i,j,nL,nC;

 nL = matIn->nx[0];
 nC = matIn->nx[1];

 if (nL != matOut->nx[0]  ||  nC != matOut->nx[1])
   {UJError(3,"UACDsymLines",
              "Input and output matrices must have same line and column numbers");}

 PhiIn  = matIn->dataD;
 PhiOut = matOut->dataD;

 for (j=0; j<2*nC; j++)
    {for (i=0; i<=(nL-1)/2; i++)
        {phi                     = PhiIn[2*(nL-1-i)*nC+j];
         PhiOut[2*(nL-1-i)*nC+j] = PhiIn[2*i*nC+j];
         PhiOut[2*i*nC+j]        = phi;
        }
    }

 return;
}


/* --------------------------------------------------- UACDunitSub ---*/
void UACDunitSub(VectIn, VectOut)
struct UArrayCD *VectIn, *VectOut;
/*
     VectOut = 1 - VectIn  where 1 is the unity matrix
*/
/*--------------------------------------------------------------------*/
{
 double *PhiIn, *PhiOut;
 long i,j,index,nBin;
    
 if (VectIn->nData  != VectOut->nData)  UJError(3,"UACDunitSub","nData missmatch");
 if (VectIn->nx[0]  != VectIn->nx[1])   UJError(3,"UACDunitSub","not a square matrix");
 if (VectOut->nx[0] != VectOut->nx[1])  UJError(3,"UACDunitSub","not a square matrix");

 PhiIn  = VectIn->dataD;
 PhiOut = VectOut->dataD;
 nBin   = VectIn->nx[0];

 for (i=0; i<nBin; i++)
    {for (j=0; j<nBin; j++)
        {index = i*nBin + j;
         if (i == j)  PhiOut[2*index] = 1. - PhiIn[2*index];
         else         PhiOut[2*index] =    - PhiIn[2*index];
         PhiOut[2*index+1] = - PhiIn[2*index+1];
        }
    }
		  
 return;
}


/*----------------------------------------------------- UACFfft --------*/
void UACFfft(Vect, msign)
struct UArrayCF *Vect;
long msign;
/*--------------------------------------------------------------------*/
/* cffft computes the finite Fourier transform of a complex periodic  */
/* sequence, whose period n must be a power of 2 (n=2**|msign|).      */
/* To ensure optimum use of storage, the same array is used for input */
/* and output. If msign is positive the direct Fourier transform is   */
/* computed. If msign is negative, the inverse Fourier transform is   */
/* computed. This code is a c translation of the routine D704 (CFFT)  */
/* from CERNLIB. See the CERN writeup for more information.           */
/* Author: B. Mours        27/02/91                                   */
/*--------------------------------------------------------------------*/
 
{
 float s,ur,ui,ur0,wr,wi,tr,ti,*a,temp;
 double c;
 long  m,n,nv2,nm1,i,j,k,le,l,le1,ip;
 long indexI,indexJ,indexIp;

 a = Vect->dataF;
 
 if (msign == 0) return;
 m = msign;
 if (msign < 0) m = -m;
 n = 1;
 for (i=0; i<m; i++) n=2*n;
 nv2 = n/2;
 nm1 = n-1;
 j   = 0;
 
 for (i=0; i<nm1; i++)
    {indexI=2*i;
     if (i<j)
        {indexJ      = 2*j;
         tr          = a[indexJ];
         ti          = a[indexJ+1];
         a[indexJ  ] = a[indexI];
         a[indexJ+1] = a[indexI+1];
         a[indexI  ] = tr;
         a[indexI+1] = ti;
        }
     k = nv2 - 1;
     while (k<j)
        {j = j - k - 1;
         k = (k-1) / 2;
        }
 
     j = j + k + 1;
    }
 
 for (i=0; i<n; i=i+2)
    {indexI       = 2*i;
     tr           = a[2*(i+1)];
     ti           = a[2*(i+1)+1];
     a[2*(i+1)  ] = a[indexI  ] - tr;
     a[2*(i+1)+1] = a[indexI+1] - ti;
     a[indexI  ]  = a[indexI  ] + tr;
     a[indexI+1]  = a[indexI+1] + ti;
    }
 
 if (m == 1) return;
 c  = 0;
 s  = 1.;
 if (msign < 0) s = -1.;
 le = 2;
 
 for (l=2; l<=m; l++)
    {wr  = c;
     wi  = s;
     ur  = wr;
     ui  = wi;
     c   = sqrt(c*.5+.5);
     s   = wi / (c+c);
     le1 = le;
     le  = le1 + le1;
 
     for (i=0; i<n; i=i+le)
        {ip           = i + le1;
         indexI       = 2*i;
         indexIp      = 2*ip;
         tr           = a[indexIp];
         ti           = a[indexIp+1];
         a[indexIp  ] = a[indexI  ] - tr;
         a[indexIp+1] = a[indexI+1] - ti;
         a[indexI  ]  = a[indexI  ] + tr;
         a[indexI+1]  = a[indexI+1] + ti;
        }
 
     for (j=1; j<le1; j++)
        {for (i=j; i<n; i=i+le)
            {ip           = i + le1;
             indexI       = 2*i;
             indexIp      = 2*ip;
             tr           = a[indexIp  ]*ur - a[indexIp+1]*ui;
             ti           = a[indexIp+1]*ur + a[indexIp  ]*ui;
             a[indexIp  ] = a[indexI  ] - tr;
             a[indexIp+1] = a[indexI+1] - ti;
             a[indexI  ]  = a[indexI  ] + tr;
             a[indexI+1]  = a[indexI+1] + ti;
            }
 
         ur0 = ur;
         ur  = ur*wr - ui*wi;
         ui  = ui*wr + ur0*wi;
        }
    }

 for (i=1; i<n/2; i++)
    {temp         = a[2*i];
     a[2*i]       = a[2*(n-i)];
     a[2*(n-i)]   = temp;
     temp         = a[2*i+1];
     a[2*i+1]     = a[2*(n-i)+1];
     a[2*(n-i)+1] = temp;
    }
 
 return;
}


/* ----------------------------------------------------- UADAdd ------*/
void UADAdd(mat1, mat2, matOut)
struct UArrayD *mat1, *mat2, *matOut;
/*                          Add 2 real matrices: matOut = mat1 + mat2 */
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut;
 long index;
    
 if (mat1->nData != mat2->nData)    UJError(3,"UADAdd","nData missmatch");
 if (mat1->nData != matOut->nData)  UJError(3,"UADAdd","nData missmatch");

 Phi1   = mat1->dataD;
 Phi2   = mat2->dataD;
 PhiOut = matOut->dataD;
 
 for (index=0; index<mat1->nData; index++)
    {PhiOut[index] = Phi1[index] + Phi2[index];}
		
 return;
}


/*-------------------------------------------------------UADC2d ---*/
void UADC2d(A,B,Ad,Bd,t)
struct UArrayD *A, *B,*Ad,*Bd;
double t;
/*    compute discrete matrix representation from continuous one       */
/*-----------------------------------------------------------------*/
{
 long i,j,indice1,indice2;
 struct UArrayD *AB,*ABE;

 AB  = (struct UArrayD*) UADNew((A->nx[0]+B->nx[1]),(A->nx[1]+B->nx[1]));
 ABE = (struct UArrayD*) UADNew((A->nx[0]+B->nx[1]),(A->nx[1]+B->nx[1]));

 for (i=0; i<A->nx[1]; i++)
    {for (j=0; j<A->nx[0]; j++)
        {indice1            = j * AB->nx[1] + i;
         indice2            = j * A->nx[1]  + i;
         AB->dataD[indice1] = A->dataD[indice2] * t;
        }
    }

 for (i=0; i<B->nx[1]; i++)
    {for (j=0; j<B->nx[0]; j++)
        {indice1            = j * AB->nx[1] + A->nx[1] + i;
         indice2            = j * B->nx[1] + i;
         AB->dataD[indice1] = B->dataD[indice2] * t;
        }
    }

 for (i=0; i<AB->nx[1]; i++)
    {for (j=A->nx[0]; j<AB->nx[0]; j++)
        {indice1            = j * AB->nx[1] + i;
         AB->dataD[indice1] = 0.;
        }
    }

 UADExpm(AB,ABE);

 for (i=0; i<A->nx[1]; i++)
    {for (j=0; j<A->nx[0]; j++)
        {indice1            = j * A->nx[1]   + i;
         indice2            = j * ABE->nx[1] + i;
         Ad->dataD[indice1] = ABE->dataD[indice2];
        }
    }

 for (i=0; i<B->nx[1]; i++)
    {for (j=0; j<A->nx[0]; j++)
        {indice1            = j * B->nx[1]   + i;
         indice2            = j * ABE->nx[1] + A->nx[1] + i;
         Bd->dataD[indice1] = ABE->dataD[indice2];
        }
    }

 UADFree(AB);
 UADFree(ABE);

 return;
}

/*----------------------------------------------------UADCopy --------*/
void UADCopy(In, Out)
struct UArrayD *In, *Out;
/*
  Copy the real matrix PhiIn into PhiOut.
      Input  ->  PhiIn : Matrix which will BE COPIED.
      Output ->  PhiOut: Matrix INTO which PhiIn will be copied.
*/
/*--------------------------------------------------------------------*/
{
 double *PhiIn, *PhiOut;
 long index;
 
 if (In->nData != Out->nData)  UJError(3,"UADCopy","nData missmatch");

 PhiIn  = In->dataD;
 PhiOut = Out->dataD;

 for (index=0; index<In->nData; index++)
    {PhiOut[index] = PhiIn[index];}
		
 return;
}


/*------------------------------------------------------ UADfft -----*/
void  UADfft(Vect, msign)
struct UArrayD *Vect;
long msign;
/*  n=2**|msign| msign>0 => FFT,msign<0 => IFFT                          
    Calculates the Fourier transfrom of a set of n double data points.   
    Replace this data by the positive frequency half of its the complex  
    Fourier transform. The real-valued first and last components of     
    complex transform are returned as elements Vect[0] and Vect[1],   
    respectively.  n must be a power of 2. This routine also calculates 
    the inverse transform of  a complex data array if it is the tranform 
    of real data. (Result in the case must be multiplied by 2/n         
    From numerical recipes :  (realft)   See ''Numerial Recipes in C''   */
/*--------------------------------------------------------------------*/
{
 unsigned long i,i1,i2,i3,i4,np1,n;
 short isign,m;
 double c1=0.5,c2,h1r,h1i,h2r,h2i,*data;
 double wr,wi,wpr,wpi,wtemp,theta;
 data=Vect->dataD;

 if (msign == 0) return;
 if (msign < 0)
    {m     = -msign;
     isign = -1;
    }
 else
    {isign = 1;
     m     = msign; 
    }

 n = 1;
 for (i=0; i<m; i++)  n = 2*n;
 theta = 3.141592653589793 / (double) (n>>1);

 if (isign == 1)
    {c2 = -0.5;
     UACDfft(Vect,(long) (m-1));
    }
 else
    {c2    = 0.5;
     theta = -theta;
    }

 wtemp = sin(0.5*theta);
 wpr   = -2.0 * wtemp * wtemp;
 wpi   = sin(theta);
 wr    = wpr + 1.;
 wi    = wpi;
 np1   = n + 1;
 for (i=1; i<(n>>2); i++)
    {i4       = 1 + (i3=np1 - (i2=1 + (i1=i+i)));
     h1r      =  c1 * (data[i1] + data[i3]);
     h1i      =  c1 * (data[i2] - data[i4]);
     h2r      = -c2 * (data[i2] + data[i4]);
     h2i      =  c2 * (data[i1] - data[i3]);
     data[i1] =  h1r + wr*h2r - wi*h2i;
     data[i2] =  h1i + wr*h2i + wi*h2r;
     data[i3] =  h1r - wr*h2r + wi*h2i;
     data[i4] = -h1i + wr*h2i + wi*h2r;
     wr       = (wtemp=wr)*wpr - wi*wpi    + wr;
     wi       =         wi*wpr + wtemp*wpi + wi;
    }

 if (isign == 1)
    {data[0] = (h1r=data[0]) + data[1];
     data[1] =  h1r          - data[1];
    }
 else
    {data[0] = c1 * ((h1r=data[0]) + data[1]);
     data[1] = c1 * ( h1r          - data[1]);
     UACDfft(Vect,-1*(long) (m-1));
    }
 return;
}


/*----------------------------------------------------------UADInv ---*/
double UADInv(in, out)
struct UArrayD *in, *out;
               /* invert the real matrix in using the 'pivots' method `
               return the determinant of in Matrix */
/*--------------------------------------------------------------------*/
{
 long *l,*m;
 long nk,j,ji,ki,i,jp,jk,iz,k,kk,ij,ik,jq,jr,kj,n;
 double biga,hold,determinant, *a;
 
 if (out->nData != in->nData)   UJError(3,"UADInv","nData missmatch");
 if (out->nx[1] != in->nx[1])   UJError(3,"UADInv","nC missmatch");
 if (out->nx[1] != out->nx[0])  UJError(3,"UADInv","nL missmatch");
 
 a = out->dataD;
 n = out->nx[1];

 for (i=0; i<n*n; i++)  out->dataD[i] = in->dataD[i];
 
 l = (long *) calloc(n,sizeof(long));
 m = (long *) calloc(n,sizeof(long));
 if (l == NULL  ||  m == NULL)  UJError(3,"UADInv","malloc failed");
 
 determinant = 1.;
 nk = -n;
 for (k=1; k<=n; k++)
    {nk     = nk + n;
     l[k-1] = k;
     m[k-1] = k;
     kk     = nk + k;
     biga   = a[kk-1];
     for (j=k; j<=n; j++)
        {iz = n*(j-1);
         for (i=k; i<=n; i++)
            {ij = iz + i;
             if (fabs(biga) < fabs(a[ij-1]))
                {biga = a[ij-1];
                 l[k-1] = i;
                 m[k-1] = j;
                }
            }
        }
     j = l[k-1];
     if (j > k)
        {ki = k - n;
         for (i=1; i<=n; i++)
            {ki      = ki + n;
             hold    = -a[ki-1];
             ji      = ki - k + j;
             a[ki-1] = a[ji-1];
             a[ji-1] = hold;
            }
        }
     i = m[k-1];
     if (i > k)
        {jp = n*(i-1);
         for (j=1; j<=n; j++)
            {jk      = nk + j;
             ji      = jp + j;
             hold    = -a[jk-1];
             a[jk-1] = a[ji-1];
             a[ji-1] = hold;
            }
        }
     if (biga == 0)
        {determinant = 0.;
         free(l);
         free(m);
         UJError(3,"UADInv","determinant close to zero");
         return(determinant);
        }

     for (i=1; i<=n; i++)
        {if (i != k)
            {ik      = nk + i;
             a[ik-1] = - a[ik-1] / biga;
            }
        }
     for (i=1; i<=n; i++)
        {ik   = nk + i;
         hold = a[ik-1];
         ij   = i - n;
         for (j=1; j<=n; j++)
            {ij = ij + n;
             if (i != k)
                {if (j != k)
                    {kj      = ij - i + k;
                     a[ij-1] = hold*a[kj-1] + a[ij-1];
                    }
                }
            }
        }
     kj = k-n;
     for (j=1; j<=n; j++)
        {kj = kj + n;
         if (j != k) a[kj-1] = a[kj-1] / biga;
        }
     determinant = determinant * biga;
     a[kk-1]     = 1. / biga;
    }

 k = n;
 while (k > 1)
    {k = k - 1;
     i = l[k-1];
     if (i > k)
        {jq = n*(k-1);
         jr = n*(i-1);
         for (j=1; j<=n; j++)
            {jk      = jq + j;
             hold    = a[jk-1];
             ji      = jr + j;
             a[jk-1] = -a[ji-1];
             a[ji-1] = hold;
            }
        }
     j = m[k-1];
     if (j > k)
        {ki = k - n;
         for (i=1; i<=n; i++)
            {ki      = ki + n;
             hold    = a[ki-1];
             ji      = ki-k+j;
             a[ki-1] = -a[ji-1];
             a[ji-1] = hold;
            }
        }
    }
 
 free(l);
 free(m);
 return(determinant);
}


/*--------------------------------------------------------UADPolProdV--*/
void UADPolProdV(Vect1, Vect2, VectOut)
struct UArrayD *Vect1, *Vect2, *VectOut;
                             /*  Computes the product of two polynoms */
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut;
 long i,j;
 
 if (Vect1->nData + Vect2->nData != VectOut->nData)
    {UJError(3,"UADPolProdV","nData missmatch");}
 if (Vect1->nx[0] != 1  ||  Vect2->nx[0] != 1  ||  VectOut->nx[0] != 1)
    {UJError(3,"UADPolProdV","nData missmatch");}

 Phi1   = Vect1->dataD;
 Phi2   = Vect2->dataD;
 PhiOut = VectOut->dataD;

 for (i=0; i<Vect1->nData+Vect2->nData; i++)  PhiOut[i] = 0;

 for (i=0; i<Vect1->nData; i++)
    {for (j=0; j<Vect2->nData; j++)
        {PhiOut[i+j] = PhiOut[i+j] + Phi1[i] * Phi2[j];}
    }

 return;
}


/*--------------------------------------------------------UADPolRootV--*/
long UADPolRootV(polyn, root)
struct UArrayD *polyn;
struct UArrayCD *root;
                                /*  Computes the roots of polynom pol */
/*--------------------------------------------------------------------*/
{
 long i,k,n,l,ierr, igh,low,idimpoly;
 double *origin;
 struct UArrayD *matrice,*zmatrice;

 if (polyn->nx[0] != 1  ||  root->nx[0] != 1)  UJError(3,"UADPolRootV","nData missmatch");

 idimpoly = polyn->nData - 1;

 if (polyn->dataD[idimpoly] == 0.)  return(-2);
 ierr = 0;

 i = 0;
 while (polyn->dataD[i] == 0.)
    {root->dataD[2*i]   = 0.;
     root->dataD[2*i+1] = 0.;
     i++;
    }

 n = idimpoly - i;
 if (n > 0)
    {matrice  = UADNew(n,n);
     zmatrice = UADNew(n,n);

     /*--- n contains the number of roots to find = matrix dimension ---*/

     for (k=0; k<n; k++)
        {for (l=0; l<n; l++) matrice->dataD[k*n+l] = 0.;}

    for (k=1; k<n; k++)
       {matrice->dataD[k*n+k-1] = 1.;}
    for (k=0; k<n; k++)
       {matrice->dataD[k] = - polyn->dataD[idimpoly-1-k]/polyn->dataD[idimpoly];}
                                         
    origin      = root->dataD;
    root->dataD = &(root->dataD[2*i]);
    UADValVec (matrice, zmatrice, root, &low, &igh, &ierr);
    root->dataD = origin;

    UADFree(matrice);
    UADFree(zmatrice);
    return(0);
   }

 return (ierr);
}


/*----------------------------------------------------------UADProd---*/

void UADProd(mat1, mat2, matOut)
struct UArrayD *mat1, *mat2, *matOut;
/*                                  compute the produce of 2 matrices */
/*--------------------------------------------------------------------*/

{
 long kjn,jm,ni,i,j,k,l,m,n;
 double *a, *b, *bStart, *bEnd, *r, *rStart, *rEnd, tmp;

 if (mat1->nx[1] !=  mat2->nx[0])    UJError(3,"UADProd","nData missmatch");
 if (mat1->nx[0] !=  matOut->nx[0])  UJError(3,"UADProd","nData missmatch");
 if (mat2->nx[1] !=  matOut->nx[1])  UJError(3,"UADProd","nData missmatch");

 a = mat1->dataD;
 b = mat2->dataD;
 r = matOut->dataD;

 l = mat1->nx[0];
 m = mat1->nx[1];
 n = mat2->nx[1];

 if (n == 1)       /*--- matrix-vector: simplified code (A.Vicere', PISA) ---*/
    {
     bStart = b;
     bEnd   = b + m;
     rStart = r;
     rEnd   = r + l;
     for (r=rStart; r<rEnd; r++)
        {tmp  = 0.0; 
	 for (b=bStart; b < bEnd; b++, a++)  tmp += (*a) * (*b);
	 *r   = tmp;
        }
    }
 else             /*--- general code ---*/
    {
     kjn = -1;
     jm  = -m;
     for (j=0; j<l; j++)
        {jm = jm + m;
         for (k=0; k<n; k++)
            {kjn++;
             r[kjn] = 0.;
             ni     = -n;
             for (i=0; i<m; i++)
                {ni     = ni + n;
                 tmp    = a[jm+i]*b[k+ni];
                 r[kjn] = r[kjn] + tmp;
                }
            }
        }
    }

 return;
}


/*-----------------------------------------------------------UADrk4--*/
void UADrk4(matA, matB, matC, matD, matY, matU, matX0, path)
struct UArrayD *matA, *matB, *matC, *matD, *matY, *matU, *matX0;
double path;
/*
    Integration of differiential equation:
	.
	X =A X  + B U
	Y =C X  + D U
    matA of dim n,n
	matB of dim n,m
	matC of dim p,n
	matD of dim p,m
	matY of dim 1,p
	matU of dim m,1
    matX0: state before integration and after integration
	     n   :number of states
	     m   :number of inputs
	     p   :number of outputs
	path: time of integration
	     (should be carefully chosen)
*/
/*-----------------------------------------------------------UADrk4--*/
{
 long i,j,k,q;
 double *X,*k1,*l1,*m1,*n1, *A, *B, *C, *D, *U, *Y, *X0;
 double time,h,hmax,delta,tol;
 double tau,constfix;

 A  = matA->dataD;
 B  = matB->dataD;
 C  = matC->dataD;
 D  = matD->dataD;
 U  = matU->dataD;
 Y  = matY->dataD;
 X0 = matX0->dataD;

 if (matA->nx[0] !=  matA->nx[1])  UJError(3,"UADrk4","nC missmatch");
 if (matB->nx[0] !=  matA->nx[0])  UJError(3,"UADrk4","nC missmatch");
 if (matC->nx[1] !=  matA->nx[0])  UJError(3,"UADrk4","nC missmatch");
 if (matD->nx[0] !=  matC->nx[0])  UJError(3,"UADrk4","nC missmatch");
 if (matD->nx[1] !=  matB->nx[1])  UJError(3,"UADrk4","nC missmatch");
 if (matU->nx[0] !=  matD->nx[1])  UJError(3,"UADrk4","nC missmatch");
 if (matY->nx[0] !=  matD->nx[0])  UJError(3,"UADrk4","nC missmatch");

 k1 = (double*) calloc(matA->nx[0],sizeof(double));
 l1 = (double*) calloc(matA->nx[0],sizeof(double));
 m1 = (double*) calloc(matA->nx[0],sizeof(double));
 n1 = (double*) calloc(matA->nx[0],sizeof(double));
 X  = (double*) calloc(matA->nx[0],sizeof(double));

 if (k1 == NULL)  UJError(2,"UADrk4"," malloc failed");
 if (l1 == NULL)  UJError(2,"UADrk4"," malloc failed");
 if (m1 == NULL)  UJError(2,"UADrk4"," malloc failed");
 if (n1 == NULL)  UJError(2,"UADrk4"," malloc failed");
 if (X  == NULL)  UJError(2,"UADrk4"," malloc failed");

 /*--- constants ---*/
 hmax = path / 2.;
 h    = hmax / 4.;
 time = 0.; 
 tol  = 1.e-6;

 while (time < path  &&  time+h > time) 
    {
     if (time+h > path)  h = path - time;
   
     /*--- compute new state using 3d order with variable path ---*/

     for (j=0; j<matA->nx[0]; j++)  X[j] = X0[j];

     for (i=0; i<matA->nx[0]; i++)
        {k1[i] = 0;
         for (k=0; k<matA->nx[0]; k++)
            {k1[i] = k1[i] + A[i*(matA->nx[1])+k] * X[k];}
         for (q=0; q<matB->nx[1]; q++)
            {k1[i] = k1[i] + B[i*(matB->nx[1])+q] * U[q];}
         k1[i] = k1[i] * h;
        }

     for (j=0; j<matA->nx[0]; j++)   X[j] = X0[j] + k1[j];

     for (i=0; i<matA->nx[0]; i++)
        {l1[i] = 0;
         for (k=0; k<matA->nx[0]; k++)
            {l1[i] = l1[i] + A[i*(matA->nx[1])+k] * X[k];}
         for (q=0; q<matB->nx[1]; q++)
            {l1[i] = l1[i] + B[i*(matB->nx[1])+q] * U[q];}
         l1[i] = l1[i] * h;
        }

     for (j=0; j<matA->nx[0]; j++)  X[j] = X0[j] + (k1[j]+l1[j]) / 4;

     for (i=0; i<matA->nx[0]; i++)
        {m1[i] = 0;
         for (k=0; k<matA->nx[0]; k++)
            {m1[i] = m1[i] + A[i*(matA->nx[1])+k] * X[k];}
         for (q=0; q<matB->nx[1]; q++)
            {m1[i] = m1[i] + B[i*(matB->nx[1])+q] * U[q];}
         m1[i] = m1[i] * h;
        }
     delta = 0.;
     for (j=0; j<matA->nx[0]; j++)
        {constfix = fabs((k1[j]-2*m1[j]+l1[j])/3.);
         if (constfix > delta)  delta = constfix;     /* modif ici */
        }
     tau = 0.;
     for (j=0; j<matA->nx[0]; j++)
        {if (fabs(X0[j]) > tau)  tau = fabs(X0[j]);}
     if (tau < 1.)  tau = 1.;
     tau = tau * tol;
     if (delta <= tau)
        {time = time + h;
         for (i=0; i<matA->nx[0]; i++)
            {X0[i] = X0[i] + (k1[i]+4.0*m1[i]+l1[i])/6.;}
        }
     if (delta != 0.)
        {h = 0.9 * h * pow(tau/delta,1.0/3.0);
         if (hmax < h)  h = hmax;
        }
    }
 if (time < path)  UJError(2,"UADrk4"," Singularity");
 
 /*--- compute Y=CX+DU ---*/
 for (q=0; q<matC->nx[0]; q++)
    {Y[q] = 0;
     for (i=0; i<matA->nx[0]; i++)  Y[q] = Y[q] + C[q*matC->nx[1]+i] * X0[i];
     for (i=0; i<matB->nx[1]; i++)  Y[q] = Y[q] + D[q*matD->nx[1]+i] * U[i];
    }
 free(k1);
 free(l1);
 free(m1);
 free(n1);
 free(X);
 return;
}


/* ---------------------------------------------------- UADScal ----- */                           
void UADScal( in, factor, out)
struct UArrayD *in, *out;
double factor;
/*    This function multiplies the real matrix in by a real factor    */
/*--------------------------------------------------------------------*/
{
 double *phiIn, *phiOut;
 long index;
 
 if (in->nData != out->nData)  UJError(3,"UADscal","nData missmatch");
 
 phiIn  = in->dataD;
 phiOut = out->dataD;

 for (index=0; index<in->nData; index++)
    {phiOut[index] = phiIn[index] * factor;}
      
 return;
}

  
/*----------------------------------------------------------UADSub ---*/
void UADSub(mat1, mat2, matOut)
struct UArrayD *mat1, *mat2, *matOut;
/*                     Subtract 2 real matrices: matOut = mat1 - mat2 */
/*--------------------------------------------------------------------*/
{
 double *Phi1, *Phi2, *PhiOut;
 long index;
 
 if (mat1->nData != mat2->nData)    UJError(3,"UADSub","nData missmatch");
 if (mat1->nData != matOut->nData)  UJError(3,"UADSub","nData missmatch");

 Phi1   = mat1->dataD;
 Phi2   = mat2->dataD;
 PhiOut = matOut->dataD;

 for (index=0; index<mat1->nData; index++)
    {PhiOut[index] = Phi1[index] - Phi2[index];}

 return;
}


/*----------------------------------------------------------UADTran---*/
void UADTran(in, out)
struct UArrayD *in, *out;
                                            /* tranpose a real matrix */
/*--------------------------------------------------------------------*/
{
 long ir,mi,i,j,n,m;
 double *a, *r;
 
 if (in->nx[1] != out->nx[0])  UJError(3,"UADInv","nC missmatch");
 if (in->nx[0] != out->nx[1])  UJError(3,"UADInv","nL missmatch");
 
 a = in->dataD;
 r = out->dataD;
 n = in->nx[0];
 m = in->nx[1];
 
 ir = -1;
 for (i=0; i<m; i++)
    {mi = i - m;
     for (j=0; j<n; j++)
        {ir++;
         mi    = mi + m;
         r[ir] = a[mi];
        }
    }
 return;
}


/*---------------------------------------------------------UADValVec*/
void UADValVec(matA, matZ, matW, low, igh, ierr)
struct UArrayD *matA, *matZ;
struct UArrayCD *matW;
long *low, *igh, *ierr;
/*------------------------------------------------------------------*/
/*
    Calcule les valeurs propres et vecteurs propres (complexes)
    d'une matrice reelle

    input parameters:
            a : tableau (ou pointeur) de flottants : matrice
	    n     : nombre de lignes
	    nm    : nombre de colonnes
    output parameters:
	    z : tableu des vecteurs propres
	    w : tableau (ou pointeur) de flottants : valeurs propres
	    low : pointeur sur entier
	    igh : pointeur sur entier
	    ierr : pointeur sur entier : type d'erreur : 0 =O.K.
*/
/*------------------------------------------------------------------*/
{
 long m, n, nm, *inte;
 double *scale;

 n  = matA->nx[0];
 nm = matA->nx[1];
 if (n  !=  matZ->nx[0])  UJError(3,"UADrk4","nC missmatch");
 if (nm !=  matZ->nx[1])  UJError(3,"UADrk4","nC missmatch");

 scale = (double*) calloc(matA->nx[0],sizeof(double));
 inte  = (long*)   calloc(matA->nx[0],sizeof(long));
 if (scale == NULL)  UJError(2,"UADValVec"," malloc failed");
 if (inte  == NULL)  UJError(2,"UADValVec"," malloc failed");

 UADvvBalanc(matA, low, igh, scale);
 UADvvElmhes(matA, low, igh, inte);
 UADvvEltran(matA, matZ, low, igh, inte);
 UADvvHqr2  (matA, matW, matZ, low, igh, ierr);

 if (*ierr != 0) return;
 
 m = 0;
 UADvvBalbak(matZ, low, igh, scale, m);

 return;
}


/*-------------------------------------------------------UADvvBalanc*/
void UADvvBalanc(matA, low, igh, scale)
struct UArrayD *matA;
long *low, *igh;
double *scale;                   /* function only used by UADValVec */
/*------------------------------------------------------------------*/
{
 double c,f,g,r,s,b2,radix, *a;
 long i,j,k,l,jj,m, flag, iexc, noconv,n,nm;

 a  = matA->dataD;
 n  = matA->nx[0];
 nm = matA->nx[1];
  
 radix = 2.;
 b2    = radix * radix;
 k     = 1;
 l     = n;
 goto l100;

 l20:
 scale[m-1] = j;
 if (j != m)
    {for (i=1; i<=l; i++)
        {f               = a[(i-1)*nm+j-1];
         a[(i-1)*nm+j-1] = a[(i-1)*nm+m-1];
         a[(i-1)*nm+m-1] = f;
        }
     for (i=k; i<=n; i++)
        {f               = a[(j-1)*nm+i-1];
         a[(j-1)*nm+i-1] = a[(m-1)*nm+i-1];
         a[(m-1)*nm+i-1] = f;
        }
    }
 if (iexc == 2) goto l130;
 if (l == 1)    goto l280;
 l--;

 l100:
 for (jj=1; jj<=l; jj++)
    {j    = l - jj + 1;
     flag = 0;
     for (i=1; i<=l; i++)
        {if (i != j  &&  a[(j-1)*nm+i-1] != 0.)  {flag = 1; break;}}
     if (flag == 0)
        {m    = l;
         iexc = 1;
         goto l20;
        }
    }
 goto l140;

 l130:
 k++;

 l140:
 for (j=k; j<=l; j++)
    {flag = 0;
     for (i=k; i<=l; i++)
        {if (i != j  &&  a[(i-1)*nm+j-1] != 0.)  {flag = 1; break;}}
     if (flag == 0)
        {m    = k;
         iexc = 2;
         goto l20;
        }
    }
 for (i=k; i<=l; i++)  scale[i-1] = 1.;

 l190:
 noconv = 0;
 for (i=k; i<=l; i++)
    {c = 0.;
     r = 0.;
     for (j=k; j<=l; j++)
        {if (j != i)
            {c += fabs(a[(j-1)*nm+i-1]);
             r += fabs(a[(i-1)*nm+j-1]);
            }
        }
     g = r / radix;
     f = 1.;
     s = c + r;
     while(c < g)
        {f = f * radix;
         c = c * b2;
        }
     g = r * radix;
     while(c >= g)
        {f = f / radix;
         c = c / b2;
        }
     if ((c+r)/f < 0.95*s)
        {scale[i-1] = scale[i-1] * f;
         g          = 1. / f;
         noconv     = 1;
         for (j=k; j<=n; j++)  a[(i-1)*nm+j-1] = a[(i-1)*nm+j-1] * g;
         for (j=1; j<=l; j++)  a[(j-1)*nm+i-1] = a[(j-1)*nm+i-1] * f;
        }
    }
 if (noconv == 1)  goto l190;

 l280:
 *low = k;
 *igh = l;
 return;
}


/*-------------------------------------------------------UADvvElmhes*/
void UADvvElmhes(matA, low, igh, inte)
struct UArrayD *matA;
long *low, *igh, *inte;   /* function only used by UADValVec */
/*------------------------------------------------------------------*/
{
 double x,y, *a;
 long i,j,m,mm1,la,kp1,mp1,n,nm;
 
 a   = matA->dataD;
 n   = matA->nx[0];
 nm  = matA->nx[1];

 la  = *igh - 1;
 kp1 = *low + 1;
 if (la < kp1) return;

 for (m=kp1; m<=la; m++)
    {mm1 = m - 1;
     x   = 0.;
     i   = m;
     for (j=m; j<=*igh; j++)
        {if (fabs(a[(j-1)*nm+mm1-1]) > fabs(x))
            {x = a[(j-1)*nm+mm1-1];
             i = j;
            }
        }
     inte[m-1] = i;
     if (i != m)
        {for (j=mm1; j<=n; j++)
            {y               = a[(i-1)*nm+j-1];
             a[(i-1)*nm+j-1] = a[(m-1)*nm+j-1];
             a[(m-1)*nm+j-1] = y;
            }
         for (j=1; j<=*igh; j++)
            {y               = a[(j-1)*nm+i-1];
             a[(j-1)*nm+i-1] = a[(j-1)*nm+m-1];
             a[(j-1)*nm+m-1] = y;
            }
        }
     if (x != 0.0)
        {mp1 = m + 1;
         for (i=mp1; i<=*igh; i++)
            {y = a[(i-1)*nm+mm1-1];
             if (y != 0.)
                {y = y / x;
                 a[(i-1)*nm+mm1-1] = y;
                 for (j=m; j<=n; j++)     a[(i-1)*nm+j-1] = a[(i-1)*nm+j-1] - y*a[(m-1)*nm+j-1];
                 for (j=1; j<=*igh; j++)  a[(j-1)*nm+m-1] = a[(j-1)*nm+m-1] + y*a[(j-1)*nm+i-1];
                }
            }
        }
    }

 return;
}


/*-------------------------------------------------------UADvvEltran*/
void UADvvEltran(matA, matZ, low, igh, inte)
struct UArrayD *matA,  *matZ;
long *low, *igh, *inte;   /* function only used by UADValVec */
/*------------------------------------------------------------------*/
{
 long i,j,mm,kl,mp,mp1,n,nm;
 double *a, *z;
 
 a  = matA->dataD;
 z  = matZ->dataD;
 n  = matA->nx[0];
 nm = matA->nx[1];

 for (i=1; i<=n; i++)
    {for (j=1; j<=n; j++)  z[(i-1)*nm+j-1] = 0.;
     z[(i-1)*nm+i-1] = 1.;
    }

 /*--- warning ---*/
 kl = *igh - *low - 1;
 if (kl >= 1)
    {for (mm=1; mm<=kl; mm++)
        {mp = *igh - mm;
         /*--- warning ---*/
         mp1 = mp + 1;

         for (i=mp1; i<=*igh; i++)  z[(i-1)*nm+mp-1] = a[(i-1)*nm+mp-1-1];
         i = inte[mp-1];
         if (i != mp)
            {for (j=mp; j<=*igh; j++)
                {z[(mp-1)*nm+j-1] = z[(i-1)*nm+j-1];
                 z[(i-1)*nm+j-1]  = 0.;
                }
             z[(i-1)*nm+mp-1] = 1.;
            }
        }
    }

 return;
}


/*------------------------------------------------------UADvvMulcpx2*/
void UADvvMulcpx2(a,b,c,d,x,y)
double a,b,c,d,*x,*y;   /* function only used by UADValVec */
/*------------------------------------------------------------------*/
{
 *x = (a*c + b*d) / (c*c + d*d);
 *y = (b*c - a*d) / (c*c + d*d);
 return;
}


/*-------------------------------------------------------UADvvBalbak*/
void UADvvBalbak(matZ, low, igh, scale, m)
struct UArrayD *matZ;
double *scale;
long *low, *igh, m;              /* function only used by UADValVec */
/*------------------------------------------------------------------*/
{
 double s, *z;
 long i,j,k,ii, n, nm;

 z  = matZ->dataD;
 n  = matZ->nx[0];
 nm = matZ->nx[1];

 if (*igh != *low)
    {for (i=*low; i<=*igh; i++)
        {s = scale[i-1];
         for (j=1; j<=m; j++)  z[(i-1)*nm+j-1] = z[(i-1)*nm+j-1] * s;
        }
    }
 for (ii=1; ii<=n; ii++)
    {i = ii;
     if (i < *low  ||  i > *igh)
        {if (i < *low) i = *low - ii;
         k = scale[i-1];
         if (k != i)
            {for (j=1; j<=m; j++)
                {s               = z[(i-1)*nm+j-1];
                 z[(i-1)*nm+j-1] = z[(k-1)*nm+j-1];
                 z[(k-1)*nm+j-1] = s;
                }
            }
        }
    }

return;
}


/*-------------------------------------------------------UADvvHqr2--*/
void UADvvHqr2(matA, matW, matZ, low, igh, ierr)
struct UArrayD *matA, *matZ;
struct UArrayCD *matW;
long *low, *igh, *ierr;          /* function only used by UADValVec */
/*------------------------------------------------------------------*/
{
 double p,q,r,s,t,w,x,y,ra,sa,vi,vr,zz,norm,machep;
 double t3[2], *h, *z, *ww;
 long notlas, rien, i,j,k,l,en,na,its,enm2,ll,mp2,mm,m,nn,ii,jj,n,nm;

 h  = matA->dataD;
 z  = matZ->dataD;
 ww = matW->dataD;
 n  = matA->nx[0];
 nm = matA->nx[1];

 machep = pow(2.,-50.);
 *ierr  = 0;

 for (i=1; i<=n; i++)
    {if (i < *low  ||  i > *igh)
        {ww[(i-1)*2]   = h[(i-1)*nm+i-1];
         ww[2*(i-1)+1] = 0.;
        }
    }
 en = *igh;
 t  = 0.;

 l60:
 if (en < *low)  goto l340;
 its  = 0;
 na   = en - 1;
 enm2 = na - 1;

 l70:
 for (ll=*low; ll<=en; ll++)
    {l = en + *low - ll;
     if (l == *low) break;
     if (fabs(h[(l-1)*nm+l-1-1]) <= machep * 
        (fabs(h[(l-1-1)*nm+l-1-1]) + fabs(h[(l-1)*nm+l-1]))) break;
    }
 x = h[(en-1)*nm+en-1];
 if (l == en)    goto l270;
 y = h[(na-1)*nm+na-1];
 w = h[(en-1)*nm+na-1] * h[(na-1)*nm+en-1];
 if (l == na)    goto l280;
 if (its == 30)  goto l1000;
 if (its != 10  &&  its != 20)  goto l130;
 t = t + x;
 for (i=*low; i<=en; i++)  h[(i-1)*nm+i-1] = h[(i-1)*nm+i-1] - x;
 s = fabs(h[(en-1)*nm+na-1]) + fabs(h[(na-1)*nm+enm2-1]);
 x = 0.75 * s;
 y = x;
 w = -.4375 * s * s;

 l130:
 its++;
 for (mm=l; mm<=enm2; mm++)
    {m  = enm2 + l - mm;
     zz = h[(m-1)*nm+m-1];
     r  = x - zz;
     s  = y - zz;
     p  = (r*s-w) / h[(m+1-1)*nm+m-1] + h[(m-1)*nm+m+1-1];
     q  = h[(m+1-1)*nm+m+1-1] - zz - r - s;
     r  = h[(m+2-1)*nm+m+1-1];
     s  = fabs(p) + fabs(q) + fabs(r);
     p  = p / s;
     q  = q / s;
     r  = r / s;
     if (m == l)  break;
     if (fabs(h[(m-1)*nm+m-1-1])*(fabs(q)+fabs(r)) <= machep * fabs(p) *
        (fabs(h[(m-1-1)*nm+m-1-1]) + fabs(zz)+fabs(h[(m+1-1)*nm+m+1-1]))) break;
    }

 mp2 = m + 2;
 for (i=mp2; i<=en; i++)
    {h[(i-1)*nm+i-2-1] = 0.;
     if (i != mp2)  h[(i-1)*nm+i-3-1] = 0.;
    }
 for (k=m; k<=na; k++)
    {if (k != na)  notlas = 1;
     else          notlas = 0;
     if (k == m)  goto l170;
     p = h[(k-1)*nm+k-1-1];
     q = h[(k+1-1)*nm+k-1-1];
     r = 0.;
     if (notlas == 1) r = h[(k+2-1)*nm+k-1-1];
     x = fabs(p) + fabs(q) + fabs(r);
     p = p / x;
     q = q / x;
     r = r / x;
     l170:
     s = sqrt(p*p+q*q+r*r);
     if (p >= 0) s = fabs(s);
     else       s = -fabs(s);
     if (k == m)  goto l180;
     h[(k-1)*nm+k-1-1] = -s * x;
     goto l190;
     l180:
     if (l != m)  h[(k-1)*nm+k-1-1] = -h[(k-1)*nm+k-1-1];
     l190:
     p  = p + s;
     x  = p / s;
     y  = q / s;
     zz = r / s;
     q  = q / p;
     r  = r / p;
     for (j=k; j<=n; j++)
        {p = h[(k-1)*nm+j-1] + q*h[(k+1-1)*nm+j-1];
         if (notlas == 1)
            {p = p + r*h[(k+2-1)*nm+j-1];
             h[(k+2-1)*nm+j-1] = h[(k+2-1)*nm+j-1] - p*zz;
            }
         h[(k+1-1)*nm+j-1] = h[(k+1-1)*nm+j-1] - p*y;
         h[(k-1)*nm+j-1]   = h[(k-1)*nm+j-1]   - p*x;
        }
     if (en < k+3) j = en;
     else          j = k + 3;

     for (i=1; i<=j; i++)
        {p = x * h[(i-1)*nm+k-1]  +  y * h[(i-1)*nm+k+1-1];
         if (notlas == 1)
            {p = p + zz*h[(i-1)*nm+k+2-1];
             h[(i-1)*nm+k+2-1] = h[(i-1)*nm+k+2-1] - p*r;
            }
         h[(i-1)*nm+k+1-1] = h[(i-1)*nm+k+1-1] - p*q;
         h[(i-1)*nm+k-1]   = h[(i-1)*nm+k-1]   - p;
        }
     for (i=*low; i<=*igh; i++)
        {p = x * z[(i-1)*nm+k-1]  +  y * z[(i-1)*nm+k+1-1];
         if (notlas == 1)
            {p = p + zz*z[(i-1)*nm+k+2-1];
             z[(i-1)*nm+k+2-1] = z[(i-1)*nm+k+2-1] - p*r;
            }
         z[(i-1)*nm+k+1-1] = z[(i-1)*nm+k+1-1] - p*q;
         z[(i-1)*nm+k-1]   = z[(i-1)*nm+k-1]   - p;
        }
    }
 goto l70;

 l270:
 h[(en-1)*nm+en-1] = x + t;
 ww[2*(en-1)]      = h[(en-1)*nm+en-1];
 ww[2*(en-1)+1]    = 0.;
 en = na;
 goto l60;

 l280:
 p                 = (y-x) / 2.;
 q                 = p*p + w;
 zz                = sqrt(fabs(q));
 h[(en-1)*nm+en-1] = x + t;
 x                 = h[(en-1)*nm+en-1];
 h[(na-1)*nm+na-1] = y + t;
 if (q < 0.)  goto l320;
 if (p >= 0)  zz = p + fabs(zz);
 else         zz = p - fabs(zz);
 ww[2*(na-1)]   = x + zz;
 ww[2*(en-1)]   = ww[2*(na-1)];
 if (zz != 0.)  ww[2*(en-1)] = x - w/zz;
 ww[2*(na-1)+1] = 0.;
 ww[2*(en-1)+1] = 0.;
 x              = h[(en-1)*nm+na-1];
 r              = sqrt(x*x+zz*zz);
 p              = x  / r;
 q              = zz / r;

 for (j=na; j<=n; j++)
    {zz               = h[(na-1)*nm+j-1];
     h[(na-1)*nm+j-1] = q*zz + p*h[(en-1)*nm+j-1];
     h[(en-1)*nm+j-1] = q*h[(en-1)*nm+j-1] - p*zz;
    }
 for (i=1; i<=en; i++)
    {zz               = h[(i-1)*nm+na-1];
     h[(i-1)*nm+na-1] = q*zz + p*h[(i-1)*nm+en-1];
     h[(i-1)*nm+en-1] = q*h[(i-1)*nm+en-1] - p*zz;
    }
 for (i=*low; i<=*igh; i++)
    {zz               = z[(i-1)*nm+na-1];
     z[(i-1)*nm+na-1] = q*zz + p*z[(i-1)*nm+en-1];
     z[(i-1)*nm+en-1] = q*z[(i-1)*nm+en-1] - p*zz;
    }
 goto l330;

 l320:
 ww[2*(na-1)]   = x + p;
 ww[2*(en-1)]   = x + p;
 ww[2*(na-1)+1] = zz;
 ww[2*(en-1)+1] =-zz;

 l330:
 en = enm2;
 goto l60;

 l340:
 norm = 0.;
 k    = 1;
 for (i=1; i<=n; i++)
    {for (j=k; j<=n; j++)  norm += fabs(h[(i-1)*nm+j-1]);
     k = i;
    }
 if (norm == 0.)  goto l1001;
 for (nn=1; nn<=n; nn++)
    {en = n + 1 - nn;
     p = ww[2*(en-1)];
     q = ww[2*(en-1)+1];
     na = en - 1;
     if (q > 0.)  goto l800;
     if (q == 0.) goto l600;
     if (q < 0.)  goto l710;

     l600:
     m = en;
     h[(en-1)*nm+en-1] = 1.;
     if (na == 0)  goto l800;
     for (ii=1; ii<=na; ii++)
        {i = en - ii;
         w = h[(i-1)*nm+i-1] - p;
         r = h[(i-1)*nm+en-1];
         if (m <= na)
            {for (j=m; j<=na; j++) r = r + h[(i-1)*nm+j-1]*h[(j-1)*nm+en-1]; }
         if (ww[2*(i-1)+1] >= 0.)  goto l630;
         zz = w;
         s  = r;
         goto l700;
         l630:
         m = i;
         if (ww[2*(i-1)+1] != 0.)  goto l640;
         t = w;
         if (w == 0.)  t = machep * norm;
         h[(i-1)*nm+en-1] = -r/t;
         goto l700;
         l640:
         x = h[(i-1)*nm+i+1-1];
         y = h[(i+1-1)*nm+i-1];
         q = (ww[2*(i-1)]-p) * (ww[2*(i-1)]-p)  +  ww[2*(i-1)+1] * ww[2*(i-1)+1];
         t = (x*s-zz*r) / q;
         h[(i-1)*nm+en-1] = t;
         if (fabs(x) <= fabs(zz))  goto l650;
         h[(i+1-1)*nm+en-1] = (-r-w*t) / x;
         goto l700;
         l650:
         h[(i+1-1)*nm+en-1] = (-s-y*t) / zz;
         l700:
         rien = rien;
        }
     goto l800;
     l710:
     m = na;
     if (fabs(h[(en-1)*nm+na-1]) <= fabs(h[(na-1)*nm+en-1])) goto l720;
     h[(na-1)*nm+na-1] = q / h[(en-1)*nm+na-1];
     h[(na-1)*nm+en-1] = -(h[(en-1)*nm+en-1]-p) / h[(en-1)*nm+na-1];
     goto l730;
     l720:
     UADvvMulcpx2(0.,-h[(na-1)*nm+en-1],h[(na-1)*nm+na-1]-p,q,&t3[0],&t3[1]);
     h[(na-1)*nm+na-1] = t3[0];
     h[(na-1)*nm+en-1] = t3[1];
     l730:
     h[(en-1)*nm+na-1] = 0.;
     h[(en-1)*nm+en-1] = 1.;
     enm2 = na - 1;
     if (enm2 == 0.)  goto l800;
     for (ii=1; ii<=enm2; ii++)
        {i = na - ii;
         w  = h[(i-1)*nm+i-1]-p;
         ra = 0.;
         sa = h[(i-1)*nm+en-1];
         for (j=m; j<=na; j++)
            {ra = ra + h[(i-1)*nm+j-1]*h[(j-1)*nm+na-1];
             sa = sa + h[(i-1)*nm+j-1]*h[(j-1)*nm+en-1];
            }
         if (ww[2*(i-1)+1] >= 0.)  goto l770;
         r = sa;
         s = sa;
         goto l790;
         l770:
         m = i;
         if (ww[2*(i-1)+1] != 0.) goto l780;
         UADvvMulcpx2(-ra,-sa,w,q,&t3[0],&t3[1]);
         h[(i-1)*nm+na-1] = t3[0];
         h[(i-1)*nm+en-1] = t3[1];
         goto l790;
         l780:
         x  = h[(i-1)*nm+i+1-1];
         y  = h[(i+1-1)*nm+i-1];
         vr = (ww[2*(i-1)]-p)*(ww[2*(i-1)]-p) + ww[2*(i-1)+1]*ww[2*(i-1)+1] - q*q;
         vi = (ww[2*(i-1)]-p) * 2. * q;
         if (vr == 0.  &&  vi == 0.)
            {vr=machep*norm*(fabs(w)+fabs(q)+fabs(x)+fabs(y)+fabs(zz));}
         UADvvMulcpx2(x*r-zz*ra+q*sa,x*s-zz*sa-q*ra,vr,vi,&t3[0],&t3[1]);
         h[(i-1)*nm+na-1] = t3[0];
         h[(i-1)*nm+en-1] = t3[1];
         if (fabs(x) <= fabs(zz)+fabs(q))  goto l785;
         h[(i+1-1)*nm+na-1] = (-ra-w*h[(i-1)*nm+na-1]+q*h[(i-1)*nm+en-1]) / x;
         h[(i+1-1)*nm+en-1] = (-sa-w*h[(i-1)*nm+en-1]-q*h[(i-1)*nm+na-1]) / x;
         goto l790;
         l785:
         UADvvMulcpx2(-r-y*h[(i-1)*nm+na-1],-s-y*h[(i-1)*nm+en-1],zz,q,&t3[0],&t3[1]);
         h[(i+1-1)*nm+na-1] = t3[0];
         h[(i+1-1)*nm+en-1] = t3[1];
         l790:
         rien = rien;
        }
     l800:
     rien = rien;
    }

 for (i=1; i<=n; i++)
    {if (i < *low  ||  i > *igh)
        {for (j=1; j<=n; j++)  z[(i-1)*nm+j-1] = h[(i-1)*nm+j-1];}
    }
 for (jj=*low; jj<=n; jj++)
    {j = n + *low - jj;
     if (j < *igh)  m = j;
     else           m = *igh;
     for (i=*low; i<=*igh; i++)
        {zz = 0.;
         for (k=*low; k<=m; k++)  zz += z[(i-1)*nm+k-1]*h[(k-1)*nm+j-1];
         z[(i-1)*nm+j-1] = zz;
        }
    }
 goto l1001;
 l1000:
 *ierr = en;
 l1001:
 rien = rien;

 return;
}


/*----------------------------------------------------------UADExpm --*/
void UADExpm(matIn, matOut)
struct UArrayD *matIn, *matOut;
/*                     compute exponential of a real matrix           */
/*   matIn is destroyed by expm                                       */
/*--------------------------------------------------------------------*/
{
 double maxx,maxmax,e,c,pp;
 long i,j,k,q,p,s;
 int exp;
 struct UArrayD *XA,*X,*X2,*D,*cX;

 if (matIn->nData != matOut->nData)  UJError(3,"UADExpm","nData missmatch 1");
 if (matIn->nx[0] != matIn->nx[1])   UJError(3,"UADExpm","nData missmatch 2");

 maxmax = 0.;
 for (i=0; i<matIn->nx[1]; i++)
    {maxx=0.;
     for (j=0;j<matIn->nx[0];j++)
        {maxx = maxx + fabs(matIn->dataD[j*matIn->nx[1]+i]);}
     if( maxx > maxmax)  maxmax = maxx;
    }

 e = frexp(maxmax,&exp);
 s = 0;
 if (exp > -1)  s = exp + 1;

 XA = (struct UArrayD*) UADNew(matIn->nx[0],matIn->nData/matIn->nx[0]);
 UADCopy(matIn, XA);
 pp = pow(2.,(double)s);
 for (i=0; i<XA->nData; i++)  XA->dataD[i] = XA->dataD[i] / pp;

 /*--- Pade approximation for exp(A) ---*/
 X = (struct UArrayD*) UADNew(matIn->nx[0],matIn->nData/matIn->nx[0]);
 UADCopy(XA, X);
 c  = 0.5;
 X2 = (struct UArrayD*) UADNew(matIn->nx[0],matIn->nData/matIn->nx[0]);
 for (i=0; i<XA->nx[1]; i++)
    {for (j=0; j<matIn->nx[0]; j++)
        {matOut->dataD[j*matIn->nx[1]+i] = 0;
         if (i == j)  matOut->dataD[j*matIn->nx[1]+j] = 1.;
        }
    }

 UADScal(XA, c, X2);      
 UADAdd(X2,matOut,matOut);
 D = (struct UArrayD*) UADNew(matIn->nx[0],matIn->nData/matIn->nx[0]);
 for (i=0; i<XA->nx[1]; i++)
    {for (j=0;j<matIn->nx[0];j++)
        {D->dataD[j*matIn->nx[1]+i] = 0;
         if (i == j)  D->dataD[j*matIn->nx[1]+j] = 1.;
        }
    }

 UADSub(D,X2,D);
 q  = 6;
 p  = 1;
 cX = (struct UArrayD*) UADNew(matIn->nx[0],matIn->nData/matIn->nx[0]);
 for (k=2; k<=q; k++)
    {c = c * (q-k+1) / (k*(2*q-k+1));
     UADProd(XA, X, X2);
     UADCopy(X2,X);
     UADScal(X,c,cX);
     UADAdd(matOut,cX,matOut);
     if (p != 0)  UADAdd(D,cX,D);
     else         UADSub(D,cX,D);
     if (p == 0)  p = 1;
     else         p = 0;
    }

 UADInv(D,X2);
 UADProd(X2,matOut,D);
 UADCopy(D,matOut);

 /*--- Undo scaling by repeated squaring ---*/
 for (k=1; k<=s; k++)
    {UADProd(matOut,D,X2);
     UADCopy(X2,D);
     UADCopy(X2,matOut);
    }

 UADFree(X2);
 UADFree(X);
 UADFree(D);
 UADFree(cX);
 UADFree(XA);
 return;

}


/*----------------------------------------------------- UAfillDRead ----*/
void UAfillDRead(iFile)
struct UJfile *iFile;
/*----------------------------------------------------------------------*/
{
 struct UArrayFillD *fill;

 UJ_READ_HEADER_WITH_NAME(UArrayFillD,fill)
 UJ_READ_LONG(fill->iClock);
 UJ_READ_LONG(fill->nData);
 UJ_READ_STRUCT(USignal,fill->input);

 strcpy(bankName,fill->name);
 strcat(bankName,".out");
 fill->vect = UADNew(1,fill->nData);
 free(fill->vect->name);
 fill->vect->name = (char *) calloc(strlen(bankName)+1,sizeof(char));
 sprintf(fill->vect->name,"%s",bankName);
 UJSetBA(fill->vect->name,"UArrayD",fill->vect);

 UJCObAdd(fill->iClock,fill,UArrayFillDUJstructH,fill->name);

 return;
}


/*------------------------------------------------------ UAfillDSim ----*/
void UAfillDSim(fill,time)
struct UArrayFillD *fill;
long time;
/*----------------------------------------------------------------------*/
{
 fill->vect->dataD[time%fill->nData] = (double) fill->input->value;
 return;
}


/*----------------------------------------------------- UAfillFRead ----*/
void UAfillFRead(iFile)
struct UJfile *iFile;
/*----------------------------------------------------------------------*/
{
 struct UArrayFillF *fill;

 UJ_READ_HEADER_WITH_NAME(UArrayFillF,fill)
 UJ_READ_LONG(fill->iClock);
 UJ_READ_LONG(fill->nData);
 UJ_READ_STRUCT(USignal,fill->input);

 strcpy(bankName,fill->name);
 strcat(bankName,".out");
 fill->vect = UAFNew(1,fill->nData);
 free(fill->vect->name);
 fill->vect->name = (char *) calloc(strlen(bankName)+1,sizeof(char));
 sprintf(fill->vect->name,"%s",bankName);
 UJSetBA(fill->vect->name,"UArrayF",fill->vect);

 UJCObAdd(fill->iClock,fill,UArrayFillFUJstructH,fill->name);

 return;
}


/*------------------------------------------------------ UAfillFSim ----*/
void UAfillFSim(fill,time)
struct UArrayFillF *fill;
long time;
/*----------------------------------------------------------------------*/
{
 fill->vect->dataF[time%fill->nData] = (float) fill->input->value;
 return;
}

