/* *******************************************************************
********** UFrameL.h ************************** UFrameL.h ************
********** UFrameL.h ************************** UFrameL.h ************
******************************************************************* */
/* by B.Mours, D.Verkindt LAPP (Annecy); April 06, 1998 */

#if !defined(UFRAMELIB_DEFINED)
#define UFRAMELIB_DEFINED DONE

#include <time.h>
#include <string.h>
/*
#include "UMathL.h"
*/
#include "UJobL.h"
#include "USignalL.h"
#include "dataDisplay.h"

#ifndef NOCM
#include "CmMessage.h"
#endif

#define UFr_MAX_ID   10000
#define LBUFF_IN     540000


struct UFrVectOp {           /* Describes operations on vectors         */
  long  iClock;              /* clock number                            */
  long  id;                  /* algorithm's object id                   */
  char *className;           /* algorithm's class name                  */
  char *inputVect;           /* list of input vectors                   */
  char *inputPar;            /* list of input parameters                */
  struct FrameH *frameH;     /* root pointer for the frame              */
                             /* -------------- end_of_SIO parameters ---*/
  char  *name;               /* algorithm's object name                 */
  int   realUnits;           /* flag: >0 if real units of vectors used  */
  int   nvect;               /* number of input vectors                 */
  int   nData;               /* max of input vectors nData              */
  double inputParam[64];     /* parameters for each vectors             */
  char  *inputName[64];      /* names of input vectors                  */
  char  *outName;            /* names of output vector                  */
  double lastValue;          /* last output value (end of each frame)   */
};


struct UFrShm {                 /* Describes input shared memory           */
  char  *name;                  /* name                                    */
  long  iClock;                 /* clock number                            */
  char *producerName;           /* shared memory producer name             */
                                /* -------------- end_of_SIO parameters ---*/
  int    lastFrameN;            /* frame number of previous frame read     */
  long   iread;                 /* read index of shared memory             */
  struct ShmInfo  *shm;         /* pointer to the shared memory            */
  struct FrameH *frameH;        /* the name of this frameH struct is 
                                   the name of the UFrIFile struct.        */
};


struct UFrWave {                /* Describes wave input file               */
  char  *name;                  /* name                                    */
  long  iClock;                 /* clock number                            */
  char *fileName;               /* shared memory producer name             */
                                /* -------------- end_of_SIO parameters ---*/
  FILE   *file;                 /* pointer to wave input file stream       */
  int    vectnData;             /* number of sample per vector in frame    */
  int    i;                     /* counter of samples read in wave file    */
  int    nBytes_per_frame;      /* number of data bytes per frame          */
  double framedt;               /* length of the frames created            */
  double gtime;                 /* GPS time of the frames created          */
  long   nData;                 /* total number of sample in wave file     */
  struct FrameH *frameH;        /* the name of this frameH struct is 
                                   the name of the UFrWave struct.         */
};


struct UFrIFile {               /* Describes input file                    */
  char  *name;                  /* name                                    */
  long  iClock;                 /* clock number                            */
  char *fileName;               /* input file name                         */
  char *itag;                   /* input tag (channels list)               */
                                /* -------------- end_of_SIO parameters ---*/
  double gtime;                 /* GPS time of the current frame read      */
  double framedt;               /* length (sec) of the current frame read  */
  long   status;                /* gives file status info(see flag bellow) */
  long   iRead;                 /* number of UFrame read for this file     */
  long   nFiles;                /* number of input files                   */
  long   iFiles;                /* current input files index               */
  char   *fSubName[UFr_MAX_ID]; /* array of files names                    */
  double dt;                    /* clock dt                                */
  struct FrFile *file;
  struct FrameH *frameH;        /* the name of this frameH struct is 
                                   the name of the UFrIFile struct.        */
  int    lastFrameN;            /* frame number of previous frame read     */
  struct USignal *run;          /* Frame number                            */
  struct USignal *frame;        /* Frame number                            */
  struct USignal *length;       /* Frame number                            */
  struct USignal *time;         /* Frame number                            */
  struct USignal *timeofrun;    /* Frame number                            */
  struct UFrDetector *detectSim;/*detector used for simulation             */
  struct UFrDetector *detectRec;/*detector used for reconstruction         */
  BOOL endOfFile;               /* YES means we have reached the end of file */
  FILE *trigAsciiFile;
};


struct UFrIFileCm {          /* Describes input Cm                      */
  char  *name;               /* name                                    */
  long iClock;               /* clock number                            */
  long waitMax;              /* waiting time to get Cm message          */
  char *CmName;              /* local Cm Name                           */
  char *tag;                 /* tag to select channels of the frame     */
                             /* -------------- end_of_SIO parameters ---*/
  long   status;             /* gives file status info(see flag bellow) */
  long   iFrame;             /* current frame number                    */
  double dt;                 /* clock dt                                */
  struct FrFile *file;
  struct FrameH *frameH;     /* the name of this frameH struct is 
                                the name of the UFrIFile struct.          */
  int    lastFrameN;         /* frame number of previous frame read       */
  int    previousSize;       /* size (bytes) of previous frame received   */
  struct USignal *run;       /* Frame number                              */
  struct USignal *frame;     /* Frame number                              */
  struct USignal *length;    /* Frame number                              */
  struct USignal *time;      /* Frame number                              */
  struct USignal *timeofrun; /* Frame number                              */
  BOOL endOfFile;            /* YES means we have reached the end of file */
}; 


struct UFrIFileOnline {         /* Describes input file                    */
  char  *name;                  /* name                                    */
  long iClock;                  /* clock number                            */
  char *directoryName;          /* input directory name                    */
  long waitMax;                 /* waiting time to get next frame's file   */
                                /* has to be integer number                */
  long startTime;               /* first frame number                      */
  char *itag;                   /* input tag (channels list)               */
                                /* -------------- end_of_SIO parameters ---*/
  double gtime;                 /* GPS time of the current frame read      */
  double framedt;               /* length (sec) of the current frame read  */
  long   frames_per_file;       /* number of frames per input file         */
  long   status;                /* gives file status info(see flag bellow) */
  long   iFirst;                /* flag>0 if pass in OnlineSim first time  */
  long   iRead;                 /* number of UFrame read for this file     */
  long   iFrame;                /* current frame number                    */
  long   nFiles;                /* number of input files                   */
  long   iFiles;                /* current input files index               */
  char   *fSubName[UFr_MAX_ID]; /* array of files names                    */
  char   *fname;                /* first part of the generic file name     */
  char   *suffix;               /* second part of the generic file name    */
  double dt;                    /* clock dt                                */
  struct FrFile *file;
  struct FrameH *frameH;         /* the name of this frameH struct is 
                                    the name of the UFrIFile struct.          */
  int    lastFrameN;             /* frame number of previous frame read       */
  struct USignal *run;           /* Frame number                              */
  struct USignal *frame;         /* Frame number                              */
  struct USignal *length;        /* Frame number                              */
  struct USignal *time;          /* Frame number                              */
  struct USignal *timeofrun;     /* Frame number                              */
  struct UFrDetector *detectSim; /* detector used for simulation              */
  struct UFrDetector *detectRec; /* detector used for reconstruction          */
  BOOL endOfFile;                /* YES means we have reached the end of file */
}; 


struct UFrUnpack{           /* unpack frame data into signal struct  */
  char  *name;              /* name                                  */
  long   iClock;            /* clock number                          */
  char   *paramName;        /* parameter name like: sms.TOBS.G31
                               for the sms TOBS, parameter G31       */
  struct FrameH *frameH;    /* root pointer for the frame            */
                            /* ------ end_of_SIO parameters ---------*/
  double dt;                /* sampling UFrequency                   */
  double previousValue;     /* signal ampl. at previous sampling     */
  double smsPeriod;         /* positive if sms period is greater than frame period */
  struct USignal *out;      /* output signal                         */
  struct FrVect *data;      /* pointer on current data vector read   */
  int frameNold;            /* frame nbr of the previous frame read  */
  int GPSold;               /* GPS time of the previous frame read   */
  int GPSNold;               /* GPS time of the previous frame read   */
  long  type;               /* type is one of the followin UFRSMS,...*/
  char  *smsName;           /* sms name                              */
  char  *param;             /* short parameter name                  */
  long   i;                 /* indice in data vector                 */
  long   nError;            /* flag to reduce the prints             */
  long   nError2;           /* flag to reduce the prints             */
  int valid;                /* this flag is zero if vector data are valid */
};


#define UFRSMS 0
#define UFRADC 1
#define UFRREC 2
#define UFRSIM 3

void UJLibIni2 (char *jobName, long arg, char **argv);
void UFrLibIni(void);

void UFrWaveIni(struct UFrWave *wave, double dt);
void UFrWaveRead(struct UJfile *iFile);
void UFrWaveSim(struct UFrWave *wave, long time);
void UFrWaveEnd(struct UFrWave *wave);

void UFrVectOpEnd (struct UFrVectOp *vectop);
void UFrVectOpRead (struct UJfile *iFile);
void UFrVectOpIni (struct UFrVectOp *vectop, double dt);
void UFrVectOpSim (struct UFrVectOp *vectop, long time);

void UFrShmIni(struct UFrShm *oShm, double dt);
void UFrShmRead(struct UJfile *iFile);
void UFrShmSim(struct UFrShm *oShm, long time);
void UFrShmEnd(struct UFrShm *oShm);

void UFrIFileIni(struct UFrIFile *iFile, double dt);
void UFrIFileRead(struct UJfile *iFile);
double DyFrFileIGetNextEventTime(FrFile *iFile,char *name,double tStart,double aMin);
void UFrIFileSim(struct UFrIFile *iFile, long time);
void UFrIFileEnd(struct UFrIFile *iFile);

void UFrIFileCmIni(struct UFrIFileCm *iFile, double dt);
void UFrIFileCmRead(struct UJfile *iFile);
void UFrIFileCmSim(struct UFrIFileCm *iFile, long time);
void UFrIFileCmEnd();

void UFrIFileOnlineIni(struct UFrIFileOnline *iFile, double dt);
void UFrIFileOnlineRead(struct UJfile *iFile);
void UFrIFileOnlineSim(struct UFrIFileOnline *iFile, long time);

void UFrUnpackIni(struct UFrUnpack *UFrUnpack, double dt);
struct UFrUnpack *UFrUnpackNew(char *name, double dt, struct FrameH *fh);
void UFrUnpackRead(struct UJfile *iFile);
void UFrUnpackSim(struct UFrUnpack *UFrUnpack, long time);
void UFrUnpackEnd(struct UFrUnpack *UFrUnpack);

extern struct UJstructH *FrameHUJstructH;
extern struct UJstructH *UFrIFileUJstructH;
extern struct UJstructH *UFrWaveUJstructH;
extern struct UJstructH *UFrShmUJstructH;
extern struct UJstructH *UFrIFileOnlineUJstructH;
extern struct UJstructH *UFrUnpackUJstructH;
extern struct UJstructH *UFrVectOpUJstructH;

#ifndef NO_CM
extern struct UJstructH *UFrIFileCmUJstructH;
#endif

extern double   	Time0;
extern int              lastFrameN;

#endif


