/* ***************************************************************
************* UMath.c *************************** UMath.c ********
************* UMath.c *************************** UMath.c ********
*************************************************************** */
/* LAPP (Annecy) */
/* Modifications by Cavalier Fabien LAL (Orsay) */
 
char UMversion[] = "October 24, 1995";
 
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "UMathL.h"
#ifdef MEMWATCH
#    include "memwatch.h"
#endif

/*------------------------------------------UMpHermite----------*/
double UMpHermite(n,x)
long n;
double x;
/*--------------------------------------------------------------*/
{ 
 switch (n)
    {
     case  0:  return (1.); break;
     case  1:  return (2.*x); break; 
     case  2:  return 4.*x*x-2.; break;
     default:  return(2.*x*UMpHermite(n-1,x)-
                          2.*(n-1)*UMpHermite(n-2,x)); break;
    }           
}


/*------------------------------------------ UMrandom -----*/
/* random : fonction donnant un nombre pseudo_aleatoire    */
/*          entre 0 et 1                                   */
/* Cette fonction est basee sur le generateur de lehmer.   */
/* (Voir l'article de Stephen K. Park and Keith W Miller   */
/* dans Cmmunications of the ACM October 1988 Volume 31    */
/* Number 10)                                              */
/* En particulier on est assure d'obtenir une periode      */
/* "complete" sur une machine capable de representer les   */
/* entiers compris entre -2**31 et 2**31 - 1               */
/* Si l'argument d'entree est 0, la racine ("seed") sera   */
/* la derniere racine calculee (la premiere fois la racine */
/* vaut 2**31 - 2).                                        */
/* Si on veut changer de racine l'argument doit etre un    */
/* nombre compris entre 1 et 2**31 - 2.                    */
/* Author: A. Pellier      17/01/92                        */
/* Correction by: X. Grave 13/04/95                        */
/*-------------------------------------------------------- */
float UMrandom(i)
long i;
{
 long a = 16807;
 long m = 2147483647;
 long q = 127773;
 long p = 2836;
 static long seed = 2147483646;
 long hi, lo;
 
 if (i != 0)  seed = i;

 hi   = seed / q;
 lo   = seed - hi * q;
 seed = a*lo - p*hi;
 if (seed <= 0) seed = seed + m;

 return ((float)seed / (float)m);
}
 
/*----------------------------------------------------- UMrgausd -----*/
/* rangausd generates a pseudo random number (double) with normal     */
/* (Gaussian) distribution with zero mean and unit variance.          */
/* Author: B. Mours        27/02/91                                   */
/* Correction by: X. Grave 13/04/95                                   */
/*--------------------------------------------------------------------*/
double UMrgausd(i)
long i;
{
 static long iset=0;
 static double gset;
 double fac,r,x,y;
 long a = 16807;
 long m = 2147483647;
 long q = 127773;
 long p = 2836;
 static long seed = 2147483646;
 long hi, lo;

 if (i != 0)  seed = i;
 
 if (iset == 1 )
    {iset = 0;
     return(gset);
    }
 
 iset = 1;
 do {
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0) seed = seed + m;
     x    = -1. + 2. * (double)seed / (double)m;
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0) seed = seed + m;
     y    = -1. + 2. * (double)seed / (double)m;
     r    = x*x + y*y;
    }
 while (r >= 1.0);
 
 if (r == 0.0)  r = 0.000001;
 fac  = sqrt(-2.*log(r)/r);
 gset = x * fac;

 return(y*fac);
}


/*----------------------------------------------------- UMrgausf -----*/
/* rangausf generates a pseudo random number (float) with normal      */
/* (Gaussian) distribution with zero mean and unit variance.          */
/* Author: B. Mours        27/02/91                                   */
/* Correction by: X. Grave 13/04/95                                   */
/*--------------------------------------------------------------------*/
float UMrgausf(i)
long i;
{
 static long iset=0;
 static float gset;
 float fac,r,x,y;
 long a = 16807;
 long m = 2147483647;
 long q = 127773;
 long p = 2836;
 static long seed = 2147483646;
 double hi, lo;
 
 if (i != 0)  seed = i;
 
 if (iset == 1 )
    {iset = 0;
     return(gset);
    }
 
 iset = 1;
 do {
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0)  seed = seed + m;
     x    = -1. + 2. * (double)seed / (double)m;
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0)  seed = seed + m;
     y    = -1. + 2. * (double)seed / (double)m;
     r    = x*x + y*y;
    }
 while (r >= 1.0);
 
 if (r == 0.0)  r = 0.000001;
 fac  = sqrt(-2.*log(r)/r);
 gset = x * fac;

 return(y*fac);
}


/*-------------------------------------------------------- UMfit -----*/
double UMminimize(xmesu,ymesu,yerr,fctfit,deriv,param,active,errmatx)
struct UArrayD *xmesu;
struct UArrayD *ymesu;
struct UArrayD *yerr;
double (*fctfit)();
void (*deriv)();
struct UArrayD *param;
struct UArrayD *active;
struct UArrayD *errmatx;
/* fitting routine                                                    */
/* Minimisation of chi2                                               */
/* Entries :                                                          */
/* xmesured, ymesured = vector of data points                         */
/* yerrors = vector of erros on the data points                       */
/* fctfit = theoretical function to fit on data points                */
/* fctfit should be a function of the following type                  */
/*   double fctfit(double x, struct UArrayD *param)                    */
/* deriv = derivative of theoretical function to fit on data points   */
/* deriv should be a function of the following type                   */
/*   void deriv(double x, struct UArrayD *param, struct UArrayD *dyda)  */
/* in deriv, you must fill all the derivative of the theoretical      */
/* function with respect to the fitted parameters                     */
/* param = vector of initial values of the parameters                 */
/* Results :                                                          */
/* param = vector of the fitted parameters                            */
/* acitve = vector telling which parameters have to be fitted         */
/* (0 = not fitted)                                                   */
/* errmatx = covariance error matrix of the parameters                */
/* Return the value of chi2 if the fit is OK else return -999         */
/*                                                                    */
/* Algorithm taken in :                                               */
/* Numerical Recipes : the art of scientific computing                */
/* (FORTRAN version), page 526.                                       */
/* Adapted for C by Cavalier Fabien LAL (Orsay) February 16, 1995     */
/*--------------------------------------------------------------------*/
{
 struct UMmesu *mesu;
 struct UMmini *mini;
 struct UArrayD *covar;
 struct UArrayD *invcovar;
 struct UArrayD *deltaa;
 double chi2_min,lambda,deter;
 long i,j,size_a,ntry,test;

 /*--- Book memory space ---*/

 mesu = (struct UMmesu *) malloc(sizeof(struct UMmesu));
 mesu->xmesured   = xmesu;
 mesu->ymesured   = ymesu;
 mesu->yerrors    = yerr;
 mesu->ypredicted = fctfit;
 mesu->derivative = deriv;
 mesu->areturned  = param;
 size_a           = 0;
 for (i=0; i<active->nData; i++)
    {if (active->dataD[i] != 0) size_a++;}
 if (size_a == 0)
    {printf("All the parameters are constant \n");
     return(-999);
    }
 mesu->aestimated = UADNew(1,size_a);
 mesu->acovar     = UADNew(size_a,size_a);
 covar            = UADNew(size_a,size_a);
 invcovar         = UADNew(size_a,size_a);
 deltaa           = UADNew(size_a,1);
 mini             = (struct UMmini *) malloc(sizeof(struct UMmini));
 mini->alpha      = UADNew(size_a,size_a);
 mini->beta       = UADNew(size_a,1);
 mini->atry       = UADNew(mesu->areturned->nData,1);

 /*--- Some Initialisations ---*/

 lambda = 1.e-3;
 test   = 0;
 ntry   = 0;
 j      = 0;

 for (i=0; i<mini->atry->nData; i++)
    {if (active->dataD[i] != 0) 
        {mesu->aestimated->dataD[j] = mesu->areturned->dataD[i];
         j ++;
        }
     mini->atry->dataD[i] = mesu->areturned->dataD[i];  
    }

 UMchi2(mesu,mini);
 UMhessian(mesu,mini);
 chi2_min = mini->chi2;

 /*--- Test of convergence and loop for minimisation ---*/

 while (test == 0  &&  ntry < MAXTRY)
    {
     ntry ++;
     /*--- Computatiom of the matrix covar ---*/
     UADCopy(mini->alpha,covar);
     for (i=0; i<size_a; i++)
        {covar->dataD[i*(size_a+1)] = covar->dataD[i*(size_a+1)] + lambda;}
     /*--- Solve equation : [covar]*[deltaa]=[beta] ---*/
     deter = UADInv(covar,invcovar);
     UADProd(invcovar,mini->beta,deltaa);
     /*--- New tested values of the parameters ---*/
     j = 0;
     for (i=0; i<mini->atry->nData; i++)
        {if (active->dataD[i] != 0) 
            {mini->atry->dataD[i] = mesu->aestimated->dataD[j] + deltaa->dataD[j];
             j ++;
            }
        }
     UMchi2(mesu,mini);
     /*--- Chi2 test ---*/
     if (mini->chi2 <= chi2_min) 
        {if (fabs(mini->chi2-chi2_min) < 1.e-7) test = 1;
         chi2_min = mini->chi2;
         lambda   = lambda / 10;
         UMhessian(mesu,mini);
         /*--- New values of the parameters ---*/
         j = 0;
         for (i=0; i<mini->atry->nData; i++)
            {if (active->dataD[i] != 0) 
                {mesu->areturned->dataD[i]  = mini->atry->dataD[i];
                 mesu->aestimated->dataD[j] = mini->atry->dataD[i];
                 j ++;
                }
            }
        }
     else
        {lambda = lambda * 10.;
         if (lambda > 1.e70) return(-chi2_min);
        }
    }

 /*--- End of While loop ---*/

 if (ntry == MAXTRY)  return(-999.);

 /*--- Computation of the covariance matrix of errors ---*/
/*
 deter = UADInv(mini->alpha,mesu->acovar);
*/

 UADFree (covar);
 UADFree (invcovar);
 UADFree (deltaa);
 UADFree (mini->alpha);
 UADFree (mini->beta);
 UADFree (mini->atry);
 free(mini);
 UADFree (mesu->aestimated);
 UADFree (mesu->acovar); 
 free(mesu);

 return(chi2_min);
}


/*------------------------------------------------------- UMchi2 -----*/
void UMchi2(mesu,mini)
struct UMmesu *mesu;
struct UMmini *mini;
/* Calculation of chi2 ( use by UMminimize )                          */
/* Author : Cavalier Fabien LAL (Orsay) February 16, 1995             */ 
/*--------------------------------------------------------------------*/
{
 struct UArrayD *param;
 double dy,sig2;
 long i;

 param      = UADNew(1,mini->atry->nData);
 mini->chi2 = 0.;
 for (i=0; i<mini->atry->nData; i++)  param->dataD[i] = mini->atry->dataD[i];

 /*--- Loop over ymesured ---*/

 for (i=0; i<mesu->ymesured->nData; i++)
    {dy         = mesu->ymesured->dataD[i] - mesu->ypredicted(mesu->xmesured->dataD[i],param);
     sig2       = 1./ (mesu->yerrors->dataD[i] * mesu->yerrors->dataD[i]);
     mini->chi2 = mini->chi2 + dy*dy*sig2;
    }

 UADFree(param);
 return;
}


/*------------------------------------------------------ UMhessian -----*/
void UMhessian(mesu,mini)
struct UMmesu *mesu;
struct UMmini *mini;
/* Calculation of hessian matrix ( use by UMminimize )                  */
/* Author : Cavalier Fabien LAL (Orsay) February 16, 1995             */ 
/*--------------------------------------------------------------------*/
{
 struct UArrayD *dyda,*param;
 double dy,wt,sig2;
 long i,j,k,indis,indis2;

 dyda  = UADNew(1,mesu->aestimated->nData);
 param = UADNew(1,mini->atry->nData);
 for (i=0; i<mini->alpha->nData; i++)  mini->alpha->dataD[i] = 0;
 for (i=0; i<mini->beta->nData; i++)   mini->beta->dataD[i]  = 0;
 for (i=0; i<mini->atry->nData; i++)   param->dataD[i] = mini->atry->dataD[i];

 /*--- Loop over ymesured ---*/

 for (i=0; i<mesu->ymesured->nData; i++)
    {
     dy   = mesu->ymesured->dataD[i] - mesu->ypredicted(mesu->xmesured->dataD[i],param);
     sig2 = 1./ (mesu->yerrors->dataD[i] * mesu->yerrors->dataD[i]);
     mesu->derivative(mesu->xmesured->dataD[i],param,dyda);

     for (j=0; j<mesu->aestimated->nData; j++)
        {wt = dyda->dataD[j] * sig2;
         for (k=0; k<=j; k++)
            {indis = j + mesu->aestimated->nData * k;
             mini->alpha->dataD[indis] += wt * dyda->dataD[k];
            }
         mini->beta->dataD[j] += wt * dy;
        }

     for (j=1; j<mesu->aestimated->nData; j++)
        {for (k=0; k<j; k++)
            {indis  = j * mesu->aestimated->nData + k;
             indis2 = j + mesu->aestimated->nData * k;
             mini->alpha->dataD[indis] = mini->alpha->dataD[indis2];
            }
        }
    }

 UADFree(dyda);
 UADFree(param);
 return;
}
