/**************************************************/
/*                                                */
/* File: URootAccess.cc                           */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/
 
// Wrappers allowing access to ROOT functions

using namespace std;

#include <fstream>
#include <iostream>
#include <time.h>
#include "FrameL.h"
#include "TSystem.h"
#include "TClass.h"
#include "TEnv.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TH2.h"
#include "TAxis.h"
#include "TStyle.h"
#include "TDirectory.h"
#include "TFile.h"
#include "TKey.h"
#include "TApplication.h"
#include "TROOT.h"
#include "TImage.h"
#include "TColor.h"

#if ROOT_VERSION_CODE >= ROOT_VERSION(5,0,0)
 #include "TAttImage.h"
 #include "TImageDump.h"
#endif

#include "TList.h"
#include "TLegend.h"
#include "TPaveText.h"
#include "TPaveStats.h"
#include "TTimer.h"
#include "TObjectTable.h"
#include "TStorage.h"
#include "TFrame.h"
#include "TString.h"
#include "TSystem.h"
#include "dataDisplay.h"
#include "URootAccess.h"
#include "VDyPlot.h"
#include "VDyPlot2.h"
#include "TInterpreter.h"
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,3,4)
#include "TClassMenuItem.h"
#endif
#include "TMethod.h"
#include <sys/time.h>

#ifdef R__ALPHA
#include <sys/table.h>
#endif

#ifndef USE_VEGA
#if ROOT_VERSION_CODE <= ROOT_VERSION(3,3,9)
TROOT root("DD", "Data Display");
#endif
#endif

TCanvas*         gDDCanvas=NULL;
TVirtualPad*     gLastPad=NULL;
double           gLowestInterval;
TTimer*          gCheckTimer;
TString          gDDCanvasString;
char*            gDyTmpMacro;


//_____________________________________________________________________________
void RA_Main()
{
// Main for the root event loop

// Sets the timer to call periodically check_forms
 gCheckTimer = new TTimer(50);
 TQObject::Connect(gCheckTimer, "Timeout()", 0,0, "Wrapper_fl_check_forms()");
 gCheckTimer->Start();

#ifndef USE_VEGA
// Runs the root event loop
 gApplication->Run();
#endif
}


//_____________________________________________________________________________
void RA_ZoomAllStat (double zoomstep)
{
// Larger or smaller StatBox on all the plots

 int padmodified, padmodified2;
 double x1, y1;
 TPaveStats* pavestat;

 if (zoomstep >= 0.)
    {if (zoomstep > 0.5) zoomstep = 0.5;
     if (zoomstep < 0.1) zoomstep = 0.1;
    }
 else
    {if (zoomstep < -0.5) zoomstep = -0.5;
     if (zoomstep > -0.1) zoomstep = -0.1;
    }

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;
             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TH1::Class()))
                     {
                      TH1* h = (TH1 *)obj2;

                      pavestat= (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                      if (pavestat)
                         {x1    = pavestat->GetX1NDC();
                          y1    = pavestat->GetY1NDC();
                          y1 -= y1*zoomstep;
                          x1 -= x1*zoomstep;
                          if (y1 > 0.3 && y1 < 0.95) {pavestat->SetX1NDC(x1); pavestat->SetY1NDC(y1); padmodified = 1;}
                         }
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(TH1::Class()))
                             {
                              TH1* h = (TH1 *)obj3;
                              pavestat = (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                              if (pavestat)
                                 {x1    = pavestat->GetX1NDC();
                                  y1    = pavestat->GetY1NDC();
                                  y1 -= y1*zoomstep;
                                  x1 -= x1*zoomstep;
                                  if (y1 > 0.3 && y1 < 0.95) {pavestat->SetX1NDC(x1); pavestat->SetY1NDC(y1); padmodified2 = 1;}
                                 }
                             }
                         }
                     if (padmodified2 > 0) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                    }
                }
             if (padmodified > 0) {((TPad *)obj)->Modified(); ((TPad *)obj)->Update();}
            }
        }
    }
}


//_____________________________________________________________________________
void RA_ZoomAllLegends (double zoomstep)
{
// Larger or smaller legends for all the plots

 double yleg1;
 TLegend* leg;

 if (zoomstep >= 0.)
    {if (zoomstep > 0.5) zoomstep = 0.5;
     if (zoomstep < 0.1) zoomstep = 0.1;
    }
 else
    {if (zoomstep < -0.5) zoomstep = -0.5;
     if (zoomstep > -0.1) zoomstep = -0.1;
    }

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             leg = (TLegend*)(((TPad *)obj)->FindObject("DyLegendString"));
             if (leg)
                {yleg1 = leg->GetY1NDC();
                 leg->SetTextSize(0);
                 yleg1 -= 0.2*zoomstep;
                 if (yleg1 < 0.5)  yleg1 = 0.5;
                 if (yleg1 > 0.95) yleg1 = 0.95;
                 leg->SetY1NDC(yleg1);
                 ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      leg = (TLegend*)(((TPad *)obj2)->FindObject("DyLegendString"));
                      if (leg)
                         {yleg1 = leg->GetY1NDC();
                          leg->SetTextSize(0);
                          yleg1 -= 0.4*zoomstep;
                          if (yleg1 < 0.5)  yleg1 = 0.5;
                          if (yleg1 > 0.95) yleg1 = 0.95;
                          leg->SetY1NDC(yleg1);
                          ((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();
                         }
                    }
                }
            }
        }
    }

}


//_____________________________________________________________________________
void RA_ZoomAllLabels (double zoomstep, char *axisname)
{
// Larger Labels on axis of all the plots, and larger GPSorigin, title and unitsY

 int padmodified, padmodified2;
 double lsize, offset;
 char axis[64];

 if (zoomstep >= 0.)
    {if (zoomstep > 0.5) zoomstep = 0.5;
     if (zoomstep < 0.1) zoomstep = 0.1;
    }
 else
    {if (zoomstep < -0.5) zoomstep = -0.5;
     if (zoomstep > -0.1) zoomstep = -0.1;
    }


 if (axisname == NULL) strcpy(axis,"xaxis");
 else                  strcpy(axis, axisname);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TH1::Class()))
                     {
                      TH1* h = (TH1 *)obj2;
                      if (strcmp(axis,"xaxis") == 0)
                         {lsize = h->GetLabelSize("X"); lsize += 0.1*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetLabelSize(lsize, "X");
                          lsize = h->GetTitleSize("X"); lsize += 0.1*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetTitleSize(lsize, "X");
                          offset = 1.3-9.85*lsize; if (offset < 0.1) offset = 0.1; if (lsize > 0.04) h->SetTitleOffset(offset, "X");
                         }
                      else
                         {lsize = h->GetLabelSize("Y"); lsize += 0.1*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetLabelSize(lsize, "Y");
                          lsize = h->GetTitleSize("Y"); lsize += 0.1*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetTitleSize(lsize, "Y");
                          offset = 2.4-20.2*lsize; if (offset < 0.4) offset = 0.4; if (lsize > 0.04) h->SetTitleOffset(offset, "Y");
                         }
                      padmodified = 1;  
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;

                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(TH1::Class()))
                             {
                              TH1* h = (TH1 *)obj3;
                              if (strcmp(axis,"xaxis") == 0)
                                 {lsize = h->GetLabelSize("X"); lsize += 0.2*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetLabelSize(lsize, "X");
                                  lsize = h->GetTitleSize("X"); lsize += 0.2*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetTitleSize(lsize, "X");
                                  offset = 1.3-9.85*lsize; if (offset < 0.1) offset = 0.1; if (lsize > 0.04) h->SetTitleOffset(offset, "X");
                                 }
                              else
                                 {lsize = h->GetLabelSize("Y"); lsize += 0.2*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetLabelSize(lsize, "Y");
                                  lsize = h->GetTitleSize("Y"); lsize += 0.2*zoomstep; if (lsize < 0.04) lsize = 0.04; h->SetTitleSize(lsize, "Y");
                                  offset = 2.4-20.2*lsize; if (offset < 0.4) offset = 0.4; if (lsize > 0.04) h->SetTitleOffset(offset, "Y");
                                 }
                              padmodified2 = 1;
                             }
                         }
                      if (padmodified2 > 0){((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                     }
                 }
             if (padmodified > 0) {((TPad *)obj)->Modified(); ((TPad *)obj)->Update();}
            }
        }
    }
}


//_____________________________________________________________________________
void RA_ZoomAllTimeString (double zoomstep)
{
// Larger or smaller all plots GPSorigin time string

 double lsize;
 TPaveText* timepave;

 if (zoomstep >= 0.)
    {if (zoomstep > 0.5) zoomstep = 0.5;
     if (zoomstep < 0.1) zoomstep = 0.1;
    }
 else
    {if (zoomstep < -0.5) zoomstep = -0.5;
     if (zoomstep > -0.1) zoomstep = -0.1;
    }

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             timepave = (TPaveText*)(((TPad *)obj)->FindObject("DyTimeString"));
             if (timepave)
                {lsize = timepave->GetTextSize();
                 if (lsize == 0.) lsize = 0.05;
                 lsize += 0.05*zoomstep;
                 if (lsize < 0.03) lsize = 0.;
                 timepave->SetTextSize(lsize);
                 if (lsize > 0.07) timepave->SetTextAlign(12);
                 else              timepave->SetTextAlign(22);
                 ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      timepave = (TPaveText*)(((TPad *)obj2)->FindObject("DyTimeString"));
                      if (timepave)
                         {lsize = timepave->GetTextSize();
                          if (lsize == 0.) lsize = 0.05;
                          lsize += 0.05*zoomstep;
                          if (lsize < 0.03) lsize = 0.;
                          timepave->SetTextSize(lsize);
                          if (lsize > 0.07) timepave->SetTextAlign(12);
                          else              timepave->SetTextAlign(22);
                          ((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();
                         }
                     }
                 }
            }
        }
    }
}


//_____________________________________________________________________________
void RA_ZoomAllTitle (double zoomstep)
{
// Larger or smaller all plots titles

 double lsize;
 TPaveText* timepave;

 if (zoomstep >= 0.)
    {if (zoomstep > 0.5) zoomstep = 0.5;
     if (zoomstep < 0.1) zoomstep = 0.1;
    }
 else
    {if (zoomstep < -0.5) zoomstep = -0.5;
     if (zoomstep > -0.1) zoomstep = -0.1;
    }

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             timepave = (TPaveText*)(((TPad *)obj)->FindObject("title"));
             if (timepave)
                {lsize = timepave->GetTextSize();
                 if (lsize == 0) lsize = 0.05;
                 lsize += 0.1*zoomstep;
                 if (lsize < 0.03) lsize = 0.;
                 timepave->SetTextSize(lsize);
                 ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      timepave = (TPaveText*)(((TPad *)obj2)->FindObject("title"));
                      if (timepave)
                         {lsize = timepave->GetTextSize();
                          if (lsize == 0) lsize = 0.05;
                          lsize += 0.2*zoomstep;
                          if (lsize < 0.03) lsize = 0.;
                          timepave->SetTextSize(lsize);
                          ((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();
                         }
                     }
                 }
            }
        }
    }
}


//_____________________________________________________________________________
int RA_GetLog (int plot2d)
{
// Say if current pad plot axis is in log scale
  
 if (plot2d > 0)      {if (gPad->GetLogz() > 0) return(1);}
 else if (plot2d < 0) {if (gPad->GetLogx() > 0) return(1);}
 else                 {if (gPad->GetLogy() > 0) return(1);}
 return(0);
}


//_____________________________________________________________________________
void RA_LogLinAll (char *axisname)
{
// Put all plots in logx/linx or logy/liny

 char axis[64];
 int padmodified, padmodified2;
 static int logxFlip=0;
 static int logyFlip=0;

 if (axisname == NULL) strcpy(axis,"xaxis");
 else                  strcpy(axis, axisname);

 if (strcmp(axis,"xaxis") == 0) logxFlip = abs(logxFlip-1);
 else                           logyFlip = abs(logyFlip-1);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TH1::Class()))
                     {
                       TH1 *h = (TH1 *)obj2;
                      padmodified = 1;
                      if (strcmp(axis,"xaxis") == 0)
                         {if (logxFlip == 0 || (logxFlip == 1 &&
                                                h->GetXaxis()->GetBinLowEdge(h->GetXaxis()->GetFirst()) > 0 &&
                                                strstr(h->GetTitle(),"TIME") == NULL)) ((TPad *)obj)->SetLogx(logxFlip);
                         }
                      else
                         {if (logyFlip == 0 || (logyFlip == 1 && h->GetMinimum() > 0)) ((TPad *)obj)->SetLogy(logyFlip);}
                      break;
                     }

                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;

                      while ((obj3 = next()))
                        {
                         if (obj3->InheritsFrom(TH1::Class()))
                            {
                             padmodified2 = 1;
                             if (strcmp(axis,"xaxis") == 0) ((TPad *)obj)->SetLogx(logxFlip);
                             else                           ((TPad *)obj)->SetLogy(logyFlip);
                             break;
                            }
                        }

                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }

                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }

}


//______________________________________________________________________________
void RA_HSetColor(char *name, int color)
{
// Set histogram line color (1=black, 2=red, 3=green, 4=blue, 5=yellow, 6=pink)

 if (color == 3)  color  = 30; /* or 32 or 36 or 8 */
 if (color == 5)  color  = 41; /* or 44 or 28 */
 if (color == 7)  color  = 38; /* or 36 or 9 */
 if (color == 10) color = 28;

 TH1* h=(TH1*)gDirectory->Get(name);
 if (h) h->SetLineColor((Color_t)color);
}


//______________________________________________________________________________
void RA_HSetLineStyle(char *name, int style)
{
// Set histogram line style (1=solid, 2=dash, 3=dash-dot, 4=dot)

 TH1* h=(TH1*)gDirectory->Get(name);
 if (h) h->SetLineStyle(style);
}


//______________________________________________________________________________
void RA_HSetStat(char *name, char *options)
{
// Put stat box on plot if chosen in options

 TH1* h=(TH1*)gDirectory->Get(name);

 if (h)
    {
     if (options == NULL)
        {h->SetStats(0);}
     else
        {if (strstr(options,"STA") != NULL) h->SetStats(1);
         else                               h->SetStats(0);
        }
    }
}


//______________________________________________________________________________
void RA_HSetLegend (char *name, char *text_orig, int color, int nsame)
{
 TLegend* leg;
 char *p, *text;
 double xorig, yorig, yoffset, length;

// Add legend for superimposed plots in Pad

 TH1* h=(TH1*)gDirectory->Get(name);

 if (!h)         return;
 if (!gPad)      return;

 leg = (TLegend*)gPad->FindObject("DyLegendString");

 if (!text_orig) {if (leg) delete leg; return;}

 text = (char *)malloc(strlen(text_orig)+1);
 strcpy(text, text_orig);

 if ((p = strstr(text,SEPTITLE)) != NULL)
    {
     const char *drawOption = h->GetDrawOption();
     if (drawOption)
       {if (strstr(drawOption,"same") != NULL || strstr(drawOption,"SAME") != NULL) *p = '\0';}
    }

 if (!leg)
    {
     if (text)
        {
         xorig = 0.23; length = 0.75;
         yoffset = 0.;
         if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)
            {if (strcmp(((VDyPlot*)h)->GetPlotType(),"1D") == 0) {xorig = 0.10; length = 0.75;}
             if (strcmp(((VDyPlot*)h)->GetPlotType(),"TRFCT") == 0) yoffset = 0.05;
            }
         if (nsame > 4)      yorig = 0.56;
         else if (nsame > 0) yorig = 0.96 - nsame*0.08;
         leg = new TLegend(xorig, yorig-yoffset, xorig+length, 0.99);
         leg->AddEntry(h,text,"l");
         leg->SetName("DyLegendString");
         if (Dy->nicefont == OFF) leg->SetTextFont(DY_FONT_TIMES);
         else                     leg->SetTextFont(DY_FONT_HELVETICA);
         leg->SetFillColor(0);
//         leg->SetTextSize(0.04);
         leg->SetTextSize(0);
         leg->Draw();
        }
    }
 else
    {
     if (text)
        {leg->AddEntry(h,text,"l");
/*         leg->SetTextColor(color); */
        }
     leg->Draw();
    }
 free(text);
}


//______________________________________________________________________________
void RA_HSetWarningString (char *text)
{
// Add warning string under plot in the all Pad

 int color=1;
 TPaveText* warningpave;

 if (gPad && text)
    {if (gPad->GetBottomMargin() < 0.08+0.10) gPad->SetBottomMargin(0.08+0.10);}

 if (gPad)
    {
     if (text)
        {if (strstr(text,"FRAME") != NULL)        color = 4;
         else if (strstr(text,"CHANNEL") != NULL) color = 3;
         else if (strstr(text,"DATA") != NULL)    color = 2;
         else                                     color = 1;
        }

     warningpave = (TPaveText*)gPad->FindObject("DyWarningString");
     if (!warningpave)
        {if (text)
            {warningpave = new TPaveText(0.05, 0.08, 0.40, 0.12,"bNDC");
             warningpave->SetName("DyWarningString");
             TString warningString = text;
             warningpave->AddText(warningString.Data());
             if (Dy->nicefont == OFF) warningpave->SetTextFont(DY_FONT_TIMES);
             else                     warningpave->SetTextFont(DY_FONT_HELVETICA);
             warningpave->SetTextColor(color);
             warningpave->SetFillColor(0);
             warningpave->Draw("NDC");
            }
        }

     else
        {
         if (text)
            {warningpave->Clear();
             TString warningString = text;
             warningpave->AddText(warningString.Data());
             warningpave->SetTextColor(color);
             warningpave->Draw("NDC");
            }
         else
            {delete warningpave; warningpave = NULL;}
        }
    }
}


//_____________________________________________________________________________
void RA_SetLabelFont (int fonttype, char *plotName)
{
 if (plotName != NULL)
    {
     TH1* h=(TH1*)gDirectory->Get(plotName);
     if (h)
        {h->SetLabelFont(fonttype,"X");
         h->SetLabelFont(fonttype,"Y");
        }
    }
 else
    {
     if (!gStyle) return;
     gStyle->SetTextFont(fonttype);
     gStyle->SetLabelFont(fonttype,"X");
     gStyle->SetLabelFont(fonttype,"Y");
     //   gStyle->SetTitleFont(fonttype);
    }
}


//_____________________________________________________________________________
void RA_SetLabelSize (double size, char *plotName)
{
 if (plotName != NULL)
    {
     TH1* h=(TH1*)gDirectory->Get(plotName);
     if (h)
        {h->SetLabelSize(size,"X");
         h->SetLabelSize(size,"Y");
        }
    }
 else
    {
     if (!gStyle) return;
     gStyle->SetLabelSize(size,"X");
     gStyle->SetLabelSize(size,"Y");
    }
}


//_____________________________________________________________________________
void RA_SetCanvasColor(int fullwhite)
{
 if (!gDDCanvas) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;

 if (fullwhite == 1) gDDCanvas->SetFillColor(0);
 else                gDDCanvas->SetFillColor(19);

 gDDCanvas->Modified();
 gDDCanvas->Update();

}


//_____________________________________________________________________________
void RA_InitROOT()
{
// Initialize the ROOT environment
   
#ifndef USE_VEGA

 DyDebugPrint(__FUNCTION__,0);

 DyDebugTimeNeeded(__FUNCTION__,"start",0);

 if (Dy->BgRun == ON) gROOT->SetBatch();
// if (Dy->BgRun == ON) gApplication->MakeBatch();

/**
 gEnv->SetValue("Unix.*.Root.UseTTFonts","false");
**/
 new TApplication("App", 0, 0);

 /*--- Init of gStyle ---*/

 if (Dy->nicefont == OFF)
    {
     gStyle->SetTextFont(DY_FONT_TIMES);
     gStyle->SetTitleFont(DY_FONT_TIMES);
     gStyle->SetLabelFont(DY_FONT_TIMES,"X");
     gStyle->SetLabelFont(DY_FONT_TIMES,"Y");
    }
 else 
    {
     gStyle->SetTextFont(DY_FONT_HELVETICA);
     gStyle->SetTitleFont(DY_FONT_HELVETICA);
     gStyle->SetLabelFont(DY_FONT_HELVETICA,"X");
     gStyle->SetLabelFont(DY_FONT_HELVETICA,"Y");
    }

 gStyle->SetTextSize(0.05);
 gStyle->SetTitleBorderSize(0);
 gStyle->SetTitleSize(0.05);
 gStyle->SetLabelSize(0.05,"X");
 gStyle->SetLabelSize(0.05,"Y");
 gStyle->SetTitleW(0.8);
 gStyle->SetTitleH(0.07);
 //gStyle->SetTitleY(0.99);
 //gStyle->SetTitleX(0.1);
 gStyle->SetTitleStyle(0);
 gStyle->SetStatY(0.90);
 gStyle->SetOptStat(1);
 gStyle->SetNdivisions(505,"X");
 gStyle->SetNdivisions(505,"Y");
 gStyle->SetPadLeftMargin(0.13);
 gStyle->SetPadTopMargin(0.12);
// gStyle->SetPadRightMargin(0.15);
// gStyle->SetPadBottomMargin(0.18);
// gROOT->ForceStyle();
// gROOT->SetStyle("Plain");

 gStyle->SetCanvasBorderMode(0);
 gStyle->SetCanvasColor(19);
 gStyle->SetPadBorderMode(0);
 gStyle->SetPadColor(0);
 gStyle->SetFrameBorderMode(0);
// gStyle->SetFillColor(19);
// gStyle->SetFrameFillColor(19);
 gStyle->SetGridColor(14);

/**
 RA_SetColorPalette(0);
**/
 Dy->colorpalette = 0;

// TStorage::EnableStatistics();
 gSystem->ResetSignal(kSigBus);
 gSystem->ResetSignal(kSigSegmentationViolation);
 gSystem->ResetSignal(kSigSystem);
 gSystem->ResetSignal(kSigPipe);
 gSystem->ResetSignal(kSigIllegalInstruction);
 gSystem->ResetSignal(kSigQuit);
 gSystem->ResetSignal(kSigInterrupt);
 gSystem->ResetSignal(kSigWindowChanged);
 gSystem->ResetSignal(kSigAlarm);
 gSystem->ResetSignal(kSigChild);
 gSystem->ResetSignal(kSigUrgent);
 gSystem->ResetSignal(kSigFloatingException);
 gSystem->ResetSignal(kSigTermination);
 gSystem->ResetSignal(kSigUser1);
 gSystem->ResetSignal(kSigUser2);
#endif
      
 gDyTmpMacro = RA_CreateTempFileName(".C");

// Customize the Axis menu if possible
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,3,4)
 TClass* claxis = gROOT->GetClass("TAxis");
 claxis->MakeCustomMenuList();
   
 TList* laxmenu = claxis->GetMenuList();

// Remove the access to class information

 TClassMenuItem* mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(1);
 delete mi;

// Change Unzoom position in the menu

 TClassMenuItem* miUnzoom = (TClassMenuItem*)laxmenu->At(12);
 laxmenu->Remove(miUnzoom);
 laxmenu->AddFirst(miUnzoom);

// Add the other specific commands in the menu

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoom ALL Z","URUnZoomAllZ",0,"",0,0));

 laxmenu->AddFirst(new TClassMenuItem(TClassMenuItem::kPopupSeparator,claxis));

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoomY","URUnZoomY",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoom ALL Y","URUnZoomAllY",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Scale ALL Y as last","URScaleAllYAsLast",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Scale Y as last","URScaleYAsLast",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom Y axis as last","URZoomYAsLast",0,"TObject*",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom ALL Y as last","URZoomAllYAsLast",0,"",0,0));

 laxmenu->AddFirst(new TClassMenuItem(TClassMenuItem::kPopupSeparator,claxis));

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoomX","URUnZoomX",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoom ALL X","URUnZoomAllX",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom X axis as last","URZoomXAsLast",0,"TObject*",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom ALL X as last","URZoomAllXAsLast",0,"",0,0));

 laxmenu->AddFirst(new TClassMenuItem(TClassMenuItem::kPopupSeparator,claxis));

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom All StatBox","URZoomAllStat",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom All Legends","URZoomAllLegends",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom All Labels","URZoomAllLabels",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Log/Lin All","URLogLinAll",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Update Time/Freq","URUpdateTimeFreq",0,"",0,0));
// laxmenu->AddFirst(new TClassMenuItem(
//       TClassMenuItem::kPopupUserFunction,claxis,"Draw All as this one","URDrawOptionsAllAsThis",0,"",0,0));

#endif
   
 gDDCanvasString = "";

/*--- Execute the logon macro as in Root ---*/
 RA_ExecLogon();

 DyDebugTimeNeeded(__FUNCTION__,"stop",0);
}


//_____________________________________________________________________________
void RA_ExecLogon()
{
// Copy of TRint's "ExecLogon" method (could not use it since it is private)
//
// Execute logon macro's. There are three levels of logon macros that
// will be executed: the system logon etc/system.rootlogon.C, the global
// user logon ~/.rootlogon.C and the local ./.rootlogon.C. For backward
// compatibility also the logon macro as specified by the Rint.Logon
// environment setting, by default ./rootlogon.C, will be executed.
// No logon macros will be executed when the system is started with
// the -n option.

 DyDebugPrint(__FUNCTION__,0);

 if (gApplication->NoLogOpt()) return;

 TString name = ".rootlogon.C";
 TString sname = "system";
 sname += name;

#ifdef ROOTETCDIR
 char *s = gSystem->ConcatFileName(ROOTETCDIR, sname);
#else
 TString etc = gRootDir;
#ifdef WIN32
 etc += "\\etc";
#else
 etc += "/etc";
#endif
 char *s = gSystem->ConcatFileName(etc, sname);
#endif
 if (!gSystem->AccessPathName(s, kReadPermission)) gApplication->ProcessFile(s);
 delete [] s;

 s = gSystem->ConcatFileName(gSystem->HomeDirectory(), name);
 if (!gSystem->AccessPathName(s, kReadPermission)) gApplication->ProcessFile(s);
 delete [] s;

 // avoid executing ~/.rootlogon.C twice
 if (strcmp(gSystem->HomeDirectory(), gSystem->WorkingDirectory()))
    {if (!gSystem->AccessPathName(name, kReadPermission)) gApplication->ProcessFile(name);}

 // execute also the logon macro specified by "Rint.Logon"
 const char *logon = gEnv->GetValue("Rint.Logon", (char*)0);
 if (logon)
    {char *mac = gSystem->Which(TROOT::GetMacroPath(), logon, kReadPermission);
     if (mac) gApplication->ProcessFile(logon);
     delete [] mac;
    }
}


//_____________________________________________________________________________
int RA_SetColorPalette (int choice)
{
// Set color palette
   
#ifndef USE_VEGA

 DyDebugPrint(__FUNCTION__,0);

 if (choice == 0)
   {
    gStyle->SetPalette(1);   /*--- Standard rainbow palette ---*/
   }

 else if (choice == 1)  /*--- rainbow palette more contrasted ---*/
   {int ncolors = 128;
    const Int_t NRGBs = 5;
    Double_t Stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
    Double_t Green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
    Double_t Blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 2)  /*--- rainbow palette more contrasted ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 7;
    Double_t Stops[NRGBs] = { 0.00, 0.20, 0.35, 0.50, 0.65, 0.80, 1.00 };
    Double_t Red[NRGBs]   = { 0.20, 0.00, 0.00, 0.00, 0.97, 0.97, 0.30 };
    Double_t Green[NRGBs] = { 0.00, 0.30, 0.97, 0.40, 0.97, 0.00, 0.00 };
    Double_t Blue[NRGBs]  = { 0.30, 0.97, 0.97, 0.00, 0.00, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 3)  /*--- violet-->green-->red-->dark red ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.20, 0.40, 0.97, 0.00, 0.00, 0.97, 0.97, 0.40 };
    Double_t Green[NRGBs] = { 0.00, 0.04, 0.00, 0.97, 0.97, 0.97, 0.00, 0.00 };
    Double_t Blue[NRGBs]  = { 0.20, 0.30, 0.50, 0.97, 0.00, 0.00, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 4)   /*--- black-->red-->yellow-->white ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.10, 0.20, 0.40, 0.60, 0.97, 0.97, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.20, 0.97, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 5)   /*--- black-->green-->yellow ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.60, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.10, 0.20, 0.30, 0.70, 0.80, 0.97, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 6)  /*--- black-->yellow-->blue-->white ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.10, 0.60, 0.97, 0.00, 0.00, 0.70, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.10, 0.60, 0.97, 0.20, 0.50, 0.70, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.10, 0.00, 0.00, 0.20, 0.50, 0.70, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 7)  /*--- black-->red-->green-->violet ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.30, 0.00, 0.00, 0.20, 0.40, 0.97, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.00, 0.97, 0.97, 0.00, 0.04, 0.97, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.00, 0.97, 0.00, 0.20, 0.30, 0.00, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 8)  /*--- black-->gray-->white ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.10, 0.20, 0.50, 0.60, 0.80, 0.90, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.10, 0.20, 0.50, 0.60, 0.80, 0.90, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.10, 0.20, 0.50, 0.60, 0.80, 0.90, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else
   {
    gStyle->SetPalette(1);   /*--- Standard rainbow palette ---*/
    choice = 0;
   }

 RA_MainCanvasUpdate();

#endif

 return (choice);
}


//_____________________________________________________________________________
void RA_InitDDWindow(char* title, int posx, int posy, int width, int height)
{
// Initialize global data display canvas
   
 DyDebugPrint(__FUNCTION__,0);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) && (strcmp(title,"CLEAR") == 0))
    {delete gDDCanvas;
     gDDCanvas=0;
    }

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
       {if (obj->InheritsFrom(TPad::Class())) RA_PadClear((void*)obj);}
    }
 else
    {gDDCanvas = new TCanvas("gDDCanvas",title,posx,posy,width,height);
     gSystem->ProcessEvents();
     gDDCanvas->ToggleEventStatus();
     gDDCanvas->Resize();
    }

 gDDCanvas->cd();
}


//_____________________________________________________________________________
void RA_MainCanvasClear()
{
// Clear the global canvas
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas)) gDDCanvas->Clear();
}


//_____________________________________________________________________________
void RA_MainCanvasDelete()
{
 if (!gDDCanvas) return;

// Clear the global canvas
   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas)) {
      delete gDDCanvas;
      gDDCanvas=0;
   }
}


//_____________________________________________________________________________
void RA_MainCanvasUpdate()
{
// Update the global canvas on the screen

 DyDebugPrint(__FUNCTION__,0);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     if (RA_SystemCheckSwap()) 
        {gDDCanvas->Paint();
         gDDCanvas->Modified();
         gDDCanvas->Update();
        } 
    }
}


//_____________________________________________________________________________
void RA_FileOpenCreate(char* name)
{
// Open a new file for creation
   
 TDirectory* tmpdir;
 tmpdir = gDirectory;
 new TFile(name,"RECREATE");
 tmpdir->cd();
}


//_____________________________________________________________________________
void RA_FileWrite(char* name)
{
// Write out all histograms to a file
   
 TH1F* idcurcopy;
 TH2F* idcurcopy2;
 TDirectory* tmpdir;

 DyDebugPrint(__FUNCTION__,0);

 tmpdir = gDirectory;

 TFile* f = (TFile*)(gROOT->GetListOfFiles()->FindObject(name));

 if (!f) {printf("RA_FileWrite> file %s not opened\n",name); return;}
 f->cd();

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());

     /*--- Loop on all the objects of the root Canvas ---*/
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             while ((obj2 = next()))   /*--- If Pad encountered, loop on all the objects of this pad ---*/
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                         {printf("RA_FileWrite> Writing as TH1F the VDyPlot plot %s \n", h->GetTitle());}
                      idcurcopy = new TH1F(*((TH1F*)obj2));
                      idcurcopy->Write();
                      delete idcurcopy;
                     }
                  else if (obj2->InheritsFrom(VDyPlot2::Class()))
                     {
                      VDyPlot2* h = (VDyPlot2 *)obj2;
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                         {printf("RA_FileWrite> Writing as TH2F the VDyPlot2 plot %s \n", h->GetTitle());}
                      idcurcopy2 = new TH2F(*((TH2F*)obj2));
                      idcurcopy2->Write();
                      delete idcurcopy2;
                     }
                  else if (obj2->InheritsFrom(TH1::Class()))
                     {obj2->Write();
                      TH1* h = (TH1 *)obj2;
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                         {printf("RA_FileWrite> Writing TH1 plot %s \n", h->GetTitle());}
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))  /*--- If SubPad encountered, loop on all the objects of this Subpad ---*/
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                                 {printf("RA_FileWrite> Writing as TH1F the VDyPlot plot %s \n", h->GetTitle());}
                              idcurcopy = new TH1F(*((TH1F*)obj3));
                              idcurcopy->Write();
                              delete idcurcopy;
                             }
                          else if (obj3->InheritsFrom(VDyPlot2::Class()))
                             {
                              VDyPlot2* h = (VDyPlot2 *)obj3;
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                                 {printf("RA_FileWrite> Writing as TH2F the VDyPlot2 plot %s \n", h->GetTitle());}
                              idcurcopy2 = new TH2F(*((TH2F*)obj3));
                              idcurcopy2->Write();
                              delete idcurcopy2;
                             }
                          else if (obj3->InheritsFrom(TH1::Class()))
                             {obj3->Write();
                              TH1* h = (TH1 *)obj3;
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                                 {printf("RA_FileWrite> Writing TH1 plot %s \n", h->GetTitle());}
                             }
                          else
                             {obj3->Write();
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                                 {printf("RA_FileWrite> Writing object %s  class=%s\n", obj3->GetName(), obj3->ClassName());}
                             }
                         }
                     }
                  else
                     {obj2->Write();
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                         {printf("RA_FileWrite> Writing object %s  class=%s\n", obj2->GetName(), obj2->ClassName());}
                     }
                 }
             }
         }
     } 

/***
 TIter next(tmpdir->GetList());
 while (idcur = next())
    {
     if (idcur->InheritsFrom(TH1F::Class()))
        {
         idcurcopy = new TH1F(*((TH1F*)idcur));
         idcurcopy->Write();
         delete idcurcopy;
        }
     else
        {idcur->Write();}

     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
        {TH1 *h = (TH1*)idcur; 
         printf("RA_FileWrite> %s: %s=%s written in %s\n", idcur->IsA()->GetName(), h->GetName(), h->GetTitle(), name);
        }
    }
***/

 tmpdir->cd();
}


//_____________________________________________________________________________
void RA_FileClose(char* name)
{
// Write out all histograms to a file
   
 TFile* f = (TFile*)(gROOT->GetListOfFiles()->FindObject(name));
 if (!f) {printf("RA_FileClose> file %s was not open\n",name); return;}
 f->Close();
 delete f;
}


//_____________________________________________________________________________
int RA_SavePlots(char* filename)
{
// Save the plots into a file with format specified by the suffix

 char *str1, *str2, *str3, *str4;
 char name[1024], jpgFileName[256], epsFileName[256], rootFileName[256];
 int suffixType;
#if ROOT_VERSION_CODE < ROOT_VERSION(5,0,0)
 char execConvert[1024];
 int xsize, ysize;
#endif

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->outPicture) strcpy(name, Dy->outPicture);
 else                strcpy(name, filename);

 str1 = strstr(name,CONFIG_SUFFIX);
 str2 = strstr(name,".jpg");
 str3 = strstr(name,".eps");
 str4 = strstr(name,ROOT_SUFFIX);
 if      (str1 && strlen(name) <= str1-name+4) {suffixType=1; *str1 = '\0';}
 else if (str2 && strlen(name) <= str2-name+4) {suffixType=1; *str2 = '\0';}
 else if (str3 && strlen(name) <= str3-name+4) {suffixType=2; *str3 = '\0';}
 else if (str4 && strlen(name) <= str4-name+5) {suffixType=3; *str4 = '\0';}
 else                                           suffixType=1;

 printf("RA_SavePlots> filename=%s  name=%s  suffix=%d \n", filename, name, suffixType);

 if (suffixType == 1)
    {
     if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) {printf("RA_SavePlots> No plot to save\n"); return(0);}
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,0,0)
     //======= Real speed-up, suggested by Valeriy Onuchin 16-05-07 ======
     //   uses the "SaveAs" feature of ROOT 5, but with almost no compression
     sprintf(jpgFileName, "%s.jpg", name);
     printf("RA_SavePlots> Creating %s \n", jpgFileName);
     TImageDump *imgdump = new TImageDump(jpgFileName); 
     gDDCanvas->Paint(); 
     TImage *img = imgdump->GetImage();
     img->SetImageQuality(img->kImgFast);
//     img->SetImageQuality(img->kImgGood);
//     img->SetImageQuality(img->kImgBest); 
     img->SetImageCompression(60); 
     imgdump->Close();
     imgdump->Delete();
#else
     sprintf(jpgFileName, "%s.jpg", name);
     sprintf(epsFileName, "%s.eps", name);
     printf("RA_SavePlots> Creating %s and converting it into jpg\n", epsFileName);
     xsize = gDDCanvas->GetWindowWidth();
     ysize = gDDCanvas->GetWindowHeight();
     gDDCanvas->Print(epsFileName);
     sprintf(execConvert," convert -cache 512 -geometry %dx%d -density 288x288 -colors 256 -sharpen 0.2 %s %s",xsize,ysize,epsFileName,jpgFileName);
     gSystem->Exec(execConvert);
     gSystem->Exec(Form("rm -f %s",epsFileName));
#endif
    }

 else if (suffixType == 2)
    {if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) {printf("RA_SavePlots> No plot to save\n"); return(0);}
     sprintf(epsFileName, "%s.eps", name);
     printf("RA_SavePlots> Creating %s \n", epsFileName);
     gDDCanvas->Print(epsFileName);
    }

 else if (suffixType == 3)
    {sprintf(rootFileName, "%s%s", name, ROOT_SUFFIX);
     printf("RA_SavePlots> Creating %s \n", rootFileName);
//     gDDCanvas->SaveAs(rootFileName);
     RA_FileOpenCreate(rootFileName);
     RA_FileWrite(rootFileName);
     RA_FileClose(rootFileName);
    }

 else
    {printf("RA_SavePlots> Suffix must be .root , .jpg or .ps\n"); return(0);}

 return(1);
}


//_____________________________________________________________________________
void RA_AllPadUpdate()
{
// Update each pad (should be useless since gDDCanvas->Paint() do the same in RA_MainCanvasUpdate) 

 DyDebugPrint(__FUNCTION__,0);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             while ((obj2 = next()))
                {if (obj2->InheritsFrom(TPad::Class())) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}}
            }
        }
    }
}


//_____________________________________________________________________________
int RA_DeleteUnusedPlot (int nvisible)
{
// List the plots in memory

 TObject *obj;
 char name[1024];
 int nall, nplot;

 nall  = 0;
 nplot = 0;

 TIter  next(gROOT->GetList());
 while ((obj = next()))
    {if (obj->InheritsFrom(TH1::Class()))
        {TH1 *h = (TH1*)obj;
         if (strstr(h->GetName(),REF_SUFFIX) == NULL) nall++;
        }
    }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0) printf("RA_DeleteUnusedPlot> nall=%d  nvisible=%d \n", nall, nvisible);

 if (nall > nvisible)
    {
     TIter  next(gROOT->GetList());
     while ((obj = next()) && nplot < nall-nvisible)
        {
         if (obj->InheritsFrom(TH1::Class()))
            {TH1 *h = (TH1*)obj;
             if (strstr(h->GetName(),REF_SUFFIX) != NULL) continue;
             if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
                {printf("RA_DeleteUnusedPlot> deleted plot %s = %s \n", h->GetName(), h->GetTitle());}
             delete h;
             nplot++;
            }
        }
    }

 return(nplot);
}


//_____________________________________________________________________________
int RA_CheckAllPlot (int showall)
{
// List the plots in memory

 TObject *obj;
 int nplot;

 TIter    next(gROOT->GetList());
 nplot = 0;
 while ((obj = next()))
    {
     if (obj->InheritsFrom(VDyPlot::Class()))
        {VDyPlot *h = (VDyPlot*)obj;
         nplot++;
         if (showall) printf("  Plot%02d : VDyPlot  : %s = %s  in pad %d \n", nplot, h->GetName(), h->GetTitle(), h->numpad);
        }
     else if (obj->InheritsFrom(VDyPlot2::Class()))
        {VDyPlot2 *h = (VDyPlot2*)obj;
         nplot++;
         if (showall) printf("  Plot%02d : VDyPlot2 : %s = %s  in pad %d \n", nplot, h->GetName(), h->GetTitle(), h->numpad);
        }
     else if (obj->InheritsFrom(TH1::Class()))
        {TH1 *h = (TH1*)obj;
         nplot++;
         if (showall) printf("  Plot%02d : TH1      : %s = %s \n", nplot, h->GetName(), h->GetTitle());
        }
    }

 if (showall > 1) gROOT->GetList()->Print();

 return(nplot);
}


// obsolete function
//_____________________________________________________________________________
void RA_DelAllPlot()
{
// Delete all the plots in memory which are not reference plots

 TObject *obj;

 DyDebugPrint(__FUNCTION__,0);

 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *h = (TH1*)obj;
         if (strstr(h->GetName(),REF_SUFFIX) != NULL) continue;
         printf("RA_DelAllPlot> deleted plot %s=%s \n", h->GetName(), h->GetTitle());
         delete h;
        }
    }

 return;
}


//_____________________________________________________________________________
int RA_DelReferencePlot (char *filename)
{
// Delete all the reference plots in memory, for a given filename

 int hexist, stillone;
 int numPicture, subpad, color;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char hname[64], dummy[64], *str, *str2, filename2[LSTRING_MAX];
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (filename) basename = gSystem->BaseName(filename);
 else basename = NULL;

 hexist = 0;
 stillone = 0;
 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *href = (TH1*)obj;
         str2 = (char *)strstr(href->GetName(),REF_SUFFIX);
         if (str2 == NULL) continue;
         color = href->GetLineColor();
         str = (char *)strstr(href->GetName(),SEPTITLE);
         if (filename && strstr(href->GetName(),basename) == NULL) {stillone = 1; continue;}
         if (filename == NULL)
            {strncpy(filename2, str+strlen(SEPTITLE), str2-str-strlen(SEPTITLE));
             filename2[str2-str-strlen(SEPTITLE)] = '\0';
             basename = gSystem->BaseName(filename2);
            }
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);
         if (obj2)
            {if (obj2->InheritsFrom(VDyPlot2::Class()))
                {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             else
                {VDyPlot *hv = (VDyPlot *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             RA_SetActivePad(numPicture);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
             if (gDDCanvas && gPad)
                {sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                 timepave = (TPaveText*)(gPad->FindObject(dummy));
                 if (timepave) delete timepave;
                }
            }
         printf("RA_DelReferencePlot> deleted plot %s=%s \n", href->GetName(), href->GetTitle());
         delete href;
         hexist = 1;
        }
    }

 if (hexist == 1) RA_MainCanvasUpdate();

 return(stillone);
}


//_____________________________________________________________________________
int RA_Del2DReferencePlot (char *filename)
{
// Delete all the 2-dimensional reference plots in memory for a given filename

 int hexist;
 int numPicture, subpad, color;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char hname[64], dummy[64], *str;
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (filename == NULL) return(0);

 basename = gSystem->BaseName(filename);

 hexist = 0;
 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *href = (TH1*)obj;
         if (strstr(href->GetName(),REF_SUFFIX) == NULL)           continue;
         if (filename && strstr(href->GetName(),basename) == NULL) continue;
         if (strstr(href->GetTitle(),"__2D") == NULL        &&
             strstr(href->GetTitle(),"__FFTTIME") == NULL   &&
             strstr(href->GetTitle(),"__1DTIME") == NULL    &&
             strstr(href->GetTitle(),"__COHETIME") == NULL  &&
             strstr(href->GetTitle(),"__TRTIME") == NULL    &&
             strstr(href->GetTitle(),"__RAWTIME") == NULL)  continue;
         if (strstr(href->GetTitle(),"__RAW") != NULL || strstr(href->GetTitle(),"__COHE") != NULL)
            {int nx=href->GetNbinsX();
             int ny=href->GetNbinsY();
             if (nx<=1 || ny<=1) continue;
            }

         printf("RA_DelReferencePlot> deleted plot %s=%s \n", href->GetName(), href->GetTitle());
         color = href->GetLineColor();
         str = (char *)strstr(href->GetName(),SEPTITLE);
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);
         if (obj2)
            {if (obj2->InheritsFrom(VDyPlot2::Class()))
                {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             else
                {VDyPlot *hv = (VDyPlot *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             RA_SetActivePad(numPicture);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
             sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
             timepave = (TPaveText*)(gPad->FindObject(dummy));
             if (timepave) delete timepave;
            }
         delete href;
         hexist = 1;
        }
    }

 if (hexist == 1) RA_MainCanvasUpdate();

 return(hexist);
}


// Obsolete function
//_____________________________________________________________________________
void RA_CheckReferencePlot()
{
// Check if reference plots are superposed plots and change their name accordingly

 TObject *obj;
 char hname[64], *str;
 VDyPlot *hv;

 gROOT->GetList()->Print();

 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *href = (TH1*)obj;
         if ((str = (char *)strstr(href->GetName(),REF_SUFFIX)) == NULL) continue;
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         hv = (VDyPlot *)gROOT->Get(hname);
         if (hv) printf("RA_CheckReferencePlot> %s is superposed to %s=%s in pad %d \n", href->GetName(), hv->GetName(), hv->GetTitle(), hv->numpad);
         else    printf("RA_CheckReferencePlot> %s=%s is superposed to nothing! \n", href->GetName(), href->GetTitle());
        }
    }
}


//_____________________________________________________________________________
void RA_ModifyReferencePlot (char *fileName, int color)
{
// Modify the color of a set of reference plots

 int numPicture, subpad;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char hname[64], dummy[64], *str;
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (fileName == NULL) return;

 basename = gSystem->BaseName(fileName);

 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {
         TH1 *href = (TH1*)obj;
         if (strstr(href->GetName(),REF_SUFFIX) == NULL || strstr(href->GetName(),basename) == NULL) continue;
         href->SetLineColor(color);

         str = (char *)strstr(href->GetName(),SEPTITLE);
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);
         if (obj2)
            {if (obj2->InheritsFrom(VDyPlot2::Class()))
                {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             else
                {VDyPlot *hv = (VDyPlot *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }

             RA_SetActivePad(numPicture);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
           
             sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
             timepave = (TPaveText*)(gPad->FindObject(dummy));
             if (timepave) timepave->SetTextColor(color);
            }
        }
    }

 RA_MainCanvasUpdate();
 return;
}


//_____________________________________________________________________________
int RA_ReadReferencePlot (char *fileName, int color)
{
// Read from file all the reference plots and put them in memory

 TKey *key;
 char dummy[256];
 const char *basename;
 int nref;

 DyDebugPrint(__FUNCTION__,0);

 if (fileName == NULL) return(0);

 TFile *fp = new TFile(fileName);

 if (fp->GetListOfKeys() == NULL) {printf("RA_ReadReferencePlot> file %s not found\n", fileName); return(0);}

 nref = 0;
 TIter next(fp->GetListOfKeys());
 while ((key = (TKey*)next()))
    {
     if (strncmp(key->GetClassName(),"TH",2) == 0)
        {TH1 *href = (TH1*)fp->Get(key->GetName());
         if (strstr(href->GetName(),REF_SUFFIX) != NULL) continue;
         href->SetDirectory(gROOT);
         if (color > 1) href->SetLineColor(color);
         else           href->SetLineColor(51);
         basename = gSystem->BaseName(fileName);
         sprintf(dummy, "%s%s%s%s", href->GetName(), SEPTITLE, basename, REF_SUFFIX);
         href->SetName(dummy);
         printf("RA_ReadReferencePlot> found in %s plot %s=%s  color=%d\n", fileName, href->GetName(), href->GetTitle(), href->GetLineColor());
         nref++;
        }
    }
 fp->Close();

 return(nref);
}


//_____________________________________________________________________________
void RA_DrawReferencePlot()
{
// Draws all the reference plots present in memory

 int numPicture, subpad, color, ok;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char filename[LSTRING_MAX], hname[64], dummy[64], dummy2[LSTRING_MAX];
 char *str2, *str, *dytimestring;

 if (gDDCanvas == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;

 if (Dy->refplot_indelebile == OFF) return;

 DyDebugPrint(__FUNCTION__,0);
 DyDebugTimeNeeded(__FUNCTION__,"start",3);

 numPicture = 0;

 TIter  next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {
         TH1 *href = (TH1*)obj;
         str2 = (char *)strstr(href->GetName(),REF_SUFFIX);
         if (str2 == NULL) continue;
         str = (char *)strstr(href->GetName(),SEPTITLE);

         /*--- If a 2-dimensional reference plot but not 2D, we do not draw it ---*/
         strcpy(dummy2, href->GetTitle());
         if (strstr(dummy2,"__RAW")!=0 || strstr(dummy2,"__FFTTIME")!=0 || strstr(dummy2,"__1DTIME")!=0 ||
             strstr(dummy2,"__RAWTIME")!=0 || strstr(dummy2,"__TRFCTTIME")!=0 || strstr(dummy2,"__COHETIME")!=0)
            {
             int nx=href->GetNbinsX();
             int ny=href->GetNbinsY();
             if (nx > 1 && ny > 1) continue;
            }

         strncpy(filename, str+strlen(SEPTITLE), str2-str-strlen(SEPTITLE));
         filename[str2-str-strlen(SEPTITLE)] = '\0';
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);
         if (obj2)
            {if (obj2->InheritsFrom(VDyPlot2::Class()))
                {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             else
                {VDyPlot *hv = (VDyPlot *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }

             RA_SetActivePad(numPicture);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
           
             color = href->GetLineColor(); 
             sprintf(dummy,"DyRefString%s%s", SEPTITLE, filename);
             timepave = (TPaveText*)(gPad->FindObject(dummy));

             if (!timepave)
                {
                 timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
                 timepave->SetName(dummy);
                 timepave->AddText(href->GetTitle());
                 timepave->SetTextFont(DY_FONT_HELVETICA);
                 timepave->SetTextColor(color);
                 timepave->SetFillColor(0);
                 timepave->Draw("NDC");
                }
/**
             if (href->GetXaxis()->GetTimeFormat() && !timepave)
                {
                 dytimestring = (char *)malloc(256);
                 ok = RA_GetTimeString(dytimestring, href->GetXaxis()->GetTimeFormat());
                 if (ok)
                    {
                     timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
                     timepave->SetName(dummy);
                     timepave->AddText(dytimestring);
                     timepave->SetTextFont(DY_FONT_HELVETICA);
                     timepave->SetTextColor(color);
                     timepave->SetFillColor(0);
                     timepave->Draw("NDC");
                    }
                 free(dytimestring);
                }
**/

             if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
                {printf("RA_DrawReferencePlot> drawing plot %s=%s=pad%d  color=%d \n",
                         href->GetName(), href->GetTitle(), numPicture, href->GetLineColor());
                }
             if (obj2->InheritsFrom(VDyPlot2::Class())) href->Draw("same colz");
             else
                {if (href->GetNbinsX() > 16383) href->Draw("l same");
                 else                           href->Draw("h same");
                }
            }
         else
            {printf("RA_DrawReferencePlot> Could not find in which pad to put %s = %s  hname=%s\n", href->GetName(), href->GetTitle(), hname);}
        }
    }
 DyDebugTimeNeeded(__FUNCTION__,"stop",3);
 return;
}


/*------------------------------------------------- RA_GetReferencePlot ---*/
/* Superpose to the plots the ones from reference file */

int RA_GetReferencePlot (char *fileName, int color)
{
 int padmodified, padmodified2, nsuperposed;
 double value, valueref, diff, mindiff, maxdiff;
 int ok, hnum;
 char dummy[256], dummy2[256], *dytimestring, *str;
 TPaveText* timepave;
 TPaveText* timepave1;
 TPaveText* timepave_ref;
 char *strtype, *strtyperef;
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (fileName == NULL) return(0);

 TFile *fp = new TFile(fileName);

 if (fp->GetListOfKeys() == NULL) {printf("RA_GetReferencePlot> file %s not found\n", fileName); return(0);}

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             nsuperposed = 0;
             while ((obj2 = next()))
                {
                 if (obj2->InheritsFrom(TH1::Class()))
                    {
                     ((TPad *)obj)->cd();

                     TH1* h = (TH1 *)obj2;
                     TH1 *href = (TH1 *)(fp->Get(h->GetName()));
                     if (strstr(h->GetName(),REF_SUFFIX) != NULL) continue;

                     if (href == NULL)
                        {printf("RA_GetReferencePlot> Reference plot does not exist for plot %s=%s\n", h->GetName(), h->GetTitle());
                         continue;
                        }

                     strtype    = (char *)strstr(h->GetTitle(),SEPTITLE);
                     strtyperef = (char *)strstr(href->GetTitle(),SEPTITLE);

                     if (strcmp(strtype, strtyperef) != 0)
                        {printf("RA_GetReferencePlot> Not same type for h=%s=%s  and  href=%s=%s\n",h->GetName(), h->GetTitle(),href->GetName(), href->GetTitle());
                         continue;
                        }

                     href->SetDirectory(gROOT);
                     basename = gSystem->BaseName(fileName);
                     sprintf(dummy, "%s%s%s%s", h->GetName(), SEPTITLE, basename, REF_SUFFIX);
                     href->SetName(dummy);
                     href->SetLineColor(color);

                     nsuperposed++;

                     if (nsuperposed == 1)
                        {
                         timepave_ref = (TPaveText *)(fp->Get("DyTimeString"));
                         href->SetTitle(timepave_ref->GetLine(0)->GetTitle());
                         printf("RA_GetReferencePlot> href->name%s  href->title==%s\n", href->GetName(), href->GetTitle());
                         timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
                         sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                         timepave->SetName(dummy);
                         timepave->AddText(timepave_ref->GetLine(0)->GetTitle());
                         timepave->SetTextFont(DY_FONT_HELVETICA);
                         timepave->SetTextColor(color);
                         timepave->SetFillColor(0);
                         timepave->Draw("NDC");
                        }
/**
                     if (href->GetXaxis()->GetTimeFormat() && nsuperposed == 1)
                        {
                         dytimestring = (char *)malloc(256);
                         ok = RA_GetTimeString(dytimestring, href->GetXaxis()->GetTimeFormat());
                         if (ok)
                            {
                             timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
                             sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                             timepave->SetName(dummy);
                             timepave->AddText(dytimestring);
                             timepave->SetTextFont(DY_FONT_HELVETICA);
                             timepave->SetTextColor(color);
                             timepave->SetFillColor(0);
                             timepave->Draw("NDC");
                            }
                          free(dytimestring);
                         }
**/

                     strcpy(dummy2, href->GetTitle());

                     /*--- If 2-dimensional plots but not 2D, we compute the difference between plot and reference plot ---*/
                     /*--- Then, we need to redraw the plot and to redraw the time string ---*/
                     if (strstr(dummy2,"__RAW")!=0 || strstr(dummy2,"__FFTTIME")!=0 || strstr(dummy2,"__1DTIME")!=0 ||
                         strstr(dummy2,"__RAWTIME")!=0 || strstr(dummy2,"__TRFCTTIME")!=0 || strstr(dummy2,"__COHETIME")!=0)
                        {
                         int nx=h->GetNbinsX();
                         int ny=h->GetNbinsY();

                         if (nx>1 && ny>1)
                            {
                             gPad->SetLogz(0);
                             for (int i=0;i<nx;i++)
                                {for (int j=0;j<ny;j++)
                                    {value    = h->GetBinContent(i+1,j+1);
                                     valueref = href->GetBinContent(i+1,j+1);
                                     diff = value - valueref;
                                     if (i==0 && j==0) {mindiff = diff; maxdiff = diff;}
                                     if (diff < mindiff) mindiff = diff;
                                     if (diff > maxdiff) maxdiff = diff;
                                     h->SetBinContent(i+1,j+1,diff);
                                    }
                                }

                             h->Draw("colz");
                             h->SetMinimum(mindiff);
                             h->SetMaximum(maxdiff);

                             if (nsuperposed == 1)
                               {
                                timepave_ref = (TPaveText *)(fp->Get("DyTimeString"));
                                href->SetTitle(timepave_ref->GetLine(0)->GetTitle());
                                timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
                                sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                                timepave->SetName(dummy);
                                timepave->AddText(timepave_ref->GetLine(0)->GetTitle());
                                timepave->SetTextFont(DY_FONT_HELVETICA);
                                timepave->SetTextColor(color);
                                timepave->SetFillColor(0);
                                timepave->Draw("NDC");
                               }
/**
                             if (h->GetXaxis()->GetTimeFormat() && nsuperposed == 1)
                                {
                                 dytimestring = (char *)malloc(256);
                                 ok = RA_GetTimeString(dytimestring, href->GetXaxis()->GetTimeFormat());
                                 if (ok)
                                    {
                                     timepave1 = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
                                     sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                                     timepave1->SetName(dummy);
                                     timepave1->AddText(dytimestring);
                                     if (Dy->nicefont == OFF) timepave1->SetTextFont(DY_FONT_TIMES);
                                     else                     timepave1->SetTextFont(DY_FONT_HELVETICA);
                                     timepave1->SetTextColor(color);
                                     timepave1->SetFillColor(0);
                                     timepave1->Draw("NDC");
                                    }
                                 free(dytimestring);
                                }

                             if (href->GetXaxis()->GetTimeFormat())
                                {if (timepave) timepave->Draw("NDC");}
**/
                            if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3)
                               {printf("RA_GetReferencePlot> FOUND href=%s=%s  (%d)\n", href->GetName(), href->GetTitle(), padmodified+1);}
                             padmodified++;
                             break;
                            }

                          else if (ny == 1)
                             {
                              if (href->GetNbinsX() > 16383) href->Draw("l same");
                              else                           href->Draw("h same");
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                             }
                         }

                      else
                         {
                          strcpy(dummy2, href->GetTitle());

                          if (strstr(dummy2,"__2D")!=0)
                             {
                              href->Draw("colz same");
/**
                              double ymin = h->GetYaxis()->GetXmin();
                              double ymax = h->GetYaxis()->GetXmax();
                              int bymin = h->GetYaxis()->FindBin(ymin);
                              int bymax = h->GetYaxis()->FindBin(ymax);
                              int firstYbin = h->GetYaxis()->GetFirst();
                              int lastYbin  = h->GetYaxis()->GetLast();
                              double xmin = h->GetXaxis()->GetXmin();
                              double xmax = h->GetXaxis()->GetXmax();
                              int bxmin = h->GetXaxis()->FindBin(xmin);
                              int bxmax = h->GetXaxis()->FindBin(xmax);
                              int firstXbin = h->GetXaxis()->GetFirst();
                              int lastXbin  = h->GetXaxis()->GetLast();

                              double yminref = href->GetYaxis()->GetXmin();
                              double ymaxref = href->GetYaxis()->GetXmax();
                              int byminref = href->GetYaxis()->FindBin(yminref);
                              int bymaxref = href->GetYaxis()->FindBin(ymaxref);
                              int firstYbinref = href->GetYaxis()->GetFirst();
                              int lastYbinref  = href->GetYaxis()->GetLast();
                              double xminref = href->GetXaxis()->GetXmin();
                              double xmaxref = href->GetXaxis()->GetXmax();
                              int bxminref = href->GetXaxis()->FindBin(xminref);
                              int bxmaxref = href->GetXaxis()->FindBin(xmaxref);
                              int firstXbinref = href->GetXaxis()->GetFirst();
                              int lastXbinref  = href->GetXaxis()->GetLast();
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3)
                                 {printf("\n");
                                  printf("RA_GetReferencePlot> plot=%s  xmin=%g xmax=%g bxmin=%d mxmax=%d firstXbin=%d lastXbin=%d \n",
                                          h->GetTitle(), xmin, xmax, bxmin, bxmax, firstXbin, lastXbin);
                                  printf("RA_GetReferencePlot> plot=%s  ymin=%g ymax=%g bymin=%d bymax=%d firstYbin=%d lastYbin=%d \n",
                                          h->GetTitle(), ymin, ymax, bymin, bymax, firstYbin, lastYbin);
                                  printf("RA_GetReferencePlot> plotref=%s  xminref=%g xmaxref=%g bxminref=%d bxmaxref=%d firstXbinref=%d lastXbinref=%d \n",
                                          href->GetTitle(), xminref, xmaxref, bxminref, bxmaxref, firstXbinref, lastXbinref);
                                  printf("RA_GetReferencePlot> plotref=%s  yminref=%g ymaxref=%g byminref=%d bymaxref=%d firstYbinref=%d lastYbinref=%d \n",
                                          href->GetTitle(), yminref, ymaxref, byminref, bymaxref, firstYbinref, lastYbinref);
                                 }
                              if (xminref < xmin) h->GetXaxis()->SetLimits(xminref,xmax);
                              if (xmaxref > xmax) h->GetXaxis()->SetLimits(xmin, xmaxref);
                              if (yminref < ymin) h->GetYaxis()->SetLimits(yminref, ymax);
                              if (ymaxref > ymax) h->GetYaxis()->SetLimits(ymin, ymaxref);
**/
                             }

                          else if (strstr(dummy2,"__1D")!=0)
                             {
                              if (href->GetNbinsX() > 16383) href->Draw("l same");
                              else                           href->Draw("h same");
/**
                              double xmin = h->GetXaxis()->GetXmin();
                              double xmax = h->GetXaxis()->GetXmax();
                              double xminref = href->GetXaxis()->GetXmin();
                              double xmaxref = href->GetXaxis()->GetXmax();
                              if (xminref < xmin) h->GetXaxis()->SetLimits(xminref,xmax);
                              if (xmaxref > xmax) h->GetXaxis()->SetLimits(xmin, xmaxref);
**/
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                             }

                          else
                             {
                              if (href->GetNbinsX() > 16383) href->Draw("l same");
                              else                           href->Draw("h same");
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                             }
                         }

                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3)
                         {printf("RA_GetReferencePlot> FOUND href=%s=%s  (%d)\n", href->GetName(), href->GetTitle(), padmodified+1);}
                      padmodified++;
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      nsuperposed = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              ((TPad *)obj2)->cd();
                              VDyPlot* h = (VDyPlot *)obj3;
                              VDyPlot *href = (VDyPlot *)(fp->Get(h->GetName()));

                              if (href == NULL)
                                 {printf("RA_GetReferencePlot> References plot does not exist for %s=%s\n", h->GetName(), h->GetTitle());
                                  continue;
                                 }

                              if (strcmp(h->GetTitle(), href->GetTitle()) != 0)
                                 {printf("RA_GetReferencePlot> Not same types for h=%s=%s  and  href=%s=%s\n",h->GetName(), h->GetTitle(),href->GetName(), href->GetTitle());
                                  continue;
                                 }

                              href->SetDirectory(gROOT);
                              basename = gSystem->BaseName(fileName);
                              sprintf(dummy, "%s%s%s%s", h->GetName(), SEPTITLE, basename, REF_SUFFIX);
                              href->SetName(dummy);
                              href->SetLineColor(color);

                              strcpy(dummy2, href->GetName());
                              hnum = atoi(dummy2+1);

                              if (hnum>=1000) nsuperposed++;

                              if (hnum>1000 && nsuperposed == 1)
                                 {
                                  timepave_ref = (TPaveText *)(fp->Get("DyTimeString"));
                                  href->SetTitle(timepave_ref->GetLine(0)->GetTitle());
                                  timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
                                  sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                                  timepave->SetName(dummy);
                                  timepave->AddText(timepave_ref->GetLine(0)->GetTitle());
                                  timepave->SetTextFont(DY_FONT_HELVETICA);
                                  timepave->SetTextColor(color);
                                  timepave->SetFillColor(0);
                                  timepave->Draw("NDC");
                                 }
/**
                              if (href->GetXaxis()->GetTimeFormat() && hnum>=1000 && nsuperposed == 1)
                                 {
                                  dytimestring = (char *)malloc(256);
                                  ok = RA_GetTimeString(dytimestring, href->GetXaxis()->GetTimeFormat());
                                  if (ok)
                                     {
                                      timepave = new TPaveText(0.02, 0.06, 0.98, 0.12,"bNDC");
                                      sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                                      timepave->SetName(dummy);
                                      timepave->AddText(dytimestring);
                                      if (Dy->nicefont == OFF) timepave->SetTextFont(DY_FONT_TIMES);
                                      else                     timepave->SetTextFont(DY_FONT_HELVETICA);
                                      timepave->SetTextColor(color);
                                      timepave->SetFillColor(0);
                                      timepave->Draw("NDC");
                                     }
                                  free(dytimestring);
                                  }
**/
                              if (href->GetNbinsX() > 16383) href->Draw("l same");
                              else                           href->Draw("h same");
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3)
                                 {printf("RA_GetReferencePlot> FOUND href=%s=%s  (%d)\n", href->GetName(), href->GetTitle(), padmodified+padmodified2+1);}
                              padmodified2++;
                             }
                         }
                     if (padmodified2 > 0) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                     }
                 }
             if (padmodified > 0){((TPad *)obj)->Modified(); ((TPad *)obj)->Update();}
            }
        }
    }  /* end of if FindObject(gDDCanvas) */

 else
    {padmodified = RA_ReadReferencePlot(fileName,color);}

 fp->Close();

 printf("RA_GetReferencePlot> Current plots in gROOT are:\n");
 gROOT->ls();

 return(padmodified+padmodified2);
}


/*------------------------------------------------- RA_GetTimeString ---*/
/* Get the GPS time string from the plot time format */

int RA_GetTimeString (char *dytimestring, const char *timeformat)
{
 char dummy[256], *str;
 double gtime;

 gtime = DyTimeFormatToGps(timeformat);

 if (gtime == 0.) return(0);

 DyGpsToTimeString ((int)gtime, NULL, 0, 0, dytimestring);

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3)
    {printf("RA_GetTimeString> Reference plot timeformat=%s  GPS=%.4f  dytimestring=%s\n",
            timeformat, gtime, dytimestring);
    }

 return(1);
}


/*------------------------------------------------- RA_ScanPlots ---*/
/* Change to time axis area visualized in the plot */

void RA_ScanPlots (double percentage)
{
 int padmodified, padmodified2;
 double value, plotymin, plotymax;
 TH1* horig;
 char *str, mPlotType[64], gLastPlotType[64];
 int nbintot, binmin_vis, binmax_vis, binmin, binmax, newbinmin, newbinmax;
 double xmin, xmax, deltax, xmin2, xmax2, xmin_vis, xmax_vis, deltax_vis, newxmin_vis, newxmax_vis;

// gPad = (gPad->GetCanvas())->GetSelectedPad();

 if (gLastModifiedPlot == NULL && gLastModifiedPlot2 == NULL) return;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;
             horig = NULL;

             while ((obj2 = next()))
                {
                 if (obj2->InheritsFrom(TH1::Class()))
                    {
                     ((TPad *)obj)->cd();

                       TH1 *h   = (TH1 *)obj2;
                       str = (char *)strstr(h->GetTitle(),SEPTITLE);
                       if (str) strcpy(mPlotType,&str[0]+2);
                       else     strcpy(mPlotType,"NULL");
                       str = (char *)strstr(Dy->gLastTitle,SEPTITLE);
                       if (str) strcpy(gLastPlotType,&str[0]+2);
                       else     strcpy(gLastPlotType,"NULL");

                     if (strcmp(mPlotType, gLastPlotType) == 0 ||
                         (strstr(mPlotType,"TIME") != NULL && strstr(gLastPlotType,"TIME") != NULL) ||
                         (strcmp(mPlotType,"FFT") == 0 && strcmp(gLastPlotType,"COHE") == 0) ||
                         (strcmp(mPlotType,"COHE") == 0 && strcmp(gLastPlotType,"FFT") == 0) ||
                         (strcmp(mPlotType,"FFT") == 0 && strcmp(gLastPlotType,"TRFCT") == 0) ||
                         (strcmp(mPlotType,"COHE") == 0 && strcmp(gLastPlotType,"TRFCT") == 0))
                        {
                         nbintot = h->GetNbinsX();
                         xmin    = h->GetXaxis()->GetBinLowEdge(0);
                         xmax    = h->GetXaxis()->GetBinUpEdge(nbintot);
                         deltax  = xmax - xmin;

                         xmin2   = h->GetXaxis()->GetXmin();
                         xmax2   = h->GetXaxis()->GetXmax();

                         xmin_vis = gPad->GetUxmin();                  
                         xmax_vis = gPad->GetUxmax();                  
                         if (((TPad *)obj)->GetLogx() > 0)
                            {xmin_vis = pow(10,xmin_vis);
                             xmax_vis = pow(10,xmax_vis);
                            }
                         deltax_vis  = xmax_vis - xmin_vis;
                         binmin_vis  = h->GetXaxis()->GetFirst();
                         binmax_vis  = h->GetXaxis()->GetLast();
                         binmin      = h->GetXaxis()->FindBin(xmin_vis);
                         binmax      = h->GetXaxis()->FindBin(xmax_vis);

                         DyXformSetScanStep(0.2*deltax_vis/deltax);

                         newxmin_vis = percentage * (xmax-xmin-deltax_vis) + xmin;
                         newxmax_vis = newxmin_vis + deltax_vis; 
                         newbinmin   = h->GetXaxis()->FindBin(newxmin_vis);
                         newbinmax   = h->GetXaxis()->FindBin(newxmax_vis);
//                         printf("RA_ScanPlots> plot=%s  percentage=%g  nbintot=%d  xmin=%g  xmax=%g  xmin2=%g  xmax2=%g  \n",
//                                 h->GetName(), percentage, nbintot, xmin, xmax, xmin2, xmax2);
//                         printf("RA_ScanPlots> plot=%s  xmin_vis=%g  xmax_vis=%g  binmin=%d (%d)  binmax=%d (%d)  ||  newxmin_vis=%g  newxmax_vis=%g  newbinmin=%d  newbinmax=%d \n",
//                                 h->GetName(), xmin_vis, xmax_vis, binmin, binmin_vis, binmax, binmax_vis, newxmin_vis, newxmax_vis, newbinmin, newbinmax);
                         if (xmin_vis == xmin2 && xmax_vis == xmax2) continue;
//                         if (newxmin_vis <= xmin2 || newxmax_vis >= xmax2) continue;
                         if (newbinmin < 0) newbinmin = 0;
                         if (newbinmax > nbintot-1) newbinmax = nbintot-1;

//                         binmin_vis  = h->GetXaxis()->GetFirst();
//                         binmax_vis  = h->GetXaxis()->GetLast();
//                         xmin_vis2   = h->GetXaxis()->GetBinUpEdge(binmin_vis);
//                         xmax_vis2   = h->GetXaxis()->GetBinUpEdge(binmax_vis);
//                         newbinmin2  = (int)(newxmin_vis * nbintot / deltax);
//                         newbinmax2  = (int)(newxmax_vis * nbintot / deltax);

                         if (strcmp(h->IsA()->GetName(),"VDyPlot2") != 0)
                            {if (horig == NULL)
                                {horig = h;
                                 plotymin = h->GetBinContent(newbinmin);
                                 plotymax = h->GetBinContent(newbinmin); 
                                }
                             for (int i=newbinmin;i<=newbinmax;i++)
                                {value = h->GetBinContent(i);
                                 if (value > plotymax) plotymax = value;
                                 if (value<= 0. && ((TPad *)obj)->GetLogy() > 0) continue;
                                 if (value < plotymin ) plotymin = value;
                                }
                            }

                         h->GetXaxis()->SetRange(newbinmin, newbinmax);
                         padmodified++;
                        }
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(TH1::Class()))
                             {
                              ((TPad *)obj2)->cd();
                              TH1* h = (TH1 *)obj3;
                              str = (char *)strstr(h->GetTitle(),SEPTITLE);
                              if (str) strcpy(mPlotType,&str[0]+2);
                              else     strcpy(mPlotType,"NULL");
                              str = strstr(Dy->gLastTitle,SEPTITLE);
                              if (str) strcpy(gLastPlotType,&str[0]+2);
                              else     strcpy(gLastPlotType,"NULL");

                              if (strcmp(mPlotType, gLastPlotType) == 0 ||
                                  (strcmp(mPlotType,"TRFCT") == 0 && strcmp(gLastPlotType,"COHE") == 0) ||
                                  (strcmp(mPlotType,"TRFCT") == 0 && strcmp(gLastPlotType,"FFT") == 0))
                                 {
                                  nbintot = h->GetNbinsX();
                                  xmin    = h->GetXaxis()->GetBinLowEdge(0);
                                  xmax    = h->GetXaxis()->GetBinUpEdge(nbintot);
                                  deltax  = xmax - xmin;

                                  xmin2   = h->GetXaxis()->GetXmin();
                                  xmax2   = h->GetXaxis()->GetXmax();

                                  xmin_vis = gPad->GetUxmin();                  
                                  xmax_vis = gPad->GetUxmax();                  
                                  if (((TPad *)obj2)->GetLogx() > 0)
                                     {xmin_vis = pow(10,xmin_vis);
                                      xmax_vis = pow(10,xmax_vis);
                                     }
                                  deltax_vis = xmax_vis - xmin_vis;
                                  binmin_vis = h->GetXaxis()->GetFirst();
                                  binmax_vis = h->GetXaxis()->GetLast();
                                  binmin     = h->GetXaxis()->FindBin(xmin_vis);
                                  binmax     = h->GetXaxis()->FindBin(xmax_vis);

                                  DyXformSetScanStep(0.2*deltax_vis/deltax);

                                  newxmin_vis = percentage * (xmax-xmin-deltax_vis) + xmin;
                                  newxmax_vis = newxmin_vis + deltax_vis; 
                                  newbinmin   = h->GetXaxis()->FindBin(newxmin_vis);
                                  newbinmax   = h->GetXaxis()->FindBin(newxmax_vis);

                                  if (xmin_vis == xmin2 && xmax_vis == xmax2) continue;
                                  if (newbinmin < 0) newbinmin = 0;
                                  if (newbinmax > nbintot-1) newbinmax = nbintot-1;
                                  if (strcmp(h->IsA()->GetName(),"VDyPlot2") != 0)
                                     {if (horig == NULL)
                                         {horig = h;
                                          plotymin = h->GetBinContent(newbinmin);
                                          plotymax = h->GetBinContent(newbinmin); 
                                         }
                                      for (int i=newbinmin;i<=newbinmax;i++)
                                         {value = h->GetBinContent(i);
                                          if (value > plotymax) plotymax = value;
                                          if (value<= 0. && ((TPad *)obj2)->GetLogy() > 0) continue;
                                          if (value < plotymin ) plotymin = value;
                                         }
                                     }
//                               printf("%s new range set: (%d %d) --> (%d %d) \n", h->GetName(), binmin, binmax, newbinmin, newbinmax);
                                  h->GetXaxis()->SetRange(newbinmin, newbinmax);
                                  padmodified2++;
                                 }
                             }
                         }  /* end of while on obj3 */
                     if (padmodified2 > 0) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                    } /* end of if on obj2 */
                } /* end of while on obj2 */

             if (padmodified > 0)
                {
/**
                 if (horig && strcmp(horig->IsA()->GetName(),"VDyPlot2") != 0)
                    {if (((TPad *)obj)->GetLogy() == 0) plotymin = plotymin - 0.2 * (plotymax - plotymin);
                     plotymax = plotymax + 0.2 * (plotymax - plotymin);
                     horig->SetMinimum(plotymin);
                     horig->SetMaximum(plotymax);
                    }
**/
                 ((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            } /* end of if on obj */
        } /* end of while on obj */
    } /* end of if gDDCanvas */

 return;
}


//_____________________________________________________________________________
void RA_NtupleNew(char* name,char* title,int nvar,char* tags)
{
// 
}

//_____________________________________________________________________________
void RA_NtupleFill(char* name,float* x)
{
//
}

//_____________________________________________________________________________
void RA_SetStyle(char* options)
{
// Sets the global style gStyle when needed and the pad options

   DyDebugPrint(__FUNCTION__,0);

   if (strstr(options,"LINX") != NULL)  gPad->SetLogx(0);
   if (strstr(options,"LOGX") != NULL)  gPad->SetLogx(1);
   if (strstr(options,"LINY") != NULL)  gPad->SetLogy(0);
   if (strstr(options,"LOGY") != NULL)  gPad->SetLogy(1);
   if (strstr(options,"LINZ") != NULL)  gPad->SetLogz(0);
   if (strstr(options,"LOGZ") != NULL)  gPad->SetLogz(1);
/*
   if (strstr(options,"STA ") != NULL)  gStyle->SetOptStat(1);
   if (strstr(options,"NSTA") != NULL)  gStyle->SetOptStat(0);
   if (strstr(options,"GRID") != NULL) {gPad->SetGridx(1);gPad->SetGridy(1);}
*/
   if (strstr(options,"NGRI") != NULL) {gPad->SetGridx(0);gPad->SetGridy(0);}
   if (strstr(options,"GRIDX") != NULL) gPad->SetGridx(1);
   if (strstr(options,"GRIDY") != NULL) gPad->SetGridy(1);
   if (strstr(options,"UTIT") != NULL)  gStyle->SetOptTitle(0);
   if (strstr(options,"HTIT") != NULL)  gStyle->SetOptTitle(1);
}


//_____________________________________________________________________________
void RA_HUpdateEnd(char *plotName, char *options)
{
 if (!gPad)     return;
 if (!plotName) return;

 DyDebugPrint(__FUNCTION__,0);

 TH1* h=(TH1*)gDirectory->Get(plotName);
 if (!h) return;

 TString optbis = options;

 if (h->GetNbinsX()<20480)
    {optbis = "h" + optbis; h->SetDrawOption(optbis.Data());}
 else
    {optbis = "l9" + optbis; h->SetDrawOption(optbis.Data());}

 h->SetLabelFont(DY_FONT_HELVETICA,"X");
 h->SetLabelFont(DY_FONT_HELVETICA,"Y");
/**
 if (Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
    {h->SetLabelFont(DY_FONT_HELVETICA,"X");
     h->SetLabelFont(DY_FONT_HELVETICA,"Y");
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
        {printf("RA_HUpdateEnd> plot=%s=%s : Set Label Font to 62\n", h->GetTitle(), h->GetName());}
    }
**/

}


//_____________________________________________________________________________
int RA_HUpdate (long numPicture, char *name, char *title, char *options)
{
// Update canvas if the pad contains the plot which updates the fastest
 int update_done, i, numplot;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 update_done = 0;

 sscanf(name, "h%d", &numplot);
 if (Dy->plotReady[numplot] == 0) {Dy->plotReady[numplot] = 1; Dy->nready++;}

/**
 if (strstr(options,"SAME") != NULL && Dy->plotReady[numplot] == 0)
    {Dy->plotReady[numplot] = 1; Dy->nready++;}
**/

/***
 if (strstr(options,"SAME") == NULL)
    {for (i=0; i<Dy->nplot; i++)
        {if (Dy->refnumplot[i] = numplot && Dy->plotReady[i] == 1) {Dy->plotReady[i] = 0; Dy->nready--;}}
    }
In some cases, the previous lines of code have a bad consequence: plots are never shown.
The problem of refresh synchronization should be transfered to DyPlot: find a ppcm among refreshPeriods that
guarantees that a Draw of a superposed plot is always done after the Draw of the reference plot on which it is superposed
***/

 RA_SetActivePad(numPicture);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("RA_HUpdate> plot%ld=%s=%s  numplot=%d  plotReady[%d]=%d  nready=%d  nsame=%d  nplot=%d \n",
        numPicture, name, title, numplot, numplot, Dy->plotReady[numplot],
        Dy->nready, Dy->nsame, Dy->nplot);
    }

/**
 if ((gPad == gLastPad && Dy->nsame == 0) || (Dy->nready == Dy->nsame && Dy->nsame > 0))
**/
 if ((gPad == gLastPad && Dy->nsame == 0) || (Dy->nready == Dy->nplot && Dy->nsame > 0))
    {
     DyDebugTimeNeeded("All Plots ready to be paint","stop",0);
     DyDebugTimeNeeded(__FUNCTION__,"start",2);

     if (((Dy->debugType)&(1<<DBG_URPLOT)))
        {sprintf(dummy,"Canvas Update made at plot %s=%s of pad%d", name, title, numPicture);
         DyInfoLine(dummy);
        }
     Dy->nready = 0;
     for (i=0; i<=Dy->nplot; i++) Dy->plotReady[i] = 0;
     RA_DrawReferencePlot();

     if (RA_SystemCheckSwap() && gDDCanvas) gDDCanvas->Update();
     update_done = 1;

     DyDebugTimeNeeded(__FUNCTION__,"stop",2);
     DyDebugTimeNeeded("All Plots ready to be paint","start",0);
    }

 return(update_done);
}



//_____________________________________________________________________________
void RA_HDraw(char* name, char* options, int nsame)
{
// Draws the plot named "name" in the current pad

 DyDebugPrint(__FUNCTION__,0);
   
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HDraw> no plot %s \n", name); return;}

 DyDebugTimeNeeded(__FUNCTION__,"start",3);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 TString optbis=options;
 TAxis* ax = h->GetXaxis();
 
 if (optbis.Contains("TIMX"))
    {ax->SetTimeDisplay(1);
     optbis.Remove(optbis.Index("TIMX"),4);
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>10) printf("RA_HDraw> Set Time display on X axis for plot %s\n", h->GetName());
    }

// If plots superposed, we do not show first plot title
 if (nsame > 0) h->SetBit(TH1::kNoTitle);

// If VDyPlot, take care of number of points displayed to choose the line option
 if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)
    {
     if ((unsigned int)(ax->GetLast()-ax->GetFirst())<3*gPad->GetWw()) {optbis = "h" + optbis;}
     else                                                              {optbis = "l" + optbis;}
     ((VDyPlot*)h)->Draw(optbis.Data());
/**
     ((VDyPlot*)h)->Draw("");
**/
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
        {printf("RA_HDraw> Drawing plot=%s=%s=pad%d  color=%d  option=%s\n", h->GetName(), h->GetTitle(), ((VDyPlot *)h)->numpad, h->GetLineColor(), options);}
    }

// If VDyPlot2, take care of right margin for the color palette
 else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0)
    {gPad->SetRightMargin(0.15);
     ((VDyPlot2*)h)->Draw(options);
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
        {printf("RA_HDraw> Drawing plot=%s=%s=pad%d  color=%d  option=%s\n", h->GetName(), h->GetTitle(), ((VDyPlot2 *)h)->numpad, h->GetLineColor(), options);}
    }

// If TH2F, take care of right margin for the color palette
 else if (strcmp(h->IsA()->GetName(),"TH2F") == 0)
    {gPad->SetRightMargin(0.15);
     ((TH2F*)h)->Draw(options);
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
        {printf("RA_HDraw> Drawing plot=%s=%s  color=%d  option=%s\n", h->GetName(), h->GetTitle(), h->GetLineColor(), options);}
    }

// Sets or removes the stat box if we don't want to see it after stop
// if (!gStyle->GetOptStat()) h->SetStats(0); else h->SetStats(1);

 oldsty->cd();

 DyDebugTimeNeeded(__FUNCTION__,"stop",3);
}


//_____________________________________________________________________________
void RA_HSetXRange(char* name, float fmin, float fmax, float df)
{
// Set the X axis range of the 1D histogram named "name".
 int xmin, xmax;

 TH1* h = (TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetXRange> no plot %s\n",name); return;}

 if (df > 0)
    {xmin = (int)(fmin / df);
     xmax = (int)(fmax / df);
    }
 else
    {xmin = (int)fmin;
     xmax = (int)fmax;
    }

 TAxis* ax = h->GetXaxis();
 ax->SetRange(xmin, xmax);
}


//_____________________________________________________________________________
void RA_HSetYRange(char* name, float fmin, float fmax, float df)
{
// Set the Y axis range of the 2D histogram named "name".
 int ymin, ymax;

 TH1* h = (TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetYRange> no plot %s\n",name); return;}

 if (df > 0)
    {ymin = (int)(fmin / df);
     ymax = (int)(fmax / df);
    }
 else
    {ymin = (int)fmin;
     ymax = (int)fmax;
    }

 TAxis* ay = h->GetYaxis();
 ay->SetRange(ymin, ymax);
}


//_____________________________________________________________________________
void RA_HReset(char* name)
{
// Resets the histogram named "name"
  
 DyDebugPrint(__FUNCTION__,0);
 
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HReset> no plot %s\n",name); return;}

 if (!strcmp(h->IsA()->GetName(),"VDyPlot"))       ((VDyPlot*)h)->Reset();
 else if (!strcmp(h->IsA()->GetName(),"TH2F"))     ((TH2F*)h)->Reset();
 else if (!strcmp(h->IsA()->GetName(),"VDyPlot2")) ((VDyPlot2*)h)->Reset();
}


//_____________________________________________________________________________
void RA_H1Fill(char* name, float x , float weight)
{
// Fills the 1D histogram names "name"

 static int counter=0;

 VDyPlot* h=(VDyPlot*)gDirectory->Get(name);
 if (!h && counter == 0) {printf("RA_H1Fill> no plot %s\n",name); return;}
 counter++;
 if (counter > 100000) counter = 0;

 h->Fill(x,weight);
}


//_____________________________________________________________________________
void RA_H2Fill(char* name, float x, float y, float weight)
{
// Fills the 2D histogram names "name"

 static int counter=0;

 TH2F* h=(TH2F*)gDirectory->Get(name);
 if (!h && counter == 0) {printf("RA_H2Fill> no plot %s\n",name); return;}
 counter++;
 if (counter > 100000) counter = 0;

 h->Fill(x,y,weight);
}


//_____________________________________________________________________________
void RA_HDelete(char* name)
{
// Deletes the histogram named "name"

 DyDebugPrint(__FUNCTION__,0);

 if (!name) {gDirectory->Delete("*"); return;}

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HDelete> no plot %s\n",name); return;}

 if (!strcmp(h->IsA()->GetName(),"VDyPlot"))       delete ((VDyPlot*)h);
 else if (!strcmp(h->IsA()->GetName(),"TH2F"))     delete ((TH2F*)h);
 else if (!strcmp(h->IsA()->GetName(),"VDyPlot2")) delete ((VDyPlot2*)h);
}


// obsolete
//_____________________________________________________________________________
void RA_H1New(char* name,char* title,int nbins, float xmin, float xmax)
{
// Creates a new 1D histogram

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();
 VDyPlot* h = new VDyPlot(name,title,nbins,xmin,xmax);
 h->UnlinkTimePave();
 oldsty->cd();
}


//_____________________________________________________________________________
void RA_H1NewReuse(int pad, int subpad, char* name, char* title, int nbins, float xmin, float xmax)
{
// Reuses an already existing histogram in pad/subpad
// or if it is not good, creates a new 1D histogram

 char* oldname;

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
    {printf("RA_H1NewReuse> calling GoodHist with pad=%d subpad=%d nbins=%d name=%s\n", pad, subpad, nbins, name);}

 oldname = RA_PadContainsGoodHist (pad, subpad, 1, nbins, 0, name);

 if (!oldname)
    {
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
        {printf("RA_H1NewReuse> oldname=NULL : new plot created in pad=%d subpad=%d: %s=%s\n", pad, subpad, name, title);}
     VDyPlot *h0 = new VDyPlot(name,title,nbins,xmin,xmax);
     h0->numpad    = pad;
     h0->numsubpad = subpad;
     return;
    }

 else
    {
     VDyPlot* h = (VDyPlot*)gDirectory->Get(oldname);
     if (!h)
        {
         if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
            {printf("RA_H1NewReuse> oldname=%s but plot does not exist: new plot created in pad=%d subpad=%d: %s=%s\n", oldname, pad, subpad, name, title);}
         VDyPlot *h0 = new VDyPlot(name,title,nbins,xmin,xmax);
         h0->numpad    = pad;
         h0->numsubpad = subpad;
        }
     else
        {if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
            {printf("RA_H1NewReuse> Plot reused in pad=%d subpad=%d: %s=%s\n", pad, subpad, name, title);}
         h->UnlinkTimePave();
         h->SetName(name);
         h->SetTitle(title);
         h->GetXaxis()->SetLimits(xmin,xmax);
         h->Reset();
         h->numpad    = pad;
         h->numsubpad = subpad;
        }
    }

 oldsty->cd();
}


//_____________________________________________________________________________
void RA_HSetMinimum(char* name, float ymin)
{
// Sets the minimum for histo "name"
   
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetMinimum> no plot %s\n",name); return;}

 h->SetMinimum(ymin);
}


//_____________________________________________________________________________
void RA_HSetMaximum(char* name, float ymax)
{
// Sets the maximum for histo "name"
   
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetMaximum> no plot %s\n",name); return;}

 h->SetMaximum(ymax);
}


//_____________________________________________________________________________
void RA_HSetTitle(char* name, char* title)
{
// Sets the title for histo "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTitle> no plot %s title=%s\n",name, title); return;}

 if (title) h->SetTitle(title);
 else   h->SetBit(TH1::kNoTitle);
}


//_____________________________________________________________________________
void RA_HSetUnitsX(char* name, char* units)
{
// Sets the units of horizontal axis for histo "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetUnitsX> no plot %s\n",name); return;}

 h->GetXaxis()->SetTitle(units);
 h->GetXaxis()->SetTitleOffset(0.9);
}


//_____________________________________________________________________________
void RA_HSetUnitsY(char* name, char* units)
{
// Sets the units of vertical axis for histo "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetUnitsY> no plot %s\n",name); return;}

 h->GetYaxis()->SetTitle(units);
 h->GetYaxis()->SetTitleOffset(1.63);
}


//_____________________________________________________________________________
void RA_HSetUnitsZ(char* name, char* units)
{
// Sets the units of vertical axis for histo "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTitle> no histogram %s\n",name); return;}

 h->GetZaxis()->SetTitle(units);
// h->GetZaxis()->RotateTitle(kTRUE);
// h->GetZaxis()->SetTitleOffset(1.63);
}


//_____________________________________________________________________________
void RA_HSetTimeString(char* name, char* timestring)
{
// Sets the time string displayed below the plot "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTimeString> no plot %s\n",name); return;}

 if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)       ((VDyPlot*)h)->SetTimeString(timestring);
 else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0) ((VDyPlot2*)h)->SetTimeString(timestring);
 else printf("RA_HSetTimeString> NO TIME STRING for plot %s=%s\n", h->GetName(), h->GetTitle());
}

//_____________________________________________________________________________
void RA_HSetTimeFlag(char* name, int flag)
{
// Sets the flag that allows drawing the time string

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTimeFlag> no plot %s\n",name); return;}

 if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)       ((VDyPlot*)h)->SetTimeFlag(flag); 
 else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0) ((VDyPlot2*)h)->SetTimeFlag(flag);
 else printf("RA_HSetTimeString> NO TIME FLAG for plot %s=%s\n", h->GetName(), h->GetTitle());
}

//_____________________________________________________________________________
void RA_HSetTimeOffset(char* name, double offset)
{
// Sets the offset date for the time axis
 time_t tt;

 DyDebugPrint(__FUNCTION__,0);

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTimeOffset> no plot %s\n",name); return;}

 if (offset > 0)
    {
     if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)
        {((VDyPlot*)h)->SetTimeUTC(offset);}  /* keep and use date later when zooming */
     else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0)
        {((VDyPlot2*)h)->SetTimeUTC(offset);}  /* keep and use date later when zooming */

#if ROOT_VERSION_CODE < ROOT_VERSION(5,14,0)
     /*--- offset is a C time, which is GPS time + 315964800 ---*/
     /*    315964800 = GPS origin - C time origin (Jan  6 00:00:00 1980 - Jan 1 00:00:00 1970)    */
     /* In root/v3r10p1 we need to convert UTC time into a localtime before calling SetTimeOffset */
     /* This is not needed anymore after root/v5r14p0.                                            */
     tt = (time_t)offset;
     if ((localtime(&tt))->tm_isdst) offset += 3600.;  /* Summer time is before 26 October 2003 */
//     printf("RA_HSetTimeOffset> tt=%d  utc=%s  local-gmt=%g  offset=%g  daylight_local=%d  daylight_gmt=%d \n",
//             tt,  asctime(gmtime(&tt)), (double)(mktime(localtime(&tt)) - mktime(gmtime(&tt))), offset,
//             (localtime(&tt))->tm_isdst, (gmtime(&tt))->tm_isdst);
#endif
    }

/*
 h->GetXaxis()->SetTimeDisplay(1);
*/
 h->GetXaxis()->SetTimeOffset(offset,"gmt");
}


//_____________________________________________________________________________
void RA_SetAxisNdiv(char* name, int ndiv)
{
// Sets the X axis ndivisions
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_SetAxisNdiv> no plot %s\n",name); return;}
 h->GetXaxis()->SetNdivisions(ndiv,"X");
}


//_____________________________________________________________________________
void RA_SetLimitsX(char* name, double xmin, double xmax)
{
// Sets the X axis limits (xmin and xmax) of a plot
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_SetLimitsX> no plot %s\n",name); return;}
 h->GetXaxis()->SetLimits(xmin,xmax);
}


//_____________________________________________________________________________
void RA_SetLimitsY(char* name, double ymin, double ymax)
{
// Sets the Y axis limits (ymin and ymax) of a plot
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_SetLimitsY> no plot %s\n",name); return;}
 h->GetYaxis()->SetLimits(ymin,ymax);
}


// obsolete
//_____________________________________________________________________________
void RA_H2New(char* name, char* title, int nbinx, float xmin, float xmax, int nbiny, float ymin, float ymax)
{
// Creates a new 2D histogram

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 VDyPlot2* h = new VDyPlot2(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);
// TH2F* h = new TH2F(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);

// Sets the kCanDelete bit so that any drawing
// replacing this plot will delete it
 h->SetBit(kCanDelete);

 oldsty->cd();
}


//_____________________________________________________________________________
void RA_H2NewReuse(int pad, int subpad, char* name, char* title, int nbinx, float xmin, float xmax, int nbiny, float ymin, float ymax)
{
// Reuses an already existing histogram in pad/subpad
// or if it is not good, creates a new 2D histogram

 char* oldname;

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
    {printf("RA_H2NewReuse> calling GoodHist with pad=%d  subpad=%d  nbinx=%d  nbiny=%d  name=%s\n", pad, subpad, nbinx, nbiny, name);}

 oldname = RA_PadContainsGoodHist(pad, subpad, 2, nbinx, nbiny, name);

 if (!oldname)
    {
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
        {printf("RA_H2NewReuse> oldname=NULL : new plot created in pad=%d subpad=%d: %s=%s\n", pad, subpad, name, title);}
     VDyPlot2* h0 = new VDyPlot2(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);
     h0->SetBit(kCanDelete);   // Sets the kCanDelete bit so that any drawing replacing this plot will delete it
     h0->numpad    = pad;
     h0->numsubpad = subpad;
     return;
    }
 else
    {
     VDyPlot2* h = (VDyPlot2*)gDirectory->Get(oldname);
     if (!h)
        {if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
            {printf("RA_H2NewReuse> oldname=%s but plot does not exist: new plot created in pad=%d subpad=%d: %s=%s\n", oldname, pad, subpad, name, title);}
         VDyPlot2 *h0 = new VDyPlot2(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);
         h0->numpad    = pad;
         h0->numsubpad = subpad;
        }
     else
        {if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
            {printf("RA_H2NewReuse> Plot reused in pad=%d subpad=%d: %s=%s\n", pad, subpad, name, title);}
         h->UnlinkTimePave();
         h->SetName(name);
         h->SetTitle(title);
         h->GetXaxis()->SetLimits(xmin,xmax);
         h->GetYaxis()->SetLimits(ymin,ymax);
         h->Reset();
         h->numpad    = pad;
         h->numsubpad = subpad;
        }
    }

 oldsty->cd();
}


// Not used anymore
//_____________________________________________________________________________
void RA_H1SetBinColor(char* name, int bin, int colorValue)
{
// Set a vertical colored line at bin of a 1D plot
 if (bin < 0) return;
 VDyPlot* h = (VDyPlot*)gDirectory->Get(name);
 if (!h) {printf("RA_H1SetBinColor> no plot %s\n",name); return;}
 h->SetIndicator(bin,colorValue);
// h->SetBinContent(bin,0);
}


//_____________________________________________________________________________
void RA_H1Pack(char* name, float* buff)
{
// Fills a 1D histogram with the contents of a buffer
 double lastValue;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = (VDyPlot*)gDirectory->Get(name);

 if (!h) {printf("RA_H1Pack> no plot %s\n",name); return;}

 /*--- 2=red=baddata, 3=green=nochannel, 4=blue=noframe, 17=gray ---*/

 int nx = h->GetNbinsX();

 lastValue = buff[0];
 if (lastValue >= DY_NOCHANNEL) lastValue = 0.;
 if (lastValue >= DY_NOFRAME)   lastValue = 0.;
 if (lastValue >= DY_BADDATA)   lastValue = 0.;

 for (int i=0;i<nx;i++)
    {
     if (buff[i] >= DY_NOCHANNEL)
        {h->SetIndicator(i+1,3);
         h->SetBinContent(i+1,lastValue);
        }

     else if (buff[i] >= DY_NOFRAME)
        {h->SetIndicator(i+1,4);
         h->SetBinContent(i+1,lastValue);
        }

     else if (buff[i] >= DY_BADDATA)
        {h->SetIndicator(i+1,2);
         h->SetBinContent(i+1,lastValue);
        }

     else
        {h->SetIndicator(i+1,0);
         h->SetBinContent(i+1,buff[i]);
         lastValue = buff[i];
        }
    }
}


//_____________________________________________________________________________
void RA_H2Pack(char* name, float* buff)
{
// Fills a 2D histogram with the contents of a buffer
   
 DyDebugPrint(__FUNCTION__,0);

 VDyPlot2* h = (VDyPlot2*)gDirectory->Get(name);

 if (!h) {printf("RA_H2Pack> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();
 int ny=h->GetNbinsY();

 for (int i=0;i<nx;i++)
    {for (int j=0;j<ny;j++) h->SetBinContent(i+1,j+1,buff[i+nx*j]);}
}


//_____________________________________________________________________________
void RA_H2PackBits(char* name, float* buff)
{
// Fills a 2D histogram with the bits of the contents of a buffer

 int bitValue, valueInt;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot2* h = (VDyPlot2*)gDirectory->Get(name);
 
 if (!h) {printf("RA_H2PackBits> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();

 for (int i=0;i<nx;i++)
    {
     valueInt = (int)buff[i];
     for (int j=0;j<32;j++)
        {bitValue = valueInt&(1<<j);
         if (bitValue != 0) h->SetBinContent(i+1,j+1,1.);
         else               h->SetBinContent(i+1,j+1,0.3);
        }
    }
}


//_____________________________________________________________________________
void RA_H1Unpack(char* name, float* buff)
{
// Fills a buffer with the contents of a 1D histo
   
 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = (VDyPlot*)gDirectory->Get(name);

 if (!h) {printf("RA_H1Unpack> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();

 for (int i=0;i<nx;i++) buff[i] = h->GetBinContent(i+1);
}


//_____________________________________________________________________________
void RA_H2Unpack(char* name, float* buff)
{
// Fills a buffer with the contents of a 2D histo

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot2* h = (VDyPlot2*)gDirectory->Get(name);

 if (!h) {printf("RA_H2Unpack> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();
 int ny=h->GetNbinsY();

 for (int i=0;i<nx;i++)
    {for (int j=0;j<ny;j++) buff[i+nx*j] = h->GetBinContent(i+1,j+1);}
}


//_____________________________________________________________________________
void RA_SetActivePad(int num)
{
// Sets the active pad as the number num
 static long lastNum = -1;
 long num2;

 if (num < 0) {lastNum = -1; return;}
 
 if (num > 0)  num2 = num;
 else          num2 = lastNum;
 lastNum = num2;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) && gPad)
    {gDDCanvas->cd(num2);
     gPad->SetFillColor(0);
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3) printf("SetActivePad> numpad=%d gPad=%x\n", num, (long)gPad);   
    }
}


//_____________________________________________________________________________
void RA_SetActiveSubPad(int num)
{
// Sets the active pad as the number num

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) && gPad)
    {gPad->cd(num);
     gPad->SetFillColor(0);
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3) printf("SetActiveSubPad> numsubpad=%d gPad=%x\n",num, (long)gPad);
    }
}


//_____________________________________________________________________________
void RA_PadDivide(int nx,int ny)
{
// Divides the current pad in nx.ny subpads

 DyDebugPrint(__FUNCTION__,0);

 if (!gPad->IsEditable()) return;

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();
 if (gPad) gPad->Divide(nx,ny);
 oldsty->cd();
}


//_____________________________________________________________________________
void RA_PadDivideWithTitle(int nx,int ny)
{
// Divides the current pad in nx.ny subpads
// Special version of the TPad::Divide() method which leaves a given space
// at the top of the pad in order to allow writing a title or a global time

 DyDebugPrint(__FUNCTION__,0);

#ifndef USE_VEGA
 Float_t xmargin=0.005;
 Float_t ymargin=0.005;
 Int_t color=0;
 Float_t topmargin=0.06;
   
 if (!gPad->IsEditable()) return;

 TPad *padsav = (TPad*)gPad;
 gPad->cd();
 if (nx <= 0) nx = 1;
 if (ny <= 0) ny = 1;
 Int_t ix,iy;
 Double_t x1,y1,x2,y2;
 Double_t dy = (1-topmargin)/Double_t(ny);
 Double_t dx = 1/Double_t(nx);
 TPad *pad;
 char *name = new char [strlen(padsav->GetName())+6];
 Int_t n = 0;
 if (color == 0) color = padsav->GetFillColor();

 for (iy=0;iy<ny;iy++)
    {y2 = 1 - topmargin - iy*dy - ymargin;
     y1 = y2 - dy + 2*ymargin;
     if (y1 < 0) y1 = 0;
     if (y1 > y2) continue;
     for (ix=0;ix<nx;ix++)
        {x1 = ix*dx + xmargin;
         x2 = x1 +dx -2*xmargin;
         if (x1 > x2) continue;
         n++;
         sprintf(name,"%s_%d",padsav->GetName(),n);
         pad = new TPad(name,name,x1,y1,x2,y2,color);
         pad->SetNumber(n);
         pad->Draw();
        }
    }

 delete [] name;
 padsav->Modified();
 if (padsav) padsav->cd();

#else
 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();
 iif (gPad) gPad->Divide(nx,ny);
 oldsty->cd();
#endif

}


//_____________________________________________________________________________
void RA_PadClear(void* pad)
{
// Clear a pad, leaving all subpads and histograms intact
// Commented because it seems that not needed anymore
// and because it created a problem for restarting and using mTimePave
// of VDyPlot.

/***
 TObject *obj;
 TList* listofprim=((TPad*)pad)->GetListOfPrimitives();
 TIter    next(listofprim);
 while ((obj = next()))
   {
    if (obj->InheritsFrom(TPad::Class()))
       {RA_PadClear((void*)obj);}
    else
       {if (!obj->InheritsFrom(VDyPlot::Class()) && !obj->InheritsFrom(VDyPlot2::Class()) &&
             !obj->InheritsFrom(TH2F::Class()) &&
             !obj->InheritsFrom(TFrame::Class()) &&
             !obj->InheritsFrom(TPaveText::Class()) ) {listofprim->Remove(obj); obj->Delete();}
       }
   }
***/

}


//_____________________________________________________________________________
void RA_PadCompleteClear(int ipad)
{
// Clear a pad
 
 if (gDDCanvas == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;
  
 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();
 gDDCanvas->cd(ipad);
 gPad->Clear();
 oldsty->cd();
}


//_____________________________________________________________________________
int RA_PadIsDivided(int npads)
{
// Return 1 if the current pad is already divided with npads subpads
// if npads == -1, returns 1 if there is at least one subpad
   
 TObject* idcur;
 int npadinpad=0;
   
// loop on all objects in the canvas to see if there are subpads
 TList* l = gPad->GetListOfPrimitives();
 TIter next(l);
 while ((idcur = next())) {if (!strcmp(idcur->IsA()->GetName(),"TPad")) npadinpad++;}
 if (npads==npadinpad || (npads==-1&&npadinpad>0)) return 1;

 return 0;
}


//_____________________________________________________________________________
void RA_SetTestTrigPad(int numpad, int numsubpad, double interval)
{
// Tests and sets the current pad to be the one triggering the canvas screen update 
  
 if (numpad==-1) {gLowestInterval = 999999999.; return;}

 if (numpad == 0) return;

 if (gDDCanvas == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2)
    {printf("RA_SetTestTrigPad> Before: numpad=%d  numsubpad=%d  interval=%g  gPad=%x  gLastPad=%x  gLowestInterval=%g \n ",
             numpad, numsubpad, interval, gPad, gLastPad, gLowestInterval);
    }

 if (interval<=gLowestInterval && interval > 0.)
    {
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0) printf("RA_SetTestTrigPad> pad%d subpad%d accepted \n", numpad, numsubpad);
     TVirtualPad* oldpad=gPad;
     gDDCanvas->cd(numpad);
     if (numsubpad) gPad->cd(numsubpad);
     gLastPad = gPad;
     gLowestInterval = interval;
     gPad = oldpad;
    }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2)
    {printf("RA_SetTestTrigPad> After: numpad=%d  numsubpad=%d  interval=%g .. gPad=%x  gLastPad=%x  gLowestInterval=%g\n",
             numpad, numsubpad, interval, gPad, gLastPad, gLowestInterval);
    }

}


//_____________________________________________________________________________
char* RA_PadContainsGoodHist(int numpad, int numsubpad, int type, int nbinsx, int nbinsy, char *name)
{
// Tests the pad number and subpad number for containing a plot of type
// "type" (1=1D, 2=2D) and of number of bins "nbins".
// Returns the name of the plot if ok.
// Allow to reuse the plot when one wants to keep the style
   
 static char oldname[255];

 DyDebugPrint(__FUNCTION__,0);

 if (gDDCanvas == NULL) return(NULL);
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return(NULL);

 TVirtualPad* curpad = gPad;

// Go to the right pad or subpad
 gDDCanvas->cd(numpad);
 if (numsubpad)
    {if (!RA_PadIsDivided(-1)) {curpad->cd(); return NULL;}
     gPad->cd(numsubpad);
    }
  
// Loop on all objects contained in the pad
 TObject *obj;
 TIter    next(gPad->GetListOfPrimitives());
 while ((obj = next()))
   {
    if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3)
       {printf("RA_PadContainsGoodHist> numpad=%d numsubpad=%d obj=%s\n", numpad, numsubpad, obj->GetName());}

    // If 1Dim plot (VDyPlot) with right number of bins, return it's name
    if (obj->InheritsFrom(VDyPlot::Class()) && type==1)
       {
        VDyPlot* h=(VDyPlot*)obj;
        if (h->GetNbinsX()==nbinsx && strcmp(name,h->GetName())==0)
           {
            if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3) printf("RA_PadContainsGoodHist> previous oldname=%s\n", oldname);
            strcpy(oldname,h->GetName());
            if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>3)
               {printf("RA_PadContainsGoodHist> Found already existing plot: %s=%s=%s nbinx=%d (requested nbinx=%d)\n",
                        oldname, h->GetName(), h->GetTitle(), h->GetNbinsX(), nbinsx);
               }
            curpad->cd();
            return oldname;
           }
       }

    // If 2Dim plot (VDyPlot2) with right number of bins, return it's name
    else if (obj->InheritsFrom(VDyPlot2::Class()) && type==2)
       {
        VDyPlot2* h=(VDyPlot2*)obj;
        if (h->GetNbinsX()==nbinsx && h->GetNbinsY()==nbinsy && strcmp(name,h->GetName())==0)
           {
            strcpy(oldname,h->GetName());
            curpad->cd();
            return oldname;
           }
       }

    // If 2Dim plot (but not VDyPlot2) with right number of bins, return it's name
    else if (obj->InheritsFrom(TH2F::Class()) && type==2)
       {
        TH2F* h=(TH2F*)obj;
        if (h->GetNbinsX()==nbinsx && h->GetNbinsY()==nbinsy && strcmp(name,h->GetName())==0)
           {
            strcpy(oldname,h->GetName());
            curpad->cd();
            return oldname;
           }
       }

   }

 curpad->cd();
 return NULL;
}


//_____________________________________________________________________________
double RA_HGetXmin(char* name)
{
// Returns the lower limit of the X axis for a histogram named "name"
 TAxis* ax=((TH1*)gDirectory->Get(name))->GetXaxis();
 return ax->GetBinLowEdge(ax->GetFirst());
}


//_____________________________________________________________________________
double RA_HGetXmax(char* name)
{
// Returns the upper limit of the X axis for a histogram named "name"
 TAxis* ax=((TH1*)gDirectory->Get(name))->GetXaxis();
 return ax->GetBinUpEdge(ax->GetLast());
}


//_____________________________________________________________________________
double RA_HGetYmin(char* name)
{
// Returns the lower limit of the Y axis for a histogram named "name"
 TAxis* ay=((TH1*)gDirectory->Get(name))->GetYaxis();
 return ay->GetBinLowEdge(ay->GetFirst());
}


//_____________________________________________________________________________
double RA_HGetYmax(char* name)
{
// Returns the upper limit of the Y axis for a histogram named "name"
 TAxis* ay=((TH1*)gDirectory->Get(name))->GetYaxis();
 return ay->GetBinUpEdge(ay->GetLast());
}


//_____________________________________________________________________________
void RA_GlobalTitleDraw (int gps, int uLeapS)
{
// Draws the global title in the Pavelabel named "DyGlobalTitle"
// input : gTimeS = GPS time in seconds
//         ULeapS = Leap seconds for the time

#ifndef USE_VEGA

 if (gDDCanvas == NULL || gPad == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;
   
 TPad* padsave = (TPad*) gPad;
   
 static char stringTime[30];
 time_t utc;
 TPaveText* globalTitle;
 TString titlestring = "dataDisplay ";
 
 titlestring += DYVERSION;
 titlestring += " : started by ";

 gDDCanvas->cd();
 globalTitle = (TPaveText*)gDDCanvas->FindObject("DyGlobalTitle");
   
 if (!globalTitle)
    {globalTitle = new TPaveText(0.05,0.955,0.95,0.985,"");
     globalTitle->SetName("DyGlobalTitle");
     globalTitle->AddText(titlestring);
    }

 TText* firstline = globalTitle->GetLine(0);

 if (!strncmp(titlestring.Data(),firstline->GetTitle(), strlen(titlestring.Data())))
    {
     globalTitle->Clear();
     utc = DyConvertToUTC(gps,uLeapS);
     utc = time(NULL);                      /***--- Do not use anymore gps and uLeapS ---***/
     strftime(stringTime,30,"%b %e %Y %H:%M:%S UTC ",gmtime(&utc));
     titlestring += getenv("USER");
     titlestring += " on ";
     titlestring += stringTime;

     if (Dy->titleCmd != NULL)
        {titlestring += " : ";
         if (strncmp(Dy->titleCmd,"::",2) == 0) titlestring += Dy->titleCmd+2;
         else                                   titlestring  = Dy->titleCmd;
        }

     globalTitle->AddText(titlestring);
     if (Dy->nicefont == OFF) globalTitle->SetTextFont(DY_FONT_TIMES);
     else                     globalTitle->SetTextFont(DY_FONT_HELVETICA);
//     globalTitle->SetTextSize(0.035);
     globalTitle->Draw();
    }
  
 if (padsave) padsave->cd();
#endif
}

   
//_____________________________________________________________________________
const char* RA_CanvasMacroString()
{
// Fills the global variable gDDCanvasString with the instructions
// created by the SaveAs command
// if no global canvas present, return 0

 DyDebugPrint(__FUNCTION__,0);

 gDDCanvasString = "";  // clear the canvas macro string
 if (!gDDCanvas) return gDDCanvasString.Data();

 gDDCanvas->SaveAs(gDyTmpMacro);
 ifstream incan(gDyTmpMacro,ios::in);
 TString linecan;

// Reload the saved canvas macro, with cleaning of some parts
 linecan.ReadLine(incan);
 while(linecan.Length()!=0)
   {
    if (!linecan.Contains("pt->") &&
        !linecan.Contains("TPaveText(") &&
        !linecan.Contains("->SetBinContent(") &&
        !linecan.Contains(REF_SUFFIX) ) gDDCanvasString += linecan + "\n";

    if (linecan.Contains("gDDCanvas_") && linecan.Contains("->Draw();"))
       {
        linecan.ReplaceAll("gDDCanvas_","");
        linecan.ReplaceAll("->Draw();","");
        linecan.ReplaceAll(" ","");
        TString stmpnum = linecan;
        stmpnum.ReplaceAll("_"," ");
        int f,g;
        f=0;
        g=0;
        char stmpnum1[5];
        sscanf(stmpnum.Data(),"%d %d",&f,&g);
        if (g) sprintf(stmpnum1,"%d",g);
        else sprintf(stmpnum1,"%d",f);
        gDDCanvasString += "gDDCanvas_"+linecan+"->SetNumber("+stmpnum1+");\n";
       }

    linecan.ReadLine(incan);
   }
   
 incan.close();
 remove(gDyTmpMacro);
 return gDDCanvasString.Data();
}


//_____________________________________________________________________________
void RA_InterpreterProcessLine(char * cline)
{
// Executes the commands contained in the string cline
// Processing is made by the interpreter

 fstream tmpout(gDyTmpMacro,ios::out);
 tmpout << cline;
 tmpout.close();
 gInterpreter->ExecuteMacro(gDyTmpMacro);
 remove(gDyTmpMacro);

 if (gPad != 0) {gDDCanvas = gPad->GetCanvas(); gDDCanvas->cd();}
}


//_____________________________________________________________________________
char* RA_CreateTempFileName(const char* suffix)
{
// Creates a unique temporary file name, including directory
   
 static char s[100];
// char* tmps = tempnam(0,"Dy_");
 char* tmps = (char *)malloc(10*sizeof(char));
 strcpy(tmps,"Dy_XXXXXX");
 mkstemp(tmps);
 remove(tmps);
 strcpy(s,tmps);
 strcat(s,suffix);
 free(tmps);
 return s;
}


//_____________________________________________________________________________
void RA_MacroCreateCanvas(char * cline)
{
// Executes the commands contained in the string cline
// Processing is made by the interpreter
 if (gDDCanvas) return;
 RA_InterpreterProcessLine(cline);
}


//_____________________________________________________________________________
void  RA_ObjectTablePrint() {}
// void  RA_ObjectTablePrint() {gObjectTable->Print(); TStorage::PrintStatistics();}


//_____________________________________________________________________________
void  RA_ProcessRootEvents(){gSystem->ProcessEvents();}
// void  RA_ProcessRootEvents(){}


//_____________________________________________________________________________
int RA_SystemCheckSwap()
{
// test of the swap space. If there is still enough swap, returns 1, else 0
// Only works for OSF1
// This routine is needed because of a "memory leak" in the Xwindows managing
// of ROOT. Hope will be obsoleted in a near future

#ifdef R__ALPHA
 struct tbl_swapinfo swbuf;
 int swap_pages=0,swap_free=0,i;

 i=0;
 while(table(TBL_SWAPINFO,i,&swbuf,1,sizeof(struct tbl_swapinfo))>0)
    {swap_pages += swbuf.size;
     swap_free  += swbuf.free;
     i++;
    }

 if (swap_free < 0.1*swap_pages)
    {printf("RA_SystemCheckSwap> Warning : swap space left %d%%, cannot continue to display plots\n", (100*swap_free)/swap_pages);
     return 0;
    }
#endif
 return 1;
}


//_____________________________________________________________________________
void RA_BuildDDStyle()
{
// If it doesn't exist, build a style used for Data Display

#ifdef USE_VEGA
 TStyle* sty;
 TStyle* oldsty = gStyle;

 if (!gROOT->GetListOfStyles()->FindObject("dyStyle"))
    {
     sty = new TStyle("dyStyle","style for data display");
     sty->SetTextFont(DY_FONT_TIMES);
     sty->SetTextSize(0.05);
     sty->SetTitleBorderSize(0);
     sty->SetTitleSize(0.05);
     sty->SetLabelFont(DY_FONT_TIMES,"X");
     sty->SetLabelSize(0.05,"X");
     sty->SetLabelFont(DY_FONT_TIMES,"Y");
     sty->SetLabelSize(0.05,"Y");
     sty->SetTitleW(0.8);
     sty->SetTitleH(0.07);
     sty->SetTitleY(0.99);
     sty->SetTitleX(0.1);
     sty->SetTitleStyle(0);
     sty->SetOptStat(1);
     sty->SetNdivisions(505,"X");
     sty->SetNdivisions(505,"Y");
     sty->SetPadLeftMargin(0.13);
     sty->SetPadTopMargin(0.12);
      
     sty->cd();
     int ncolors = 64;
     const Int_t NRGBs = 8;
     Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
     Double_t Red[NRGBs]   = { 0.20, 0.30, 0.00, 0.00, 0.00, 0.97, 0.97, 0.40 };
     Double_t Green[NRGBs] = { 0.00, 0.04, 0.00, 0.97, 0.97, 0.97, 0.00, 0.00 };
     Double_t Blue[NRGBs]  = { 0.20, 0.30, 0.50, 0.97, 0.00, 0.00, 0.00, 0.00 };
     TStyle::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
//     sty->SetNumberContours(64);
     oldsty->cd();
    }
#endif

}


