/**************************************************/
/*                                                */
/* File: URootProcess.cc                          */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/
 
// Handling of user processing functions

using namespace std;

#include <iostream>
//#include "Api.h"
#include "FrameL.h"
#include "TSystem.h"
#include "TString.h"
#include "URootProcess.h"
#include "VViewerSelector.h"
#include "TString.h"
#include "TDatime.h"
#include "TClass.h"
#include "dyincpath.h"
#include "Api.h"
extern "C" {
#include "dataDisplay.h"
}

struct UserProcList*    gProcList=0;
struct UserFileList*    gFileList=0;


//_____________________________________________________________________________
void RA_GetOpString (char *className)
{
 int i,n;
 char *opstring, dummy[LBIGSTRING_MAX], fccimp[LSTRING_MAX];
 FILE *fp;

 if (className == NULL) return;

 sprintf(fccimp,"%s.C",className);

 fp = fopen(fccimp,"r");

 if (fp)
    {n = 0;
     strcpy(dummy," ");
     while (n<7 && strstr(dummy,OPSTRING_PREFIX)==NULL)
        {n++;
         fgets(dummy,LBIGSTRING_MAX-1,fp);
         dummy[strlen(dummy)-1] = '\0';
        }
    }

 if (n>= 7) {strcpy(dummy,"no operation defined"); opstring = dummy;}
 else       opstring = dummy + strlen(OPSTRING_PREFIX);

 RA_GetListOfProc();

 if (gProcList)
    {
     for (i=0;i<gProcList->numProcess;i++)
        {if (strcmp(className,gProcList->name[i]) == 0)
            {gProcList->opstring[i] = (char *)malloc(strlen(opstring)+1);
             strcpy(gProcList->opstring[i], opstring);
             if (strstr(className,TRIGGER) != NULL) strcpy(Dy->lastTrigString,opstring);
             break;
            }
        }
    }

 return;
}


//_____________________________________________________________________________
int RA_LoadScript(char* scriptfilename, int compile)
{
// Loads a script that is in a file named "scriptfilename".
// compile = 1 : compile the script with the automatic compiler
// compile = 0 : do not compile
// return 0 on correct loading, -1 if loading failed
// The script contains a class implementation. The name of the class is
// the file name without .C (or .cc, or whatever)

   char line[256];
   const char* selname;
   int i, iloaded;
   
   TString incpathorig = gSystem->GetIncludePath();
   TString incpath;

   strcpy(line,scriptfilename);
   if (compile) {
      strcat(line,"+");
      incpath = incpathorig + " -I";
      incpath = incpath + gSystem->DirName(scriptfilename);
      incpath = incpath + " "+DYINCPATH;
      printf("RA_LoadScript> DYINCPATH=%s = %s \n", DYINCPATH,incpath.Data());
      gSystem->SetIncludePath(incpath.Data());
   }
   VViewerSelector* sel = VViewerSelector::GetSelector(line);
   if (sel==0) {
      printf("RA_LoadScript> Warning : the script could not be loaded\n");
      return -1;
   }
   selname = sel->GetViewerClassName();

// Add it to the list of loaded process objects
   if (!gProcList) {
      gProcList = new UserProcList;
      for (i=0;i<DYMAXUSERFUNC;i++) gProcList->name[i]=0;
      for (i=0;i<DYMAXUSERFUNC;i++) gProcList->procCompiled[i]=0;
      for (i=0;i<DYMAXUSERFUNC;i++) gProcList->selector[i]=0;
      for (i=0;i<DYMAXUSERFUNC;i++) gProcList->opstring[i]=0;
      gProcList->numProcess = 0;
   }

// Detect if it was already loaded once
   iloaded=-1;
   for (i = 0;i<gProcList->numProcess;i++) {
      if (!strcmp(gProcList->name[i], selname)) iloaded = i;
   }
   
   if (iloaded == -1) {
      gProcList->name[gProcList->numProcess] = new char[strlen(selname)+1];
      strcpy(gProcList->name[gProcList->numProcess], selname);
      gProcList->procCompiled[gProcList->numProcess] = compile;
      gProcList->selector[gProcList->numProcess] = (void*) sel;
      gProcList->numProcess++;
      RA_GetOpString((char *)selname);
   } else {
      gProcList->procCompiled[iloaded] = compile;
      gProcList->selector[iloaded] = (void*) sel;
   }
   
// Add it to the list of loaded files
   if (!gFileList) {
      gFileList = new UserFileList;
      for (i=0;i<DYMAXUSERFUNC;i++) gFileList->fileName[i]=0;
      gFileList->numFiles = 0;
   }

// Detect if it was already loaded once
   iloaded=-1;
   for (i = 0;i<gFileList->numFiles;i++) {
      if (!strcmp(gFileList->fileName[i], scriptfilename)) iloaded = i;
   }
   
   if (iloaded == -1) {
      gFileList->fileName[gFileList->numFiles] = new char[strlen(scriptfilename)+1];
      strcpy(gFileList->fileName[gFileList->numFiles], scriptfilename);
      gFileList->fileCompiled[gFileList->numFiles] = compile;
      gFileList->numFiles++;
   } else {
      gFileList->fileCompiled[iloaded] = compile;
   }

   gSystem->SetIncludePath(incpathorig.Data());
   return 0;
}

//_____________________________________________________________________________
int RA_InitDefaultScripts()
{
// Initialize scripts present at startup time. Those scripts are
// actually classes compiled and linked within dataDisplay code.
// Search all the classes known to CINT and if a class derives from
// VViewerSelector, make an instance of it. Fill the ProcList list
// with these objects.

   int i, iloaded;
   const char* selname;
   VViewerSelector *sel;

   if (Dy->debug > 0) printf("RA_InitDefaultScripts> Make libs : %s\n",gSystem->GetMakeSharedLib());

   G__ClassInfo cl;
   Bool_t OK = kFALSE;
   while (cl.Next()) {
      if (cl.IsBase("VViewerSelector") && strcmp(cl.Name(),"VViewerSelectorCint")!=0) {
         OK = kTRUE;
       // we can now create an instance of the class
         sel = (VViewerSelector*)cl.New();
         if (sel==0) {
            printf("RA_InitDefaultScripts> Warning : a script could not be initialized\n");
         } else {
            selname = sel->GetViewerClassName();
          // Add it to the list of loaded process objects
            if (!gProcList) {
               gProcList = new UserProcList;
               for (i=0;i<DYMAXUSERFUNC;i++) gProcList->name[i]=0;
               for (i=0;i<DYMAXUSERFUNC;i++) gProcList->procCompiled[i]=0;
               for (i=0;i<DYMAXUSERFUNC;i++) gProcList->selector[i]=0;
               for (i=0;i<DYMAXUSERFUNC;i++) gProcList->opstring[i]=0;
               gProcList->numProcess = 0;
            }
         // Detect if it was already loaded once
            iloaded=-1;
            for (i = 0;i<gProcList->numProcess;i++) {
               if (!strcmp(gProcList->name[i], selname)) iloaded = i;
            }

            if (iloaded == -1) {
               gProcList->name[gProcList->numProcess] = new char[strlen(selname)+1];
               strcpy(gProcList->name[gProcList->numProcess], selname);
               gProcList->procCompiled[gProcList->numProcess] = 1;
               gProcList->selector[gProcList->numProcess] = (void*) sel;
               gProcList->numProcess++;
            } else {
               gProcList->procCompiled[iloaded] = 1;
               gProcList->selector[iloaded] = (void*) sel;
            }
         }
      }
   }
   if (OK==kFALSE) printf("RA_InitDefaultScripts> Warning : No script classes initialized\n");
   
   return 0;
}


//_____________________________________________________________________________
void* RA_GetSelector(char* objectname)
{
// returns a pointer to the selector object corresponding to objectname
   int iloaded,i;

   if (gProcList == NULL) return(0);

   iloaded=-1;
   for (i = 0;i<gProcList->numProcess;i++) {
      if (!strcmp(gProcList->name[i], objectname)) iloaded = i;
   }
   if (iloaded == -1) return 0;

   return gProcList->selector[iloaded];
}


//_____________________________________________________________________________
int RA_GetSelectorI(char* objectname)
{
// returns the index of the gProcList object corresponding to objectname
   int iloaded,i;

   if (gProcList == NULL) return(-1);

   iloaded=-1;
   for (i=0; i<gProcList->numProcess; i++)
      {if (!strcmp(gProcList->name[i], objectname)) iloaded = i;}

   return(iloaded);
}


//_____________________________________________________________________________
void RA_UserInit(char* objectname)
{
// Calls the user Init function corresponding to objectname
   VViewerSelector* sel;

   sel = (VViewerSelector*)RA_GetSelector(objectname);
   if (sel) sel->Init();
}


//_____________________________________________________________________________
void RA_UserBegin(char* objectname)
{
// Calls the user Begin function corresponding to objectname
   VViewerSelector* sel;

   sel = (VViewerSelector*)RA_GetSelector(objectname);
   if (sel) sel->Begin();
}


//_____________________________________________________________________________
void RA_UserTerminate(char* objectname)
{
// Calls the user Terminate function corresponding to objectname
   VViewerSelector* sel;

   sel = (VViewerSelector*)RA_GetSelector(objectname);
   if (sel) sel->Terminate();
}


//_____________________________________________________________________________
int RA_UserProcess(char* objectname, int numvect, FrVect** vectlist, FrVect* outvect, double* par)
{
// Calls the user ProcessVect function corresponding to objectname
// The object should be known to the interpreter,  i.e. already loaded
// (compiled or interpreted)
// The output vector is already allocated, it's size being the size of
// the first vector in the vector input list vectlist
// (vectlist is of size numvect)
// A user may NOT delete this vector. But he can allocate
// memory for a new one, best suiting his wishes (size, type ...)
// This vector should then be returned. The returned vector
// is not owned by the user, so no need to do a free afterwards.
   
   VViewerSelector* sel;

   sel = (VViewerSelector*)RA_GetSelector(objectname);
   if (sel) return sel->ProcessVect(numvect,vectlist,outvect,par);
   return 0;
}


//_____________________________________________________________________________
struct UserProcList* RA_GetListOfProc()
{
// Returns a list of names of functions that were loaded.
   
   return(gProcList);
}


//______________________________________________________________________________
int RA_MakeClass(const char *classname, char *opstring, char *opstring_orig)
{
//*-*-*-*-*Generate skeleton analysis class for the Data Display *-*-*-*-*-*
//*-*      ======================================================
//
//   The following files are produced: classname.h and classname.C
//   if classname is NULL, classname will be "MyDySelector".
//
//   The generated code in classname.h includes the following:
//      - Definition of class (data and functions)
//      - the following class methods:
//         - constructor and destructor
//         - void    Begin()
//         - void    Init()
//         - Bool_t  ProcessVect(int nbvect, FrVect** invects, FrVect* ouvect, double* par);
//         - void    Terminate
//
//   The class classname derives from VViewerSelector.
//   The generated code in classname.C includes empty functions defined above:
//
//   The generated code should also be usable in vega for analysis purposes
//

   TString defaultname = "MyDySelector";

   int statusflag = 0;

   // Connect output files
   char *dummy  = new char[256];
   char *fchead = new char[256];
   char *fccimp = new char[256];
 
   if (!classname) classname = defaultname.Data();

   sprintf(fchead,"%s.h",classname);
   sprintf(fccimp,"%s.C",classname);

   FILE *fp = fopen(fchead,"w");
   FILE *fpc = fopen(fccimp,"w");
   if (fp && fpc)
      {
       RA_MakeClassWrite(classname, opstring, opstring_orig, fchead, fp, fpc);
       printf("RA_MakeClass> Files: %s and %s generated for dataDisplay or viewer\n",fchead,fccimp);
      }
   else
      {printf("RA_MakeClass> Cannot open files: %s and %s\n",fchead, fccimp);
       statusflag = 3;
      }

   delete [] fchead;
   delete [] fccimp;
   if (fp)  fclose(fp);
   if (fpc) fclose(fpc);

   /*--- Create the same files in DYALGO_DIR just for archive ---*/

   if (is_a_dir((char *)DYALGO_DIR))
      {
       char *fchead_copy = new char[256];
       char *fccimp_copy = new char[256];

       sprintf(dummy,"%s/%s", DYALGO_DIR, getenv("UNAME"));
       if (is_a_dir(dummy)) sprintf(fccimp_copy, "%s/%s.C", dummy, classname);
       else                 sprintf(fccimp_copy, "%s/%s.C", DYALGO_DIR, classname);
       if (is_a_dir(dummy)) sprintf(fchead_copy, "%s/%s.h", dummy, classname);
       else                 sprintf(fchead_copy, "%s/%s.h", DYALGO_DIR, classname);

       FILE *fp_copy = fopen(fchead_copy,"w");
       FILE *fpc_copy = fopen(fccimp_copy,"w");
       RA_MakeClassWrite(classname, opstring, opstring_orig, fchead_copy, fp_copy, fpc_copy);

       delete [] fchead_copy;
       delete [] fccimp_copy;
       if (fp_copy)  fclose(fp_copy);
       if (fpc_copy) fclose(fpc_copy);
      }

   return statusflag;
}



//______________________________________________________________________________
void RA_MakeClassWrite(const char *classname, char *opstring, char *opstring_orig,
                       char *fchead, FILE *fp, FILE *fpc)
{

 //======================Generate classname.h=====================
   // Print header
   TDatime td;
   fprintf(fp,"//////////////////////////////////////////////////////////\n");
   fprintf(fp,"//   This class has been automatically generated \n");
   fprintf(fp,"//     (%s by Data Display version %s)\n",td.AsString(),DYVERSION);
   fprintf(fp,"//////////////////////////////////////////////////////////\n");
   fprintf(fp,"\n");
   fprintf(fp,"\n");
   fprintf(fp,"#ifndef %s_h\n",classname);
   fprintf(fp,"#define %s_h\n",classname);
   fprintf(fp,"\n");
   fprintf(fp,"#if !defined(__CINT__) || defined(__MAKECINT__)\n");
   fprintf(fp,"#include \"FrameL.h\"\n");
   fprintf(fp,"#include \"VViewerSelector.h\"\n");
   fprintf(fp,"#endif\n");

   fprintf(fp,"\n");
   fprintf(fp,"class %s : public VViewerSelector {\n",classname);
   fprintf(fp,"   public :\n");

// generate class member functions prototypes
   fprintf(fp,"\n");
   fprintf(fp,"   %s() { }\n",classname) ;
   fprintf(fp,"   ~%s() { }\n",classname);
   fprintf(fp,"   void    Begin();\n");
   fprintf(fp,"   void    Init();\n");
   fprintf(fp,"   Bool_t  ProcessVect(int nbvect, FrVect** invects, FrVect* outvect, double* par);\n");
   fprintf(fp,"   void    Terminate();\n");
   fprintf(fp,"   ClassDef(%s,0);\n",classname);
   fprintf(fp,"};\n");
   fprintf(fp,"\n");
   fprintf(fp,"#endif\n");
   fprintf(fp,"\n");

//======================Generate classname.C=====================
   // generate usage comments and list of includes
   fprintf(fpc,"#define %s_cxx\n",classname);
   fprintf(fpc,"// The class definition in %s.h has been generated automatically\n",classname);
   fprintf(fpc,"// by the Data Display utility MakeClass().\n");
   if (opstring_orig) fprintf(fpc,"%s%s\n", OPSTRING_PREFIX, opstring_orig);
   fprintf(fpc,"//\n");
   fprintf(fpc,"// This class is derived from the class VViewerSelector.\n");
   fprintf(fpc,"// The following members functions are called by the dataDisplay Process() functions:\n");
   fprintf(fpc,"//    Init():        called once per dataDisplay session\n");
   fprintf(fpc,"//    Begin():       called everytime a loop on the data starts,\n");
   fprintf(fpc,"//                   a convenient place to create your histograms.\n");
   fprintf(fpc,"//    ProcessVect(): called for each vector(s) that should be analyzed\n");
   fprintf(fpc,"//    Terminate():   called at the end of a loop in the dataDisplay,\n");
   fprintf(fpc,"//                   a convenient place to draw/fit your histograms.\n");
   fprintf(fpc,"//\n");
   fprintf(fpc,"#include \"%s\"\n",fchead);
   fprintf(fpc,"\n");
   fprintf(fpc,"#if !defined(__CINT__) || defined(__MAKECINT__)\n");
//   fprintf(fpc,"extern \"C\" { \n");
//   fprintf(fpc,"#include \"userfct.h\"\n");
//   fprintf(fpc,"} \n");
   fprintf(fpc,"// Here come the user includes like\n");
   fprintf(fpc,"// #include \"TH1F.h\"\n");
   fprintf(fpc,"#endif\n");
   fprintf(fpc,"\n");
// generate code for class member function Init()
   fprintf(fpc,"void %s::Init()\n",classname);
   fprintf(fpc,"{\n");
   fprintf(fpc,"}\n");
   fprintf(fpc,"\n");
   // generate code for class member function Begin
   fprintf(fpc,"\n");
   fprintf(fpc,"void %s::Begin()\n",classname);
   fprintf(fpc,"{\n");
   fprintf(fpc,"   // Function called before starting the analysis loop.\n");
   fprintf(fpc,"\n");
   fprintf(fpc,"}\n");
   // generate code for class member function ProcessVect
   fprintf(fpc,"\n");
   fprintf(fpc,"Bool_t %s::ProcessVect(int nbvect, FrVect** invects, FrVect* outvect, double* par)\n",classname);
   fprintf(fpc,"{\n");
   fprintf(fpc,"   // Function called for selected vectors only.\n");
   fprintf(fpc,"   // invects is an array of vectors, nbvect is the number.\n");
   fprintf(fpc,"   // of vectors in the array and outvect is the resulting.\n");
   fprintf(fpc,"   // output vector.\n");
   fprintf(fpc,"   // par is a set of parameters.\n");
   fprintf(fpc,"   // Return kFALSE to stop processing.\n");
   fprintf(fpc,"   // The following code is an example, where sum of squares is computed.\n");
   fprintf(fpc,"   // Just replace it by your own code.\n");
   fprintf(fpc,"\n");
   fprintf(fpc,"int i, j, ndata_max, indix;\n");
   fprintf(fpc,"double value;\n");
   if (opstring)
      {fprintf(fpc,"double valueIn[256];\n");}
   fprintf(fpc,"\n");
   fprintf(fpc,"ndata_max = 0;\n");
   fprintf(fpc,"for (i=0; i<nbvect; i++)\n");
   fprintf(fpc,"   {if (invects[i])\n");
   fprintf(fpc,"       {if (ndata_max < invects[i]->nData) ndata_max = invects[i]->nData;}\n");
   fprintf(fpc,"   }\n");
   fprintf(fpc,"\n");
   fprintf(fpc,"for (j=0; j<ndata_max; j++)\n");
   fprintf(fpc,"   {\n");
   fprintf(fpc,"    outvect->dataD[j] = 0.;\n");
   if (opstring == NULL)
      {
       fprintf(fpc,"    for (i=0; i<nbvect; i++)\n");
       fprintf(fpc,"       {\n");
       fprintf(fpc,"        if (invects[i])\n");
       fprintf(fpc,"           {indix = (int)((double)j * invects[i]->nData / ndata_max);\n");
       fprintf(fpc,"            value = FrVectGetV (invects[i], indix);\n");
       fprintf(fpc,"            if (par) outvect->dataD[j] += value * value * par[i];\n");
       fprintf(fpc,"            else     outvect->dataD[j] += value * value;\n");
       fprintf(fpc,"           }\n");
       fprintf(fpc,"        else\n");
       fprintf(fpc,"           {value = 0.;}\n");
       fprintf(fpc,"       }\n");
      }
   else
      {
       fprintf(fpc,"    for (i=0; i<nbvect; i++)\n");
       fprintf(fpc,"       {\n");
       fprintf(fpc,"        if (invects[i])\n");
       fprintf(fpc,"           {indix = (int)((double)j * invects[i]->nData / ndata_max);\n");
       fprintf(fpc,"            %s[i] = FrVectGetV (invects[i], indix);\n", VALUEIN);
       fprintf(fpc,"           }\n");
       fprintf(fpc,"        else\n");
       fprintf(fpc,"           {%s[i] = 0.;}\n", VALUEIN);
       fprintf(fpc,"        }\n");
       fprintf(fpc,"     outvect->dataD[j] = %s;\n", opstring);
      }
   fprintf(fpc,"   }\n");
   fprintf(fpc,"\n");
   fprintf(fpc,"   return kTRUE;\n");
   fprintf(fpc,"\n");
   fprintf(fpc,"}\n");
   // generate code for class member function Terminate
   fprintf(fpc,"\n");
   fprintf(fpc,"void %s::Terminate()\n",classname);
   fprintf(fpc,"{\n");
   fprintf(fpc,"   // Function called at the end of the analysis loop.\n");
   fprintf(fpc,"\n");
   fprintf(fpc,"\n");
   fprintf(fpc,"}\n");

   return;
}
