#ifndef UROOTPROCESS_DEFINED
#define UROOTPROCESS_DEFINED

/* Access to the Cint interpreter. Possibility to launch user scripts */

#include "FrameL.h"

#ifdef __cplusplus
extern "C" {
#endif

#define OPSTRING_PREFIX "//opstring="
#define DYMAXUSERFUNC 256

/* Structure to hold the list of user processing objects */ 
   struct UserProcList {
      int   numProcess;                  /* number of loaded objects         */
      char* name[DYMAXUSERFUNC];         /* names of the loaded objects      */
      char* opstring[DYMAXUSERFUNC];     /* operation string of the loaded objects      */
      int   procCompiled[DYMAXUSERFUNC]; /* 0=interpreted function, 1=compiled */
      void* selector[DYMAXUSERFUNC];     /* selector object build for processing */
   };
   
/* Structure to hold the list of user script files */ 
   struct UserFileList {
      int   numFiles;                    /* number of loaded files         */
      char* fileName[DYMAXUSERFUNC];     /* names of the files */
      int   fileCompiled[DYMAXUSERFUNC]; /* 0=interpreted file, 1=compiled */
   };
   
/* Functions */
          void          RA_GetOpString (char *className);
          int           RA_LoadScript(char* scriptfilename, int compile);
          int           RA_InitDefaultScripts();
          void*         RA_GetSelector(char* objectname);
          int           RA_GetSelectorI(char* objectname);
          void          RA_UserInit(char* objectname); 
          void          RA_UserBegin(char* objectname); 
          void          RA_UserTerminate(char* objectname);
          int           RA_UserProcess(char* objectname, int numvect, FrVect** vectlist, FrVect* outvect, double* par); 
          int           RA_MakeClass(const char *classname, char *opstring, char *opstring_orig);
          void          RA_MakeClassWrite(const char *classname, char *opstring, char *opstring_orig, char *fchead, FILE *fp, FILE *fpc);
   struct UserProcList* RA_GetListOfProc();

/* global to hold the list of user functions */
   extern struct UserProcList*  gProcList;
   extern struct UserFileList*  gFileList;

#ifdef __cplusplus
}
#endif

#endif
