/**************************************************/
/*                                                */
/* File: VDyPlot.c                                */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// VDyPlot: class describing a plot (representation) of a series        //
// Used to adapt the behaviour of TH1F to datadisplay needs             //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

using namespace std;

#include <fstream>
#include <iostream>
#include <sys/time.h>
#include <time.h>
#include "TSystem.h"
#include "TROOT.h"
#include "TClass.h"
#include "VDyPlot.h"
#include "TString.h"
#include "TGaxis.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TFile.h"
#include "TKey.h"
#include "dataDisplay.h"
/*
#include "forms.h"
*/

VDyPlot*  gLastModifiedPlot=0; 
VDyPlot*  gLastModifiedPlotY=0; 


ClassImp(VDyPlot)

//______________________________________________________________________________
VDyPlot::VDyPlot() : TH1F()
{
   alreadyDrawn  = 0;
   numpad        = 0;
   numsubpad     = 0;
   superposed    = 0;
   mTimeFlag     = 0;
   mTimeString   = "";
   mTimePave     = 0;
   mTimeX1       = 0.03;
   mTimeY1       = 0.01;
   mTimeX2       = 0.97;
   mTimeY2       = 0.08;
   mTimeUTC      = 0.;
   mOldFirstXbin = GetXaxis()->GetFirst();
   mOldLastXbin  = GetXaxis()->GetLast();
   mOldMinY      = 0.;
   mOldMaxY      = 0.;
   mPreviousMinY = 0.;
   mPreviousMaxY = 0.;
   mIndicator.Set(0);
}

//______________________________________________________________________________
VDyPlot::VDyPlot(Text_t* name, Text_t* title, Int_t nbinsx, Axis_t xlow, Axis_t xup)
           :TH1F(name,title,nbinsx,xlow,xup)
{
   char *str;

   alreadyDrawn  = 0;
   numpad        = 0;
   numsubpad     = 0;
   superposed    = 0;
   mTimeFlag     = 0;
   mTimeString   = "";
   mTimePave     = 0;
   mTimeX1       = 0.03;
   mTimeY1       = 0.01;
   mTimeX2       = 0.97;
   mTimeY2       = 0.08;
   mTimeUTC      = 0.;
   mOldFirstXbin = GetXaxis()->GetFirst();
   mOldLastXbin  = GetXaxis()->GetLast();
   mOldMinY      = GetMinimum();
   mOldMaxY      = GetMaximum();
   mIndicator.Set(fNcells);

   str = strstr(title,SEPTITLE);
   if (str) strcpy(mPlotType,&str[0]+2);
   else     strcpy(mPlotType,"NULL");
   if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0) printf("VDyPlot> Created VDyPlot %s=%s \n", GetName(), GetTitle());
}

//______________________________________________________________________________
VDyPlot::VDyPlot(VDyPlot &plot)
{
   ((VDyPlot&)plot).Copy(*this);
}

//______________________________________________________________________________
void VDyPlot::Copy(TObject& newvsp)
{
   TH1F::Copy((VDyPlot&)newvsp);
}

//______________________________________________________________________________
void VDyPlot::Draw(Option_t* option)
{
 char option2[256];

// Draw the plot. Added the drawing of the time string, with respect to
// the TH1F base class

 char dbgmsg[LSTRING_MAX];

 sprintf(dbgmsg,"VDyPlotDraw> plot=%s=%s pad=%d", GetName(), GetTitle(), numpad);
 DyDebugPrint(dbgmsg,0);
 DyDebugTimeNeeded(dbgmsg,"start",4);

 if (gPad && mTimeFlag  && strstr(option,"SAME") == NULL)
    {if (gPad->GetBottomMargin() < mTimeY2+0.10) gPad->SetBottomMargin(mTimeY2+0.10);}

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0) 
    {printf("VDyPlotDraw> plot=%s=%s  pad=%d  option=%s \n", GetName(), GetTitle(), numpad, option);}

/**
   printf("VDyPlotDraw> plot=%s=%s=pad%d  alreadyDrawn=%d  mTimeFlag=%d  option=%s mTimePave=%x\n",
           GetName(), GetTitle(), numpad, alreadyDrawn, mTimeFlag, option, mTimePave);
   if (alreadyDrawn == 1) sprintf(option2,"%s same",option);
   else                   {strcpy(option2,option); alreadyDrawn = 1; Dy->nplotdrawn++;}
   printf("VDyPlotDraw> option2=%s\n", option2);
**/

   if (strstr(option,"SAME") != NULL) superposed = 1;
   else                               superposed = 0;

   TH1F::Draw(option);

   if (gPad && mTimeFlag && strstr(option,"SAME") == NULL)
     {
      if (!mTimePave)
        {
         mTimePave = new TPaveText(mTimeX1, mTimeY1, mTimeX2, mTimeY2,"bNDC");
         mTimePave->ResetBit(kCanDelete);
         mTimePave->SetName("DyTimeString");
         mTimePave->AddText(mTimeString.Data());
//         if (strstr(mTimeString.Data(),"dt:") != NULL) {mTimePave->SetTextSize(0.04); mTimePave->SetTextAlign(12);}
//         if (strstr(mTimeString.Data(),"dt:") != NULL) {mTimePave->SetX1NDC(0.07); mTimePave->SetX2NDC(0.93);}
         mTimePave->SetFillColor(0);
         mTimePave->Draw("NDC");
        }
      else
        {
         mTimePave->Clear();
         mTimePave->AddText(mTimeString.Data());
//         if (strstr(mTimeString.Data(),"dt:") != NULL) {mTimePave->SetX1NDC(0.07); mTimePave->SetX2NDC(0.93);}
         mTimePave->SetFillColor(0);
         if (alreadyDrawn == 0) mTimePave->Draw("NDC");
        }
     }

 DyDebugTimeNeeded(dbgmsg,"stop",4);

}


//______________________________________________________________________________
char *VDyPlot::GetObjectInfo(Int_t px, Int_t py) const
{
//   Redefines TH1::GetObjectInfo.
//   Displays the histogram info (bin number, contents, integral up to bin
//   corresponding to cursor position px,py
//   Switches to the right pad before doing so.
//   Correction of a ROOT bug (feature ?).
//
   static char info[128];
   TVirtualPad* savepad;
   savepad = gPad;
   gPad = (gPad->GetCanvas())->GetSelectedPad();

   if (!gPad) return (char*)"";
   Double_t x  = gPad->PadtoX(gPad->AbsPixeltoX(px));
   Double_t y  = gPad->PadtoY(gPad->AbsPixeltoY(py));
   Double_t x1 = gPad->PadtoX(gPad->AbsPixeltoX(px+1));
   Int_t binx,binmin,binx1;
   
   binx   = fXaxis.FindFixBin(x);
   binmin = fXaxis.GetFirst();
   binx1  = fXaxis.FindFixBin(x1);
// special case if more than 1 bin in x per pixel
   if (binx1-binx>1 && GetDimension() == 1) {
      Double_t binval=GetBinContent(binx);
      Int_t binnear=binx;
      for (Int_t ibin=binx+1; ibin<binx1; ibin++) {
         Double_t binvaltmp = GetBinContent(ibin);
         if (TMath::Abs(y-binvaltmp) < TMath::Abs(y-binval)) {
            binval=binvaltmp;
            binnear=ibin;
         }
      }
      binx = binnear;
   }
   Double_t integ = 0;
   for (Int_t bin=binmin;bin<=binx;bin++) {integ += GetBinContent(bin);}
   sprintf(info,"x=%g y=%g bin=%d value=%g Sum=%g",x,y,binx,GetBinContent(binx),integ);

   gPad = savepad;
   return info;
}


//______________________________________________________________________________
void VDyPlot::SavePrimitive(ofstream &out, Option_t *option)
{
 // Save primitive as a C++ statement(s) on output stream out
 // Note the following restrictions in the code generated:
 // - variable bin size not implemented
 // - Objects in list of functions not saved (fits)
 // Adapted for Data Display: DO NOT save the bin contents, unnecessary

 char dbgmsg[LSTRING_MAX];

 sprintf(dbgmsg, "VDyPlotSavePrimitive> plot=%s=%s pad=%d", GetName(), GetTitle(), numpad);
 DyDebugPrint(dbgmsg,0);

 char quote = '"';
 out<<"   "<<endl;
 out<<"   "<<"TH1"<<" *";

 out<<GetName()<<" = new "<<ClassName()<<"("<<quote<<GetName()<<quote<<","<<quote<<GetTitle()<<quote
               <<","<<GetXaxis()->GetNbins()
               <<","<<GetXaxis()->GetXmin()
               <<","<<GetXaxis()->GetXmax();
 if (fDimension > 1) {
            out<<","<<GetYaxis()->GetNbins()
               <<","<<GetYaxis()->GetXmin()
               <<","<<GetYaxis()->GetXmax();
 }
 if (fDimension > 2) {
            out<<","<<GetZaxis()->GetNbins()
               <<","<<GetZaxis()->GetXmin()
               <<","<<GetZaxis()->GetXmax();
 }
            out<<");"<<endl;

 if (TMath::Abs(GetBarOffset()) > 1e-5)  {out<<"   "<<GetName()<<"->SetBarOffset("<<GetBarOffset()<<");"<<endl;}
 if (TMath::Abs(GetBarWidth()-1) > 1e-5) {out<<"   "<<GetName()<<"->SetBarWidth("<<GetBarWidth()<<");"<<endl;}
 if (fMinimum != -1111)                  {out<<"   "<<GetName()<<"->SetMinimum("<<fMinimum<<");"<<endl;}
 if (fMaximum != -1111)                  {out<<"   "<<GetName()<<"->SetMaximum("<<fMaximum<<");"<<endl;}
 if (fNormFactor != 0)                   {out<<"   "<<GetName()<<"->SetNormFactor("<<fNormFactor<<");"<<endl;}
 if (fEntries != 0)                      {out<<"   "<<GetName()<<"->SetEntries("<<fEntries<<");"<<endl;}
 if (fDirectory == 0)                    {out<<"   "<<GetName()<<"->SetDirectory(0);"<<endl;}
 if (TestBit(kNoStats))                  {out<<"   "<<GetName()<<"->SetStats(0);"<<endl;}
 if (fOption.Length() != 0)              {out<<"   "<<GetName()<<"->SetOption("<<quote<<fOption.Data()<<quote<<");"<<endl;}

 Int_t bin;
//    for (bin=0;bin<fNcells;bin++) {
//       Double_t bc = GetBinContent(bin);
//       if (bc) {
//          out<<"   "<<GetName()<<"->SetBinContent("<<bin<<","<<bc<<");"<<endl;
//       }
//    }
//    if (fSumw2.fN) {
//       for (bin=0;bin<fNcells;bin++) {
//          Double_t be = GetBinError(bin);
//          if (be) {
//             out<<"   "<<GetName()<<"->SetBinError("<<bin<<","<<be<<");"<<endl;
//          }
//       }
//    }
 Int_t ncontours = GetContour();
 if (ncontours > 0) {
    out<<"   "<<GetName()<<"->SetContour("<<ncontours<<");"<<endl;
    for (bin=0;bin<ncontours;bin++) {
       out<<"   "<<GetName()<<"->SetContourLevel("<<bin<<","<<GetContourLevel(bin)<<");"<<endl;
    }
 }

 SaveFillAttributes(out,GetName(),0,1001);
 SaveLineAttributes(out,GetName(),1,1,1);
 SaveMarkerAttributes(out,GetName(),1,1,1);
 fXaxis.SaveAttributes(out,GetName(),"->GetXaxis()");
 fYaxis.SaveAttributes(out,GetName(),"->GetYaxis()");
 fZaxis.SaveAttributes(out,GetName(),"->GetZaxis()");
 TString opt = option;
 opt.ToLower();
 if (!opt.Contains("nodraw")) {
    out<<"   "<<GetName()<<"->Draw("
    <<quote<<option<<quote<<");"<<endl;
 }
}


//______________________________________________________________________________
void VDyPlot::SetIndicator(int bin, short colorValue)
{
// Sets the value for indicator on bin. This will be used to draw a colored vertical
// line indicating something went wrong.

 if (bin < 0) return;
 mIndicator.AddAt((short)colorValue,bin);
}


//______________________________________________________________________________
void VDyPlot::Paint(Option_t* option)
{
// Paint the plot and add indicator lines

 double utc, gps;
 int nxbin_visible, i, num;
 char title[256], h2name[64];
 double indicymin, indicymax, indicxmin,indicxmax;
 Float_t savexloffset;
 char optbis[256];
 Option_t *optionold;
 TVirtualPad *curpad;
 char dbgmsg[LSTRING_MAX];

 sprintf(dbgmsg, "VDyPlotPaint> plot=%s=%s pad=%d", GetName(), GetTitle(), numpad);

 DyDebugPrint(dbgmsg,0);
 DyDebugTimeNeeded(dbgmsg,"start",4);

 savexloffset = GetXaxis()->GetLabelOffset();
 if (gPad->GetLogx() == 1) GetXaxis()->SetLabelOffset(savexloffset+0.005);

 if (strstr(option,"same") != NULL || strstr(option,"SAME") != NULL)
    {if (gLastModifiedPlot) nxbin_visible = gLastModifiedPlot->GetXaxis()->GetLast() - gLastModifiedPlot->GetXaxis()->GetFirst();
     else                   nxbin_visible = GetXaxis()->GetLast() - GetXaxis()->GetFirst();
    }
 else
    {nxbin_visible = GetXaxis()->GetLast() - GetXaxis()->GetFirst();}

 int length = strlen(option);

// If dataDisplay stopped, set special option of drawing 

 if (Dy->Order == STOP)
    {
     if (nxbin_visible < 16384)
        {if (strstr(option,"l") != NULL)
            {for (i=0; i<length; i++)
                {if (option[i] != 'l') optbis[i] = option[i];
                 else                  optbis[i] = 'h';
                }
             optbis[length] = '\0';
             SetDrawOption(optbis);
            }
        }

     else if (strstr(option,"h") != NULL)
        {for (i=0; i<length; i++)
             {if (option[i] != 'h') optbis[i] = option[i];
              else                  optbis[i] = 'l';
             }
         optbis[length] = '\0';
         SetDrawOption(optbis);
        }
    }

// Paint the plot  

/**
gLastModified est NULL si plot TIME ou plot 1D, mais pas apres le premier passage de plot FFT ?? 
if (gLastModifiedPlot) gLastModifiedPlot->Paint(option);
**/

 TH1F::Paint(option);

 /*** Ce calcul de nxbin ne fonctionne pas si logX **/
 optionold = GetDrawOption();
 int nxbin_visible2 = (int)(GetNbinsX() * (gPad->GetUxmax() - gPad->GetUxmin())/(GetXaxis()->GetXmax() - GetXaxis()->GetXmin()));
 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 1)
    {printf("VDyPlotPaint> plot=%s=%s=pad%d  option=%s  optionold=%s  nxbin_visible=%d  zoom_min=%g  zoom_max=%g  nbinx=%d  nxbin_visible2=%d\n",
             GetName(), GetTitle(), numpad, option, optionold, nxbin_visible, gPad->GetUxmin(), gPad->GetUxmax(), GetNbinsX(), nxbin_visible2);
    }


// If a zoom on X axis was made, set new timeString

 if ((mOldFirstXbin != GetXaxis()->GetFirst() || mOldLastXbin != GetXaxis()->GetLast())
      && (strcmp(mPlotType,"TIME")==0 || strcmp(mPlotType,"CORR")==0) )
    {
     TPaveText* timepave = (TPaveText*)gPad->FindObject("DyTimeString");
     if (timepave)
        {
         double zoom_min2 = gPad->GetUxmin();
         utc = GetTimeUTC() + zoom_min2;
         gps = utc - FRGPSOFF + FRGPSDELTA;
         if (FRGPSDELTA <= 13 && utc-FRGPSOFF+FRGPSDELTA>=820108813) gps =  utc - FRGPSOFF + FRGPSDELTA + 1;
         if (FRGPSDELTA <= 13 && utc-FRGPSOFF+FRGPSDELTA>=914803213) gps =  utc - FRGPSOFF + FRGPSDELTA + 2;
         if (FRGPSDELTA == 14 && utc-FRGPSOFF+FRGPSDELTA>=914803213) gps =  utc - FRGPSOFF + FRGPSDELTA + 1;
         strcpy(title, timepave->GetLine(0)->GetTitle());
         DyGpsToTimeString (gps, NULL, 0, 0, &title[0]);
         timepave->Clear();
         TString timeString = title;
         timepave->AddText(timeString.Data());
         timepave->SetFillColor(0);
         timepave->Draw("NDC");
        }
    }


// Add colored indicator lines

 double zoom_min = gPad->GetUxmin();
 double zoom_max = gPad->GetUxmax();
 indicymin = gPad->GetUymin(); /** + (gPad->GetUymax()-gPad->GetUymin())/3; **/
 indicymax = gPad->GetUymax(); /** - (gPad->GetUymax()-gPad->GetUymin())/3; **/

 int flag = 0;
 for (int i=1;i<fNcells;i++)
    {
     if (i>1 && mIndicator[i-1] > 0 && mIndicator[i] != mIndicator[i-1])
          {
         indicxmax = GetXaxis()->GetBinUpEdge(i-1);
         if (indicxmax > zoom_max) indicxmax = zoom_max;
         if (indicxmin >= zoom_min  &&  indicxmax <= zoom_max  &&  indicxmin < indicxmax && (Dy->noBox == 0 || mIndicator[i-1] != 3))
            {
             gVirtualX->SetFillStyle(1001);
             gVirtualX->SetFillColor(mIndicator[i-1]); //is redefined in TPad::PaintBox
             gPad->PaintBox(indicxmin, indicymin, indicxmax, indicymax);
            }
         flag = 0;
        }
     if (mIndicator[i] > 0  &&  flag == 0)
        {flag = 1;
         indicxmin = GetXaxis()->GetBinLowEdge(i);
         if (indicxmin < zoom_min) indicxmin = zoom_min;
        }
    }


 GetXaxis()->SetLabelOffset(savexloffset);


// If a zoom on X axis was made, set this plot as last modified plot

 if (mOldFirstXbin != GetXaxis()->GetFirst() || mOldLastXbin != GetXaxis()->GetLast())
    {
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1) 
        {printf("VDyPlotPaint>ZOOMX> plot=%s=%s=pad%d  type=%s  gPadname=%s  gPadtitle=%s  mOldFirstXbin=%d  mOldLastXbin=%d  GetFirst=%d  GetLast=%d\n",
                GetName(), GetTitle(), numpad, mPlotType, gPad->GetName(), gPad->GetTitle(),  mOldFirstXbin, mOldLastXbin, GetXaxis()->GetFirst(), GetXaxis()->GetLast()); 
        }
     mOldFirstXbin = GetXaxis()->GetFirst();
     mOldLastXbin = GetXaxis()->GetLast();
     gLastModifiedPlot = this;
     strcpy(Dy->gLastTitle,gLastModifiedPlot->GetTitle()); 
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2) 
        {printf("VDyPlotPaint>ZOOMX> plot=%s=%s  GetMin=%g GetMax=%g  gLastModifiedPlot=%s\n",
                 GetName(), GetTitle(), GetMinimum(), GetMaximum(), gLastModifiedPlot->GetName());
        }
     if (strcmp(mPlotType,"TRFCT") == 0)
        {sscanf(GetName(),"h%ld",&num);
         if (num < 1000) sprintf(h2name,"h%d", num+1000);
         else            sprintf(h2name,"h%d", num-1000);
         VDyPlot *h2 = (VDyPlot *)(gROOT->Get(h2name));
         h2->GetXaxis()->SetRange(GetXaxis()->GetFirst(), GetXaxis()->GetLast());
         curpad = gPad;
         if (num < 1000) ((TPad *)gPad->GetMother())->cd(2);
         else            ((TPad *)gPad->GetMother())->cd(1);
         if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2) 
            {printf("VDyPlotPaint>TRFCT> num=%d  h2name=%s  gPadname=%s  gPadtitle=%s  gPadMotherName=%s  getfirst=%d  getlast=%d\n",
                    num, h2name, gPad->GetName(), gPad->GetTitle(), (gPad->GetMother())->GetName(), GetXaxis()->GetFirst(), GetXaxis()->GetLast());
            }
         gPad->Paint();
         gPad->Modified();
         gPad->Update();
         curpad->cd();
        }
    } 


// If a zoom on Y axis was made, set this plot as last modified plot

 if ((mOldMinY != GetMinimum()) || (mOldMaxY != GetMaximum()))
    {
     mPreviousMinY = mOldMinY;
     mPreviousMaxY = mOldMaxY;
     mOldMinY = GetMinimum();
     mOldMaxY = GetMaximum();
     gLastModifiedPlotY = this;
     strcpy(Dy->gLastTitleY,gLastModifiedPlotY->GetTitle()); 
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2) 
        {printf("VDyPlotPaint> ZOOMY plot=%s=%s  pad=%d  type=%s  mOldMinY=%g  mOldMaxY=%g  Uymin=%g Uymax=%g  gLastModifiedPlot=%s\n",
                 GetName(), GetTitle(), numpad, mPlotType, mOldMinY, mOldMaxY, gPad->GetUymin(), gPad->GetUymax(), gLastModifiedPlotY->GetName());
        }
    }

// If a zoom on Y axis was made, set this plot as last modified plot
// use GetUymax and GetUymin if GetMaximum and GetMinimum
// do not work for superposed plots (option "same").
// But take care if logy is selected: Uymax=1e-4 becomes Uymax=-4
/**
 if ((mOldMinY != gPad->GetUymin()) || (mOldMaxY != gPad->GetUymax()))
    {
     mPreviousMinY = mOldMinY;
     mPreviousMaxY = mOldMaxY;
     mOldMinY = gPad->GetUymin();
     mOldMaxY = gPad->GetUymax();
     gLastModifiedPlotY = this;
    }
**/

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>2) 
    {printf("VDyPlotPaint> Paint done for plot=%s=%s  pad=%d  type=%s  gLastModifiedPlotY=%s\n",
            GetName(), GetTitle(), numpad, mPlotType, gLastModifiedPlotY->GetName());
    }

 DyDebugTimeNeeded(dbgmsg,"stop",4);

}


//______________________________________________________________________________
void VDyPlot::ZoomAsLast()
{

// Zoom with the same range as the last plot

 if (gLastModifiedPlot)
    {GetXaxis()->SetRange(gLastModifiedPlot->GetXaxis()->GetFirst(), gLastModifiedPlot->GetXaxis()->GetLast());}
}


//______________________________________________________________________________
void VDyPlot::ExportAscii(const char* fileName)
{
// Create a file containing plot data in ASCII format: col1=x col2=y

 int i, nBin;
 double x, y, gtime;
 char realName[256];
 FILE *fp_ascii;

 DyDebugPrint(__FUNCTION__,0);

 if (strcmp(fileName,"Default_file_name_=_Plot_name") == 0) sprintf(realName,"%s.txt",GetTitle());
 else                                                       strcpy(realName, fileName);

 fp_ascii = fopen(realName,"w");
 if (fp_ascii == NULL) {printf("VDyPlotExportAscii> Could not open file %s \n", realName); return;}

 nBin = GetNbinsX();
 gtime = DyTimeFormatToGps(GetXaxis()->GetTimeFormat());

 for (i=1; i<nBin+1; i++)
    {
     x = GetBinCenter(i);
     y = GetBinContent(i);
     if (strstr(GetTitle(),"TIME") != NULL) fprintf(fp_ascii,"%f  %g \n", gtime+x, y);
     else                                   fprintf(fp_ascii,"%g  %g \n", x, y);
    }

 fclose(fp_ascii);
 printf("VDyPlotExportAscii> Exported data of %s into ASCII file %s \n", GetTitle(), realName);

 return;
}


//______________________________________________________________________________
void VDyPlot::Scale_And_Offset(Double_t scale, Double_t offset)
{
// Scale the plot in Y axis

 double value;
 double ymin, ymax, yminNew, ymaxNew;
 int i, imin, imax;

// imin = GetXaxis()->GetFirst();
// imax = GetXaxis()->GetLast();

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
    {printf("VDyPlotScale_And_Offset> scale=%g  offset=%g \n", scale, offset);}

 if (scale == 0.  && offset == 0.) return;
 if (scale == 0.) scale = 1.;

 imin = 0;
 imax = GetNbinsX();

 ymin    = GetMinimum();
 ymax    = GetMaximum();
 yminNew = scale * GetBinContent(imin);
 ymaxNew = scale * GetBinContent(imin);

 for (i=imin; i<=imax; i++)
    {
     value = offset + scale * GetBinContent(i);
     SetBinContent(i, value);
     if (yminNew > value) yminNew = value;
     if (ymaxNew < value) ymaxNew = value;
    }

 if (ymaxNew > ymax) SetMaximum(ymaxNew);
 if (yminNew < ymin) SetMinimum(yminNew);

}


//______________________________________________________________________________
void VDyPlot::Scale_AroundMean(Double_t scale, Double_t offset)
{
// Scale the plot in Y axis, around the mean value

 double value, ymean;
 double ymin, ymax, yminNew, ymaxNew;
 int i, imin, imax;

// imin = GetXaxis()->GetFirst();
// imax = GetXaxis()->GetLast();

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>0)
    {printf("VDyPlotScale_AroundMean> scale=%g  offset=%g \n", scale, offset);}

 if (scale == 0.  && offset == 0.) return;
 if (scale == 0.) scale = 1.;

 imin = 0;
 imax = GetNbinsX();

 ymean = 0.;
 ymin = GetMinimum();
 ymax = GetMaximum();
/*
 for (i=imin; i<=imax; i++)
    {
     value = GetBinContent(i);
     ymean += value;
    }
 ymean = ymean / (double)(imax-imin);
*/
 ymean = (ymax+ymin)/2.;

 yminNew = ymean + scale * (GetBinContent(imin) - ymean);
 ymaxNew = ymean + scale * (GetBinContent(imin) - ymean);
 for (i=imin; i<=imax; i++)
    {
     value = offset + ymean + scale * (GetBinContent(i) - ymean);
     SetBinContent(i, value);
     if (yminNew > value) yminNew = value;
     if (ymaxNew < value) ymaxNew = value;
    }

 if (ymaxNew > ymax) SetMaximum(ymaxNew);
 if (yminNew < ymin) SetMinimum(yminNew);

}


//______________________________________________________________________________
void VDyPlot::FFTOfPrimitive(int order)
{
// Superpose to the plot the FFT of the primitive of the signal

 double value, xmin, xmax, freq;
 int i, imin, imax;

   DyDebugPrint(__FUNCTION__,0);

   VDyPlot* h = this;

   if (strcmp(h->mPlotType,"FFT") != 0 && strcmp(h->mPlotType,"TRFCT") != 0 && strcmp(h->mPlotType,"COHE") != 0) return;
 
   imin = 0;
   imax = h->GetNbinsX();

   xmin = h->GetXaxis()->GetBinUpEdge(imin);
   xmax = h->GetXaxis()->GetBinUpEdge(imax);

// Update the pad
   TVirtualPad* pad = gPad;

   TH1F* h2 = new TH1F("diff", "diff", imax-imin, xmin, xmax);

   for (i=imin+1; i<=imax; i++)
      {freq  = h->GetXaxis()->GetBinUpEdge(i);
       if (freq <= 0.) continue; 
       if (order <= 1)      value = h->GetBinContent(i) / freq / TWOPI;
       else if (order == 2) value = h->GetBinContent(i) / freq / freq / TWOPI;
       else                 value = h->GetBinContent(i) / pow(freq, (double)order) / TWOPI;
       h2->SetBinContent(i,value);
      }

   h2->SetLineColor(kRed);
   h2->Draw("h same");

   if (pad->GetUymax() < h2->GetMaximum()) SetMaximum(h2->GetMaximum());
   if (pad->GetUymin() > h2->GetMinimum()) SetMinimum(h2->GetMinimum());
   pad->Modified();
}


//______________________________________________________________________________
void VDyPlot::FFTOfDerivative(int order)
{
// Superpose to the plot the FFT of the derivative of the signal

 double value, xmin, xmax, freq;
 int i, imin, imax;

   DyDebugPrint(__FUNCTION__,0);

   VDyPlot* h = this;
 
   if (strcmp(h->mPlotType,"FFT") != 0 && strcmp(h->mPlotType,"TRFCT") != 0 && strcmp(h->mPlotType,"COHE") != 0) return;

   imin = 0;
   imax = h->GetNbinsX();

   xmin = h->GetXaxis()->GetBinUpEdge(imin);
   xmax = h->GetXaxis()->GetBinUpEdge(imax);

// Update the pad
   TVirtualPad* pad = gPad;

   TH1F* h2 = new TH1F("FFTdiff", "FFTdiff", imax-imin, xmin, xmax);

   for (i=imin+1; i<=imax; i++)
      {freq  = h->GetXaxis()->GetBinUpEdge(i);
       if (freq < 0.) continue; 
       if (order <= 1)      value = h->GetBinContent(i) * freq * TWOPI;
       else if (order == 2) value = h->GetBinContent(i) * freq * freq * TWOPI;
       else                 value = h->GetBinContent(i) * pow(freq,(double)order) * TWOPI;
       h2->SetBinContent(i,value);
      }

   h2->SetLineColor(kRed);
   h2->Draw("h same");

   if (pad->GetUymax() < h2->GetMaximum()) SetMaximum(h2->GetMaximum());
   if (pad->GetUymin() > h2->GetMinimum()) SetMinimum(h2->GetMinimum());
   pad->Modified();
}


//______________________________________________________________________________
void VDyPlot::Differentiate()
{
// Superpose to the plot its differential

 double value1, value2, xmin, xmax, dx;
 int i, imin, imax;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = this;

   imin = 0;
   imax = h->GetNbinsX();

   xmin = h->GetXaxis()->GetBinLowEdge(imin);
   xmax = h->GetXaxis()->GetBinUpEdge(imax);

// Update the pad
   TVirtualPad* pad = gPad;

   dx = (xmax-xmin)/(double)(imax-imin);
   TH1F* h2 = new TH1F("diff", "diff", imax-imin, xmin-dx/2., xmax-dx/2.);

   for (i=imin; i<=imax-1; i++)
      {value1 = h->GetBinContent(i);
       value2 = h->GetBinContent(i+1);
       h2->SetBinContent(i, value2-value1);
      }
   h2->SetBinContent(imax-1, 0.);

   h2->SetLineColor(kRed);
   h2->Draw("h same");

   if (pad->GetUymax() < h2->GetMaximum()) SetMaximum(h2->GetMaximum());
   if (pad->GetUymin() > h2->GetMinimum()) SetMinimum(h2->GetMinimum());
   pad->Modified();
}


//______________________________________________________________________________
void VDyPlot::IntegrateUp()
{
// Superpose to the plot its integral summed from end of plot

 double value, xmin, xmax, ymean, dx;
 int i, imin, imax;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = this;

   imin = h->GetXaxis()->GetFirst();
   imax = h->GetXaxis()->GetLast();
   xmin = h->GetXaxis()->GetBinLowEdge(imin);
   xmax = h->GetXaxis()->GetBinUpEdge(imax);

// Update the pad
   TVirtualPad* pad = gPad;

   dx = (xmax-xmin)/(double)(imax-imin);
   TH1F* h2 = new TH1F("integralup", "integralup", imax-imin, xmin-dx/2, xmax-dx/2);

   ymean = 0.;
   for (i=imax-1; i>=imin; i--)
      {
       value = h->GetBinContent(i);
       ymean += value;
       h2->SetBinContent(i, ymean);
      }

   //scale h2 to the pad coordinates
/**
   Float_t rightmax = 1.1*h2->GetMaximum();
   Float_t scale = pad->GetUymax()/rightmax;
   h2->Scale(scale);
   h2->SetLineColor(50);
   h2->Draw("h same");
   
   //draw an axis on the right side
   TGaxis *axis2 = new TGaxis(pad->GetUxmax(),pad->GetUymin(),
                              pad->GetUxmax(), pad->GetUymax(),0,rightmax,510,"+L");
   axis2->SetLineColor(50);
   axis2->SetTextColor(50);
   axis2->Draw();
**/

   h2->SetLineColor(50);
   h2->Draw("h same");

   if (pad->GetUymax() < h2->GetMaximum()) SetMaximum(h2->GetMaximum());
   if (pad->GetUymin() > h2->GetMinimum()) SetMinimum(h2->GetMinimum());
   pad->Modified();
}


//______________________________________________________________________________
void VDyPlot::IntegrateDown()
{
// Superpose to the plot its integral summed from start of plot

 double value, xmin, xmax, ymean, dx;
 int i, imin, imax;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = this;

   imin = h->GetXaxis()->GetFirst();
   imax = h->GetXaxis()->GetLast();
   xmin = h->GetXaxis()->GetBinLowEdge(imin);
   xmax = h->GetXaxis()->GetBinUpEdge(imax);

// Update the pad
   TVirtualPad* pad = gPad;

   dx = (xmax-xmin)/(double)(imax-imin);
   TH1F* h2 = new TH1F("integraldown", "integraldown", imax-imin, xmin-dx/2., xmax-dx/2.);

   ymean = 0.;
   for (i=imin; i<imax; i++)
      {
       value = h->GetBinContent(i);
       ymean += value;
       h2->SetBinContent(i, ymean);
      }

   //scale h2 to the pad coordinates
/***
   Float_t rightmax = 1.1*h2->GetMaximum();
   Float_t scale = pad->GetUymax()/rightmax;
   h2->Scale(scale);
   h2->SetLineColor(50);
   h2->Draw("h same");
 
   //draw an axis on the right side
   TGaxis *axis2 = new TGaxis(pad->GetUxmax(), pad->GetUymin(),
                              pad->GetUxmax(), pad->GetUymax(),0,rightmax,510,"+L");
   axis2->SetLineColor(50);
   axis2->SetTextColor(50);
   axis2->Draw();
***/

   h2->SetLineColor(51);
   h2->Draw("h same");

   if (pad->GetUymax() < h2->GetMaximum()) SetMaximum(h2->GetMaximum());
   if (pad->GetUymin() > h2->GetMinimum()) SetMinimum(h2->GetMinimum());
   pad->Modified();
}


//______________________________________________________________________________
void VDyPlot::IntegrateRms()
{
// Superpose to the plot its rms integrated from end of plot

 double value, xmin, xmax, rms, dx;
 int i, imin, imax;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = this;

   imin = h->GetXaxis()->GetFirst();
   imax = h->GetXaxis()->GetLast();
   xmin = h->GetXaxis()->GetBinLowEdge(imin);
   xmax = h->GetXaxis()->GetBinUpEdge(imax);

// Update the pad
   TVirtualPad* pad = gPad;

   dx = (xmax-xmin)/(double)(imax-imin);
   TH1F* h2 = new TH1F("integratedRMS", "integratedRMS", imax-imin, xmin-dx/2, xmax-dx/2);

   rms = 0.;
   for (i=imax-1; i>=imin; i--)
      {
       value = h->GetBinContent(i);
       rms += value * value;
       h2->SetBinContent(i, sqrt(rms*dx));
      }

   h2->SetLineColor(50);
   h2->Draw("h same");

   if (pad->GetUymax() < h2->GetMaximum()) SetMaximum(h2->GetMaximum());
   if (pad->GetUymin() > h2->GetMinimum()) SetMinimum(h2->GetMinimum());
   pad->Modified();
}


//______________________________________________________________________________
void VDyPlot::SuperposeExternalPlot(const char *fileName, const char *plotName)
{
// Superpose to the plot an other one read from file

 double xmin, xmax;
 int imin, imax, ok, color;
 char dummy[LSTRING_MAX], *dytimestring;
 TPaveText *timepave;
 const char *basename;

   DyDebugPrint(__FUNCTION__,0);

   imin = GetXaxis()->GetFirst();
   imax = GetXaxis()->GetLast();
   xmin = GetXaxis()->GetBinLowEdge(imin);
   xmax = GetXaxis()->GetBinUpEdge(imax);

// Update the pad
   TVirtualPad* pad = gPad;

   TFile *fp = new TFile(fileName);

   if (fp == NULL)
      {printf("VDyPlotSuperPoseExternalPlot> file %s not found\n", fileName);
       return;
      }

   TH1 *h2 = (TH1 *)(fp->Get(plotName));

   if (h2 == NULL)
      {
       printf("VDyPlotSuperPoseExternalPlot> plot %s not found in file %s\n", plotName, fileName);
       printf("VDyPlotSuperPoseExternalPlot> Available plot in this file are:\n");
       fp->GetListOfKeys()->Print();
       TKey *key;
       TIter nextkey(fp->GetListOfKeys());
       while ((key = (TKey*)nextkey()))
         {
          TObject *obj = (TObject *)(key->ReadObj());
          if (obj->IsA()->InheritsFrom("TH1"))
             {TH1 *h = (TH1*)obj; 
              printf("VDyPlotSuperPoseExternalPlot> %s: %s=%s \n", obj->IsA()->GetName(), h->GetName(), h->GetTitle());
             }
          else printf("VDyPlotSuperPoseExternalPlot> %s \n", obj->IsA()->GetName());
         }
       fp->Close();
       return;
      }

   h2->SetDirectory(gROOT);
   fp->Close();

   basename = gSystem->BaseName(fileName);
   sprintf(dummy, "%s%s%s%s", GetName(), SEPTITLE, basename, REF_SUFFIX);
   h2->SetName(dummy);
   if (h2->GetLineColor() < 2) h2->SetLineColor(Dy->refplot_color);
   color = h2->GetLineColor();
   DyXformAddToRefList((char *)fileName,color);

   if (h2->GetXaxis()->GetTimeFormat())
      {
       dytimestring = (char *)malloc(256);
       ok = GetTimeString(dytimestring, h2->GetXaxis()->GetTimeFormat());
       if (ok)
          {timepave = new TPaveText(0.02, 0.06, 0.98, 0.11,"bNDC");
           sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
           timepave->SetName(dummy);
           timepave->AddText(dytimestring);
           timepave->SetTextFont(DY_FONT_HELVETICA);
           timepave->SetTextColor(color);
           timepave->SetFillColor(0);
           timepave->Draw("NDC");
          }
       free(dytimestring);
      }

   h2->Draw("h same");

   if (pad->GetUymax() < h2->GetMaximum()) SetMaximum(h2->GetMaximum());
   if (pad->GetUymin() > h2->GetMinimum()) SetMinimum(h2->GetMinimum());

   pad->Modified();
   return;
}


//______________________________________________________________________________
void VDyPlot::RemoveRefPlot()
{
// Delete the reference plots superposed to the plot

 int padmodified;
 TObject *obj;
 char filename[LSTRING_MAX], dummy[LSTRING_MAX], *str, *str2;
 TPaveText *timepave;

   DyDebugPrint(__FUNCTION__,0);

   TIter  next((gPad)->GetListOfPrimitives());
   padmodified = 0;

   while ((obj = next()))
       {
        if (obj->InheritsFrom(TH1::Class()))
           {
            TH1* href = (TH1 *)obj;
            str2 = (char *)strstr(href->GetName(),REF_SUFFIX);
            if (str2 == NULL) continue;
            str = (char *)strstr(href->GetName(),SEPTITLE);
            strncpy(filename, str+strlen(SEPTITLE), str2-str-strlen(SEPTITLE));
            filename[str2-str-strlen(SEPTITLE)] = '\0';
            sprintf(dummy,"DyRefString%s%s", SEPTITLE, filename);
            timepave = (TPaveText*)(gPad->FindObject(dummy));
            if (timepave) delete timepave;
            delete href;
            padmodified = 1;
           }
       }

   gPad->Modified();
   return;
}


//______________________________________________________________________________
int VDyPlot::GetTimeString (char *dytimestring, const char *timeformat)
{
// Get the GPS time string from the plot time format

 char dummy[256], *str;
 double gtime;

 gtime = DyTimeFormatToGps(timeformat);
 if (gtime == 0.) return(0);

 DyGpsToTimeString ((int)gtime, NULL, 0, 0, dytimestring);

 return(1);
}

