#!/bin/sh

LOCALDIR=`dirname ${0}`
DyServer=$LOCALDIR/../$UNAME/dataDisplay.exe
DYDIR=/virgoData/Dy/ddevent

if [ ${#} -eq 0 ] ; then
  echo " "
  echo " Syntax is:  $0  [event's GPS time] [duration] [type]."
  echo " "
  echo " This script starts a dataDisplay around the event's GPS time."
  echo " The plots will be centered on event's GPS time"
  echo " and default duration will be the one of the [type] selected (often 4 sec)"
  echo " Put duration=0, to get the default value."
  echo " "
  echo " Various types are available:"
  echo "   time (default): time plots of interesting channels"
  echo "   trend         : time plots of interesting channels in the trend data"
  echo "   ffttime       : time-frequency plots of interesting channels"
  echo "   environment   : time and coherence plots of environment channels"
  echo "   suspension    : time and coherence plots of suspensions channels"
  echo "   photodiode    : time-frequency and coherence plots of photodiode channels"
  echo "   injection     : time-frequency and coherence plots of beam source channels"
  echo "   lock          : time-frequency and coherence plots of locking control loops signals"
  echo "   alignement    : time-frequency and coherence plots of alignement control loops signals"
  echo " "
  echo " You can also provide directly a configuration's name."
  echo " For instance: $0 967013136 10 /virgoData/Dy/ddevent/ddevent_outliermoni.car"
  echo " will show the whitened dark fringe spectrogram from 967013131 to 967013141"
  echo " "
  exit 0 
fi

#------------------------------- Get arguments
GPS=$1
if [ "$GPS" -lt "700000000" ]; then
 echo " "
 echo "$GPS is not a valid GPS time"
 echo " "
 exit 0
fi

LOCKSTATUS=`${LOCALDIR}/../../../Help/v0r1/src/lock.sh $GPS | tail -1`
if [ "$LOCKSTATUS" != "1" ]; then
 echo " "
 echo " The interferometer is not locked for GPS=$GPS"
 echo " Do you want to continue?"
 read answer
 if [ "$answer" != "yes" ] && [ "$answer" != "y" ] ; then exit 0; fi
fi

TYPE=time
if [ "$3" != "" ]; then TYPE=$3; fi

JPGOUT=
if [ "$4" = "gif" ]; then JPGOUT=ddevent_$GPS.gif
elif [ "$4" != "" ]; then JPGOUT=ddevent_$GPS.jpg; fi


#------------------------------- Get dataDisplay configuration

CONFIG=${DYDIR}/ddevent_${TYPE}.car
if [ ! -f "$CONFIG" ]; then
 echo " "
 echo "Configuration file $CONFIG does not exist"
 echo "I try config $TYPE"
 CONFIG=$TYPE
 if [ ! -f "$CONFIG" ]; then
  echo "Configuration file $CONFIG does not exist"
  echo " " 
  exit 0
 fi
 echo " "
fi

#----------------------------- Choose duration and start dataDisplay

DURATION=`grep "\/\*>" $CONFIG | awk '{print $5}'`
if [ "$DURATION" = "-1" ] || [ "$DURATION" = "" ]; then DURATION=4; fi
if [ "$2" != "" ] && [ "$2" != "0" ]; then DURATION=$2; fi

GPSSTART=$((GPS-DURATION/2))

echo "Starting $DyServer : CONFIG=$CONFIG  GPSSTART=$GPSSTART  DURATION=$DURATION"

if [ "$JPGOUT" = "" ]; then
  $DyServer $CONFIG -f $GPSSTART -l $DURATION &
else
  echo "Running ... will output a picture $JPGOUT"
  $DyServer $CONFIG -f $GPSSTART -l $DURATION -o $JPGOUT -b -q
fi

