#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#define NFRAMEMAX           8

struct ShmInfo
{
  key_t key;                  /* key to sign the shared memory segment */
  int  shmid;                 /* id of the shared memory segment (deduced from the key) */
  void *segptr;               /*address shared memory segment */
  long size;                  /* size of the shared memory segment (bytes) */
  char version[64];           /* version of FdShm */
  long dynamic;               /* flag>0 if optimize shm for variable size frames */
  long statusSize;            /* size of the shm header (not used anymore) */
  long frameSizeMax;          /* maximum size of a frame in shm */
  long nFrameMax;             /* maximum number of frames in shm (if dynamic=0) */
  int latency;                /* max latency of the producer */

  long frameSh[NFRAMEMAX];    /* address of frames in shm */
  long frameSize[NFRAMEMAX];  /* size of frames in shm (bytes) */
  int  frameGPS[NFRAMEMAX];   /* GPS time of frames in shm */
  long iwrite;                /* write counter of the shm producer */

  long nConsumer;              /* number of consumers connected to shm */
  int  producerPid;            /* producer pid */
  char producerName[128];      /* producer Cm name */
  char outputName[128];        /* name of the common output of the consumers (triggermanager) */

  long iwriteMod;             /* write counter modulo number of frames in shm */
  long iwriteMax;             /* iwrite of the last frame in shm */
  long curSizeMax;            /* size of the current largest frame in shm */
};


int getshm(char shmid[][128]);
void   FdShmClose (struct ShmInfo *shm, char *name);
struct ShmInfo *FdShmOpen (char *idName, char *name);

/*------------------------------------------------------------------------*/

int main(long argc, char **argv)
{
 char shmid[16][128];
 char shmname[128];
 int i, nshm;
 struct ShmInfo *shm;

 nshm = getshm(shmid);

 printf("--------------------------------------------------\n");
 printf("List of shared memory segments on %s\n", getenv("HOST"));
 printf("No  shmID \t name\n");
 for (i=0; i<nshm; i++)
    {shm = FdShmOpen(shmid[i], "shmlist");
     if (shm) {strcpy(shmname, shm->producerName); FdShmClose(shm, "shmlist");}
     else     strcpy(shmname,"NOT ACCESSED");
     printf("%d  %s \t %s\n", i, shmid[i], shmname);
    }
 printf("--------------------------------------------------\n");
 printf("\n\n");

 return(0);
}

/*----------------------------------------*/
int getshm(char shmid[][128])
{
 FILE *fp;
 char buff[128], *str, *str2;
 int i;

 fp = popen("ipcs -m","r");

 i = 0;
 fgets(buff,128,fp);
 while (!feof(fp))
   {printf(">> %s",buff);
    if (strncmp(buff,"0x",2) == 0)
       {str = strstr(buff," ") + 1; str2 = strstr(str," ");
        *str2='\0'; strcpy(shmid[i],str);
        i++;
       }
    fgets(buff,128,fp);
   }

 pclose(fp);
 return(i);
}



/*------------------------------------------ FdShmClose --------*/

void FdShmClose (struct ShmInfo *shm, char *name)
{
 if (name == NULL  ||  shm == NULL) return;
 shm->nConsumer--;
 shmdt((void *)shm);

 return;
}

/*-------------------------------------------  FdShmOpen -------*/

struct ShmInfo *FdShmOpen (char *idName, char *name)
{
 struct ShmInfo *temp;
 void *segptr;
 key_t key;
 int shmid;

 shmid = atoi(idName);

 /*--- Attach (map) the shared memory segment into the current process ---*/

 if ((segptr = shmat(shmid, 0, 0)) == (void *)-1) return(NULL);
 if (segptr == NULL)                              return(NULL);

 temp = (struct ShmInfo *)segptr;
 temp->nConsumer++;
 printf ("FdShmOpen> New consumer %s (%ld) connected to shm %s (shmid=%d=%d)\n", name, temp->nConsumer, temp->producerName, shmid, temp->shmid);
 printf ("FdShmOpen> nframeMax=%d  iwrite=%d  version=%s \n", temp->nFrameMax, temp->iwrite, temp->version);

 return(temp);
}

