/**************************************************/
/*                                                */
/* File: URPlot.c                                 */
/* Copyright (C) 2003, D. Verkindt,               */
/*    X.Grave, F.Marion, B.Mours, D.Castellazzi   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#ifdef OSF1
#include "dysoundcard.h"
#else
#include <sys/soundcard.h>
#endif

#include "UMathL.h"
#include "UJobL.h"
#include "USignalL.h"
#include "URPlot.h"
#include "UArrayL.h"
#include "URootAccess.h"
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif

struct UJstructH *UHplotFFTUJstructH;
struct UJstructH *UHplotTimeUJstructH;
struct UJstructH *UHplot1DUJstructH;
struct UJstructH *UHplot2DUJstructH;
struct UJstructH *UHplot3DUJstructH;
struct UJstructH *UHpTfctUJstructH;
struct UJstructH *UHpCoheUJstructH;
struct UJstructH *UHpCorrUJstructH;
struct UJstructH *UHsoundUJstructH;
struct UJstructH *UHplotFFTTimeUJstructH;
struct UJstructH *UHpTfctTimeUJstructH;
struct UJstructH *UHpCoheTimeUJstructH;
struct UJstructH *UHplot1DTimeUJstructH;
struct UJstructH *UHplotRawUJstructH;
struct UJstructH *UHplotRawTimeUJstructH;
struct UJstructH *UHdummyUJstructH;


/*------------------------------------------------------ UHLibIni -*/
void UHLibIni()
/*--------------------------------------------------------------------*/
{
 static BOOL first=YES;

 if (first == NO) return;
 first = YES; 

 UHplotFFTUJstructH     = UJDicAddS ("UHplotFFT", UHplotFIni, UHplotFSim, UHplotFEnd, UHplotFRead, NULL);
 UHplotTimeUJstructH    = UJDicAddS ("UHplotTime", UHplotTIni, UHplotTSim, UHplotTEnd, UHplotTRead, NULL);
 UHplot1DUJstructH      = UJDicAddS ("UHplot1D", UHplot1DIni, UHplot1DSim, UHplot1DEnd, UHplot1DRead, NULL);
 UHplot2DUJstructH      = UJDicAddS ("UHplot2D", UHplot2DIni, UHplot2DSim, UHplot2DEnd, UHplot2DRead, NULL);
 UHplot3DUJstructH      = UJDicAddS ("UHplot3D", UHplot3DIni, UHplot3DSim, UHplot3DEnd, UHplot3DRead, NULL);
 UHpTfctUJstructH       = UJDicAddS ("UHpTfct", UHpTfctIni, UHpTfctSim,UHpTfctEnd, UHpTfctRead, NULL);
 UHpCoheUJstructH       = UJDicAddS ("UHpCohe", UHpCoheIni, UHpCoheSim,UHpCoheEnd, UHpCoheRead, NULL);
 UHpCorrUJstructH       = UJDicAddS ("UHpCorr", UHpCorrIni, UHpCorrSim,UHpCorrEnd, UHpCorrRead, NULL);
 UHsoundUJstructH       = UJDicAddS ("UHsound", UHsoundIni, UHsoundSim, UHsoundEnd, UHsoundRead, NULL);
 UHplotFFTTimeUJstructH = UJDicAddS ("UHplotFFTTime", UHplotFTIni, UHplotFTSim, UHplotFTEnd, UHplotFTRead, NULL);
 UHpTfctTimeUJstructH   = UJDicAddS ("UHpTfctTime", UHpTfctTIni, UHpTfctTSim, UHpTfctTEnd, UHpTfctTRead, NULL);
 UHpCoheTimeUJstructH   = UJDicAddS ("UHpCoheTime", UHpCoheTIni, UHpCoheTSim, UHpCoheTEnd, UHpCoheTRead, NULL);
 UHplot1DTimeUJstructH  = UJDicAddS ("UHplot1DTime", UHplot1DTIni, UHplot1DTSim, UHplot1DTEnd, UHplot1DTRead, NULL);
 UHplotRawUJstructH     = UJDicAddS ("UHplotRaw", UHplotRawIni, UHplotRawSim, UHplotRawEnd, UHplotRawRead, NULL);
 UHplotRawTimeUJstructH = UJDicAddS ("UHplotRawTime", UHplotRawTIni, UHplotRawTSim, UHplotRawTEnd, UHplotRawTRead, NULL);
 UHdummyUJstructH       = UJDicAddS ("UHdummy", NULL, UHdummySim, NULL, UHdummyRead, NULL);

 return;
}


/*----------------------------------------------------- UHdummyRead -----*/
void UHdummyRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHdummy *dummy;
 long i;

 UJ_READ_HEADER_WITHOUT_NAME(UHdummy, dummy);
 
 UJ_READ_LONG(dummy->iClock)
    
 UJCObAdd(dummy->iClock,dummy,UHdummyUJstructH,NULL);
  
 return;
} 


/*---------------------------------------------------- UHdummySim -----*/
void UHdummySim(dummy,time)
struct UHdummy *dummy;
long time;
/*--------------------------------------------------------------------*/
{
 return;
}


/*----------------------------------------------------- UHplotUpdateEnv ---*/
void UHplotUpdateEnv (int plotNum, char *plotName, char *plotOptions, char *refplotName, int refplotNum, int nsame,
                       char *title, int color, float *buff, int nBin, double *corrupt)
/*--------------------------------------------------------------------*/
{
 int j;

 /*--- Set color of the plot ---*/

 if (color > 0) RA_HSetColor(plotName, color);

 /*--- Set title of the plot ---*/
/**
 if (strstr(plotOptions,"SAME") == NULL)
    {if (nsame == 0) RA_HSetTitle(plotName, title);
     else            RA_HSetTitle(plotName,NULL);
    }
**/

 /*--- Set legend if several plots are superposed ---*/

 if (nsame > 0 && plotName != NULL)
    {if (strstr(plotOptions,"SAME") == NULL) RA_HSetLegend(plotName, NULL, 0, nsame);
     RA_HSetLegend(plotName, title, color, nsame);
    }

 /*--- Check if data is missing or corrupted ---*/

  if (buff)
     {
      if (strstr(plotOptions,"SAME") == NULL) *corrupt = 0.;
      for (j=0; j<nBin-1; j++)
         {if (buff[j] >= DY_NOCHANNEL)    *corrupt = DY_NOCHANNEL * 1.01;
          else if (buff[j] >= DY_NOFRAME) *corrupt = DY_NOFRAME * 1.01;
          else if (buff[j] >= DY_BADDATA) *corrupt = DY_BADDATA * 1.01;
         }
     }
  else
     {*corrupt = nBin;}

     /*--- Set warning string if data missing or corrupted ---*/

  if (*corrupt != 0)
    {
     RA_SetActivePad(refplotNum);
     if (*corrupt >= DY_NOCHANNEL)    RA_HSetWarningString (DYMSG_NOCHANNEL);
     else if (*corrupt >= DY_NOFRAME) RA_HSetWarningString (DYMSG_NOFRAME);
     else if (*corrupt >= DY_BADDATA) RA_HSetWarningString (DYMSG_BADDATA);
     else                             RA_HSetWarningString (NULL);
    }

 return;
}


/*----------------------------------------------------- UHplotSetTime ---*/
void UHplotSetTime (double origin, char *plotName, double deltat, int nAverage, int timeaxis, struct FrameH *frame)
/*--------------------------------------------------------------------*/
{
 int fractime;
 char title[256];
 char stringtime[40];
 time_t starttimeutc;
 double origin2;

 if (origin-(int)origin >= 0.9999) origin = (double)((int)(origin+0.1));

 if (frame) starttimeutc = DyConvertToUTC((int)origin, frame->ULeapS);
 else       starttimeutc = DyConvertToUTC((int)origin,0);

 strcpy(title,DYEMPTY);
 DyGpsToTimeString (origin, frame, deltat, nAverage, &title[0]);

 if (timeaxis == ON) RA_HSetTimeOffset(plotName,(double)starttimeutc+origin-(int)origin);
 RA_HSetTimeFlag(plotName,1);
 RA_HSetTimeString(plotName,title);

 return;
}


/*------------------------------------------------- UHoptDecode-----*/
void UHoptDecode(options,intervalle,plotOpt)
char *options, *plotOpt;
long *intervalle;
/*--------------------------------------------------------------------*/
/* This function split the option string into HPLOPT and HPLOT options*/
{
 sscanf(options,"%ld", intervalle);

 /*------ HPLOT options -------------*/

 if (strstr(options,"same")  != NULL)  strcat(plotOpt," SAME");
 if (strstr(options,"SAME")  != NULL)  strcat(plotOpt," SAME");
 if (strstr(options,"COL")   != NULL)  strcat(plotOpt," COLZ");
 if (strstr(options,"ARR")   != NULL)  strcat(plotOpt," ARR");
 if (strstr(options,"BOX")   != NULL)  strcat(plotOpt," BOX");  
 if (strstr(options,"CONT")  != NULL)  strcat(plotOpt," CONT");
 if (strstr(options,"LEGO")  != NULL)  strcat(plotOpt," LEGO");
 if (strstr(options,"SURF")  != NULL)  strcat(plotOpt," SURF");
 if (strstr(options,"SCAT")  != NULL)  strcat(plotOpt," SCAT");
/* new option x axis with time */
 if (strstr(options,"TIMX")  != NULL)  strcat(plotOpt," TIMX");

/* bits values shown on y axis of time plot */
 if (strstr(options,"BITS")   != NULL)  strcat(plotOpt," COLZ");
 if (strstr(options,"CROSSCOHE_FMIN") != NULL)  strcat(plotOpt," COLZ");

 return;
}


/*------------------------------------------------- UHoptPlopt ---------*/
void UHoptPlopt(options, numPicture, numsubpad)
char *options;
long numPicture;
int numsubpad;
/*--------------------------------------------------------------------*/
/* This function performed the HPLOPT calls following the options*/
{
 static long lastNum = -1;
 long num;

 if (numPicture < 0) {lastNum = -1; return;}
 
 if (numPicture > 0)  num = numPicture;
 else                 num = lastNum;
 lastNum = num;

 RA_SetActivePad(num);
 if (numsubpad) RA_SetActiveSubPad(numsubpad);

 RA_SetStyle("LINX");
 if (strstr(options,"LOGX") != NULL) {RA_SetStyle("LOGX");}
 RA_SetStyle("LINY");
 if (strstr(options,"LOGY") != NULL) {RA_SetStyle("LOGY");}
 RA_SetStyle("LINZ");
 if (strstr(options,"LOGZ") != NULL) {RA_SetStyle("LOGZ");}
/*
 RA_SetStyle("NSTA");
 if (strstr(options,"STAT") != NULL) {RA_SetStyle("STA ");}
*/
 RA_SetStyle("NGRI");
 if (strstr(options,"GRIDX") != NULL) {RA_SetStyle("GRIDX");}
 if (strstr(options,"GRIDY") != NULL) {RA_SetStyle("GRIDY");}

 return;
}


/*---------------------------------------------------- UHplot --------*/
void UHplot(name, numPicture, options, plotOpt, numsubpad, nsame)
char* name;
long numPicture;
char *options, *plotOpt;
int numsubpad;
int nsame;
/*--------------------------------------------------------------------*/
{
 static long lastNum = -1;
 long num;
 char dummy[2048];
 char *blank;
 int color;

 DyDebugPrint(__FUNCTION__,1);

 if (numPicture < 0) {lastNum = -1; return;}

 UJpauseAtTheEnd = YES;
 
 if (numPicture > 0)  num = numPicture;
 else                 num = lastNum;
 lastNum = num;

 RA_SetActivePad(num);
 if (numsubpad) RA_SetActiveSubPad(numsubpad);

/*
 UHoptPlopt(options, num, numsubpad);
*/

/**
 if (strstr(plotOpt, "SAME") == NULL || strstr(plotOpt, "same") == NULL)
    {sprintf(dummy,"%s same", plotOpt);
     RA_HDraw(name, dummy, nsame);
    }
 else
    {RA_HDraw(name, plotOpt, nsame);}
**/

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot> plot=%s=pad%d  nplot=%d  nplotdrawn=%d  plotOpt=%s  options=%s\n",
             name, numPicture, Dy->nplot, Dy->nplotdrawn, plotOpt, options);}

 RA_HDraw(name, plotOpt, nsame);

 return;
}



/*====================================================================================================*/
/*================================================ PLOT FFT ==========================================*/

/*---------------------------------------------------- UHplotFEnd -----*/
void UHplotFEnd(plotFFT)
struct UHplotFFT *plotFFT;
/*---------------------------------------------------------------------*/
{
 long j;
 double coef;
 double origin;

 if (plotFFT->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (plotFFT->nAverage > 0 && plotFFT->intervalle == 0)
    {
     RA_HReset(plotFFT->name);
     coef = plotFFT->norm / plotFFT->nAverage;
     for (j=0 ;j<plotFFT->nBin/2+1 ;j++)
        {RA_H1Fill(plotFFT->name, plotFFT->dnu * (double)j, sqrt(plotFFT->fft->dataD[j]*coef));}
    }

 if (Dy->plotsNotDisplayed == OFF)
    {
     /*--- Set time string and axis time offset ---*/
     origin = Dy->timeOrigin + plotFFT->tmin;
     UHplotSetTime (origin, plotFFT->name, plotFFT->nBin*plotFFT->dt, plotFFT->nAverage, OFF, UFrCurFrameH);

     /*--- Draw the plot ---*/
     UHplot(plotFFT->name, plotFFT->numPicture, plotFFT->options, plotFFT->plotOpt, 0, plotFFT->refplot->nsame);
     if (plotFFT->rmsflag > 0) UHplot(plotFFT->name_rms, 0, "SAME", " SAME", 0, plotFFT->refplot->nsame);

     /*--- Update plot environment ---*/
     RA_SetActivePad(plotFFT->refplot->numPicture);
     UHplotUpdateEnv (plotFFT->numPicture, plotFFT->name, plotFFT->options, plotFFT->refplot->name,
                      plotFFT->refplot->numPicture, plotFFT->refplot->nsame, plotFFT->title,
                      plotFFT->color, plotFFT->buff, plotFFT->nBin, &plotFFT->corrupt);
    }

 RA_SetActivePad(plotFFT->refplot->numPicture);
 RA_HUpdateEnd(plotFFT->name,plotFFT->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotFEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  nAverage=%d\n",
         plotFFT->name, plotFFT->title, plotFFT->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plotFFT->nAverage);
    }

 if (plotFFT->vect     != NULL)  UACDFree(plotFFT->vect);
 if (plotFFT->fft      != NULL)  UADFree(plotFFT->fft);
 if (plotFFT->window   != NULL)  UADFree(plotFFT->window);
 if (plotFFT->options  != NULL)  free(plotFFT->options);
 if (plotFFT->name_rms != NULL)  free(plotFFT->name_rms);
 if (plotFFT->buff     != NULL)  free(plotFFT->buff);

 if (plotFFT != plotFFT->refplot)
    {if (plotFFT->title    != NULL)  free(plotFFT->title);
     if (plotFFT->name     != NULL)  free(plotFFT->name);
     free(plotFFT);
    }

 return;
}


/*---------------------------------------------------- UHplotFIni -----*/
void UHplotFIni(plotFFT,dt)
struct UHplotFFT *plotFFT;
double dt;
/*--------------------------------------------------------------------*/
{
 long i, numplot, refnumplot;
 char *oldname, dummy[LSTRING_MAX], normString[32];
 static struct UHplotFFT *previous=NULL;
 static int color;

 if (plotFFT->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (plotFFT->resampFactor <= 0) plotFFT->resampFactor = 1;
/*
 if (plotFFT->smsPeriod > 0.) plotFFT->resampFactor = plotFFT->resampFactor * plotFFT->smsPeriod;
*/

 plotFFT->dt       = dt * (double)plotFFT->resampFactor;
 plotFFT->dnu      = 1./(plotFFT->nBin*plotFFT->dt);
 plotFFT->i        = 0;
 plotFFT->j        = 0;
 plotFFT->dNbin    = plotFFT->nBin/2;
 plotFFT->nAverage = 0;
 plotFFT->nRefresh = 0;
 plotFFT->previousGPS = 0;
 plotFFT->corrupt  = 0;
 plotFFT->iFirst   = ON;

 plotFFT->tmin = plotFFT->dt * (double)plotFFT->t0 * plotFFT->resampFactor;

 plotFFT->vect = UACDNew(1,plotFFT->nBin);
 plotFFT->fft  = UADNew(1,plotFFT->nBin/2+1);

 bzero(plotFFT->vect->data, plotFFT->vect->nBytes);
 bzero(plotFFT->fft->data, plotFFT->fft->nBytes);

 /*--- Define the Hanning window ---*/

 plotFFT->window  = UADNew(1,plotFFT->nBin);
 bzero(plotFFT->window->data,plotFFT->window->nBytes);
  
 for (i=0; i<plotFFT->nBin; i++)
     {plotFFT->window->dataD[i] = 0.5 * (1. + cos( TWOPI * 
                                  ((double)i - ((double)plotFFT->nBin - 1.)/2.)/
                                  ((double)plotFFT->nBin - 1.)));}
 plotFFT->norm = 0.;
 for (i=0; i < plotFFT->nBin; i++)
     {plotFFT->norm += plotFFT->window->dataD[i]*plotFFT->window->dataD[i];}

 if (plotFFT->hertz) plotFFT->norm = plotFFT->dt*plotFFT->dt / plotFFT->norm; 
 else                plotFFT->norm = plotFFT->dt / plotFFT->norm; 
 plotFFT->norm = plotFFT->norm * 2.; 

 /*---- decode the options string --------*/

 UHoptDecode(plotFFT->options, &plotFFT->intervalle, plotFFT->plotOpt);

 if (plotFFT->intervalle <= 0) plotFFT->intervalle = plotFFT->nBin;

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plotFFT->options,"SAME") == NULL) plotFFT->numPicture = DyGrafIni(0);
     else                                         plotFFT->numPicture = previous->numPicture;
     plotFFT->dNbin = plotFFT->intervalle;
    }

/*  Get the limits of the previous histogram in the pad if it is good  */ 
 oldname = RA_PadContainsGoodHist(plotFFT->numPicture,0, 1, plotFFT->nBin/2+1, 0, plotFFT->name);

 RA_H1NewReuse(plotFFT->numPicture,
               0,
               plotFFT->name,
               plotFFT->title,
               plotFFT->nBin/2+1,
               plotFFT->dnu*(-.5),
               plotFFT->dnu*((double)(plotFFT->nBin/2+1) - .5));

 if (plotFFT->rmsflag > 0)
    {
     plotFFT->name_rms = (char *) calloc(strlen(plotFFT->name)+10,sizeof(char));
     sprintf(plotFFT->name_rms,"%s_rms",plotFFT->name);
     plotFFT->title_rms = (char *) calloc(strlen(plotFFT->title)+10,sizeof(char));
     sprintf(plotFFT->title_rms,"%s_rms",plotFFT->title);

     RA_H1NewReuse(0,
                   0,
                   plotFFT->name_rms,
                   plotFFT->title_rms,
                   plotFFT->nBin/2+1,
                   plotFFT->dnu*(-.5),
                   plotFFT->dnu*((double)(plotFFT->nBin/2+1) - .5));
    }

 RA_SetTestTrigPad(plotFFT->numPicture, 0, plotFFT->intervalle * plotFFT->dt * plotFFT->maxRefresh);

 if (plotFFT->autoScale == 0 && plotFFT->yMin > 0. && plotFFT->yMax > 0.)
    {RA_HSetMinimum(plotFFT->name, plotFFT->yMin);
     RA_HSetMaximum(plotFFT->name, plotFFT->yMax);
    }

 if (!oldname)
    {if (plotFFT->fmax <= plotFFT->fmin) plotFFT->fmax = 0.5 / plotFFT->dt;
     RA_HSetXRange (plotFFT->name, plotFFT->fmin, plotFFT->fmax, plotFFT->dnu);
     UHoptPlopt (plotFFT->options, plotFFT->numPicture, 0);
    }

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plotFFT->name);

 RA_HSetStat(plotFFT->name, NULL);
 RA_HSetUnitsX(plotFFT->name, "Hz");

 plotFFT->buff = (float *)calloc(plotFFT->nBin+1, sizeof(float));
 if (plotFFT->buff == NULL) printf("UHplotFIni> plotFFT->buff malloc failed\n");

 sscanf(plotFFT->name, "h%ld", &numplot);

 if (strstr(plotFFT->options,"SAME") == NULL)
    {previous = plotFFT;
     previous->nsame = 0;
     plotFFT->refplot = previous;
     color = 1;
     Dy->refnumplot[numplot] = numplot;
    }
 else if (previous)
    {previous->nsame++;
     plotFFT->refplot = previous;
     color++;
     if (color == 8)  color = 40+color-7;
     if (color == 48) color = 30+color-47;
     sscanf(plotFFT->refplot->name, "h%ld", &refnumplot);
     Dy->refnumplot[numplot] = refnumplot;
    }

 RA_PadContainsGoodHist(1,0, 1, plotFFT->nBin/2+1, 0, plotFFT->name);

 plotFFT->decay = 0.;
 if (strstr(plotFFT->options,"DECAYFFT") != NULL)
    {plotFFT->decay = 1./(double)plotFFT->maxAverage;}

 plotFFT->color = color;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplotFIni> plot=%s=%s=pad%d  color=%d  refplot=%s=%s=pad%d \n",
            plotFFT->name, plotFFT->title, plotFFT->numPicture, plotFFT->color,
            plotFFT->refplot->name, plotFFT->refplot->title, plotFFT->refplot->numPicture);
    }

 return;
}



/*---------------------------------------------------- UHplotFIniFrame -----*/

void UHplotFIniFrame (struct UHplotFFT *plotFFT)
{
 long i;
 char *temp, *name1, *oldname, dummy[LSTRING_MAX], normString[32];
 struct FrAdcData *adc=NULL;
 struct FrVect *vect=NULL;
 long length;

 if (plotFFT->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Get Adc and vector ---*/

 temp   = strstr(plotFFT->title,SEPTITLE);
 length = strlen(plotFFT->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plotFFT->title, length);

 if (UFrCurFrameH)
    {adc  = FrAdcDataFind(UFrCurFrameH, name1);
     vect = FrameFindVect(UFrCurFrameH, name1);
    }

 if (adc == NULL  && vect == NULL)
    {adc  = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(Dy->Frame, name1);
    }
 free(name1);

 /*--- Set Slope and Bias ---*/

 plotFFT->slope = 1.;
 plotFFT->bias  = 0.;

 if (adc != NULL)
    {if (adc->slope != 0.) plotFFT->slope = adc->slope;
     plotFFT->bias  = adc->bias;
    }

 /*--- Set Units ---*/

 if (plotFFT->hertz) strcpy(normString,"Hz");
 else                strcpy(normString,"sqrt(Hz)");
 sprintf(dummy,"a.u./%s", normString);
 RA_HSetUnitsY(plotFFT->name, dummy);

 if (vect != NULL)
    {
     sprintf(dummy,"%s/%s",vect->unitY, normString);
     if (plotFFT->realUnits > 0)
        {RA_HSetUnitsY(plotFFT->name, dummy);}
     if (plotFFT->realUnits == 0 && plotFFT->slope == 1.  &&  plotFFT->bias == 0.)
        {RA_HSetUnitsY(plotFFT->name, dummy);}
    }
 else if (adc != NULL)
    {
     sprintf(dummy,"%s/%s",adc->units, normString);
     if (plotFFT->realUnits > 0)
        {RA_HSetUnitsY(plotFFT->name, dummy);}
     if (plotFFT->realUnits == 0 && plotFFT->slope == 1.  &&  plotFFT->bias == 0.)
        {RA_HSetUnitsY(plotFFT->name, dummy);}
    }

 return;
}


/*----------------------------------------------------- UHplotFRead --*/
void UHplotFRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplotFFT *plotFFT;
 long i;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplotFFT, plotFFT);
 
 UJ_READ_LONG(plotFFT->iClock)
 UJ_READ_LONG(plotFFT->id)
 if (!plotFFT->name) plotFFT->name = (char *) calloc(10,sizeof(char));
 sprintf(plotFFT->name,"h%ld",plotFFT->id);
 UJ_READ_S_CHAR(plotFFT->title)
 UJ_READ_LONG(plotFFT->nBinP2)
 UJ_READ_LONG(plotFFT->resampFactor)
 UJ_READ_LONG(plotFFT->t0)
 UJ_READ_STRUCT(USignal, plotFFT->input)
 UJ_READ_S_CHAR(plotFFT->options)

 i = 1; 
 plotFFT->nBin = (i << plotFFT->nBinP2);
 if (plotFFT->nBin <= 0) plotFFT->nBin = 2;
    
 UJCObAdd(plotFFT->iClock,plotFFT,UHplotFFTUJstructH,NULL);

 plotFFT->hidden = OFF;
 s = strstr(plotFFT->options,"HIDDEN");
 if (s != NULL)  plotFFT->hidden = ON;

 plotFFT->realUnits = 0;
 s = strstr(plotFFT->options,"REALUNITS");
 if (s != NULL)  plotFFT->realUnits = 1;

 plotFFT->maxAverage = 1;
 s = strstr(plotFFT->options,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plotFFT->maxAverage);

 plotFFT->maxRefresh = 1;
 s = strstr(plotFFT->options,"NFFT_REFRESH");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plotFFT->maxRefresh);
 if (plotFFT->maxRefresh <= 0) plotFFT->maxRefresh = 1;

 plotFFT->fmin = 0.;
 s = strstr(plotFFT->options,"FMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFT->fmin);

 plotFFT->fmax = 0.;
 s = strstr(plotFFT->options,"FMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFT->fmax);

 plotFFT->yMin = 0.;
 s = strstr(plotFFT->options,"YMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFT->yMin);

 plotFFT->yMax = 0.;
 s = strstr(plotFFT->options,"YMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFT->yMax);

 plotFFT->autoScale = 0;
 s = strstr(plotFFT->options,"AUTOSCALE");
 if (s != NULL)  plotFFT->autoScale = 1;

 plotFFT->nodc = 0;
 s = strstr(plotFFT->options,"NODC");
 if (s != NULL)  plotFFT->nodc = 1;

 plotFFT->hertz = 0;
 s = strstr(plotFFT->options,"HERTZ");
 if (s != NULL)  plotFFT->hertz = 1;

 plotFFT->rmsflag = 0;
 s = strstr(plotFFT->options,"RMSFFT");
 if (s != NULL)  plotFFT->rmsflag = 1;

/**
 plotFFT->smsPeriod = 0.;
 s = strstr(plotFFT->options,"SMSPERIOD");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFT->smsPeriod);
**/

 return;
}
 

/*--------------------------------------------------- UHplotFSim -----*/
void UHplotFSim(plotFFT,time)
struct UHplotFFT *plotFFT;
long time;
/*--------------------------------------------------------------------*/
{
 long  i, j, inter, nvalue;
 double *tab, *fft, coef, dc;
 double value, localMin, localMax;
 char title[256], stringtime[40], refname[256];
 time_t starttimeutc;
 double origin;
 double rmsSum, rms_value, deltat, normpsd;

 if (plotFFT->hidden == ON) return;

 if (time < plotFFT->t0)  return;

 if (plotFFT->iFirst == ON)
    {plotFFT->iFirst = OFF;
     UHplotFIniFrame(plotFFT);
    }

 /*--- Take the input value ---*/

 if (plotFFT->input == NULL) 
    {UFrUnpackSim(plotFFT->FrUnpack, 0);
     value = plotFFT->FrUnpack->out->value;
    }
 else
    {value = plotFFT->input->value;}

 if (plotFFT->realUnits != 0  &&  value < DY_BADDATA) 
    {value = value * plotFFT->slope + plotFFT->bias;}


 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reach resampFactor ---*/

 if (value < DY_BADDATA)         plotFFT->sumvalue += value;
 else if (value >= DY_NOCHANNEL) plotFFT->sumvalue  = DY_NOCHANNEL * 1.01 * plotFFT->resampFactor;
 else if (value >= DY_NOFRAME)   plotFFT->sumvalue  = DY_NOFRAME * 1.01 * plotFFT->resampFactor;
 else                            plotFFT->sumvalue  = DY_BADDATA * 1.01 * plotFFT->resampFactor;

 plotFFT->j++;

 if (plotFFT->j < plotFFT->resampFactor) return;


 /*---------- fill buffer -----------------*/

 plotFFT->buff[plotFFT->i] = plotFFT->sumvalue / plotFFT->resampFactor;
 plotFFT->i++;
 plotFFT->sumvalue = 0.;
 plotFFT->j = 0;

 /*---- do FFT and shift data if we reach the end of the plot --------*/

 if (plotFFT->i == plotFFT->nBin)
    {
     DyDebugPrint(__FUNCTION__,0);
     DyDebugTimeNeeded(__FUNCTION__,"start",1);

     plotFFT->nAverage++;
     plotFFT->nRefresh++;

     /*--- Remove the mean DC value if requested ---*/

     dc     = 0.;
     nvalue = 0;
     if (plotFFT->nodc > 0)
        {
         for (j=0 ;j<plotFFT->nBin; j++)
             {if (plotFFT->buff[j] < DY_BADDATA)
                 {dc += plotFFT->buff[j]; nvalue++;}
             }
         if (nvalue > 0) dc = dc / (double)nvalue;
        }

      /*--- Get data into FFT buffer ---*/

      tab = plotFFT->vect->dataD;

      for (i=0; i< plotFFT->nBin; i++)
         {
          if (plotFFT->buff[i] < DY_BADDATA)
             {inter = 2*i;
              tab[inter]   = (plotFFT->buff[i]-dc) * plotFFT->window->dataD[i];
              tab[inter+1] = 0.;
              plotFFT->lastGoodData = plotFFT->buff[i];
             }
          else
             {inter = 2*i;
              tab[inter]   = (plotFFT->lastGoodData-dc) * plotFFT->window->dataD[i];
              tab[inter+1] = 0.;
             }
         }

     /*--- Compute the FFT ---*/

     DyDebugTimeNeeded("UHplotFSim Compute FFT","start",2);

     UACDfft(plotFFT->vect, plotFFT->nBinP2);

     DyDebugTimeNeeded("UHplotFSim Compute FFT","stop",2);

     /*--- Compute averaged FFT module ---*/

     fft = plotFFT->fft->dataD;
     for (j=0 ;j<plotFFT->nBin/2+1 ;j++)
        {
         if (plotFFT->decay>0 && plotFFT->decay<1) fft[j] = (1.-plotFFT->decay) * fft[j] + plotFFT->decay * (tab[2*j]*tab[2*j] + tab[2*j+1]*tab[2*j+1]);
         else                                      fft[j] = fft[j] + (tab[2*j]*tab[2*j] + tab[2*j+1]*tab[2*j+1]);
        }
 
     /*--- Reset content of the plot ---*/

     RA_HReset(plotFFT->name);

     /*--- Compute normalization coefficient ---*/

     coef = plotFFT->norm / plotFFT->nAverage;
     if (plotFFT->decay>0 && plotFFT->decay<1) coef = coef / plotFFT->decay;

     /*--- Fill the plot of FFT module and compute min and max ---*/

     if (plotFFT->autoScale > 0) {plotFFT->localMin =  DY_BADDATA; plotFFT->localMax = -DY_BADDATA;}

     for (j=0 ;j<plotFFT->nBin/2+1 ;j++)
        {
         value = sqrt(plotFFT->fft->dataD[j] * coef);
         RA_H1Fill(plotFFT->name, plotFFT->dnu * (double) j, value);
         if (plotFFT->autoScale > 0)
            {if (plotFFT->localMin > value  &&  value < DY_BADDATA)  plotFFT->localMin = value;
             if (plotFFT->localMax < value  &&  value < DY_BADDATA)  plotFFT->localMax = value;
            }
        }

     /*--- Superpose the integrated rms ---*/

    if (plotFFT->rmsflag > 0)
       {
        RA_HSetColor(plotFFT->name_rms, 50);
        RA_HReset(plotFFT->name_rms);

        rmsSum = 0.;
        deltat = plotFFT->nBin * plotFFT->dt;
        for (j=plotFFT->nBin/2 ;j>=0 ;j--)
           {
            value = plotFFT->fft->dataD[j] * coef;
            rmsSum += value;
            rms_value = sqrt(rmsSum / deltat);
            RA_H1Fill(plotFFT->name_rms, plotFFT->dnu * (double) j, rms_value);
            if (plotFFT->autoScale > 0)
               {if (plotFFT->localMin > rms_value)  plotFFT->localMin = rms_value;
                if (plotFFT->localMax < rms_value)  plotFFT->localMax = rms_value;
               }
           }
       }

     /*--- Compute the min and max of all superposed plots and set them ---*/

     if (plotFFT->autoScale > 0)
        {
         if (plotFFT->refplot->localMin > plotFFT->localMin) plotFFT->refplot->localMin = plotFFT->localMin;
         if (plotFFT->refplot->localMax < plotFFT->localMax) plotFFT->refplot->localMax = plotFFT->localMax;
         if (Dy->plotsNotDisplayed == OFF &&  plotFFT->nRefresh == plotFFT->maxRefresh)
            {RA_HSetMinimum(plotFFT->refplot->name, plotFFT->refplot->localMin*0.8);
             RA_HSetMaximum(plotFFT->refplot->name, plotFFT->refplot->localMax*1.2);
            }
        }

     if (Dy->plotsNotDisplayed == OFF &&  plotFFT->nRefresh == plotFFT->maxRefresh)
        {
         /*--- Update the time string under the plot (and the time axis offset if needed) ---*/
         origin = Dy->timeOrigin + plotFFT->tmin;
         UHplotSetTime (origin, plotFFT->name, plotFFT->nBin*plotFFT->dt, plotFFT->nAverage, OFF, UFrCurFrameH);

         /*--- Draw the plot ---*/
         UHplot(plotFFT->name, plotFFT->numPicture, plotFFT->options, plotFFT->plotOpt, 0, plotFFT->refplot->nsame);

         /*--- Draw the rms plot ---*/
         if (plotFFT->rmsflag > 0) UHplot(plotFFT->name_rms, 0, "SAME", " SAME", 0, plotFFT->refplot->nsame);

         /*--- Update plot environment ---*/
         RA_SetActivePad(plotFFT->refplot->numPicture);
         UHplotUpdateEnv (plotFFT->numPicture, plotFFT->name, plotFFT->options, plotFFT->refplot->name,
                          plotFFT->refplot->numPicture, plotFFT->refplot->nsame, plotFFT->title,
                          plotFFT->color, plotFFT->buff, plotFFT->nBin, &plotFFT->corrupt);

         /*--- Update the canvas ---*/
         RA_HUpdate(plotFFT->numPicture, plotFFT->name, plotFFT->title, plotFFT->options);
        }

     /*--- Check user interaction with the data Display ---*/

     DyCheckRunning();

     /*--- Update parameters ---*/

     if (plotFFT->nRefresh == plotFFT->maxRefresh) plotFFT->nRefresh = 0;
     if (plotFFT->decay>0 && plotFFT->decay<1)
        {if (plotFFT->nAverage >= plotFFT->maxAverage) plotFFT->nAverage = plotFFT->maxAverage-1;}
     else
        {if (plotFFT->nAverage == plotFFT->maxAverage)
            {plotFFT->nAverage = 0;
             for (j=0 ;j<plotFFT->nBin/2+1 ;j++) plotFFT->fft->dataD[j] = 0.;
            }
        }
    DyDebugTimeNeeded(__FUNCTION__,"stop",1);
   }

 /*--- Shift buffer and shift the plot's start time ---*/

 if (plotFFT->i == plotFFT->nBin)
    {for (j=0; j<plotFFT->nBin-plotFFT->dNbin; j++) plotFFT->buff[j] = plotFFT->buff[j+plotFFT->dNbin];
     plotFFT->i    = plotFFT->i - plotFFT->dNbin;
     plotFFT->tmin = plotFFT->tmin + plotFFT->dt * plotFFT->dNbin;
    }

 /*--- Check missing frames and update tmin accordingly ---*/

 if (UFrCurFrameH)
    {
     if ((plotFFT->previousGPS > 0) &&  ((double)(UFrCurFrameH->GTimeS-plotFFT->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {plotFFT->tmin += (double)(UFrCurFrameH->GTimeS - plotFFT->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) plotFFT->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*================================================ PLOT TIME =========================================*/

/*---------------------------------------------------- UHplotTEnd -----*/
void UHplotTEnd(plotTime)
struct UHplotTime *plotTime;
/*--------------------------------------------------------------------*/
{
 double tmax;
 double dc;
 int j, delta_past;
 double localMin, localMax;
 char title[256];
 char stringtime[40];
 double hours, minutes, seconds;
 time_t starttimeutc;
 double newMin, newMax;
 double origin, deltaMinMax;
 int fractime, i;

 if (plotTime->hidden == ON) return;

 delta_past = plotTime->nBin - plotTime->i;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplotTEnd> plot %s : i=%d  dNbin=%d  nBin=%d  mod(i,dNbin)=%d  "
            "plotTime->buffEndFilled=%d  delta_past=%d\n",
             plotTime->title, plotTime->i, plotTime->dNbin, plotTime->nBin,
             plotTime->i % plotTime->dNbin, plotTime->buffEndFilled, plotTime->nBin - plotTime->i);
    }

 /*--- Add last data in the buffer if needed ---*/

 if (plotTime->i > 0 && plotTime->buffEndFilled > 0  &&  delta_past <= plotTime->dNbin)
    {for (j=plotTime->i-1 ; j>=0; j--) plotTime->buff[j+delta_past] = plotTime->buff[j];
     for (j=0; j<delta_past; j++)      plotTime->buff[j] = plotTime->buffEnd[j+plotTime->dNbin-delta_past];
     plotTime->tmin -= plotTime->dt * delta_past * plotTime->resampFactor;
    }
 else if (plotTime->i > 0)
    {for (j=plotTime->i ;j<plotTime->nBin; j++) plotTime->buff[j] = 0.;}


 /*--- Last draw of plots if needed ---*/

 if (plotTime->i > 0 && plotTime->i % plotTime->dNbin >= 0 && Dy->skipFrame == OFF &&
     Dy->triggerType != TRIG_ASCII && Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     /*--- Compute min and max and set them ---*/

     if (plotTime->autoScale > 0)
        {plotTime->localMin =  DY_BADDATA;
         plotTime->localMax = -DY_BADDATA;
         for (j=0; j<plotTime->i; j++)
            {if (plotTime->localMin > plotTime->buff[j]  &&  plotTime->buff[j] < DY_BADDATA)  plotTime->localMin = plotTime->buff[j];
             if (plotTime->localMax < plotTime->buff[j]  &&  plotTime->buff[j] < DY_BADDATA)  plotTime->localMax = plotTime->buff[j];
            } 
         if (plotTime->refplot->localMin > plotTime->localMin) plotTime->refplot->localMin = plotTime->localMin;
         if (plotTime->refplot->localMax < plotTime->localMax) plotTime->refplot->localMax = plotTime->localMax;
        }

     if (plotTime->autoScale > 0  &&  plotTime->bits != 1)
        {deltaMinMax = (plotTime->refplot->localMax-plotTime->refplot->localMin)/2.;
         RA_HSetMinimum(plotTime->refplot->name, plotTime->refplot->localMin - deltaMinMax);
         RA_HSetMaximum(plotTime->refplot->name, plotTime->refplot->localMax + deltaMinMax);
        }

     /*--- Put data in the plot ---*/

     if (plotTime->bits == 1) RA_H2PackBits(plotTime->name, plotTime->buff);
     else                     RA_H1Pack(plotTime->name, plotTime->buff);

     /*--- Visualize data ---*/

     if (Dy->plotsNotDisplayed == OFF)
        {
         /*--- Set time string and time axis offset ---*/
         origin = Dy->timeOrigin + plotTime->tmin - plotTime->timeOffset;
         UHplotSetTime (origin, plotTime->name, 0, 0, ON, UFrCurFrameH);

         /*--- Draw the plot ---*/
         UHplot(plotTime->name, plotTime->numPicture, plotTime->options, plotTime->plotOpt, 0, plotTime->refplot->nsame);

         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("UHplotTEnd> plotTime=%s  plotTime->refplot=%s  color=%d  dNbin=%d i=%d nBin=%d\n",
                     plotTime->title, plotTime->refplot->title, plotTime->color, plotTime->dNbin, plotTime->i, plotTime->nBin);
            }

         /*--- Update plot environment ---*/
         RA_SetActivePad(plotTime->refplot->numPicture);
         UHplotUpdateEnv (plotTime->numPicture, plotTime->name, plotTime->options, plotTime->refplot->name,
                          plotTime->refplot->numPicture, plotTime->refplot->nsame, plotTime->title,
                          plotTime->color, plotTime->buff, plotTime->nBin, &plotTime->corrupt);

         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1) 
            {printf("UHplotTEnd> plot %s (%s): i=%d  nBin=%d  dNbin=%d  tmin=%g \n",
                     plotTime->title, plotTime->name, plotTime->i, plotTime->nBin, plotTime->dNbin,plotTime->tmin);
            }
        }
    }

 /*--- Do the last update of plot painting ---*/

 RA_SetActivePad(plotTime->refplot->numPicture);
 RA_HUpdateEnd(plotTime->name,plotTime->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotTEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  color=%d  (refplot=%s=%s=pad%d)\n",
             plotTime->name, plotTime->title, plotTime->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plotTime->color,
             plotTime->refplot->name, plotTime->refplot->title, plotTime->refplot->numPicture);
    }

 /*--- free buffers and strings ---*/
 if (plotTime)
    {
     if (plotTime->buff     != NULL)  free(plotTime->buff);
     if (plotTime->buffEnd  != NULL)  free(plotTime->buffEnd);
     if (plotTime->nodc)
        {if (plotTime->buffWithDC != NULL) free(plotTime->buffWithDC);}

     if (strstr(plotTime->options,"SAME") != NULL)
        {if (plotTime->name  != NULL)  free(plotTime->name);
         if (plotTime->title != NULL)  free(plotTime->title);
         if (plotTime->options  != NULL)  free(plotTime->options);
         free(plotTime);
        }
     else
        {if (plotTime->options  != NULL)  free(plotTime->options);}
    }

 return;
}


/*---------------------------------------------------- UHplotTIni -----*/
void UHplotTIni(plotTime,dt)
struct UHplotTime *plotTime;
double dt;
/*--------------------------------------------------------------------*/
{
 long length, numplot, refnumplot;
 double tmax;
 char *oldname;
 double f0, w0, r, fs, df;
 double offset, framedt;
 static struct UHplotTime *previous=NULL;
 static int color;
 int j;

 if (plotTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Initialize parameters ---*/

 if (plotTime->bits == 1) plotTime->resampFactor = 1;

 if (plotTime->resampFactor <= 0) plotTime->resampFactor = 1;
 if (plotTime->smsPeriod > 0.) plotTime->resampFactor =  plotTime->resampFactor * plotTime->smsPeriod;

 if (plotTime->resampFactor%2 == 1 && plotTime->minmax_resample > 0) plotTime->resampFactor++;
 if (plotTime->resampFactor > 1 && plotTime->minmax_resample > 0)    plotTime->resampFactor = plotTime->resampFactor / 2;

 plotTime->iFirst   = ON;
 plotTime->nBin     = plotTime->nBin / plotTime->resampFactor;
 if (plotTime->smsPeriod > 0.) plotTime->nBin =  plotTime->nBin * plotTime->smsPeriod;
 plotTime->dt       = dt;
 plotTime->i        = 0;
 plotTime->j        = 0;
 plotTime->sumvalue = 0.;
 plotTime->previousGPS = 0;
 plotTime->dc       = 0.;
 plotTime->dNbin    = plotTime->nBin/2;
 plotTime->t0       = 0;
 plotTime->timeOffset = 0.;
 plotTime->buffEndFilled = 0;

 tmax = plotTime->dt * (double)plotTime->nBin * plotTime->resampFactor;


 /*---- Decode the options string --------*/

 UHoptDecode(plotTime->options, &plotTime->intervalle, plotTime->plotOpt);

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plotTime->options,"SAME") == NULL)  plotTime->numPicture = DyGrafIni(0);
     else                                           plotTime->numPicture = previous->numPicture;
     if (plotTime->intervalle > 0) plotTime->dNbin = plotTime->intervalle;
    }
 if (plotTime->dNbin <= 0) plotTime->dNbin = 1;

/*--- Initialize pass band filter parameters ---*/

 if (plotTime->filterMin < 0.)                   plotTime->filterMin = -plotTime->filterMin;
 if (plotTime->filterMax <= plotTime->filterMin) plotTime->filterMax = plotTime->filterMin;

 if (plotTime->filterMax != 0.  ||  plotTime->filterMin != 0.)
    {df     = plotTime->filterMax - plotTime->filterMin;
     f0     = (plotTime->filterMax + plotTime->filterMin) / 2.;
     fs     = (double)plotTime->resampFactor / plotTime->dt;
     r      = 1. - PI * df/fs;
     w0     = 2. * PI * f0/fs;
     plotTime->a1   = -2.*r*cos(w0);
     plotTime->a2   = r*r;
     plotTime->G    = (1.-r) * sqrt(1.-2.*r*cos(2.*w0)+r*r);
     plotTime->y[0] = 0.;
     plotTime->y[1] = 0.;
     plotTime->y[2] = 0.;
    }

 /*--- Clear pad and create new plot ---*/

 if (plotTime->bits == 1)
    {
     oldname = RA_PadContainsGoodHist(plotTime->numPicture, 0, 2, plotTime->nBin, 32, plotTime->name);
     if (!oldname)
       {RA_PadCompleteClear(plotTime->numPicture);
        UHoptPlopt(plotTime->options, plotTime->numPicture, 0);
       }
     RA_H2NewReuse(plotTime->numPicture, 0, plotTime->name, plotTime->title, plotTime->nBin, 0, tmax, 32, 0., 32.);
     RA_HSetMinimum(plotTime->name, 0);
    }

 else
    {
     /*  Get the limits of the previous histogram in the pad if it is good  */ 
     oldname = RA_PadContainsGoodHist(plotTime->numPicture, 0, 1, plotTime->nBin, 0, plotTime->name);
     RA_H1NewReuse(plotTime->numPicture, 0, plotTime->name, plotTime->title, plotTime->nBin, 0., tmax);
     if (!oldname)
        {if (plotTime->autoScale == 0) RA_HSetMinimum(plotTime->name, plotTime->yMin);
         if (plotTime->autoScale == 0) RA_HSetMaximum(plotTime->name, plotTime->yMax);
         UHoptPlopt(plotTime->options, plotTime->numPicture, 0);
        }
    }

 /*--- Test if this plot will trigger the canvas update ---*/

 RA_SetTestTrigPad(plotTime->numPicture, 0, (double)plotTime->intervalle * dt * (double)plotTime->resampFactor);

 /*--- Set a different x axis Ndivision if plot is over a long time scale ---*/

 if (tmax > 3600.)  RA_SetAxisNdiv(plotTime->name,505);

 /*--- Set a faster font if dataDisplay is running online ---*/

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plotTime->name);

 /*--- No stat box in this plot ---*/

 RA_HSetStat(plotTime->name, NULL);

 /*--- Allocate buffers for data (buff), for last time slice (buffEnd) and for data with DC mean value (buffWithDC) ---*/

 plotTime->buff = (float *)calloc(plotTime->nBin+1, sizeof(float));
 if (plotTime->buff == NULL) printf("UHplotTIni> plotTime->buff malloc failed\n");
 plotTime->buffEnd = (float *)calloc(plotTime->dNbin+1, sizeof(float));
 if (plotTime->buffEnd == NULL) printf("UHplotTIni> plotTime->buffEnd malloc failed\n");

 if (plotTime->nodc)
    {plotTime->buffWithDC = (float *)calloc(plotTime->nBin+1, sizeof(float));
     if (plotTime->buffWithDC == NULL) printf("UHplotTIni> plotTime->buffWithDC malloc failed\n");
    }

 /*--- Get plot number from its name ---*/

 sscanf(plotTime->name, "h%ld", &numplot);

 /*--- Set first plot as reference plot ---*/

 if (strstr(plotTime->options,"SAME") == NULL)
    {previous = plotTime;
     previous->nsame = 0;
     plotTime->refplot = previous;
     color = 1;
     Dy->refnumplot[numplot] = numplot;
    }

 /*--- Set color of superposed plots and set pointer to reference plot ---*/

 else if (previous)
    {previous->nsame++;
     plotTime->refplot = previous;
     color++;
     if (color == 16)  color = 40+color-15;
     if (color == 48) color = 30+color-47;
     sscanf(plotTime->refplot->name, "h%ld", &refnumplot);
     Dy->refnumplot[numplot] = refnumplot;
    }

 plotTime->color = color;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplotTIni> plot=%s=%s=pad%d  color=%d  refplot=%s=%s=pad%d\n",
             plotTime->name, plotTime->title, plotTime->numPicture, plotTime->color,
             plotTime->refplot->name, plotTime->refplot->title, plotTime->refplot->numPicture);
    }

 return;
}




/*---------------------------------------------------- UHplotTIniFrame -----*/

void UHplotTIniFrame (struct UHplotTime *plotTime)
{
 long length;
 struct FrAdcData *adc=NULL;
 struct FrVect *vect=NULL;
 char *temp, *name1, *oldname;
 double offset, framedt;
 int j;

 if (plotTime->hidden == ON) return;

 if (UFrCurFrameH == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 plotTime->iFirst = OFF;

 if (plotTime->resampFactor <= 0) plotTime->resampFactor = 1;

 /*--- Get Adc and vector ---*/

 temp   = strstr(plotTime->title,SEPTITLE);
 length = strlen(plotTime->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plotTime->title, length); 

 if (UFrCurFrameH)
    {adc  = FrAdcDataFind(UFrCurFrameH, name1);
     if (adc == NULL) adc = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(UFrCurFrameH, name1);
     if (vect == NULL) vect = FrameFindVect(Dy->Frame, name1);
    }

 if (adc == NULL  &&  vect == NULL)
    {adc  = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(Dy->Frame, name1);
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1) 
        {printf("UHplotTIniFrame> Using DyFrame: name1=%s  adc=%x  vect=%x\n", name1, adc, vect);}
    }
 free(name1);

 /*--- Set Slope and Bias ---*/

 plotTime->slope  = 1.;
 plotTime->bias   = 0.;

 if (adc != NULL)
    {if (adc->slope != 0.) plotTime->slope = adc->slope;
     plotTime->bias  = adc->bias;
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
        {printf("UHplotTIniFrame> slope=%g  bias=%g\n", plotTime->slope, plotTime->bias);}
    }


 /*--- Set starting time offset if data is taken offline ---*/

 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     if (UFrCurFrameH != NULL)
        {framedt = DyGetFramedt(UFrCurFrameH);
         offset = ((double)Dy->timeOrigin - (double)UFrCurFrameH->GTimeS) / framedt;
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
            {printf("UHplotTIniFrame> offset=%g (%ld-%d)  dt=%f\n", offset, Dy->timeOrigin, UFrCurFrameH->GTimeS, framedt);}

         if (offset>0.) offset = offset - (int)offset;
         if (fabs(offset*UFrCurFrameH->dt) < 1.)
            {if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
                {printf("UHplotTIniFrame>  offset*dt=%.12f < 1. Forced offset=0.\n", offset*UFrCurFrameH->dt);}
             offset = 0.;
            }
        }
     else                     
        {
         printf("UHplotTIniFrame> UFrCurFrameH=NULL. Forced offset=0.\n");
         offset = 0.;
        }

     plotTime->timeOffset = offset * framedt * plotTime->resampFactor;

     if (offset > 0.)
        {if (vect) plotTime->t0 = (int)(offset*framedt / vect->dx[0]);
         else      plotTime->t0 = (int)(offset*framedt);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
            {printf("UHplotTIniFrame> OFFSET>0  startTime=%ld  offset=%.12f  t0=%ld  plotTime->timeOffset=%.12f ",
                     Dy->timeOrigin, offset, plotTime->t0, plotTime->timeOffset);
             if (UFrCurFrameH) printf("UHplotTIniFrame> frame->GTimeS=%d  frame->dt=%g ", UFrCurFrameH->GTimeS, UFrCurFrameH->dt);
             else              printf("UHplotTIniFrame> frame=NULL ");
             if (vect) printf("vect=%s:dx[0]=%g \n",vect->name, vect->dx[0]);
             else      printf("vect=NULL\n");
            }
        }
    }

 /*--- Set starting time ---*/

 plotTime->tmin = plotTime->dt * (double)plotTime->t0 * plotTime->resampFactor;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotTIniFrame> tmin=%.12f  offset=%.12f  t0=%ld  plotTime->timeOffset=%.12f\n",
             plotTime->tmin, offset, plotTime->t0, plotTime->timeOffset);
    }

 /*--- Set Units ---*/

 if (vect != NULL)
    {
     if (plotTime->realUnits > 0)
        {RA_HSetUnitsY(plotTime->name, vect->unitY);}
     else if (plotTime->realUnits == 0 && plotTime->slope == 1.  &&  plotTime->bias == 0.)
        {RA_HSetUnitsY(plotTime->name, vect->unitY);}
     else
        {RA_HSetUnitsY(plotTime->name, "");}
    }
 else if (adc != NULL)
    {
     if (plotTime->realUnits > 0)
        {RA_HSetUnitsY(plotTime->name, adc->units);}
     else if (plotTime->realUnits == 0 && plotTime->slope == 1.  &&  plotTime->bias == 0.)
        {RA_HSetUnitsY(plotTime->name, adc->units);}
     else
        {RA_HSetUnitsY(plotTime->name, "");}
    }

 /*--- Forbid 2d plot showing bits of the values if vector type is not integer ---*/

/***
 if (vect != NULL)
    {if (vect->type != FR_VECT_C  && vect->type != FR_VECT_2S && vect->type != FR_VECT_4S &&
         vect->type != FR_VECT_8S && vect->type != FR_VECT_2U && vect->type != FR_VECT_4U &&
         vect->type != FR_VECT_8U && vect->type != FR_VECT_1U && vect->type != FR_VECT_END) plotTime->bits = 0;
    }
***/

 /*--- Initialize previousGPS to deal with missing frames ---*/

 if (UFrCurFrameH) plotTime->previousGPS = UFrCurFrameH->GTimeS;

 return;
}


/*------------------------------------------------------ UHplotTRead --*/
void UHplotTRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplotTime *plotTime;
 char *s,dummy[64], *str;

 UJ_READ_HEADER_WITHOUT_NAME(UHplotTime,plotTime)
 
 UJ_READ_LONG(plotTime->iClock)
 UJ_READ_LONG(plotTime->id)
 if (!plotTime->name) plotTime->name = (char *) calloc(10,sizeof(char));
 sprintf(plotTime->name,"h%ld",plotTime->id);
 UJ_READ_S_CHAR(plotTime->title)
 UJ_READ_LONG(plotTime->nBin)
 UJ_READ_LONG(plotTime->resampFactor)
 UJ_READ_LONG(plotTime->t0)
 UJ_READ_STRUCT(USignal, plotTime->input) 
 UJ_READ_S_CHAR(plotTime->options)

 UJCObAdd(plotTime->iClock,plotTime,UHplotTimeUJstructH,NULL);

 plotTime->hidden = OFF;
 s = strstr(plotTime->options,"HIDDEN");
 if (s != NULL)  plotTime->hidden = ON;

 if (plotTime->title)
    {str = strstr(plotTime->title,SEP);
     if (str) {str[0] = '\0'; sprintf(plotTime->title, "%s_%s",plotTime->title, str+2);}
    }

 plotTime->minmax_resample = 0;
 s = strstr(plotTime->options,"MINMAX_RESAMPLE");
 if (s != NULL)  plotTime->minmax_resample = 1;

 plotTime->realUnits = 0;
 s = strstr(plotTime->options,"REALUNITS");
 if (s != NULL)  plotTime->realUnits = 1;

 plotTime->yMin = 0.;
 s = strstr(plotTime->options,"YMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotTime->yMin);

 plotTime->yMax = 0.;
 s = strstr(plotTime->options,"YMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotTime->yMax);

 plotTime->autoScale = 0;
 s = strstr(plotTime->options,"AUTOSCALE");
 if (s != NULL)  plotTime->autoScale = 1;

 plotTime->nodc = 0;
 s = strstr(plotTime->options,"NODC");
 if (s != NULL)  plotTime->nodc = 1;

 plotTime->filterMin = 0.;
 s = strstr(plotTime->options,"FILTERMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotTime->filterMin);

 plotTime->filterMax = 0.;
 s = strstr(plotTime->options,"FILTERMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotTime->filterMax);

 plotTime->yoffset = 0.;
 s = strstr(plotTime->options,"YOFFSET");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotTime->yoffset);

 plotTime->yscale = 0.;
 s = strstr(plotTime->options,"YSCALE");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotTime->yscale);

 plotTime->smsPeriod = 0.;
 s = strstr(plotTime->options,"SMSPERIOD");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotTime->smsPeriod);

 plotTime->bits == 0;
 s = strstr(plotTime->options,"BITS");
 if (s != NULL)  plotTime->bits = 1;

 if (plotTime->yMax <= plotTime->yMin)  plotTime->autoScale = 1;  

 if (plotTime->bits == 1)
    { plotTime->filterMin = 0.;
      plotTime->filterMax = 0.;
    }

 return;
} 


/*---------------------------------------------------- UHplotTSim -----*/
void UHplotTSim(plotTime,time)
struct UHplotTime *plotTime;
long time;
/*--------------------------------------------------------------------*/
{
 long   j, i, deltaBin, shift, imin;
 int  status=1;
 double dc, value;
 char title[256];
 char stringtime[40];
 time_t starttimeutc;
 double dataValue;
 double origin, deltaMinMax;
 int fractime;

/**
 if (time<10) printf("plotTSim START: 1 plot=%s  frame=%d  currenttime=%f  tmin=%f  t0=%d  time=%d  iFirst=%d \n",
                     plotTime->title, UFrCurFrameH->GTimeS, Dy->currentTime, plotTime->tmin, plotTime->t0, time, plotTime->iFirst);
**/
 if (plotTime->hidden == ON) return;
 if (plotTime->iFirst == ON) UHplotTIniFrame (plotTime);
 if (time < plotTime->t0) return;

 /*--- Take care of missing frames ---*/

 if (UFrCurFrameH)
    {
     if ((plotTime->previousGPS > 0)  &&  ((double)(UFrCurFrameH->GTimeS-plotTime->previousGPS) > UFrCurFrameH->dt)
         && ((double)(UFrCurFrameH->GTimeS-plotTime->previousGPS) > plotTime->smsPeriod))
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
            {printf("UHplotTSim> plot=%s=%s=pad%d  i=%d : Missed %.1f frames  (between %d and %d)\n",
                     plotTime->name, plotTime->title, plotTime->numPicture, plotTime->i,
                     (double)(UFrCurFrameH->GTimeS-plotTime->previousGPS)/UFrCurFrameH->dt-1.,
                     plotTime->previousGPS, UFrCurFrameH->GTimeS);
            }

         /*--- Deal with automatic pause used with trigger mode ---*/

         if (Dy->triggerUsed == ON && Dy->triggerPause == ON)
            {Dy->Order = PAUSE; while (Dy->Order == PAUSE) fl_check_forms();
             if (Dy->Order == STOP) return;
            }

         /*--- In case of missing frames, compute the shift to be done in the plot ---*/

         if (Dy->triggerUsed == ON || Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {
             deltaBin = (int)((double)(UFrCurFrameH->GTimeS-plotTime->previousGPS-UFrCurFrameH->dt) / (plotTime->dt * plotTime->resampFactor));
             if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1) printf("UHplotTSim> deltaBin=%d  nBin-i=%d-%d=%d\n", deltaBin, plotTime->nBin, plotTime->i, plotTime->nBin-plotTime->i);
             if (deltaBin < plotTime->nBin-plotTime->i)
                {for (i=plotTime->i; i<plotTime->i+deltaBin; i++)
                    {plotTime->buff[i] = DY_NOFRAME;
                     if (plotTime->nodc > 0) plotTime->buffWithDC[i] = DY_NOFRAME;
                    }
                 plotTime->i += deltaBin;
                 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1) printf("UHplotTSim> Case1: delta<nBin-i: i=%d \n", plotTime->i);
                }

             else
                {
                 shift = plotTime->i + deltaBin - plotTime->nBin;
                 if (shift < plotTime->nBin)
                    {for (i=0; i<plotTime->nBin-shift; i++)
                        {plotTime->buff[i] = plotTime->buff[i+shift];
                         if (plotTime->nodc > 0) plotTime->buffWithDC[i] = plotTime->buffWithDC[i+shift];
                        }
                    }
                 imin = plotTime->i - shift;

                 if (imin < 0) imin = 0;
                 for (i=imin; i<plotTime->nBin; i++)
                    {plotTime->buff[i] = DY_NOFRAME;
                     if (plotTime->nodc > 0) plotTime->buffWithDC[i] = DY_NOFRAME;
                    }
                 plotTime->i = plotTime->nBin-1;

                 plotTime->tmin += shift * plotTime->dt * plotTime->resampFactor;
                 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1) printf("UHplotTSim> Case2: shift=%d imin=%d  i=%d  tmin=%f \n", shift, imin, plotTime->i, plotTime->tmin);
                }
            }
        }
     if (UFrCurFrameH->GTimeS > 0) plotTime->previousGPS = UFrCurFrameH->GTimeS;
    }


 /*--- Get input data ---*/

 if (plotTime->input == NULL) 
    {UFrUnpackSim(plotTime->FrUnpack, 0);
     value = plotTime->FrUnpack->out->value;
    }
 else
    {value = plotTime->input->value;}

 if (plotTime->realUnits != 0  &&  value < DY_BADDATA) 
    {value = value * plotTime->slope + plotTime->bias;}

 if (value < DY_BADDATA)         plotTime->sumvalue += value;
 else if (value >= DY_NOCHANNEL) plotTime->sumvalue  = DY_NOCHANNEL * 1.01 * plotTime->resampFactor;
 else if (value >= DY_NOFRAME)   plotTime->sumvalue  = DY_NOFRAME * 1.01 * plotTime->resampFactor;
 else                            plotTime->sumvalue  = DY_BADDATA * 1.01 * plotTime->resampFactor;

 if (plotTime->minmax_resample > 0)
    {if (plotTime->j == 0) {plotTime->valueMin = DY_BADDATA; plotTime->valueMax = -DY_BADDATA;}
     if (value < DY_BADDATA)
        {if (value < plotTime->valueMin) plotTime->valueMin = value;
         if (value > plotTime->valueMax) plotTime->valueMax = value;
        }
     else if (value >= DY_NOCHANNEL) {plotTime->valueMin = DY_NOCHANNEL * 1.01; plotTime->valueMax = DY_NOCHANNEL * 1.01;}
     else if (value >= DY_NOFRAME)   {plotTime->valueMin = DY_NOFRAME * 1.01;   plotTime->valueMax = DY_NOFRAME * 1.01;}
     else                            {plotTime->valueMin = DY_BADDATA * 1.01;   plotTime->valueMax = DY_BADDATA * 1.01;}
    }

 plotTime->j++;

 if (plotTime->minmax_resample > 0) {if (plotTime->j < plotTime->resampFactor*2) return;}
 else                               {if (plotTime->j < plotTime->resampFactor) return;}


 /*--- Deal with resampling option using min and max instead of averaging ---*/
 /*--------------------------------------------------------------------------*/

 if (plotTime->minmax_resample > 0)
    {
     /*--- fill data and plot ---*/

     dataValue = plotTime->valueMin;

     /*--- remove DC (using a moving average) ---*/

     if (plotTime->nodc > 0  &&  dataValue < DY_BADDATA)
        {plotTime->buffWithDC[plotTime->i] = dataValue;
         plotTime->dc += dataValue;
         if (plotTime->i > plotTime->dNbin)
            {plotTime->dc -= plotTime->buffWithDC[plotTime->i-plotTime->dNbin];
             if (plotTime->dNbin > 0)  dc = plotTime->dc / plotTime->dNbin;
             else                      dc = 0.;
            }
         else
            {if (plotTime->i+1 > 0)  dc = plotTime->dc /(plotTime->i+1);
             else                    dc = 0.;
            }
         dataValue -= dc;
        }

     /*--- pass band filter on the data ---*/

     if (plotTime->filterMax > 0.)
        {if (dataValue < DY_BADDATA)
            {plotTime->y[0]  = dataValue * plotTime->G;
             plotTime->y[0] -= plotTime->a2 * plotTime->y[2];
             plotTime->y[2]  = plotTime->y[1];
             plotTime->y[0] -= plotTime->a1 * plotTime->y[1];
             plotTime->y[1] = plotTime->y[0];
             dataValue = plotTime->y[0];
            }
         else
            {plotTime->y[0] = 0.;
             plotTime->y[1] = 0.;
             plotTime->y[2] = 0.;
            }
        }
     if (plotTime->yscale  != 0 && dataValue < DY_BADDATA) dataValue *= plotTime->yscale;
     if (plotTime->yoffset != 0 && dataValue < DY_BADDATA) dataValue += plotTime->yoffset;
     plotTime->buff[plotTime->i] = dataValue;
     plotTime->i++;
     plotTime->j = 0;

     /*--- fill data and plot ---*/
 
    dataValue = plotTime->valueMax;

     /*--- remove DC (using a moving average) ---*/

     if (plotTime->nodc > 0  &&  dataValue < DY_BADDATA)
        {plotTime->buffWithDC[plotTime->i] = dataValue;
         plotTime->dc += dataValue;
         if (plotTime->i > plotTime->dNbin)
            {plotTime->dc -= plotTime->buffWithDC[plotTime->i-plotTime->dNbin];
             if (plotTime->dNbin > 0)  dc = plotTime->dc / plotTime->dNbin;
             else                      dc = 0.;
            }
         else
            {if (plotTime->i+1 > 0)  dc = plotTime->dc /(plotTime->i+1);
             else                    dc = 0.;
            }
         dataValue -= dc;
        }

     /*--- pass band filter on the data ---*/

     if (plotTime->filterMax > 0.)
        {if (dataValue < DY_BADDATA)
            {plotTime->y[0]  = dataValue * plotTime->G;
             plotTime->y[0] -= plotTime->a2 * plotTime->y[2];
             plotTime->y[2]  = plotTime->y[1];
             plotTime->y[0] -= plotTime->a1 * plotTime->y[1];
             plotTime->y[1] = plotTime->y[0];
             dataValue = plotTime->y[0];
            }
         else
            {plotTime->y[0] = 0.;
             plotTime->y[1] = 0.;
             plotTime->y[2] = 0.;
            }
        }
     if (plotTime->yscale  != 0 && dataValue < DY_BADDATA) dataValue *= plotTime->yscale;
     if (plotTime->yoffset != 0 && dataValue < DY_BADDATA) dataValue += plotTime->yoffset;
     plotTime->buff[plotTime->i] = dataValue;
     plotTime->i++;
     plotTime->j = 0;
    }

 /*--- Normal resampling  (use averaging of data) ---*/
 /*--------------------------------------------------*/

 else
    {
     /*--- fill data and plot ---*/

     dataValue = plotTime->sumvalue / plotTime->resampFactor;

     /*--- remove DC (using a moving average) ---*/

     if (plotTime->nodc > 0  &&  dataValue < DY_BADDATA)
        {
         plotTime->buffWithDC[plotTime->i] = dataValue;
         plotTime->dc += dataValue;
         if (plotTime->i > plotTime->dNbin)
            {plotTime->dc -= plotTime->buffWithDC[plotTime->i-plotTime->dNbin];
             if (plotTime->dNbin > 0)  dc = plotTime->dc / plotTime->dNbin;
             else                      dc = 0.;
            }
         else
            {if (plotTime->i+1 > 0)  dc = plotTime->dc /(plotTime->i+1);
             else                    dc = 0.;
            }
         dataValue -= dc;
        }

     /*--- pass band filter on the data ---*/

     if (plotTime->filterMax > 0.)
        {
         if (dataValue < DY_BADDATA)
            {
             plotTime->y[0]  = dataValue * plotTime->G;
             plotTime->y[0] -= plotTime->a2 * plotTime->y[2];
             plotTime->y[2]  = plotTime->y[1];
             plotTime->y[0] -= plotTime->a1 * plotTime->y[1];
             plotTime->y[1] = plotTime->y[0];
             dataValue = plotTime->y[0];
            }
         else
            {plotTime->y[0] = 0.;
             plotTime->y[1] = 0.;
             plotTime->y[2] = 0.;
            }
        }

     if (plotTime->yscale  != 0 && dataValue < DY_BADDATA) dataValue *= plotTime->yscale;
     if (plotTime->yoffset != 0 && dataValue < DY_BADDATA) dataValue += plotTime->yoffset;

     plotTime->buff[plotTime->i] = dataValue;
     plotTime->i++;
     plotTime->j = 0;
     plotTime->sumvalue = 0.;
    }

 /*--- Draw the plot each time a slice has been completed ---*/

 if ((plotTime->i % plotTime->dNbin == 0  &&  plotTime->buff[plotTime->nBin-1]==0. &&  plotTime->buff[plotTime->nBin-2]==0.) ||  plotTime->i == plotTime->nBin)
    {
     DyDebugPrint(__FUNCTION__,0);
     DyDebugTimeNeeded(__FUNCTION__,"start",1);

     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
        {printf("UHplotTSim> plot=%s=%s  i=%ld  dNbin=%ld  nBin=%ld  mod(i,dNbin)=%ld  Dy->unpacki=%d  nsame=%d  nready=%d\n",
                 plotTime->name, plotTime->title, plotTime->i, plotTime->dNbin, plotTime->nBin, plotTime->i % plotTime->dNbin,
                 Dy->unpacki, plotTime->refplot->nsame, Dy->nready);
        }
/**
     if (strstr(plotTime->options,"SAME") != NULL) 
        {
         RA_HDelete(plotTime->name);
         RA_H1NewReuse(plotTime->numPicture, 0, plotTime->name, plotTime->title, plotTime->nBin, 0.,
                       plotTime->dt * (double)plotTime->nBin * plotTime->resampFactor);
        }
**/
     /*--- Set color of the plot ---*/

     RA_HSetColor(plotTime->name, plotTime->color);

     /*--- Put data in the plot ---*/

     if (plotTime->bits == 1) RA_H2PackBits(plotTime->name, plotTime->buff);
     else                     RA_H1Pack(plotTime->name, plotTime->buff);

     /*--- Compute min and max ---*/

     if (plotTime->autoScale > 0 && plotTime->bits != 1)
        {
         plotTime->localMin =  DY_BADDATA;
         plotTime->localMax = -DY_BADDATA;
         for (j=0; j<plotTime->i; j++)
            {if (plotTime->localMin > plotTime->buff[j]  &&  plotTime->buff[j] < DY_BADDATA)  plotTime->localMin = plotTime->buff[j];
             if (plotTime->localMax < plotTime->buff[j]  &&  plotTime->buff[j] < DY_BADDATA)  plotTime->localMax = plotTime->buff[j];
            } 
         if (plotTime->refplot->localMin > plotTime->localMin) plotTime->refplot->localMin = plotTime->localMin;
         if (plotTime->refplot->localMax < plotTime->localMax) plotTime->refplot->localMax = plotTime->localMax;
        }


     /*--- Visualize data ---*/

     if (Dy->plotsNotDisplayed == OFF)
        {
         /*--- Set min and max ---*/

         if (plotTime->autoScale > 0  &&  plotTime->bits != 1)
            {deltaMinMax = (plotTime->refplot->localMax-plotTime->refplot->localMin)/2.;
             RA_HSetMinimum(plotTime->refplot->name, plotTime->refplot->localMin - deltaMinMax);
             RA_HSetMaximum(plotTime->refplot->name, plotTime->refplot->localMax + deltaMinMax);
             RA_HSetMinimum(plotTime->name, plotTime->refplot->localMin - deltaMinMax);
             RA_HSetMaximum(plotTime->name, plotTime->refplot->localMax + deltaMinMax);
           }

         /*--- Set time string and time axis offset ---*/

         origin = Dy->timeOrigin + plotTime->tmin - plotTime->timeOffset;
         UHplotSetTime (origin, plotTime->name, 0., 0, ON, UFrCurFrameH);

         /*--- Draw the plot ---*/

         UHplot(plotTime->name, plotTime->numPicture, plotTime->options, plotTime->plotOpt, 0, plotTime->refplot->nsame);

         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
            {printf("UHplotTSim> startTime=%ld  tmin=%.12f  timeOffset=%.12f  origin=%.12f \n",
                     Dy->timeOrigin, plotTime->tmin, plotTime->timeOffset, origin);
            }

         /*--- Update the plot environment ---*/

         UHplotUpdateEnv (plotTime->numPicture, plotTime->name, plotTime->options, plotTime->refplot->name,
                          plotTime->refplot->numPicture, plotTime->refplot->nsame, plotTime->title,
                          plotTime->color, plotTime->buff, plotTime->nBin, &plotTime->corrupt);

         /*--- Update the canvas ---*/
 
         RA_HUpdate(plotTime->numPicture, plotTime->name, plotTime->title, plotTime->options);
        }

     /*--- Check user interaction with the dataDisplay ---*/

     DyCheckRunning();

     DyDebugTimeNeeded(__FUNCTION__,"stop",1);
    }

 /*--- Shift buffers and shift the plot's start time tmin ---*/

 if (plotTime->i == plotTime->nBin)
    {
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>4  &&  plotTime->nBin < 256)
        {for (j=0; j<plotTime->nBin; j++) printf("UHplotTSim> buff[%d]=%g\n",j,plotTime->buff[j]);}

     for (j=0; j<plotTime->dNbin; j++)
        {plotTime->buffEnd[j] = plotTime->buff[j];}
     for (j=0; j<plotTime->nBin-plotTime->dNbin; j++)
        {plotTime->buff[j]    = plotTime->buff[j+plotTime->dNbin];}

     if (plotTime->nodc > 0)
        {
         for (j=0; j<plotTime->nBin-plotTime->dNbin; j++)
            {plotTime->buffWithDC[j] = plotTime->buffWithDC[j+plotTime->dNbin];}
        }

     plotTime->buffEndFilled = 1;
     plotTime->i = plotTime->i - plotTime->dNbin;
     plotTime->tmin = plotTime->tmin + plotTime->dt * plotTime->dNbin * plotTime->resampFactor;
    }

 return;
}



/*====================================================================================================*/
/*================================================ PLOT 1D ===========================================*/

/*----------------------------------------------------- UHplot1DEnd ---*/
void UHplot1DEnd(plot1D)
struct UHplot1D *plot1D;
/*--------------------------------------------------------------------*/
{
 long i;
 double localMax;
 static double maxofMax;

 if (plot1D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_H1NewReuse(plot1D->numPicture, 0, plot1D->name, plot1D->title, plot1D->nBin, plot1D->xMin, plot1D->xMax);

 RA_HSetColor(plot1D->name, plot1D->color);

 RA_HSetStat(plot1D->name, plot1D->options);
 RA_HSetUnitsX(plot1D->name, "a.u.");
 if (plot1D->realUnits > 0)
    {RA_HSetUnitsX(plot1D->name, plot1D->units);}
 if (plot1D->realUnits == 0 && plot1D->slope == 1.  &&  plot1D->bias == 0.)
    {RA_HSetUnitsX(plot1D->name, plot1D->units);}

 for (i=0; i<plot1D->nBin; i++) plot1D->buff[i] = (float)(plot1D->data[i]);
 localMax = 0;
 for (i=0; i<plot1D->nBin; i++)
    {if (localMax < plot1D->buff[i]) localMax = plot1D->buff[i];}

 if (plot1D->maxofMax < localMax)       plot1D->maxofMax = localMax;

 RA_H1Pack(plot1D->name, plot1D->buff);
    
 if (Dy->plotsNotDisplayed == OFF)
    {
     RA_HSetMaximum(plot1D->refplot->name, plot1D->refplot->maxofMax*1.2);

     UHplot(plot1D->name, plot1D->refplot->numPicture, plot1D->options, plot1D->plotOpt, 0, plot1D->refplot->nsame);

     RA_SetActivePad(plot1D->refplot->numPicture);
     if (plot1D->refplot->nsame > 0)
        {if (strstr(plot1D->options,"SAME") == NULL) RA_HSetLegend(plot1D->name, NULL, 0, plot1D->nsame);
         RA_HSetLegend(plot1D->name, plot1D->title, plot1D->color, plot1D->refplot->nsame);
        }
    }

 RA_SetActivePad(plot1D->refplot->numPicture);
 RA_HUpdateEnd(plot1D->name,plot1D->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot1DEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  color=%d \n",
             plot1D->name, plot1D->title, plot1D->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plot1D->color);
    }

 if (plot1D->buff)            free(plot1D->buff);
 if (plot1D->data)            free(plot1D->data);
 if (plot1D->options != NULL) free(plot1D->options);

 if (plot1D != plot1D->refplot)
    {if (plot1D->name     != NULL) free(plot1D->name);
     if (plot1D->title    != NULL) free(plot1D->title);
     free(plot1D);
    }

 return;
}


/*---------------------------------------------------- UHplot1DIni -----*/
void UHplot1DIni(plot1D,dt)
struct UHplot1D *plot1D;
double dt;
/*--------------------------------------------------------------------*/
{
 long numplot, refnumplot;
 char *oldname;
 static struct UHplot1D *previous=NULL;
 static int color;

 if (plot1D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 plot1D->iFirst   = ON;
 plot1D->dt       = dt;
 plot1D->j        = 0;
 plot1D->sumvalue = 0.;

 /*---- decode the options string --------*/

 UHoptDecode(plot1D->options, &plot1D->intervalle, plot1D->plotOpt);
  
 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plot1D->options,"SAME") == NULL)  plot1D->numPicture = DyGrafIni(0);
     else                                         plot1D->numPicture = previous->numPicture;
    }

 if (strstr(plot1D->options,"AUTO") == NULL) 
    {plot1D->adjust = NO;
     plot1D->nData  = 2;
    }
 else
    {plot1D->adjust = YES;
     plot1D->nData = 0;
     if (plot1D->nBin % 2 == 1)  plot1D->nBin = plot1D->nBin + 1;
    }

 oldname = RA_PadContainsGoodHist(plot1D->numPicture, 0, 1, plot1D->nBin, 0, plot1D->name);
 RA_SetTestTrigPad(plot1D->numPicture, 0, (double)plot1D->intervalle * dt);
 if (!oldname) UHoptPlopt(plot1D->options, plot1D->numPicture, 0);

 RA_H1NewReuse(plot1D->numPicture, 0, plot1D->name, plot1D->title, plot1D->nBin, plot1D->xMin, plot1D->xMax);

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plot1D->name);

 RA_HSetStat(plot1D->name, plot1D->options);
 RA_HSetUnitsX(plot1D->name, "a.u.");

 plot1D->data = (double *) calloc(plot1D->nBin, sizeof(double));
 if (plot1D->data == NULL) printf("UHplot1DIni> plot1D->data calloc failed\n");

 plot1D->buff = (float *) calloc(plot1D->nBin, sizeof(float));
 if (plot1D->buff == NULL) printf("UHplot1DIni> plot1D->buff calloc failed\n");

 if (strstr(plot1D->options,"SAME") == NULL)
    {plot1D->maxofMax  = 0;
     plot1D->minofxMin = DY_BADDATA;
     plot1D->maxofxMax = -DY_BADDATA;
    }

 sscanf(plot1D->name, "h%ld", &numplot);

 if (strstr(plot1D->options,"SAME") == NULL)
    {previous = plot1D;
     previous->nsame = 0;
     plot1D->refplot = previous;
     color = 1;
     Dy->refnumplot[numplot] = numplot;
    }
 else if (previous)
    {previous->nsame++;
     plot1D->refplot = previous;
     color++;
     if (color == 8)  color = 40+color-7;
     if (color == 48) color = 30+color-47;
     sscanf(plot1D->refplot->name, "h%ld", &refnumplot);
     Dy->refnumplot[numplot] = refnumplot;
    }

 plot1D->color = color;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplot1DIni> plot=%s=%s=pad%d  color=%d  refplot=%s=%s=pad%d\n",
             plot1D->name, plot1D->title, plot1D->numPicture, plot1D->color,
             plot1D->refplot->name, plot1D->refplot->title, plot1D->refplot->numPicture);
    }


 return;
}


/*---------------------------------------------------- UHplot1DIniFrame -----*/

void UHplot1DIniFrame (struct UHplot1D *plot1D)
{
 struct FrAdcData *adc=NULL;
 struct FrVect *vect=NULL;
 char *temp, *name1, *oldname;
 long length;

 if (plot1D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 temp   = strstr(plot1D->title,SEPTITLE);
 length = strlen(plot1D->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plot1D->title, length);

 adc  = FrAdcDataFind(UFrCurFrameH, name1);
 vect = FrameFindVect(UFrCurFrameH, name1);

 if (adc == NULL)  adc  = FrAdcDataFind(Dy->Frame, name1);
 if (vect == NULL) vect = FrameFindVect(Dy->Frame, name1);

 free(name1);

 plot1D->slope = 1.;
 plot1D->bias  = 0.;

 if (adc != NULL)
    {if (adc->slope != 0.) plot1D->slope = adc->slope;
     plot1D->bias  = adc->bias;
    }

 if (vect != NULL)
    {
     if (plot1D->realUnits > 0)
        {strcpy(plot1D->units, vect->unitY);}
     if (plot1D->realUnits == 0 && plot1D->slope == 1.  &&  plot1D->bias == 0.)
        {strcpy(plot1D->units, vect->unitY);}
     else
        {strcpy(plot1D->units, "");}
    }
 else if (adc != NULL)
    {
     if (plot1D->realUnits > 0)
        {strcpy(plot1D->units, adc->units);}
     else if (plot1D->realUnits == 0 && plot1D->slope == 1.  &&  plot1D->bias == 0.)
        {strcpy(plot1D->units, adc->units);}
     else
        {strcpy(plot1D->units, "");}
    }

 return;
}


/*----------------------------------------------------- UHplot1DRead --*/
void UHplot1DRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplot1D *plot1D;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplot1D, plot1D)
 
 UJ_READ_LONG(plot1D->iClock)
 UJ_READ_LONG(plot1D->id)
 if (!plot1D->name) plot1D->name = (char *) calloc(10,sizeof(char));
 sprintf(plot1D->name,"h%ld",plot1D->id);
 UJ_READ_S_CHAR(plot1D->title)
 UJ_READ_LONG(plot1D->nBin)
 UJ_READ_DOUBLE(plot1D->xMin)
 UJ_READ_DOUBLE(plot1D->xMax)
 UJ_READ_STRUCT(USignal, plot1D->input) 
 UJ_READ_LONG(plot1D->t0)
 UJ_READ_LONG(plot1D->tMax)
 UJ_READ_S_CHAR(plot1D->options)
   
 UJCObAdd(plot1D->iClock,plot1D,UHplot1DUJstructH,NULL);

 plot1D->hidden = OFF;
 s = strstr(plot1D->options,"HIDDEN");
 if (s != NULL)  plot1D->hidden = ON;

 plot1D->realUnits = 0;
 s = strstr(plot1D->options,"REALUNITS");
 if (s != NULL)  plot1D->realUnits = 1;

 if (plot1D->xMax < plot1D->xMin  &&  plot1D->xMin > 0.)  plot1D->xMax = 2. * plot1D->xMin;
 if (plot1D->xMax < plot1D->xMin  &&  plot1D->xMin < 0.)  plot1D->xMax = 0.5 * plot1D->xMin;
 if (plot1D->xMax == 0.  &&  plot1D->xMin == 0.)  plot1D->xMax = plot1D->xMin + 1.;

 plot1D->resampFactor = 1;
 s = strstr(plot1D->options,"RESAMPFACTOR");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plot1D->resampFactor);

 return;
}

 
/*-------------------------------------------------- UHplot1DSim -----*/
void UHplot1DSim(plot1D,time)
struct UHplot1D *plot1D;
long time;
/*--------------------------------------------------------------------*/
{
 long localTime,i,last, intervalle;
 float vMin,vMax,*buff,band;
 double value,*data, localMax, delta;
 double origin;

 if (plot1D->hidden == ON) return;

 if (time < plot1D->t0)  return;
 if (time > plot1D->tMax && plot1D->tMax > 0) return;

 if (plot1D->iFirst == ON)
    {plot1D->iFirst = -1;
     UHplot1DIniFrame(plot1D);
    }

 data = plot1D->data;
 buff = plot1D->buff;

 if (plot1D->input == NULL) 
    {UFrUnpackSim(plot1D->FrUnpack, 0);
     value = plot1D->FrUnpack->out->value;
    }
 else
    {value = plot1D->input->value;}

 if (plot1D->realUnits != 0  &&  value < DY_BADDATA) 
    {value = value * plot1D->slope + plot1D->bias;}

 if (value < DY_BADDATA)         plot1D->sumvalue += value;
 else if (value >= DY_NOCHANNEL) plot1D->sumvalue  = DY_NOCHANNEL * 1.01 * plot1D->resampFactor;
 else if (value >= DY_NOFRAME)   plot1D->sumvalue  = DY_NOFRAME * 1.01 * plot1D->resampFactor;
 else                            plot1D->sumvalue  = DY_BADDATA * 1.01 * plot1D->resampFactor;

 plot1D->j++;

 if (plot1D->j < plot1D->resampFactor) return;

 /*---------- fill data and plot -----------------*/

 value = plot1D->sumvalue / plot1D->resampFactor; 

 plot1D->j = 0;
 plot1D->sumvalue = 0.;


 /*------ check the histogram limits ------*/

 if (plot1D->adjust == YES)
    {
     if (plot1D->nData == 0)
        {
         if (value < DY_BADDATA)  plot1D->firstValue = value;
         else                     plot1D->firstValue = plot1D->xMin;
         plot1D->xMin = plot1D->firstValue;
         plot1D->nFirstValue = 1;
         plot1D->nData       = 1;
        }
     else if (plot1D->nData == 1)
        {
         if (plot1D->firstValue == value)
            {data[0] += 1.;
             if (value < DY_BADDATA) plot1D->xMin = value;
             if (value < DY_BADDATA) plot1D->xMax = value + 1.;
            }
         else
	    {plot1D->nData = 2;
             vMin = DyGetMin (value,plot1D->firstValue);
             if (value < DY_BADDATA)  vMax = DyGetMax (value,plot1D->firstValue);
             else                     vMax = plot1D->xMax;
             band = 0.5*(vMax-vMin);
             if (band < 1.e-30) band = 1.e-30;
             plot1D->xMin = 0.5*(vMin+vMax) - band;
             plot1D->xMax = 0.5*(vMin+vMax) + band;
             if (value == vMax)
	        {data[plot1D->nBin-1] = 1.;}
             else
	        {data[plot1D->nBin-1] = data[0];
                 data[0] = 1.;
                }
             plot1D->nData = 3;
            }
        }

     else   /*--- automatically update Min and Max of the plot ---*/
        {
         if (plot1D->refplot->nsame == 0)  /*--- change limit according to value ---*/
            {
             while (value > plot1D->xMax  &&  value < DY_BADDATA)
               {
                plot1D->xMax += fabs(plot1D->xMax - plot1D->xMin); 
                for (i=0; i<plot1D->nBin/2; i++) data[i] = data[2*i] + data[2*i+1];
                for (i=plot1D->nBin/2; i<plot1D->nBin; i++) data[i] = 0.;
               }

             while (value < plot1D->xMin)
               {
                plot1D->xMin -= fabs(plot1D->xMax-plot1D->xMin);
                last = plot1D->nBin-1;
                for (i=0; i<plot1D->nBin/2; i++) data[last-i] = data[last-2*i] + data[last-2*i-1];
                for (i=0; i<plot1D->nBin/2; i++) data[i] = 0.;
               }
            }

         if (plot1D->refplot->nsame > 0 && value < DY_BADDATA)  /*--- change limit according to minofxMin and maxofxMax ---*/
            {
             if (plot1D->refplot->maxofxMax < value) plot1D->refplot->maxofxMax = value;
             if (plot1D->refplot->minofxMin > value) plot1D->refplot->minofxMin = value;

             while (plot1D->refplot->maxofxMax > plot1D->xMax)
               {
                plot1D->xMax += fabs(plot1D->xMax - plot1D->xMin); 
                for (i=0; i<plot1D->nBin/2; i++)
                   {plot1D->data[i] = plot1D->data[2*i] + plot1D->data[2*i+1];}
                for (i=plot1D->nBin/2; i<plot1D->nBin; i++) plot1D->data[i] = 0.;
               }
             while (plot1D->refplot->minofxMin < plot1D->xMin)
               {
                plot1D->xMin -= fabs(plot1D->xMax-plot1D->xMin);
                last = plot1D->nBin-1;
                for (i=0; i<plot1D->nBin/2; i++)
                   {plot1D->data[last-i] = plot1D->data[last-2*i] + plot1D->data[last-2*i-1];}
                for (i=0; i<plot1D->nBin/2; i++) plot1D->data[i] = 0.;
               }
            }
        }
    }

 /*--- Put data in the histogram (in buffer data) ---*/

 if (plot1D->nData > 2 || plot1D->adjust == NO)
    {i = floor( (value - plot1D->xMin)*(double)(plot1D->nBin) / (plot1D->xMax - plot1D->xMin));
     if (i >= 0 && i < plot1D->nBin) data[i] += 1.;
    }

 intervalle = plot1D->intervalle;
 if (intervalle <= 0) intervalle = 10;

 /*---Display histogram ---*/

 localTime = time - plot1D->t0;

 if (localTime % intervalle == 0  && localTime != 0)
    {
     DyDebugPrint(__FUNCTION__,0);
     DyDebugTimeNeeded(__FUNCTION__,"start",1);

     if (strstr(plot1D->options,"SAME") != NULL) RA_HDelete(plot1D->name);

     RA_H1NewReuse(plot1D->numPicture, 0, plot1D->name, plot1D->title, plot1D->nBin, plot1D->xMin, plot1D->xMax);

     RA_HSetStat(plot1D->name, plot1D->options);
     RA_HSetUnitsX(plot1D->name, "a.u.");
     if (plot1D->realUnits > 0)
        {RA_HSetUnitsX(plot1D->name, plot1D->units);}
     if (plot1D->realUnits == 0 && plot1D->slope == 1.  &&  plot1D->bias == 0.)
        {RA_HSetUnitsX(plot1D->name, plot1D->units);}

     for (i=0; i<plot1D->nBin; i++) buff[i] = (float)data[i];
     for (i=0; i<plot1D->nBin; i++)
        {if (plot1D->refplot->maxofMax  < buff[i]) plot1D->refplot->maxofMax = buff[i];}

     RA_H1Pack(plot1D->name, buff);

     if (Dy->plotsNotDisplayed == OFF)
        {
         RA_HSetMaximum(plot1D->refplot->name, plot1D->refplot->maxofMax*1.2);

         /*--- Set time string and time axis offset ---*/
         origin = Dy->timeOrigin + plot1D->dt * time;
         if (time*plot1D->dt < Dy->duration) UHplotSetTime (origin, plot1D->name, origin-Dy->timeOrigin, 0, OFF, UFrCurFrameH);

         /*--- Draw the plot ---*/
         UHplot(plot1D->name, plot1D->refplot->numPicture, plot1D->options, plot1D->plotOpt, 0, plot1D->refplot->nsame);

         /*--- Update plot environment ---*/
         RA_SetActivePad(plot1D->refplot->numPicture);
         UHplotUpdateEnv (plot1D->numPicture, plot1D->name, plot1D->options, plot1D->refplot->name,
                          plot1D->refplot->numPicture, plot1D->refplot->nsame, plot1D->title,
                          plot1D->color, NULL, plot1D->nBin, &plot1D->corrupt);

         RA_HUpdate(plot1D->numPicture, plot1D->name, plot1D->title, plot1D->options);
        }

     DyCheckRunning();    /* interaction with the data Display */
     DyDebugTimeNeeded(__FUNCTION__,"stop",1);
    }

 return;
}


/*====================================================================================================*/
/*================================================== PLOT 2D =========================================*/

/*----------------------------------------------------- UHplot2DEnd ---*/
void UHplot2DEnd(plot2D)
struct UHplot2D *plot2D;
/*--------------------------------------------------------------------*/
{
 if (plot2D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(plot2D->numPicture);
 RA_HUpdateEnd(plot2D->name,plot2D->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot2DEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  plotOpt=%s \n",
             plot2D->name, plot2D->title, plot2D->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plot2D->plotOpt);
    }

 if (plot2D->name    != NULL)  free(plot2D->name);
 if (plot2D->title   != NULL)  free(plot2D->title);
 if (plot2D->options != NULL)  free(plot2D->options);
 free(plot2D);
 
 return;
}


/*---------------------------------------------------- UHplot2DIni -----*/
void UHplot2DIni(plot2D,dt)
struct UHplot2D *plot2D;
double dt;
/*--------------------------------------------------------------------*/
{
 char param1[40],param2[40];
 char *oldname;
 long length, length2;
 int i;
 double value;

 if (plot2D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);
 
 UHoptDecode(plot2D->options, &plot2D->intervalle, plot2D->plotOpt);

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plot2D->options,"SAME") == NULL)  plot2D->numPicture = DyGrafIni(0);
     else                                         plot2D->numPicture = 0;
    }

 if (strstr(plot2D->options,"AUTO") == NULL) 
    {plot2D->adjust = NO;}
 else
    {plot2D->adjust = YES;
     if (plot2D->nxBin % 2 == 1)  plot2D->nxBin = plot2D->nxBin + 1;
     if (plot2D->nyBin % 2 == 1)  plot2D->nyBin = plot2D->nyBin + 1;
    }

/*  Get the limits of the previous histogram in the pad if it is good  */ 
 oldname = RA_PadContainsGoodHist(plot2D->numPicture, 0, 2, plot2D->nxBin, plot2D->nyBin, plot2D->name);
 if (!oldname)
   {RA_PadCompleteClear(plot2D->numPicture);
    UHoptPlopt(plot2D->options, plot2D->numPicture, 0);
   }

 RA_H2NewReuse (plot2D->numPicture, 0,
                plot2D->name,  plot2D->title,
                plot2D->nxBin, plot2D->xMin, plot2D->xMax,
                plot2D->nyBin, plot2D->yMin, plot2D->yMax);

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plot2D->name);

 RA_HSetStat(plot2D->name, plot2D->options);

 RA_SetTestTrigPad(plot2D->numPicture, 0, (double)plot2D->intervalle * dt);

 plot2D->iSampling     = 0;
 plot2D->previousValue = 0.;
 plot2D->iFirst        = ON;
 plot2D->dt = dt;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot2DIni> plot=%s=%s=pad%d  plotOpt=%s \n",
             plot2D->name, plot2D->title, plot2D->numPicture, plot2D->plotOpt);
    }
 return;
}



/*---------------------------------------------------- UHplot2DIniFrame -----*/

void UHplot2DIniFrame (struct UHplot2D *plot2D)
{
 char param1[40],param2[40], dummy[LSTRING_MAX];
 char *name1, *name2, *temp, *oldname;
 long length, length2;
 struct FrAdcData *adc1=NULL;
 struct FrAdcData *adc2=NULL;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;
 int i;
 double value;

 if (plot2D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 temp   = strstr(plot2D->title,SEPTITLE);
 length = strlen(plot2D->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plot2D->title, length);
 temp   = strstr(name1,SEP2VAR);
 length2 = strlen(name1)-strlen(temp);
 name2 = (char *) calloc(length2+1,sizeof(char));
 strncpy (name2, name1, length2);
 strcpy (name1, temp+strlen(SEP2VAR));
 
 adc1  = FrAdcDataFind(UFrCurFrameH, name1);
 adc2  = FrAdcDataFind(UFrCurFrameH, name2);
 vect1 = FrameFindVect(UFrCurFrameH, name1);
 vect2 = FrameFindVect(UFrCurFrameH, name2);

 if (adc1 == NULL)  adc1  = FrAdcDataFind(Dy->Frame, name1);
 if (vect1 == NULL) vect1 = FrameFindVect(Dy->Frame, name1);
 if (adc2 == NULL)  adc2  = FrAdcDataFind(Dy->Frame, name2);
 if (vect2 == NULL) vect2 = FrameFindVect(Dy->Frame, name2);

 plot2D->xslope = 1.;
 plot2D->xbias  = 0.;
 plot2D->yslope = 1.;
 plot2D->ybias  = 0.;

 if (adc1 != NULL)
    {if (adc1->slope != 0.) plot2D->xslope = adc1->slope;
     plot2D->xbias  = adc1->bias;
    }
 if (adc2 != NULL)
    {if (adc2->slope != 0.) plot2D->yslope = adc2->slope;
     plot2D->ybias  = adc2->bias;
    }

 if (plot2D->adjust == YES)
    {
     if (vect1 != NULL && vect1->nData > 2 && FrVectIsValid(vect1)==0)
        {
         plot2D->xMax = -DY_BADDATA;
         plot2D->xMin =  DY_BADDATA;
         for (i=0; i<vect1->nData; i++)
            {value = FrVectGetV(vect1,i);
             if (plot2D->realUnits != 0  &&  value < DY_BADDATA)
                {value = value * plot2D->xslope + plot2D->xbias;}
             if (plot2D->xMax < value) plot2D->xMax = value;
             if (plot2D->xMin > value) plot2D->xMin = value;
            }
        }
     if (vect2 != NULL && vect2->nData > 2 && FrVectIsValid(vect2)==0)
        {
         plot2D->yMax = -DY_BADDATA;
         plot2D->yMin =  DY_BADDATA;
         for (i=0; i<vect2->nData; i++)
            {value = FrVectGetV(vect2,i);
             if (plot2D->realUnits != 0  &&  value < DY_BADDATA)
                {value = value * plot2D->yslope + plot2D->ybias;}
             if (plot2D->yMax < value) plot2D->yMax = value;
             if (plot2D->yMin > value) plot2D->yMin = value;
            }
        }
     if (plot2D->xMax == plot2D->xMin)  plot2D->xMax = plot2D->xMin + 1e-15;
     if (plot2D->yMax == plot2D->yMin)  plot2D->yMax = plot2D->yMin + 1e-15; 
    }

 sprintf(dummy,"%s  a.u.", name1);
 RA_HSetUnitsX(plot2D->name, dummy);
 if (vect1 != NULL)
    {
     sprintf(dummy,"%s  %s", name1, vect1->unitY);
     if (plot2D->realUnits > 0)
        {RA_HSetUnitsX(plot2D->name, dummy);}
     if (plot2D->realUnits == 0 && plot2D->xslope == 1.  &&  plot2D->xbias == 0.)
        {RA_HSetUnitsX(plot2D->name, dummy);}
    }
 if (adc1 != NULL)
    {
     sprintf(dummy,"%s  %s", name1, adc1->units);
     if (plot2D->realUnits > 0)
        {RA_HSetUnitsX(plot2D->name, dummy);}
     if (plot2D->realUnits == 0 && plot2D->xslope == 1.  &&  plot2D->xbias == 0.)
        {RA_HSetUnitsX(plot2D->name, dummy);}
    }

 RA_HSetUnitsY(plot2D->name, "a.u.");
 if (vect2 != NULL)
    {
     sprintf(dummy,"%s  %s", name2, vect2->unitY);
     if (plot2D->realUnits > 0)
        {RA_HSetUnitsY(plot2D->name, dummy);}
     if (plot2D->realUnits == 0 && plot2D->yslope == 1.  &&  plot2D->ybias == 0.)
        {RA_HSetUnitsY(plot2D->name, dummy);}
    }
 if (adc2 != NULL)
    {
     sprintf(dummy,"%s  %s", name2, adc2->units);
     if (plot2D->realUnits > 0)
        {RA_HSetUnitsY(plot2D->name, dummy);}
     if (plot2D->realUnits == 0 && plot2D->yslope == 1.  &&  plot2D->ybias == 0.)
        {RA_HSetUnitsY(plot2D->name, dummy);}
    }

 free(name1);
 free(name2);

 return;
}


/*----------------------------------------------------- UHplot2DRead --*/
void UHplot2DRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplot2D *plot2D;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplot2D, plot2D)
 
 UJ_READ_LONG(plot2D->iClock)
 UJ_READ_LONG(plot2D->id)
 if (!plot2D->name) plot2D->name = (char *) calloc(10,sizeof(char));
 sprintf(plot2D->name,"h%d",plot2D->id);
 UJ_READ_S_CHAR(plot2D->title)
 UJ_READ_LONG(plot2D->nxBin)
 UJ_READ_DOUBLE(plot2D->xMin)
 UJ_READ_DOUBLE(plot2D->xMax)
 UJ_READ_LONG(plot2D->nyBin)
 UJ_READ_DOUBLE(plot2D->yMin)
 UJ_READ_DOUBLE(plot2D->yMax)
 UJ_READ_STRUCT(USignal, plot2D->xInput)
 UJ_READ_STRUCT(USignal, plot2D->yInput)
 UJ_READ_LONG(plot2D->t0)
 UJ_READ_LONG(plot2D->tMax)
 UJ_READ_S_CHAR(plot2D->options)
    
 UJCObAdd(plot2D->iClock,plot2D,UHplot2DUJstructH,NULL);

 plot2D->hidden = OFF;
 s = strstr(plot2D->options,"HIDDEN");
 if (s != NULL)  plot2D->hidden = ON;

 plot2D->realUnits = 0;
 s = strstr(plot2D->options,"REALUNITS");
 if (s != NULL)  plot2D->realUnits = 1;

 plot2D->nSampling = 0;
 s = strstr(plot2D->options,"NSAMPLING");
 if (s != NULL) sscanf(s,"%s %ld",dummy,&plot2D->nSampling);

 if (plot2D->xMax < plot2D->xMin  &&  plot2D->xMin > 0.)  plot2D->xMax = 2. * plot2D->xMin;
 if (plot2D->yMax < plot2D->yMin  &&  plot2D->xMin > 0.)  plot2D->yMax = 2. * plot2D->yMin;

 if (plot2D->xMax < plot2D->xMin  &&  plot2D->xMin < 0.)  plot2D->xMax = 0.5 * plot2D->xMin;
 if (plot2D->yMax < plot2D->yMin  &&  plot2D->xMin < 0.)  plot2D->yMax = 0.5 * plot2D->yMin;

 if (plot2D->xMax == plot2D->xMin)  plot2D->xMax = plot2D->xMin + 1e-15;
 if (plot2D->yMax == plot2D->yMin)  plot2D->yMax = plot2D->yMin + 1e-15;

 return;
}


/*-------------------------------------------------- UHplot2DSim -----*/
void UHplot2DSim(plot2D,time)
struct UHplot2D *plot2D;
long time;
/*--------------------------------------------------------------------*/
{
 long localTime,i,j,last,nx,intervalle;
 double xValue, yValue;
 float *buff;
 char *name1, *name2, *temp;
 long length, length2;
 double value, origin;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;

 if (plot2D->hidden == ON) return;

 if (strstr(plot2D->options,"SAME") != NULL) return;

 if (time < plot2D->t0)  return;
 if (time > plot2D->tMax && plot2D->tMax > 0) return;

 if (plot2D->iFirst == ON)
    {plot2D->iFirst = -1;
     UHplot2DIniFrame(plot2D);
    }

 /*--- Read x axis signal ---*/

 if (plot2D->xInput == NULL)
    {UFrUnpackSim(plot2D->FrUnpackx, 0);
     xValue = plot2D->FrUnpackx->out->value;
    }
 else
    {xValue = plot2D->xInput->value;}

 if (plot2D->realUnits != 0  &&  xValue < DY_BADDATA)
    {xValue = xValue * plot2D->xslope + plot2D->xbias;}

 /*--- Read y axis signal ---*/

 if (plot2D->nSampling == 0)
    {
     if (plot2D->yInput == NULL) 
        {UFrUnpackSim(plot2D->FrUnpacky, 0);
         yValue = plot2D->FrUnpacky->out->value;
        }
     else
        {yValue = plot2D->yInput->value;}

     if (plot2D->realUnits != 0  &&  yValue < DY_BADDATA)
        {yValue = yValue * plot2D->yslope + plot2D->ybias;}
    }

 else
    {
     if (plot2D->yInput == NULL) 
        {UFrUnpackSim(plot2D->FrUnpacky, 0);
         yValue = plot2D->previousValue;
         if (plot2D->iSampling > plot2D->nSampling)
            {
             plot2D->previousValue = plot2D->FrUnpacky->out->value;
             if (plot2D->realUnits != 0  &&  plot2D->previousValue < DY_BADDATA)
                {plot2D->previousValue = plot2D->previousValue * plot2D->yslope + plot2D->ybias;}
            }
         else
            {(plot2D->iSampling)++;}
        }
     else
        {
         yValue = plot2D->previousValue;
         if (plot2D->iSampling > plot2D->nSampling)
            {
             plot2D->previousValue = plot2D->yInput->value;
             if (plot2D->realUnits != 0  &&  plot2D->previousValue < DY_BADDATA)
                {plot2D->previousValue = plot2D->previousValue * plot2D->yslope + plot2D->ybias;}
            }
         else
            {(plot2D->iSampling)++;}
        }
    }


 if (plot2D->adjust == YES)
    {
     /*--- Initialize min and max for auto-adjust of X axis and Y axis ---*/

     if (plot2D->iFirst == -1)
        {if (xValue != 0. && yValue != 0) plot2D->iFirst = OFF;
         if (xValue < DY_BADDATA && xValue != 0.) {plot2D->xMax = xValue + 0.00001*fabs(xValue); plot2D->xMin  = xValue - 0.00001*fabs(xValue);}
         if (yValue < DY_BADDATA && yValue != 0.) {plot2D->yMax = yValue + 0.00001*fabs(yValue); plot2D->yMin  = yValue - 0.00001*fabs(yValue);}
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("UHplot2DSim> xValue=%g  xMax=%g  xMin=%g  ..  yValue=%g  yMax=%g yMin=%g\n",
                     xValue, plot2D->xMax, plot2D->xMin, yValue, plot2D->yMax, plot2D->yMin);
            }
        }

     /*--- Auto-adjust max on X axis ---*/

     while (xValue > plot2D->xMax  &&  xValue < DY_BADDATA)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot2DSim> xValue=%g  xMax=%g  \n", xValue, plot2D->xMax);

         buff = (float *) calloc(plot2D->nxBin*plot2D->nyBin,sizeof(float));
         RA_H2Unpack(plot2D->name, buff);

         plot2D->xMax = 2 * plot2D->xMax - plot2D->xMin;
         RA_SetLimitsX(plot2D->name, plot2D->xMin, plot2D->xMax);

         nx = plot2D->nxBin;
         for (j=0; j<plot2D->nyBin; j++)
             {for (i=0; i<plot2D->nxBin/2; i++)
                  {buff[i+j*nx] = buff[2*i+j*nx]+buff[(2*i+1)+j*nx];}
              for (i=plot2D->nxBin/2; i<plot2D->nxBin; i++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot2D->name, buff);
         free(buff);
        }

     /*--- Auto-adjust min on X axis ---*/

     while (xValue < plot2D->xMin)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot2DSim> xValue=%g  xMin=%g \n", xValue, plot2D->xMin);

         buff = (float *) calloc(plot2D->nxBin*plot2D->nyBin,sizeof(float));
         RA_H2Unpack(plot2D->name, buff);

         plot2D->xMin = 2 * plot2D->xMin - plot2D->xMax;
         RA_SetLimitsX(plot2D->name, plot2D->xMin, plot2D->xMax);

         last = plot2D->nxBin-1;
         nx   = plot2D->nxBin;
         for (j=0; j<plot2D->nyBin; j++)
             {for (i=0; i<plot2D->nxBin/2; i++)
                  {buff[(last-i)+j*nx] = buff[(last-2*i)+j*nx]+buff[(last-2*i-1)+j*nx];}
              for (i=0; i<plot2D->nxBin/2; i++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot2D->name, buff);
         free(buff);
        }

     /*--- Auto-adjust max on Y axis ---*/

     while (yValue > plot2D->yMax  &&  yValue < DY_BADDATA)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot2DSim> yValue=%g  yMax=%g  \n", yValue, plot2D->yMax);

         buff = (float *) calloc(plot2D->nxBin*plot2D->nyBin, sizeof(float));
         RA_H2Unpack(plot2D->name, buff);

         plot2D->yMax = 2 * plot2D->yMax - plot2D->yMin;
         RA_SetLimitsY(plot2D->name, plot2D->yMin, plot2D->yMax);

         nx   = plot2D->nxBin;
         for (i=0; i<plot2D->nxBin; i++)
             {for (j=0; j<plot2D->nyBin/2; j++)
                  {buff[i+j*nx] = buff[i+2*j*nx]+buff[i+(2*j+1)*nx];}
              for (j=plot2D->nyBin/2; j<plot2D->nyBin; j++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot2D->name, buff);
         free(buff);
        }

     /*--- Auto-adjust min on Y axis ---*/

     while (yValue < plot2D->yMin)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot2DSim> yValue=%g  yMin=%g  \n", yValue, plot2D->yMin);

         buff = (float *) calloc(plot2D->nxBin*plot2D->nyBin, sizeof(float));
         RA_H2Unpack(plot2D->name, buff);

         plot2D->yMin = 2 * plot2D->yMin - plot2D->yMax;
         RA_SetLimitsY(plot2D->name, plot2D->yMin, plot2D->yMax);

         last = plot2D->nyBin-1;
         nx   = plot2D->nxBin;
         for (i=0; i<plot2D->nxBin; i++)
             {for (j=0; j<plot2D->nyBin/2; j++)
                  {buff[i+(last-j)*nx] = buff[i+(last-2*j)*nx]+buff[i+(last-2*j-1)*nx];}
              for (j=0; j<plot2D->nyBin/2; j++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot2D->name, buff);
         free(buff);
        }
    }

 /*--- Fill the plot ---*/

 RA_H2Fill(plot2D->name, xValue, yValue, 1);

 intervalle = plot2D->intervalle;
 if (intervalle <= 0) intervalle = 10;

 localTime = time - plot2D->t0;

 /*--- Show the plot ---*/

 if (localTime % intervalle == 0  && localTime != 0)
   {
    DyDebugPrint(__FUNCTION__,0);
    DyDebugTimeNeeded(__FUNCTION__,"start",1);

    if (Dy->plotsNotDisplayed == OFF)
       {
        /*--- Set time string and time axis offset ---*/
        origin = Dy->timeOrigin + plot2D->dt * time;
        UHplotSetTime (origin, plot2D->name, origin-Dy->timeOrigin, 0, OFF, UFrCurFrameH);

        /*--- Draw the plot ---*/
        UHplot(plot2D->name, plot2D->numPicture, plot2D->options, plot2D->plotOpt, 0, 0);

        RA_SetActivePad(plot2D->numPicture);

        RA_HUpdate(plot2D->numPicture, plot2D->name, plot2D->title, plot2D->options);
       }

    DyCheckRunning();    /* interaction with the data Display */
    DyDebugTimeNeeded(__FUNCTION__,"stop",1);
   }
 
 return;
}


/*====================================================================================================*/
/*================================================ PLOT 3D ===========================================*/

/*----------------------------------------------------- UHplot3DEnd ---*/
void UHplot3DEnd(plot3D)
struct UHplot3D *plot3D;
/*--------------------------------------------------------------------*/
{
 if (plot3D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(plot3D->numPicture);
 RA_HUpdateEnd(plot3D->name,plot3D->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot3DEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  plotOpt=%s\n",
             plot3D->name, plot3D->title, plot3D->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plot3D->plotOpt);
    }

 if (plot3D->name    != NULL)  free(plot3D->name);
 if (plot3D->title   != NULL)  free(plot3D->title);
 if (plot3D->options != NULL)  free(plot3D->options);
 free(plot3D);
 
 return;
}


/*---------------------------------------------------- UHplot3DIni -----*/
void UHplot3DIni(plot3D,dt)
struct UHplot3D *plot3D;
double dt;
/*--------------------------------------------------------------------*/
{
 char param1[40],param2[40];
 char *oldname;
 long length, length2;
 int i;
 double value;

 if (plot3D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 UHoptDecode(plot3D->options, &plot3D->intervalle, plot3D->plotOpt);

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plot3D->options,"SAME") == NULL)  plot3D->numPicture = DyGrafIni(0);
     else                                         plot3D->numPicture = 0;
    }

 if (strstr(plot3D->options,"AUTO") == NULL) 
    {plot3D->adjust = NO;}
 else
    {plot3D->adjust = YES;
     if (plot3D->nxBin % 2 == 1)  plot3D->nxBin = plot3D->nxBin + 1;
     if (plot3D->nyBin % 2 == 1)  plot3D->nyBin = plot3D->nyBin + 1;
    }

/*  Get the limits of the previous histogram in the pad if it is good  */ 
 oldname = RA_PadContainsGoodHist(plot3D->numPicture, 0, 2, plot3D->nxBin, plot3D->nyBin, plot3D->name);
 if (!oldname)
   {RA_PadCompleteClear(plot3D->numPicture);
    UHoptPlopt(plot3D->options, plot3D->numPicture, 0);
   }

 RA_H2NewReuse (plot3D->numPicture, 0,
                plot3D->name,  plot3D->title,
                plot3D->nxBin, plot3D->xMin, plot3D->xMax,
                plot3D->nyBin, plot3D->yMin, plot3D->yMax);

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plot3D->name);

 RA_HSetStat(plot3D->name, plot3D->options);

 RA_SetTestTrigPad(plot3D->numPicture, 0, (double)plot3D->intervalle * dt);

 plot3D->iSampling     = 0;
 plot3D->previousValue = 0.;
 plot3D->iFirst        = ON;
 plot3D->dt            = dt;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot3DIni> plot=%s=%s=pad%d  plotOpt=%s \n",
             plot3D->name, plot3D->title, plot3D->numPicture, plot3D->plotOpt);
    }

 return;
}



/*---------------------------------------------------- UHplot3DIniFrame -----*/

void UHplot3DIniFrame (struct UHplot3D *plot3D)
{
 char param1[40],param2[40];
 char *name1, *name2, *name3, *temp, *temp2, *oldname;
 long length, length1, length2, length3;
 struct FrAdcData *adc1=NULL;
 struct FrAdcData *adc2=NULL;
 struct FrAdcData *adc3=NULL;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;
 struct FrVect *vect3=NULL;
 int i;
 double value;

 if (plot3D->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 /*--- The title is formed by name2.vs.name1.vs.name3 --*/

 temp   = strstr(plot3D->title,SEPTITLE);
 length = strlen(plot3D->title)-strlen(temp);
 name3 = (char *) calloc(length+1,sizeof(char));
 strncpy (name3, plot3D->title, length);

 temp   = strstr(name3,SEP2VAR);
 length2 = strlen(name3)-strlen(temp);
 name2 = (char *) calloc(length2+1,sizeof(char));
 strncpy (name2, name3, length2);
 strcpy (name3, temp+strlen(SEP2VAR));

 temp   = strstr(name3,SEP2VAR);
 length1 = strlen(name3)-strlen(temp);
 name1 = (char *) calloc(length1+1,sizeof(char));
 strncpy (name1, name3, length1);
 strcpy (name3, temp+strlen(SEP2VAR));

 adc1  = FrAdcDataFind(UFrCurFrameH, name1);
 adc2  = FrAdcDataFind(UFrCurFrameH, name2);
 adc3  = FrAdcDataFind(UFrCurFrameH, name3);
 vect1 = FrameFindVect(UFrCurFrameH, name1);
 vect2 = FrameFindVect(UFrCurFrameH, name2);
 vect3 = FrameFindVect(UFrCurFrameH, name3);

 if (adc1 == NULL)  adc1  = FrAdcDataFind(Dy->Frame, name1);
 if (vect1 == NULL) vect1 = FrameFindVect(Dy->Frame, name1);
 if (adc2 == NULL)  adc2  = FrAdcDataFind(Dy->Frame, name2);
 if (vect2 == NULL) vect2 = FrameFindVect(Dy->Frame, name2);
 if (adc3 == NULL)  adc3  = FrAdcDataFind(Dy->Frame, name3);
 if (vect3 == NULL) vect3 = FrameFindVect(Dy->Frame, name3);

 free(name1);
 free(name2);
 free(name3);

 plot3D->xslope = 1.;
 plot3D->xbias  = 0.;
 plot3D->yslope = 1.;
 plot3D->ybias  = 0.;
 plot3D->zslope = 1.;
 plot3D->zbias  = 0.;

 if (adc1 != NULL)
    {if (adc1->slope != 0.) plot3D->xslope = adc1->slope;
     plot3D->xbias  = adc1->bias;
    }
 if (adc2 != NULL)
    {if (adc2->slope != 0.) plot3D->yslope = adc2->slope;
     plot3D->ybias  = adc2->bias;
    }
 if (adc3 != NULL)
    {if (adc3->slope != 0.) plot3D->zslope = adc3->slope;
     plot3D->zbias  = adc3->bias;
    }

 if (plot3D->adjust == YES)
    {
     if (vect1 != NULL && vect1->nData > 2 && FrVectIsValid(vect1)==0)
        {
         plot3D->xMax = -DY_BADDATA;
         plot3D->xMin =  DY_BADDATA;
         for (i=0; i<vect1->nData; i++)
            {value = FrVectGetV(vect1,i);
             if (plot3D->realUnits != 0  &&  value < DY_BADDATA)
                {value = value * plot3D->xslope + plot3D->xbias;}
             if (plot3D->xMax < value) plot3D->xMax = value;
             if (plot3D->xMin > value) plot3D->xMin = value;
            }
        }
     if (vect2 != NULL && vect2->nData > 2 && FrVectIsValid(vect2)==0)
        {
         plot3D->yMax = -DY_BADDATA;
         plot3D->yMin =  DY_BADDATA;
         for (i=0; i<vect2->nData; i++)
            {value = FrVectGetV(vect2,i);
             if (plot3D->realUnits != 0  &&  value < DY_BADDATA)
                {value = value * plot3D->yslope + plot3D->ybias;}
             if (plot3D->yMax < value) plot3D->yMax = value;
             if (plot3D->yMin > value) plot3D->yMin = value;
            }
        }
     if (plot3D->xMax == plot3D->xMin)  plot3D->xMax = plot3D->xMin + 1e-15;
     if (plot3D->yMax == plot3D->yMin)  plot3D->yMax = plot3D->yMin + 1e-15; 
    }

 RA_HSetUnitsX(plot3D->name, "a.u.");
 if (vect1 != NULL)
    {
     if (plot3D->realUnits > 0)
        {RA_HSetUnitsX(plot3D->name, vect1->unitY);}
     if (plot3D->realUnits == 0 && plot3D->xslope == 1.  &&  plot3D->xbias == 0.)
        {RA_HSetUnitsX(plot3D->name, vect1->unitY);}
    }
 if (adc1 != NULL)
    {
     if (plot3D->realUnits > 0)
        {RA_HSetUnitsX(plot3D->name, adc1->units);}
     if (plot3D->realUnits == 0 && plot3D->xslope == 1.  &&  plot3D->xbias == 0.)
        {RA_HSetUnitsX(plot3D->name, adc1->units);}
    }

 RA_HSetUnitsY(plot3D->name, "a.u.");
 if (vect2 != NULL)
    {
     if (plot3D->realUnits > 0)
        {RA_HSetUnitsY(plot3D->name, vect2->unitY);}
     if (plot3D->realUnits == 0 && plot3D->yslope == 1.  &&  plot3D->ybias == 0.)
        {RA_HSetUnitsY(plot3D->name, vect2->unitY);}
    }
 if (adc2 != NULL)
    {
     if (plot3D->realUnits > 0)
        {RA_HSetUnitsY(plot3D->name, adc2->units);}
     if (plot3D->realUnits == 0 && plot3D->yslope == 1.  &&  plot3D->ybias == 0.)
        {RA_HSetUnitsY(plot3D->name, adc2->units);}
    }

 RA_HSetUnitsZ(plot3D->name, "a.u.");
 if (vect3 != NULL)
    {
     if (plot3D->realUnits > 0)
        {RA_HSetUnitsZ(plot3D->name, vect3->unitY);}
     if (plot3D->realUnits == 0 && plot3D->zslope == 1.  &&  plot3D->zbias == 0.)
        {RA_HSetUnitsZ(plot3D->name, vect3->unitY);}
    }
 if (adc3 != NULL)
    {
     if (plot3D->realUnits > 0)
        {RA_HSetUnitsZ(plot3D->name, adc3->units);}
     if (plot3D->realUnits == 0 && plot3D->zslope == 1.  &&  plot3D->zbias == 0.)
        {RA_HSetUnitsZ(plot3D->name, adc3->units);}
    }

 return;
}


/*----------------------------------------------------- UHplot3DRead --*/
void UHplot3DRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplot3D *plot3D;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplot3D, plot3D)
 
 UJ_READ_LONG(plot3D->iClock)
 UJ_READ_LONG(plot3D->id)
 if (!plot3D->name) plot3D->name = (char *) calloc(10,sizeof(char));
 sprintf(plot3D->name,"h3D%d",plot3D->id);
 UJ_READ_S_CHAR(plot3D->title)
 UJ_READ_LONG(plot3D->nxBin)
 UJ_READ_DOUBLE(plot3D->xMin)
 UJ_READ_DOUBLE(plot3D->xMax)
 UJ_READ_LONG(plot3D->nyBin)
 UJ_READ_DOUBLE(plot3D->yMin)
 UJ_READ_DOUBLE(plot3D->yMax)
 UJ_READ_STRUCT(USignal, plot3D->xInput)
 UJ_READ_STRUCT(USignal, plot3D->yInput)
 UJ_READ_STRUCT(USignal, plot3D->zInput)
 UJ_READ_LONG(plot3D->t0)
 UJ_READ_LONG(plot3D->tMax)
 UJ_READ_S_CHAR(plot3D->options)
    
 UJCObAdd(plot3D->iClock,plot3D,UHplot3DUJstructH,NULL);

 plot3D->hidden = OFF;
 s = strstr(plot3D->options,"HIDDEN");
 if (s != NULL)  plot3D->hidden = ON;

 plot3D->realUnits = 0;
 s = strstr(plot3D->options,"REALUNITS");
 if (s != NULL)  plot3D->realUnits = 1;

 plot3D->nSampling = 0;
 s = strstr(plot3D->options,"NSAMPLING");
 if (s != NULL) sscanf(s,"%s %ld",dummy,&plot3D->nSampling);

 if (plot3D->xMax < plot3D->xMin  &&  plot3D->xMin > 0.)  plot3D->xMax = 2. * plot3D->xMin;
 if (plot3D->yMax < plot3D->yMin  &&  plot3D->xMin > 0.)  plot3D->yMax = 2. * plot3D->yMin;

 if (plot3D->xMax < plot3D->xMin  &&  plot3D->xMin < 0.)  plot3D->xMax = 0.5 * plot3D->xMin;
 if (plot3D->yMax < plot3D->yMin  &&  plot3D->xMin < 0.)  plot3D->yMax = 0.5 * plot3D->yMin;

 if (plot3D->xMax == plot3D->xMin)  plot3D->xMax = plot3D->xMin + 1e-15;
 if (plot3D->yMax == plot3D->yMin)  plot3D->yMax = plot3D->yMin + 1e-15;

 return;
}


/*-------------------------------------------------- UHplot3DSim -----*/
void UHplot3DSim(plot3D,time)
struct UHplot3D *plot3D;
long time;
/*--------------------------------------------------------------------*/
{
 long localTime,i,j,last,nx,intervalle;
 double xValue, yValue, zValue;
 float *buff;

 char *name1, *name2, *temp;
 long length, length2;
 double value, origin;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;

 if (plot3D->hidden == ON) return;

 if (strstr(plot3D->options,"SAME") != NULL) return;

 if (time < plot3D->t0)  return;
 if (time > plot3D->tMax && plot3D->tMax > 0) return;

 if (plot3D->iFirst == ON)
    {plot3D->iFirst = -1;
     UHplot3DIniFrame(plot3D);
    }

 intervalle = plot3D->intervalle;
 if (intervalle <= 0) intervalle = 10;
 localTime = time - plot3D->t0;
 if (localTime % intervalle == 1 && localTime != 0) RA_HReset(plot3D->name);

 /*--- ponderate z axis with a third signal ---*/

 if (plot3D->zInput == NULL)
    {UFrUnpackSim(plot3D->FrUnpackz, 0);
     zValue = plot3D->FrUnpackz->out->value;
    }
 else
    {zValue = plot3D->zInput->value;}

 /*--- Read x axis signal ---*/

 if (plot3D->xInput == NULL)
    {UFrUnpackSim(plot3D->FrUnpackx, 0);
     xValue = plot3D->FrUnpackx->out->value;
    }
 else
    {xValue = plot3D->xInput->value;}

 if (plot3D->realUnits != 0  &&  xValue < DY_BADDATA)
    {xValue = xValue * plot3D->xslope + plot3D->xbias;}

 /*--- Read y axis signal ---*/

 if (plot3D->nSampling == 0)
    {
     if (plot3D->yInput == NULL) 
        {UFrUnpackSim(plot3D->FrUnpacky, 0);
         yValue = plot3D->FrUnpacky->out->value;
        }
     else
        {yValue = plot3D->yInput->value;}

     if (plot3D->realUnits != 0  &&  yValue < DY_BADDATA)
        {yValue = yValue * plot3D->yslope + plot3D->ybias;}
    }

 else
    {
     if (plot3D->yInput == NULL) 
        {UFrUnpackSim(plot3D->FrUnpacky, 0);
         yValue = plot3D->previousValue;
         if (plot3D->iSampling > plot3D->nSampling)
            {
             plot3D->previousValue = plot3D->FrUnpacky->out->value;
             if (plot3D->realUnits != 0  &&  plot3D->previousValue < DY_BADDATA)
                {plot3D->previousValue = plot3D->previousValue * plot3D->yslope + plot3D->ybias;}
            }
         else
            {(plot3D->iSampling)++;}
        }
     else
        {
         yValue = plot3D->previousValue;
         if (plot3D->iSampling > plot3D->nSampling)
            {
             plot3D->previousValue = plot3D->yInput->value;
             if (plot3D->realUnits != 0  &&  plot3D->previousValue < DY_BADDATA)
                {plot3D->previousValue = plot3D->previousValue * plot3D->yslope + plot3D->ybias;}
            }
         else
            {(plot3D->iSampling)++;}
        }
    }


 if (plot3D->adjust == YES)
    {
     if (plot3D->iFirst == -1)
        {if (xValue != 0. && yValue != 0) plot3D->iFirst = OFF;
         if (xValue < DY_BADDATA && xValue != 0.) {plot3D->xMax = xValue + 0.00001*fabs(xValue); plot3D->xMin  = xValue - 0.00001*fabs(xValue);}
         if (yValue < DY_BADDATA && yValue != 0.) {plot3D->yMax = yValue + 0.00001*fabs(yValue); plot3D->yMin  = yValue - 0.00001*fabs(yValue);}
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("UHplot3DSim> xValue=%g  xMax=%g  xMin=%g  ..  yValue=%g  yMax=%g yMin=%g\n",
                     xValue, plot3D->xMax, plot3D->xMin, yValue, plot3D->yMax, plot3D->yMin);
            }
        }

     /*--- Auto-adjust max on X axis ---*/

     while (xValue > plot3D->xMax  &&  xValue < DY_BADDATA)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot3DSim> xValue=%g  xMax=%g  \n", xValue, plot3D->xMax);

         buff = (float *) calloc(plot3D->nxBin*plot3D->nyBin,sizeof(float));
         RA_H2Unpack(plot3D->name, buff);

         plot3D->xMax = 2 * plot3D->xMax - plot3D->xMin;
         RA_SetLimitsX(plot3D->name, plot3D->xMin, plot3D->xMax);

         nx = plot3D->nxBin;
         for (j=0; j<plot3D->nyBin; j++)
             {for (i=0; i<plot3D->nxBin/2; i++)
                  {buff[i+j*nx] = buff[2*i+j*nx]+buff[(2*i+1)+j*nx];}
              for (i=plot3D->nxBin/2; i<plot3D->nxBin; i++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot3D->name, buff);
         free(buff);
        }

     /*--- Auto-adjust min on X axis ---*/

     while (xValue < plot3D->xMin)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot3DSim> xValue=%g  xMin=%g \n", xValue, plot3D->xMin);

         buff = (float *) calloc(plot3D->nxBin*plot3D->nyBin,sizeof(float));
         RA_H2Unpack(plot3D->name, buff);

         plot3D->xMin = 2 * plot3D->xMin - plot3D->xMax;
         RA_SetLimitsX(plot3D->name, plot3D->xMin, plot3D->xMax);

         last = plot3D->nxBin-1;
         nx   = plot3D->nxBin;
         for (j=0; j<plot3D->nyBin; j++)
             {for (i=0; i<plot3D->nxBin/2; i++)
                  {buff[(last-i)+j*nx] = buff[(last-2*i)+j*nx]+buff[(last-2*i-1)+j*nx];}
              for (i=0; i<plot3D->nxBin/2; i++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot3D->name, buff);
         free(buff);
        }

     /*--- Auto-adjust max on Y axis ---*/

     while (yValue > plot3D->yMax  &&  yValue < DY_BADDATA)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot3DSim> yValue=%g  yMax=%g  \n", yValue, plot3D->yMax);

         buff = (float *) calloc(plot3D->nxBin*plot3D->nyBin, sizeof(float));
         RA_H2Unpack(plot3D->name, buff);

         plot3D->yMax = 2 * plot3D->yMax - plot3D->yMin;
         RA_SetLimitsY(plot3D->name, plot3D->yMin, plot3D->yMax);

         nx   = plot3D->nxBin;
         for (i=0; i<plot3D->nxBin; i++)
             {for (j=0; j<plot3D->nyBin/2; j++)
                  {buff[i+j*nx] = buff[i+2*j*nx]+buff[i+(2*j+1)*nx];}
              for (j=plot3D->nyBin/2; j<plot3D->nyBin; j++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot3D->name, buff);
         free(buff);
        }

     /*--- Auto-adjust min on Y axis ---*/

     while (yValue < plot3D->yMin)
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) printf("UHplot3DSim> yValue=%g  yMin=%g  \n", yValue, plot3D->yMin);

         buff = (float *) calloc(plot3D->nxBin*plot3D->nyBin, sizeof(float));
         RA_H2Unpack(plot3D->name, buff);

         plot3D->yMin = 2 * plot3D->yMin - plot3D->yMax;
         RA_SetLimitsY(plot3D->name, plot3D->yMin, plot3D->yMax);

         last = plot3D->nyBin-1;
         nx   = plot3D->nxBin;
         for (i=0; i<plot3D->nxBin; i++)
             {for (j=0; j<plot3D->nyBin/2; j++)
                  {buff[i+(last-j)*nx] = buff[i+(last-2*j)*nx]+buff[i+(last-2*j-1)*nx];}
              for (j=0; j<plot3D->nyBin/2; j++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot3D->name, buff);
         free(buff);
        }
    }

 /*--- Fill the plot ---*/

/** RESET NEEDED FOR EACH FILL ? **/

 RA_H2Fill(plot3D->name, xValue, yValue, zValue);

 intervalle = plot3D->intervalle;
 if (intervalle <= 0) intervalle = 10;

 /*--- Show the plot ---*/

 localTime = time - plot3D->t0;
 if (localTime % intervalle == 0  && localTime != 0)
   {
    DyDebugPrint(__FUNCTION__,0);
    DyDebugTimeNeeded(__FUNCTION__,"start",1);

    if (Dy->plotsNotDisplayed == OFF)
       {
        /*--- Set time string and time axis offset ---*/
        origin = Dy->timeOrigin + plot3D->dt * time;
        UHplotSetTime (origin, plot3D->name, origin-Dy->timeOrigin, 0, OFF, UFrCurFrameH);

        /*--- Draw the plot ---*/
        UHplot(plot3D->name, plot3D->numPicture, plot3D->options, plot3D->plotOpt, 0, 0);

        RA_SetActivePad(plot3D->numPicture);

        RA_HUpdate(plot3D->numPicture, plot3D->name, plot3D->title, plot3D->options);
       }

    DyCheckRunning();    /* interaction with the data Display */
    DyDebugTimeNeeded(__FUNCTION__,"stop",1);
   }
 
 return;
}




/*====================================================================================================*/
/*========================================= PLOT TRANSFERT FUNCTION ==================================*/

/*---------------------------------------------------- UHpTfctEnd -----*/
void UHpTfctEnd(pTfct)
struct UHpTfct *pTfct;
/*--------------------------------------------------------------------*/
{
 long j;
 double origin;

 if (pTfct->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHpTfctEnd> plot=%s idModule=%ld idPhase=%ld nAverage=%ld\n", pTfct->title, pTfct->idModule,pTfct->idPhase,pTfct->nAverage);}

 if (pTfct->nAverage > 0)
    {
     RA_HReset(pTfct->nameModule);
     RA_HReset(pTfct->namePhase);

     for (j=0 ;j<pTfct->nBin/2+1 ;j++)
        {RA_H1Fill(pTfct->nameModule, pTfct->dnu * (double) j ,
                 pTfct->module->dataD[j]);
         RA_H1Fill(pTfct->namePhase, pTfct->dnu * (double) j ,
                 pTfct->phase->dataD[j]);
        }
    }

 if (Dy->plotsNotDisplayed == OFF && ((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug<0)
    {
     /*--- Set time string and time axis offset ---*/
     origin = Dy->timeOrigin + pTfct->tmin;
     UHplotSetTime (origin, pTfct->namePhase, pTfct->nBin*pTfct->dt, pTfct->nAverage, OFF, UFrCurFrameH);

     /*--- Draw the plot ---*/
     UHplot (pTfct->nameModule, pTfct->numPicture, pTfct->option1, pTfct->plotOpt1, 1, pTfct->refplot->nsame);
     UHplot (pTfct->namePhase, pTfct->numPicture, pTfct->option2, pTfct->plotOpt2, 2, pTfct->refplot->nsame);
    }

 RA_SetActivePad(pTfct->refplot->numPicture);
 RA_SetActiveSubPad(1);
 RA_HUpdateEnd(pTfct->nameModule, pTfct->plotOpt1);

 RA_SetActivePad(pTfct->refplot->numPicture);
 RA_SetActiveSubPad(2);
 RA_HUpdateEnd(pTfct->namePhase, pTfct->plotOpt2);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpTfctEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  color=%d \n",
             pTfct->nameModule, pTfct->title, pTfct->numPicture, Dy->nready, Dy->nsame, Dy->nplot, pTfct->color);
    }

 if (pTfct->option1 != NULL)  free(pTfct->option1);
 if (pTfct->option2 != NULL)  free(pTfct->option2);
 if (pTfct->vect1   != NULL)  UACDFree(pTfct->vect1);
 if (pTfct->vect2   != NULL)  UACDFree(pTfct->vect2);
 if (pTfct->module  != NULL)  UADFree(pTfct->module);
 if (pTfct->window  != NULL)  UADFree(pTfct->window);
 if (pTfct->buff1    !=NULL)  free(pTfct->buff1);
 if (pTfct->buff2    !=NULL)  free(pTfct->buff2);

 if (pTfct != pTfct->refplot)
    {if (pTfct->nameModule  != NULL)  free(pTfct->nameModule);
     if (pTfct->namePhase   != NULL)  free(pTfct->namePhase);
     if (pTfct->title   != NULL)      free(pTfct->title);
     free(pTfct);
    }

 return;
}


/*---------------------------------------------------- UHpTfctIni -----*/
void UHpTfctIni(pTfct,dt)
struct UHpTfct *pTfct;
double dt;
/*--------------------------------------------------------------------*/
{
 long i, numplot, refnumplot;
 char param1[40],param2[40], dummy[LSTRING_MAX];
 char *name1, *name2, *temp, *oldname;
 long length, length2;
 static struct UHpTfct *previous=NULL;
 static int color;

 if (pTfct->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (pTfct->resampFactor <= 0)  pTfct->resampFactor = 1;

 dt              = dt * (double)pTfct->resampFactor;
 pTfct->dt       = dt;
 pTfct->dnu      = 1./(pTfct->nBin*dt);
 pTfct->i        = 0;
 pTfct->j        = 0;
 pTfct->dNbin    = pTfct->nBin;
 pTfct->nAverage = 0;
 pTfct->nRefresh = 0;
 pTfct->previousGPS = 0;
 pTfct->iFirst   = ON;

 pTfct->tmin = pTfct->dt * (double)pTfct->t0;

 pTfct->vect1    = UACDNew(1,pTfct->nBin);
 pTfct->vect2    = UACDNew(1,pTfct->nBin);
 pTfct->module   = UADNew(1,pTfct->nBin/2+1);
 pTfct->phase    = UADNew(1,pTfct->nBin/2+1);
 pTfct->rer      = UADNew(1,pTfct->nBin/2+1);
 pTfct->imr      = UADNew(1,pTfct->nBin/2+1);
 pTfct->mod1sq   = UADNew(1,pTfct->nBin/2+1);
 pTfct->window   = UADNew(1,pTfct->nBin);

 bzero(pTfct->vect1->data, pTfct->vect1->nBytes);
 bzero(pTfct->vect2->data, pTfct->vect2->nBytes);
 bzero(pTfct->module->data, pTfct->module->nBytes);
 bzero(pTfct->phase->data, pTfct->phase->nBytes);
 bzero(pTfct->rer->data, pTfct->rer->nBytes);
 bzero(pTfct->imr->data, pTfct->imr->nBytes);
 bzero(pTfct->mod1sq->data, pTfct->mod1sq->nBytes);
 bzero(pTfct->window->data, pTfct->window->nBytes);


 for (i=0; i<pTfct->nBin; i++)
    {pTfct->window->dataD[i] = 0.5 * (1. + cos( TWOPI * 
                               ((double)i - ((double)pTfct->nBin - 1.)/2.)/
                               ((double)pTfct->nBin - 1.)));}

 /*---- decode the options string --------*/

 UHoptDecode (pTfct->option1, &pTfct->intervalle, pTfct->plotOpt1);
 UHoptDecode (pTfct->option2, &pTfct->intervalle, pTfct->plotOpt2);

 if (pTfct->intervalle <= 0) pTfct->intervalle = pTfct->nBin;

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(pTfct->option1,"SAME") == NULL) pTfct->numPicture = DyGrafIni(0);
     else                                       pTfct->numPicture = previous->numPicture;
     pTfct->dNbin = pTfct->intervalle;
    }


/*  Get the limits of the previous histogram in the pad if it is good  */ 
 oldname = RA_PadContainsGoodHist(pTfct->numPicture, 1, 1, pTfct->nBin/2+1, 0, pTfct->nameModule);

 RA_H1NewReuse(pTfct->numPicture,
               1,
               pTfct->nameModule,
               pTfct->title,
               pTfct->nBin/2+1,
               pTfct->dnu*(-.5),
               pTfct->dnu*((double)(pTfct->nBin/2+1) - .5));

 RA_H1NewReuse(pTfct->numPicture,
               2,
               pTfct->namePhase,
               pTfct->title,
               pTfct->nBin/2+1,
               pTfct->dnu*(-.5),
               pTfct->dnu*((double)(pTfct->nBin/2+1) - .5));

 RA_SetTestTrigPad(pTfct->numPicture, 0, (double)pTfct->intervalle * dt * pTfct->maxRefresh);

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, pTfct->nameModule);
 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, pTfct->namePhase);

 RA_SetLabelSize(0.07, pTfct->nameModule);
 RA_SetLabelSize(0.07, pTfct->namePhase);

 RA_HSetStat(pTfct->nameModule, NULL);
 RA_HSetStat(pTfct->namePhase, NULL);

 if (pTfct->autoScale == 0 && pTfct->yMin > 0. && pTfct->yMax > 0.)
    {RA_HSetMinimum(pTfct->nameModule, pTfct->yMin);
     RA_HSetMaximum(pTfct->nameModule, pTfct->yMax);
    }

 if (!oldname)
    {
     if (strstr(pTfct->option1,"SAME") == NULL)
        {RA_PadCompleteClear(pTfct->numPicture);
         RA_SetActivePad(pTfct->numPicture);
         if (!RA_PadIsDivided(-1)  &&  strstr(pTfct->option1,"SAME") == NULL) 
            {RA_PadDivide(1,2);
/*             RA_SetTestTrigPad(pTfct->numPicture, 1, (double)pTfct->intervalle * pTfct->dt * pTfct->maxRefresh); */
            }
        }
     if (pTfct->fmax <= pTfct->fmin) pTfct->fmax = 0.5 / pTfct->dt;
     RA_HSetXRange(pTfct->nameModule, pTfct->fmin, pTfct->fmax, pTfct->dnu);
     RA_HSetXRange(pTfct->namePhase,  pTfct->fmin, pTfct->fmax, pTfct->dnu);

     UHoptPlopt(pTfct->option1, pTfct->numPicture, 1);
     UHoptPlopt(pTfct->option2, pTfct->numPicture, 2);
    }

 RA_HSetUnitsX(pTfct->nameModule, "Hz");
 RA_HSetUnitsX(pTfct->namePhase, "Hz");
 RA_HSetUnitsY(pTfct->namePhase, "Phase (rad)");

 pTfct->buff1 = (float *)calloc(pTfct->nBin+1, sizeof(float));
 if (pTfct->buff1 == NULL) printf("UHpTFctIni> pTfct->buff1 malloc failed\n");
 pTfct->buff2 = (float *)calloc(pTfct->nBin+1, sizeof(float));
 if (pTfct->buff2 == NULL) printf("UHpTFctIni> pTfct->buff2 malloc failed\n");

 sscanf(pTfct->nameModule, "h%ld", &numplot);

 if (strstr(pTfct->option1,"SAME") == NULL)
    {previous = pTfct;
     previous->nsame = 0;
     pTfct->refplot = previous;
     color = 1;
     Dy->refnumplot[numplot] = numplot;
    }
 else if (previous)
    {previous->nsame++;
     pTfct->refplot = previous;
     color++;
     if (color == 8)  color = 40+color-7;
     if (color == 48) color = 30+color-47;
     sscanf(pTfct->refplot->nameModule, "h%ld", &refnumplot);
     Dy->refnumplot[numplot] = refnumplot;
    }

 pTfct->color = color;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpTfctIni> plot=%s=%s=pad%d  color=%d  refplot=%s=%s=pad%d \n",
             pTfct->nameModule, pTfct->title, pTfct->numPicture, pTfct->color,
             pTfct->refplot->nameModule, pTfct->refplot->title, pTfct->refplot->numPicture);
    }

 pTfct->decay = 0.;
 if (strstr(pTfct->option1,"DECAYFFT") != NULL)
    {pTfct->decay = 1./(double)pTfct->maxAverage;}

 return;
}


/*---------------------------------------------------- UHpTfctIniFrame -----*/

void UHpTfctIniFrame (struct UHpTfct *pTfct)
{
 long i;
 char param1[40],param2[40], dummy[LSTRING_MAX];
 char *name1, *name2, *temp, *oldname;
 long length, length2;
 struct FrAdcData *adc1=NULL;
 struct FrAdcData *adc2=NULL;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;

 if (pTfct->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 temp   = strstr(pTfct->title,SEPTITLE);
 length = strlen(pTfct->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, pTfct->title, length);
 temp   = strstr(name1,SEP2VARTR);
 length2 = strlen(name1)-strlen(temp);
 name2 = (char *) calloc(length2+1,sizeof(char));
 strncpy (name2, name1, length2);
 strcpy (name1, temp+strlen(SEP2VARTR));

 if (UFrCurFrameH)
    {adc1  = FrAdcDataFind(UFrCurFrameH, name1);
     adc2  = FrAdcDataFind(UFrCurFrameH, name2);
     vect1 = FrameFindVect(UFrCurFrameH, name1);
     vect2 = FrameFindVect(UFrCurFrameH, name2);
    }

 if (adc1 == NULL  &&  vect1 == NULL)
    {adc1  = FrAdcDataFind(Dy->Frame, name1);
     vect1 = FrameFindVect(Dy->Frame, name1);
    }
 if (adc2 == NULL  &&  vect2 == NULL)
    {adc2  = FrAdcDataFind(Dy->Frame, name2);
     vect2 = FrameFindVect(Dy->Frame, name2);
    }
 free(name1);
 free(name2);

 pTfct->slope1 = 1.;
 pTfct->bias1  = 0.;
 pTfct->slope2 = 1.;
 pTfct->bias2  = 0.;

 if (adc1 != NULL)
    {if (adc1->slope != 0.) pTfct->slope1 = adc1->slope;
     pTfct->bias1  = adc1->bias;
    }
 if (adc2 != NULL)
    {if (adc2->slope != 0.) pTfct->slope2 = adc2->slope;
     pTfct->bias2  = adc2->bias;
    }

 RA_HSetUnitsY(pTfct->nameModule, "a.u.");

 if (vect1 != NULL  &&  vect2 != NULL)
    {
     sprintf(dummy,"%s/%s", vect2->unitY, vect1->unitY);
     if (pTfct->realUnits > 0)
        {RA_HSetUnitsY(pTfct->nameModule, dummy);}
     if (pTfct->realUnits == 0 && pTfct->slope1 == 1. && pTfct->bias1 == 0.)
        {RA_HSetUnitsY(pTfct->nameModule, dummy);}
    }
 else if (adc1 != NULL  &&  adc2 != NULL)
    {
     sprintf(dummy,"%s/%s", adc2->units, adc1->units);
     if (pTfct->realUnits > 0)
        {RA_HSetUnitsY(pTfct->nameModule, dummy);}
     if (pTfct->realUnits == 0 && pTfct->slope1 == 1. && pTfct->bias1 == 0.)
        {RA_HSetUnitsY(pTfct->nameModule, dummy);}
    }

 return;
}


/*----------------------------------------------------- UHpTfctRead --*/
void UHpTfctRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHpTfct *pTfct;
 long i;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHpTfct, pTfct);

 UJ_READ_LONG(pTfct->iClock)
 UJ_READ_LONG(pTfct->idModule)
 if (!pTfct->nameModule) pTfct->nameModule  = (char *) calloc(10,sizeof(char));
 sprintf(pTfct->nameModule,"h%ld",pTfct->idModule);
 UJ_READ_LONG(pTfct->idPhase)
 if (!pTfct->namePhase) pTfct->namePhase  = (char *) calloc(10,sizeof(char));
 sprintf(pTfct->namePhase,"h%ld",pTfct->idPhase);
 UJ_READ_S_CHAR(pTfct->title)
 UJ_READ_LONG(pTfct->nBinP2)
 UJ_READ_LONG(pTfct->resampFactor)
 UJ_READ_LONG(pTfct->t0)
 UJ_READ_STRUCT(USignal,pTfct->input1)
 UJ_READ_STRUCT(USignal,pTfct->input2)
 UJ_READ_S_CHAR(pTfct->option1)
 UJ_READ_S_CHAR(pTfct->option2)

 i = 1; 
 pTfct->nBin = (i << pTfct->nBinP2);
 if (pTfct->nBin <= 0) pTfct->nBin = 1;

 UJCObAdd(pTfct->iClock,pTfct,UHpTfctUJstructH,NULL);

 pTfct->hidden = OFF;
 s = strstr(pTfct->option1,"HIDDEN");
 if (s != NULL)  pTfct->hidden = ON;

 pTfct->realUnits = 0;
 s = strstr(pTfct->option1,"REALUNITS");
 if (s != NULL)  pTfct->realUnits = 1;

 pTfct->maxAverage = 1;
 s = strstr(pTfct->option1,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&pTfct->maxAverage);

 pTfct->maxRefresh = 1;
 s = strstr(pTfct->option1,"NFFT_REFRESH");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&pTfct->maxRefresh);
 if (pTfct->maxRefresh <= 0) pTfct->maxRefresh = 1;

 pTfct->fmin = 0.;
 s = strstr(pTfct->option1,"FMIN");
 if (s != NULL)  sscanf(s,"%s %lf", dummy, &pTfct->fmin);

 pTfct->fmax = 0.;
 s = strstr(pTfct->option1,"FMAX");
 if (s != NULL)  sscanf(s,"%s %lf", dummy, &pTfct->fmax);

 pTfct->yMin = 0.;
 s = strstr(pTfct->option1,"YMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pTfct->yMin);

 pTfct->yMax = 0.;
 s = strstr(pTfct->option1,"YMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pTfct->yMax);

 pTfct->autoScale = 0;
 s = strstr(pTfct->option1,"AUTOSCALE");
 if (s != NULL)  pTfct->autoScale = 1;

 return;
} 


/*--------------------------------------------------- UHpTfctSim -----*/
void UHpTfctSim(pTfct,time)
struct UHpTfct *pTfct;
long time;
/*--------------------------------------------------------------------*/
{
 long i,j,inter,inter2;
 double *tab1,*tab2,*module,*phase,value1,value2;
 double *mod1sq,*rer,*imr;
 char title[256], stringtime[40];
 time_t starttimeutc;
 double localMin, localMax;
 static float lastGoodData1=0.;
 static float lastGoodData2=0.;
 static double minofMin=DY_BADDATA;
 static double maxofMax=-DY_BADDATA;
 double origin;
 int status;

 if (pTfct->hidden == ON) return;

 if (time < pTfct->t0)  return;

 if (pTfct->iFirst == ON)
    {pTfct->iFirst = OFF;
     UHpTfctIniFrame(pTfct);
    }

 if (pTfct->input1 == NULL)
    {UFrUnpackSim(pTfct->FrUnpack1, 0);
     value1 = pTfct->FrUnpack1->out->value;
    }
 else
    {value1 = pTfct->input1->value;}

 if (pTfct->realUnits != 0  &&  value1 < DY_BADDATA) 
    {value1 = value1 * pTfct->slope1 + pTfct->bias1;}

 if (pTfct->input2 == NULL)
    {UFrUnpackSim(pTfct->FrUnpack2, 0);
     value2 = pTfct->FrUnpack2->out->value;
    }
 else
    {value2 = pTfct->input2->value;}

 if (pTfct->realUnits != 0  &&  value2 < DY_BADDATA) 
    {value2 = value2 * pTfct->slope2 + pTfct->bias2;}


 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reaches resampFactor ---*/

 if (value1 < DY_BADDATA)         pTfct->sumvalue1 += value1;
 else if (value1 >= DY_NOCHANNEL) pTfct->sumvalue1  = DY_NOCHANNEL * 1.01 * pTfct->resampFactor;
 else if (value1 >= DY_NOFRAME)   pTfct->sumvalue1  = DY_NOFRAME * 1.01 * pTfct->resampFactor;
 else                            pTfct->sumvalue1  = DY_BADDATA * 1.01 * pTfct->resampFactor;

 if (value2 < DY_BADDATA)         pTfct->sumvalue2 += value2;
 else if (value2 >= DY_NOCHANNEL) pTfct->sumvalue2  = DY_NOCHANNEL * 1.01 * pTfct->resampFactor;
 else if (value2 >= DY_NOFRAME)   pTfct->sumvalue2  = DY_NOFRAME * 1.01 * pTfct->resampFactor;
 else                             pTfct->sumvalue2  = DY_BADDATA * 1.01 * pTfct->resampFactor;

 pTfct->j++;

 if (pTfct->j < pTfct->resampFactor) return;

 /*--- increment vector index and do the job if enough data has been taken ---*/

 pTfct->buff1[pTfct->i] = pTfct->sumvalue1 / pTfct->resampFactor;
 pTfct->buff2[pTfct->i] = pTfct->sumvalue2 / pTfct->resampFactor;
 pTfct->i++;
 pTfct->sumvalue1 = 0;
 pTfct->sumvalue2 = 0;
 pTfct->j = 0;

 if (pTfct->i == pTfct->nBin)
   {
    DyDebugPrint(__FUNCTION__,0);
    DyDebugTimeNeeded(__FUNCTION__,"start",1);

    pTfct->nAverage++;
    pTfct->nRefresh++;

    tab1 = pTfct->vect1->dataD;
    tab2 = pTfct->vect2->dataD;

    /*--- Ponderate data of channel 1 with a Hanning window ---*/

    for (i=0; i<pTfct->nBin; i++)
       {
        if (pTfct->buff1[i] < DY_BADDATA)
           {inter = 2*i;
            tab1[inter]   = pTfct->buff1[i] * pTfct->window->dataD[i];
            tab1[inter+1] = 0.;
            lastGoodData1 = pTfct->buff1[i];
           }
        else
           {inter = 2*i;
            tab1[inter]   = lastGoodData1 * pTfct->window->dataD[i];
            tab1[inter+1] = 0.;
           }
       }


    /*--- Ponderate data of channel 2 with a Hanning window ---*/

    for (i=0; i<pTfct->nBin; i++)
       {
        if (pTfct->buff2[i] < DY_BADDATA)
           {inter = 2*i;
            tab2[inter]   = pTfct->buff2[i] * pTfct->window->dataD[i];
            tab2[inter+1] = 0.;
            lastGoodData2 = pTfct->buff2[i];
           }
        else
           {inter = 2*i;
            tab2[inter]   = lastGoodData2 * pTfct->window->dataD[i];
            tab2[inter+1] = 0.;
           }
       }

    /*--- Do the FFTs ---*/

    UACDfft(pTfct->vect1,pTfct->nBinP2);
    UACDfft(pTfct->vect2,pTfct->nBinP2);

    module = pTfct->module->dataD;
    phase  = pTfct->phase->dataD;
    rer    = pTfct->rer->dataD;
    imr    = pTfct->imr->dataD;
    mod1sq = pTfct->mod1sq->dataD;

    /*--- Compute the transfer function ---*/

    for (j=0 ;j<pTfct->nBin/2+1 ;j++)
       {inter  = 2*j;
        inter2 = inter+1;

        /*--- Z = Z2 / Z1 = Z2 * Z1conj. / |Z1|**2   ---*/

        if (pTfct->decay>0 && pTfct->decay<1)
           {
            mod1sq[j] =  (1.-pTfct->decay) * mod1sq[j] + pTfct->decay * (tab1[inter]*tab1[inter] + tab1[inter2]*tab1[inter2]);
            rer[j]    =  (1.-pTfct->decay) * rer[j]    + pTfct->decay * (tab2[inter]*tab1[inter] + tab2[inter2]*tab1[inter2]);
            imr[j]    =  (1.-pTfct->decay) * imr[j]    + pTfct->decay * (-tab2[inter]*tab1[inter2] + tab2[inter2]*tab1[inter]);
           }
        else
           {
            mod1sq[j] += tab1[inter]*tab1[inter] + tab1[inter2]*tab1[inter2];
            rer[j]    += (tab2[inter]*tab1[inter] + tab2[inter2]*tab1[inter2]);
            imr[j]    += (-tab2[inter]*tab1[inter2] + tab2[inter2]*tab1[inter]);
           }

        if (mod1sq[j] == 0) module[j] = 0.;
        else                module[j] = sqrt(rer[j]*rer[j]+imr[j]*imr[j])/mod1sq[j];
        phase[j] = atan2(imr[j],rer[j]);
       }

/**
printf("nAverage=%d/%d  module[10]=%g  mod1sq[10]=%g  rer[10]=%g  imr[10]=%g phase[10]=%g \n",
        pTfct->nAverage, pTfct->maxAverage, module[10], mod1sq[10], rer[10], imr[10], phase[10]);
**/

    /*--- Set automatic min and max ---*/

    if (pTfct->autoScale > 0)
       {
        localMin = DY_BADDATA;
        localMax = -DY_BADDATA;
        if (module[0] < DY_BADDATA) localMin = module[0];
        if (module[0] < DY_BADDATA) localMax = module[0];
        for (j=0 ;j<pTfct->nBin/2+1 ;j++)
           {if (localMin > module[j]  &&  module[j] < DY_BADDATA)  localMin = module[j];
            if (localMax < module[j]  &&  module[j] < DY_BADDATA)  localMax = module[j];
           }
        if (strstr(pTfct->option1,"SAME") == NULL)
            {minofMin = localMin;
             maxofMax = localMax;
            }
         else
            {
             if (minofMin > localMin) minofMin = localMin;
             if (maxofMax < localMax) maxofMax = localMax;
            }
       }

    /*--- Fill the plots ---*/

    RA_HReset(pTfct->nameModule);
    RA_HReset(pTfct->namePhase);

    for (j=0 ;j<pTfct->nBin/2+1 ;j++)
       {
        RA_H1Fill (pTfct->nameModule, pTfct->dnu*(double)j, module[j]);
        RA_H1Fill (pTfct->namePhase, pTfct->dnu*(double)j,  phase[j]);
      }

    /*--- Reset the FFTs buffers ---*/

    for (j=0; j<2*pTfct->nBin; j++)
       {
        pTfct->vect1->dataD[j] = 0.;
        pTfct->vect2->dataD[j] = 0.;
       }

    /*--- Draw the plots ---*/

    if (Dy->plotsNotDisplayed == OFF && pTfct->nRefresh == pTfct->maxRefresh)
       {
        if (pTfct->autoScale > 0)
           {
            if (strstr(pTfct->option1,"SAME") == NULL)
               {
                if (pTfct->nsame > 0)
                   {RA_HSetMinimum(pTfct->nameModule, localMin*0.8);
                    RA_HSetMaximum(pTfct->nameModule, localMax*1.2);
                   }
               }
            else if (pTfct->refplot)
               {RA_HSetMinimum(pTfct->refplot->nameModule, minofMin*0.8);
                RA_HSetMaximum(pTfct->refplot->nameModule, maxofMax*1.2);
               }
           }

        /*--- Set time string and time axis offset ---*/
        origin = Dy->timeOrigin + pTfct->tmin;
        UHplotSetTime (origin, pTfct->namePhase, pTfct->nBin*pTfct->dt, pTfct->nAverage, OFF, UFrCurFrameH);

        /*--- Draw the plot ---*/
        UHplot (pTfct->nameModule, pTfct->numPicture, pTfct->option1, pTfct->plotOpt1, 1, pTfct->refplot->nsame);
        UHplot (pTfct->namePhase, pTfct->numPicture, pTfct->option2, pTfct->plotOpt2, 2, pTfct->refplot->nsame);

        /*--- Update plot environment ---*/
        RA_SetActivePad(pTfct->refplot->numPicture);
        RA_SetActiveSubPad(1);
        UHplotUpdateEnv (pTfct->numPicture, pTfct->nameModule, pTfct->option1, pTfct->refplot->nameModule,
                         pTfct->refplot->numPicture, pTfct->refplot->nsame, pTfct->title,
                         pTfct->color, pTfct->buff1, pTfct->nBin, &pTfct->corrupt);
        UHplotUpdateEnv (pTfct->numPicture, NULL, pTfct->option1, pTfct->refplot->nameModule,
                         pTfct->refplot->numPicture, pTfct->refplot->nsame, pTfct->title,
                         pTfct->color, pTfct->buff2, pTfct->nBin, &pTfct->corrupt);

        RA_SetActivePad(pTfct->refplot->numPicture);
        RA_SetActiveSubPad(2);
        UHplotUpdateEnv (pTfct->numPicture, pTfct->namePhase, pTfct->option2, pTfct->refplot->namePhase,
                         pTfct->refplot->numPicture, pTfct->refplot->nsame, NULL,
                         pTfct->color, NULL, pTfct->nBin, &pTfct->corrupt);

        /*--- Update the plot canvas ---*/
        RA_HUpdate(pTfct->numPicture, pTfct->nameModule, pTfct->title, pTfct->option1);
       }

    DyCheckRunning();    /* interaction with the data Display */

    if (pTfct->nRefresh == pTfct->maxRefresh) pTfct->nRefresh = 0;

    if (pTfct->decay>0 && pTfct->decay<1)
       {
        if (pTfct->nAverage >= pTfct->maxAverage) pTfct->nAverage = pTfct->maxAverage-1;
       }
    else
       {
        if (pTfct->nAverage == pTfct->maxAverage)
           {pTfct->nAverage = 0;
            for (j=0 ;j<pTfct->nBin/2+1 ;j++) pTfct->mod1sq->dataD[j] = 0.;
            for (j=0 ;j<pTfct->nBin/2+1 ;j++) pTfct->rer->dataD[j] = 0.;
            for (j=0 ;j<pTfct->nBin/2+1 ;j++) pTfct->imr->dataD[j] = 0.;
           }
       }

    /*--- shift buffers and update tmin ---*/
    for (j=0; j<pTfct->nBin-pTfct->dNbin; j++)
       {pTfct->buff1[j] = pTfct->buff1[j+pTfct->dNbin];
        pTfct->buff2[j] = pTfct->buff2[j+pTfct->dNbin];
       }
     pTfct->i = pTfct->i - pTfct->dNbin;
     pTfct->tmin = pTfct->tmin + pTfct->dt * pTfct->dNbin;

    DyDebugTimeNeeded(__FUNCTION__,"stop",1);
   }

 if (UFrCurFrameH)
    {
     if ((pTfct->previousGPS > 0) &&  ((double)(UFrCurFrameH->GTimeS-pTfct->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {pTfct->tmin += (double)(UFrCurFrameH->GTimeS - pTfct->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) pTfct->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*============================================== PLOT COHERENCE ======================================*/

/*---------------------------------------------------- UHpCoheEnd -----*/
void UHpCoheEnd(pCohe)
struct UHpCohe *pCohe;
/*--------------------------------------------------------------------*/
{
 long j, i, jj, nbin;
 double origin;

 if (pCohe->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (pCohe->nAverage != 0)
    {
     RA_HReset(pCohe->nameModule);

     if (pCohe->crossCohe_fmin >= 0.) 
        {
         nbin = pCohe->nBin/2+1;
         for (i=0 ;i<nbin ;i++)
            {for (j=0 ;j<nbin ;j++)
                {jj = j + i*nbin;
                 RA_H2Fill (pCohe->nameModule, pCohe->dnu*(double)i, pCohe->dnu*(double)j , pCohe->module2D->dataD[jj]);
                }
            }
        }

     else
        {
         for (j=0 ;j<pCohe->nBin/2+1 ;j++)
            {RA_H1Fill(pCohe->nameModule, pCohe->dnu * (double)j, pCohe->module->dataD[j]);}
        }
    }

 if (Dy->plotsNotDisplayed == OFF && strstr(pCohe->option1,"SAME") != NULL)
    {
     UHplot (pCohe->nameModule, pCohe->numPicture, pCohe->option1, pCohe->plotOpt1, 0, pCohe->refplot->nsame);
    }

 RA_SetActivePad(pCohe->refplot->numPicture);
 RA_HUpdateEnd(pCohe->nameModule,pCohe->plotOpt1);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpCoheEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  color=%d \n",
            pCohe->nameModule, pCohe->title, pCohe->numPicture, Dy->nready, Dy->nsame, Dy->nplot, pCohe->color);
    }

 if (pCohe->option1 != NULL)  free(pCohe->option1);
 if (pCohe->option2 != NULL)  free(pCohe->option2);
 if (pCohe->vect1   != NULL)  UACDFree(pCohe->vect1);
 if (pCohe->vect2   != NULL)  UACDFree(pCohe->vect2);
 if (pCohe->window  != NULL)  UADFree(pCohe->window);
 if (pCohe->buff1   !=NULL)   free(pCohe->buff1);
 if (pCohe->buff2   !=NULL)   free(pCohe->buff2);

 if (pCohe->crossCohe_fmin >= 0.)
    {
     if (pCohe->module2D != NULL)  UADFree(pCohe->module2D);
     if (pCohe->rer2D    != NULL)  UADFree(pCohe->rer2D);
     if (pCohe->imr2D    != NULL)  UADFree(pCohe->imr2D);
     if (pCohe->mod1sq2D != NULL)  UADFree(pCohe->mod1sq2D);
     if (pCohe->mod2sq2D != NULL)  UADFree(pCohe->mod2sq2D);
    }
 else
    {
     if (pCohe->module != NULL)  UADFree(pCohe->module);
     if (pCohe->rer    != NULL)  UADFree(pCohe->rer);
     if (pCohe->imr    != NULL)  UADFree(pCohe->imr);
     if (pCohe->mod1sq != NULL)  UADFree(pCohe->mod1sq);
     if (pCohe->mod2sq != NULL)  UADFree(pCohe->mod2sq);
    }

 if (pCohe != pCohe->refplot)
    {if (pCohe->nameModule  != NULL)  free(pCohe->nameModule);
     if (pCohe->title   != NULL)      free(pCohe->title);
     free(pCohe);
    }

 return;
}


/*---------------------------------------------------- UHpCoheIni -----*/
void UHpCoheIni(pCohe,dt)
struct UHpCohe *pCohe;
double dt;
/*--------------------------------------------------------------------*/
{
 long i, numplot, refnumplot;
 char param1[40],param2[40];
 char *oldname;
 static struct UHpCohe *previous=NULL;
 static int color;

 if (pCohe->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (pCohe->crossCohe_fmin >= 0.)
   {while (pCohe->nBin > 4096) {pCohe->nBin = pCohe->nBin/2.; pCohe->nBinP2--;}}

 if (pCohe->resampFactor <= 0)  pCohe->resampFactor = 1;

 dt              = dt * (double)pCohe->resampFactor;
 pCohe->dt       = dt;
 pCohe->dnu      = 1./(pCohe->nBin*dt);
 pCohe->i        = 0;
 pCohe->j        = 0;
 pCohe->dNbin    = pCohe->nBin;
 pCohe->nAverage = 0;
 pCohe->nRefresh = 0;
 pCohe->previousGPS = 0;
 pCohe->iFirst   = ON;

 pCohe->tmin = pCohe->dt * (double)pCohe->t0;

 pCohe->vect1    = UACDNew(1,pCohe->nBin);
 pCohe->vect2    = UACDNew(1,pCohe->nBin);
 bzero(pCohe->vect1->data, pCohe->vect1->nBytes);
 bzero(pCohe->vect2->data, pCohe->vect2->nBytes);

 if (pCohe->crossCohe_fmin >= 0.)
    {
     pCohe->module2D   = UADNew(pCohe->nBin/2+1, pCohe->nBin/2+1);
     pCohe->rer2D      = UADNew(pCohe->nBin/2+1, pCohe->nBin/2+1);
     pCohe->imr2D      = UADNew(pCohe->nBin/2+1, pCohe->nBin/2+1);
     pCohe->mod1sq2D   = UADNew(pCohe->nBin/2+1, pCohe->nBin/2+1);
     pCohe->mod2sq2D   = UADNew(pCohe->nBin/2+1, pCohe->nBin/2+1);
     if (pCohe->module2D==NULL || pCohe->rer2D==NULL || pCohe->imr2D==NULL)
        {printf("UHpCoheIni> crossCohe vectors not allocated for %d x %d bytes. I stop.\n",
                 sizeof(double)*pCohe->nBin, sizeof(double)*pCohe->nBin);
         exit(0);
        }
     bzero(pCohe->module2D->data, pCohe->module2D->nBytes);
     bzero(pCohe->rer2D->data, pCohe->rer2D->nBytes);
     bzero(pCohe->imr2D->data, pCohe->imr2D->nBytes);
     bzero(pCohe->mod1sq2D->data, pCohe->mod1sq2D->nBytes);
     bzero(pCohe->mod2sq2D->data, pCohe->mod2sq2D->nBytes);
    }

 else
    {
     pCohe->module   = UADNew(1,pCohe->nBin/2+1);
     pCohe->rer      = UADNew(1,pCohe->nBin/2+1);
     pCohe->imr      = UADNew(1,pCohe->nBin/2+1);
     pCohe->mod1sq   = UADNew(1,pCohe->nBin/2+1);
     pCohe->mod2sq   = UADNew(1,pCohe->nBin/2+1);

     bzero(pCohe->module->data, pCohe->module->nBytes);
     bzero(pCohe->rer->data, pCohe->rer->nBytes);
     bzero(pCohe->imr->data, pCohe->imr->nBytes);
     bzero(pCohe->mod1sq->data, pCohe->mod1sq->nBytes);
     bzero(pCohe->mod2sq->data, pCohe->mod2sq->nBytes);
    }

 pCohe->window   = UADNew(1,pCohe->nBin);
 bzero(pCohe->window->data, pCohe->window->nBytes);

 for (i=0; i<pCohe->nBin; i++)
    {pCohe->window->dataD[i] = 0.5 * (1. + cos( TWOPI * 
                               ((double)i - ((double)pCohe->nBin - 1.)/2.)/
                               ((double)pCohe->nBin - 1.)));}

 /*---- decode the options string --------*/

 UHoptDecode (pCohe->option1, &pCohe->intervalle, pCohe->plotOpt1);

 if (pCohe->crossCohe_fmin >= 0.) pCohe->intervalle = pCohe->nBin/2;
 if (pCohe->intervalle <= 0) pCohe->intervalle = pCohe->nBin;

 if (Dy->plotsNotDisplayed == OFF) 
    {if (strstr(pCohe->option1,"SAME") == NULL) pCohe->numPicture = DyGrafIni(0);
     else                                       pCohe->numPicture = previous->numPicture;
     pCohe->dNbin = pCohe->intervalle;
    }

 if (pCohe->crossCohe_fmin >= 0.)
    {
     oldname = RA_PadContainsGoodHist(pCohe->numPicture, 0, 2, pCohe->nBin/2+1, pCohe->nBin/2+1, pCohe->nameModule);
     if (!oldname)
       {RA_PadCompleteClear(pCohe->numPicture);
        UHoptPlopt(pCohe->option1, pCohe->numPicture, 0);
       }
     RA_H2NewReuse(pCohe->numPicture, 0, pCohe->nameModule, pCohe->title,
                   pCohe->nBin/2+1, pCohe->dnu*(-.5), pCohe->dnu*((double)(pCohe->nBin/2+1) - .5),
                   pCohe->nBin/2+1, pCohe->dnu*(-.5), pCohe->dnu*((double)(pCohe->nBin/2+1) - .5));
    }

 else
    {
     /*  Get the limits of the previous histogram in the pad if it is good  */
     oldname = RA_PadContainsGoodHist(pCohe->numPicture, 0, 1, pCohe->nBin/2+1, 0, pCohe->nameModule);
     RA_H1NewReuse(pCohe->numPicture, 0, pCohe->nameModule, pCohe->title,
                   pCohe->nBin/2+1, pCohe->dnu*(-.5), pCohe->dnu*((double)(pCohe->nBin/2+1) - .5));
    }

 RA_SetTestTrigPad(pCohe->numPicture, 0, (double)pCohe->intervalle * dt * pCohe->maxRefresh);

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, pCohe->nameModule);

 RA_HSetStat(pCohe->nameModule, NULL);

 RA_HSetMinimum(pCohe->nameModule, 0.);
 RA_HSetMaximum(pCohe->nameModule, 1.);

 if (pCohe->autoScale == 0 && pCohe->yMin > 0. && pCohe->yMax > 0.)
     {RA_HSetMinimum(pCohe->nameModule, pCohe->yMin);
      RA_HSetMaximum(pCohe->nameModule, pCohe->yMax);
     }

 if (!oldname)
    {if (pCohe->fmax <= pCohe->fmin) pCohe->fmax = 0.5 / pCohe->dt;
     RA_HSetXRange(pCohe->nameModule, pCohe->fmin, pCohe->fmax, pCohe->dnu);
     if (pCohe->crossCohe_fmin >= 0.)
        {if (pCohe->crossCohe_fmax <= pCohe->crossCohe_fmin) pCohe->crossCohe_fmax = 0.5 / pCohe->dt;
         RA_HSetYRange(pCohe->nameModule, pCohe->crossCohe_fmin, pCohe->crossCohe_fmax, pCohe->dnu);
        }
     UHoptPlopt(pCohe->option1, pCohe->numPicture, 1);
    }

 RA_HSetUnitsX(pCohe->nameModule, "Hz");

 pCohe->buff1 = (float *)calloc(pCohe->nBin+1, sizeof(float));
 if (pCohe->buff1 == NULL) printf("UHpTFctIni> pCohe->buff1 malloc failed\n");
 pCohe->buff2 = (float *)calloc(pCohe->nBin+1, sizeof(float));
 if (pCohe->buff2 == NULL) printf("UHpTFctIni> pCohe->buff2 malloc failed\n");

 sscanf(pCohe->nameModule, "h%ld", &numplot);

 if (strstr(pCohe->option1,"SAME") == NULL)
    {previous = pCohe;
     previous->nsame = 0;
     pCohe->refplot = previous;
     color = 1;
     Dy->refnumplot[numplot] = numplot;
    }
 else if (previous)
    {previous->nsame++;
     pCohe->refplot = previous;
     color++;
     if (color == 8)  color = 40+color-7;
     if (color == 48) color = 30+color-47;
     sscanf(pCohe->refplot->nameModule, "h%ld", &refnumplot);
     Dy->refnumplot[numplot] = refnumplot;
    }

 RA_PadContainsGoodHist(1,0, 1, pCohe->nBin/2+1, 0, pCohe->nameModule);

 pCohe->decay = 0.;
 if (strstr(pCohe->option1,"DECAYFFT") != NULL)
    {pCohe->decay = 1./(double)pCohe->maxAverage;}

 pCohe->color = color;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpCoheIni> plot=%s=%s=pad%d  color=%d  refplot=%s=%s=pad%d\n",
            pCohe->nameModule, pCohe->title, pCohe->numPicture, pCohe->color,
            pCohe->refplot->nameModule, pCohe->refplot->title, pCohe->refplot->numPicture);
    }

 return;
}



/*---------------------------------------------------- UHpCoheIniFrame -----*/

void UHpCoheIniFrame (struct UHpCohe *pCohe)
{
 long i;
 char param1[40],param2[40], dummy[LSTRING_MAX];
 char *name1, *name2, *temp, *oldname;
 long length, length2;
 struct FrAdcData *adc1=NULL;
 struct FrAdcData *adc2=NULL;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;
 static struct UHpCohe *previous=NULL;

 if (pCohe->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 temp   = strstr(pCohe->title,SEPTITLE);
 length = strlen(pCohe->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, pCohe->title, length);
 temp   = strstr(name1,SEP2VAR);
 length2 = strlen(name1)-strlen(temp);
 name2 = (char *) calloc(length2+1,sizeof(char));
 strncpy (name2, name1, length2);
 strcpy (name1, temp+strlen(SEP2VAR));

 if (UFrCurFrameH)
    {adc1  = FrAdcDataFind(UFrCurFrameH, name1);
     adc2  = FrAdcDataFind(UFrCurFrameH, name2);
     vect1 = FrameFindVect(UFrCurFrameH, name1);
     vect2 = FrameFindVect(UFrCurFrameH, name2);
    }

 if (adc1 == NULL  &&  vect1 == NULL)
    {adc1  = FrAdcDataFind(Dy->Frame, name1);
     vect1 = FrameFindVect(Dy->Frame, name1);
    }

 if (adc2 == NULL  &&  vect2 == NULL)
    {adc2  = FrAdcDataFind(Dy->Frame, name2);
     vect2 = FrameFindVect(Dy->Frame, name2);
    }

 if (pCohe->crossCohe_fmin >= 0.)
    {RA_SetStyle("LINX");
     RA_SetStyle("LINY");
     sprintf(dummy,"%s  Hz", name1);
     RA_HSetUnitsX(pCohe->nameModule, dummy);
     sprintf(dummy,"%s  Hz", name2);
     RA_HSetUnitsY(pCohe->nameModule, dummy);
    }

 free(name1);
 free(name2);

 pCohe->slope1 = 1.;
 pCohe->bias1  = 0.;
 pCohe->slope2 = 1.;
 pCohe->bias2  = 0.;

 if (adc1 != NULL)
    {if (adc1->slope != 0.) pCohe->slope1 = adc1->slope;
     pCohe->bias1  = adc1->bias;
    }
 if (adc2 != NULL)
    {if (adc2->slope != 0.) pCohe->slope2 = adc2->slope;
     pCohe->bias2  = adc2->bias;
    }

 return;
}



/*----------------------------------------------------- UHpCoheRead --*/
void UHpCoheRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHpCohe *pCohe;
 long i;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHpCohe, pCohe);

 UJ_READ_LONG(pCohe->iClock)
 UJ_READ_LONG(pCohe->idModule)
 if (!pCohe->nameModule) pCohe->nameModule  = (char *) calloc(10,sizeof(char));
 sprintf(pCohe->nameModule,"h%ld",pCohe->idModule);
 UJ_READ_LONG(pCohe->idPhase)
 UJ_READ_S_CHAR(pCohe->title)
 UJ_READ_LONG(pCohe->nBinP2)
 UJ_READ_LONG(pCohe->resampFactor)
 UJ_READ_LONG(pCohe->t0)
 UJ_READ_STRUCT(USignal,pCohe->input1)
 UJ_READ_STRUCT(USignal,pCohe->input2)
 UJ_READ_S_CHAR(pCohe->option1)
 UJ_READ_S_CHAR(pCohe->option2)

 i = 1; 
 pCohe->nBin = (i << pCohe->nBinP2);
 if (pCohe->nBin <= 0) pCohe->nBin = 1;

 UJCObAdd(pCohe->iClock,pCohe,UHpCoheUJstructH,NULL);

 pCohe->hidden = OFF;
 s = strstr(pCohe->option1,"HIDDEN");
 if (s != NULL)  pCohe->hidden = ON;

 pCohe->maxAverage = 1000000;
 s = strstr(pCohe->option1,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&pCohe->maxAverage);

 pCohe->maxRefresh = 1;
 s = strstr(pCohe->option1,"NFFT_REFRESH");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&pCohe->maxRefresh);
 if (pCohe->maxRefresh <= 0) pCohe->maxRefresh = 1;

 pCohe->fmin = 0.;
 s = strstr(pCohe->option1,"FMIN");
 if (s != NULL)  sscanf(s,"%s %lf", dummy, &pCohe->fmin);

 pCohe->fmax = 0.;
 s = strstr(pCohe->option1,"FMAX");
 if (s != NULL)  sscanf(s,"%s %lf", dummy, &pCohe->fmax);

 pCohe->yMin = 0.;
 s = strstr(pCohe->option1,"YMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCohe->yMin);

 pCohe->yMax = 0.;
 s = strstr(pCohe->option1,"YMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCohe->yMax);

 pCohe->autoScale = 0;
 s = strstr(pCohe->option1,"AUTOSCALE");
 if (s != NULL)  pCohe->autoScale = 1;

 pCohe->crossCohe_fmin = -1.;
 s = strstr(pCohe->option1,"CROSSCOHE_FMIN");
 if (s != NULL)  sscanf(s,"%s %lf", dummy, &pCohe->crossCohe_fmin);

 pCohe->crossCohe_fmax = -1.;
 s = strstr(pCohe->option1,"CROSSCOHE_FMAX");
 if (s != NULL)  sscanf(s,"%s %lf", dummy, &pCohe->crossCohe_fmax);

 pCohe->modulesqrt = 0;
 s = strstr(pCohe->option1,"MODULESQRT");
 if (s != NULL)  pCohe->modulesqrt = 1;

 return;
} 


/*--------------------------------------------------- UHpCoheSim -----*/
void UHpCoheSim(pCohe,time)
struct UHpCohe *pCohe;
long time;
/*--------------------------------------------------------------------*/
{
 long i,j, inter, inter2, jj, inter2x, interx, nbin;
 double *tab1,*tab2,*module,*phase,value1,value2;
 double *mod1sq,*mod2sq,*rer,*imr, mod1;
 char title[256], stringtime[40];
 time_t starttimeutc;
 double localMin, localMax;
 static float lastGoodData1=0.;
 static float lastGoodData2=0.;
 static double minofMin=DY_BADDATA;
 static double maxofMax=-DY_BADDATA;
 double origin;

 if (pCohe->hidden == ON) return;

 if (time < pCohe->t0)  return;

 if (pCohe->iFirst == ON)
    {pCohe->iFirst = OFF;
     UHpCoheIniFrame(pCohe);
    }

 if (pCohe->input1 == NULL)
    {UFrUnpackSim(pCohe->FrUnpack1, 0);
     value1 = pCohe->FrUnpack1->out->value;
    }
 else
    {value1 = pCohe->input1->value;}

 if (pCohe->realUnits != 0  &&  value1 < DY_BADDATA) 
    {value1 = value1 * pCohe->slope1 + pCohe->bias1;}

 if (pCohe->input2 == NULL)
    {UFrUnpackSim(pCohe->FrUnpack2, 0);
     value2 = pCohe->FrUnpack2->out->value;
    }
 else
    {value2 = pCohe->input2->value;}

 if (pCohe->realUnits != 0  &&  value2 < DY_BADDATA) 
    {value2 = value2 * pCohe->slope2 + pCohe->bias2;}


 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reaches resampFactor ---*/

 if (value1 < DY_BADDATA)         pCohe->sumvalue1 += value1;
 else if (value1 >= DY_NOCHANNEL) pCohe->sumvalue1  = DY_NOCHANNEL * 1.01 * pCohe->resampFactor;
 else if (value1 >= DY_NOFRAME)   pCohe->sumvalue1  = DY_NOFRAME * 1.01 * pCohe->resampFactor;
 else                             pCohe->sumvalue1  = DY_BADDATA * 1.01 * pCohe->resampFactor;

 if (value2 < DY_BADDATA)         pCohe->sumvalue2 += value2;
 else if (value2 >= DY_NOCHANNEL) pCohe->sumvalue2  = DY_NOCHANNEL * 1.01 * pCohe->resampFactor;
 else if (value2 >= DY_NOFRAME)   pCohe->sumvalue2  = DY_NOFRAME * 1.01 * pCohe->resampFactor;
 else                             pCohe->sumvalue2  = DY_BADDATA * 1.01 * pCohe->resampFactor;

 pCohe->j++;

 if (pCohe->j < pCohe->resampFactor) return;

 /*--- increment vector index and do the job if enough data has been taken ---*/

 pCohe->buff1[pCohe->i] = pCohe->sumvalue1 / pCohe->resampFactor;
 pCohe->buff2[pCohe->i] = pCohe->sumvalue2 / pCohe->resampFactor;
 pCohe->i++;
 pCohe->sumvalue1 = 0;
 pCohe->sumvalue2 = 0;
 pCohe->j = 0;

 if (pCohe->i == pCohe->nBin)
   {
    DyDebugPrint(__FUNCTION__,0);
    DyDebugTimeNeeded(__FUNCTION__,"start",1);

    pCohe->nAverage++;
    pCohe->nRefresh++;

    tab1 = pCohe->vect1->dataD;
    tab2 = pCohe->vect2->dataD;

    /*--- Ponderate data of channel 1 with a Hanning window ---*/

    for (i=0; i<pCohe->nBin; i++)
       {
        if (pCohe->buff1[i] < DY_BADDATA)
           {inter = 2*i;
            tab1[inter]   = pCohe->buff1[i] * pCohe->window->dataD[i];
            tab1[inter+1] = 0.;
            lastGoodData1 = pCohe->buff1[i];
           }
        else
           {inter = 2*i;
            tab1[inter]   = lastGoodData1 * pCohe->window->dataD[i];
            tab1[inter+1] = 0.;
           }
       }

    /*--- Ponderate data of channel 1 with a Hanning window ---*/

    for (i=0; i<pCohe->nBin; i++)
       {
        if (pCohe->buff2[i] < DY_BADDATA)
           {inter = 2*i;
            tab2[inter]   = pCohe->buff2[i] * pCohe->window->dataD[i];
            tab2[inter+1] = 0.;
            lastGoodData2 = pCohe->buff2[i];
           }
        else
           {inter = 2*i;
            tab2[inter]   = lastGoodData2 * pCohe->window->dataD[i];
            tab2[inter+1] = 0.;
           }
       }

    /*--- Do FFT on windowed buffers 1 and 2 ---*/

    UACDfft(pCohe->vect1,pCohe->nBinP2);
    UACDfft(pCohe->vect2,pCohe->nBinP2);

    /*--- Fill a 2D plot that shows the full cross-coherence ---*/

    if (pCohe->crossCohe_fmin >= 0.)
       {
        module = pCohe->module2D->dataD;
        rer    = pCohe->rer2D->dataD;
        imr    = pCohe->imr2D->dataD;
        mod1sq = pCohe->mod1sq2D->dataD;
        mod2sq = pCohe->mod2sq2D->dataD;
        nbin   = pCohe->nBin/2+1;

        for (i=0 ;i<nbin ;i++)
           {
            interx = 2*i;
            inter2x = interx+1;

            mod1 = (tab1[interx]*tab1[interx] + tab1[inter2x]*tab1[inter2x]);

            for (j=0 ;j<nbin ;j++)
               {
                inter  = 2*j;
                inter2 = inter+1;
	        jj = j + i*nbin;

                if (pCohe->decay>0 && pCohe->decay<1)
                   {mod1sq[jj]  = (1.-pCohe->decay) * mod1sq[jj] + pCohe->decay * mod1;
                    mod2sq[jj] = (1.-pCohe->decay) * mod2sq[jj] + pCohe->decay * (tab2[inter]*tab2[inter] + tab2[inter2]*tab2[inter2]);
                    rer[jj]   = (1.-pCohe->decay) * rer[jj]   + pCohe->decay * (tab2[inter]*tab1[interx] + tab2[inter2]*tab1[inter2x]);
                    imr[jj]   = (1.-pCohe->decay) * imr[jj]   + pCohe->decay * (-tab2[inter]*tab1[inter2x] + tab2[inter2]*tab1[interx]);
                   }
                else
                   {mod1sq[jj] += mod1;
                    mod2sq[jj] += tab2[inter]*tab2[inter] + tab2[inter2]*tab2[inter2];
                    rer[jj]    += (tab2[inter]*tab1[interx] + tab2[inter2]*tab1[inter2x]);
                    imr[jj]    += (-tab2[inter]*tab1[inter2x] + tab2[inter2]*tab1[interx]);
                   }

                if (mod1sq[jj] == 0. || mod2sq[jj] == 0.) module[jj] = 0.;
                else                                      module[jj] = (rer[jj]*rer[jj]+imr[jj]*imr[jj])/(mod1sq[jj]*mod2sq[jj]);
               }
           }
       }

    /*--- Show only the classical coherence function ---*/

    else 
       {
        module = pCohe->module->dataD;
        rer    = pCohe->rer->dataD;
        imr    = pCohe->imr->dataD;
        mod1sq = pCohe->mod1sq->dataD;
        mod2sq = pCohe->mod2sq->dataD;

        for (j=0 ;j<pCohe->nBin/2+1 ;j++)
           {inter  = 2*j;
            inter2 = inter+1;
	
            if (pCohe->decay>0 && pCohe->decay<1)
               {
                mod1sq[j] =  (1.-pCohe->decay) * mod1sq[j] + pCohe->decay * (tab1[inter]*tab1[inter] + tab1[inter2]*tab1[inter2]);
                mod2sq[j] =  (1.-pCohe->decay) * mod2sq[j] + pCohe->decay * (tab2[inter]*tab2[inter] + tab2[inter2]*tab2[inter2]);
                rer[j]    =  (1.-pCohe->decay) * rer[j]    + pCohe->decay * (tab2[inter]*tab1[inter] + tab2[inter2]*tab1[inter2]);
                imr[j]    =  (1.-pCohe->decay) * imr[j]    + pCohe->decay * (-tab2[inter]*tab1[inter2] + tab2[inter2]*tab1[inter]);
               }
            else
               {
                mod1sq[j] += tab1[inter]*tab1[inter] + tab1[inter2]*tab1[inter2];
                mod2sq[j] += tab2[inter]*tab2[inter] + tab2[inter2]*tab2[inter2];
                rer[j]    += (tab2[inter]*tab1[inter] + tab2[inter2]*tab1[inter2]);
                imr[j]    += (-tab2[inter]*tab1[inter2] + tab2[inter2]*tab1[inter]);
               }

            if (pCohe->modulesqrt == 1)
               {if (mod1sq[j] == 0. || mod2sq[j] == 0.) module[j] = 0.;
                else                                    module[j] = sqrt((rer[j]*rer[j]+imr[j]*imr[j])/(mod1sq[j]*mod2sq[j]));
               }
            else
               {if (mod1sq[j] == 0. || mod2sq[j] == 0.) module[j] = 0.;
                else                                    module[j] = (rer[j]*rer[j]+imr[j]*imr[j])/(mod1sq[j]*mod2sq[j]);
               }
           }
       }

    /*--- Search for max and min over the current plot ---*/

    if (pCohe->autoScale > 0)
       {
        localMin = DY_BADDATA;
        localMax = -DY_BADDATA;
        if (module[0] < DY_BADDATA) localMin = module[0];
        if (module[0] < DY_BADDATA) localMax = module[0];
        for (j=0 ;j<pCohe->nBin/2+1 ;j++)
           {if (localMin > module[j]  &&  module[j] < DY_BADDATA)  localMin = module[j];
            if (localMax < module[j]  &&  module[j] < DY_BADDATA)  localMax = module[j];
           }
       }

    /*--- Set color of the plot line and reset the plot content ---*/

    RA_HSetColor(pCohe->nameModule, pCohe->color);

    RA_HReset(pCohe->nameModule);

    /*--- Fill the plot ---*/

    if (pCohe->crossCohe_fmin >= 0.) 
       {
        nbin = pCohe->nBin/2+1;
        for (i=0 ;i<nbin ;i++)
           {for (j=0 ;j<nbin ;j++)
               {jj = j + i*nbin;
                RA_H2Fill (pCohe->nameModule, pCohe->dnu*(double)i, pCohe->dnu*(double)j , module[jj]);
               }
           }
       }

    else
       {
        for (j=0 ;j<pCohe->nBin/2+1 ;j++)
           {RA_H1Fill (pCohe->nameModule, pCohe->dnu*(double)j , module[j]);}
       }

    /*--- Put to zero the input data vectors ---*/

    for (j=0; j<2*pCohe->nBin; j++)
       {pCohe->vect1->dataD[j] = 0.;
        pCohe->vect2->dataD[j] = 0.;
       }

    /*--- Set max and min over all the superposed plots ---*/

    if (pCohe->autoScale > 0)
       {
        if (strstr(pCohe->option1,"SAME") == NULL)
            {minofMin = localMin;
             maxofMax = localMax;
            }
         else
            {if (minofMin > localMin) minofMin = localMin;
             if (maxofMax < localMax) maxofMax = localMax;
            }
       }

   /*--- Draw the plot ---*/

    if (Dy->plotsNotDisplayed == OFF && pCohe->nRefresh == pCohe->maxRefresh)
       {
        if (pCohe->autoScale > 0)
           {
            if (strstr(pCohe->option1,"SAME") == NULL)
               {
                if (pCohe->nsame > 0)
                   {RA_HSetMinimum(pCohe->nameModule, localMin*0.8);
                    RA_HSetMaximum(pCohe->nameModule, localMax*1.2);
                   }
               }
            else if (pCohe->refplot)
               {RA_HSetMinimum(pCohe->refplot->nameModule, minofMin*0.8);
                RA_HSetMaximum(pCohe->refplot->nameModule, maxofMax*1.2);
               }
           }

        /*--- Set time string and time axis offset ---*/
        origin = Dy->timeOrigin + pCohe->tmin;
        UHplotSetTime (origin, pCohe->nameModule, pCohe->nBin*pCohe->dt, pCohe->nAverage, OFF, UFrCurFrameH);

        /*--- Draw the plot ---*/ 
        UHplot (pCohe->nameModule, pCohe->numPicture, pCohe->option1, pCohe->plotOpt1, 0, pCohe->refplot->nsame);

        /*--- Update plot environment ---*/
        RA_SetActivePad(pCohe->refplot->numPicture);
        UHplotUpdateEnv (pCohe->numPicture, pCohe->nameModule, pCohe->option1, pCohe->refplot->nameModule,
                         pCohe->refplot->numPicture, pCohe->refplot->nsame, pCohe->title,
                         pCohe->color, pCohe->buff1, pCohe->nBin, &pCohe->corrupt);
        UHplotUpdateEnv (pCohe->numPicture, NULL, pCohe->option1, pCohe->refplot->nameModule,
                         pCohe->refplot->numPicture, pCohe->refplot->nsame, pCohe->title,
                         pCohe->color, pCohe->buff2, pCohe->nBin, &pCohe->corrupt);

        RA_HUpdate(pCohe->numPicture, pCohe->nameModule, pCohe->title, pCohe->option1);
       }

    DyCheckRunning();    /* interaction with the data Display */

    if (pCohe->nRefresh == pCohe->maxRefresh) pCohe->nRefresh = 0;

    if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) 
       {printf("pCoheSim>  module[1]=%g  module[%d/2]=%g module[%d]=%g  nAv=%d maxAv=%d\n",
                module[1], pCohe->nBin/2, module[pCohe->nBin/4], pCohe->nBin/2, module[pCohe->nBin/2],
                pCohe->nAverage, pCohe->maxAverage);
       }

    if (pCohe->decay>0 && pCohe->decay<1)
       {
        if (pCohe->nAverage >= pCohe->maxAverage) pCohe->nAverage = pCohe->maxAverage-1;
       }
    else
       {
        if (pCohe->nAverage == pCohe->maxAverage && pCohe->decay == 0)
           {pCohe->nAverage = 0;
            for (j=0 ;j<pCohe->nBin/2+1 ;j++)  mod1sq[j] = 0.;
            for (j=0 ;j<pCohe->nBin/2+1 ;j++)  mod2sq[j] = 0.;
            for (j=0 ;j<pCohe->nBin/2+1 ;j++)  rer[j]    = 0.;
            for (j=0 ;j<pCohe->nBin/2+1 ;j++)  imr[j]    = 0.;
           }
       }

     /*--- Translate buffers and update tmin value ---*/
     for (j=0; j<pCohe->nBin-pCohe->dNbin; j++)
        {pCohe->buff1[j] = pCohe->buff1[j+pCohe->dNbin];
         pCohe->buff2[j] = pCohe->buff2[j+pCohe->dNbin];
        }
     pCohe->i = pCohe->i - pCohe->dNbin;
     pCohe->tmin = pCohe->tmin + pCohe->dt * pCohe->dNbin;

    DyDebugTimeNeeded(__FUNCTION__,"stop",1);
   }

 if (UFrCurFrameH)
    {
     if ((pCohe->previousGPS > 0) &&  ((double)(UFrCurFrameH->GTimeS-pCohe->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {pCohe->tmin += (double)(UFrCurFrameH->GTimeS - pCohe->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) pCohe->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*============================================== PLOT CORRELATION ====================================*/

/*---------------------------------------------------- UHpCorrEnd -----*/
void UHpCorrEnd(pCorr)
struct UHpCorr *pCorr;
/*--------------------------------------------------------------------*/
{
 double tmax;

 if (pCorr->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(pCorr->refplot->numPicture);
 RA_HUpdateEnd(pCorr->name,pCorr->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotCorrEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  color=%d\n",
             pCorr->name, pCorr->title, pCorr->numPicture, Dy->nready, Dy->nsame, Dy->nplot, pCorr->color);
    }

 if (pCorr->options   != NULL)  free(pCorr->options);
 if (pCorr->vect1     != NULL)  UACDFree(pCorr->vect1);
 if (pCorr->vect2     != NULL)  UACDFree(pCorr->vect2);
 if (pCorr->buff      != NULL)  free(pCorr->buff);
 if (pCorr->buff1     != NULL)  free(pCorr->buff1);
 if (pCorr->buff2     != NULL)  free(pCorr->buff2);

 if (pCorr != pCorr->refplot)
    {if (pCorr->name      != NULL)  free(pCorr->name);
     if (pCorr->title     != NULL)  free(pCorr->title);
     free(pCorr);
    }

 return;
}


/*---------------------------------------------------- UHpCorrIni -----*/
void UHpCorrIni(pCorr,dt)
struct UHpCorr *pCorr;
double dt;
/*--------------------------------------------------------------------*/
{
 long numplot, refnumplot;
 char param1[64], param2[64];
 double tmax;
 long i, length;
 char *oldname;
 static struct UHpCorr *previous=NULL;
 static int color=0;

 if (pCorr->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (pCorr->resampFactor <= 0)  pCorr->resampFactor = 1;

 dt              = dt * (double)pCorr->resampFactor;
 pCorr->dNbin   = pCorr->nBin / 4;
 pCorr->dNbinP2 = pCorr->nBinP2 - 2;
 pCorr->dt      = dt;
 pCorr->dnu     = 1./(pCorr->dNbin*dt);
 pCorr->j       = 0;
 pCorr->i       = 0;
 pCorr->i2      = 0;
 pCorr->sumvalue1   = 0.;
 pCorr->sumvalue2   = 0.;
 pCorr->previousGPS = 0;
 pCorr->iFirst  = ON;

 tmax = pCorr->dt * (double)pCorr->nBin;

 pCorr->vect1   = UACDNew (1, pCorr->dNbin);
 pCorr->vect2   = UACDNew (1, pCorr->dNbin);
 pCorr->window  = UADNew(1,pCorr->dNbin);

 bzero(pCorr->vect1->data, pCorr->vect1->nBytes);
 bzero(pCorr->vect2->data, pCorr->vect2->nBytes);
 bzero(pCorr->window->data, pCorr->window->nBytes);

 for (i=0; i<pCorr->dNbin; i++)
    {pCorr->window->dataD[i] = 0.5 * (1. + cos( TWOPI * 
                               ((double)i - ((double)pCorr->dNbin - 1.)/2.)/
                               ((double)pCorr->dNbin - 1.)));}

 /*---- decode the options string --------*/

 UHoptDecode (pCorr->options, &pCorr->intervalle, pCorr->plotOpt);

 pCorr->intervalle = pCorr->dNbin/2;

 if (Dy->plotsNotDisplayed == OFF)  pCorr->numPicture = DyGrafIni(0);

 /*----  --------*/
 oldname = RA_PadContainsGoodHist(pCorr->numPicture, 0, 1, pCorr->nBin, 0, pCorr->name);

 RA_H1NewReuse(pCorr->numPicture, 0, pCorr->name, pCorr->title, pCorr->nBin, 0., tmax);
 RA_SetTestTrigPad(pCorr->numPicture, 0, (double)pCorr->intervalle * dt);

 if (!oldname)
    {UHoptPlopt(pCorr->options, pCorr->numPicture, 0);
     RA_HSetMinimum(pCorr->name, pCorr->yMin);
     RA_HSetMaximum(pCorr->name, pCorr->yMax);
    }

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, pCorr->name);

 RA_HSetStat(pCorr->name, NULL);
 RA_HSetUnitsY(pCorr->name, "a.u.");

 pCorr->buff1 = (float *)calloc(pCorr->dNbin+1, sizeof(float));
 if (pCorr->buff1 == NULL) printf("UHpCorrIni> pCorr->buff1 malloc failed\n");
 pCorr->buff2 = (float *)calloc(pCorr->nBin+1, sizeof(float));
 if (pCorr->buff2 == NULL) printf("UHpCorrIni> pCorr->buff2 malloc failed\n");
 pCorr->buff = (float *)calloc(pCorr->nBin+1, sizeof(float));
 if (pCorr->buff == NULL) printf("UHpCorrIni> pCorr->buff malloc failed\n");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHpCorrIni> nBin=%ld  nBinP2=%ld  dNbin=%ld  dNbinP2=%d  tmax=%g  intervalle=%d\n",
             pCorr->nBin, pCorr->nBinP2, pCorr->dNbin,  pCorr->dNbinP2, tmax, pCorr->intervalle);
    }

 sscanf(pCorr->name, "h%ld", &numplot);

 if (strstr(pCorr->options,"SAME") == NULL)
    {previous = pCorr;
     previous->nsame = 0;
     pCorr->refplot = previous;
     Dy->refnumplot[numplot] = numplot;
     pCorr->color = 1;
     color = 0;
    }
 else if (previous)
    {previous->nsame++;
     pCorr->refplot = previous;
     sscanf(pCorr->refplot->name, "h%ld", &refnumplot);
     Dy->refnumplot[numplot] = refnumplot;
     color++;
     pCorr->color = color;
    }

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpCorrIni> plot=%s=%s=pad%d  color=%d  refplot=%s=%s=pad%d \n",
             pCorr->name, pCorr->title, pCorr->numPicture, pCorr->color,
             pCorr->refplot->name, pCorr->refplot->title, pCorr->refplot->numPicture);
    }

 return;
}



/*---------------------------------------------------- UHpCorrIniFrame -----*/

void UHpCorrIniFrame (struct UHpCorr *pCorr)
{
 char param1[64], param2[64];
 double tmax;
 long i, length;
 char *oldname;
 static struct UHpCorr *previous=NULL;
 double offset, framedt;
 struct FrAdcData *adc=NULL;
 struct FrVect *vect=NULL;
 char *temp, *name1;

 if (pCorr->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 temp   = strstr(pCorr->title,SEPTITLE);
 length = strlen(pCorr->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, pCorr->title, length); 
 if (UFrCurFrameH)
    {adc  = FrAdcDataFind(UFrCurFrameH, name1);
     if (adc == NULL) adc = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(UFrCurFrameH, name1);
     if (vect == NULL) vect = FrameFindVect(Dy->Frame, name1);
    }

 if (adc == NULL  &&  vect == NULL)
    {adc  = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(Dy->Frame, name1);
    }
 free(name1);

 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     if (UFrCurFrameH != NULL)
        {framedt = DyGetFramedt(UFrCurFrameH);
         offset = ((double)Dy->timeOrigin - (double)UFrCurFrameH->GTimeS) / framedt;
         offset = offset - (int)offset;
         if (offset*UFrCurFrameH->dt < 1.) offset = 0.;

        }
     else                     
        {
         printf("UHpCorrIniFrame> UFrCurFrameH=NULL. Forced offset=0.\n");
         offset = 0.;
        }

     pCorr->timeOffset = offset * framedt * pCorr->resampFactor;

     if (offset > 0.)
        {if (vect) pCorr->t0 = (int)(offset*UFrCurFrameH->dt / vect->dx[0]);
         else      pCorr->t0 = (int)(offset*UFrCurFrameH->dt);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
            {printf("UHpCorrIniFrame> startTime=%ld  GTimeS=%d  framedt=%f  offset=%f  t0=%ld  pCorr->timeOffset=%f  ",
                     Dy->timeOrigin, UFrCurFrameH->GTimeS, UFrCurFrameH->dt, offset, pCorr->t0, pCorr->timeOffset);
             if (vect) printf("vect=%s:dx[0]=%f \n", vect->name, vect->dx[0]);
             else      printf("vect=NULL\n");
            }
        }
    }

 pCorr->tmin = pCorr->dt * (double)pCorr->t0;

 return;
}


/*----------------------------------------------------- UHpCorrRead --*/
void UHpCorrRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHpCorr *pCorr;
 long i;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHpCorr, pCorr);

 UJ_READ_LONG(pCorr->iClock)
 UJ_READ_LONG(pCorr->id)
 if (!pCorr->name) pCorr->name = (char *) calloc(10,sizeof(char));
 sprintf(pCorr->name,"h%ld",pCorr->id);
 UJ_READ_S_CHAR(pCorr->title)
 UJ_READ_LONG(pCorr->nBinP2)
 UJ_READ_LONG(pCorr->resampFactor)
 UJ_READ_LONG(pCorr->t0)
 UJ_READ_STRUCT(USignal,pCorr->input1)
 UJ_READ_STRUCT(USignal,pCorr->input2)
 UJ_READ_S_CHAR(pCorr->options)

 i = 1; 
 pCorr->nBin = (i << pCorr->nBinP2);
 if (pCorr->nBin <= 0) pCorr->nBin = 1;

 UJCObAdd(pCorr->iClock,pCorr,UHpCorrUJstructH,NULL);

 pCorr->hidden = OFF;
 s = strstr(pCorr->options,"HIDDEN");
 if (s != NULL)  pCorr->hidden = ON;

 pCorr->yMin = 0.;
 s = strstr(pCorr->options,"YMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCorr->yMin);

 pCorr->yMax = 0.;
 s = strstr(pCorr->options,"YMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCorr->yMax);

 pCorr->autoScale = 0;
 s = strstr(pCorr->options,"AUTOSCALE");
 if (s != NULL)  pCorr->autoScale = 1;

 if (pCorr->yMax <= pCorr->yMin)  pCorr->autoScale = 1;  

 return;
} 


/*--------------------------------------------------- UHpCorrSim -----*/
void UHpCorrSim(pCorr,time)
struct UHpCorr *pCorr;
long time;
/*--------------------------------------------------------------------*/
{
 long   i, j, inter, inter2;
 double *tab1,*tab2;
 double tmax, value1, value2, a, b;
 double localMin, localMax;
 static float lastGoodData1=0.;
 static float lastGoodData2=0.;
 static double minofMin=DY_BADDATA;
 static double maxofMax=-DY_BADDATA;
 double origin;
 int status;

 if (pCorr->hidden == ON) return;

 if (time < pCorr->t0)  return;

 if (pCorr->iFirst == ON)
    {pCorr->iFirst = OFF;
     UHpCorrIniFrame(pCorr);
    }

 if (pCorr->input1 == NULL)
    {UFrUnpackSim(pCorr->FrUnpack1, 0);
     value1 = pCorr->FrUnpack1->out->value;
    }
 else
    {value1 = pCorr->input1->value;}

 if (pCorr->input2 == NULL)
    {UFrUnpackSim(pCorr->FrUnpack2, 0);
     value2 = pCorr->FrUnpack2->out->value;
    }
 else
    {value2 = pCorr->input2->value;}

 if (value1 < DY_BADDATA)         pCorr->sumvalue1 += value1;
 else if (value1 >= DY_NOCHANNEL) pCorr->sumvalue1  = DY_NOCHANNEL * 1.01 * pCorr->resampFactor;
 else if (value1 >= DY_NOFRAME)   pCorr->sumvalue1  = DY_NOFRAME * 1.01 * pCorr->resampFactor;
 else                             pCorr->sumvalue1  = DY_BADDATA * 1.01 * pCorr->resampFactor;

 if (value2 < DY_BADDATA)         pCorr->sumvalue2 += value2;
 else if (value2 >= DY_NOCHANNEL) pCorr->sumvalue2  = DY_NOCHANNEL * 1.01 * pCorr->resampFactor;
 else if (value2 >= DY_NOFRAME)   pCorr->sumvalue2  = DY_NOFRAME * 1.01 * pCorr->resampFactor;
 else                             pCorr->sumvalue2  = DY_BADDATA * 1.01 * pCorr->resampFactor;

 pCorr->j++;

 if (pCorr->j < pCorr->resampFactor) return;

 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reaches resampFactor ---*/

 pCorr->buff1[pCorr->i2] = pCorr->sumvalue1 / pCorr->resampFactor;
 pCorr->buff2[pCorr->i]  = pCorr->sumvalue2 / pCorr->resampFactor;
 pCorr->i2++;
 pCorr->i++;
 pCorr->sumvalue1 = 0;
 pCorr->sumvalue2 = 0;
 pCorr->j = 0;

 if (pCorr->i2 >= pCorr->dNbin)
    {
     tab1 = pCorr->vect1->dataD;
     tab2 = pCorr->vect2->dataD;

     /*--- Ponderate data of channel1 with a Hanning window ---*/

     for (i=0; i<pCorr->dNbin; i++)
        {
         if (pCorr->buff1[i] < DY_BADDATA)
            {inter = 2*i;
             tab1[inter]   = pCorr->buff1[i] * pCorr->window->dataD[i];
             tab1[inter+1] = 0.;
             lastGoodData1 = pCorr->buff1[i];
            }
         else
            {inter = 2*i;
             tab1[inter]   = lastGoodData1 * pCorr->window->dataD[i];
             tab1[inter+1] = 0.;
            }
        }

     /*--- Ponderate data of channel2 with a Hanning window ---*/

     for (i=0; i<pCorr->dNbin; i++)
        {
         if (pCorr->buff2[i] < DY_BADDATA)
            {inter = 2*i;
             tab2[inter]   = pCorr->buff2[i] * pCorr->window->dataD[i];
             tab2[inter+1] = 0.;
             if (i>= pCorr->dNbin/2) tab2[inter] = 0.;
             lastGoodData2 = pCorr->buff2[i];
            }
         else
            {inter = 2*i;
             tab2[inter]   = lastGoodData2 * pCorr->window->dataD[i];
             tab2[inter+1] = 0.;
             if (i>= pCorr->dNbin/2) tab2[inter] = 0.;
            }
        }

     /*--- Compute FFTs ---*/

     UACDfft (pCorr->vect1, pCorr->dNbinP2);
     UACDfft (pCorr->vect2, pCorr->dNbinP2);

     for (j=0 ; j<pCorr->dNbin; j++)
        {inter  = 2*j;
         inter2 = inter+1;
         a      = tab1[inter];
         b      = tab1[inter2];
         tab1[inter]  = a*tab2[inter]  - b*tab2[inter2];
         tab1[inter2] = a*tab2[inter2] + b*tab2[inter];
        }

     UACDfft(pCorr->vect1,-1*pCorr->dNbinP2);

     for (j=0; j<pCorr->dNbin/2; j++) pCorr->buff[pCorr->i-pCorr->dNbin/2+j] = tab1[2*j];
     for (j=0; j<pCorr->dNbin/2; j++) pCorr->buff1[j] = pCorr->buff1[j+pCorr->dNbin/2];
     pCorr->i2 = pCorr->i2 - pCorr->dNbin/2;
    }


 if ((pCorr->i % (pCorr->dNbin/2) == 0  &&  pCorr->buff[pCorr->nBin-1]==0. &&  pCorr->buff[pCorr->nBin-2]==0.)
      ||  pCorr->i == pCorr->nBin)
    {
     DyDebugPrint(__FUNCTION__,0);

     /*--- Fill the plots ---*/
     RA_H1Pack(pCorr->name, pCorr->buff);

     /*--- Autoscale on Y axis ---*/
     if (pCorr->autoScale > 0)
        {
         localMin =  DY_BADDATA;
         localMax = -DY_BADDATA;
         if (pCorr->buff[0] < DY_BADDATA) localMin = pCorr->buff[0];
         if (pCorr->buff[0] < DY_BADDATA) localMax = pCorr->buff[0];
         for (j=0; j<pCorr->i; j++)
            {if (localMin > pCorr->buff[j]  &&  pCorr->buff[j] < DY_BADDATA)  localMin = pCorr->buff[j];
             if (localMax < pCorr->buff[j]  &&  pCorr->buff[j] < DY_BADDATA)  localMax = pCorr->buff[j];
            }

         if (strstr(pCorr->options,"SAME") == NULL)
            {minofMin = localMin;
             maxofMax = localMax;
            }
         else
            {if (minofMin > localMin) minofMin = localMin;
             if (maxofMax < localMax) maxofMax = localMax;
            }
        }

     /*---------- Visualize data -----------------*/

     if (Dy->plotsNotDisplayed == OFF)
        {
         if (pCorr->autoScale > 0)
            {
             if (strstr(pCorr->options,"SAME") == NULL)
                {
                 if (pCorr->nsame == 0)
                    {RA_HSetMinimum(pCorr->name, localMin - (localMax-localMin)/2.);
                     RA_HSetMaximum(pCorr->name, localMax + (localMax-localMin)/2.);
                    }
                }
             else if (pCorr->refplot)
                {RA_HSetMinimum(pCorr->refplot->name, minofMin - (maxofMax-minofMin)/2.);
                 RA_HSetMaximum(pCorr->refplot->name, maxofMax + (maxofMax-minofMin)/2.);
                }
            }

         /*--- Draw the plot ---*/
         UHplot(pCorr->name,pCorr->numPicture, pCorr->options, pCorr->plotOpt, 0, pCorr->refplot->nsame);

         /*--- Update the plot environment ---*/
         RA_SetActivePad(pCorr->numPicture);
         UHplotUpdateEnv (pCorr->numPicture, pCorr->name, pCorr->options, pCorr->refplot->name,
                          pCorr->refplot->numPicture, pCorr->refplot->nsame, pCorr->title,
                          pCorr->color, pCorr->buff1, pCorr->dNbin, &pCorr->corrupt);
         UHplotUpdateEnv (pCorr->numPicture, pCorr->name, pCorr->options, pCorr->refplot->name,
                          pCorr->refplot->numPicture, pCorr->refplot->nsame, pCorr->title,
                          pCorr->color, pCorr->buff2, pCorr->dNbin, &pCorr->corrupt);

         /*--- Update the canvas ---*/
         RA_HUpdate(pCorr->numPicture, pCorr->name, pCorr->title, pCorr->options);
        }

     /*--- Check user interaction with the dataDisplay ---*/
     DyCheckRunning();
    }

 /*--- shift buffer and begin time ---*/

 if (pCorr->i == pCorr->nBin)
    {
     for (j=0; j<pCorr->nBin-pCorr->dNbin/2; j++)
        {pCorr->buff[j]  = pCorr->buff[j+pCorr->dNbin/2];
         pCorr->buff2[j] = pCorr->buff2[j+pCorr->dNbin/2];
        }
     pCorr->i = pCorr->i - pCorr->dNbin/2;
     pCorr->tmin = pCorr->tmin + pCorr->dt * pCorr->dNbin/2;
    }

 /*--- Take care of missing frames ---*/

 if (UFrCurFrameH)
    {
     if ((pCorr->previousGPS > 0) && ((double)(UFrCurFrameH->GTimeS-pCorr->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {pCorr->tmin += (double)(UFrCurFrameH->GTimeS - pCorr->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) pCorr->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*================================================ PLOT SOUND ========================================*/

/*--------------------------------------------------- UHsoundEnd ----*/
void UHsoundEnd(struct UHsound *sound)
/*--------------------------------------------------------------------*/
{
 char cmd_play[2048];
 char cmd_volume[256];
 struct soundhdr hdr;
 long last;

 if (sound->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (sound->fd_audio > 0) close(sound->fd_audio);

 if (sound->i < sound->end_nb - sound->begin_nb && sound->fout != NULL)
    {last = ftell(sound->fout);
     rewind(sound->fout);
     DySoundFillHeader(&hdr,(long)(sound->Fsamp * sound->resampFactor), (sound->i-sound->begin_nb)*sound->encoding/8, (short)sound->encoding);
     fwrite(&hdr, 1, 44, sound->fout);
     fseek(sound->fout, last, SEEK_SET);
     printf("UHsoundEnd> END AUDIO FILE RECORDING: only %.2f sec (%ld samples) recorded at %.2f Hz\n",
            sound->i/sound->Fsamp, sound->i * sound->resampFactor, sound->Fsamp * sound->resampFactor);
    }

 if (sound->fout) fclose(sound->fout);
 sound->fout = NULL;

 /*--- Run audioplay with right parameters ---*/

#ifdef OSF1
     sprintf(cmd_play,"%s -filename %s -rate %.2f -bitspersample %d -encoding pcm",
             AUDIOPLAY_OSF1, sound->outfileName, sound->Fsamp*sound->resampFactor, sound->encoding);
     sprintf(cmd_volume,"%s &", AUDIOCONTROL_OSF1);
#else
     sprintf(cmd_play,"%s -r %.2f %s", AUDIOPLAY, sound->Fsamp*sound->resampFactor, sound->outfileName);
     sprintf(cmd_volume,"%s &", AUDIOCONTROL);
#endif

 printf("UHsoundEnd> A file has been created. You can play it with the following command:\n");
 printf("UHsoundEnd> play_command = %s\n",cmd_play);
 printf("UHsoundEnd> volume_control = %s\n",cmd_volume);

 free(sound);

 return;
}


/*--------------------------------------------------- UHsoundIni------*/
void UHsoundIni(struct UHsound *sound, double dt)
/*--------------------------------------------------------------------*/
{
 int tmp, sample_rate;
 char *str;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Set real sampling frequency ---*/

 if (dt > 0) sound->Fsamp = 1./dt;
 else        printf("UHsoundIni> Clock frequency not defined (dt=%f)\n", dt);

 /*--- Set new sampling frequency if needed ---*/

 sound->resampFactor = 1;
 if (sound->Fsamp < 5512.)
    {
     while (sound->resampFactor * sound->Fsamp < 5512.) sound->resampFactor++;
     printf("UHsoundIni> WARNING: Sampling frequency of %.2f Hz is too low for sound playing\n",sound->Fsamp);
     printf("UHsoundIni> Data will be interpolated to %.2f Hz to be in allowed frequency band: 5512 to 44100 Hz\n", sound->Fsamp*sound->resampFactor);
    }

 /*--- Initialise ---*/ 

 sound->iFirst      = ON;
 sound->begin_nb    = (long)(sound->beginTime*sound->Fsamp); 
 sound->end_nb      = (long)((sound->beginTime+sound->totTime)*sound->Fsamp);  
 sound->i           = 0;
 sound->nsamp       = 0;
 sound->fout        = NULL;
 sound->fd_audio    = 0;
 if (sound->encoding < 8) sound->encoding = 8;
 if (sound->begin_nb < AUDIO_BUF_SIZE) {sound->begin_nb += AUDIO_BUF_SIZE; sound->end_nb += AUDIO_BUF_SIZE;}

 /*--- Audio output file ---*/

 if (strlen(sound->outfileName) >= 2) {str = strstr(sound->outfileName,":"); if (str) strcpy(sound->outfileName, str+1);}
 if (strlen(sound->outfileName) < 2) sprintf(sound->outfileName,"%s.wav", sound->in_signal->name);
 else                                sprintf(sound->outfileName,"%s.wav", sound->outfileName);
 sound->data_nBytes = (unsigned long)(sound->totTime * sound->Fsamp * sound->resampFactor) * sound->encoding/8;

 printf("UHsoundIni> beginTime=%ld  totTime=%ld  Fsamp=%g  begin_nb=%ld  end_nb=%ld  data_nBytes=%ld  AUDIO_BUF_SIZE=%d  outfileName=%s\n",
         sound->beginTime, sound->totTime, sound->Fsamp, sound->begin_nb, sound->end_nb, sound->data_nBytes, AUDIO_BUF_SIZE, sound->outfileName);

 /*--- Set audio device for audio online output stream ---*/

 if (sound->soundonline > 0)
    {
     if (sound->encoding == 16)
        {if ((sound->fd_audio = open ("/dev/audio", O_WRONLY)) < 0)
            {printf("UHsoundIni> could not open audio device\n"); sound->fd_audio = 0;}
        }
     else
        {if ((sound->fd_audio = open ("/dev/dsp", O_WRONLY)) < 0)
            {printf("UHsoundIni> could not open dsp audio device\n"); sound->fd_audio = 0;}
        }

     if (sound->fd_audio)
        {
         if (sound->encoding == 16)
            {tmp = AFMT_S16_NE;
             if (ioctl (sound->fd_audio, SNDCTL_DSP_SETFMT, &tmp)==-1) {perror("SNDCTL_DSP_SETFMT"); exit(-1);}
             if (tmp != AFMT_S16_NE)
                {printf("UHsoundIni> The device doesn't support the 16 bit sample format.\n"); sound->fd_audio = 0; return;}
            }
         printf("UHsoundIni> audio device format set to %d bits\n", sound->encoding);

         tmp = 1;
         if (ioctl (sound->fd_audio, SNDCTL_DSP_CHANNELS, &tmp)==-1) {perror("SNDCTL_DSP_CHANNELS"); exit(-1);}
         if (tmp != 1) {printf("UHsoundIni> The device doesn't support mono mode.\n"); exit(-1);}

         sample_rate = sound->Fsamp * sound->resampFactor;
         if (ioctl (sound->fd_audio, SNDCTL_DSP_SPEED, &sample_rate)==-1) {perror("SNDCTL_DSP_SPEED"); exit(-1);}
         printf("UHsoundIni> audio device sample rate set to %d \n", sample_rate);
        }
    }
 
 return;
}

/*---------------------------------------------------- UHsoundIniFrame -----*/

void UHsoundIniFrame (struct UHsound *sound)
{
 struct FrVect *vect;
 char *name1, *temp;
 long length;

 if (sound->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 temp   = strstr(sound->outfileName,SEPTITLE);
 length = strlen(sound->outfileName)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, sound->outfileName, length);
 name1[length] = '\0';

 if (UFrCurFrameH)
    {vect = FrameFindVect(UFrCurFrameH, name1);
     if (vect == NULL) vect = FrameFindVect(Dy->Frame, name1);
    }

 if (vect) sound->vecttype = vect->type;
 else      sound->vecttype = FR_VECT_4R;

 printf("UHsoundIniFrame> channel=%s  vecttype=%d \n", name1, sound->vecttype);
 free(name1);

 return;
}


/*-------------------------------------------------- UHsoundRead------*/
void UHsoundRead(struct UJfile *iFile)
/*--------------------------------------------------------------------*/
{
 struct UHsound *sound;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHsound,sound)

 UJ_READ_LONG(sound->iClock)
 UJ_READ_STRUCT(USignal,sound->in_signal)
 UJ_READ_LONG(sound->beginTime)
 UJ_READ_LONG(sound->totTime)
 UJ_READ_S_CHAR(sound->options)
  
 UJCObAdd(sound->iClock,sound,UHsoundUJstructH,NULL);

 sound->hidden = OFF;
 s = strstr(sound->options,"HIDDEN");
 if (s != NULL)  sound->hidden = ON;

 s = strstr(sound->options,"FILENAME");
 if (s != NULL) sscanf(s,"%s %s", dummy, sound->outfileName);

 sound->encoding = 8;
 s = strstr(sound->options,"SOUND16BITS");
 if (s != NULL) sound->encoding = 16;

 sound->soundonline = 1;
 s = strstr(sound->options,"SOUNDOFF");
 if (s != NULL) sound->soundonline = 0;

 return;
}

 
/*--------------------------------------------------- UHsoundSim ----*/
void UHsoundSim(struct UHsound *sound, long time)           
/*--------------------------------------------------------------------*/
{
 int nwritten, z;
 double value, value_norm, valueBuf, valueBuf2, delta, timeNeeded;
 struct soundhdr hdr;
 unsigned char outchar;
 short outshort, outshort0;
 
 if (sound->hidden == ON) return;

 if (sound->iFirst == ON)
    {sound->iFirst = OFF;
     UHsoundIniFrame(sound);
    }

 if (sound->in_signal == NULL) printf("UHsoundSim> Input structure not found\n");
 else                          value = sound->in_signal->value;

 if      (sound->vecttype == FR_VECT_C)  outshort = (short)(value*256);
 else if (sound->vecttype == FR_VECT_1U) outshort = (short)(value*256-32768);
 else if (sound->vecttype == FR_VECT_2U) outshort = (short)(value-32768);
 else if (sound->vecttype == FR_VECT_4U) outshort = (short)(value/65536-32768);
 else if (sound->vecttype == FR_VECT_4S) outshort = (short)(value/65536.);
 else if (sound->vecttype == FR_VECT_2S) outshort = (short)value;

 else
    {
     if (time == 0) {sound->max_value = value; sound->min_value = value;}
     if (time < AUDIO_BUF_SIZE)
        {sound->max_value = (value > sound->max_value) ? value : sound->max_value; 
         sound->min_value = (value < sound->min_value) ? value : sound->min_value;
        }
     if (time == AUDIO_BUF_SIZE)
        {delta = (sound->max_value - sound->min_value);
         sound->min_value -= 0.5*delta;
         sound->max_value += 0.5*delta;
        }
     delta = sound->max_value - sound->min_value;
     if (delta == 0.) delta = 1.;
     value_norm = (value - sound->min_value) / delta;
     if (value_norm > 1.) value_norm = 1.;
     if (value_norm < 0.) value_norm = 0.;
     valueBuf = value_norm * (pow(2.0,(double)sound->encoding)-1.);
     outshort = (short)((float)valueBuf-32768.);
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1 && time%100000 < 10)
        {printf("UHsoundSim> vecttype=float  output=%s  samp=%d  i=%d  value=%g  value_normalized=%g  valueBuf=%g  min=%g  max=%g  outshort=%d\n",
                 sound->outfileName, time, sound->i, value, value_norm, valueBuf, sound->min_value, sound->max_value, outshort);
        }
    }


 /*--- Start audio file recording only after begin_nb ---*/
 if (time == sound->begin_nb)
    {printf("UHsoundSim> START AUDIO FILE RECORDING at tick=%ld\n", time);
     sound->fout = fopen(sound->outfileName,"w");
     if (sound->fout == NULL) printf("UHsoundSim> output file %s could not be opened\n", sound->outfileName);
     DySoundFillHeader(&hdr,(long)(sound->Fsamp * sound->resampFactor), sound->data_nBytes, (short)(sound->encoding));
     DySoundReadHeader((char *)(&hdr));
     if (sound->fout) fwrite(&hdr, 1, sizeof(hdr), sound->fout);
     sound->i = 0; 
    }

 /*--- Loop for oversampling if needed ---*/
 if (time == 0) sound->previousShort = outshort;
 outshort0 = outshort;
 for (z=1; z<=sound->resampFactor; z++)
    {
     /*--- Linearly interpolate for resampling a too low frequency signal ---*/
     if (sound->resampFactor > 1)
        {valueBuf2 = ((sound->resampFactor - z) * (double)sound->previousShort + z * outshort0) / sound->resampFactor;
         outshort = (short)((float)valueBuf2);
        }

     /*--- Put data in buffer for audio stream and in output file if requested ---*/
     if (sound->encoding == 16)
        {if (sound->fd_audio) sound->bufstreamS[sound->nsamp] = outshort;
         if (sound->fout)     fwrite(&outshort, sizeof(short), 1, sound->fout);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1 && sound->fout && time%100000 < 10)
            {printf("UHsoundSim> encoding=16bits  vecttype=%d  output=%s  samp=%ld  i+z=%ld+%ld  value=%g  outshort=%d\n",
                    sound->vecttype, sound->outfileName, time, sound->i, z, value, outshort);
            }
        }
     else
        {outchar = (char)((outshort+32768)/256);
         if (sound->fd_audio) sound->bufstream[sound->nsamp] = outchar;
         if (sound->fout)     fwrite(&outchar, sizeof(unsigned char), 1, sound->fout);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1 && sound->fout && time%100000 < 10)
            {printf("UHsoundSim> encoding=8bits  vecttype=%d  output=%s  samp=%ld  i+z=%ld+%ld  value=%g  outshort=%d  outchar=%d\n",
                    sound->vecttype, sound->outfileName, time, sound->i, z, value, outshort, outchar);
            }
        }

     /*--- Output data online to audio device each time the buffer is full ---*/

     if (sound->fd_audio && time > AUDIO_BUF_SIZE)
        {
         if (sound->nsamp == 0) DyDebugTime();
         sound->nsamp++;
         if (sound->nsamp == AUDIO_BUF_SIZE)
            {
             if (sound->encoding == 16)
                {if ((nwritten = write (sound->fd_audio, (char *)sound->bufstreamS, 2*AUDIO_BUF_SIZE)) < 0) perror ("/dev/audio");
                 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0 && time < 10*AUDIO_BUF_SIZE)
                    {printf("UHsoundSim> 16 bits Buffer %d written in dev/audio (tick=%d)\n", time/AUDIO_BUF_SIZE, time);}
                }
             else
                {if ((nwritten = write (sound->fd_audio, sound->bufstream, AUDIO_BUF_SIZE)) < 0) perror ("/dev/dsp");
                 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2 && time < 10*AUDIO_BUF_SIZE)
                    {printf("UHsoundSim> 8 bits Buffer %d written in dev/dsp (tick=%d)\n", time/AUDIO_BUF_SIZE, time);}
                }

             timeNeeded = DyDebugTime();
             if (sound->nsamp > 5512.  &&  timeNeeded < sound->nsamp / (sound->Fsamp * sound->resampFactor))
                {timeNeeded = 100000. * 0.8 * sound->nsamp / (sound->Fsamp * sound->resampFactor) - timeNeeded;
                 printf("UHsoundSim> Waiting %g microseconds to align plots update on audio buffering speed\n", timeNeeded);
                 usleep(timeNeeded);
                }
             sound->nsamp = 0;
            }
        }
    }

 sound->previousShort = outshort;
 if (sound->fout) sound->i++;

 /*--- Stop audio data recording in file ---*/

 if (sound->fout && (sound->i >= sound->end_nb-sound->begin_nb))
    {printf("UHsoundSim> END AUDIO FILE RECORDING at tick=%ld: %ld samples recorded\n", time, sound->i * sound->resampFactor);
     fclose(sound->fout);
     sound->fout = NULL;
    }
                 
 return;
}


/*====================================================================================================*/
/*============================================== PLOT FFT_TIME =======================================*/

/*---------------------------------------------------- UHplotFTEnd -----*/
void UHplotFTEnd(plotFFTTime)
struct UHplotFFTTime *plotFFTTime;
/*---------------------------------------------------------------------*/
{
 double origin;

 if (plotFFTTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(plotFFTTime->numPicture);

 /*--- Set time string and time axis offset ---*/
 origin = Dy->timeOrigin + plotFFTTime->tmin - plotFFTTime->timeOffset;
 UHplotSetTime (origin, plotFFTTime->name, plotFFTTime->nBin*plotFFTTime->dt, plotFFTTime->nAverage, ON, UFrCurFrameH);

 /*--- Do last painting for last data received but not yet displayed ---*/
 RA_HUpdateEnd(plotFFTTime->name,plotFFTTime->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotFTEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  nAverage=%d  plotOpt=%s\n",
             plotFFTTime->name, plotFFTTime->title, plotFFTTime->numPicture,
             Dy->nready, Dy->nsame, Dy->nplot, plotFFTTime->nAverage, plotFFTTime->plotOpt);
    }

 if (plotFFTTime->title   != NULL)  free(plotFFTTime->title);
 if (plotFFTTime->vect    != NULL)  UACDFree(plotFFTTime->vect);
 if (plotFFTTime->fft     != NULL)  UADFree(plotFFTTime->fft);
 if (plotFFTTime->window  != NULL)  UADFree(plotFFTTime->window);
 if (plotFFTTime->options != NULL)  free(plotFFTTime->options);
 if (plotFFTTime->name    != NULL)  free(plotFFTTime->name);
 if (plotFFTTime->buff    != NULL)  free(plotFFTTime->buff);
 if (plotFFTTime->buffTot != NULL)  free(plotFFTTime->buffTot);

 free(plotFFTTime);

 return;
}


/*---------------------------------------------------- UHplotFTIni -----*/
void UHplotFTIni(plotFFTTime,dt)
struct UHplotFFTTime *plotFFTTime;
double dt;
/*--------------------------------------------------------------------*/
{
 long i;
 char* oldname;

 if (plotFFTTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (plotFFTTime->resampFactor <= 0) plotFFTTime->resampFactor = 1;

 plotFFTTime->dt           = dt * (double)plotFFTTime->resampFactor;
 plotFFTTime->dnu          = 1./(plotFFTTime->nBin*plotFFTTime->dt);
 plotFFTTime->i            = 0;
 plotFFTTime->j            = 0;
 plotFFTTime->itime        = 0;
 plotFFTTime->itimeView    = 0;
 plotFFTTime->dNbin        = plotFFTTime->nBin/10;
 plotFFTTime->nAverage     = 0;
 plotFFTTime->previousGPS  = 0;
 plotFFTTime->t0           = 0;
 plotFFTTime->timeOffset   = 0.;
 plotFFTTime->refresh_nBin = 1;
 plotFFTTime->iFirst       = ON;

 plotFFTTime->vect = UACDNew(1,plotFFTTime->nBin);
 plotFFTTime->fft  = UADNew(1,plotFFTTime->nBin/2+1);

 bzero(plotFFTTime->vect->data, plotFFTTime->vect->nBytes);
 bzero(plotFFTTime->fft->data, plotFFTTime->fft->nBytes);

 /*--- Define the Hanning window ---*/

 plotFFTTime->window  = UADNew(1,plotFFTTime->nBin);
 bzero(plotFFTTime->window->data,plotFFTTime->window->nBytes);
  
 for (i=0; i<plotFFTTime->nBin; i++)
     {plotFFTTime->window->dataD[i] = 0.5 * (1. + cos( TWOPI * 
                                  ((double)i - ((double)plotFFTTime->nBin - 1.)/2.)/
                                  ((double)plotFFTTime->nBin - 1.)));}
 plotFFTTime->norm = 0.;
 for (i=0; i < plotFFTTime->nBin; i++)
     {plotFFTTime->norm += plotFFTTime->window->dataD[i]*plotFFTTime->window->dataD[i];}
 plotFFTTime->norm = plotFFTTime->dt / plotFFTTime->norm; 
 plotFFTTime->norm = plotFFTTime->norm * sqrt(2.); 

 /*---- decode the options string --------*/

 UHoptDecode(plotFFTTime->options, &plotFFTTime->intervalle, plotFFTTime->plotOpt);

 if (plotFFTTime->intervalle <= 0) plotFFTTime->intervalle = plotFFTTime->nBin;

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plotFFTTime->options,"SAME") == NULL) plotFFTTime->numPicture = DyGrafIni(0);
     else                                             plotFFTTime->numPicture = 0;
     plotFFTTime->dNbin = plotFFTTime->intervalle;
    }

/*---  Define the 2D plot used for time/freq and set limits on frequency axis  ---*/

 plotFFTTime->nBinTime = (long)(plotFFTTime->timeSize
                         / (plotFFTTime->dNbin * plotFFTTime->maxAverage * plotFFTTime->dt));

 if (plotFFTTime->refresh_percent > 0)
    {
     plotFFTTime->refresh_nBin = (long)((plotFFTTime->refresh_percent * plotFFTTime->timeSize/100.) 
                                 / (plotFFTTime->dNbin * plotFFTTime->maxAverage * plotFFTTime->dt));
     if (plotFFTTime->refresh_nBin <= 0) plotFFTTime->refresh_nBin = 1;
    }

 if (plotFFTTime->intervalle < plotFFTTime->maxAverage * plotFFTTime->dNbin)
    {plotFFTTime->intervalle = plotFFTTime->maxAverage * plotFFTTime->dNbin;}

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplotFTIni> plotFFTTime=%s  timeSize=%g  dNbin=%ld  dt=%g  nBinTime=%ld\n",
             plotFFTTime->title, plotFFTTime->timeSize, plotFFTTime->dNbin, plotFFTTime->dt, plotFFTTime->nBinTime);
    }

/*---  Get the limits of the previous histogram in the pad if it is good  ---*/ 
 oldname = RA_PadContainsGoodHist(plotFFTTime->numPicture, 0, 2, plotFFTTime->nBinTime, plotFFTTime->nBin/2+1, plotFFTTime->name);
 if (!oldname)
   {RA_PadCompleteClear(plotFFTTime->numPicture);
    UHoptPlopt(plotFFTTime->options, plotFFTTime->numPicture, 0);
   }

 RA_H2NewReuse(plotFFTTime->numPicture, 0,
               plotFFTTime->name, plotFFTTime->title,
               plotFFTTime->nBinTime,
               plotFFTTime->maxAverage * plotFFTTime->dNbin * plotFFTTime->dt * (-0.5),
               plotFFTTime->maxAverage * plotFFTTime->dNbin * plotFFTTime->dt * ((double)plotFFTTime->nBinTime - 0.5),
               plotFFTTime->nBin/2+1,
               plotFFTTime->dnu*(-0.5),
               plotFFTTime->dnu*((double)(plotFFTTime->nBin/2+1) - 0.5));

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plotFFTTime->name);

 RA_HSetStat(plotFFTTime->name, NULL);

 RA_SetTestTrigPad(plotFFTTime->numPicture, 0, plotFFTTime->refresh_percent * plotFFTTime->timeSize/100.);

 if (!oldname)
    {if (plotFFTTime->fmax <= plotFFTTime->fmin) plotFFTTime->fmax = 0.5 / plotFFTTime->dt;
     RA_HSetYRange(plotFFTTime->name, plotFFTTime->fmin, plotFFTTime->fmax, plotFFTTime->dnu);
    }

 RA_HSetUnitsX(plotFFTTime->name, "sec");
 RA_HSetUnitsY(plotFFTTime->name, "Hz");

 if (plotFFTTime->zmin < DY_BADDATA) RA_HSetMinimum(plotFFTTime->name, plotFFTTime->zmin);
 if (plotFFTTime->zmax > 0)          RA_HSetMaximum(plotFFTTime->name, plotFFTTime->zmax);

 plotFFTTime->buff = (float *)calloc(plotFFTTime->nBin+1, sizeof(float));
 if (plotFFTTime->buff == NULL) printf("UHplotFTIni> plotFFTTime->buff calloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplotFTIni> plot=%s=%s  Doing a calloc for %ld times %ld floats = %ld bytes\n",
             plotFFTTime->name, plotFFTTime->title, plotFFTTime->nBinTime, plotFFTTime->nBin/2+1, plotFFTTime->nBinTime * (plotFFTTime->nBin/2+1));
    }

 plotFFTTime->buffTot = (float *) calloc(plotFFTTime->nBinTime*(plotFFTTime->nBin/2+1), sizeof(float));
 if (plotFFTTime->buffTot == NULL) printf("UHplotFTIni> plotFFTTime->buffTot calloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotFTIni> plot=%s=%s=pad%d  plotOpt=%s \n",
             plotFFTTime->name, plotFFTTime->title, plotFFTTime->numPicture, plotFFTTime->plotOpt);
    }

 return;
}


/*---------------------------------------------------- UHplotFTIniFrame -----*/

void UHplotFTIniFrame (struct UHplotFFTTime *plotFFTTime)
{
 long i;
 char* oldname;
 double offset, framedt;
 struct FrAdcData *adc;
 struct FrVect *vect;
 char *name1, *temp;
 long length;

 if (plotFFTTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (UFrCurFrameH != NULL) plotFFTTime->iFirst = OFF;

 temp   = strstr(plotFFTTime->title,SEPTITLE);
 length = strlen(plotFFTTime->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plotFFTTime->title, length);
 if (UFrCurFrameH)
    {adc  = FrAdcDataFind(UFrCurFrameH, name1);
     if (adc == NULL) adc = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(UFrCurFrameH, name1);
     if (vect == NULL) vect = FrameFindVect(Dy->Frame, name1);
    }

 if (adc == NULL  &&  vect == NULL)
    {adc  = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(Dy->Frame, name1);
    }
 free(name1);

 plotFFTTime->slope = 1.;
 plotFFTTime->bias  = 0.;

 if (adc != NULL && Dy->InputType != INDD)
    {if (adc->slope != 0.) plotFFTTime->slope = adc->slope;
     plotFFTTime->bias  = adc->bias;
    }

 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     if (UFrCurFrameH != NULL)
        {framedt = DyGetFramedt(UFrCurFrameH);
         offset = ((double)Dy->timeOrigin - (double)UFrCurFrameH->GTimeS) / framedt;
         offset = offset - (int)offset;
         if (offset*UFrCurFrameH->dt < 1.) offset = 0.;
        }
     else                     
        {printf("UHplotFTIniFrame> UFrCurFrameH=NULL. Forced offset=0.\n");
         offset = 0.;
        }

     plotFFTTime->timeOffset = offset * framedt * plotFFTTime->resampFactor;

     if (offset > 0.)
        {if (vect) plotFFTTime->t0 = (int)(offset*UFrCurFrameH->dt / vect->dx[0]);
         else      plotFFTTime->t0 = (int)(offset*UFrCurFrameH->dt);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
            {printf("UHplotFTIniFrame> startTime=%d  GTimeS=%d  framedt=%f  offset=%f  t0=%ld  plotFFTTime->timeOffset=%f  ",
                          Dy->timeOrigin, UFrCurFrameH->GTimeS, UFrCurFrameH->dt, offset, plotFFTTime->t0, plotFFTTime->timeOffset);
             if (vect) printf("vect=%s:dx[0]=%f \n", vect->name, vect->dx[0]);
             else      printf("vect=NULL\n");
            }
        }
    }

 plotFFTTime->tmin = plotFFTTime->dt * (double)plotFFTTime->t0;

 RA_HSetUnitsZ(plotFFTTime->name, "a.u.");
 if (vect != NULL)
    {
     if (plotFFTTime->realUnits > 0)
        {RA_HSetUnitsZ(plotFFTTime->name, vect->unitY);}
     if (plotFFTTime->realUnits == 0 && plotFFTTime->slope == 1.  &&  plotFFTTime->bias == 0.)
        {RA_HSetUnitsZ(plotFFTTime->name, vect->unitY);}
    }
 if (adc != NULL)
    {
     if (plotFFTTime->realUnits > 0)
        {RA_HSetUnitsZ(plotFFTTime->name, adc->units);}
     if (plotFFTTime->realUnits == 0 && plotFFTTime->slope == 1.  &&  plotFFTTime->bias == 0.)
        {RA_HSetUnitsZ(plotFFTTime->name, adc->units);}
    }
 
 return;
}


/*----------------------------------------------------- UHplotFTRead --*/
void UHplotFTRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplotFFTTime *plotFFTTime;
 long i;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplotFFTTime, plotFFTTime);
 
 UJ_READ_LONG(plotFFTTime->iClock)
 UJ_READ_LONG(plotFFTTime->id)
 if (!plotFFTTime->name) plotFFTTime->name = (char *) calloc(10,sizeof(char));
 sprintf(plotFFTTime->name,"h%ld",plotFFTTime->id);
 UJ_READ_S_CHAR(plotFFTTime->title)
 UJ_READ_LONG(plotFFTTime->nBinP2)
 UJ_READ_DOUBLE(plotFFTTime->timeSize)
 UJ_READ_LONG(plotFFTTime->resampFactor)
 UJ_READ_LONG(plotFFTTime->t0)
 UJ_READ_STRUCT(USignal, plotFFTTime->input)
 UJ_READ_S_CHAR(plotFFTTime->options)

 i = 1; 
 plotFFTTime->nBin = (i << plotFFTTime->nBinP2);
 if (plotFFTTime->nBin <= 0) plotFFTTime->nBin = 2;
   
 UJCObAdd(plotFFTTime->iClock,plotFFTTime,UHplotFFTTimeUJstructH,NULL);

 plotFFTTime->hidden = OFF;
 s = strstr(plotFFTTime->options,"HIDDEN");
 if (s != NULL)  plotFFTTime->hidden = ON;

 plotFFTTime->realUnits = 0;
 s = strstr(plotFFTTime->options,"REALUNITS");
 if (s != NULL)  plotFFTTime->realUnits = 1;

 plotFFTTime->maxAverage = 1;
 s = strstr(plotFFTTime->options,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plotFFTTime->maxAverage);

 plotFFTTime->fmin = 0.;
 s = strstr(plotFFTTime->options,"FMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFTTime->fmin);

 plotFFTTime->fmax = 0.;
 s = strstr(plotFFTTime->options,"FMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFTTime->fmax);

 plotFFTTime->nodc = 0;
 s = strstr(plotFFTTime->options,"NODC");
 if (s != NULL)  plotFFTTime->nodc = 1;

 plotFFTTime->autoScale = 0;
 s = strstr(plotFFTTime->options,"AUTOSCALE");
 if (s != NULL)  plotFFTTime->autoScale = 1;

 plotFFTTime->zmin = DY_BADDATA;
 plotFFTTime->zmax = 0.;
 s = strstr(plotFFTTime->options,"ZMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFTTime->zmin);
 s = strstr(plotFFTTime->options,"ZMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFTTime->zmax);

 plotFFTTime->refresh_percent = 0.;
 s = strstr(plotFFTTime->options,"REFRESH_PERCENT");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotFFTTime->refresh_percent);

 return;
}
 

/*--------------------------------------------------- UHplotFTSim -----*/
void UHplotFTSim(plotFFTTime,time)
struct UHplotFFTTime *plotFFTTime;
long time;
/*--------------------------------------------------------------------*/
{
 long  i, j, inter;
 double *tab, *fft, coef, value, dc;
 char *name1, *temp;
 long length;
 double origin;
 struct FrAdcData *adc;
 int fractime, nx, status;
 char title[256], stringtime[40];
 time_t starttimeutc;

 if (plotFFTTime->hidden == ON) return;

 if (Dy->Order == STOP) return;

 if (time < plotFFTTime->t0) return;

 if (plotFFTTime->iFirst == ON) UHplotFTIniFrame(plotFFTTime);

 /*--- Take the input value ---*/

 if (plotFFTTime->input == NULL) 
    {UFrUnpackSim(plotFFTTime->FrUnpack, 0);
     value = plotFFTTime->FrUnpack->out->value;
    }
 else
    {value = plotFFTTime->input->value;}

 if (plotFFTTime->realUnits != 0  &&  value < DY_BADDATA) 
    {value = value * plotFFTTime->slope + plotFFTTime->bias;}


 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reach resampFactor ---*/

 if (value < DY_BADDATA)         plotFFTTime->sumvalue += value;
 else if (value >= DY_NOCHANNEL) plotFFTTime->sumvalue  = DY_NOCHANNEL * 1.01 * plotFFTTime->resampFactor;
 else if (value >= DY_NOFRAME)   plotFFTTime->sumvalue  = DY_NOFRAME   * 1.01 * plotFFTTime->resampFactor;
 else                            plotFFTTime->sumvalue  = DY_BADDATA   * 1.01 * plotFFTTime->resampFactor;

 plotFFTTime->j++;

 if (plotFFTTime->j < plotFFTTime->resampFactor) return;


 /*---------- fill data and plot -----------------*/

 plotFFTTime->buff[plotFFTTime->i] = plotFFTTime->sumvalue / plotFFTTime->resampFactor;
 plotFFTTime->i++;
 plotFFTTime->sumvalue = 0.;
 plotFFTTime->j = 0;

 /*---- do FFT and shift data if we reach the end of the plot --------*/

 if (plotFFTTime->i == plotFFTTime->nBin)
    {
     plotFFTTime->nAverage++;

     dc = 0.;
     if (plotFFTTime->nodc > 0)
        {
         for (j=0 ;j<plotFFTTime->nBin; j++)  dc += plotFFTTime->buff[j];
         if (plotFFTTime->nBin > 0)  dc = dc / (double)plotFFTTime->nBin;
        }

     tab = plotFFTTime->vect->dataD;
     for (i=0; i< plotFFTTime->nBin; i++)
        {
         inter = 2*i;
         tab[inter]   = (plotFFTTime->buff[i]-dc) * plotFFTTime->window->dataD[i];
         tab[inter+1] = 0.;
        }

     UACDfft(plotFFTTime->vect, plotFFTTime->nBinP2);

     fft = plotFFTTime->fft->dataD;
     for (j=0 ;j<plotFFTTime->nBin/2+1 ;j++)
          {fft[j] = fft[j] + tab[2*j]*tab[2*j] + tab[2*j+1]*tab[2*j+1];}

     if (plotFFTTime->nAverage == plotFFTTime->maxAverage)
        {  
         coef = plotFFTTime->norm / plotFFTTime->nAverage;

         nx = plotFFTTime->nBinTime;
         for (j=0 ;j<plotFFTTime->nBin/2+1 ;j++)
            {plotFFTTime->buffTot[plotFFTTime->itime+nx*j] = sqrt(plotFFTTime->fft->dataD[j]*coef);
             if (plotFFTTime->autoScale > 0 && plotFFTTime->zmin > plotFFTTime->buffTot[plotFFTTime->itime+nx*j]) plotFFTTime->zmin = plotFFTTime->buffTot[plotFFTTime->itime+nx*j];
            }

         RA_H2Pack(plotFFTTime->name, plotFFTTime->buffTot);
         plotFFTTime->itimeView++;

         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("plotFTSim> itimeView=%d  refresh_nBin=%d intervalle=%d  buffTot[1]=%g  nBin=%d i=%d itime=%d nBinTime=%d nAv=%d maxAv=%d\n",
                     plotFFTTime->itimeView, plotFFTTime->refresh_nBin, plotFFTTime->intervalle, plotFFTTime->buffTot[1],
                     plotFFTTime->nBin, plotFFTTime->i, plotFFTTime->itime, plotFFTTime->nBinTime, plotFFTTime->nAverage, plotFFTTime->maxAverage);
            }

         if (Dy->plotsNotDisplayed == OFF && plotFFTTime->itimeView%plotFFTTime->refresh_nBin == 0)
            {
             DyDebugPrint(__FUNCTION__,0);
             DyDebugTimeNeeded(__FUNCTION__,"start",1);

             if (plotFFTTime->autoScale > 0 && plotFFTTime->zmin > 0.) RA_HSetMinimum(plotFFTTime->name, plotFFTTime->zmin);

             /*--- Set time string and time axis offset ---*/
             origin = Dy->timeOrigin + plotFFTTime->tmin - plotFFTTime->timeOffset;
             UHplotSetTime (origin, plotFFTTime->name, plotFFTTime->nBin*plotFFTTime->dt, plotFFTTime->nAverage, ON, UFrCurFrameH);

             /*--- Draw the plot ---*/
             UHplot(plotFFTTime->name, plotFFTTime->numPicture, plotFFTTime->options, plotFFTTime->plotOpt, 0, 0);

             /*--- Update plot environment ---*/
             RA_SetActivePad(plotFFTTime->numPicture);
             UHplotUpdateEnv (plotFFTTime->numPicture, plotFFTTime->name, plotFFTTime->options, plotFFTTime->name,
                              plotFFTTime->numPicture, 0, plotFFTTime->title,
                              0, plotFFTTime->buff, plotFFTTime->nBin, &plotFFTTime->corrupt);

             RA_HUpdate(plotFFTTime->numPicture, plotFFTTime->name, plotFFTTime->title, plotFFTTime->options);
             DyDebugTimeNeeded(__FUNCTION__,"stop",1);
            }

         DyCheckRunning();    /* interaction with the data Display */
        }
    }

 /*--- shift buffer used for FFT ---*/

 if (plotFFTTime->i == plotFFTTime->nBin)
    {
     for (j=0; j<plotFFTTime->nBin-plotFFTTime->dNbin; j++)
        {plotFFTTime->buff[j] = plotFFTTime->buff[j+plotFFTTime->dNbin];}
     plotFFTTime->i = plotFFTTime->i - plotFFTTime->dNbin;

     if (plotFFTTime->nAverage == plotFFTTime->maxAverage)
        {plotFFTTime->itime++;
         plotFFTTime->nAverage = 0;
         for (j=0 ;j<plotFFTTime->nBin/2+1 ;j++) plotFFTTime->fft->dataD[j] = 0.;
        }
    }


 /*--- Update buffers ---*/

 if (plotFFTTime->itime == plotFFTTime->nBinTime)
    {
     plotFFTTime->itime--;

     RA_H2Unpack(plotFFTTime->name, plotFFTTime->buffTot);

     /*--- Shift the buffer content ---*/

     nx = plotFFTTime->nBinTime;
     for (i=0; i<plotFFTTime->nBinTime-1; i++)
        {for (j=0; j<plotFFTTime->nBin/2+1; j++)
            {plotFFTTime->buffTot[i+j*nx] = plotFFTTime->buffTot[i+1+j*nx];}
        }
     for (j=0; j<plotFFTTime->nBin/2+1; j++)  plotFFTTime->buffTot[nx-1+nx*j] = 0.;

     plotFFTTime->tmin += plotFFTTime->dt * plotFFTTime->dNbin * plotFFTTime->resampFactor * plotFFTTime->maxAverage;
    }

 if (UFrCurFrameH)
    {
     if ((plotFFTTime->previousGPS > 0) && ((double)(UFrCurFrameH->GTimeS-plotFFTTime->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {plotFFTTime->tmin += (double)(UFrCurFrameH->GTimeS - plotFFTTime->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) plotFFTTime->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*============================================== PLOT COHE_TIME ======================================*/

/*---------------------------------------------------- UHpCoheTEnd -----*/
void UHpCoheTEnd(pCoheTime)
struct UHpCoheTime *pCoheTime;
/*---------------------------------------------------------------------*/
{
 if (pCoheTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(pCoheTime->numPicture);
 RA_HUpdateEnd(pCoheTime->name,pCoheTime->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpCoheEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  nAverage=%d  plotOpt=%s\n",
            pCoheTime->name, pCoheTime->title, pCoheTime->numPicture, Dy->nready, Dy->nsame, Dy->nplot, pCoheTime->nAverage, pCoheTime->plotOpt);
    }

 if (pCoheTime->title   != NULL)  free(pCoheTime->title);
 if (pCoheTime->window  != NULL)  UADFree(pCoheTime->window);
 if (pCoheTime->options != NULL)  free(pCoheTime->options);
 if (pCoheTime->name    != NULL)  free(pCoheTime->name);
 if (pCoheTime->buffTot != NULL)  free(pCoheTime->buffTot);

 if (pCoheTime->vect1   != NULL)  UACDFree(pCoheTime->vect1);
 if (pCoheTime->vect2   != NULL)  UACDFree(pCoheTime->vect2);
 if (pCoheTime->module  != NULL)  UADFree(pCoheTime->module);
 if (pCoheTime->buff1   !=NULL)   free(pCoheTime->buff1);
 if (pCoheTime->buff2   !=NULL)   free(pCoheTime->buff2);

 free(pCoheTime);

 return;
}


/*---------------------------------------------------- UHpCoheTIni -----*/
void UHpCoheTIni(pCoheTime,dt)
struct UHpCoheTime *pCoheTime;
double dt;
/*--------------------------------------------------------------------*/
{
 long i;
 char* oldname;

 if (pCoheTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (pCoheTime->resampFactor <= 0) pCoheTime->resampFactor = 1;

 pCoheTime->dt           = dt * (double)pCoheTime->resampFactor;
 pCoheTime->dnu          = 1./(pCoheTime->nBin*pCoheTime->dt);
 pCoheTime->i            = 0;
 pCoheTime->j            = 0;
 pCoheTime->itime        = 0;
 pCoheTime->itimeView    = 0;
 pCoheTime->dNbin        = pCoheTime->nBin/10;
 pCoheTime->nAverage     = 0;
 pCoheTime->previousGPS  = 0;
 pCoheTime->t0           = 0;
 pCoheTime->timeOffset   = 0.;
 pCoheTime->refresh_nBin = 1;
 pCoheTime->iFirst       = ON;

 pCoheTime->vect1    = UACDNew(1,pCoheTime->nBin);
 pCoheTime->vect2    = UACDNew(1,pCoheTime->nBin);
 pCoheTime->module   = UADNew(1,pCoheTime->nBin/2+1);
 pCoheTime->phase    = UADNew(1,pCoheTime->nBin/2+1);
 pCoheTime->rer      = UADNew(1,pCoheTime->nBin/2+1);
 pCoheTime->imr      = UADNew(1,pCoheTime->nBin/2+1);
 pCoheTime->mod1sq   = UADNew(1,pCoheTime->nBin/2+1);
 pCoheTime->mod2sq   = UADNew(1,pCoheTime->nBin/2+1);

 bzero(pCoheTime->vect1->data, pCoheTime->vect1->nBytes);
 bzero(pCoheTime->vect2->data, pCoheTime->vect2->nBytes);
 bzero(pCoheTime->module->data, pCoheTime->module->nBytes);
 bzero(pCoheTime->phase->data, pCoheTime->phase->nBytes);
 bzero(pCoheTime->rer->data, pCoheTime->rer->nBytes);
 bzero(pCoheTime->imr->data, pCoheTime->imr->nBytes);
 bzero(pCoheTime->mod1sq->data, pCoheTime->mod1sq->nBytes);
 bzero(pCoheTime->mod2sq->data, pCoheTime->mod2sq->nBytes);

 /*--- Define the Hanning window ---*/

 pCoheTime->window  = UADNew(1,pCoheTime->nBin);
 bzero(pCoheTime->window->data,pCoheTime->window->nBytes);
  
 for (i=0; i<pCoheTime->nBin; i++)
     {pCoheTime->window->dataD[i] = 0.5 * (1. + cos( TWOPI * 
                                  ((double)i - ((double)pCoheTime->nBin - 1.)/2.)/
                                  ((double)pCoheTime->nBin - 1.)));}
 pCoheTime->norm = 0.;
 for (i=0; i < pCoheTime->nBin; i++)
     {pCoheTime->norm += pCoheTime->window->dataD[i]*pCoheTime->window->dataD[i];}
 pCoheTime->norm = pCoheTime->dt / pCoheTime->norm; 
 pCoheTime->norm = pCoheTime->norm * sqrt(2.); 

 /*---- decode the options string --------*/

 UHoptDecode(pCoheTime->options, &pCoheTime->intervalle, pCoheTime->plotOpt);

 if (pCoheTime->intervalle <= 0) pCoheTime->intervalle = pCoheTime->nBin;

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(pCoheTime->options,"SAME") == NULL) pCoheTime->numPicture = DyGrafIni(0);
     else                                           pCoheTime->numPicture = 0;
     pCoheTime->dNbin = pCoheTime->intervalle;
    }

/*---  Define the 2D plot used for time/freq and set limits on frequency axis  ---*/

 pCoheTime->nBinTime = (long)(pCoheTime->timeSize
                         / (pCoheTime->dNbin * pCoheTime->maxAverage * pCoheTime->dt));

 if (pCoheTime->refresh_percent > 0)
    {
     pCoheTime->refresh_nBin = (long)((pCoheTime->refresh_percent * pCoheTime->timeSize/100.) 
                                 / (pCoheTime->dNbin * pCoheTime->maxAverage * pCoheTime->dt));
     if (pCoheTime->refresh_nBin <= 0) pCoheTime->refresh_nBin = 1;
    }

 if (pCoheTime->intervalle < pCoheTime->maxAverage * pCoheTime->dNbin)
    {pCoheTime->intervalle = pCoheTime->maxAverage * pCoheTime->dNbin;}

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHpCoheTIni> pCoheTime=%s  timeSize=%g  dNbin=%ld  dt=%g  nBinTime=%ld\n",
             pCoheTime->title, pCoheTime->timeSize, pCoheTime->dNbin, pCoheTime->dt, pCoheTime->nBinTime);
    }

/*---  Get the limits of the previous histogram in the pad if it is good  ---*/ 
 oldname = RA_PadContainsGoodHist(pCoheTime->numPicture, 0, 2, pCoheTime->nBinTime, pCoheTime->nBin/2+1, pCoheTime->name);
 if (!oldname)
   {RA_PadCompleteClear(pCoheTime->numPicture);
    UHoptPlopt(pCoheTime->options, pCoheTime->numPicture, 0);
   }

 RA_H2NewReuse(pCoheTime->numPicture, 0,
               pCoheTime->name, pCoheTime->title,
               pCoheTime->nBinTime,
               pCoheTime->maxAverage * pCoheTime->dNbin * pCoheTime->dt * (-0.5),
               pCoheTime->maxAverage * pCoheTime->dNbin * pCoheTime->dt * ((double)pCoheTime->nBinTime - 0.5),
               pCoheTime->nBin/2+1,
               pCoheTime->dnu*(-0.5),
               pCoheTime->dnu*((double)(pCoheTime->nBin/2+1) - 0.5));

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, pCoheTime->name);

 RA_HSetStat(pCoheTime->name, NULL);

 RA_SetTestTrigPad(pCoheTime->numPicture, 0, pCoheTime->refresh_percent * pCoheTime->timeSize/100.);

 if (!oldname)
    {if (pCoheTime->fmax <= pCoheTime->fmin) pCoheTime->fmax = 0.5 / pCoheTime->dt;
     RA_HSetYRange(pCoheTime->name, pCoheTime->fmin, pCoheTime->fmax, pCoheTime->dnu);
    }

 RA_HSetUnitsX(pCoheTime->name, "sec");
 RA_HSetUnitsY(pCoheTime->name, "Hz");

 if (pCoheTime->zmin < DY_BADDATA) RA_HSetMinimum(pCoheTime->name, pCoheTime->zmin);
 if (pCoheTime->zmax > 0.)         RA_HSetMaximum(pCoheTime->name, pCoheTime->zmax);

 pCoheTime->buff1 = (float *)calloc(pCoheTime->nBin+1, sizeof(float));
 if (pCoheTime->buff1 == NULL) printf("UHpCoheTIni> pCoheTime->buff1 malloc failed");
 pCoheTime->buff2 = (float *)calloc(pCoheTime->nBin+1, sizeof(float));
 if (pCoheTime->buff2 == NULL) printf("UHpCoheTIni> pCoheTime->buff2 malloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHpCoheTIni> plot=%s=%s  Doing a calloc for %ld times %ld floats = %ld bytes\n",
             pCoheTime->name, pCoheTime->title, pCoheTime->nBinTime, pCoheTime->nBin/2+1, pCoheTime->nBinTime * (pCoheTime->nBin/2+1));
    }

 pCoheTime->buffTot = (float *) calloc(pCoheTime->nBinTime*(pCoheTime->nBin/2+1), sizeof(float));
 if (pCoheTime->buffTot == NULL) printf("UHpCoheTIni> pCoheTime->buffTot calloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpCoheTIni> plot=%s=%s=pad%d  plotOpt=%s \n",
             pCoheTime->name, pCoheTime->title, pCoheTime->numPicture, pCoheTime->plotOpt);
    }

 return;
}


/*---------------------------------------------------- UHpCoheTIniFrame -----*/

void UHpCoheTIniFrame (struct UHpCoheTime *pCoheTime)
{
 long i;
 double offset, framedt;
 char *name1, *name2, *temp, *oldname;
 long length, length2;
 struct FrAdcData *adc1=NULL;
 struct FrAdcData *adc2=NULL;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;

 if (pCoheTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (UFrCurFrameH != NULL) pCoheTime->iFirst = OFF;

 temp   = strstr(pCoheTime->title,SEPTITLE);
 length = strlen(pCoheTime->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, pCoheTime->title, length);
 temp   = strstr(name1,SEP2VAR);
 length2 = strlen(name1)-strlen(temp);
 name2 = (char *) calloc(length2+1,sizeof(char));
 strncpy (name2, name1, length2);
 strcpy (name1, temp+strlen(SEP2VAR));

 if (UFrCurFrameH)
    {adc1  = FrAdcDataFind(UFrCurFrameH, name1);
     adc2  = FrAdcDataFind(UFrCurFrameH, name2);
     vect1 = FrameFindVect(UFrCurFrameH, name1);
     vect2 = FrameFindVect(UFrCurFrameH, name2);
    }

 if (adc1 == NULL  &&  vect1 == NULL)
    {adc1  = FrAdcDataFind(Dy->Frame, name1);
     vect1 = FrameFindVect(Dy->Frame, name1);
    }

 if (adc2 == NULL  &&  vect2 == NULL)
    {adc2  = FrAdcDataFind(Dy->Frame, name2);
     vect2 = FrameFindVect(Dy->Frame, name2);
    }

 free(name1);
 free(name2);

 pCoheTime->slope1 = 1.;
 pCoheTime->bias1  = 0.;
 pCoheTime->slope2 = 1.;
 pCoheTime->bias2  = 0.;

 if (adc1 != NULL)
    {if (adc1->slope != 0.) pCoheTime->slope1 = adc1->slope;
     pCoheTime->bias1  = adc1->bias;
    }
 if (adc2 != NULL)
    {if (adc2->slope != 0.) pCoheTime->slope2 = adc2->slope;
     pCoheTime->bias2  = adc2->bias;
    }

 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     if (UFrCurFrameH != NULL)
        {framedt = DyGetFramedt(UFrCurFrameH);
         offset = ((double)Dy->timeOrigin - (double)UFrCurFrameH->GTimeS) / framedt;
         offset = offset - (int)offset;
         if (offset*UFrCurFrameH->dt < 1.) offset = 0.;
        }
     else                     
        {printf("UHpCoheTIniFrame> UFrCurFrameH=NULL. Forced offset=0.\n");
         offset = 0.;
        }

     pCoheTime->timeOffset = offset * framedt * pCoheTime->resampFactor;

     if (offset > 0.)
        {if (vect1) pCoheTime->t0 = (int)(offset*UFrCurFrameH->dt / vect1->dx[0]);
         else      pCoheTime->t0 = (int)(offset*UFrCurFrameH->dt);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
            {printf("UHpCoheTIniFrame> startTime=%d  GTimeS=%d  framedt=%f  offset=%f  t0=%ld  pCoheTime->timeOffset=%f  ",
                          Dy->timeOrigin, UFrCurFrameH->GTimeS, UFrCurFrameH->dt, offset, pCoheTime->t0, pCoheTime->timeOffset);
             if (vect1) printf("UHpCoheTIniFrame> vect1=%s:dx[0]=%f \n", vect1->name, vect1->dx[0]);
             else      printf("UHpCoheTIniFrame> vect1=NULL\n");
            }
        }
    }

 pCoheTime->tmin = pCoheTime->dt * (double)pCoheTime->t0;

 RA_HSetUnitsZ(pCoheTime->name, "a.u.");
 
 return;
}


/*----------------------------------------------------- UHpCoheTRead --*/
void UHpCoheTRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHpCoheTime *pCoheTime;
 long i;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHpCoheTime, pCoheTime);
 
 UJ_READ_LONG(pCoheTime->iClock)
 UJ_READ_LONG(pCoheTime->id)
 if (!pCoheTime->name) pCoheTime->name = (char *) calloc(10,sizeof(char));
 sprintf(pCoheTime->name,"h%ld",pCoheTime->id);
 UJ_READ_S_CHAR(pCoheTime->title)
 UJ_READ_LONG(pCoheTime->nBinP2)
 UJ_READ_DOUBLE(pCoheTime->timeSize)
 UJ_READ_LONG(pCoheTime->resampFactor)
 UJ_READ_LONG(pCoheTime->t0)
 UJ_READ_STRUCT(USignal,pCoheTime->input1)
 UJ_READ_STRUCT(USignal,pCoheTime->input2)
 UJ_READ_S_CHAR(pCoheTime->options)

 i = 1; 
 pCoheTime->nBin = (i << pCoheTime->nBinP2);
 if (pCoheTime->nBin <= 0) pCoheTime->nBin = 2;
   
 UJCObAdd(pCoheTime->iClock,pCoheTime,UHpCoheTimeUJstructH,NULL);

 pCoheTime->hidden = OFF;
 s = strstr(pCoheTime->options,"HIDDEN");
 if (s != NULL)  pCoheTime->hidden = ON;

 pCoheTime->realUnits = 0;
 s = strstr(pCoheTime->options,"REALUNITS");
 if (s != NULL)  pCoheTime->realUnits = 1;

 pCoheTime->maxAverage = 1;
 s = strstr(pCoheTime->options,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&pCoheTime->maxAverage);

 pCoheTime->fmin = 0.;
 s = strstr(pCoheTime->options,"FMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCoheTime->fmin);

 pCoheTime->fmax = 0.;
 s = strstr(pCoheTime->options,"FMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCoheTime->fmax);

 pCoheTime->nodc = 0;
 s = strstr(pCoheTime->options,"NODC");
 if (s != NULL)  pCoheTime->nodc = 1;

 pCoheTime->zmin = DY_BADDATA;
 pCoheTime->zmax = 0.;
 s = strstr(pCoheTime->options,"ZMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCoheTime->zmin);
 s = strstr(pCoheTime->options,"ZMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCoheTime->zmax);

 pCoheTime->refresh_percent = 0.;
 s = strstr(pCoheTime->options,"REFRESH_PERCENT");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pCoheTime->refresh_percent);

 return;
}
 

/*--------------------------------------------------- UHpCoheTSim -----*/
void UHpCoheTSim(pCoheTime,time)
struct UHpCoheTime *pCoheTime;
long time;
/*--------------------------------------------------------------------*/
{
 long  i, j, inter, inter2;
 double *tab1,*tab2,*module,*phase,value1,value2;
 double *mod1sq,*mod2sq,*rer,*imr;
 long length, origin;
 int fractime, nx, status;
 char title[256], stringtime[40];
 time_t starttimeutc;
 static float lastGoodData1=0.;
 static float lastGoodData2=0.;

 if (pCoheTime->hidden == ON) return;

 if (Dy->Order == STOP) return;

 if (time < pCoheTime->t0) return;

 if (pCoheTime->iFirst == ON) UHpCoheTIniFrame(pCoheTime);

 /*--- Take the input value ---*/

 if (pCoheTime->input1 == NULL) 
    {UFrUnpackSim(pCoheTime->FrUnpack1, 0);
     value1 = pCoheTime->FrUnpack1->out->value;
    }
 else
    {value1 = pCoheTime->input1->value;}

 if (pCoheTime->realUnits != 0  &&  value1 < DY_BADDATA) 
    {value1 = value1 * pCoheTime->slope1 + pCoheTime->bias1;}

 if (pCoheTime->input2 == NULL) 
    {UFrUnpackSim(pCoheTime->FrUnpack2, 0);
     value2 = pCoheTime->FrUnpack2->out->value;
    }
 else
    {value2 = pCoheTime->input2->value;}

 if (pCoheTime->realUnits != 0  &&  value2 < DY_BADDATA) 
    {value2 = value2 * pCoheTime->slope2 + pCoheTime->bias2;}


 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reach resampFactor ---*/

 if (value1 < DY_BADDATA)         pCoheTime->sumvalue1 += value1;
 else if (value1 >= DY_NOCHANNEL) pCoheTime->sumvalue1  = DY_NOCHANNEL * 1.01 * pCoheTime->resampFactor;
 else if (value1 >= DY_NOFRAME)   pCoheTime->sumvalue1  = DY_NOFRAME   * 1.01 * pCoheTime->resampFactor;
 else                             pCoheTime->sumvalue1  = DY_BADDATA   * 1.01 * pCoheTime->resampFactor;

 if (value2 < DY_BADDATA)         pCoheTime->sumvalue2 += value2;
 else if (value2 >= DY_NOCHANNEL) pCoheTime->sumvalue2  = DY_NOCHANNEL * 1.01 * pCoheTime->resampFactor;
 else if (value2 >= DY_NOFRAME)   pCoheTime->sumvalue2  = DY_NOFRAME   * 1.01 * pCoheTime->resampFactor;
 else                             pCoheTime->sumvalue2  = DY_BADDATA   * 1.01 * pCoheTime->resampFactor;

 pCoheTime->j++;

 if (pCoheTime->j < pCoheTime->resampFactor) return;


 /*---------- fill data and plot -----------------*/

 pCoheTime->buff1[pCoheTime->i] = pCoheTime->sumvalue1 / pCoheTime->resampFactor;
 pCoheTime->buff2[pCoheTime->i] = pCoheTime->sumvalue2 / pCoheTime->resampFactor;
 pCoheTime->i++;
 pCoheTime->sumvalue1 = 0.;
 pCoheTime->sumvalue2 = 0.;
 pCoheTime->j = 0;

 /*---- do FFT and shift data if we reach the end of the plot --------*/

 if (pCoheTime->i == pCoheTime->nBin)
    {
     pCoheTime->nAverage++;

     tab1 = pCoheTime->vect1->dataD;
     tab2 = pCoheTime->vect2->dataD;

     for (i=0; i<pCoheTime->nBin; i++)
        {
         if (pCoheTime->buff1[i] < DY_BADDATA)
            {inter = 2*i;
             tab1[inter]   = pCoheTime->buff1[i] * pCoheTime->window->dataD[i];
             tab1[inter+1] = 0.;
             lastGoodData1 = pCoheTime->buff1[i];
            }
         else
            {inter = 2*i;
             tab1[inter]   = lastGoodData1 * pCoheTime->window->dataD[i];
             tab1[inter+1] = 0.;
            }
        }

     for (i=0; i<pCoheTime->nBin; i++)
        {
         if (pCoheTime->buff2[i] < DY_BADDATA)
            {inter = 2*i;
             tab2[inter]   = pCoheTime->buff2[i] * pCoheTime->window->dataD[i];
             tab2[inter+1] = 0.;
             lastGoodData2 = pCoheTime->buff2[i];
            }
         else
            {inter = 2*i;
             tab2[inter]   = lastGoodData2 * pCoheTime->window->dataD[i];
             tab2[inter+1] = 0.;
            }
        }

     UACDfft(pCoheTime->vect1,pCoheTime->nBinP2);
     UACDfft(pCoheTime->vect2,pCoheTime->nBinP2);

     module = pCoheTime->module->dataD;
     phase  = pCoheTime->phase->dataD;
     rer    = pCoheTime->rer->dataD;
     imr    = pCoheTime->imr->dataD;
     mod1sq = pCoheTime->mod1sq->dataD;
     mod2sq = pCoheTime->mod2sq->dataD;

     for (j=0 ;j<pCoheTime->nBin/2+1 ;j++)
        {inter  = 2*j;
         inter2 = inter+1;
         mod1sq[j] += tab1[inter]*tab1[inter] + tab1[inter2]*tab1[inter2];
         mod2sq[j] += tab2[inter]*tab2[inter] + tab2[inter2]*tab2[inter2];
         rer[j]    += (tab2[inter]*tab1[inter] + tab2[inter2]*tab1[inter2]);
         imr[j]    += (-tab2[inter]*tab1[inter2] + tab2[inter2]*tab1[inter]);
         if (mod1sq[j] == 0. || mod2sq[j] == 0.)
            {module[j]  = 0.;}
         else
            {module[j]  = (rer[j]*rer[j]+imr[j]*imr[j])/(mod1sq[j]*mod2sq[j]);}
/**
         phase[j]   = atan2(imr[j],rer[j]);
**/
        }

    if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2) 
       {printf("plotCoheTSim>  module[1]=%g  module[%d/2]=%g module[%d]=%g  itime=%d nBinTime=%d nAv=%d maxAv=%d\n",
                module[1], pCoheTime->nBin/2, module[pCoheTime->nBin/4], pCoheTime->nBin/2, module[pCoheTime->nBin/2],
                pCoheTime->itime, pCoheTime->nBinTime, pCoheTime->nAverage, pCoheTime->maxAverage);
       }

     if (pCoheTime->nAverage == pCoheTime->maxAverage)
        {
         nx = pCoheTime->nBinTime;
         for (j=0 ;j<pCoheTime->nBin/2+1 ;j++)
            {pCoheTime->buffTot[pCoheTime->itime+nx*j] = module[j];
             if (pCoheTime->zmin > module[j]) pCoheTime->zmin = module[j];
            }

         RA_H2Pack(pCoheTime->name, pCoheTime->buffTot);
         pCoheTime->itimeView++;

         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("plotCoheTSim> itimeView=%d  refresh_nBin=%d intervalle=%d  buffTot[1]=%g module[1]=%g  nBin=%d i=%d itime=%d nBinTime=%d nAv=%d maxAv=%d\n",
                     pCoheTime->itimeView, pCoheTime->refresh_nBin, pCoheTime->intervalle, pCoheTime->buffTot[1], module[1],
                     pCoheTime->nBin, pCoheTime->i,  pCoheTime->itime, pCoheTime->nBinTime, pCoheTime->nAverage, pCoheTime->maxAverage);
            }

         if (Dy->plotsNotDisplayed == OFF && pCoheTime->itimeView%pCoheTime->refresh_nBin == 0)
            {
             DyDebugPrint(__FUNCTION__,0);
             DyDebugTimeNeeded(__FUNCTION__,"start",1);

             if (pCoheTime->zmin > 0.) RA_HSetMinimum(pCoheTime->name, pCoheTime->zmin);

             /*--- Set time string and time axis offset ---*/
             origin = Dy->timeOrigin + pCoheTime->tmin - pCoheTime->timeOffset;
             UHplotSetTime (origin, pCoheTime->name, pCoheTime->nBin*pCoheTime->dt, pCoheTime->nAverage, ON, UFrCurFrameH);

             /*--- Draw the plot ---*/
             UHplot(pCoheTime->name, pCoheTime->numPicture, pCoheTime->options, pCoheTime->plotOpt, 0, 0);

             /*--- Update plot environment ---*/
             RA_SetActivePad(pCoheTime->numPicture);
             UHplotUpdateEnv (pCoheTime->numPicture, pCoheTime->name, pCoheTime->options, pCoheTime->name,
                              pCoheTime->numPicture, 0, pCoheTime->title,
                              0, pCoheTime->buff1, pCoheTime->nBin, &pCoheTime->corrupt);
             UHplotUpdateEnv (pCoheTime->numPicture, pCoheTime->name, pCoheTime->options, pCoheTime->name,
                              pCoheTime->numPicture, 0, pCoheTime->title,
                              0, pCoheTime->buff2, pCoheTime->nBin, &pCoheTime->corrupt);

             RA_HUpdate(pCoheTime->numPicture, pCoheTime->name, pCoheTime->title, pCoheTime->options);
            }

         DyCheckRunning();    /* interaction with the data Display */
         DyDebugTimeNeeded(__FUNCTION__,"stop",1);
        }
    }

 /*--- shift buffer used for FFT ---*/

 if (pCoheTime->i == pCoheTime->nBin)
    {
     for (j=0; j<pCoheTime->nBin-pCoheTime->dNbin; j++)
        {pCoheTime->buff1[j] = pCoheTime->buff1[j+pCoheTime->dNbin];
         pCoheTime->buff2[j] = pCoheTime->buff2[j+pCoheTime->dNbin];
        }
     pCoheTime->i = pCoheTime->i - pCoheTime->dNbin;

     if (pCoheTime->nAverage == pCoheTime->maxAverage)
        {
         pCoheTime->itime++;
         pCoheTime->nAverage = 0;
         for (j=0 ;j<pCoheTime->nBin/2+1 ;j++)  pCoheTime->mod1sq->dataD[j] = 0.;
         for (j=0 ;j<pCoheTime->nBin/2+1 ;j++)  pCoheTime->mod2sq->dataD[j] = 0.;
         for (j=0 ;j<pCoheTime->nBin/2+1 ;j++)  pCoheTime->rer->dataD[j]    = 0.;
         for (j=0 ;j<pCoheTime->nBin/2+1 ;j++)  pCoheTime->imr->dataD[j]    = 0.;
        }
    }


 /*--- Update buffers ---*/

 if (pCoheTime->itime == pCoheTime->nBinTime)
    {
     pCoheTime->itime--;

     RA_H2Unpack(pCoheTime->name, pCoheTime->buffTot);

     /*--- Shift the buffer content ---*/

     nx = pCoheTime->nBinTime;
     for (i=0; i<pCoheTime->nBinTime-1; i++)
        {for (j=0; j<pCoheTime->nBin/2+1; j++)
            {pCoheTime->buffTot[i+j*nx] = pCoheTime->buffTot[i+1+j*nx];}
        }
     for (j=0; j<pCoheTime->nBin/2+1; j++)  pCoheTime->buffTot[nx-1+nx*j] = 0.;

     pCoheTime->tmin += pCoheTime->dt * pCoheTime->dNbin * pCoheTime->resampFactor * pCoheTime->maxAverage;
    }

 if (UFrCurFrameH)
    {
     if ((pCoheTime->previousGPS > 0) && ((double)(UFrCurFrameH->GTimeS-pCoheTime->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {pCoheTime->tmin += (double)(UFrCurFrameH->GTimeS - pCoheTime->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) pCoheTime->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*============================================= PLOT TRFCT_TIME ======================================*/


/*---------------------------------------------------- UHpTfctTEnd -----*/
void UHpTfctTEnd(pTfctTime)
struct UHpTfctTime *pTfctTime;
/*---------------------------------------------------------------------*/
{
 if (pTfctTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(pTfctTime->numPicture);
 RA_HUpdateEnd(pTfctTime->name,pTfctTime->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpTfctTime> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  nAverage=%d  plotOpt=%s\n",
             pTfctTime->name, pTfctTime->title, pTfctTime->numPicture, Dy->nready, Dy->nsame, Dy->nplot, pTfctTime->nAverage, pTfctTime->plotOpt);
    }

 if (pTfctTime->title   != NULL)  free(pTfctTime->title);
 if (pTfctTime->window  != NULL)  UADFree(pTfctTime->window);
 if (pTfctTime->options != NULL)  free(pTfctTime->options);
 if (pTfctTime->name    != NULL)  free(pTfctTime->name);
 if (pTfctTime->buffTot != NULL)  free(pTfctTime->buffTot);

 if (pTfctTime->vect1   != NULL)  UACDFree(pTfctTime->vect1);
 if (pTfctTime->vect2   != NULL)  UACDFree(pTfctTime->vect2);
 if (pTfctTime->module  != NULL)  UADFree(pTfctTime->module);
 if (pTfctTime->buff1   !=NULL)   free(pTfctTime->buff1);
 if (pTfctTime->buff2   !=NULL)   free(pTfctTime->buff2);

 free(pTfctTime);

 return;
}


/*---------------------------------------------------- UHpTfctTIni -----*/
void UHpTfctTIni(pTfctTime,dt)
struct UHpTfctTime *pTfctTime;
double dt;
/*--------------------------------------------------------------------*/
{
 long i;
 char* oldname;

 if (pTfctTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (pTfctTime->resampFactor <= 0) pTfctTime->resampFactor = 1;

 pTfctTime->dt           = dt * (double)pTfctTime->resampFactor;
 pTfctTime->dnu          = 1./(pTfctTime->nBin*pTfctTime->dt);
 pTfctTime->i            = 0;
 pTfctTime->j            = 0;
 pTfctTime->itime        = 0;
 pTfctTime->itimeView    = 0;
 pTfctTime->dNbin        = pTfctTime->nBin/10;
 pTfctTime->nAverage     = 0;
 pTfctTime->previousGPS  = 0;
 pTfctTime->t0           = 0;
 pTfctTime->timeOffset   = 0.;
 pTfctTime->refresh_nBin = 1;
 pTfctTime->iFirst       = ON;

 pTfctTime->vect1    = UACDNew(1,pTfctTime->nBin);
 pTfctTime->vect2    = UACDNew(1,pTfctTime->nBin);
 pTfctTime->module   = UADNew(1,pTfctTime->nBin/2+1);
 pTfctTime->phase    = UADNew(1,pTfctTime->nBin/2+1);
 pTfctTime->rer      = UADNew(1,pTfctTime->nBin/2+1);
 pTfctTime->imr      = UADNew(1,pTfctTime->nBin/2+1);
 pTfctTime->mod1sq   = UADNew(1,pTfctTime->nBin/2+1);

 bzero(pTfctTime->vect1->data, pTfctTime->vect1->nBytes);
 bzero(pTfctTime->vect2->data, pTfctTime->vect2->nBytes);
 bzero(pTfctTime->module->data, pTfctTime->module->nBytes);
 bzero(pTfctTime->phase->data, pTfctTime->phase->nBytes);
 bzero(pTfctTime->rer->data, pTfctTime->rer->nBytes);
 bzero(pTfctTime->imr->data, pTfctTime->imr->nBytes);
 bzero(pTfctTime->mod1sq->data, pTfctTime->mod1sq->nBytes);

 /*--- Define the Hanning window ---*/

 pTfctTime->window  = UADNew(1,pTfctTime->nBin);
 bzero(pTfctTime->window->data,pTfctTime->window->nBytes);
  
 for (i=0; i<pTfctTime->nBin; i++)
     {pTfctTime->window->dataD[i] = 0.5 * (1. + cos( TWOPI * 
                                  ((double)i - ((double)pTfctTime->nBin - 1.)/2.)/
                                  ((double)pTfctTime->nBin - 1.)));}
 pTfctTime->norm = 0.;
 for (i=0; i < pTfctTime->nBin; i++)
     {pTfctTime->norm += pTfctTime->window->dataD[i]*pTfctTime->window->dataD[i];}
 pTfctTime->norm = pTfctTime->dt / pTfctTime->norm; 
 pTfctTime->norm = pTfctTime->norm * sqrt(2.); 

 /*---- decode the options string --------*/

 UHoptDecode(pTfctTime->options, &pTfctTime->intervalle, pTfctTime->plotOpt);

 if (pTfctTime->intervalle <= 0) pTfctTime->intervalle = pTfctTime->nBin;

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(pTfctTime->options,"SAME") == NULL) pTfctTime->numPicture = DyGrafIni(0);
     else                                           pTfctTime->numPicture = 0;
     pTfctTime->dNbin = pTfctTime->intervalle;
    }

/*---  Define the 2D plot used for time/freq and set limits on frequency axis  ---*/

 pTfctTime->nBinTime = (long)(pTfctTime->timeSize
                         / (pTfctTime->dNbin * pTfctTime->maxAverage * pTfctTime->dt));

 if (pTfctTime->refresh_percent > 0)
    {
     pTfctTime->refresh_nBin = (long)((pTfctTime->refresh_percent * pTfctTime->timeSize/100.) 
                                 / (pTfctTime->dNbin * pTfctTime->maxAverage * pTfctTime->dt));
     if (pTfctTime->refresh_nBin <= 0) pTfctTime->refresh_nBin = 1;
    }

 if (pTfctTime->intervalle < pTfctTime->maxAverage * pTfctTime->dNbin)
    {pTfctTime->intervalle = pTfctTime->maxAverage * pTfctTime->dNbin;}

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHpTfctTIni> plot=%s=%s  timeSize=%g  dNbin=%ld  dt=%g  nBinTime=%ld\n",
             pTfctTime->name, pTfctTime->title, pTfctTime->timeSize, pTfctTime->dNbin, pTfctTime->dt, pTfctTime->nBinTime);
    }

/*---  Get the limits of the previous histogram in the pad if it is good  ---*/ 
 oldname = RA_PadContainsGoodHist(pTfctTime->numPicture, 0, 2, pTfctTime->nBinTime, pTfctTime->nBin/2+1, pTfctTime->name);
 if (!oldname)
   {RA_PadCompleteClear(pTfctTime->numPicture);
    UHoptPlopt(pTfctTime->options, pTfctTime->numPicture, 0);
   }

 RA_H2NewReuse(pTfctTime->numPicture, 0,
               pTfctTime->name, pTfctTime->title,
               pTfctTime->nBinTime,
               pTfctTime->maxAverage * pTfctTime->dNbin * pTfctTime->dt * (-0.5),
               pTfctTime->maxAverage * pTfctTime->dNbin * pTfctTime->dt * ((double)pTfctTime->nBinTime - 0.5),
               pTfctTime->nBin/2+1,
               pTfctTime->dnu*(-0.5),
               pTfctTime->dnu*((double)(pTfctTime->nBin/2+1) - 0.5));

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, pTfctTime->name);

 RA_HSetStat(pTfctTime->name, NULL);

 RA_SetTestTrigPad(pTfctTime->numPicture, 0, pTfctTime->refresh_percent * pTfctTime->timeSize/100.);

 if (!oldname)
    {if (pTfctTime->fmax <= pTfctTime->fmin) pTfctTime->fmax = 0.5 / pTfctTime->dt;
     RA_HSetYRange(pTfctTime->name, pTfctTime->fmin, pTfctTime->fmax, pTfctTime->dnu);
    }

 RA_HSetUnitsX(pTfctTime->name, "sec");
 RA_HSetUnitsY(pTfctTime->name, "Hz");

 if (pTfctTime->zmin < DY_BADDATA) RA_HSetMinimum(pTfctTime->name, pTfctTime->zmin);
 if (pTfctTime->zmax > 0.)         RA_HSetMaximum(pTfctTime->name, pTfctTime->zmax);

 pTfctTime->buff1 = (float *)calloc(pTfctTime->nBin+1, sizeof(float));
 if (pTfctTime->buff1 == NULL) printf("UHpTfctTIni> pTfctTime->buff1 malloc failed");
 pTfctTime->buff2 = (float *)calloc(pTfctTime->nBin+1, sizeof(float));
 if (pTfctTime->buff2 == NULL) printf("UHpTfctTIni> pTfctTime->buff2 malloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpTfctTIni> plot=%s=%s  Doing a calloc for %ld times %ld floats = %ld bytes\n",
             pTfctTime->name, pTfctTime->title, pTfctTime->nBinTime, pTfctTime->nBin/2+1, pTfctTime->nBinTime * (pTfctTime->nBin/2+1));
    }

 pTfctTime->buffTot = (float *) calloc(pTfctTime->nBinTime*(pTfctTime->nBin/2+1), sizeof(float));
 if (pTfctTime->buffTot == NULL) printf("UHpTfctTIni> pTfctTime->buffTot calloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHpTfctTIni> plot=%s=%s=pad%d  plotOpt=%s \n",
             pTfctTime->name, pTfctTime->title, pTfctTime->numPicture, pTfctTime->plotOpt);
    }

 return;
}


/*---------------------------------------------------- UHpTfctTIniFrame -----*/

void UHpTfctTIniFrame (struct UHpTfctTime *pTfctTime)
{
 long i;
 double offset, framedt;
 char *name1, *name2, *temp, *oldname, dummy[LSTRING_MAX];
 long length, length2;
 struct FrAdcData *adc1=NULL;
 struct FrAdcData *adc2=NULL;
 struct FrVect *vect1=NULL;
 struct FrVect *vect2=NULL;

 if (pTfctTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (UFrCurFrameH != NULL) pTfctTime->iFirst = OFF;

 temp   = strstr(pTfctTime->title,SEPTITLE);
 length = strlen(pTfctTime->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, pTfctTime->title, length);
 temp   = strstr(name1,SEP2VARTR);
 length2 = strlen(name1)-strlen(temp);
 name2 = (char *) calloc(length2+1,sizeof(char));
 strncpy (name2, name1, length2);
 strcpy (name1, temp+strlen(SEP2VARTR));

 if (UFrCurFrameH)
    {adc1  = FrAdcDataFind(UFrCurFrameH, name1);
     adc2  = FrAdcDataFind(UFrCurFrameH, name2);
     vect1 = FrameFindVect(UFrCurFrameH, name1);
     vect2 = FrameFindVect(UFrCurFrameH, name2);
    }

 if (adc1 == NULL  &&  vect1 == NULL)
    {adc1  = FrAdcDataFind(Dy->Frame, name1);
     vect1 = FrameFindVect(Dy->Frame, name1);
    }

 if (adc2 == NULL  &&  vect2 == NULL)
    {adc2  = FrAdcDataFind(Dy->Frame, name2);
     vect2 = FrameFindVect(Dy->Frame, name2);
    }

 free(name1);
 free(name2);

 pTfctTime->slope1 = 1.;
 pTfctTime->bias1  = 0.;
 pTfctTime->slope2 = 1.;
 pTfctTime->bias2  = 0.;

 if (adc1 != NULL)
    {if (adc1->slope != 0.) pTfctTime->slope1 = adc1->slope;
     pTfctTime->bias1  = adc1->bias;
    }
 if (adc2 != NULL)
    {if (adc2->slope != 0.) pTfctTime->slope2 = adc2->slope;
     pTfctTime->bias2  = adc2->bias;
    }

 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     if (UFrCurFrameH != NULL)
        {framedt = DyGetFramedt(UFrCurFrameH);
         offset = ((double)Dy->timeOrigin - (double)UFrCurFrameH->GTimeS) / framedt;
         offset = offset - (int)offset;
         if (offset*UFrCurFrameH->dt < 1.) offset = 0.;
        }
     else                     
        {printf("UHpTfctTIniFrame> UFrCurFrameH=NULL. Forced offset=0.\n");
         offset = 0.;
        }

     pTfctTime->timeOffset = offset * framedt * pTfctTime->resampFactor;

     if (offset > 0.)
        {if (vect1) pTfctTime->t0 = (int)(offset*UFrCurFrameH->dt / vect1->dx[0]);
         else      pTfctTime->t0 = (int)(offset*UFrCurFrameH->dt);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
            {printf("UHpTfctTIniFrame> startTime=%d  GTimeS=%d  framedt=%f  offset=%f  t0=%ld  pTfctTime->timeOffset=%f  ",
                          Dy->timeOrigin, UFrCurFrameH->GTimeS, UFrCurFrameH->dt, offset, pTfctTime->t0, pTfctTime->timeOffset);
             if (vect1) printf("UHpTfctTIniFrame> vect1=%s:dx[0]=%f \n", vect1->name, vect1->dx[0]);
             else      printf("UHpTfctTIniFrame> vect1=NULL\n");
            }
        }
    }

 pTfctTime->tmin = pTfctTime->dt * (double)pTfctTime->t0;

 if (vect1 != NULL  &&  vect2 != NULL)
    {
     sprintf(dummy,"%s/%s/sqrt(Hz)", vect1->unitY, vect2->unitY);
     if (pTfctTime->realUnits > 0)
        {RA_HSetUnitsZ(pTfctTime->name, dummy);}
     if (pTfctTime->realUnits == 0 && pTfctTime->slope1 == 1.  &&  pTfctTime->bias1 == 0.)
        {RA_HSetUnitsZ(pTfctTime->name, dummy);}
    }
 else if (adc1 != NULL  &&  adc2 != NULL)
    {
     sprintf(dummy,"%s/%s/sqrt(Hz)", adc1->units, adc2->units);
     if (pTfctTime->realUnits > 0)
        {RA_HSetUnitsZ(pTfctTime->name, dummy);}
     if (pTfctTime->realUnits == 0 && pTfctTime->slope1 == 1.  &&  pTfctTime->bias1 == 0.)
        {RA_HSetUnitsZ(pTfctTime->name, dummy);}
    }
 
 return;
}


/*----------------------------------------------------- UHpTfctTRead --*/
void UHpTfctTRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHpTfctTime *pTfctTime;
 long i;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHpTfctTime, pTfctTime);
 
 UJ_READ_LONG(pTfctTime->iClock)
 UJ_READ_LONG(pTfctTime->id)
 if (!pTfctTime->name) pTfctTime->name = (char *) calloc(10,sizeof(char));
 sprintf(pTfctTime->name,"h%ld",pTfctTime->id);
 UJ_READ_S_CHAR(pTfctTime->title)
 UJ_READ_LONG(pTfctTime->nBinP2)
 UJ_READ_DOUBLE(pTfctTime->timeSize)
 UJ_READ_LONG(pTfctTime->resampFactor)
 UJ_READ_LONG(pTfctTime->t0)
 UJ_READ_STRUCT(USignal,pTfctTime->input1)
 UJ_READ_STRUCT(USignal,pTfctTime->input2)
 UJ_READ_S_CHAR(pTfctTime->options)

 i = 1; 
 pTfctTime->nBin = (i << pTfctTime->nBinP2);
 if (pTfctTime->nBin <= 0) pTfctTime->nBin = 2;
   
 UJCObAdd(pTfctTime->iClock,pTfctTime,UHpTfctTimeUJstructH,NULL);

 pTfctTime->hidden = OFF;
 s = strstr(pTfctTime->options,"HIDDEN");
 if (s != NULL)  pTfctTime->hidden = ON;

 pTfctTime->realUnits = 0;
 s = strstr(pTfctTime->options,"REALUNITS");
 if (s != NULL)  pTfctTime->realUnits = 1;

 pTfctTime->maxAverage = 1;
 s = strstr(pTfctTime->options,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&pTfctTime->maxAverage);

 pTfctTime->fmin = 0.;
 s = strstr(pTfctTime->options,"FMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pTfctTime->fmin);

 pTfctTime->fmax = 0.;
 s = strstr(pTfctTime->options,"FMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pTfctTime->fmax);

 pTfctTime->nodc = 0;
 s = strstr(pTfctTime->options,"NODC");
 if (s != NULL)  pTfctTime->nodc = 1;

 pTfctTime->zmin = DY_BADDATA;
 pTfctTime->zmax = 0.;
 s = strstr(pTfctTime->options,"ZMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pTfctTime->zmin);
 s = strstr(pTfctTime->options,"ZMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pTfctTime->zmax);

 pTfctTime->refresh_percent = 0.;
 s = strstr(pTfctTime->options,"REFRESH_PERCENT");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&pTfctTime->refresh_percent);

 return;
}
 

/*--------------------------------------------------- UHpTfctTSim -----*/
void UHpTfctTSim(pTfctTime,time)
struct UHpTfctTime *pTfctTime;
long time;
/*--------------------------------------------------------------------*/
{
 long  i, j, inter, inter2;
 double *tab1,*tab2,*module,*phase,value1,value2;
 double *mod1sq,*rer,*imr;
 long length, origin;
 int fractime, nx, status;
 char title[256], stringtime[40];
 time_t starttimeutc;
 static float lastGoodData1=0.;
 static float lastGoodData2=0.;

 if (pTfctTime->hidden == ON) return;

 if (Dy->Order == STOP) return;

 if (time < pTfctTime->t0) return;

 if (pTfctTime->iFirst == ON) UHpTfctTIniFrame(pTfctTime);

 /*--- Take the input value ---*/

 if (pTfctTime->input1 == NULL) 
    {UFrUnpackSim(pTfctTime->FrUnpack1, 0);
     value1 = pTfctTime->FrUnpack1->out->value;
    }
 else
    {value1 = pTfctTime->input1->value;}

 if (pTfctTime->realUnits != 0  &&  value1 < DY_BADDATA) 
    {value1 = value1 * pTfctTime->slope1 + pTfctTime->bias1;}

 if (pTfctTime->input2 == NULL) 
    {UFrUnpackSim(pTfctTime->FrUnpack2, 0);
     value2 = pTfctTime->FrUnpack2->out->value;
    }
 else
    {value2 = pTfctTime->input2->value;}

 if (pTfctTime->realUnits != 0  &&  value2 < DY_BADDATA) 
    {value2 = value2 * pTfctTime->slope2 + pTfctTime->bias2;}


 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reach resampFactor ---*/

 if (value1 < DY_BADDATA)         pTfctTime->sumvalue1 += value1;
 else if (value1 >= DY_NOCHANNEL) pTfctTime->sumvalue1  = DY_NOCHANNEL * 1.01 * pTfctTime->resampFactor;
 else if (value1 >= DY_NOFRAME)   pTfctTime->sumvalue1  = DY_NOFRAME   * 1.01 * pTfctTime->resampFactor;
 else                             pTfctTime->sumvalue1  = DY_BADDATA   * 1.01 * pTfctTime->resampFactor;

 if (value2 < DY_BADDATA)         pTfctTime->sumvalue2 += value2;
 else if (value2 >= DY_NOCHANNEL) pTfctTime->sumvalue2  = DY_NOCHANNEL * 1.01 * pTfctTime->resampFactor;
 else if (value2 >= DY_NOFRAME)   pTfctTime->sumvalue2  = DY_NOFRAME   * 1.01 * pTfctTime->resampFactor;
 else                             pTfctTime->sumvalue2  = DY_BADDATA   * 1.01 * pTfctTime->resampFactor;

 pTfctTime->j++;

 if (pTfctTime->j < pTfctTime->resampFactor) return;


 /*---------- fill data and plot -----------------*/

 pTfctTime->buff1[pTfctTime->i] = pTfctTime->sumvalue1 / pTfctTime->resampFactor;
 pTfctTime->buff2[pTfctTime->i] = pTfctTime->sumvalue2 / pTfctTime->resampFactor;
 pTfctTime->i++;
 pTfctTime->sumvalue1 = 0.;
 pTfctTime->sumvalue2 = 0.;
 pTfctTime->j = 0;

 /*---- do FFT and shift data if we reach the end of the plot --------*/

 if (pTfctTime->i == pTfctTime->nBin)
    {
     pTfctTime->nAverage++;

     tab1 = pTfctTime->vect1->dataD;
     tab2 = pTfctTime->vect2->dataD;

     for (i=0; i<pTfctTime->nBin; i++)
        {
         if (pTfctTime->buff1[i] < DY_BADDATA)
            {inter = 2*i;
             tab1[inter]   = pTfctTime->buff1[i] * pTfctTime->window->dataD[i];
             tab1[inter+1] = 0.;
             lastGoodData1 = pTfctTime->buff1[i];
            }
         else
            {inter = 2*i;
             tab1[inter]   = lastGoodData1 * pTfctTime->window->dataD[i];
             tab1[inter+1] = 0.;
            }
        }

     for (i=0; i<pTfctTime->nBin; i++)
        {
         if (pTfctTime->buff2[i] < DY_BADDATA)
            {inter = 2*i;
             tab2[inter]   = pTfctTime->buff2[i] * pTfctTime->window->dataD[i];
             tab2[inter+1] = 0.;
             lastGoodData2 = pTfctTime->buff2[i];
            }
         else
            {inter = 2*i;
             tab2[inter]   = lastGoodData2 * pTfctTime->window->dataD[i];
             tab2[inter+1] = 0.;
            }
        }

     UACDfft(pTfctTime->vect1,pTfctTime->nBinP2);
     UACDfft(pTfctTime->vect2,pTfctTime->nBinP2);

     module = pTfctTime->module->dataD;
     phase  = pTfctTime->phase->dataD;
     rer    = pTfctTime->rer->dataD;
     imr    = pTfctTime->imr->dataD;
     mod1sq = pTfctTime->mod1sq->dataD;

     for (j=0 ;j<pTfctTime->nBin/2+1 ;j++)
        {inter  = 2*j;
         inter2 = inter+1;
         mod1sq[j] += tab1[inter]*tab1[inter] + tab1[inter2]*tab1[inter2];
         rer[j]    += (tab2[inter]*tab1[inter] + tab2[inter2]*tab1[inter2]);
         imr[j]    += (-tab2[inter]*tab1[inter2] + tab2[inter2]*tab1[inter]);
         if (mod1sq[j] == 0) module[j]  = 0.;
         else                module[j]  = sqrt(rer[j]*rer[j]+imr[j]*imr[j])/mod1sq[j];
/**
         phase[j]   = atan2(imr[j],rer[j]);
**/
        }

     if (pTfctTime->nAverage == pTfctTime->maxAverage)
        {
         nx = pTfctTime->nBinTime;
         for (j=0 ;j<pTfctTime->nBin/2+1 ;j++)
            {pTfctTime->buffTot[pTfctTime->itime+nx*j] = module[j];
             if (pTfctTime->zmin > module[j]) pTfctTime->zmin = module[j];
            }

         RA_H2Pack(pTfctTime->name, pTfctTime->buffTot);
         pTfctTime->itimeView++;

         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("plotFTSim> itimeView=%d  refresh_nBin=%d intervalle=%d  buffTot[1]=%g module[1]=%g  nBin=%d i=%d itime=%d nBinTime=%d nAv=%d maxAv=%d\n",
                     pTfctTime->itimeView, pTfctTime->refresh_nBin, pTfctTime->intervalle, pTfctTime->buffTot[1], module[1],
                     pTfctTime->nBin, pTfctTime->i,  pTfctTime->itime, pTfctTime->nBinTime, pTfctTime->nAverage, pTfctTime->maxAverage);
            }

         if (Dy->plotsNotDisplayed == OFF && pTfctTime->itimeView%pTfctTime->refresh_nBin == 0)
            {
             DyDebugPrint(__FUNCTION__,0);
             DyDebugTimeNeeded(__FUNCTION__,"start",1);

             if (pTfctTime->zmin > 0.) RA_HSetMinimum(pTfctTime->name, pTfctTime->zmin);

             /*--- Set time string and time axis offset ---*/
             origin = Dy->timeOrigin + pTfctTime->tmin - pTfctTime->timeOffset;
             UHplotSetTime (origin, pTfctTime->name, pTfctTime->nBin*pTfctTime->dt, pTfctTime->nAverage, ON,  UFrCurFrameH);

             /*--- Draw the plot ---*/
             UHplot(pTfctTime->name, pTfctTime->numPicture, pTfctTime->options, pTfctTime->plotOpt, 0, 0);

             /*--- Update plot environment ---*/
             RA_SetActivePad(pTfctTime->numPicture);
             UHplotUpdateEnv (pTfctTime->numPicture, pTfctTime->name, pTfctTime->options, pTfctTime->name,
                              pTfctTime->numPicture, 0, pTfctTime->title,
                              0, pTfctTime->buff1, pTfctTime->nBin, &pTfctTime->corrupt);
             UHplotUpdateEnv (pTfctTime->numPicture, pTfctTime->name, pTfctTime->options, pTfctTime->name,
                              pTfctTime->numPicture, 0, pTfctTime->title,
                              0, pTfctTime->buff2, pTfctTime->nBin, &pTfctTime->corrupt);

             RA_HUpdate(pTfctTime->numPicture, pTfctTime->name, pTfctTime->title, pTfctTime->options);
             DyDebugTimeNeeded(__FUNCTION__,"stop",1);
            }
        }

     DyCheckRunning();    /* interaction with the data Display */
    }

 /*--- shift buffer used for FFT ---*/

 if (pTfctTime->i == pTfctTime->nBin)
    {
     for (j=0; j<pTfctTime->nBin-pTfctTime->dNbin; j++)
        {pTfctTime->buff1[j] = pTfctTime->buff1[j+pTfctTime->dNbin];
         pTfctTime->buff2[j] = pTfctTime->buff2[j+pTfctTime->dNbin];
        }
     pTfctTime->i = pTfctTime->i - pTfctTime->dNbin;

     if (pTfctTime->nAverage == pTfctTime->maxAverage)
        {
         pTfctTime->itime++;
         pTfctTime->nAverage = 0;
         for (j=0 ;j<pTfctTime->nBin/2+1 ;j++)  pTfctTime->mod1sq->dataD[j] = 0.;
         for (j=0 ;j<pTfctTime->nBin/2+1 ;j++)  pTfctTime->rer->dataD[j]    = 0.;
         for (j=0 ;j<pTfctTime->nBin/2+1 ;j++)  pTfctTime->imr->dataD[j]    = 0.;
        }
    }

 /*--- Update buffers ---*/

 if (pTfctTime->itime == pTfctTime->nBinTime)
    {
     pTfctTime->itime--;

     RA_H2Unpack(pTfctTime->name, pTfctTime->buffTot);

     /*--- Shift the buffer content ---*/

     nx = pTfctTime->nBinTime;
     for (i=0; i<pTfctTime->nBinTime-1; i++)
        {for (j=0; j<pTfctTime->nBin/2+1; j++)
            {pTfctTime->buffTot[i+j*nx] = pTfctTime->buffTot[i+1+j*nx];}
        }
     for (j=0; j<pTfctTime->nBin/2+1; j++)  pTfctTime->buffTot[nx-1+nx*j] = 0.;

     pTfctTime->tmin += pTfctTime->dt * pTfctTime->dNbin * pTfctTime->resampFactor * pTfctTime->maxAverage;
    }

 if (UFrCurFrameH)
    {
     if ((pTfctTime->previousGPS > 0) && ((double)(UFrCurFrameH->GTimeS-pTfctTime->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {pTfctTime->tmin += (double)(UFrCurFrameH->GTimeS - pTfctTime->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) pTfctTime->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*============================================== PLOT 1D_TIME ========================================*/



/*---------------------------------------------------- UHplot1DTEnd -----*/
void UHplot1DTEnd(plot1DTime)
struct UHplot1DTime *plot1DTime;
/*---------------------------------------------------------------------*/
{
 if (plot1DTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(plot1DTime->numPicture);
 RA_HUpdateEnd(plot1DTime->name,plot1DTime->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot1DTEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  plotOpt=%s\n",
             plot1DTime->name, plot1DTime->title, plot1DTime->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plot1DTime->plotOpt);
    }

 if (plot1DTime->title   != NULL)  free(plot1DTime->title);
 if (plot1DTime->options != NULL)  free(plot1DTime->options);
 if (plot1DTime->name    != NULL)  free(plot1DTime->name);
 if (plot1DTime->buff    != NULL)  free(plot1DTime->buff);
 if (plot1DTime->buffTot != NULL)  free(plot1DTime->buffTot);

 free(plot1DTime);

 return;
}


/*---------------------------------------------------- UHplot1DTIni -----*/
void UHplot1DTIni(plot1DTime,dt)
struct UHplot1DTime *plot1DTime;
double dt;
/*--------------------------------------------------------------------*/
{
 long i;
 char* oldname;

 if (plot1DTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (plot1DTime->resampFactor <= 0) plot1DTime->resampFactor = 1;

 plot1DTime->dt           = dt * (double)plot1DTime->resampFactor;
 plot1DTime->i            = 0;
 plot1DTime->j            = 0;
 plot1DTime->itime        = 0;
 plot1DTime->itimeView    = 0;
 plot1DTime->previousGPS  = 0;
 plot1DTime->t0           = 0;
 plot1DTime->timeOffset   = 0.;
 plot1DTime->refresh_nBin = 1;
 plot1DTime->iFirst       = ON;


 /*---- decode the options string --------*/

 UHoptDecode(plot1DTime->options, &plot1DTime->intervalle, plot1DTime->plotOpt);

 if (plot1DTime->intervalle <= 0) plot1DTime->intervalle = plot1DTime->nBin;

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plot1DTime->options,"SAME") == NULL) plot1DTime->numPicture = DyGrafIni(0);
     else                                            plot1DTime->numPicture = 0;
    }

/*---  Define the 2D plot used for time/freq and set limits on frequency axis  ---*/

 plot1DTime->nBinTime = (long)(plot1DTime->timeSize / (plot1DTime->nBin * plot1DTime->dt));

 if (plot1DTime->refresh_percent > 0)
    {
     plot1DTime->refresh_nBin = (long)((plot1DTime->refresh_percent * plot1DTime->timeSize/100.) 
                                 / (plot1DTime->nBin * plot1DTime->dt));
     if (plot1DTime->refresh_nBin <= 0) plot1DTime->refresh_nBin = 1;
    }
 else
    {plot1DTime->refresh_nBin = 1;}

 if (plot1DTime->intervalle < plot1DTime->nBin)
    {plot1DTime->intervalle = plot1DTime->nBin;}


 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplot1DTIni> plot1DTime=%s  timeSize=%g  dt=%g  nBinTime=%ld  nyBin=%ld nBin=%ld \n",
             plot1DTime->title, plot1DTime->timeSize, plot1DTime->dt, plot1DTime->nBinTime, plot1DTime->nyBin, plot1DTime->nBin);
    }

 if (strstr(plot1DTime->options,"AUTO") == NULL) 
    {plot1DTime->adjust = NO;}
 else
    {plot1DTime->adjust = YES;
     if (plot1DTime->nyBin % 2 == 1)  plot1DTime->nyBin = plot1DTime->nyBin + 1;
    }

/*---  Get the limits of the previous histogram in the pad if it is good  ---*/ 
 oldname = RA_PadContainsGoodHist(plot1DTime->numPicture, 0, 2, plot1DTime->nBinTime, plot1DTime->nyBin, plot1DTime->name);
 if (!oldname)
   {RA_PadCompleteClear(plot1DTime->numPicture);
    UHoptPlopt(plot1DTime->options, plot1DTime->numPicture, 0);
   }

 RA_H2NewReuse(plot1DTime->numPicture, 0,
               plot1DTime->name, plot1DTime->title,
               plot1DTime->nBinTime,
               plot1DTime->nBin * plot1DTime->dt * (-0.5),
               plot1DTime->nBin * plot1DTime->dt * ((double)plot1DTime->nBinTime - 0.5),
               plot1DTime->nyBin,
               plot1DTime->yMin,
               plot1DTime->yMax);

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plot1DTime->name);

 RA_HSetStat(plot1DTime->name, NULL);

 RA_SetTestTrigPad(plot1DTime->numPicture, 0, plot1DTime->refresh_percent * plot1DTime->timeSize/100.);

 RA_HSetUnitsX(plot1DTime->name, "sec");
 RA_HSetUnitsY(plot1DTime->name, "a.u.");

 if (plot1DTime->zmin < DY_BADDATA) RA_HSetMinimum(plot1DTime->name, plot1DTime->zmin);
 if (plot1DTime->zmax > 0.)         RA_HSetMaximum(plot1DTime->name, plot1DTime->zmax);

 plot1DTime->buff = (float *)calloc(plot1DTime->nBin+1, sizeof(float));
 if (plot1DTime->buff == NULL) printf("UHplot1DTIni> plot1DTime->buff calloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplot1DTIni> plot=%s=%s  Doing a calloc for %ld times %ld floats = %ld bytes\n",
             plot1DTime->name, plot1DTime->title, plot1DTime->nBinTime, plot1DTime->nyBin,
             plot1DTime->nBinTime * (plot1DTime->nyBin));
    }

 plot1DTime->buffTot = (float *) calloc(plot1DTime->nBinTime*(plot1DTime->nyBin), sizeof(float));
 if (plot1DTime->buffTot == NULL) printf("UHplot1DTIni> plot1DTime->buffTot calloc failed");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplot1DTIni> plot=%s=%s=pad%d  plotOpt=%s \n",
             plot1DTime->name, plot1DTime->title, plot1DTime->numPicture, plot1DTime->plotOpt);
    }

 return;
}



/*---------------------------------------------------- UHplot1DTIniFrame -----*/

void UHplot1DTIniFrame (struct UHplot1DTime *plot1DTime)
{
 long i;
 char* oldname;
 double offset, framedt, value;
 struct FrAdcData *adc;
 struct FrVect *vect;
 char *name1, *temp;
 long length;

 if (plot1DTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 temp   = strstr(plot1DTime->title,SEPTITLE);
 length = strlen(plot1DTime->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plot1DTime->title, length);

 adc  = FrAdcDataFind(UFrCurFrameH, name1);
 vect = FrameFindVect(UFrCurFrameH, name1);

 if (adc == NULL)  adc  = FrAdcDataFind(Dy->Frame, name1);
 if (vect == NULL) vect = FrameFindVect(Dy->Frame, name1);

 free(name1);

 plot1DTime->slope = 1.;
 plot1DTime->bias  = 0.;

 if (adc != NULL)
    {if (adc->slope != 0.) plot1DTime->slope = adc->slope;
     plot1DTime->bias  = adc->bias;
    }

 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     if (UFrCurFrameH != NULL)
        {framedt = DyGetFramedt(UFrCurFrameH);
         offset = ((double)Dy->timeOrigin - (double)UFrCurFrameH->GTimeS) / framedt;
         offset = offset - (int)offset;
         if (offset*UFrCurFrameH->dt < 1.) offset = 0.;
        }
     else                     
        {printf("UHplot1DTIniFrame> UFrCurFrameH=NULL. Forced offset=0.\n");
         offset = 0.;
        }

     plot1DTime->timeOffset = offset * framedt * plot1DTime->resampFactor;

     if (offset > 0.)
        {if (vect) plot1DTime->t0 = (int)(offset*UFrCurFrameH->dt / vect->dx[0]);
         else      plot1DTime->t0 = (int)(offset*UFrCurFrameH->dt);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
            {printf("UHplot1DTIniFrame> startTime=%ld  GTimeS=%d  framedt=%f  offset=%f  t0=%ld  plot1DTime->timeOffset=%f  ",
                          Dy->timeOrigin, UFrCurFrameH->GTimeS, UFrCurFrameH->dt, offset, plot1DTime->t0,plot1DTime->timeOffset);
             if (vect) printf("vect=%s:dx[0]=%f \n", vect->name, vect->dx[0]);
             else      printf("vect=NULL\n");
            }
        }
    }

 plot1DTime->tmin = plot1DTime->dt * (double)plot1DTime->t0;

 if (plot1DTime->adjust == YES)
    {
     if (vect != NULL  && vect->nData > 2 && FrVectIsValid(vect)==0)
        {
         plot1DTime->yMax = -DY_BADDATA;
         plot1DTime->yMin =  DY_BADDATA;
         for (i=0; i<vect->nData; i++)
            {value = FrVectGetV(vect,i);
             if (plot1DTime->realUnits != 0  &&  value < DY_BADDATA)
                {value = value * plot1DTime->slope + plot1DTime->bias;}
             if (plot1DTime->yMax < value) plot1DTime->yMax = value;
             if (plot1DTime->yMin > value) plot1DTime->yMin = value;
            }
         if (plot1DTime->yMax == plot1DTime->yMin)  plot1DTime->yMax = plot1DTime->yMin + 1e-15; 

         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("UHplot1DTIniFrame> autoAdjust  yMin=%g  yMax=%g  nBin=%ld  vect->nData=%d\n",
                     plot1DTime->yMin, plot1DTime->yMax, plot1DTime->nyBin, vect->nData);
            }
        }

     RA_HDelete(plot1DTime->name);
     RA_H2NewReuse(plot1DTime->numPicture, 0,
                   plot1DTime->name, plot1DTime->title,
                   plot1DTime->nBinTime,
                   plot1DTime->nBin * plot1DTime->dt * (-0.5),
                   plot1DTime->nBin * plot1DTime->dt * ((double)plot1DTime->nBinTime - 0.5),
                   plot1DTime->nyBin,
                   plot1DTime->yMin,
                   plot1DTime->yMax);
     RA_HSetStat(plot1DTime->name, NULL);
     RA_SetTestTrigPad(plot1DTime->numPicture, 0, plot1DTime->refresh_percent * plot1DTime->timeSize/100.);
     RA_HSetUnitsX(plot1DTime->name, "sec");
     RA_HSetUnitsY(plot1DTime->name, "a.u.");
     if (plot1DTime->zmax > 0.)         RA_HSetMaximum(plot1DTime->name, plot1DTime->zmax);
     if (plot1DTime->zmin < DY_BADDATA) RA_HSetMinimum(plot1DTime->name, plot1DTime->zmin);
    }

 RA_HSetUnitsY(plot1DTime->name, "a.u.");

 if (vect != NULL)
    {
     if (plot1DTime->realUnits > 0)
        {strcpy(plot1DTime->units, vect->unitY);}
     if (plot1DTime->realUnits == 0 && plot1DTime->slope == 1.  &&  plot1DTime->bias == 0.)
        {strcpy(plot1DTime->units, vect->unitY);}
     RA_HSetUnitsY(plot1DTime->name, plot1DTime->units);
    }

 if (adc != NULL)
    {
     if (plot1DTime->realUnits > 0)
        {strcpy(plot1DTime->units, adc->units);}
     if (plot1DTime->realUnits == 0 && plot1DTime->slope == 1.  &&  plot1DTime->bias == 0.)
        {strcpy(plot1DTime->units, adc->units);}
     RA_HSetUnitsY(plot1DTime->name, plot1DTime->units);
    }

 return;
}


/*----------------------------------------------------- UHplot1DTRead --*/
void UHplot1DTRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplot1DTime *plot1DTime;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplot1DTime, plot1DTime);
 
 UJ_READ_LONG(plot1DTime->iClock)
 UJ_READ_LONG(plot1DTime->id)
 if (!plot1DTime->name) plot1DTime->name = (char *) calloc(10,sizeof(char));
 sprintf(plot1DTime->name,"h%ld",plot1DTime->id);
 UJ_READ_S_CHAR(plot1DTime->title)
 UJ_READ_LONG(plot1DTime->nyBin)
 UJ_READ_DOUBLE(plot1DTime->yMin)
 UJ_READ_DOUBLE(plot1DTime->yMax)
 UJ_READ_DOUBLE(plot1DTime->timeSize)
 UJ_READ_LONG(plot1DTime->resampFactor)
 UJ_READ_LONG(plot1DTime->t0)
 UJ_READ_STRUCT(USignal, plot1DTime->input)
 UJ_READ_S_CHAR(plot1DTime->options)

 if (plot1DTime->nyBin < 4) plot1DTime->nyBin = 4;
   
 UJCObAdd(plot1DTime->iClock,plot1DTime,UHplot1DTimeUJstructH,NULL);

 plot1DTime->hidden = OFF;
 s = strstr(plot1DTime->options,"HIDDEN");
 if (s != NULL)  plot1DTime->hidden = ON;

 plot1DTime->realUnits = 0;
 s = strstr(plot1DTime->options,"REALUNITS");
 if (s != NULL)  plot1DTime->realUnits = 1;

 plot1DTime->nodc = 0;
 s = strstr(plot1DTime->options,"NODC");
 if (s != NULL)  plot1DTime->nodc = 1;

 plot1DTime->zmin = DY_BADDATA;
 plot1DTime->zmax = 0.;
 s = strstr(plot1DTime->options,"ZMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plot1DTime->zmin);
 s = strstr(plot1DTime->options,"ZMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plot1DTime->zmax);

 plot1DTime->refresh_percent = 0.;
 s = strstr(plot1DTime->options,"REFRESH_PERCENT");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plot1DTime->refresh_percent);

 plot1DTime->nBin = 40;
 s = strstr(plot1DTime->options,"N1D");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plot1DTime->nBin);

 if (plot1DTime->yMax <= plot1DTime->yMin  &&  plot1DTime->yMin > 0.)
    {plot1DTime->yMax = 2. * plot1DTime->yMin;}
 if (plot1DTime->yMax <= plot1DTime->yMin  &&  plot1DTime->yMin < 0.)
    {plot1DTime->yMax = 0.5 * plot1DTime->yMin;}
 if (plot1DTime->yMax == 0.  &&  plot1DTime->yMin == 0.)
    {plot1DTime->yMin = -1.e-15; plot1DTime->yMax = 1.e-15;}

 return;
}
 

/*--------------------------------------------------- UHplot1DTSim -----*/
void UHplot1DTSim(plot1DTime,time)
struct UHplot1DTime *plot1DTime;
long time;
/*--------------------------------------------------------------------*/
{
 long  i, j, inter, last;
 double *tab, *fft, coef, value, yValue, dc;
 char *name1, *temp;
 struct FrVect *vect1=NULL;
 long length, origin;
 struct FrAdcData *adc;
 int fractime, nx, status;
 char title[256], stringtime[40];
 time_t starttimeutc;

 if (plot1DTime->hidden == ON) return;

 if (Dy->Order == STOP) return;

 if (time < plot1DTime->t0)  return;

 if (plot1DTime->iFirst == ON)
    {plot1DTime->iFirst = -1;
     UHplot1DTIniFrame(plot1DTime);
    }

 /*--- Take the input value ---*/

 if (plot1DTime->input == NULL) 
    {UFrUnpackSim(plot1DTime->FrUnpack, 0);
     value = plot1DTime->FrUnpack->out->value;
    }
 else
    {value = plot1DTime->input->value;}

 if (plot1DTime->realUnits != 0  &&  value < DY_BADDATA) 
    {value = value * plot1DTime->slope + plot1DTime->bias;}


 /*--- Accumulate the input value into tab vector ---*/
 /*--- Until number of values reach resampFactor ---*/

 if (value < DY_BADDATA)         plot1DTime->sumvalue += value;
 else if (value >= DY_NOCHANNEL) plot1DTime->sumvalue  = DY_NOCHANNEL * 1.01 * plot1DTime->resampFactor;
 else if (value >= DY_NOFRAME)   plot1DTime->sumvalue  = DY_NOFRAME   * 1.01 * plot1DTime->resampFactor;
 else                            plot1DTime->sumvalue  = DY_BADDATA   * 1.01 * plot1DTime->resampFactor;

 plot1DTime->j++;

 if (plot1DTime->j < plot1DTime->resampFactor) return;


 /*---------- fill data and plot -----------------*/

 plot1DTime->buff[plot1DTime->i] = plot1DTime->sumvalue / plot1DTime->resampFactor;
 plot1DTime->i++;
 plot1DTime->sumvalue = 0.;
 plot1DTime->j = 0;

 /*--- Auto adjust of 1D edges ---*/

 if (plot1DTime->adjust == YES)
    {
     yValue = plot1DTime->buff[plot1DTime->i-1];

     if (plot1DTime->iFirst == -1)
        {if (yValue != 0.) plot1DTime->iFirst = OFF;
         if (yValue < DY_BADDATA && yValue != 0.) {plot1DTime->yMax = yValue + 0.00001*fabs(yValue); plot1DTime->yMin  = yValue - 0.00001*fabs(yValue);}
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
            {printf("UHplot1DTSim> yValue=%g  yMax=%g  yMin=%g\n", yValue, plot1DTime->yMax, plot1DTime->yMin);}
        }

     while (yValue > plot1DTime->yMax  &&  yValue < DY_BADDATA)
        {
         plot1DTime->yMax = 2 * plot1DTime->yMax - plot1DTime->yMin;
         RA_SetLimitsY(plot1DTime->name, plot1DTime->yMin, plot1DTime->yMax);

         nx   = plot1DTime->nBinTime;
         for (i=0; i<nx; i++)
             {for (j=0; j<plot1DTime->nyBin/2; j++)
                  {plot1DTime->buffTot[i+j*nx] = plot1DTime->buffTot[i+2*j*nx]
                                               + plot1DTime->buffTot[i+(2*j+1)*nx];
                  }
              for (j=plot1DTime->nyBin/2; j<plot1DTime->nyBin; j++)
                  {plot1DTime->buffTot[i+j*nx] = 0.;}
             }
        }

     while (yValue < plot1DTime->yMin)
        {
         plot1DTime->yMin = 2 * plot1DTime->yMin - plot1DTime->yMax;
         RA_SetLimitsY(plot1DTime->name, plot1DTime->yMin, plot1DTime->yMax);

         last = plot1DTime->nyBin-1;
         nx   = plot1DTime->nBinTime;
         for (i=0; i<nx; i++)
             {for (j=0; j<plot1DTime->nyBin/2; j++)
                  {plot1DTime->buffTot[i+(last-j)*nx] = plot1DTime->buffTot[i+(last-2*j)*nx]
                                                      + plot1DTime->buffTot[i+(last-2*j-1)*nx];
                  }
              for (j=0; j<plot1DTime->nyBin/2; j++)
                  {plot1DTime->buffTot[i+j*nx] = 0.;}
             }
        }
    }


 /*---- do 1D and shift data if we reach the end of the plot --------*/

 if (plot1DTime->i == plot1DTime->nBin)
    {
     dc = 0.;
     if (plot1DTime->nodc > 0)
        {
         for (j=0 ;j<plot1DTime->nBin; j++)  dc += plot1DTime->buff[j];
         if (plot1DTime->nBin > 0)  dc = dc / (double)plot1DTime->nBin;
        }

     RA_HSetUnitsY(plot1DTime->name, "a.u.");
     if (plot1DTime->realUnits > 0)
        {RA_HSetUnitsY(plot1DTime->name, plot1DTime->units);}
     if (plot1DTime->realUnits == 0 && plot1DTime->slope == 1.  &&  plot1DTime->bias == 0.)
        {RA_HSetUnitsY(plot1DTime->name, plot1DTime->units);}

     nx = plot1DTime->nBinTime;
     for (j=0 ;j<plot1DTime->nBin ;j++)
        {value = plot1DTime->buff[j];
         i = floor( (value - plot1DTime->yMin)*(double)(plot1DTime->nyBin)
                   / (plot1DTime->yMax - plot1DTime->yMin));
         if (i >= 0 && i < plot1DTime->nyBin)
            {plot1DTime->buffTot[plot1DTime->itime+nx*i] += 1.;}
        }

     RA_H2Pack(plot1DTime->name, plot1DTime->buffTot);
     plot1DTime->itimeView++;

     if (Dy->plotsNotDisplayed == OFF && plot1DTime->itimeView%plot1DTime->refresh_nBin == 0)
        {
         DyDebugPrint(__FUNCTION__,0);
         DyDebugTimeNeeded(__FUNCTION__,"start",1);

         /*--- Set time string and time axis offset ---*/
         origin = Dy->timeOrigin + plot1DTime->tmin - plot1DTime->timeOffset;
         UHplotSetTime (origin, plot1DTime->name, plot1DTime->nBin*plot1DTime->dt, 0, ON, UFrCurFrameH);

         /*--- Draw the plot ---*/
         UHplot(plot1DTime->name, plot1DTime->numPicture, plot1DTime->options, plot1DTime->plotOpt, 0, 0);

         /*--- Update plot environment ---*/
         RA_SetActivePad(plot1DTime->numPicture);
         UHplotUpdateEnv (plot1DTime->numPicture, plot1DTime->name, plot1DTime->options, plot1DTime->name,
                          plot1DTime->numPicture, 0, plot1DTime->title,
                          0, plot1DTime->buff, plot1DTime->nBin, &plot1DTime->corrupt);

         RA_HUpdate( plot1DTime->numPicture, plot1DTime->name, plot1DTime->title, plot1DTime->options);
         DyDebugTimeNeeded(__FUNCTION__,"stop",1);
        }

     DyCheckRunning();    /* interaction with the data Display */

     plot1DTime->i = 0;
     plot1DTime->itime++;
    }

 /*--- Update buffers ---*/

 if (plot1DTime->itime == plot1DTime->nBinTime)
    {
     plot1DTime->itime--;

     RA_H2Unpack(plot1DTime->name, plot1DTime->buffTot);

     /*--- Shift the buffer content ---*/

     nx = plot1DTime->nBinTime;
     for (i=0; i<plot1DTime->nBinTime-1; i++)
        {for (j=0; j<plot1DTime->nyBin; j++)
            {plot1DTime->buffTot[i+j*nx] = plot1DTime->buffTot[i+1+j*nx];}
        }
     for (j=0; j<plot1DTime->nyBin; j++)  plot1DTime->buffTot[nx-1+nx*j] = 0.;

     plot1DTime->tmin += plot1DTime->dt * plot1DTime->nBin;
    }

 if (UFrCurFrameH)
    {
     if ((plot1DTime->previousGPS > 0) && ((double)(UFrCurFrameH->GTimeS-plot1DTime->previousGPS) > UFrCurFrameH->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {plot1DTime->tmin += (double)(UFrCurFrameH->GTimeS - plot1DTime->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) plot1DTime->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*================================================ PLOT RAW ==========================================*/

/*---------------------------------------------------- UHplotRawEnd -----*/
void UHplotRawEnd(plotRaw)
struct UHplotRaw *plotRaw;
/*--------------------------------------------------------------------*/
{
 if (plotRaw->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(plotRaw->numPicture);
 RA_HUpdateEnd(plotRaw->name,plotRaw->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotRawEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  plotOpt=%s\n",
             plotRaw->name, plotRaw->title, plotRaw->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plotRaw->plotOpt);
    }

 if (plotRaw->name     != NULL)  free(plotRaw->name);
 if (plotRaw->title    != NULL)  free(plotRaw->title);
 if (plotRaw->options  != NULL)  free(plotRaw->options);
 if (plotRaw->buff     != NULL)  free(plotRaw->buff);

 free(plotRaw);

 return;
}


/*---------------------------------------------------- UHplotRawIni -----*/
void UHplotRawIni(plotRaw,dt)
struct UHplotRaw *plotRaw;
double dt;
/*--------------------------------------------------------------------*/
{
 long length, numplot, refnumplot;
 double tmax;
 struct FrAdcData *adc=NULL;
 struct FrVect *vect=NULL;
 char *temp, *name1, *oldname;
 double f0, w0, r, fs, df;
 double offset, framedt;
 static struct UHplotRaw *previous=NULL;
 static int color;
 int j;

 if (plotRaw->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (plotRaw->iFirst == ON) return;

 if (plotRaw->resampFactor <= 0) plotRaw->resampFactor = 1;

 temp   = strstr(plotRaw->title,SEPTITLE);
 length = strlen(plotRaw->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plotRaw->title, length); 

 if (UFrCurFrameH)
    {adc  = FrAdcDataFind(UFrCurFrameH, name1);
     vect = FrameFindVect(UFrCurFrameH, name1);
    }

 if (adc != NULL && vect == NULL) vect = adc->data;

 plotRaw->dt       = dt;
 plotRaw->i        = 0;
 plotRaw->j        = 0;
 plotRaw->sumvalue = 0.;
 plotRaw->t0       = 0;
 plotRaw->resampFactor = 1;
 plotRaw->previousGPS = 0;  

 if (vect == NULL) return;

 FrVectFixVirgoImage(vect);

 plotRaw->iFirst = OFF;

 if (adc == NULL)
    {adc  = FrAdcDataFind(Dy->Frame, name1);
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
        {printf("UHplotRawIni> Using DyFrame: name1=%s  adc=%x  vect=%x\n", name1, adc, vect);}
    }

 free(name1);

 plotRaw->slope = 1.;
 plotRaw->bias  = 0.;

 if (adc != NULL)
    {if (adc->slope != 0.) plotRaw->slope = adc->slope;
     plotRaw->bias  = adc->bias;
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1) printf("UHplotRawIni> slope=%g  bias=%g\n", plotRaw->slope, plotRaw->bias);
    }

 plotRaw->raw2d = 0;
 if (vect != NULL && vect->nData > vect->nx[1]) plotRaw->raw2d = 1;


 /*---- decode the options string --------*/

 UHoptDecode(plotRaw->options, &plotRaw->intervalle, plotRaw->plotOpt);

 if (Dy->plotsNotDisplayed == OFF) 
    {if (strstr(plotRaw->options,"SAME") == NULL)  plotRaw->numPicture = DyGrafIni(0);
     else                                          plotRaw->numPicture = previous->numPicture;
    }

 if (vect != NULL)
    {
     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
        {printf("UHplotRawIni> vect=%s  nDim=%d  nData=%d  nx[0]=%d nx[1]=%d nx[2]=%d dx[0]=%g dx[1]=%g dx[2]=%g\n",
                 vect->name, vect->nDim, vect->nData, vect->nx[0], vect->nx[1], vect->nx[2], vect->dx[0], vect->dx[1], vect->dx[2]);
        }
     plotRaw->nxBin = vect->nx[0];
     plotRaw->xMin  = vect->startX[0];
     plotRaw->xMax  = plotRaw->xMin + vect->dx[0] * vect->nx[0];
     if (plotRaw->raw2d > 0)
        {plotRaw->nxBin = vect->nx[1];
         plotRaw->xMin  = vect->startX[1];
         plotRaw->xMax  = plotRaw->xMin + vect->dx[1] * vect->nx[1];
         plotRaw->nyBin = vect->nx[2];
         plotRaw->yMin = vect->startX[2];
         plotRaw->yMax = plotRaw->yMin + vect->dx[2] * vect->nx[2];
        }
    }
 else
    {
     plotRaw->nxBin = 20000;
     plotRaw->xMin  = 0.;
     plotRaw->xMax  = 1.;
     if (plotRaw->raw2d > 0)
        {plotRaw->nxBin = 256;
         plotRaw->xMin  = 0.;
         plotRaw->xMax  = 1.;
         plotRaw->nyBin = 256;
         plotRaw->yMin  = 0.;
         plotRaw->yMax  = 1.;
        }
    }

 /*--- Get the limits of the previous histogram in the pad if it is good  ---*/ 
 if (plotRaw->raw2d > 0)
    {oldname = RA_PadContainsGoodHist(plotRaw->numPicture, 0, 2, plotRaw->nxBin, plotRaw->nyBin, plotRaw->name);
     if (!oldname) RA_H2NewReuse(plotRaw->numPicture, 0, plotRaw->name, plotRaw->title,
                   plotRaw->nxBin, plotRaw->xMin, plotRaw->xMax,
                   plotRaw->nyBin, plotRaw->yMin, plotRaw->yMax);
    }
 else
    {oldname = RA_PadContainsGoodHist(plotRaw->numPicture, 0, 1, plotRaw->nxBin, 0, plotRaw->name);
     RA_H1NewReuse(plotRaw->numPicture, 0, plotRaw->name, plotRaw->title,
                   plotRaw->nxBin, plotRaw->xMin, plotRaw->xMax);
    }

 RA_SetTestTrigPad(plotRaw->numPicture, 0, (double)plotRaw->intervalle * dt);

 if (!oldname || plotRaw->autoScale == 0)
    {RA_HSetMinimum(plotRaw->name, plotRaw->zMin);
     RA_HSetMaximum(plotRaw->name, plotRaw->zMax);
    }
 if (!oldname)
    {UHoptPlopt(plotRaw->options, plotRaw->numPicture, 0);}

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plotRaw->name);

 RA_HSetStat(plotRaw->name, NULL);
 RA_HSetUnitsX(plotRaw->name, "a.u.");
 RA_HSetUnitsY(plotRaw->name, "a.u.");

 if (plotRaw->raw2d > 0)
    {
     if (vect != NULL)
        {RA_HSetUnitsX(plotRaw->name, vect->unitX[1]);
         RA_HSetUnitsY(plotRaw->name, vect->unitX[2]);
         RA_HSetUnitsZ(plotRaw->name, vect->unitY);
        }
    }
 else
    {
     if (vect != NULL)
        {RA_HSetUnitsX(plotRaw->name, vect->unitX[0]);
         if (plotRaw->realUnits > 0)
            {RA_HSetUnitsY(plotRaw->name, vect->unitY);}
         if (plotRaw->realUnits == 0 && plotRaw->slope == 1.  &&  plotRaw->bias == 0.)
            {RA_HSetUnitsY(plotRaw->name, vect->unitY);}
        }
     if (adc != NULL)
        {if (plotRaw->realUnits > 0)
            {RA_HSetUnitsY(plotRaw->name, adc->units);}
         if (plotRaw->realUnits == 0 && plotRaw->slope == 1.  &&  plotRaw->bias == 0.)
            {RA_HSetUnitsY(plotRaw->name, adc->units);}
        }
    }

 if (plotRaw->raw2d > 0) plotRaw->nBin = plotRaw->nxBin*plotRaw->nyBin;
 else                    plotRaw->nBin = plotRaw->nxBin;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplotRawIni> Malloc for %d dim : nxBin*nyBin =%d*%d = %d\n", plotRaw->raw2d+1, plotRaw->nxBin, plotRaw->nyBin, plotRaw->nBin);}

 plotRaw->buff = (float *)calloc(plotRaw->nBin, sizeof(float));
 if (plotRaw->buff == NULL) printf("UHplotRawIni> plotRaw->buff malloc failed\n");

 sscanf(plotRaw->name, "h%ld", &numplot);

 if (strstr(plotRaw->options,"SAME") == NULL)
    {previous = plotRaw;
     previous->nsame = 0;
     plotRaw->refplot = previous;
     color = 1;
     Dy->refnumplot[numplot] = numplot;
    }
 else if (previous)
    {previous->nsame++;
     plotRaw->refplot = previous;
     color++;
     if (color == 16)  color = 40+color-15;
     if (color == 48) color = 30+color-47;
     sscanf(plotRaw->refplot->name, "h%ld", &refnumplot);
     Dy->refnumplot[numplot] = refnumplot;
    }

 plotRaw->color = color;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotRawIni> plot=%s=%s=pad%d  color=%d  refplot=%s=%s=pad%d\n",
             plotRaw->name, plotRaw->title, plotRaw->numPicture, plotRaw->color,
             plotRaw->refplot->name, plotRaw->refplot->title, plotRaw->refplot->numPicture);
    }

 return;
}
 

/*------------------------------------------------------ UHplotRawRead --*/
void UHplotRawRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplotRaw *plotRaw;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplotRaw,plotRaw)
 
 UJ_READ_LONG(plotRaw->iClock)
 UJ_READ_LONG(plotRaw->id)
 if (!plotRaw->name) plotRaw->name = (char *) calloc(10,sizeof(char));
 sprintf(plotRaw->name,"h%ld",plotRaw->id);
 UJ_READ_S_CHAR(plotRaw->title)
 UJ_READ_LONG(plotRaw->t0)
 UJ_READ_STRUCT(USignal, plotRaw->input) 
 UJ_READ_S_CHAR(plotRaw->options)
    
 UJCObAdd(plotRaw->iClock,plotRaw,UHplotRawUJstructH,NULL);

 plotRaw->hidden = OFF;
 s = strstr(plotRaw->options,"HIDDEN");
 if (s != NULL)  plotRaw->hidden = ON;

 plotRaw->maxAverage = 1;
 s = strstr(plotRaw->options,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plotRaw->maxAverage);

 plotRaw->resampFactor = 1;
 s = strstr(plotRaw->options,"RESAMPFACTOR");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plotRaw->resampFactor);

 plotRaw->realUnits = 0;
 s = strstr(plotRaw->options,"REALUNITS");
 if (s != NULL)  plotRaw->realUnits = 1;

 plotRaw->zMin = 0.;
 s = strstr(plotRaw->options,"YMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotRaw->zMin);

 plotRaw->zMax = 0.;
 s = strstr(plotRaw->options,"YMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotRaw->zMax);

 plotRaw->autoScale = 0;
 s = strstr(plotRaw->options,"AUTOSCALE");
 if (s != NULL)  plotRaw->autoScale = 1;

 plotRaw->x2av = 0;
 s = strstr(plotRaw->options,"X2AV");
 if (s != NULL)  plotRaw->x2av = 1;

 if (plotRaw->zMax <= plotRaw->zMin)  plotRaw->autoScale = 1;  

 plotRaw->iFirst = ON;

 return;
} 


/*---------------------------------------------------- UHplotRawSim -----*/
void UHplotRawSim(plotRaw,time)
struct UHplotRaw *plotRaw;
long time;
/*--------------------------------------------------------------------*/
{
 long   i,j;
 int fractime, status=1;
 double dc;
 double value, localMin, localMax;
 char title[LSTRING_MAX], options[LSTRING_MAX];
 char stringtime[40];
 time_t starttimeutc;
 double dataValue, zmin, zmax, delta;
 static double minofMin=DY_BADDATA;
 static double maxofMax=-DY_BADDATA;
 double origin, deltat;
 struct FrAdcData *adc=NULL;
 struct FrVect *vect=NULL;
 char *temp, *name1;
 int length, im;

 if (plotRaw->hidden == ON) return;

 if (time < plotRaw->t0) return;

 if (UFrCurFrameH && UFrCurFrameH->GTimeS > plotRaw->previousGPS)
   {
    if (UFrCurVect == NULL) return;

    DyDebugPrint(__FUNCTION__,0);
    DyDebugTimeNeeded(__FUNCTION__,"start",1);

    if (plotRaw->iFirst == ON)
       {
        plotRaw->iFirst = OFF;
        UHplotRawIni(plotRaw,UFrCurFrameH->dt);
       }

    FrVectFixVirgoImage(UFrCurVect);

    if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>2)
       {printf("UHplotRawSim> plot=%s  nDim=%d  nData=%d  nBin=%d  nx[0]=%d  nx[1]=%d  nx[2]=%d  \n",
                plotRaw->title, UFrCurVect->nDim, UFrCurVect->nData, plotRaw->nBin, UFrCurVect->nx[0], UFrCurVect->nx[1], UFrCurVect->nx[2]);
       }

    for (im=0; im<(UFrCurVect->nData/plotRaw->nBin); im++)
       {
        if (plotRaw->raw2d > 0 && im > 0) {origin += im*UFrCurVect->dx[0]; continue;}

        for (i=0; i<plotRaw->nBin; i++)
           {
            value = FrVectGetV(UFrCurVect, i+im*plotRaw->nBin);
            if (UFrCurVect->type == FR_VECT_C && value < 0) value += 256.;
            if (plotRaw->realUnits != 0  &&  value < DY_BADDATA) value = value * plotRaw->slope + plotRaw->bias;
            if (value < DY_BADDATA)
               {if (plotRaw->x2av > 0) plotRaw->buff[i] += value*value;
                else                   plotRaw->buff[i] += value;
               }
           }
        plotRaw->nAverage++;

        /*--- Normalize before passing HPack ---*/
        for (i=0; i<plotRaw->nBin; i++)
           {if (plotRaw->x2av > 0) plotRaw->buff[i] = sqrt(plotRaw->buff[i] / plotRaw->nAverage);
            else                   plotRaw->buff[i] = plotRaw->buff[i] / plotRaw->nAverage;
            }

        /*--- Set min and max and fill the plots ---*/

        if (plotRaw->autoScale > 0)
           {
            localMin =  DY_BADDATA;
            localMax = -DY_BADDATA;
            for (j=0; j<plotRaw->nBin; j++)
               {if (j == plotRaw->nBin-1 && plotRaw->buff[j] == 0.) continue;
                if (localMin > plotRaw->buff[j]  &&  plotRaw->buff[j] < DY_BADDATA && plotRaw->buff[j] != 0.)  localMin = plotRaw->buff[j];
                if (localMax < plotRaw->buff[j]  &&  plotRaw->buff[j] < DY_BADDATA)  localMax = plotRaw->buff[j];
               }

            if (strstr(plotRaw->options,"SAME") == NULL)
               {minofMin = localMin;
                maxofMax = localMax;
               }
            else
               {if (minofMin > localMin) minofMin = localMin;
                if (maxofMax < localMax) maxofMax = localMax;
               }
           }

        if (plotRaw->raw2d == 0)
           {
            RA_H1Pack(plotRaw->name, plotRaw->buff);
           }
        else
           {RA_H2Pack(plotRaw->name, plotRaw->buff);}

        /*---------- Visualize data -----------------*/

        if (Dy->plotsNotDisplayed == OFF)
           {
            if (plotRaw->autoScale > 0)
               {if (strstr(plotRaw->options,"SAME") == NULL)
                   {if (plotRaw->nsame == 0)
                       {RA_SetActivePad(plotRaw->numPicture);
                        if (RA_GetLog(plotRaw->raw2d) > 0) {zmin = localMin/2.; zmax = localMax * 2;}
                        else                               {delta = 0.5 * (localMax-localMin); zmin = localMin - delta; zmax = localMax + delta;} 
                        if (plotRaw->raw2d > 0) {zmin = localMin; zmax = localMax;}
                        RA_HSetMinimum(plotRaw->name, zmin);
                        RA_HSetMaximum(plotRaw->name, zmax);
                       }
                   }
                else if (plotRaw->refplot)
                   {RA_SetActivePad(plotRaw->numPicture);
                    if (RA_GetLog(plotRaw->raw2d) > 0) {zmin = minofMin/2.; zmax = maxofMax * 2;}
                    else                               {delta = 0.5 * (maxofMax-minofMin); zmin = minofMin - delta; zmax = maxofMax + delta;} 
                    if (plotRaw->raw2d > 0) {zmin = localMin; zmax = localMax;}
                    RA_HSetMinimum(plotRaw->refplot->name, zmin);
                    RA_HSetMaximum(plotRaw->refplot->name, zmax);
                   }
               }

            /*--- Set time string and time axis offset ---*/
            origin = (double)(UFrCurFrameH->GTimeS);
            if (plotRaw->raw2d > 0 && UFrCurVect->nDim > 2) origin += im*UFrCurVect->dx[0];
            UHplotSetTime (origin, plotRaw->name, UFrCurFrameH->dt, plotRaw->nAverage, OFF, UFrCurFrameH);

            /*--- Draw the plot ---*/
            UHplot(plotRaw->name, plotRaw->numPicture, plotRaw->options, plotRaw->plotOpt, 0, plotRaw->refplot->nsame);

            /*--- Update plot environment ---*/
            RA_SetActivePad(plotRaw->numPicture);
            if (plotRaw->raw2d == 0)
               {UHplotUpdateEnv (plotRaw->numPicture, plotRaw->name, plotRaw->options, plotRaw->refplot->name,
                                 plotRaw->refplot->numPicture, plotRaw->refplot->nsame, plotRaw->title,
                                 plotRaw->color, plotRaw->buff, plotRaw->nBin, &plotRaw->corrupt);
               }

            /*--- Update the canvas ---*/
            RA_HUpdate(plotRaw->numPicture, plotRaw->name, plotRaw->title, plotRaw->options);
           }

        /*--- Check user interaction with the dataDisplay ---*/
        DyCheckRunning();

        if (plotRaw->nAverage == plotRaw->maxAverage)
           {plotRaw->nAverage = 0;
            for (j=0; j<plotRaw->nBin; j++) plotRaw->buff[j] = 0.;
           }
        else
           {
            /*--- Denormalize after passing HPack ---*/
            for (i=0; i<plotRaw->nBin; i++)
               {if (plotRaw->x2av > 0) plotRaw->buff[i] = plotRaw->buff[i]* plotRaw->buff[i] * plotRaw->nAverage;
                else                   plotRaw->buff[i] = plotRaw->buff[i] * plotRaw->nAverage;
               }
           }
       }
    DyDebugTimeNeeded(__FUNCTION__,"stop",1);
   }


 /*--- Take care of missing frames ---*/

 if (UFrCurFrameH)
    {
     if ((plotRaw->previousGPS > 0)  &&  ((double)(UFrCurFrameH->GTimeS-plotRaw->previousGPS) > UFrCurFrameH->dt))
        {
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
            {printf("plotRawSim> plot=%s  Missed %.1f frames (i=%d) \n",
                    plotRaw->title, (double)(UFrCurFrameH->GTimeS-plotRaw->previousGPS)/UFrCurFrameH->dt, plotRaw->i);
            }
        }
     if (UFrCurFrameH->GTimeS > 0) plotRaw->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}



/*====================================================================================================*/
/*============================================== PLOT RAW_TIME =======================================*/

/*---------------------------------------------------- UHplotRawTEnd -----*/
void UHplotRawTEnd(plotRawTime)
struct UHplotRawTime *plotRawTime;
/*---------------------------------------------------------------------*/
{
 if (plotRawTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(plotRawTime->numPicture);
 RA_HUpdateEnd(plotRawTime->name,plotRawTime->plotOpt);

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotRawTEnd> plot=%s=%s=pad%d  nready=%d  nsame=%d  nplot=%d  plotOpt=%s\n",
            plotRawTime->name, plotRawTime->title, plotRawTime->numPicture, Dy->nready, Dy->nsame, Dy->nplot, plotRawTime->plotOpt);
    }

 if (plotRawTime->title   != NULL)  free(plotRawTime->title);
 if (plotRawTime->options != NULL)  free(plotRawTime->options);
 if (plotRawTime->name    != NULL)  free(plotRawTime->name);
 if (plotRawTime->buff    != NULL)  free(plotRawTime->buff);
 if (plotRawTime->buffTot != NULL)  free(plotRawTime->buffTot);

 free(plotRawTime);

 return;
}


/*---------------------------------------------------- UHplotRawTIni -----*/
void UHplotRawTIni(plotRawTime,dt)
struct UHplotRawTime *plotRawTime;
double dt;
/*--------------------------------------------------------------------*/
{
 long i;
 char* oldname;
 double offset, framedt, value;
 struct FrAdcData *adc;
 struct FrVect *vect;
 struct FrProcData *proc;
 char *name1, *temp;
 long length;

 if (plotRawTime->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (plotRawTime->iFirst == ON) return;

 if (plotRawTime->resampFactor <= 0) plotRawTime->resampFactor = 1;

 temp   = strstr(plotRawTime->title,SEPTITLE);
 length = strlen(plotRawTime->title)-strlen(temp);
 name1 = (char *) calloc(length+1,sizeof(char));
 strncpy (name1, plotRawTime->title, length);
 if (UFrCurFrameH)
    {proc = FrProcDataFind(UFrCurFrameH, name1);
     adc  = FrAdcDataFind(UFrCurFrameH, name1);
     if (adc == NULL) adc = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(UFrCurFrameH, name1);
     if (vect == NULL) vect = FrameFindVect(Dy->Frame, name1);
    }

 if (proc == NULL)
    {proc = FrProcDataFind(Dy->Frame, name1);}

 if (adc == NULL  &&  vect == NULL)
    {adc  = FrAdcDataFind(Dy->Frame, name1);
     vect = FrameFindVect(Dy->Frame, name1);
    }
 free(name1);

 plotRawTime->slope = 1.;
 plotRawTime->bias  = 0.;

 if (adc != NULL)
    {if (adc->slope != 0.) plotRawTime->slope = adc->slope;
     plotRawTime->bias  = adc->bias;
    }

 plotRawTime->dt           = dt;
 plotRawTime->nAverage     = 0;
 plotRawTime->itime        = 0;
 plotRawTime->itimeView    = 0;
 plotRawTime->previousGPS  = 0;
 plotRawTime->t0           = 0;
 plotRawTime->timeOffset   = 0.;
 plotRawTime->refresh_nBin = 1;
 plotRawTime->raw2d        = 0;

 if (vect != NULL  && vect->nData > 2)
    {
     FrVectFixVirgoImage(vect);
     if (vect->nData > vect->nx[1]) plotRawTime->raw2d = 1;
     if (plotRawTime->nyBin <= 0) plotRawTime->nyBin = vect->nData;
     if (plotRawTime->yMin <= 0)  plotRawTime->yMin  = vect->startX[0];
     if (plotRawTime->yMax <= 0)  plotRawTime->yMax  = vect->startX[0]+vect->nData*vect->dx[0];
     if (proc)
        {if (proc->tRange > 0) plotRawTime->dt = proc->tRange;}

     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
        {printf("UHplotRawTIni> yMin=%g  yMax=%g  nBin=%ld  vect->nData=%d  dt=%g\n",
                 plotRawTime->yMin, plotRawTime->yMax, plotRawTime->nyBin, vect->nData, plotRawTime->dt);
        }
    }


 if (Dy->InputType != INCM && Dy->InputType != INSHM)
    {
     if (UFrCurFrameH != NULL)
        {framedt = DyGetFramedt(UFrCurFrameH);
         offset = ((double)Dy->timeOrigin - (double)UFrCurFrameH->GTimeS) / framedt;
         offset = offset - (int)offset;
         if (offset*UFrCurFrameH->dt < 1.) offset = 0.;
        }
     else                     
        {printf("UHplotRawIni> UFrCurFrameH=NULL. Forced offset=0.\n");
         offset = 0.;
        }

     plotRawTime->timeOffset = offset * framedt * plotRawTime->resampFactor;

     if (offset > 0.)
        {if (vect) plotRawTime->t0 = (int)(offset*UFrCurFrameH->dt / vect->dx[0]);
         else      plotRawTime->t0 = (int)(offset*UFrCurFrameH->dt);
         if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
            {printf("UHplotRawTIni> startTime=%ld  GTimeS=%d  framedt=%f  offset=%f  t0=%ld  plotRawTime->timeOffset=%f  ",
                     Dy->timeOrigin, UFrCurFrameH->GTimeS, UFrCurFrameH->dt, offset, plotRawTime->t0,plotRawTime->timeOffset);
             if (vect) printf("vect=%s:dx[0]=%f \n", vect->name, vect->dx[0]);
             else      printf("vect=NULL\n");
            }
        }
    }

 plotRawTime->tmin = plotRawTime->dt * (double)plotRawTime->t0;

 /*---- decode the options string --------*/

 UHoptDecode(plotRawTime->options, &plotRawTime->intervalle, plotRawTime->plotOpt);

 if (plotRawTime->intervalle <= 0) plotRawTime->intervalle = plotRawTime->maxAverage;

 if (Dy->plotsNotDisplayed == OFF)
    {if (strstr(plotRawTime->options,"SAME") == NULL) plotRawTime->numPicture = DyGrafIni(0);
     else                                             plotRawTime->numPicture = 0;
    }
 if (plotRawTime->intervalle < plotRawTime->maxAverage)
    {plotRawTime->intervalle = plotRawTime->maxAverage;}


/*---  Define the 2D plot used for time/freq and set limits on frequency axis  ---*/

 plotRawTime->nBinTime = (long)(plotRawTime->timeSize / (plotRawTime->maxAverage * plotRawTime->dt));

 if (plotRawTime->refresh_percent > 0)
    {
     plotRawTime->refresh_nBin = (long)((plotRawTime->refresh_percent * plotRawTime->timeSize/100.) 
                                 / (plotRawTime->maxAverage * plotRawTime->dt));
     if (plotRawTime->refresh_nBin == 0) plotRawTime->refresh_nBin = 1;
    }
 else
    {plotRawTime->refresh_nBin = 1;}


 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {printf("UHplotRawTIni> plot=%s=%s  timeSize=%g  dt=%g  nBinTime=%ld  nyBin=%ld  maxAverage=%ld \n",
             plotRawTime->name, plotRawTime->title, plotRawTime->timeSize, plotRawTime->dt, plotRawTime->nBinTime, plotRawTime->nyBin, plotRawTime->maxAverage);
    }


 if (strstr(plotRawTime->options,"AUTO") == NULL) 
    {plotRawTime->adjust = NO;}
 else
    {plotRawTime->adjust = YES;}

 /*---  Get the limits of the previous histogram in the pad if it is good  ---*/ 
 oldname = RA_PadContainsGoodHist(plotRawTime->numPicture, 0, 2, plotRawTime->nBinTime, plotRawTime->nyBin, plotRawTime->name);
 if (!oldname)
   {RA_PadCompleteClear(plotRawTime->numPicture);
    UHoptPlopt(plotRawTime->options, plotRawTime->numPicture, 0);
   }

 RA_H2NewReuse(plotRawTime->numPicture, 0,
               plotRawTime->name, plotRawTime->title,
               plotRawTime->nBinTime,
               plotRawTime->maxAverage * plotRawTime->dt * (-0.5),
               plotRawTime->maxAverage * plotRawTime->dt * ((double)plotRawTime->nBinTime - 0.5),
               plotRawTime->nyBin,
               plotRawTime->yMin,
               plotRawTime->yMax);

 if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plotRawTime->name);

 RA_HSetStat(plotRawTime->name, NULL);

 RA_SetTestTrigPad(plotRawTime->numPicture, 0, plotRawTime->refresh_percent * plotRawTime->timeSize/100.);

 RA_HSetUnitsX(plotRawTime->name, "sec");
 RA_HSetUnitsY(plotRawTime->name, "a.u.");

 if (vect != NULL)
    {
     RA_HSetUnitsY(plotRawTime->name, vect->unitX[0]);
     if (plotRawTime->realUnits > 0)
        {strcpy(plotRawTime->units, vect->unitY);}
     if (plotRawTime->realUnits == 0 && plotRawTime->slope == 1.  &&  plotRawTime->bias == 0.)
        {strcpy(plotRawTime->units, vect->unitY);}
     RA_HSetUnitsZ(plotRawTime->name, plotRawTime->units);
    }

 if (plotRawTime->zmin < DY_BADDATA)  RA_HSetMinimum(plotRawTime->name, plotRawTime->zmin);
 if (plotRawTime->zmax > -DY_BADDATA) RA_HSetMaximum(plotRawTime->name, plotRawTime->zmax);

 plotRawTime->buff = (float *)calloc(plotRawTime->nyBin+1, sizeof(float));
 if (plotRawTime->buff == NULL) printf("UHplotRawTIni> plotRawTime->buff calloc failed\n");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotRawTIni> plot=%s=%s Doing a calloc for %ld x %ld floats = %ld bytes\n",
             plotRawTime->name, plotRawTime->title, plotRawTime->nBinTime, plotRawTime->nyBin, plotRawTime->nBinTime * (plotRawTime->nyBin));
    }

 plotRawTime->buffTot = (float *) calloc(plotRawTime->nBinTime*(plotRawTime->nyBin), sizeof(float));
 if (plotRawTime->buffTot == NULL) printf("UHplotRawTIni> plotRawTime->buffTot calloc failed\n");

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {printf("UHplotRawTIni> plot=%s=%s=pad%d  plotOpt=%s \n",
             plotRawTime->name, plotRawTime->title, plotRawTime->numPicture, plotRawTime->plotOpt);
    }


 return;
}


/*----------------------------------------------------- UHplotRawTRead --*/
void UHplotRawTRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UHplotRawTime *plotRawTime;
 char *s,dummy[64];

 UJ_READ_HEADER_WITHOUT_NAME(UHplotRawTime, plotRawTime);
 
 UJ_READ_LONG(plotRawTime->iClock)
 UJ_READ_LONG(plotRawTime->id)
 if (!plotRawTime->name) plotRawTime->name = (char *) calloc(10,sizeof(char));
 sprintf(plotRawTime->name,"h%ld",plotRawTime->id);
 UJ_READ_S_CHAR(plotRawTime->title)
 UJ_READ_LONG(plotRawTime->nyBin)
 UJ_READ_DOUBLE(plotRawTime->yMin)
 UJ_READ_DOUBLE(plotRawTime->yMax)
 UJ_READ_DOUBLE(plotRawTime->timeSize)
 UJ_READ_LONG(plotRawTime->resampFactor)
 UJ_READ_LONG(plotRawTime->t0)
 UJ_READ_STRUCT(USignal, plotRawTime->input)
 UJ_READ_S_CHAR(plotRawTime->options)

 if (plotRawTime->nyBin < 0) plotRawTime->nyBin = 0;
   
 UJCObAdd(plotRawTime->iClock,plotRawTime,UHplotRawTimeUJstructH,NULL);

 plotRawTime->hidden = OFF;
 s = strstr(plotRawTime->options,"HIDDEN");
 if (s != NULL)  plotRawTime->hidden = ON;

 plotRawTime->realUnits = 0;
 s = strstr(plotRawTime->options,"REALUNITS");
 if (s != NULL)  plotRawTime->realUnits = 1;

 plotRawTime->autoScale = 0;
 s = strstr(plotRawTime->options,"AUTOSCALE");
 if (s != NULL)  plotRawTime->autoScale = 1;

 plotRawTime->zmin = DY_BADDATA;
 plotRawTime->zmax = -DY_BADDATA;
 s = strstr(plotRawTime->options,"ZMIN");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotRawTime->zmin);
 s = strstr(plotRawTime->options,"ZMAX");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotRawTime->zmax);

 plotRawTime->refresh_percent = 0.;
 s = strstr(plotRawTime->options,"REFRESH_PERCENT");
 if (s != NULL)  sscanf(s,"%s %lf",dummy,&plotRawTime->refresh_percent);

 plotRawTime->maxAverage = 1;
 s = strstr(plotRawTime->options,"NFFT");
 if (s != NULL)  sscanf(s,"%s %ld",dummy,&plotRawTime->maxAverage);

 plotRawTime->x2av = 0;
 s = strstr(plotRawTime->options,"X2AV");
 if (s != NULL)  plotRawTime->x2av = 1;

 plotRawTime->iFirst = ON;

 return;
}
 

/*--------------------------------------------------- UHplotRawTSim -----*/
void UHplotRawTSim(plotRawTime,time)
struct UHplotRawTime *plotRawTime;
long time;
/*--------------------------------------------------------------------*/
{
 long  i, j, inter, last;
 double *tab, *fft, coef, value, yValue, dc;
 char *name1, *temp;
 struct FrVect *vect1=NULL;
 long length;
 struct FrAdcData *adc;
 int fractime, nx, status;
 char title[256], stringtime[40];
 double origin;
 time_t starttimeutc;

 if (plotRawTime->hidden == ON) return;

 if (Dy->Order == STOP) return;

 if (time < plotRawTime->t0)  return;

 if (plotRawTime->raw2d > 0) return;


 /*--- Take the input values only at every frame read ---*/

 if (UFrCurFrameH && UFrCurFrameH->GTimeS > plotRawTime->previousGPS)
   {
    if (UFrCurVect == NULL)
       {printf("UHplotRawTSim> plot=%s  UFrCurFrameH=%d  previous=%d  UFrCurVect is NULL\n", plotRawTime->title, UFrCurFrameH->GTimeS, plotRawTime->previousGPS);
        plotRawTime->previousGPS = UFrCurFrameH->GTimeS;
        return;
       }

    DyDebugPrint(__FUNCTION__,0);
    DyDebugTimeNeeded(__FUNCTION__,"start",1);

    /*--- Call Ini after received first frame and getting right dt ---*/
    if (plotRawTime->iFirst == ON)
       {plotRawTime->iFirst = OFF;
        UHplotRawTIni(plotRawTime,UFrCurFrameH->dt);
        }

    /*--- Get values of input vector ---*/

    for (i=0; i<plotRawTime->nyBin; i++)
       {
        value = FrVectGetV(UFrCurVect, i);
        if (UFrCurVect->type == FR_VECT_C && value < 0) value += 256.;
        if (plotRawTime->realUnits != 0  &&  value < DY_BADDATA) 
           {value = value * plotRawTime->slope + plotRawTime->bias;}
        if (value < DY_BADDATA)
           {if (plotRawTime->x2av > 0) plotRawTime->buff[i] += value*value;
            else                       plotRawTime->buff[i] += value;
           }
        if (i == plotRawTime->nyBin-1 && plotRawTime->buff[i] == 0.) plotRawTime->buff[i] = plotRawTime->buff[i-1];
        if (plotRawTime->autoScale > 0)
           {if (plotRawTime->zmin > plotRawTime->buff[i]) plotRawTime->zmin = plotRawTime->buff[i];}
       }

    if (plotRawTime->autoScale > 0)
       {RA_HSetMinimum(plotRawTime->name, plotRawTime->zmin);}

    plotRawTime->nAverage++;

    if (plotRawTime->nAverage < plotRawTime->maxAverage) return;


    /*---- put averaged raw data and shift data if we reach the end of the plot --------*/

    if (plotRawTime->nAverage == plotRawTime->maxAverage)
       {
        RA_HSetUnitsZ(plotRawTime->name, "a.u.");
        if (plotRawTime->realUnits > 0)
           {RA_HSetUnitsZ(plotRawTime->name, plotRawTime->units);}
        if (plotRawTime->realUnits == 0 && plotRawTime->slope == 1.  &&  plotRawTime->bias == 0.)
           {RA_HSetUnitsZ(plotRawTime->name, plotRawTime->units);}

        nx = plotRawTime->nBinTime;
        for (j=0 ;j<plotRawTime->nyBin ;j++)
           {if (plotRawTime->x2av > 0) 
               {plotRawTime->buffTot[plotRawTime->itime+nx*j] = sqrt(plotRawTime->buff[j]/plotRawTime->maxAverage);}
            else
               {plotRawTime->buffTot[plotRawTime->itime+nx*j] = plotRawTime->buff[j]/plotRawTime->maxAverage;}
           }

        RA_H2Pack(plotRawTime->name, plotRawTime->buffTot);
        plotRawTime->itimeView++;

        if (Dy->plotsNotDisplayed == OFF && plotRawTime->itimeView%plotRawTime->refresh_nBin == 0)
           {
            /*--- Set time string and time axis offset ---*/
            origin = Dy->timeOrigin + plotRawTime->tmin - plotRawTime->timeOffset;
            UHplotSetTime (origin, plotRawTime->name, UFrCurFrameH->dt, plotRawTime->nAverage, ON, UFrCurFrameH);

            /*--- Draw the plot ---*/
            UHplot(plotRawTime->name, plotRawTime->numPicture, plotRawTime->options, plotRawTime->plotOpt, 0, 0);

            RA_SetActivePad(plotRawTime->numPicture);
            RA_HUpdate(plotRawTime->numPicture, plotRawTime->name, plotRawTime->title, plotRawTime->options);
           }

        plotRawTime->itime++;
        plotRawTime->nAverage = 0;
        for (j=0; j<plotRawTime->nyBin; j++) plotRawTime->buff[j] = 0.;
       }

    DyCheckRunning();    /* interaction with the data Display */

    DyDebugTimeNeeded(__FUNCTION__,"stop",1);

    /*--- Update buffers ---*/

    if (plotRawTime->itime == plotRawTime->nBinTime)
       {
        plotRawTime->itime--;
        RA_H2Unpack(plotRawTime->name, plotRawTime->buffTot);

        /*--- Shift the buffer content ---*/
        nx = plotRawTime->nBinTime;
        for (i=0; i<plotRawTime->nBinTime-1; i++)
           {for (j=0; j<plotRawTime->nyBin; j++)
               {plotRawTime->buffTot[i+j*nx] = plotRawTime->buffTot[i+1+j*nx];}
           }
        for (j=0; j<plotRawTime->nyBin; j++)  plotRawTime->buffTot[nx-1+nx*j] = 0.;
        plotRawTime->tmin += plotRawTime->dt * plotRawTime->maxAverage;
       }
   }

 /*--- Take care of missing frames ---*/

 if (UFrCurFrameH)
    {
     if ((plotRawTime->previousGPS > 0) && ((double)(UFrCurFrameH->GTimeS-plotRawTime->previousGPS) > 1.1*plotRawTime->dt))
        {
         if (Dy->skipFrame == ON || Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
            {plotRawTime->tmin += (double)(UFrCurFrameH->GTimeS - plotRawTime->previousGPS) - UFrCurFrameH->dt;}
        }
     if (UFrCurFrameH->GTimeS > 0) plotRawTime->previousGPS = UFrCurFrameH->GTimeS;
    }

 return;
}


